@@ -132,6 +132,14 @@ void _Error_Message(PGconn *pConnection)
 //														 //
 ///////////////////////////////////////////////////////////
 
+//---------------------------------------------------------
+CSG_PG_Connection::CSG_PG_Connection(void)
+{
+	m_bTransaction	= false;
+
+	m_pConnection	= NULL;
+}
+
 //---------------------------------------------------------
 CSG_PG_Connection::CSG_PG_Connection(const CSG_String &Host, int Port, const CSG_String &Name, const CSG_String &User, const CSG_String &Password, bool bAutoCommit)
 {
@@ -142,7 +150,9 @@ bool CSG_PG_Connection::Create(const CSG_String &Host, int Port, const CSG_Strin
 {
 	m_bTransaction	= false;
 
-	m_pConnection	= PQsetdbLogin(Host, CSG_String::Format(SG_T("%d"), Port), NULL, NULL, Name, User, Password);
+	m_pConnection	= Name.is_Empty()
+		? PQsetdbLogin(Host, CSG_String::Format("%d", Port), NULL, NULL, NULL, User, Password)
+		: PQsetdbLogin(Host, CSG_String::Format("%d", Port), NULL, NULL, Name, User, Password);
 
 	if( PQstatus(m_pgConnection) != CONNECTION_OK )
 	{
@@ -219,19 +229,16 @@ TSG_Data_Type CSG_PG_Connection::Get_Type_From_SQL(int Type)
 	switch( Type )
 	{
 	case SG_PG_VARCHAR:	return( SG_DATATYPE_String );
-	case SG_PG_NAME:	return( SG_DATATYPE_String );
-	case SG_PG_TEXT:	return( SG_DATATYPE_String );
-
-	case SG_PG_INT2:	return( SG_DATATYPE_Short  );	// 2 bytes integer
-	case SG_PG_INT4:	return( SG_DATATYPE_Int    );	// 4 bytes integer
-	case SG_PG_INT8:	return( SG_DATATYPE_Long   );	// 8 bytes integer
-	case SG_PG_FLOAT4:	return( SG_DATATYPE_Float  );	// 4 bytes floating point, inexact
-	case SG_PG_FLOAT8:	return( SG_DATATYPE_Double );	// 8 bytes floating point, inexact
-
-	case SG_PG_BYTEA:	return( SG_DATATYPE_Binary );
+	case SG_PG_NAME   :	return( SG_DATATYPE_String );
+	case SG_PG_TEXT   :	return( SG_DATATYPE_String );
+	case SG_PG_INT2   :	return( SG_DATATYPE_Short  );	// 2 bytes integer
+	case SG_PG_INT4   :	return( SG_DATATYPE_Int    );	// 4 bytes integer
+	case SG_PG_INT8   :	return( SG_DATATYPE_Long   );	// 8 bytes integer
+	case SG_PG_FLOAT4 :	return( SG_DATATYPE_Float  );	// 4 bytes floating point, inexact
+	case SG_PG_FLOAT8 :	return( SG_DATATYPE_Double );	// 8 bytes floating point, inexact
+	case SG_PG_BYTEA  :	return( SG_DATATYPE_Binary );
+	default           :	return( SG_DATATYPE_String );
 	}
-
-	return( SG_DATATYPE_String );
 }
 
 //---------------------------------------------------------
@@ -239,19 +246,18 @@ CSG_String CSG_PG_Connection::Get_Raster_Type_To_SQL(TSG_Data_Type Type)
 {
 	switch( Type )
 	{
-	case SG_DATATYPE_Bit:		return( "1BB"   );
-	case SG_DATATYPE_Char:		return( "8BSI"  );
-	case SG_DATATYPE_Byte:		return( "8BUI"  );
-	case SG_DATATYPE_Short:		return( "16BSI" );
-	case SG_DATATYPE_Word:		return( "16BUI" );
-	case SG_DATATYPE_Int:		return( "32BSI" );
-	case SG_DATATYPE_DWord:		return( "32BUI" );
-	case SG_DATATYPE_Long:		return( "32BSI" );
-	case SG_DATATYPE_Float:		return( "32BF"  );
+	case SG_DATATYPE_Bit   :	return( "1BB"   );
+	case SG_DATATYPE_Char  :	return( "8BSI"  );
+	case SG_DATATYPE_Byte  :	return( "8BUI"  );
+	case SG_DATATYPE_Short :	return( "16BSI" );
+	case SG_DATATYPE_Word  :	return( "16BUI" );
+	case SG_DATATYPE_Int   :	return( "32BSI" );
+	case SG_DATATYPE_DWord :	return( "32BUI" );
+	case SG_DATATYPE_Long  :	return( "32BSI" );
+	case SG_DATATYPE_Float :	return( "32BF"  );
 	case SG_DATATYPE_Double:	return( "64BF"  );
+	default                :	return( "64BF"  );
 	}
-
-	return( "64BF" );
 }
 
 
@@ -271,9 +277,16 @@ void CSG_PG_Connection::GUI_Update(void) const
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-CSG_String CSG_PG_Connection::Get_Connection(void) const
+CSG_String CSG_PG_Connection::Get_Connection(int Style) const
 {
-	return( Get_DBName() + " [" + Get_Host() + ":" + Get_Port() + "]" );
+	switch( Style )
+	{
+	default: return( Get_DBName() + " [" + Get_Host() + ":" + Get_Port() + "]" );
+
+	case  1: return( Get_DBName() );
+
+	case  2: return( "PGSQL:" + Get_Host() + ":" + Get_Port() + ":" + Get_DBName() );
+	}
 }
 
 //---------------------------------------------------------
@@ -344,11 +357,42 @@ bool CSG_PG_Connection::has_PostGIS(double minVersion)
 	CSG_Table	t;
 
 	return( _Table_Load(t, "SELECT PostGIS_Lib_Version()", "") && t.Get_Count() == 1 && t.Get_Field_Count() == 1
-		&& 0.0001 * t[0][0].asDouble() >= minVersion
+		&& t[0][0].asDouble() >= minVersion
 	);
 }
 
 
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+CSG_MetaData & CSG_PG_Connection::Add_MetaData(CSG_Data_Object &Object, const CSG_String &Table, const CSG_String &Select)
+{
+	CSG_String	Name(Object.Get_Name());
+	Object.Set_File_Name(Get_Connection(2) + ":" + Table);
+	Object.Set_Name(Name);
+
+	CSG_MetaData	&DB	= Object.Get_MetaData_DB();
+
+	DB.Del_Children();
+
+	DB.Add_Child("DBMS" , "PostgreSQL");
+	DB.Add_Child("HOST" , Get_Host      ());
+	DB.Add_Child("PORT" , Get_Port      ());
+	DB.Add_Child("USER" , Get_User      ());
+	DB.Add_Child("NAME" , Get_DBName    ());
+	DB.Add_Child("TABLE", Table);
+
+	if( !Select.is_Empty() )
+	{
+		DB.Add_Child("SELECT", Select);
+	}
+
+	return( DB );
+}
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 ///////////////////////////////////////////////////////////
@@ -801,7 +845,27 @@ bool CSG_PG_Connection::Table_Insert(const CSG_String &Table_Name, const CSG_Tab
 			}
 			else
 			{
-				sprintf(Values[iField], "%s", CSG_String(pRecord->asString(iField)).b_str());
+				CSG_String	Value	= pRecord->asString(iField);
+
+				if( 0 && Table.Get_Field_Type(iField) == SG_DATATYPE_String )
+				{
+					char	*s	= NULL;
+
+					if( Value.to_ASCII(&s) && s && *s )
+					{
+						sprintf(Values[iField], "%s", s);
+					}
+					else
+					{
+						Values[iField][0]	= '\0';
+					}
+
+					SG_FREE_SAFE(s);
+				}
+				else
+				{
+					sprintf(Values[iField], "%s", Value.b_str());
+				}
 
 				paramValues [iField]	= Values[iField];
 			}
@@ -860,6 +924,8 @@ bool CSG_PG_Connection::Table_Save(const CSG_String &Table_Name, const CSG_Table
 	}
 
 	//-----------------------------------------------------
+	Add_MetaData(*((CSG_Table *)&Table), Table_Name);
+
 	return( true );
 }
 
@@ -955,7 +1021,14 @@ bool CSG_PG_Connection::_Table_Load(CSG_Table &Table, void *_pResult) const
 //---------------------------------------------------------
 bool CSG_PG_Connection::Table_Load(CSG_Table &Table, const CSG_String &Table_Name)
 {
-	return( _Table_Load(Table, CSG_String::Format(SG_T("SELECT * FROM \"%s\""), Table_Name.c_str()), Table_Name) );
+	if( _Table_Load(Table, CSG_String::Format(SG_T("SELECT * FROM \"%s\""), Table_Name.c_str()), Table_Name) )
+	{
+		Add_MetaData(Table, Table_Name);
+
+		return( true );
+	}
+
+	return( false );
 }
 
 //---------------------------------------------------------
@@ -985,7 +1058,14 @@ bool CSG_PG_Connection::Table_Load(CSG_Table &Table, const CSG_String &Tables, c
 		Select	+= SG_T(" ORDER BY ") + Order;
 	}
 
-	return( _Table_Load(Table, Select, Table.Get_Name()) );
+	if( _Table_Load(Table, Select, Table.Get_Name()) )
+	{
+		Add_MetaData(Table, Table.Get_Name(), Select);
+
+		return( true );
+	}
+
+	return( false );
 }
 
 
@@ -1028,7 +1108,14 @@ bool CSG_PG_Connection::Shapes_Load(CSG_Shapes *pShapes, const CSG_String &Name)
 
 	Fields	+= (bBinary ? "ST_AsBinary(" : "ST_AsText(") + Geometry + ") AS __geometry__";
 
-	return( Shapes_Load(pShapes, Name, "SELECT " + Fields + " FROM \"" + Name + "\"", "__geometry__", bBinary, SRID) );
+	if( Shapes_Load(pShapes, Name, "SELECT " + Fields + " FROM \"" + Name + "\"", "__geometry__", bBinary, SRID) )
+	{
+		Add_MetaData(*pShapes, Name);
+
+		return( true );
+	}
+
+	return( false );
 }
 
 //---------------------------------------------------------
@@ -1168,6 +1255,8 @@ bool CSG_PG_Connection::Shapes_Load(CSG_Shapes *pShapes, const CSG_String &Name,
 	//-----------------------------------------------------
 	PQclear(pResult);
 
+	Add_MetaData(*pShapes, Name, Select);
+
 	return( true );
 }
 
@@ -1177,30 +1266,31 @@ bool CSG_PG_Connection::Shapes_Load(CSG_Shapes *pShapes, const CSG_String &Name,
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CSG_PG_Connection::Raster_Load(const CSG_String &Table, const CSG_String &Where, const CSG_String &Order, const CSG_String &Names, CSG_Parameter_Grid_List *pList)
+bool CSG_PG_Connection::_Raster_Open(CSG_Table &Info, const CSG_String &Table, const CSG_String &Where, const CSG_String &Order, bool bBinary)
 {
-	CSG_Table	Info;
-
+	//-----------------------------------------------------
 	if( !Table_Load(Info, "raster_columns", "*", CSG_String("r_table_name = '") + Table + "'") || Info.Get_Count() != 1 )
 	{
+		SG_UI_Msg_Add_Error(CSG_String::Format("[PostGIS] %s (%s)", _TL("could not access table"), SG_T("raster_columns")));
+
 		return( false );
 	}
 
-	CSG_String	Geometry	= Info[0].asString("r_raster_column");
+	CSG_String	Field	= Info[0].asString("r_raster_column");
 
 	//-----------------------------------------------------
-	if( Names.is_Empty() || !Table_Load(Info, Table, Names, Where, "", "", Order) )
+	if( !Table_Load(Info, Table, "rid, name", Where, "", "", Order) )
 	{
-		Info.Destroy();
+		SG_UI_Msg_Add_Error(CSG_String::Format("[PostGIS] %s (%s)", _TL("could not access raster table"), Table.c_str()));
+
+		return( false );
 	}
 
 	//-----------------------------------------------------
-	bool	bBinary	= true;
+	CSG_String	Select	= "COPY (SELECT ST_AsBinary(\"" + Field + "\") AS rastbin FROM \"" + Table + "\"";
 
-	CSG_String	Select	= "COPY (SELECT ST_AsBinary(\"" + Geometry + "\") AS rastbin FROM \"" + Table + "\"";
-
-	if( Where.Length() )	Select	+= SG_T(" WHERE ")    + Where;
-	if( Order.Length() )	Select	+= SG_T(" ORDER BY ") + Order;
+	if( Where.Length() )	Select	+= " WHERE "    + Where;
+	if( Order.Length() )	Select	+= " ORDER BY " + Order;
 
 	Select	+= ") TO STDOUT";	if( bBinary )	Select	+= " WITH (FORMAT 'binary')";
 
@@ -1216,72 +1306,116 @@ bool CSG_PG_Connection::Raster_Load(const CSG_String &Table, const CSG_String &W
 		return( false );
 	}
 
+	PQclear(pResult);
+
+	return( true );
+}
+
+//---------------------------------------------------------
+bool CSG_PG_Connection::_Raster_Load(CSG_Grid *pGrid, bool bFirst, bool bBinary)
+{
 	//-----------------------------------------------------
 	char	*Bytes;
-	int		nBytes, nOkay, iBand;
+	int		nBytes;
 
-	for(iBand=0, nOkay=0; SG_UI_Process_Get_Okay() && (nBytes = PQgetCopyData(m_pgConnection, &Bytes, 0)) > 0; iBand++)
+	if( (nBytes = PQgetCopyData(m_pgConnection, &Bytes, 0)) <= 0 )
 	{
-		CSG_Bytes	Band;
+		return( false );
+	}
 
-		if( bBinary )
+	//-----------------------------------------------------
+	CSG_Bytes	Band;
+
+	if( bBinary )
+	{
+		int	Offset	= bFirst ? 25 : 6;
+
+		if( *((short *)Bytes) > 0 && nBytes > Offset )
 		{
-			int		Offset	= iBand == 0 ? 25 : 6;
-
-			if( *((short *)Bytes) > 0 && nBytes > Offset )
-			{
-				Band.Create((BYTE *)(Bytes + Offset), nBytes - Offset);
-			}
-		}
-		else if( nBytes > 3 )
-		{
-			Band.fromHexString(Bytes + 3);
+			Band.Create((BYTE *)(Bytes + Offset), nBytes - Offset);
 		}
+	}
+	else if( nBytes > 3 )
+	{
+		Band.fromHexString(Bytes + 3);
+	}
 
-		PQfreemem(Bytes);
+	PQfreemem(Bytes);
 
-		//-------------------------------------------------
+	return( Band.Get_Count() > 0 && CSG_Grid_OGIS_Converter::from_WKBinary(Band, pGrid) );
+}
+
+//---------------------------------------------------------
+bool CSG_PG_Connection::Raster_Load(CSG_Parameter_Grid_List *pGrids, const CSG_String &Table, const CSG_String &Where, const CSG_String &Order)
+{
+	//-----------------------------------------------------
+	bool		bBinary	= true;
+
+	CSG_Table	Info;
+
+	if( !_Raster_Open(Info, Table, Where, Order, bBinary) )
+	{
+		return( false );
+	}
+
+	//-----------------------------------------------------
+	for(int iBand=0; iBand<Info.Get_Count() && SG_UI_Process_Get_Okay(); iBand++)
+	{
 		CSG_Grid	*pGrid	= SG_Create_Grid();
 
-		if( Band.Get_Count() > 0 && CSG_Grid_OGIS_Converter::from_WKBinary(Band, pGrid) )
+		if( !_Raster_Load(pGrid, iBand == 0, bBinary) )
 		{
-			if( iBand < Info.Get_Count() )
-			{
-				pGrid->Set_Name(CSG_String::Format(SG_T("%s [%s]"  ), Table.c_str(), Info[iBand].asString(0)));
-			}
-			else
-			{
-				pGrid->Set_Name(CSG_String::Format(SG_T("%s [%02d]"), Table.c_str(), iBand + 1));
-			}
+			delete(pGrid);
 
-			SG_Get_Data_Manager().Add(pGrid);
-
-			if( pList )
-			{
-				pList->Add_Item(pGrid);
-			}
-
-			nOkay++;
+			return( false );
 		}
 		else
 		{
-			delete(pGrid);
+			pGrid->Set_Name(Table + " [" + Info[iBand].asString(1) + "]");
+
+			Add_MetaData(*pGrid, Table + CSG_String::Format(":rid=%d", Info[iBand].asInt(0)));
+
+			SG_Get_Data_Manager().Add(pGrid);
+
+			if( pGrids )
+			{
+				pGrids->Add_Item(pGrid);
+			}
 		}
 	}
 
 	//-----------------------------------------------------
-	PQclear(pResult);
-
-	return( nOkay > 0 );
+	return( true );
 }
 
 //---------------------------------------------------------
-bool CSG_PG_Connection::Raster_Save(CSG_Grid *pGrid, int SRID, const CSG_String &Table, const CSG_String &Field)
+bool CSG_PG_Connection::Raster_Load(CSG_Grid *pGrid, const CSG_String &Table, const CSG_String &Where)
 {
 	CSG_Table	Info;
 
-	if( !Table_Load(Info, "raster_columns", "*", CSG_String("r_table_name = '") + Table + "'") || Info.Get_Count() != 1 )
+	if( _Raster_Open(Info, Table, Where) && _Raster_Load(pGrid, true) )
 	{
+		pGrid->Set_Name(Table + " [" + Info[0].asString(1) + "]");
+
+		Add_MetaData(*pGrid, Table + CSG_String::Format(":rid=%d", Info[0].asInt(0)));
+
+		SG_Get_Data_Manager().Add(pGrid);
+
+		return( true );
+	}
+
+	return( false );
+}
+
+//---------------------------------------------------------
+bool CSG_PG_Connection::Raster_Save(CSG_Grid *pGrid, int SRID, const CSG_String &Table, const CSG_String &Name)
+{
+	CSG_Table	Info;
+
+	if( !pGrid || !Table_Load(Info, "raster_columns", "*", CSG_String("r_table_name = '") + Table + "'") || Info.Get_Count() != 1 )
+	{
+		SG_UI_Msg_Add_Error(CSG_String::Format("[PostGIS] %s (%s)", _TL("could not access table"), SG_T("raster_columns")));
+
 		return( false );
 	}
 
@@ -1304,6 +1438,8 @@ bool CSG_PG_Connection::Raster_Save(CSG_Grid *pGrid, int SRID, const CSG_String
 		return( false );
 	}
 
+	PQclear(pResult);
+
 	//-----------------------------------------------------
 	CSG_Bytes	Band;
 
@@ -1317,14 +1453,26 @@ bool CSG_PG_Connection::Raster_Save(CSG_Grid *pGrid, int SRID, const CSG_String
 		{
 			CSG_String	hex(Band.toHexString());
 
-			PQputCopyData(m_pgConnection, hex, hex.Length());
+			PQputCopyData(m_pgConnection, hex, (int)hex.Length());
 		}
 
 		PQputCopyEnd (m_pgConnection, NULL);
 	}
 
 	//-----------------------------------------------------
-	PQclear(pResult);
+	Table_Load(Info, Table, "rid");	// CSG_String::Format("SELECT currval('%s_rid_seq')", Table.c_str()));
+
+	int	rid	= Info[Info.Get_Count() - 1].asInt(0);
+
+	//-----------------------------------------------------
+	Info	= Get_Field_Desc(Table);
+
+	if( !Name.is_Empty() && Info.Get_Count() > 2 && !SG_STR_CMP("varchar", Info[2].asString(1)) )
+	{
+		Execute(CSG_String::Format("UPDATE %s SET %s='%s' WHERE rid=%d", Table.c_str(), Info[2].asString(0), Name.c_str(), rid));
+	}
+
+	Add_MetaData(*pGrid, Table + CSG_String::Format(":rid=%d", rid));
 
 	return( true );
 }
@@ -1551,46 +1699,41 @@ int CSG_PG_Connections::Get_Connections(CSG_String &Connections)
 //---------------------------------------------------------
 CSG_PG_Module::CSG_PG_Module(void)
 {
-	if( !SG_UI_Get_Window_Main() )
-	{
-		Parameters.Add_String(
-			NULL	, "PG_HOST"		, _TL("Host"),
-			_TL("Password"),
-			SG_T("")
-		);
+	Parameters.Add_String(
+		NULL	, "PG_HOST"		, _TL("Host"),
+		_TL("Password"),
+		""
+	)->Set_UseInGUI(false);
 
-		Parameters.Add_Value(
-			NULL	, "PG_PORT"		, _TL("Port"),
-			_TL(""),
-			PARAMETER_TYPE_Int, 5432, 0, true
-		);
+	Parameters.Add_Value(
+		NULL	, "PG_PORT"		, _TL("Port"),
+		_TL(""),
+		PARAMETER_TYPE_Int, 5432, 0, true
+	)->Set_UseInGUI(false);
 
-		Parameters.Add_String(
-			NULL	, "PG_NAME"		, _TL("Database"),
-			_TL("Database Name"),
-			SG_T("")
-		);
+	Parameters.Add_String(
+		NULL	, "PG_NAME"		, _TL("Database"),
+		_TL("Database Name"),
+		""
+	)->Set_UseInGUI(false);
 
-		Parameters.Add_String(
-			NULL	, "PG_USER"		, _TL("User"),
-			_TL("User Name"),
-			SG_T("")
-		);
+	Parameters.Add_String(
+		NULL	, "PG_USER"		, _TL("User"),
+		_TL("User Name"),
+		""
+	)->Set_UseInGUI(false);
 
-		Parameters.Add_String(
-			NULL	, "PG_PWD"		, _TL("Password"),
-			_TL("Password"),
-			SG_T("")
-		);
-	}
-	else
-	{
-		Parameters.Add_Choice(
-			NULL	, "CONNECTION", _TL("Available Connections"),
-			_TL(""),
-			""
-		);
-	}
+	Parameters.Add_String(
+		NULL	, "PG_PWD"		, _TL("Password"),
+		_TL("Password"),
+		""
+	)->Set_UseInGUI(false);
+
+	Parameters.Add_Choice(
+		NULL	, "CONNECTION"	, _TL("Available Connections"),
+		_TL(""),
+		""
+	)->Set_UseInCMD(false);
 
 	m_pConnection	= NULL;
 }
