@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: MLB_Interface.cpp 2281 2014-10-09 15:49:41Z oconrad $
+ * Version $Id: MLB_Interface.cpp 2584 2015-09-17 16:13:56Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -115,6 +115,9 @@ CSG_String Get_Info(int i)
 #include "Grid_Accumulation_Functions.h"
 #include "Grid_IMCORR.h"
 
+#include "diversity_analysis.h"
+
+
 //---------------------------------------------------------
 // 4. Allow your modules to be created here...
 
@@ -124,8 +127,6 @@ CSG_Module *		Create_Module(int i)
 	{
 	case  0:	return( new CCost_Isotropic );
 	case  1:	return( new CCost_Anisotropic );
-	case  2:	return( MLB_INTERFACE_SKIP_MODULE );	// removed: CCost_PolarToRect
-	case  3:	return( MLB_INTERFACE_SKIP_MODULE );	// removed: CCost_RectToPolar
 	case  4:	return( new CLeastCostPathProfile );
 	case  5:	return( new CLeastCostPathProfile_Points );
 
@@ -139,17 +140,21 @@ CSG_Module *		Create_Module(int i)
 	case 13:	return( new CCrossClassification );
 
 	case 14:	return( new CSoil_Texture );
+	case 20:	return( new CSoil_Texture_Table );
 
 	case 15:	return( new CFragmentation_Standard );
 	case 16:	return( new CFragmentation_Resampling );
 	case 17:	return( new CFragmentation_Classify );
 
 	case 18:	return( new CGrid_Accumulation_Functions );
-	
-	case 19:	return( new CGrid_IMCORR );
-	}
 
-	return( NULL );
+	case 19:	return( new CGrid_IMCORR );
+
+	case 21:	return( new CDiversity_Analysis );
+
+	case 22:	return( NULL );
+	default:	return( MLB_INTERFACE_SKIP_MODULE );
+	}
 }
 
 
