@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: Grid_Calculator.cpp 2207 2014-08-26 13:39:34Z oconrad $
+ * Version $Id: Grid_Calculator.cpp 2509 2015-06-15 16:18:32Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -88,6 +88,8 @@ bool _finite(double val)
 //---------------------------------------------------------
 CGrid_Calculator::CGrid_Calculator(void)
 {
+	CSG_Parameter	*pNode;
+
 	//-----------------------------------------------------
 	Set_Name	(_TL("Grid Calculator"));
 
@@ -95,12 +97,12 @@ CGrid_Calculator::CGrid_Calculator(void)
 
 	CSG_String	s(_TW(
 		"The Grid Calculator calculates a new grid based on existing grids and a mathematical formula. "
-		"The grid variables are single characters which correspond in alphabetical order to the grid list order "
-		"('a' = first grid, 'b' = second grid, ...)\n"
-		"Alternatively you can address with letter 'g' followed by position index (g1, g2, g3, ...). "
-		"Grids from other systems are addressed similarly, just using letter 'h' (h1, h2, h3, ...)\n"
-		"Examples:\n"
-		"using letters: sin(a) * b + c\n"
+		"The grid variables in the formula begin with the letter 'g' followed by a position index, "
+		"which corresponds to the order of the grids in the input grid list "
+		"(i.e.: g1, g2, g3, ... correspond to the first, second, third, ... grid in list). "
+		"Grids from other systems than the default one can be addressed likewise using the letter 'h' "
+		"(h1, h2, h3, ...), which correspond to the \'Grids from different Systems\' list.\n"
+		"Example:\t sin(g1) * g2 + h1\n"
 		"the same using indices: sin(g1) * g2 + g3\n\n"
 		"The following operators are available for the formula definition:\n"
 	));
@@ -116,49 +118,61 @@ CGrid_Calculator::CGrid_Calculator(void)
 
 	//-----------------------------------------------------
 	Parameters.Add_Grid_List(
-		NULL	, "GRIDS"	, _TL("Grids"),
+		NULL	, "GRIDS"			, _TL("Grids"),
 		_TL("in the formula these grids are addressed in order of the list as 'g1, g2, g3, ...'"),
 		PARAMETER_INPUT_OPTIONAL
 	);
 
-	Parameters.Add_Grid_List(
-		NULL	, "XGRIDS"	, _TL("Grids from different Systems"),
+	pNode	= Parameters.Add_Grid_List(
+		NULL	, "XGRIDS"			, _TL("Grids from different Systems"),
 		_TL("in the formula these grids are addressed in order of the list as 'h1, h2, h3, ...'"),
 		PARAMETER_INPUT_OPTIONAL, false
 	);
 
+	Parameters.Add_Choice(
+		pNode	,"INTERPOLATION"	, _TL("Interpolation"),
+		_TL(""),
+		CSG_String::Format(SG_T("%s|%s|%s|%s|%s|"),
+			_TL("Nearest Neighbor"),
+			_TL("Bilinear Interpolation"),
+			_TL("Inverse Distance Interpolation"),
+			_TL("Bicubic Spline Interpolation"),
+			_TL("B-Spline Interpolation")
+		), 4
+	);
+
 	Parameters.Add_Grid(
-		NULL	, "RESULT"	, _TL("Result"),
+		NULL	, "RESULT"			, _TL("Result"),
 		_TL(""),
 		PARAMETER_OUTPUT
 	);
 	
 	Parameters.Add_String(
-		NULL	, "FORMULA"	, _TL("Formula"),
+		NULL	, "FORMULA"			, _TL("Formula"),
 		_TL(""),
 		SG_T("(g1 - g2) / (g1 + g2)")
 	);
 
-	CSG_Parameter	*pNode	= Parameters.Add_String(
-		NULL	, "NAME"	, _TL("Name"),
+	pNode	= Parameters.Add_String(
+		NULL	, "NAME"			, _TL("Name"),
 		_TL(""),
 		_TL("Calculation")
 	);
 
 	Parameters.Add_Value(
-		pNode	, "FNAME"	, _TL("Take Formula"),
+		pNode	, "FNAME"			, _TL("Take Formula"),
 		_TL(""),
 		PARAMETER_TYPE_Bool, false
 	);
 
 	Parameters.Add_Value(
-		NULL	, "USE_NODATA"	, _TL("Use NoData"),
+		NULL	, "USE_NODATA"		, _TL("Use NoData"),
 		_TL("Check this in order to include NoData cells in the calculation."),
 		PARAMETER_TYPE_Bool, false
 	);
 
 	Parameters.Add_Choice(
-		NULL	, "TYPE"	, _TL("Data Type"),
+		NULL	, "TYPE"			, _TL("Data Type"),
 		_TL(""),
 		CSG_String::Format(SG_T("%s|%s|%s|%s|%s|%s|%s|%s|%s|"),
 			SG_Data_Type_Get_Name(SG_DATATYPE_Bit   ).c_str(),
@@ -177,15 +191,13 @@ CGrid_Calculator::CGrid_Calculator(void)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 int CGrid_Calculator::On_Parameter_Changed(CSG_Parameters *pParameters, CSG_Parameter *pParameter)
 {
-	if(	!SG_STR_CMP(pParameter->Get_Identifier(), SG_T("FORMULA"))
-	||	!SG_STR_CMP(pParameter->Get_Identifier(), SG_T("FNAME")) )
+	if(	!SG_STR_CMP(pParameter->Get_Identifier(), "FORMULA")
+	||	!SG_STR_CMP(pParameter->Get_Identifier(), "FNAME") )
 	{
 		if( pParameters->Get_Parameter("FNAME")->asBool() )
 		{
@@ -193,32 +205,43 @@ int CGrid_Calculator::On_Parameter_Changed(CSG_Parameters *pParameters, CSG_Para
 		}
 	}
 
-	return( 0 );
+	return( CSG_Module_Grid::On_Parameter_Changed(pParameters, pParameter) );
+}
+
+//---------------------------------------------------------
+int CGrid_Calculator::On_Parameters_Enable(CSG_Parameters *pParameters, CSG_Parameter *pParameter)
+{
+	if( !SG_STR_CMP(pParameter->Get_Identifier(), "XGRIDS") )
+	{
+		pParameters->Set_Enabled("INTERPOLATION", pParameter->asGridList()->Get_Count() > 0);
+	}
+
+	return( CSG_Module_Grid::On_Parameters_Enable(pParameters, pParameter) );
 }
 
 
 ///////////////////////////////////////////////////////////
 //														 //
-//                                                       //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 bool CGrid_Calculator::On_Execute(void)
 {
-	bool					bUseNoData;
+	bool					bUseNoData, bPosition[4];
+	int						Interpol;
 	CSG_Formula				Formula;
 	CSG_Parameter_Grid_List	*pGrids, *pXGrids;
 	CSG_Grid				*pResult;
 
 	//-----------------------------------------------------
-	pResult		= Parameters("RESULT"    )->asGrid();
-	pGrids		= Parameters("GRIDS"     )->asGridList();
-	pXGrids		= Parameters("XGRIDS"    )->asGridList();
-	bUseNoData	= Parameters("USE_NODATA")->asBool();
+	pResult		= Parameters("RESULT"       )->asGrid();
+	pGrids		= Parameters("GRIDS"        )->asGridList();
+	pXGrids		= Parameters("XGRIDS"       )->asGridList();
+	bUseNoData	= Parameters("USE_NODATA"   )->asBool();
+	Interpol	= Parameters("INTERPOLATION")->asInt();
 
 	//-----------------------------------------------------
-	if( !Get_Formula(Formula, Parameters("FORMULA")->asString(), pGrids->Get_Count(), pXGrids->Get_Count()) )
+	if( !Get_Formula(Formula, Parameters("FORMULA")->asString(), pGrids->Get_Count(), pXGrids->Get_Count(), bPosition) )
 	{
 		return( false );
 	}
@@ -247,6 +270,12 @@ bool CGrid_Calculator::On_Execute(void)
 
 	pResult->Set_Name(Parameters("NAME")->asString());
 
+	int	nValues	= pGrids->Get_Count() + pXGrids->Get_Count()
+		+ (bPosition[0] ? 1 : 0)
+		+ (bPosition[1] ? 1 : 0)
+		+ (bPosition[2] ? 1 : 0)
+		+ (bPosition[3] ? 1 : 0);
+
 	//-----------------------------------------------------
 	for(int y=0; y<Get_NY() && Set_Progress(y); y++)
 	{
@@ -255,50 +284,41 @@ bool CGrid_Calculator::On_Execute(void)
 		#pragma omp parallel for
 		for(int x=0; x<Get_NX(); x++)
 		{
-			bool		bNoData	= false;
-			int			i, n;
+			bool		bOkay	= true;
+			int			i, n	= 0;
 			double		Result, px	= Get_XMin() + x * Get_Cellsize();
-			CSG_Vector	Values(pGrids->Get_Count() + pXGrids->Get_Count() + 4);
+			CSG_Vector	Values(nValues);
 
-			for(i=0, n=0; !bNoData && i<pGrids->Get_Count(); i++)
+			for(i=0; bOkay && i<pGrids->Get_Count() ; i++, n++)
 			{
-				if( !bUseNoData && pGrids->asGrid(i)->is_NoData(x, y) )
+				if( (bOkay = bUseNoData || !pGrids->asGrid(i)->is_NoData(x, y)) == true )
 				{
-					bNoData		= true;
-				}
-				else
-				{
-					Values[n++]	= pGrids->asGrid(i)->asDouble(x, y);
+					Values[n]	= pGrids->asGrid(i)->asDouble(x, y);
 				}
 			}
 
-			for(i=0, n=pGrids->Get_Count(); !bNoData && i<pXGrids->Get_Count(); i++)
+			for(i=0; bOkay && i<pXGrids->Get_Count(); i++, n++)
 			{
-				if( !pXGrids->asGrid(i)->Get_Value(px, py, Values[n++]) )
-				{
-					bNoData		= true;
-				}
+				bOkay	= pXGrids->asGrid(i)->Get_Value(px, py, Values[n], Interpol);
 			}
 
-			if( !bNoData )
+			if( bOkay )
 			{
-				n	= pGrids->Get_Count() + pXGrids->Get_Count();
+				if( bPosition[0] )	Values[n++]	=  x;	// col()
+				if( bPosition[1] )	Values[n++]	=  y;	// row()
+				if( bPosition[2] )	Values[n++]	= px;	// xpos()
+				if( bPosition[3] )	Values[n++]	= py;	// ypos()
 
-				Values[n++]	= y;	// col()
-				Values[n++]	= x;	// row()
-				Values[n++]	= py;	// ypos()
-				Values[n++]	= px;	// xpos()
-
-				bNoData	= !_finite(Result = Formula.Get_Value(Values));
+				bOkay	= _finite(Result = Formula.Get_Value(Values)) != 0;
 			}
 
-			if( bNoData )
+			if( bOkay )
 			{
-				pResult->Set_NoData(x, y);
+				pResult->Set_Value(x, y, Result);
 			}
 			else
 			{
-				pResult->Set_Value(x, y, Result);
+				pResult->Set_NoData(x, y);
 			}
 		}
 	}
@@ -310,42 +330,47 @@ bool CGrid_Calculator::On_Execute(void)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//                                                       //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CGrid_Calculator::Get_Formula(CSG_Formula &Formula, CSG_String sFormula, int nGrids, int nXGrids)
+bool CGrid_Calculator::Get_Formula(CSG_Formula &Formula, CSG_String sFormula, int nGrids, int nXGrids, bool bPosition[4])
 {
 	//-----------------------------------------------------
 	const int		nVars		= 27;
 
 	const SG_Char	Vars[nVars]	= SG_T("abcdefghijklmnopqrstuvwxyz");
 
+	int		nValues	= nGrids + nXGrids;
+
+	bPosition[0]	= sFormula.Find("col()" ) >= 0;	if( bPosition[0] )	nValues++;
+	bPosition[1]	= sFormula.Find("row()" ) >= 0;	if( bPosition[1] )	nValues++;
+	bPosition[2]	= sFormula.Find("xpos()") >= 0;	if( bPosition[2] )	nValues++;
+	bPosition[3]	= sFormula.Find("ypos()") >= 0;	if( bPosition[3] )	nValues++;
+
 	//-----------------------------------------------------
-	if( nGrids + nXGrids + 4 > nVars )
+	if( nValues > nVars )
 	{
-		Error_Set(_TL("too many input grids"));
+		Error_Set(_TL("too many input variables"));
 
 		return( false );
 	}
 
 	//-----------------------------------------------------
-	int		i, n = nGrids + nXGrids + 4 - 1;
+	int		i, n	= nValues;
 
-	sFormula.Replace("xpos()", Vars[n--]);
-	sFormula.Replace("ypos()", Vars[n--]);
-	sFormula.Replace("row()" , Vars[n--]);
-	sFormula.Replace("col()" , Vars[n--]);
+	if( bPosition[3] )	sFormula.Replace("ypos()", Vars[--n]);
+	if( bPosition[2] )	sFormula.Replace("xpos()", Vars[--n]);
+	if( bPosition[1] )	sFormula.Replace("row()" , Vars[--n]);
+	if( bPosition[0] )	sFormula.Replace("col()" , Vars[--n]);
 
-	for(i=nXGrids; i>0; i--, n--)
+	for(i=nXGrids; i>0 && n>0; i--)
 	{
-		sFormula.Replace(CSG_String::Format(SG_T("h%d"), i), Vars[n]);
+		sFormula.Replace(CSG_String::Format(SG_T("h%d"), i), Vars[--n]);
 	}
 
-	for(i=nGrids; i>0; i--, n--)
+	for(i=nGrids ; i>0 && n>0; i--)
 	{
-		sFormula.Replace(CSG_String::Format(SG_T("g%d"), i), Vars[n]);
+		sFormula.Replace(CSG_String::Format(SG_T("g%d"), i), Vars[--n]);
 	}
 
 	//-----------------------------------------------------
@@ -355,7 +380,7 @@ bool CGrid_Calculator::Get_Formula(CSG_Formula &Formula, CSG_String sFormula, in
 
 		if( !Formula.Get_Error(Message) )
 		{
-			Message.Printf(SG_T("%s: %s"), _TL("error in formula"), sFormula.c_str());
+			Message.Printf("%s: %s", _TL("error in formula"), sFormula.c_str());
 		}
 
 		Error_Set(Message);
@@ -363,6 +388,22 @@ bool CGrid_Calculator::Get_Formula(CSG_Formula &Formula, CSG_String sFormula, in
 		return( false );
 	}
 
+	//-----------------------------------------------------
+	CSG_String	sUsed(Formula.Get_Used_Variables());
+
+	if( nValues < sUsed.Length() )
+	{
+		Error_Fmt("%s (%d < %d)", _TL("The number of supplied grids is less than the number of variables in formula."), nValues, sUsed.Length());
+
+		return( false );
+	}
+
+	if( nValues > sUsed.Length() )
+	{
+		Message_Add(CSG_String::Format("%s: %s (%d > %d)", _TL("Warning"), _TL("The number of supplied grids exceeds the number of variables in formula."), nValues, sUsed.Length()));
+	}
+
+	//-----------------------------------------------------
 	return( true );
 }
 
