@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: Grid_Calculator.h 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: Grid_Calculator.h 2467 2015-04-15 16:55:37Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -84,13 +84,14 @@ public:
 protected:
 
 	virtual int		On_Parameter_Changed	(CSG_Parameters *pParameters, CSG_Parameter *pParameter);
+	virtual int		On_Parameters_Enable	(CSG_Parameters *pParameters, CSG_Parameter *pParameter);
 
 	virtual bool	On_Execute				(void);
 
 
 private:
 
-	bool			Get_Formula				(CSG_Formula &Formula, CSG_String sFormula, int nGrids, int nXGrids);
+	bool			Get_Formula				(CSG_Formula &Formula, CSG_String sFormula, int nGrids, int nXGrids, bool bPosition[4]);
 
 };
 
