@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: MLB_Interface.cpp 1937 2014-01-11 18:22:42Z reklov_w $
+ * Version $Id: MLB_Interface.cpp 2281 2014-10-09 15:49:41Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -74,7 +74,10 @@ CSG_String Get_Info(int i)
 	switch( i )
 	{
 	case MLB_INFO_Name:	default:
-		return( _TL("Grid - Calculus") );
+		return( _TL("Calculus") );
+
+	case MLB_INFO_Category:
+		return( _TL("Grid") );
 
 	case MLB_INFO_Author:
 		return( SG_T("O. Conrad, A. Ringeler, V. Olaya, J. Engels (c) 2001-2014") );
