@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: MLB_Interface.cpp 1246 2011-11-25 13:42:38Z oconrad $
+ * Version $Id: MLB_Interface.cpp 2281 2014-10-09 15:49:41Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -17,7 +17,10 @@ CSG_String Get_Info(int i)
 	switch( i )
 	{
 	case MLB_INFO_Name:	default:
-		return( _TL("Grid - Calculus BSL") );
+		return( _TL("Calculus BSL") );
+
+	case MLB_INFO_Category:
+		return( _TL("Grid") );
 
 	case MLB_INFO_Author:
 		return( SG_T("SAGA User Group Associaton (c) 2009") );
