@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: funktion.cpp 911 2011-02-14 16:38:15Z reklov_w $
+ * Version $Id: funktion.cpp 2210 2014-08-28 14:00:19Z oconrad $
  *********************************************************/
 
 //#include <..\stdafx.h>
@@ -587,10 +587,10 @@ public:
 			G->Set_Value(x,y, G->asDouble(x,y-1));
 
 		// die vier Randpunkte setzen
-		G->Set_Value(        0,        0, G->asDouble(        1,        1, false));
-		G->Set_Value(G->xanz-1,        0, G->asDouble(G->xanz-2,        1, false));
-		G->Set_Value(        0,G->yanz-1, G->asDouble(        1,G->yanz-2, false));
-		G->Set_Value(G->xanz-1,G->yanz-1, G->asDouble(G->xanz-2,G->yanz-2, false));
+		G->Set_Value(        0,        0, G->asDouble(        1,        1,true),true);
+		G->Set_Value(G->xanz-1,        0, G->asDouble(G->xanz-2,        1,true),true);
+		G->Set_Value(        0,G->yanz-1, G->asDouble(        1,G->yanz-2,true),true);
+		G->Set_Value(G->xanz-1,G->yanz-1, G->asDouble(G->xanz-2,G->yanz-2,true),true);
 	};
 };
 
