@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: Filter_Multi_Dir_Lee.cpp 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: Filter_Multi_Dir_Lee.cpp 2208 2014-08-27 16:00:51Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -411,7 +411,7 @@ bool CFilter_Multi_Dir_Lee::Get_Filter(bool bWeighted, bool bAbsolute)
 	//-----------------------------------------------------
 	if( !bAbsolute )
 	{
-		Noise	= Parameters("NOISE_REL")->asDouble() * m_pStdDev->Get_ArithMean();
+		Noise	= Parameters("NOISE_REL")->asDouble() * m_pStdDev->Get_Mean();
 		Noise2	= Noise*Noise;
 
 		for(y=0; y<Get_NY() && Set_Progress(y); y++)
