@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: MLB_Interface.cpp 2281 2014-10-09 15:49:41Z oconrad $
+ * Version $Id: MLB_Interface.cpp 2622 2015-09-30 13:17:47Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -113,6 +113,7 @@ CSG_String Get_Info(int i)
 #include "bin_erosion_reconst.h"
 #include "connect_analysis.h"
 #include "Filter_Sieve.h"
+#include "wombling.h"
 
 
 //---------------------------------------------------------
@@ -138,8 +139,10 @@ CSG_Module *		Create_Module(int i)
 	case 13:	return( new Cbin_erosion_reconst );
 	case 14:	return( new Cconnectivity_analysis );
 	case 15:	return( new CFilter_Sieve );
+	case 16:	return( new CWombling );
+	case 17:	return( new CWombling_MultiFeature );
 
-	case 16:	return( NULL );
+	case 18:	return( NULL );
 	default:	return( MLB_INTERFACE_SKIP_MODULE );
 	}
 }
