@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: Interpolation.cpp 2221 2014-09-04 16:02:43Z oconrad $
+ * Version $Id: Interpolation.cpp 2569 2015-09-02 10:42:56Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -84,7 +84,7 @@ CInterpolation::CInterpolation(void)
 		_TL("")
 	);
 
-	m_Grid_Target.Create(SG_UI_Get_Window_Main() ? &Parameters : Add_Parameters("TARGET", _TL("Target System"), _TL("")));
+	m_Grid_Target.Create(&Parameters, true, NULL, "TARGET_");
 }
 
 
@@ -97,9 +97,9 @@ CInterpolation::CInterpolation(void)
 //---------------------------------------------------------
 int CInterpolation::On_Parameter_Changed(CSG_Parameters *pParameters, CSG_Parameter *pParameter)
 {
-	if( !SG_STR_CMP(pParameter->Get_Identifier(), "SHAPES") && pParameter->asShapes() )
+	if( !SG_STR_CMP(pParameter->Get_Identifier(), "SHAPES") )
 	{
-		m_Grid_Target.Set_User_Defined(pParameters, pParameter->asShapes()->Get_Extent());
+		m_Grid_Target.Set_User_Defined(pParameters, pParameter->asShapes());
 	}
 
 	return( m_Grid_Target.On_Parameter_Changed(pParameters, pParameter) ? 1 : 0 );
@@ -128,7 +128,7 @@ bool CInterpolation::On_Execute(void)
 	//-----------------------------------------------------
 	bool	bResult	= false;
 
-	m_Grid_Target.Set_User_Defined(Get_Parameters("TARGET"), m_pShapes->Get_Extent());	Dlg_Parameters("TARGET");	// if called from saga_cmd
+	m_Grid_Target.Cmd_Update(m_pShapes);	// if called from saga_cmd
 
 	if( (m_pGrid = m_Grid_Target.Get_Grid()) != NULL )
 	{
@@ -138,8 +138,6 @@ bool CInterpolation::On_Execute(void)
 	}
 
 	//-----------------------------------------------------
-	m_Search.Destroy();
-
 	if( m_pShapes != Parameters("SHAPES")->asShapes() )
 	{
 		delete(m_pShapes);
@@ -232,12 +230,6 @@ CSG_Shapes * CInterpolation::Get_Points(bool bOnlyNonPoints)
 	return( m_pShapes );
 }
 
-//---------------------------------------------------------
-bool CInterpolation::Set_Search_Engine(void)
-{
-	return( m_Search.Create(m_pShapes, m_zField) );
-}
-
 
 ///////////////////////////////////////////////////////////
 //														 //
