@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: Interpolation_NaturalNeighbour.cpp 2064 2014-03-21 13:20:57Z oconrad $
+ * Version $Id: Interpolation_NaturalNeighbour.cpp 2447 2015-03-19 14:43:42Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -151,7 +151,7 @@ bool CInterpolation_NaturalNeighbour::Interpolate(void)
 
 	//-----------------------------------------------------
 	Process_Set_Text(_TL("creating interpolator"));
-	nnai	*pNN	= nnai_build(pTIN, m_pGrid->Get_NCells(), xDst, yDst);
+	nnai	*pNN	= nnai_build(pTIN, (long)m_pGrid->Get_NCells(), xDst, yDst);
 
     Process_Set_Text(_TL("interpolating"));
     nnai_interpolate(pNN, zSrc, zDst);
