@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: Interpolation_NearestNeighbour.h 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: Interpolation_NearestNeighbour.h 2381 2015-01-30 16:37:39Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -90,9 +90,15 @@ public:
 
 protected:
 
-	virtual bool			On_Initialize		(void);
+	virtual bool				On_Initialize		(void);
+	virtual bool				On_Finalize			(void);
 
-	virtual bool			Get_Value			(double x, double y, double &z);
+	virtual bool				Get_Value			(double x, double y, double &z);
+
+
+private:
+
+	CSG_PRQuadTree				m_Search;
 
 };
 
