@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: MLB_Interface.cpp 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: MLB_Interface.cpp 2281 2014-10-09 15:49:41Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -74,7 +74,10 @@ CSG_String Get_Info(int i)
 	switch( i )
 	{
 	case MLB_INFO_Name:	default:
-		return( _TL("Grid - Gridding") );
+		return( _TL("Gridding") );
+
+	case MLB_INFO_Category:
+		return( _TL("Grid") );
 
 	case MLB_INFO_Author:
 		return( SG_T("O. Conrad (c) 2002-10") );
