@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: Gridding_Spline_Base.h 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: Gridding_Spline_Base.h 2221 2014-09-04 16:02:43Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -94,7 +94,9 @@ protected:
 
 	CSG_Grid					*m_pGrid;
 
+
 	virtual int					On_Parameter_Changed	(CSG_Parameters *pParameters, CSG_Parameter *pParameter);
+	virtual int					On_Parameters_Enable	(CSG_Parameters *pParameters, CSG_Parameter *pParameter);
 
 
 	bool						Initialise				(void);
@@ -105,8 +107,6 @@ protected:
 
 private:
 
-	bool						m_bGridPoints;
-
 	CSG_Parameters_Grid_Target	m_Grid_Target;
 
 
