@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: Gridding_Spline_TPS_TIN.h 911 2011-02-14 16:38:15Z reklov_w $
+ * Version $Id: Gridding_Spline_TPS_TIN.h 2388 2015-02-01 22:28:39Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -35,7 +35,7 @@
 // You should have received a copy of the GNU General    //
 // Public License along with this program; if not,       //
 // write to the Free Software Foundation, Inc.,          //
-// 59 Temple Place - Suite 330, Boston, MA 02111-1307,   //
+// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
 // USA.                                                  //
 //                                                       //
 //-------------------------------------------------------//
@@ -72,7 +72,7 @@
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-#include "Gridding_Spline_TPS_Global.h"
+#include "Gridding_Spline_Base.h"
 
 
 ///////////////////////////////////////////////////////////
@@ -82,11 +82,10 @@
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-class grid_spline_EXPORT CGridding_Spline_TPS_TIN : public CGridding_Spline_TPS_Global
+class CGridding_Spline_TPS_TIN : public CGridding_Spline_Base
 {
 public:
 	CGridding_Spline_TPS_TIN(void);
-	virtual ~CGridding_Spline_TPS_TIN(void);
 
 
 protected:
@@ -96,6 +95,8 @@ protected:
 
 private:
 
+	double					m_Regularisation;
+
 	int						m_nPoints, m_nPoints_Buf, m_Level;
 
 	CSG_TIN_Node			**m_Points;
@@ -105,7 +106,7 @@ private:
 	bool					_Finalise		(void);
 
 	void					_Set_Triangle	(CSG_TIN_Triangle *pTriangle);
-	void					_Set_Grid		(CSG_TIN_Triangle *pTriangle);
+	void					_Set_Grid		(CSG_TIN_Triangle *pTriangle, CSG_Thin_Plate_Spline &Spline);
 
 	void					_Add_Points		(CSG_TIN_Node *Point, int Level);
 	bool					_Add_Point		(CSG_TIN_Node *Point);
