@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: MLB_Interface.cpp 2281 2014-10-09 15:49:41Z oconrad $
+ * Version $Id: MLB_Interface.cpp 2388 2015-02-01 22:28:39Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -100,7 +100,6 @@ CSG_String Get_Info(int i)
 //---------------------------------------------------------
 // 3. Include the headers of your modules here...
 
-#include "Gridding_Spline_TPS_Global.h"
 #include "Gridding_Spline_TPS_Local.h"
 #include "Gridding_Spline_TPS_TIN.h"
 #include "Gridding_Spline_BA.h"
@@ -108,6 +107,8 @@ CSG_String Get_Info(int i)
 #include "Gridding_Spline_MBA_Grid.h"
 #include "Gridding_Spline_CSA.h"
 
+#include "MBASpline_for_Categories.h"
+
 
 //---------------------------------------------------------
 // 4. Allow your modules to be created here...
@@ -116,16 +117,18 @@ CSG_Module *		Create_Module(int i)
 {
 	switch( i )
 	{
-	case 0:		return( new CGridding_Spline_TPS_Global );
 	case 1:		return( new CGridding_Spline_TPS_Local );
 	case 2:		return( new CGridding_Spline_TPS_TIN );
 	case 3:		return( new CGridding_Spline_BA );
 	case 4:		return( new CGridding_Spline_MBA );
 	case 5:		return( new CGridding_Spline_MBA_Grid );
 	case 6:		return( new CGridding_Spline_CSA );
-	}
 
-	return( NULL );
+	case 7:		return( new CMBASpline_for_Categories );
+
+	case 10:	return( NULL );
+	default:	return( MLB_INTERFACE_SKIP_MODULE );
+	}
 }
 
 
