@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: Grid_Value_Reclassify.cpp 1984 2014-02-11 16:45:09Z reklov_w $
+ * Version $Id: Grid_Value_Reclassify.cpp 2321 2014-11-03 21:10:22Z johanvdw $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -84,13 +84,13 @@ CGrid_Value_Reclassify::CGrid_Value_Reclassify(void)
 		"(a) reclassification of single values\n"
 		"(b) reclassification of a range of values\n"
 		"(c) reclassification of value ranges specified in a lookup table\n\n"
-		"In addition to theses methods, two special cases (NoData values and values not included in the "
+		"In addition to these methods, two special cases (NoData values and values not included in the "
 		"reclassification setup) are supported.\n"
 		"With reclassification mode (a) and (b), the 'NoData option' is evaluated before the 'Method' "
 		"settings. In reclassification mode (c) the option is evaluated only if the NoData value is not "
 		"included in the lookup table.\n"
 		"The 'other values' option is always evaluated after checking the 'Method' settings.\n\n"
-		"The module allows to define the NoData value of the output grid (header): by default, the "
+		"The module allows one to define the NoData value of the output grid (header): by default, the "
 		"output grid gets assigned the NoData value of the input grid. But it is also possible to "
 		"assign a user defined NoData value.\n\n"
 	));
