@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: Grid_Value_Request.cpp 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: Grid_Value_Request.cpp 2318 2014-10-31 12:10:58Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -224,7 +224,7 @@ bool CGrid_Value_Request::On_Execute_Position(CSG_Point ptWorld, TSG_Module_Inte
 
 				for(iGrid=0; iGrid<m_pGrids->Get_Count(); iGrid++)
 				{
-					if( m_pGrids->asGrid(iGrid)->Get_Value(ptWorld, Value, m_Interpolation, true) )
+					if( m_pGrids->asGrid(iGrid)->Get_Value(ptWorld, Value, m_Interpolation, false, true) )
 					{
 						m_pTable->Get_Record(iGrid + FIELD_GRIDS)->Set_Value(FIELD_VALUE, Value);
 					}
@@ -254,7 +254,7 @@ bool CGrid_Value_Request::On_Execute_Position(CSG_Point ptWorld, TSG_Module_Inte
 
 				for(iGrid=0; iGrid<m_pGrids->Get_Count(); iGrid++)
 				{
-					if( m_pGrids->asGrid(iGrid)->Get_Value(ptWorld, Value, m_Interpolation, true) )
+					if( m_pGrids->asGrid(iGrid)->Get_Value(ptWorld, Value, m_Interpolation, false, true) )
 					{
 						pRecord->Set_Value(FIELD_GRIDS + iGrid, Value);
 					}
