@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: Grid_Value_Request.h 1246 2011-11-25 13:42:38Z oconrad $
+ * Version $Id: Grid_Value_Request.h 1921 2014-01-09 10:24:11Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -35,7 +35,7 @@
 // You should have received a copy of the GNU General    //
 // Public License along with this program; if not,       //
 // write to the Free Software Foundation, Inc.,          //
-// 59 Temple Place - Suite 330, Boston, MA 02111-1307,   //
+// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
 // USA.                                                  //
 //                                                       //
 //-------------------------------------------------------//
@@ -81,12 +81,13 @@ public:
 	CGrid_Value_Request(void);
 	virtual ~CGrid_Value_Request(void);
 
-	virtual CSG_String		Get_MenuPath		(void)	{	return( _TL("R:Values") );	}
+	virtual CSG_String		Get_MenuPath		(void)	{	return( _TL("A:Grid|Values") );	}
 
 
 protected:
 
 	virtual bool			On_Execute			(void);
+
 	virtual bool			On_Execute_Position	(CSG_Point ptWorld, TSG_Module_Interactive_Mode Mode);
 
 
@@ -94,7 +95,7 @@ private:
 
 	int						m_Method, m_Interpolation;
 
-	CSG_Table					*m_pTable;
+	CSG_Table				*m_pTable;
 
 	CSG_Parameter_Grid_List	*m_pGrids;
 
