@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: MLB_Interface.cpp 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: MLB_Interface.cpp 2529 2015-07-01 07:06:33Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -74,19 +74,22 @@ CSG_String Get_Info(int i)
 	switch( i )
 	{
 	case MLB_INFO_Name:	default:
-		return( _TL("Grid - Visualisation") );
+		return( _TL("Grids") );
+
+	case MLB_INFO_Category:
+		return( _TL("Visualization") );
 
 	case MLB_INFO_Author:
-		return( SG_T("O. Conrad, V. Wichmann (c) 2002-2012") );
+		return( SG_T("O. Conrad, V. Wichmann (c) 2002-2014") );
 
 	case MLB_INFO_Description:
-		return( _TL("Special visualisation functions for grids.") );
+		return( _TL("Visualization tools for grids.") );
 
 	case MLB_INFO_Version:
 		return( SG_T("1.0") );
 
 	case MLB_INFO_Menu_Path:
-		return( _TL("Grid|Visualisation") );
+		return( _TL("Visualization|Grid") );
 	}
 }
 
@@ -102,6 +105,9 @@ CSG_String Get_Info(int i)
 #include "Grid_Color_Triangle.h"
 #include "Grid_Histogram_Surface.h"
 #include "Grid_Aspect_Slope_Map.h"
+#include "Grid_Terrain_Map.h"
+#include "Grid_LUT_Assign.h"
+
 
 //---------------------------------------------------------
 // 4. Allow your modules to be created here...
@@ -110,14 +116,20 @@ CSG_Module *		Create_Module(int i)
 {
 	switch( i )
 	{
-	case 0:		return( new CGrid_Color_Rotate );
-	case 1:		return( new CGrid_Color_Blend );
-	case 2:		return( new CGrid_Colors_Fit );
-	case 3:		return( new CGrid_RGB_Composite );
-	case 4:		return( new CGrid_3D_Image );
-	case 5:		return( new CGrid_Color_Triangle );
-	case 6:		return( new CGrid_Histogram_Surface );
-	case 7:		return( new CGrid_Aspect_Slope_Map );
+	case  0:	return( new CGrid_Color_Rotate );
+	case  1:	return( new CGrid_Color_Blend );
+	case  2:	return( new CGrid_Colors_Fit );
+	case  3:	return( new CGrid_RGB_Composite );
+	case  4:	return( new CGrid_3D_Image );
+	case  5:	return( new CGrid_Color_Triangle );
+	case  6:	return( new CGrid_Histogram_Surface );
+	case  7:	return( new CGrid_Aspect_Slope_Map );
+	case  8:	return( new CGrid_Terrain_Map );
+	case  9:	return( new CGrid_RGB_Split );
+	case 10:	return( new CGrid_LUT_Assign );
+
+	case 11:	return( NULL );
+	default:	return( MLB_INTERFACE_SKIP_MODULE );
 	}
 
 	return( NULL );
