@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: Resection.cpp 1945 2014-01-16 10:00:22Z reklov_w $
+ * Version $Id: Resection.cpp 2134 2014-05-29 10:33:16Z reklov_w $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -402,9 +402,21 @@ bool CResection::On_Execute(void)
 		SG_Matrix_Eigen_Reduction(N, eigenVecs, eigenVals, true);
 
 		// One of the Eigen Values is 0
-		if (std::any_of(eigenVals.cbegin(),
-		                eigenVals.cend(),
-		                [] (double i) { return i == 0; })) {
+		// workaround the problem that the following snippet is C++11:
+		// if (std::any_of(eigenVals.cbegin(), eigenVals.cend(), [] (double i) { return i == 0; }))
+		bool	bValid = true;
+
+		for (int i=0; i < eigenVals.Get_Size(); i++)
+		{
+			if (eigenVals.Get_Data(i) == 0.0)
+			{
+				bValid = false;
+				break;
+			}
+		}
+
+		if (!bValid)
+		{
 			msg = "The Normal Matrix has a rank defect. Please measure more points.";
 			pTabStream->Write(msg + SG_T("\n"));
 			SG_UI_Msg_Add(msg, true);
