@@ -260,6 +260,8 @@ bool CSG_GDAL_DataSet::Open_Read(const CSG_String &File_Name)
 	//-----------------------------------------------------
 	double	Transform[6];
 
+	m_File_Name	= File_Name;
+
 	m_Access	= SG_GDAL_IO_READ;
 
 	m_NX		= m_pDataSet->GetRasterXSize();
@@ -283,8 +285,8 @@ bool CSG_GDAL_DataSet::Open_Read(const CSG_String &File_Name)
 	{
 		m_bTransform	= true;
 		m_Cellsize		= 1.0;
-		m_xMin			= 0.5;
-		m_yMin			= 0.5;
+		m_xMin			= 0.0;
+		m_yMin			= 0.0;
 	}
 
 	m_TF_A[0]		= Transform[0];
@@ -299,37 +301,38 @@ bool CSG_GDAL_DataSet::Open_Read(const CSG_String &File_Name)
 }
 
 //---------------------------------------------------------
-bool CSG_GDAL_DataSet::Open_Write(const CSG_String &File_Name, const CSG_String &Driver, const CSG_String &Options,TSG_Data_Type Type, int NBands, const CSG_Grid_System &System, const CSG_Projection &Projection)
+bool CSG_GDAL_DataSet::Open_Write(const CSG_String &File_Name, const CSG_String &Driver, const CSG_String &Options, TSG_Data_Type Type, int NBands, const CSG_Grid_System &System, const CSG_Projection &Projection)
 {
-	char		**pOptions 	= NULL;
-	char		**pTokens 	= NULL;	
-	GDALDriver	*pDriver;
-	
-	if (!Options.is_Empty()){
-	  pTokens = CSLTokenizeString2( Options, " ", CSLT_STRIPLEADSPACES);
-	  
-	  for( int i = 0; pTokens != NULL && pTokens[i] != NULL; i++ ){
-	    pOptions = CSLAddString( pOptions,  pTokens[i] );
-	  }
-	}
-	
 	Close();
-	
-	
 
 	//--------------------------------------------------------
+	char	**pOptions	= NULL;
+	
+	if( !Options.is_Empty() )
+	{
+		char	**pTokens	= CSLTokenizeString2(Options, " ", CSLT_STRIPLEADSPACES);
+	  
+		for(int i=0; pTokens && pTokens[i]; i++)
+		{
+			pOptions	= CSLAddString(pOptions, pTokens[i]);
+		}
+	}
+
+	//--------------------------------------------------------
+	GDALDriver	*pDriver;
+
 	if( (pDriver = gSG_GDAL_Drivers.Get_Driver(Driver)) == NULL )
 	{
 		SG_UI_Msg_Add_Error(CSG_String::Format(SG_T("%s: %s"), _TL("driver not found."), Driver.c_str()));
 
 		return( false );
 	}
-	
-	if( !GDALValidateCreationOptions (pDriver, pOptions))
+
+	if( !GDALValidateCreationOptions(pDriver, pOptions) )
 	{
-	  SG_UI_Msg_Add_Error(CSG_String::Format(SG_T("%s: %s"), _TL("Creation option(s) not supported by the driver"), Options.c_str() ));
-	  
-	  return false;
+		SG_UI_Msg_Add_Error(CSG_String::Format(SG_T("%s: %s"), _TL("Creation option(s) not supported by the driver"), Options.c_str()));
+
+		return( false );
 	}
 
 	if( CSLFetchBoolean(pDriver->GetMetadata(), GDAL_DCAP_CREATE, false) == false )
@@ -347,6 +350,8 @@ bool CSG_GDAL_DataSet::Open_Write(const CSG_String &File_Name, const CSG_String
 	}
 
 	//--------------------------------------------------------
+	m_File_Name	= File_Name;
+
 	m_Access	= SG_GDAL_IO_WRITE;
 
 	if( Projection.is_Okay() )
@@ -383,17 +388,19 @@ bool CSG_GDAL_DataSet::Close(void)
 		m_pDataSet	= NULL;
 	}
 
+	m_File_Name.Clear();
+
 	m_Access	= SG_GDAL_IO_CLOSED;
 
-	
-	if (strlen(CPLGetLastErrorMsg()) > 3)
+	if( strlen(CPLGetLastErrorMsg()) > 3 )
 	{
-	      SG_UI_Msg_Add_Error(CSG_String::Format(SG_T("%s: %s"),_TL("Dataset creation failed") , SG_STR_MBTOSG(CPLGetLastErrorMsg())));
-	      CPLErrorReset();
-	      
-	      return false;
+		SG_UI_Msg_Add_Error(CSG_String::Format(SG_T("%s: %s"), _TL("Dataset creation failed"), SG_STR_MBTOSG(CPLGetLastErrorMsg())));
+
+		CPLErrorReset();
+
+		return( false );
 	}
-	
+
 	return( true );
 }
 
@@ -432,6 +439,12 @@ CSG_String CSG_GDAL_DataSet::Get_Description(void)	const
 	return( m_pDataSet ? m_pDataSet->GetDescription() : "" );
 }
 
+//---------------------------------------------------------
+CSG_String CSG_GDAL_DataSet::Get_File_Name(void)	const
+{
+	return( m_File_Name );
+}
+
 //---------------------------------------------------------
 const char * CSG_GDAL_DataSet::Get_MetaData_Item(const char *pszName, const char *pszDomain)	const
 {
@@ -668,19 +681,8 @@ CSG_Grid * CSG_GDAL_DataSet::Read(int i)
 	}
 
 	//-------------------------------------------------
-	int		bSuccess;
-
-	double	zScale	= pBand->GetScale (&bSuccess);	if( !bSuccess || !zScale )	zScale	= 1.0;
-	double	zMin	= pBand->GetOffset(&bSuccess);	if( !bSuccess            )	zMin	= 0.0;
-
 	TSG_Data_Type	Type	= gSG_GDAL_Drivers.Get_SAGA_Type(pBand->GetRasterDataType());
 
-	if( SG_Get_Significant_Decimals(zScale) > 0 && (Type != SG_DATATYPE_Float || Type != SG_DATATYPE_Double) )
-	{
-		Type	= SG_DATATYPE_Float;	// force to float, we will rescale data in any case!
-	}
-
-	//-------------------------------------------------
 	CSG_Grid	*pGrid	= SG_Create_Grid(Type, Get_NX(), Get_NY(), Get_Cellsize(), Get_xMin(), Get_yMin());
 
 	if( !pGrid )
@@ -689,9 +691,15 @@ CSG_Grid * CSG_GDAL_DataSet::Read(int i)
 	}
 
 	//-------------------------------------------------
+	int		bSuccess;
+
+	double	zScale	= pBand->GetScale (&bSuccess);	if( !bSuccess || !zScale )	zScale	= 1.0;
+	double	zOffset	= pBand->GetOffset(&bSuccess);	if( !bSuccess            )	zOffset	= 0.0;
+
 	pGrid->Set_Name			(Get_Name       (i));
 	pGrid->Set_Description	(Get_Description(i));
 	pGrid->Set_Unit			(CSG_String(pBand->GetUnitType()));
+	pGrid->Set_Scaling		(zScale, zOffset);
 
 	pBand->GetNoDataValue(&bSuccess);
 
@@ -716,7 +724,7 @@ CSG_Grid * CSG_GDAL_DataSet::Read(int i)
 			for(int x=0; x<Get_NX(); x++)
 			{
 			//	double	NaN	= 0.0;	NaN	= -1.0 / NaN;	if( NaN == zLine[x] )	pGrid->Set_NoData(x, yy); else
-				pGrid->Set_Value(x, yy, zMin + zScale * zLine[x]);
+				pGrid->Set_Value(x, yy, zLine[x], false);
 			}
 		}
 	}
@@ -763,7 +771,7 @@ bool CSG_GDAL_DataSet::Write(int i, CSG_Grid *pGrid, double noDataValue)
 
 	//-----------------------------------------------------
 	pBand->SetNoDataValue	(noDataValue);
-	pBand->SetStatistics	(pGrid->Get_ZMin(), pGrid->Get_ZMax(), pGrid->Get_ArithMean(), pGrid->Get_StdDev());
+	pBand->SetStatistics	(pGrid->Get_ZMin(), pGrid->Get_ZMax(), pGrid->Get_Mean(), pGrid->Get_StdDev());
 
 	return( true );	
 }
@@ -775,6 +783,177 @@ bool CSG_GDAL_DataSet::Write(int i, CSG_Grid *pGrid)
 }
 
 
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+CSG_Rect CSG_GDAL_DataSet::Get_Extent(bool bTransform)	const
+{
+	if( bTransform && Needs_Transformation() )
+	{
+		CSG_Grid_System	System;
+
+		if( Get_Transformation(System, false) )
+		{
+			return( System.Get_Extent() );
+		}
+	}
+
+	return( CSG_Rect(Get_xMin(), Get_yMin(), Get_xMax(), Get_yMax()) );
+}
+
+//---------------------------------------------------------
+CSG_Grid_System CSG_GDAL_DataSet::Get_System(void)	const
+{
+	CSG_Grid_System	System;
+
+	if( !Needs_Transformation() || !Get_Transformation(System, false) )
+	{
+		System.Assign(Get_Cellsize(), Get_xMin(), Get_yMin(), Get_NX(), Get_NY());
+	}
+
+	return( System );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CSG_GDAL_DataSet::Get_Transformation(CSG_Grid_System &System, bool bVerbose) const
+{
+	CSG_Vector	A;
+	CSG_Matrix	B;
+
+	Get_Transformation(A, B);
+
+	//-----------------------------------------------------
+	if( Needs_Transformation() )
+	{
+		CSG_Vector	v(2);
+		CSG_Rect	r;
+
+		v[0]	= Get_xMin() + 0.5;	v[1]	= Get_yMin() + 0.5;	v	= B * v + A;	r.Assign(v[0], v[1], v[0], v[1]);
+		v[0]	= Get_xMin() + 0.5;	v[1]	= Get_yMax() - 0.5;	v	= B * v + A;	r.Union(CSG_Point(v[0], v[1]));
+		v[0]	= Get_xMax() - 0.5;	v[1]	= Get_yMax() - 0.5;	v	= B * v + A;	r.Union(CSG_Point(v[0], v[1]));
+		v[0]	= Get_xMax() - 0.5;	v[1]	= Get_yMin() + 0.5;	v	= B * v + A;	r.Union(CSG_Point(v[0], v[1]));
+
+		v[0]	= 1;	v[1] = 0;	v = B * v;	double	dx	= v.Get_Length();
+		v[0]	= 0;	v[1] = 1;	v = B * v;	double	dy	= v.Get_Length();
+
+		if( dx != dy )
+		{
+			if( bVerbose )
+			{
+				SG_UI_Msg_Add_Execution(CSG_String::Format("\n%s: %s\n\t%s: %f",
+					_TL("warning"), _TL("top-to-bottom and left-to-right cell sizes differ."), _TL("Difference"), fabs(dy - dx)), false
+				);
+			}
+
+			if( dx > dy )
+			{
+				dx	= dy;
+			}
+
+			if( bVerbose )
+			{
+				SG_UI_Msg_Add_Execution(CSG_String::Format("\n\t%s: %f\n", _TL("using cellsize"), dx), false);
+			}
+		}
+
+		return( System.Assign(dx, r) );
+	}
+
+	//-----------------------------------------------------
+	return( false );
+}
+
+//---------------------------------------------------------
+bool CSG_GDAL_DataSet::Get_Transformation(CSG_Grid **ppGrid, TSG_Grid_Interpolation	Interpolation, bool bVerbose)	const
+{
+	CSG_Grid_System	System;
+
+	if( Get_Transformation(System, bVerbose) )
+	{
+		return( Get_Transformation(ppGrid, Interpolation, System, bVerbose) );
+	}
+
+	return( false );
+}
+
+//---------------------------------------------------------
+bool CSG_GDAL_DataSet::Get_Transformation(CSG_Grid **ppGrid, TSG_Grid_Interpolation	Interpolation, const CSG_Grid_System &System, bool bVerbose)	const
+{
+	if( !System.is_Valid() )
+	{
+		return( false );
+	}
+
+	//-----------------------------------------------------
+	CSG_Vector	A;
+	CSG_Matrix	B, BInv;
+
+	Get_Transformation(A, B);
+
+	BInv	= B.Get_Inverse();
+
+	//-----------------------------------------------------
+	CSG_Grid	*pImage	= *ppGrid;
+	CSG_Grid	*pWorld	= SG_Create_Grid(System, pImage->Get_Type());
+
+	if( !pWorld )
+	{
+		return( false );
+	}
+
+	*ppGrid	= pWorld;
+
+	pWorld->Set_Name              (pImage->Get_Name        ());
+	pWorld->Set_Description       (pImage->Get_Description ());
+	pWorld->Set_Unit              (pImage->Get_Unit        ());
+	pWorld->Set_Scaling           (pImage->Get_Scaling     (), pImage->Get_Offset());
+	pWorld->Set_NoData_Value_Range(pImage->Get_NoData_Value(), pImage->Get_NoData_hiValue());
+	pWorld->Get_MetaData()	     = pImage->Get_MetaData    ();
+	pWorld->Get_Projection()     = pImage->Get_Projection  ();
+
+	//-----------------------------------------------------
+//	#pragma omp parallel for
+//	for(int y=0; y<pWorld->Get_NY(); y++)
+//	{
+//		Process_Get_Okay();
+
+	for(int y=0; y<pWorld->Get_NY() && SG_UI_Process_Set_Progress(y, pWorld->Get_NY()); y++)
+	{
+		#pragma omp parallel for
+		for(int x=0; x<pWorld->Get_NX(); x++)
+		{
+			double		z;
+			CSG_Vector	vWorld(2), vImage;
+
+			vWorld[0]	= pWorld->Get_XMin() + (x - 0.5) * pWorld->Get_Cellsize();
+			vWorld[1]	= pWorld->Get_YMin() + (y + 0.5) * pWorld->Get_Cellsize();
+
+			vImage	= BInv * (vWorld - A);
+
+			if( pImage->Get_Value(vImage[0], vImage[1], z, Interpolation, false, true) )
+			{
+				pWorld->Set_Value(x, y, z);
+			}
+			else
+			{
+				pWorld->Set_NoData(x, y);
+			}
+		}
+	}
+
+	delete(pImage);
+
+	return( true );
+}
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 //														 //
