@@ -72,9 +72,9 @@ CGDAL_Import::CGDAL_Import(void)
 	CSG_Parameter	*pNode;
 
 	//-----------------------------------------------------
-	Set_Name	(_TL("GDAL: Import Raster"));
+	Set_Name	(_TL("Import Raster"));
 
-	Set_Author	(SG_T("O.Conrad (c) 2007 (A.Ringeler)"));
+	Set_Author	("O.Conrad (c) 2007 (A.Ringeler)");
 
 	CSG_String	Description;
 
@@ -86,7 +86,7 @@ CGDAL_Import::CGDAL_Import(void)
 		"  http://www.gdal.org</a>\n"
 	);
 
-	Description	+= CSG_String::Format(SG_T("\nGDAL %s:%s\n\n"), _TL("Version"), SG_Get_GDAL_Drivers().Get_Version().c_str());
+	Description	+= CSG_String::Format("\nGDAL %s:%s\n\n", _TL("Version"), SG_Get_GDAL_Drivers().Get_Version().c_str());
 
 	Description	+= _TW(
 		"Following raster formats are currently supported:\n"
@@ -95,13 +95,13 @@ CGDAL_Import::CGDAL_Import(void)
 
 	for(int i=0; i<SG_Get_GDAL_Drivers().Get_Count(); i++)
     {
-		Description	+= CSG_String::Format(SG_T("<tr><td>%s</td><td>%s</td></tr>\n"),
+		Description	+= CSG_String::Format("<tr><td>%s</td><td>%s</td></tr>\n",
 			SG_Get_GDAL_Drivers().Get_Description(i).c_str(),
 			SG_Get_GDAL_Drivers().Get_Name       (i).c_str()
 		);
     }
 
-	Description	+= SG_T("</table>");
+	Description	+= "</table>";
 
 	Set_Description(Description);
 
@@ -119,14 +119,11 @@ CGDAL_Import::CGDAL_Import(void)
 	);
 
 	//-----------------------------------------------------
-	if( SG_UI_Get_Window_Main() )
-	{
-		Parameters.Add_Value(
-			NULL	, "SELECT"		, _TL("Select from Multiple Bands"),
-			_TL(""),
-			PARAMETER_TYPE_Bool, true
-		);
-	}
+	Parameters.Add_Value(
+		NULL	, "SELECT"		, _TL("Select from Multiple Bands"),
+		_TL(""),
+		PARAMETER_TYPE_Bool, true
+	)->Set_UseInCMD(false);
 
 	//-----------------------------------------------------
 	pNode	= Parameters.Add_Value(
@@ -138,7 +135,7 @@ CGDAL_Import::CGDAL_Import(void)
 	Parameters.Add_Choice(
 		pNode	, "INTERPOL"	, _TL("Interpolation"),
 		_TL("interpolation method to use if grid needs to be aligned to coordinate system"),
-		CSG_String::Format(SG_T("%s|%s|%s|%s|%s|"),
+		CSG_String::Format("%s|%s|%s|%s|%s|",
 			_TL("Nearest Neighbor"),
 			_TL("Bilinear Interpolation"),
 			_TL("Inverse Distance Interpolation"),
@@ -161,13 +158,13 @@ CGDAL_Import::CGDAL_Import(void)
 //---------------------------------------------------------
 int CGDAL_Import::On_Parameters_Enable(CSG_Parameters *pParameters, CSG_Parameter *pParameter)
 {
-	if(	!SG_STR_CMP(pParameter->Get_Identifier(), SG_T("TRANSFORM")) )
+	if(	!SG_STR_CMP(pParameter->Get_Identifier(), "TRANSFORM") )
 	{
 		pParameters->Get_Parameter("INTERPOL")->Set_Enabled(pParameter->asBool());
 	}
 
-	if( !SG_STR_CMP(pParameters->Get_Identifier(), SG_T("SELECTION"))
-	&&  !SG_STR_CMP(pParameter ->Get_Identifier(), SG_T("ALL")) && pParameters->Get_Parameter("BANDS") )
+	if( !SG_STR_CMP(pParameters->Get_Identifier(), "SELECTION")
+	&&  !SG_STR_CMP(pParameter ->Get_Identifier(), "ALL") && pParameters->Get_Parameter("BANDS") )
 	{
 		pParameters->Get_Parameter("BANDS")->Set_Enabled(!pParameter->asBool());
 	}
@@ -200,7 +197,7 @@ bool CGDAL_Import::On_Execute(void)
 
 	for(int i=0; i<Files.Get_Count(); i++)
 	{
-		Message_Add(CSG_String::Format(SG_T("\n%s: %s"), _TL("loading"), Files[i].c_str()), false);
+		Message_Add(CSG_String::Format("\n%s: %s", _TL("loading"), Files[i].c_str()), false);
 
 		if( DataSet.Open_Read(Files[i]) == false )
 		{
@@ -236,14 +233,14 @@ bool CGDAL_Import::Load_Sub(CSG_GDAL_DataSet &DataSet)
 
 	for(i=0, n=0; i==n; i++)
 	{
-		CSG_MetaData	*pName	= MetaData.Get_Child(CSG_String::Format(SG_T("SUBDATASET_%d_NAME"), i + 1));
-		CSG_MetaData	*pDesc	= MetaData.Get_Child(CSG_String::Format(SG_T("SUBDATASET_%d_DESC"), i + 1));
+		CSG_MetaData	*pName	= MetaData.Get_Child(CSG_String::Format("SUBDATASET_%d_NAME", i + 1));
+		CSG_MetaData	*pDesc	= MetaData.Get_Child(CSG_String::Format("SUBDATASET_%d_DESC", i + 1));
 
 		if( pName )
 		{
 			n++;
 
-			Message_Add(CSG_String::Format(SG_T("\n%s"), pName->Get_Content().c_str()), false);
+			Message_Add(CSG_String::Format("\n%s", pName->Get_Content().c_str()), false);
 
 			P.Add_Value(NULL, pName->Get_Content(), pDesc ? pDesc->Get_Content().c_str() : _TL("unnamed"), SG_T(""), PARAMETER_TYPE_Bool, SG_UI_Get_Window_Main() == NULL);
 		}
@@ -290,19 +287,18 @@ bool CGDAL_Import::Load(CSG_GDAL_DataSet &DataSet, const CSG_String &Name)
 	CSG_Vector	A;
 	CSG_Matrix	B;
 
-	DataSet.Get_Transform(A, B);
+	DataSet.Get_Transformation(A, B);
 
-	//-----------------------------------------------------
-	Message_Add(SG_T("\n"), false);
-	Message_Add(CSG_String::Format(SG_T("\n%s: %s"), _TL("Driver" ), DataSet.Get_DriverID().c_str()   ), false);
-	Message_Add(CSG_String::Format(SG_T("\n%s: %d"), _TL("Bands"  ), DataSet.Get_Count()              ), false);
-	Message_Add(CSG_String::Format(SG_T("\n%s: %d"), _TL("Rows"   ), DataSet.Get_NX()                 ), false);
-	Message_Add(CSG_String::Format(SG_T("\n%s: %d"), _TL("Columns"), DataSet.Get_NY()                 ), false);
-	Message_Add(SG_T("\n"), false);
-	Message_Add(CSG_String::Format(SG_T("\n%s:"), _TL("Transformation")                               ), false);
-	Message_Add(CSG_String::Format(SG_T("\n  x' = %.6f + x * %.6f + y * %.6f"), A[0], B[0][0], B[0][1]), false);
-	Message_Add(CSG_String::Format(SG_T("\n  y' = %.6f + x * %.6f + y * %.6f"), A[1], B[1][0], B[1][1]), false);
-	Message_Add(SG_T("\n"), false);
+	Message_Add("\n", false);
+	Message_Add(CSG_String::Format("\n%s: %s", _TL("Driver" ), DataSet.Get_DriverID().c_str()   ), false);
+	Message_Add(CSG_String::Format("\n%s: %d", _TL("Bands"  ), DataSet.Get_Count()              ), false);
+	Message_Add(CSG_String::Format("\n%s: %d", _TL("Rows"   ), DataSet.Get_NX()                 ), false);
+	Message_Add(CSG_String::Format("\n%s: %d", _TL("Columns"), DataSet.Get_NY()                 ), false);
+	Message_Add("\n", false);
+	Message_Add(CSG_String::Format("\n%s:", _TL("Transformation")                               ), false);
+	Message_Add(CSG_String::Format("\n  x' = %.6f + x * %.6f + y * %.6f", A[0], B[0][0], B[0][1]), false);
+	Message_Add(CSG_String::Format("\n  y' = %.6f + x * %.6f + y * %.6f", A[1], B[1][0], B[1][1]), false);
+	Message_Add("\n", false);
 
 	//-----------------------------------------------------
 	int			i, n;
@@ -350,83 +346,6 @@ bool CGDAL_Import::Load(CSG_GDAL_DataSet &DataSet, const CSG_String &Name)
 		}
 	}
 
-	//-----------------------------------------------------
-	CSG_Grid_System	Transform;
-
-	if( Parameters("TRANSFORM")->asBool() && DataSet.Needs_Transform() )
-	{
-		double		s;
-		CSG_Vector	v(2);
-		CSG_Rect	r;
-
-		v[0]	= DataSet.Get_xMin();	v[1]	= DataSet.Get_yMin();	v	= B * v + A;	r.Assign(v[0], v[1], v[0], v[1]);
-		v[0]	= DataSet.Get_xMin();	v[1]	= DataSet.Get_yMax();	v	= B * v + A;	r.Union(CSG_Point(v[0], v[1]));
-		v[0]	= DataSet.Get_xMax();	v[1]	= DataSet.Get_yMax();	v	= B * v + A;	r.Union(CSG_Point(v[0], v[1]));
-		v[0]	= DataSet.Get_xMax();	v[1]	= DataSet.Get_yMin();	v	= B * v + A;	r.Union(CSG_Point(v[0], v[1]));
-
-		v[0]	= 0;	v[1] = 1;	v = B * v;	s	= fabs(v.Get_Length());
-		v[0]	= 1;	v[1] = 0;	v = B * v;
-
-		if( s != fabs(v.Get_Length()) )
-		{
-			if( s > fabs(v.Get_Length()) )
-			{
-				s	= fabs(v.Get_Length());
-			}
-
-			Message_Add(CSG_String::Format(SG_T("\n%s: %s\n\t%s: %f\n"), _TL("warning"), _TL("top-to-bottom and left-to-right cell sizes differ."), _TL("using cellsize"), s), false);
-		}
-
-		Transform.Assign(s, r);
-	}
-
-	//-----------------------------------------------------
-	for(i=0, n=0; i<DataSet.Get_Count() && Process_Get_Okay(); i++)
-	{
-		CSG_Table_Record	*pBand	= Bands.Get_Record_byIndex(i);
-
-		if( !Bands.Get_Selection_Count() || pBand->is_Selected() )
-		{
-			Process_Set_Text(CSG_String::Format(SG_T("%s [%d/%d]"), _TL("loading band"), i + 1, DataSet.Get_Count()));
-
-			CSG_Grid	*pGrid	= DataSet.Read(pBand->Get_Index());
-
-			if( pGrid != NULL )
-			{
-				n++;
-
-				if( Transform.is_Valid() )
-				{
-					Process_Set_Text(CSG_String::Format(SG_T("%s [%d/%d]"), _TL("band transformation"), i + 1, DataSet.Get_Count()));
-
-					Set_Transformation(&pGrid, Transform, A, B);
-				}
-
-				pGrid->Set_Name(DataSet.Get_Count() > 1
-					? CSG_String::Format(SG_T("%s [%s]"), Name.c_str(), pGrid->Get_Name()).c_str()
-					: Name.c_str()
-				);
-
-				m_pGrids->Add_Item(pGrid);
-
-				DataObject_Add			(pGrid);
-				DataObject_Set_Colors	(pGrid, CSG_Colors(100, SG_COLORS_BLACK_WHITE, false));
-			}
-		}
-    }
-
-	//-----------------------------------------------------
-	return( n > 0 );
-}
-
-
-///////////////////////////////////////////////////////////
-//														 //
-///////////////////////////////////////////////////////////
-
-//---------------------------------------------------------
-void CGDAL_Import::Set_Transformation(CSG_Grid **ppGrid, const CSG_Grid_System &System, const CSG_Vector &A, const CSG_Matrix &B)
-{
 	//-----------------------------------------------------
 	TSG_Grid_Interpolation	Interpolation;
 
@@ -440,51 +359,53 @@ void CGDAL_Import::Set_Transformation(CSG_Grid **ppGrid, const CSG_Grid_System &
 	case 4:	Interpolation	= GRID_INTERPOLATION_BSpline;			break;
 	}
 
-	//-----------------------------------------------------
-	CSG_Matrix	BInv(B.Get_Inverse());
-
-	CSG_Grid	*pImage	= *ppGrid;
-	CSG_Grid	*pWorld	= *ppGrid	= SG_Create_Grid(System, pImage->Get_Type());
-
-	pWorld->Set_Name              (pImage->Get_Name        ());
-	pWorld->Set_Description       (pImage->Get_Description ());
-	pWorld->Set_Unit              (pImage->Get_Unit        ());
-	pWorld->Set_Scaling           (pImage->Get_Scaling     (), pImage->Get_Offset());
-	pWorld->Set_NoData_Value_Range(pImage->Get_NoData_Value(), pImage->Get_NoData_hiValue());
-	pWorld->Get_MetaData()	     = pImage->Get_MetaData    ();
-	pWorld->Get_Projection()     = pImage->Get_Projection  ();
+	bool	bTransform	= Parameters("TRANSFORM")->asBool() && DataSet.Needs_Transformation();
 
 	//-----------------------------------------------------
-//	#pragma omp parallel for
-//	for(int y=0; y<pWorld->Get_NY(); y++)
-//	{
-//		Process_Get_Okay();
-
-	for(int y=0; y<pWorld->Get_NY() && Set_Progress(y, pWorld->Get_NY()); y++)
+	for(i=0, n=0; i<DataSet.Get_Count() && Process_Get_Okay(); i++)
 	{
-		#pragma omp parallel for
-		for(int x=0; x<pWorld->Get_NX(); x++)
+		CSG_Table_Record	*pBand	= Bands.Get_Record_byIndex(i);
+
+		if( !Bands.Get_Selection_Count() || pBand->is_Selected() )
 		{
-			double		z;
-			CSG_Vector	vWorld(2), vImage;
+			Process_Set_Text(CSG_String::Format("%s [%d/%d]", _TL("loading band"), i + 1, DataSet.Get_Count()));
 
-			vWorld[0]	= pWorld->Get_XMin() + x * pWorld->Get_Cellsize();
-			vWorld[1]	= pWorld->Get_YMin() + y * pWorld->Get_Cellsize();
+			CSG_Grid	*pGrid	= DataSet.Read(pBand->Get_Index());
 
-			vImage	= BInv * (vWorld - A);
-
-			if( pImage->Get_Value(vImage[0], vImage[1], z, Interpolation, false, true) )
+			if( pGrid != NULL )
 			{
-				pWorld->Set_Value(x, y, z);
-			}
-			else
-			{
-				pWorld->Set_NoData(x, y);
+				n++;
+
+				if( bTransform )
+				{
+					Process_Set_Text(CSG_String::Format("%s [%d/%d]", _TL("band transformation"), i + 1, DataSet.Get_Count()));
+
+					DataSet.Get_Transformation(&pGrid, Interpolation, true);
+				}
+
+				pGrid->Set_Name(DataSet.Get_Count() > 1
+					? CSG_String::Format("%s [%s]", Name.c_str(), pGrid->Get_Name()).c_str()
+					: Name.c_str()
+				);
+
+				pGrid->Set_File_Name(DataSet.Get_File_Name());
+
+				m_pGrids->Add_Item(pGrid);
+
+				DataObject_Add			(pGrid);
+				DataObject_Set_Colors	(pGrid, CSG_Colors(11, SG_COLORS_BLACK_WHITE, false));
+
+				if( DataSet.Get_Count() == 1 )
+				{
+					pGrid->Set_File_Name(DataSet.Get_File_Name());
+					pGrid->Get_MetaData().Add_Child("GDAL_DRIVER", DataSet.Get_DriverID());
+				}
 			}
 		}
-	}
+    }
 
-	delete(pImage);
+	//-----------------------------------------------------
+	return( n > 0 );
 }
 
 
@@ -499,7 +420,7 @@ bool	SG_GDAL_Import	(const CSG_String &File_Name)
 {
 	CGDAL_Import	Import;
 
-	if(	!Import.Get_Parameters()->Set_Parameter(SG_T("FILES"), File_Name, PARAMETER_TYPE_FilePath) )
+	if(	!Import.Get_Parameters()->Set_Parameter("FILES", File_Name, PARAMETER_TYPE_FilePath) )
 	{
 		return( false );
 	}
@@ -509,7 +430,7 @@ bool	SG_GDAL_Import	(const CSG_String &File_Name)
 		return( false );
 	}
 
-	CSG_Parameter_Grid_List	*pGrids	= Import.Get_Parameters()->Get_Parameter(SG_T("GRIDS"))->asGridList();
+	CSG_Parameter_Grid_List	*pGrids	= Import.Get_Parameters()->Get_Parameter("GRIDS")->asGridList();
 
 	for(int i=0; i<pGrids->Get_Count(); i++)
 	{
