@@ -109,6 +109,7 @@ CSG_String Get_Info(int i)
 #include "grid_table.h"
 #include "wrf.h"
 #include "citygml_import.h"
+#include "import_clip_resample.h"
 
 
 //---------------------------------------------------------
@@ -118,24 +119,27 @@ CSG_Module *		Create_Module(int i)
 {
 	switch( i )
 	{
-	case 0:		return( new CESRI_ArcInfo_Export );
-	case 1:		return( new CESRI_ArcInfo_Import );
-	case 2:		return( new CSurfer_Export );
-	case 3:		return( new CSurfer_Import );
-	case 4:		return( new CRaw_Import );
-	case 5:		return( new CXYZ_Export );
-	case 6:		return( new CXYZ_Import );
-	case 7:		return( new CUSGS_SRTM_Import );
-	case 8:		return( new CMOLA_Import );
-	case 9:		return( new CSRTM30_Import );
+	case 0 :	return( new CESRI_ArcInfo_Export );
+	case 1 :	return( new CESRI_ArcInfo_Import );
+	case 2 :	return( new CSurfer_Export );
+	case 3 :	return( new CSurfer_Import );
+	case 4 :	return( new CRaw_Import );
+	case 5 :	return( new CXYZ_Export );
+	case 6 :	return( new CXYZ_Import );
+	case 7 :	return( new CUSGS_SRTM_Import );
+	case 8 :	return( new CMOLA_Import );
+	case 9 :	return( new CSRTM30_Import );
 	case 10:	return( new CBMP_Export );
 	case 11:	return( new CErdas_LAN_Import );
 	case 12:	return( new CGrid_Table_Import );
 	case 13:	return( new CWRF_Import );
 	case 14:	return( new CWRF_Export );
 	case 15:	return( new CCityGML_Import );
+	case 16:	return( new CImport_Clip_Resample );
 
-	default:	return( NULL );
+	//-----------------------------------------------------
+	case 17:	return( NULL );
+	default:	return( MLB_INTERFACE_SKIP_MODULE );
 	}
 }
 
