@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: grid_export.cpp 2321 2014-11-03 21:10:22Z johanvdw $
+ * Version $Id: grid_export.cpp 2542 2015-07-15 12:31:48Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -81,10 +81,9 @@ CGrid_Export::CGrid_Export(void)
 	Set_Author		(SG_T("O.Conrad (c) 2005"));
 
 	Set_Description	(_TW(
-		"The module allows to save a grid as image.\n\n"
-		"On the command line, in case a shade grid is specified, "
-		"it's minimum and maximum brightness values can be specified in "
-		"percent.\n")
+		"The module allows one to save a grid as image.\n"
+		"Optionally, a shade grid can be overlayed and it's "
+		"transparency and brightness can be adjusted.\n\n")
 	);
 
 	Parameters.Add_Grid(
@@ -102,12 +101,12 @@ CGrid_Export::CGrid_Export(void)
 	Parameters.Add_FilePath(
 		NULL	, "FILE"		, _TL("Image File"),
 		_TL(""),
-		CSG_String::Format(SG_T("%s|%s|%s|%s|%s|%s|%s|%s|%s|%s"),
-			_TL("Windows or OS/2 Bitmap (*.bmp)")				, SG_T("*.bmp"),
-			_TL("JPEG - JFIF Compliant (*.jpg, *.jif, *.jpeg)")	, SG_T("*.jpg;*.jif;*.jpeg"),
-			_TL("Zsoft Paintbrush (*.pcx)")						, SG_T("*.pcx"),
+		CSG_String::Format("%s|%s|%s|%s|%s|%s|%s|%s|%s|%s",
 			_TL("Portable Network Graphics (*.png)")			, SG_T("*.png"),
-			_TL("Tagged Image File Format (*.tif, *.tiff)")		, SG_T("*.tif;*.tiff")
+			_TL("JPEG - JFIF Compliant (*.jpg, *.jif, *.jpeg)")	, SG_T("*.jpg;*.jif;*.jpeg"),
+			_TL("Tagged Image File Format (*.tif, *.tiff)")		, SG_T("*.tif;*.tiff"),
+			_TL("Windows or OS/2 Bitmap (*.bmp)")				, SG_T("*.bmp"),
+			_TL("Zsoft Paintbrush (*.pcx)")						, SG_T("*.pcx")
 		), NULL, true
 	);
 
@@ -196,14 +195,17 @@ CGrid_Export::CGrid_Export(void)
 		PARAMETER_INPUT_OPTIONAL
 	);
 
-	if( !SG_UI_Get_Window_Main() )
-	{
-		Parameters.Add_Range(
-			NULL	, "SHADE_BRIGHT", _TL("Shade Brightness"),
-			_TL("Allows one to scale shade brightness [percent]"),
-			0.0, 100.0, 0.0, true, 100.0, true
-		);
-	}
+	Parameters.Add_Value(
+		NULL	, "SHADE_TRANS"	, _TL("Shade Transparency [%]"),
+		_TL("The transparency of the shade [%]"),
+		PARAMETER_TYPE_Double, 40.0, 0.0, true, 100.0, true
+	);
+
+	Parameters.Add_Range(
+		NULL	, "SHADE_BRIGHT", _TL("Shade Brightness [%]"),
+		_TL("Allows one to scale shade brightness [%]"),
+		0.0, 100.0, 0.0, true, 100.0, true
+	);
 }
 
 
@@ -224,6 +226,12 @@ int CGrid_Export::On_Parameters_Enable(CSG_Parameters *pParameters, CSG_Paramete
 		pParameters->Get_Parameter("LUT"        )->Set_Enabled(pParameter->asInt() == 3);
 	}
 
+	if(	!SG_STR_CMP(pParameter->Get_Identifier(), SG_T("SHADE")) )
+	{
+		pParameters->Get_Parameter("SHADE_TRANS"	)->Set_Enabled(pParameter->asGrid() != NULL);
+		pParameters->Get_Parameter("SHADE_BRIGHT"	)->Set_Enabled(pParameter->asGrid() != NULL);
+	}
+
 	return( 1 );
 }
 
@@ -239,12 +247,14 @@ bool CGrid_Export::On_Execute(void)
 {
 	//-----------------------------------------------------
 	int			y, iy, Method;
+	double		dTrans;
 	CSG_Grid	*pGrid, *pShade, Grid, Shade;
 
 	//-----------------------------------------------------
-	pGrid	= Parameters("GRID"     )->asGrid();
-	pShade	= Parameters("SHADE"    )->asGrid();
-	Method	= Parameters("COLOURING")->asInt ();
+	pGrid		= Parameters("GRID"			)->asGrid();
+	pShade		= Parameters("SHADE"		)->asGrid();
+	Method		= Parameters("COLOURING"	)->asInt ();
+	dTrans		= Parameters("SHADE_TRANS"	)->asDouble() / 100.0;
 
 	if( !pGrid )
 	{
@@ -376,7 +386,7 @@ bool CGrid_Export::On_Execute(void)
 	{
 		pShade	= NULL;
 	}
-	else if( !SG_UI_DataObject_asImage(pShade, &Shade) )
+	else
 	{
 		double	dMinBright, dMaxBright;
 
@@ -426,7 +436,7 @@ bool CGrid_Export::On_Execute(void)
 		#pragma omp parallel for
 		for(int x=0; x<Get_NX(); x++)
 		{
-			if( Grid.is_NoData(x, y) )
+			if( Grid.is_NoData(x, y) || (pShade != NULL && Shade.is_NoData(x, y)) )
 			{
 				if( Image.HasAlpha() )
 				{
@@ -452,11 +462,9 @@ bool CGrid_Export::On_Execute(void)
 				{
 					c	= Shade.asInt(x, y);
 
-					double d	= (SG_GET_R(c) + SG_GET_G(c) + SG_GET_B(c)) / (3.0 * 255.0);
-
-					r	= (int)(d * r);
-					g	= (int)(d * g);
-					b	= (int)(d * b);
+					r	= dTrans * r + SG_GET_R(c) * (1.0 - dTrans);
+					g	= dTrans * g + SG_GET_G(c) * (1.0 - dTrans);
+					b	= dTrans * b + SG_GET_B(c) * (1.0 - dTrans);
 				}
 
 				Image.SetRGB(x, y, r, g, b);
@@ -474,6 +482,8 @@ bool CGrid_Export::On_Execute(void)
 	&&  !SG_File_Cmp_Extension(fName, SG_T("tif")) )
 	{
 		fName	= SG_File_Make_Path(NULL, fName, SG_T("png"));
+
+		Parameters("FILE")->Set_Value(fName);
 	}
 
 	//-----------------------------------------------------
@@ -481,7 +491,7 @@ bool CGrid_Export::On_Execute(void)
 
 	if( !SG_UI_Get_Window_Main() )
 	{
-		if( SG_File_Cmp_Extension(fName, SG_T("jpg")) )
+		if(      SG_File_Cmp_Extension(fName, SG_T("jpg")) )
 			pImgHandler = new wxJPEGHandler;
 		else if( SG_File_Cmp_Extension(fName, SG_T("pcx")) )
 			pImgHandler = new wxPCXHandler;
@@ -527,29 +537,26 @@ bool CGrid_Export::On_Execute(void)
 	}
 
 	//-----------------------------------------------------
-	if( Parameters("FILE_KML")->asBool() && Stream.Open(SG_File_Make_Path(NULL, fName, SG_T("kml")), SG_FILE_W, false) )
+	if( Parameters("FILE_KML")->asBool() )
 	{
-		Stream.Printf(SG_T("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"));
-		Stream.Printf(SG_T("<kml xmlns=\"http://www.opengis.net/kml/2.2\">\n"));
-		Stream.Printf(SG_T("  <Folder>\n"));
-		Stream.Printf(SG_T("    <name>Raster exported from SAGA</name>\n"));
-		Stream.Printf(SG_T("    <description>System for Automated Geoscientific Analyses - www.saga-gis.org</description>\n"));
-		Stream.Printf(SG_T("    <GroundOverlay>\n"));
-		Stream.Printf(SG_T("      <name>%s</name>\n")				, pGrid->Get_Name());
-		Stream.Printf(SG_T("      <description>%s</description>\n")	, pGrid->Get_Description());
-		Stream.Printf(SG_T("      <Icon>\n"));
-		Stream.Printf(SG_T("        <href>%s</href>\n")				, SG_File_Get_Name(fName, true).c_str());
-		Stream.Printf(SG_T("      </Icon>\n"));
-		Stream.Printf(SG_T("      <LatLonBox>\n"));
-		Stream.Printf(SG_T("        <north>%.10f</north>\n")			, pGrid->Get_YMax());
-		Stream.Printf(SG_T("        <south>%.10f</south>\n")			, pGrid->Get_YMin());
-		Stream.Printf(SG_T("        <east>%.10f</east>\n")				, pGrid->Get_XMax());
-		Stream.Printf(SG_T("        <west>%.10f</west>\n")				, pGrid->Get_XMin());
-		Stream.Printf(SG_T("        <rotation>0.0</rotation>\n"));
-		Stream.Printf(SG_T("      </LatLonBox>\n"));
-		Stream.Printf(SG_T("    </GroundOverlay>\n"));
-		Stream.Printf(SG_T("  </Folder>\n"));
-		Stream.Printf(SG_T("</kml>\n"));
+		CSG_MetaData	KML;	KML.Set_Name("kml");	KML.Add_Property("xmlns", "http://www.opengis.net/kml/2.2");
+
+	//	CSG_MetaData	*pFolder	= KML.Add_Child("Folder");
+	//	pFolder->Add_Child("name"       , "Raster exported from SAGA");
+	//	pFolder->Add_Child("description", "System for Automated Geoscientific Analyses - www.saga-gis.org");
+	//	CSG_MetaData	*pOverlay	= pFolder->Add_Child("GroundOverlay");
+
+		CSG_MetaData	*pOverlay	= KML.Add_Child("GroundOverlay");
+		pOverlay->Add_Child("name"       , pGrid->Get_Name());
+		pOverlay->Add_Child("description", pGrid->Get_Description());
+		pOverlay->Add_Child("Icon"       )->Add_Child("href", SG_File_Get_Name(fName, true));
+		pOverlay->Add_Child("LatLonBox"  );
+		pOverlay->Get_Child("LatLonBox"  )->Add_Child("north", pGrid->Get_YMax());
+		pOverlay->Get_Child("LatLonBox"  )->Add_Child("south", pGrid->Get_YMin());
+		pOverlay->Get_Child("LatLonBox"  )->Add_Child("east" , pGrid->Get_XMax());
+		pOverlay->Get_Child("LatLonBox"  )->Add_Child("west" , pGrid->Get_XMin());
+
+		KML.Save(fName, SG_T("kml"));
 	}
 
 	//-----------------------------------------------------
