@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: pc_get_grid_spcvf.cpp 2277 2014-10-07 13:17:58Z reklov_w $
+ * Version $Id: pc_get_grid_spcvf.cpp 2439 2015-03-16 13:35:26Z reklov_w $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -88,7 +88,9 @@ CPointCloud_Get_Grid_SPCVF_Base::~CPointCloud_Get_Grid_SPCVF_Base(void)
 
 
 //---------------------------------------------------------
-void CPointCloud_Get_Grid_SPCVF_Base::Initialise(int iOutputs, CSG_Rect AOI, CSG_Shapes *pShapes, int iFieldName, bool bMultiple, bool bAddOverlap, double dOverlap, CSG_String sFileName, CSG_Parameter_File_Name *pFilePath, CSG_Parameter_Grid_List *pGridList, double dCellsize, bool bFitToCells, int iMethod)
+void CPointCloud_Get_Grid_SPCVF_Base::Initialise(int iOutputs, CSG_Rect AOI, CSG_Shapes *pShapes, int iFieldName, bool bMultiple, bool bAddOverlap, double dOverlap,
+												 CSG_String sFileName, CSG_Parameter_File_Name *pFilePath, CSG_Parameter_Grid_List *pGridList, double dCellsize,
+												 bool bFitToCells, int iMethod, bool bConstrain, int iField, double dMinAttrRange, double dMaxAttrRange)
 {
 	m_iOutputs		= iOutputs;
 	m_AOI			= AOI;
@@ -103,6 +105,10 @@ void CPointCloud_Get_Grid_SPCVF_Base::Initialise(int iOutputs, CSG_Rect AOI, CSG
 	m_dCellsize		= dCellsize;
 	m_bFitToCells	= bFitToCells;
 	m_iMethod		= iMethod;
+	m_bConstrain	= bConstrain;
+	m_iField		= iField;
+	m_dMinAttrRange	= dMinAttrRange;
+	m_dMaxAttrRange	= dMaxAttrRange;
 
 	return;
 }
@@ -120,13 +126,13 @@ void CPointCloud_Get_Grid_SPCVF_Base::Finalise(void)
 
 
 //---------------------------------------------------------
-bool CPointCloud_Get_Grid_SPCVF_Base::Get_Subset(void)
+bool CPointCloud_Get_Grid_SPCVF_Base::Get_Subset(int iFieldToGrid)
 {
-	CSG_String		sPathSPCVF, sMethodPaths;
+	CSG_String		sVersion, sPathSPCVF, sMethodPaths;
 	double			dBBoxXMin, dBBoxYMin, dBBoxXMax, dBBoxYMax;
 	CSG_Rect		BBoxSPCVF;
 	CSG_MetaData	SPCVF;
-	
+
 
 	//-----------------------------------------------------
 	if( !SPCVF.Create(m_sFileName) || SPCVF.Get_Name().CmpNoCase(SG_T("SPCVFDataset")) )
@@ -135,7 +141,36 @@ bool CPointCloud_Get_Grid_SPCVF_Base::Get_Subset(void)
 		return( false );
 	}
 
+
+	CSG_MetaData	*pHeader = SPCVF.Get_Child(SG_T("Header"));
+
+	if( pHeader != NULL )
+	{
+		int		iFieldCount;
+
+		pHeader->Get_Child(SG_T("Attributes"))->Get_Property(SG_T("Count"), iFieldCount);
+
+		if( iFieldToGrid < 0 || iFieldToGrid >= iFieldCount )
+		{
+			SG_UI_Msg_Add_Error(_TL("Attribute field number to grid is out of range!"));
+			return( false );
+		}
+
+		if( m_bConstrain )
+		{
+			if( m_iField >= iFieldCount )
+			{
+				SG_UI_Msg_Add_Error(_TL("Constraining attribute field number is out of range!"));
+				return( false );
+			}
+		}
+	}
+
+
+
 	//-----------------------------------------------------
+	SPCVF.Get_Property(SG_T("Version"), sVersion);
+
 	SPCVF.Get_Property(SG_T("Paths"), sMethodPaths);
 
 	if( !sMethodPaths.CmpNoCase(SG_T("absolute")) )
@@ -154,10 +189,23 @@ bool CPointCloud_Get_Grid_SPCVF_Base::Get_Subset(void)
 	}
 
 	//-----------------------------------------------------
-	SPCVF.Get_Child(SG_T("BBox"))->Get_Property(SG_T("XMin"), dBBoxXMin);
-	SPCVF.Get_Child(SG_T("BBox"))->Get_Property(SG_T("YMin"), dBBoxYMin);
-	SPCVF.Get_Child(SG_T("BBox"))->Get_Property(SG_T("XMax"), dBBoxXMax);
-	SPCVF.Get_Child(SG_T("BBox"))->Get_Property(SG_T("YMax"), dBBoxYMax);
+	if( !sVersion.CmpNoCase(SG_T("1.0")) )
+	{
+		SPCVF.Get_Child(SG_T("BBox"))->Get_Property(SG_T("XMin"), dBBoxXMin);
+		SPCVF.Get_Child(SG_T("BBox"))->Get_Property(SG_T("YMin"), dBBoxYMin);
+		SPCVF.Get_Child(SG_T("BBox"))->Get_Property(SG_T("XMax"), dBBoxXMax);
+		SPCVF.Get_Child(SG_T("BBox"))->Get_Property(SG_T("YMax"), dBBoxYMax);
+	}
+	else // 1.1
+	{
+		CSG_MetaData	*pHeader = SPCVF.Get_Child(SG_T("Header"));
+
+		pHeader->Get_Child(SG_T("BBox"))->Get_Property(SG_T("XMin"), dBBoxXMin);
+		pHeader->Get_Child(SG_T("BBox"))->Get_Property(SG_T("YMin"), dBBoxYMin);
+		pHeader->Get_Child(SG_T("BBox"))->Get_Property(SG_T("XMax"), dBBoxXMax);
+		pHeader->Get_Child(SG_T("BBox"))->Get_Property(SG_T("YMax"), dBBoxYMax);
+	}
+
 	BBoxSPCVF.Assign(dBBoxXMin, dBBoxYMin, dBBoxXMax, dBBoxYMax);
 
 
@@ -200,7 +248,7 @@ bool CPointCloud_Get_Grid_SPCVF_Base::Get_Subset(void)
 		{
 			m_dOverlap = 0.0;
 		}
-	
+
 		if( m_AOI.Intersects(BBoxSPCVF) == INTERSECTION_None )
 		{
 			SG_UI_Msg_Add(_TL("AOI does not intersect bounding box of SPCVF, nothing to do!"), true);
@@ -279,7 +327,8 @@ bool CPointCloud_Get_Grid_SPCVF_Base::Get_Subset(void)
 
 			for(int iPoint=0; iPoint<pPC->Get_Count(); iPoint++)
 			{
-				if( m_AOI.Contains(pPC->Get_X(iPoint), pPC->Get_Y(iPoint)) )
+				if( m_AOI.Get_XMin() <= pPC->Get_X(iPoint) && pPC->Get_X(iPoint) < m_AOI.Get_XMax() &&
+					m_AOI.Get_YMin() <= pPC->Get_Y(iPoint) && pPC->Get_Y(iPoint) < m_AOI.Get_YMax() )
 				{
 					if( m_pShapes != NULL && !m_bAddOverlap )
 					{
@@ -288,21 +337,29 @@ bool CPointCloud_Get_Grid_SPCVF_Base::Get_Subset(void)
 						if( !pPolygon->Contains(pPC->Get_X(iPoint), pPC->Get_Y(iPoint)) )
 							continue;
 					}
-							
+
+					if( m_bConstrain )
+					{
+						if( pPC->Get_Value(iPoint, m_iField) < m_dMinAttrRange || pPC->Get_Value(iPoint, m_iField) > m_dMaxAttrRange)
+						{
+							continue;
+						}
+					}
+
 					if( System.Get_World_to_Grid(x, y, pPC->Get_X(iPoint), pPC->Get_Y(iPoint)) )
 					{
 						switch( m_iMethod )
 						{
 						default:
-						case 0:		if( pGrid->is_NoData(x, y) || pGrid->asDouble(x, y) > pPC->Get_Z(iPoint) )
+						case 0:		if( pGrid->is_NoData(x, y) || pGrid->asDouble(x, y) > pPC->Get_Value(iPoint, iFieldToGrid) )
 									{
-										pGrid->Set_Value(x, y, pPC->Get_Z(iPoint));
+										pGrid->Set_Value(x, y, pPC->Get_Value(iPoint, iFieldToGrid));
 									}
 									break;
 
-						case 1:		if( pGrid->is_NoData(x, y) || pGrid->asDouble(x, y) < pPC->Get_Z(iPoint) )
+						case 1:		if( pGrid->is_NoData(x, y) || pGrid->asDouble(x, y) < pPC->Get_Value(iPoint, iFieldToGrid) )
 									{
-										pGrid->Set_Value(x, y, pPC->Get_Z(iPoint));
+										pGrid->Set_Value(x, y, pPC->Get_Value(iPoint, iFieldToGrid));
 									}
 									break;
 						}
@@ -339,7 +396,7 @@ bool CPointCloud_Get_Grid_SPCVF_Base::Get_Subset(void)
 	}
 
 	//---------------------------------------------------------
-	if( !m_bMultiple )
+	if( !m_bMultiple && pGrid != NULL)
 	{
 		if( pGrid != NULL && dPoints == 0.0 )
 		{
@@ -412,7 +469,7 @@ CPointCloud_Get_Grid_SPCVF::CPointCloud_Get_Grid_SPCVF(void)
 	Set_Author		(SG_T("V. Wichmann, LASERDATA GmbH (c) 2014"));
 
 	Set_Description	(_TW(
-		"The module allows to retrieve a grid from a virtual "
+		"The module allows one to retrieve a grid from a virtual "
 		"point cloud dataset by applying the provided area-of-interest "
 		"(AOI). The extent of the AOI can be provided either as polygon "
 		"shapefile, grid or by coordinates. Optionally, an overlap can "
@@ -431,6 +488,8 @@ CPointCloud_Get_Grid_SPCVF::CPointCloud_Get_Grid_SPCVF(void)
 		"list or written to an output directory. For the latter, "
 		"you must provide a valid file path with the 'Optional Output "
 		"Filepath' parameter.\n"
+		"Optionally, the query can be constrained by providing an "
+		"attribute field and a value range that must be met.\n"
 		"A virtual point cloud dataset is a simple XML format "
 		"with the file extension .spcvf, which can be created "
 		"with the 'Create Virtual Point Cloud Dataset' module.\n\n"
@@ -460,6 +519,12 @@ CPointCloud_Get_Grid_SPCVF::CPointCloud_Get_Grid_SPCVF(void)
 		true, true, false
 	);
 
+	Parameters.Add_Value(
+		NULL	, "ATTR_FIELD_GRID"	, _TL("Attribute Field to Grid"),
+		_TL("The attribute field to grid. Field numbers start with 1, so elevation is attribute field 3."),
+		PARAMETER_TYPE_Int, 1, 3, true
+	);
+
 	Parameters.Add_Value(
 		NULL	, "CELL_SIZE"	, _TL("Cellsize"),
 		_TL("Cellsize of the output grid [map units]"),
@@ -474,16 +539,34 @@ CPointCloud_Get_Grid_SPCVF::CPointCloud_Get_Grid_SPCVF(void)
 			_TL("cells")
 		), 1
 	);
-	
+
 	Parameters.Add_Choice(
 		NULL	, "METHOD"	, _TL("Aggregation"),
 		_TL("Choose how to aggregate the values"),
 		CSG_String::Format(SG_T("%s|%s|"),
-			_TL("lowest z"),
-			_TL("highest z")
+			_TL("lowest"),
+			_TL("highest")
 		), 1
 	);
 
+	Parameters.Add_Value(
+		NULL	, "CONSTRAIN_QUERY"	, _TL("Constrain Query"),
+		_TL("Check this parameter to constrain the query by an attribute range."),
+		PARAMETER_TYPE_Bool, false
+	);
+
+	Parameters.Add_Value(
+		Parameters("CONSTRAIN_QUERY")	, "ATTR_FIELD"	, _TL("Attribute Field"),
+		_TL("The attribute field to use as constraint. Field numbers start with 1."),
+		PARAMETER_TYPE_Int, 1, 1, true
+	);
+
+	Parameters.Add_Range(
+		Parameters("CONSTRAIN_QUERY")	, "VALUE_RANGE"	, _TL("Value Range"),
+		_TL("Minimum and maximum of attribute range []."),
+		2.0, 2.0
+	);
+
 
 	CSG_Parameter *pNode = Parameters.Add_Node(NULL, "NODE_AOI", _TL("AOI"), _TL("AOI Input Settings"));
 
@@ -540,9 +623,13 @@ bool CPointCloud_Get_Grid_SPCVF::On_Execute(void)
 	CSG_String		sFileName;
 	CSG_Parameter_Grid_List	*pGridList = NULL;
 	CSG_Parameter_File_Name	*pFilePath = NULL;
+	int				iFieldToGrid;
 	double			dCellsize;
 	bool			bFitToCells;
 	int				iMethod;
+	bool			bConstrain;
+	int				iField;
+	double			dMinAttrRange, dMaxAttrRange;
 	CSG_Shapes		*pShapes;
 	int				iFieldName;
 	CSG_Grid		*pAOIGrid = NULL;
@@ -558,9 +645,14 @@ bool CPointCloud_Get_Grid_SPCVF::On_Execute(void)
 	sFileName		= Parameters("FILENAME")->asString();
 	pGridList		= Parameters("GRID_OUT")->asGridList();
 	pFilePath		= Parameters("FILEPATH")->asFilePath();
+	iFieldToGrid	= Parameters("ATTR_FIELD_GRID")->asInt() - 1;
 	dCellsize		= Parameters("CELL_SIZE")->asDouble();
 	bFitToCells		= Parameters("GRID_SYSTEM_FIT")->asBool();
 	iMethod			= Parameters("METHOD")->asInt();
+	bConstrain		= Parameters("CONSTRAIN_QUERY")->asBool();
+	iField			= Parameters("ATTR_FIELD")->asInt() - 1;
+	dMinAttrRange	= Parameters("VALUE_RANGE")->asRange()->Get_LoVal();
+	dMaxAttrRange	= Parameters("VALUE_RANGE")->asRange()->Get_HiVal();
 	pShapes			= Parameters("AOI_SHP")->asShapes();
 	iFieldName		= Parameters("FIELD_TILENAME")->asInt();
 	pAOIGrid		= Parameters("AOI_GRID")->asGrid();
@@ -583,7 +675,7 @@ bool CPointCloud_Get_Grid_SPCVF::On_Execute(void)
 	}
 
 	CSG_String	sPath = pFilePath->asString();
-	
+
 	if( sPath.Length() <= 1 )
 	{
 		pFilePath = NULL;
@@ -614,9 +706,10 @@ bool CPointCloud_Get_Grid_SPCVF::On_Execute(void)
 
 	//---------------------------------------------------------
 
-	m_Get_Grid_SPCVF.Initialise(iOutputs, AOI, pShapes, iFieldName, bMultiple, bAddOverlap, dOverlap, sFileName, pFilePath, pGridList, dCellsize, bFitToCells, iMethod);
+	m_Get_Grid_SPCVF.Initialise(iOutputs, AOI, pShapes, iFieldName, bMultiple, bAddOverlap, dOverlap, sFileName, pFilePath, pGridList, dCellsize, bFitToCells, iMethod,
+								bConstrain, iField, dMinAttrRange, dMaxAttrRange);
 
-	bool bResult = m_Get_Grid_SPCVF.Get_Subset();
+	bool bResult = m_Get_Grid_SPCVF.Get_Subset(iFieldToGrid);
 
 	m_Get_Grid_SPCVF.Finalise();
 
@@ -627,16 +720,22 @@ bool CPointCloud_Get_Grid_SPCVF::On_Execute(void)
 //---------------------------------------------------------
 int CPointCloud_Get_Grid_SPCVF::On_Parameters_Enable(CSG_Parameters *pParameters, CSG_Parameter *pParameter)
 {
+	if(	!SG_STR_CMP(pParameter->Get_Identifier(), SG_T("CONSTRAIN_QUERY")) )
+	{
+		pParameters->Get_Parameter("ATTR_FIELD"			)->Set_Enabled(pParameter->asBool());
+		pParameters->Get_Parameter("VALUE_RANGE"		)->Set_Enabled(pParameter->asBool());
+	}
+
 	if(	!SG_STR_CMP(pParameter->Get_Identifier(), SG_T("AOI_ADD_OVERLAP")) )
 	{
-		pParameters->Get_Parameter("OVERLAP")->Set_Enabled(pParameter->asBool());
+		pParameters->Get_Parameter("OVERLAP"			)->Set_Enabled(pParameter->asBool());
 	}
 
 	if(	!SG_STR_CMP(pParameter->Get_Identifier(), SG_T("AOI_SHP")) )
 	{
-		pParameters->Get_Parameter("FIELD_TILENAME"			)->Set_Enabled(pParameter->asShapes() != NULL);
+		pParameters->Get_Parameter("FIELD_TILENAME"		)->Set_Enabled(pParameter->asShapes() != NULL);
 	}
-	
+
 	//-----------------------------------------------------
 	return (1);
 }
@@ -656,8 +755,10 @@ CPointCloud_Get_Grid_SPCVF_Interactive::CPointCloud_Get_Grid_SPCVF_Interactive(v
 	Set_Author		(SG_T("V. Wichmann, LASERDATA GmbH (c) 2014"));
 
 	Set_Description	(_TW(
-		"The module allows to retrieve a grid from a virtual "
+		"The module allows one to retrieve a grid from a virtual "
 		"point cloud dataset by dragging a box (AOI) in a Map View.\n"
+		"Optionally, the query can be constrained by providing an "
+		"attribute field and a value range that must be met.\n"
 		"A virtual point cloud dataset is a simple XML format "
 		"with the file extension .spcvf, which can be created "
 		"with the 'Create Virtual Point Cloud Dataset' module.\n\n"
@@ -680,6 +781,12 @@ CPointCloud_Get_Grid_SPCVF_Interactive::CPointCloud_Get_Grid_SPCVF_Interactive(v
 		PARAMETER_OUTPUT_OPTIONAL
 	);
 
+	Parameters.Add_Value(
+		NULL	, "ATTR_FIELD_GRID"	, _TL("Attribute Field to Grid"),
+		_TL("The attribute field to grid. Field numbers start with 1, so elevation is attribute field 3."),
+		PARAMETER_TYPE_Int, 1, 3, true
+	);
+
 	Parameters.Add_Value(
 		NULL	, "CELL_SIZE"	, _TL("Cellsize"),
 		_TL("Cellsize of the output grid [map units]"),
@@ -699,10 +806,28 @@ CPointCloud_Get_Grid_SPCVF_Interactive::CPointCloud_Get_Grid_SPCVF_Interactive(v
 		NULL	, "METHOD"	, _TL("Aggregation"),
 		_TL("Choose how to aggregate the values"),
 		CSG_String::Format(SG_T("%s|%s|"),
-			_TL("lowest z"),
-			_TL("highest z")
+			_TL("lowest"),
+			_TL("highest")
 		), 1
 	);
+
+	Parameters.Add_Value(
+		NULL	, "CONSTRAIN_QUERY"	, _TL("Constrain Query"),
+		_TL("Check this parameter to constrain the query by an attribute range."),
+		PARAMETER_TYPE_Bool, false
+	);
+
+	Parameters.Add_Value(
+		Parameters("CONSTRAIN_QUERY")	, "ATTR_FIELD"	, _TL("Attribute Field"),
+		_TL("The attribute field to use as constraint. Field numbers start with 1."),
+		PARAMETER_TYPE_Int, 1, 1, true
+	);
+
+	Parameters.Add_Range(
+		Parameters("CONSTRAIN_QUERY")	, "VALUE_RANGE"	, _TL("Value Range"),
+		_TL("Minimum and maximum of attribute range []."),
+		2.0, 2.0
+	);
 }
 
 
@@ -740,14 +865,16 @@ bool CPointCloud_Get_Grid_SPCVF_Interactive::On_Execute_Position(CSG_Point ptWor
 		// to use CSG_Rect instead of CSG_Shape for point in polygon check in Get_Subset():
 		m_Get_Grid_SPCVF.Initialise(1, AOI, NULL, -1, false, true, 0.0, Parameters("FILENAME")->asString(),
 									NULL, Parameters("GRID_OUT")->asGridList(), Parameters("CELL_SIZE")->asDouble(),
-									Parameters("GRID_SYSTEM_FIT")->asBool(), Parameters("METHOD")->asInt());
+									Parameters("GRID_SYSTEM_FIT")->asBool(), Parameters("METHOD")->asInt(),
+									Parameters("CONSTRAIN_QUERY")->asBool(), Parameters("ATTR_FIELD")->asInt()-1,
+									Parameters("VALUE_RANGE")->asRange()->Get_LoVal(), Parameters("VALUE_RANGE")->asRange()->Get_HiVal());
 
-		bool bResult = m_Get_Grid_SPCVF.Get_Subset();
+		bool bResult = m_Get_Grid_SPCVF.Get_Subset(Parameters("ATTR_FIELD_GRID")->asInt()-1);
 
 		if( bResult )
 		{
 			CSG_Grid *pGrid = Parameters("GRID_OUT")->asGridList()->asGrid(Parameters("GRID_OUT")->asGridList()->Get_Count()-1);
-	
+
 			DataObject_Update(pGrid, SG_UI_DATAOBJECT_SHOW_LAST_MAP);
 		}
 
@@ -760,6 +887,20 @@ bool CPointCloud_Get_Grid_SPCVF_Interactive::On_Execute_Position(CSG_Point ptWor
 }
 
 
+//---------------------------------------------------------
+int CPointCloud_Get_Grid_SPCVF_Interactive::On_Parameters_Enable(CSG_Parameters *pParameters, CSG_Parameter *pParameter)
+{
+	if(	!SG_STR_CMP(pParameter->Get_Identifier(), SG_T("CONSTRAIN_QUERY")) )
+	{
+		pParameters->Get_Parameter("ATTR_FIELD"			)->Set_Enabled(pParameter->asBool());
+		pParameters->Get_Parameter("VALUE_RANGE"		)->Set_Enabled(pParameter->asBool());
+	}
+
+	//-----------------------------------------------------
+	return (1);
+}
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 //														 //
