@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: pc_get_subset_spcvf.h 2209 2014-08-28 12:24:03Z reklov_w $
+ * Version $Id: pc_get_subset_spcvf.h 2384 2015-02-01 15:18:44Z reklov_w $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -74,6 +74,8 @@
 
 //---------------------------------------------------------
 #include "MLB_Interface.h"
+#include <vector>
+#include <algorithm>
 
 
 ///////////////////////////////////////////////////////////
@@ -89,10 +91,13 @@ public:
 	CPointCloud_Get_Subset_SPCVF_Base(void);
 	virtual ~CPointCloud_Get_Subset_SPCVF_Base(void);
 
-	void					Initialise	(int iOutputs, CSG_Rect	AOI, CSG_Shapes *pShapes, int iFieldName, bool bMultiple, bool bAddOverlap, double dOverlap, CSG_String sFilename, CSG_Parameter_File_Name *pFilePath, CSG_Parameter_PointCloud_List *pPointCloudList);
+	bool					Initialise	(int iOutputs, CSG_Rect	AOI, CSG_Shapes *pShapes, int iFieldName, bool bMultiple, bool bAddOverlap, double dOverlap,
+										 CSG_String sFileNameTileInfo, CSG_String sFilename, CSG_Parameter_File_Name *pFilePath,
+										 CSG_Parameter_PointCloud_List *pPointCloudList, bool bConstrain, int iField, double dMinAttrRange, double dMaxAttrRange,
+										 bool bCopyAttr, CSG_String sAttrList);
 	void					Finalise	(void);
-	bool					Get_Subset	(void);
-	void					Write_Subset(CSG_PointCloud *pPC_out, int iAOI, int iDatasets);
+	bool					Get_Subset	(bool bCopyAttr);
+	void					Write_Subset(CSG_PointCloud *pPC_out, int iAOI, int iDatasets, CSG_MetaData *pSPCVF_Tiles, bool bAbsolutePaths);
 
 protected:
 
@@ -105,9 +110,14 @@ private:
 	int						m_iFieldName;
 	bool					m_bMultiple, m_bAddOverlap;
 	double					m_dOverlap;
+	CSG_String				m_sFileNameTileInfo;
 	CSG_String				m_sFileName;
 	CSG_Parameter_File_Name	*m_pFilePath;
 	CSG_Parameter_PointCloud_List	*m_pPointCloudList;
+	bool					m_bConstrain;
+	int						m_iField;
+	double					m_dMinAttrRange, m_dMaxAttrRange;
+	std::vector<int>		m_vAttrMapper;
 
 };
 
@@ -149,6 +159,8 @@ protected:
 	virtual bool				On_Execute			(void);
 	virtual bool				On_Execute_Position	(CSG_Point ptWorld, TSG_Module_Interactive_Mode Mode);
 
+	virtual int					On_Parameters_Enable(CSG_Parameters *pParameters, CSG_Parameter *pParameter);
+
 
 private:
 
