@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: MLB_Interface.cpp 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: MLB_Interface.cpp 2562 2015-08-30 10:50:30Z reklov_w $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -68,10 +68,13 @@ CSG_String Get_Info(int i)
 	switch( i )
 	{
 	case MLB_INFO_Name:	default:
-		return( _TL("Shapes - Point Clouds") );
+		return( _TL("Point Clouds") );
+
+	case MLB_INFO_Category:
+		return( _TL("Shapes") );
 
 	case MLB_INFO_Author:
-		return( SG_T("O.Conrad, V.Wichmann, M.Bremer (c) 2009-12") );
+		return( SG_T("O.Conrad, V.Wichmann, M.Bremer (c) 2009-15") );
 
 	case MLB_INFO_Description:
 		return( _TL("Tools for point clouds.") );
@@ -92,6 +95,7 @@ CSG_String Get_Info(int i)
 #include "pc_drop_attribute.h"
 #include "pc_from_grid.h"
 #include "pc_from_shapes.h"
+#include "pc_from_table.h"
 #include "pc_merge.h"
 #include "pc_reclass_extract.h"
 #include "pc_thinning_simple.h"
@@ -118,6 +122,7 @@ CSG_Module *		Create_Module(int i)
 	case 10:	return( new CPC_Attribute_Calculator );
 	case 11:	return( new CPC_Cluster_Analysis );
 	case 12:	return( new CPC_Merge );
+	case 13:	return( new CPC_From_Table );
 	}
 
 	return( NULL );
