@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: pc_attribute_calculator.cpp 2025 2014-02-27 12:51:31Z reklov_w $
+ * Version $Id: pc_attribute_calculator.cpp 2627 2015-10-01 09:29:25Z reklov_w $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -156,6 +156,16 @@ CPC_Attribute_Calculator::~CPC_Attribute_Calculator(void)
 //														 //
 ///////////////////////////////////////////////////////////
 
+//---------------------------------------------------------
+bool CPC_Attribute_Calculator::On_Before_Execution(void)
+{
+	if( Parameters("PC_OUT")->asPointCloud() == Parameters("PC_IN")->asPointCloud() )
+		Parameters("PC_OUT")->Set_Value(DATAOBJECT_NOTSET);
+
+	return (true);
+}
+
+
 //---------------------------------------------------------
 bool CPC_Attribute_Calculator::On_Execute(void)
 {
@@ -195,7 +205,7 @@ bool CPC_Attribute_Calculator::On_Execute(void)
 	//-----------------------------------------------------
 	CSG_Formula	Formula;
 
-	int		nFields		= 0;
+	int		nFields		= pInput->Get_Field_Count();
 	int		*pFields	= new int[nFields];
 
 	if( !Formula.Set_Formula(Get_Formula(Parameters("FORMULA")->asString(), pInput, pFields, nFields)) )
@@ -219,13 +229,14 @@ bool CPC_Attribute_Calculator::On_Execute(void)
 	}
 
 	//-----------------------------------------------------
-	CSG_String	sDatasetName;
+	CSG_String		sName;
+	CSG_MetaData	History;
 
 	if (!pResult || pResult == pInput)
 	{
+		sName = pInput->Get_Name();
 		pResult = SG_Create_PointCloud(pInput);
-
-		sDatasetName = pInput->Get_Name();
+		History = pInput->Get_History();
 	}
 	else
 	{
@@ -278,8 +289,10 @@ bool CPC_Attribute_Calculator::On_Execute(void)
 	if (!Parameters("PC_OUT")->asPointCloud() || Parameters("PC_OUT")->asPointCloud() == pInput)
 	{
 		pInput->Assign(pResult);
-		pInput->Set_Name(sDatasetName);
-		DataObject_Update(pInput);
+		pInput->Get_History() = History;
+		pInput->Set_Name(sName);
+		Parameters("PC_OUT")->Set_Value(pInput);
+
 		delete(pResult);
 	}
 	else
