@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: pc_cluster_analysis.h 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: pc_cluster_analysis.h 2295 2014-10-17 15:35:10Z reklov_w $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -96,13 +96,14 @@ protected:
 
 private:
 
-	int						nFields, clustField, *nMembers;
+	bool					m_bUpdateView;
+
+	int						*m_Features, m_nFeatures, clustField, *nMembers;
 
 	double					**Centroids, *Variances;
 
 	CSG_PointCloud			*pInput, *pResult;
 
-	std::vector<bool>					vFields;
 	std::vector< std::vector<double> >	vValues;	
 
 	void					Write_Result	(CSG_Table *pTable, long nElements, int nCluster, double SP);
