@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: MLB_Interface.cpp 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: MLB_Interface.cpp 2289 2014-10-16 13:45:05Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -72,7 +72,10 @@ CSG_String Get_Info(int i)
 	switch( i )
 	{
 	case MLB_INFO_Name:	default:
-		return( _TL("Shapes - Point Clouds Viewer") );
+		return( _TL("Point Clouds Viewer") );
+
+	case MLB_INFO_Category:
+		return( _TL("Visualization") );
 
 	case MLB_INFO_Author:
 		return( SG_T("O.Conrad (c) 2009") );
@@ -84,7 +87,7 @@ CSG_String Get_Info(int i)
 		return( SG_T("1.0") );
 
 	case MLB_INFO_Menu_Path:
-		return( _TL("Shapes|Point Clouds|Visualisation" ));
+		return( _TL("Visualization|3D Viewer" ));
 	}
 }
 
