@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: MLB_Interface.cpp 2281 2014-10-09 15:49:41Z oconrad $
+ * Version $Id: MLB_Interface.cpp 2462 2015-03-27 16:29:25Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -113,6 +113,9 @@ CSG_String Get_Info(int i)
 
 #include "crs_transform_pointcloud.h"
 
+#include "crs_distance.h"
+
+
 //---------------------------------------------------------
 // 4. Allow your modules to be created here...
 
@@ -121,10 +124,25 @@ CSG_Module *		Create_Module(int i)
 	switch( i )
 	{
 	case  0:	return( new CCRS_Assign() );
-	case  1:	return( new CCRS_Transform_Shapes(true ) );
-	case  2:	return( new CCRS_Transform_Shapes(false) );
-	case  3:	return( new CCRS_Transform_Grid  (true ) );
-	case  4:	return( new CCRS_Transform_Grid  (false) );
+
+	case  1:	return( new CCRS_Transform_Shapes    (true ) );
+	case  2:	return( new CCRS_Transform_Shapes    (false) );
+	case  3:	return( new CCRS_Transform_Grid      (true ) );
+	case  4:	return( new CCRS_Transform_Grid      (false) );
+	case 18:	return( new CCRS_Transform_PointCloud(true ) );
+	case 19:	return( new CCRS_Transform_PointCloud(false) );
+
+	case 15:	return( new CCRS_Picker() );
+
+	case 13:	return( new CGCS_Grid_Longitude_Range() );
+
+	case 14:	return( new CGCS_Graticule() );
+
+	case 16:	return( new CCRS_Indicatrix() );
+	case 17:	return( new CCRS_Grid_GeogCoords() );
+	case 20:	return( new CCRS_Distance_Lines() );
+	case 21:	return( new CCRS_Distance_Points() );
+	case 22:	return( new CCRS_Distance_Interactive() );
 
 	case  5:	return( new CPROJ4_Shapes	(PROJ4_INTERFACE_SIMPLE, false) );
 	case  6:	return( new CPROJ4_Shapes	(PROJ4_INTERFACE_DIALOG, false) );
@@ -135,19 +153,7 @@ CSG_Module *		Create_Module(int i)
 	case 11:	return( new CPROJ4_Grid		(PROJ4_INTERFACE_SIMPLE, true) );
 	case 12:	return( new CPROJ4_Grid		(PROJ4_INTERFACE_DIALOG, true) );
 
-	case 13:	return( new CGCS_Grid_Longitude_Range() );
-
-	case 14:	return( new CGCS_Graticule() );
-
-	case 15:	return( new CCRS_Picker() );
-
-	case 16:	return( new CCRS_Indicatrix() );
-	case 17:	return( new CCRS_Grid_GeogCoords() );
-
-	case 18:	return( new CCRS_Transform_PointCloud(true ) );
-	case 19:	return( new CCRS_Transform_PointCloud(false) );
-
-	case 20:	return( NULL );
+	case 23:	return( NULL );
 	default:	return( MLB_INTERFACE_SKIP_MODULE );
 	}
 }
