@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: crs_base.cpp 2150 2014-06-10 14:47:45Z reklov_w $
+ * Version $Id: crs_base.cpp 2469 2015-04-17 13:02:24Z reklov_w $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -146,7 +146,7 @@ CCRS_Base::CCRS_Base(void)
 	pNode_1	= Parameters.Add_Value(
 		pNode_0	, "CRS_EPSG"	, _TL("EPSG Code"),
 		_TL(""),
-		PARAMETER_TYPE_Int, m_Projection.Get_EPSG(), -1, true, 99999, true
+		PARAMETER_TYPE_Int, m_Projection.Get_EPSG(), -1, true
 	);
 
 	if( SG_UI_Get_Window_Main() )
@@ -338,27 +338,33 @@ int CCRS_Base::On_Parameters_Enable(CSG_Parameters *pParameters, CSG_Parameter *
 //---------------------------------------------------------
 bool CCRS_Base::Get_Projection(CSG_Projection &Projection)
 {
-	if( Parameters("CRS_METHOD") )
-	{
-		switch( Parameters("CRS_METHOD")->asInt() )
-		{
-		case 0:	default:	// Proj4 Parameters
-			Projection.Create(Parameters("CRS_PROJ4")->asString(), SG_PROJ_FMT_Proj4);
-			break;
-
-		case 1:				// EPSG Code
-			Projection.Create(Parameters("CRS_EPSG" )->asInt());
-			break;
-
-		case 2:				// Well Known Text File
-			Projection.Load (Parameters("CRS_FILE"  )->asString());
-			break;
-		}
-	}
-	else
+	if( !Parameters("CRS_METHOD") )
 	{
 		Projection	= m_Projection;
 	}
+	else switch( Parameters("CRS_METHOD")->asInt() )
+	{
+	default:	// Proj4 Parameters
+		if( !Projection.Create(Parameters("CRS_PROJ4")->asString(), SG_PROJ_FMT_Proj4) )
+		{
+			Error_Set(_TL("Proj4 definition string error"));
+		}
+		break;
+
+	case  1:	// EPSG Code
+		if( !Projection.Create(Parameters("CRS_EPSG" )->asInt()) )
+		{
+			Error_Set(_TL("EPSG code error"));
+		}
+		break;
+
+	case  2:	// Well Known Text File
+		if( !Projection.Load (Parameters("CRS_FILE"  )->asString()) )
+		{
+			Error_Set(_TL("Well Known Text file error"));
+		}
+		break;
+	}
 
 	return( Projection.is_Okay() );
 }
