@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: Grid_Classes_To_Shapes.cpp 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: Grid_Classes_To_Shapes.cpp 2241 2014-09-22 11:04:27Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -119,6 +119,12 @@ CGrid_Classes_To_Shapes::CGrid_Classes_To_Shapes(void)
 			_TL("each island as separated polygon")
 		), 0
 	);
+
+	Parameters.Add_Value(
+		NULL	, "ALLVERTICES"	, _TL("Keep Vertices on Straight Lines"),
+		_TL(""),
+		PARAMETER_TYPE_Bool, false
+	);
 }
 
 
@@ -160,7 +166,7 @@ bool CGrid_Classes_To_Shapes::On_Execute(void)
 //---------------------------------------------------------
 bool CGrid_Classes_To_Shapes::Get_Classes(void)
 {
-	long		i;
+	sLong		i;
 	int			id, x, y;
 	double		Value;
 	CSG_Grid	*pGrid;
@@ -192,6 +198,13 @@ bool CGrid_Classes_To_Shapes::Get_Classes(void)
 	//-----------------------------------------------------
 	if( Parameters("CLASS_ALL")->asInt() == 1 )
 	{
+		if( !pGrid->Set_Index() )
+		{
+			Error_Set(_TL("index creation failed"));
+
+			return( false );
+		}
+
 		for(i=0, id=-1; i<Get_NCells() && Set_Progress_NCells(i); i++)
 		{
 			if( pGrid->Get_Sorted(i, x, y, false) )
@@ -256,6 +269,8 @@ bool CGrid_Classes_To_Shapes::Get_Edges(void)
 	m_Edges.Set_NoData_Value(-2);
 	m_Edges.Assign_NoData();
 
+	m_bAllVertices	= Parameters("ALLVERTICES")->asBool();
+
 	//-----------------------------------------------------
 	for(y=0; y<Get_NY() && Process_Get_Okay(); y++)
 	{
@@ -368,7 +383,10 @@ bool CGrid_Classes_To_Shapes::Get_Edge(int x, int y, int i, int Class)
 
 			if( m_Edges.is_InGrid(ix, iy) && m_Edges.asInt(ix, iy) == -1 )	// go ahead ?
 			{
-				// nop
+				if( m_bAllVertices )
+				{
+					pPolygon->Add_Point(m_Edges.Get_System().Get_Grid_to_World(x, y), iPart);
+				}
 			}
 			else
 			{
