@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: Grid_Classes_To_Shapes.h 1246 2011-11-25 13:42:38Z oconrad $
+ * Version $Id: Grid_Classes_To_Shapes.h 2159 2014-06-12 16:07:02Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -35,7 +35,7 @@
 // You should have received a copy of the GNU General    //
 // Public License along with this program; if not,       //
 // write to the Free Software Foundation, Inc.,          //
-// 59 Temple Place - Suite 330, Boston, MA 02111-1307,   //
+// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
 // USA.                                                  //
 //                                                       //
 //-------------------------------------------------------//
@@ -90,6 +90,8 @@ protected:
 
 private:
 
+	bool					m_bAllVertices;
+
 	CSG_Grid				m_Classes, m_Edges;
 
 	CSG_Shapes				*m_pPolygons;
