@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: Grid_To_Points.cpp 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: Grid_To_Points.cpp 2208 2014-08-27 16:00:51Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -135,7 +135,7 @@ CGrid_To_Points::~CGrid_To_Points(void)
 //---------------------------------------------------------
 bool CGrid_To_Points::On_Execute(void)
 {
-	bool					bZFactor, bNoNoData;
+	bool					bNoNoData;
 	int						x, y, iGrid, iPoint, Type;
 	double					xPos, yPos;
 	CSG_Grid				*pGrid;
@@ -150,8 +150,6 @@ bool CGrid_To_Points::On_Execute(void)
 	bNoNoData	= Parameters("NODATA")	->asBool();
 	Type		= Parameters("TYPE")	->asInt();
 
-	bZFactor	= true;
-
 	//-----------------------------------------------------
 	if( pGrids->Get_Count() > 0 )
 	{
@@ -202,7 +200,7 @@ bool CGrid_To_Points::On_Execute(void)
 					{
 						pGrid	= pGrids->asGrid(iGrid);
 
-						pShape->Set_Value(iGrid + 3, pGrid->is_NoData(x, y) ? -99999 : pGrid->asDouble(x, y, bZFactor));
+						pShape->Set_Value(iGrid + 3, pGrid->is_NoData(x, y) ? -99999 : pGrid->asDouble(x, y));
 					}
 				}
 			}
