@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: Points_From_Lines.h 1246 2011-11-25 13:42:38Z oconrad $
+ * Version $Id: Points_From_Lines.h 2545 2015-07-28 07:46:45Z reklov_w $
  *********************************************************/
 
 /*******************************************************************************
@@ -18,7 +18,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, USA
 *******************************************************************************/ 
 
 ///////////////////////////////////////////////////////////
@@ -61,6 +61,13 @@ protected:
 
 	virtual bool				On_Execute		(void);
 
+	virtual int					On_Parameters_Enable		(CSG_Parameters *pParameters, CSG_Parameter *pParameter);
+
+	void						Convert_Add_Points_Segment	(CSG_Shapes *pLines, CSG_Shapes *pPoints, double dDist, bool bAddPtOrder);
+	void						Convert_Add_Points_Line		(CSG_Shapes *pLines, CSG_Shapes *pPoints, double dDist, bool bAddPtOrder);
+	void						Convert_Add_Points_Center	(CSG_Shapes *pLines, CSG_Shapes *pPoints, double dDist, bool bAddPtOrder);
+	void						Convert						(CSG_Shapes *pLines, CSG_Shapes *pPoints, bool bAddPtOrder);
+
 };
 
 
