@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: Polygons_From_Lines.h 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: Polygons_From_Lines.h 2174 2014-06-24 14:43:45Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -97,6 +97,10 @@ protected:
 
 private:
 
+	bool						Add_Part		(CSG_Shape *pPolygon, CSG_Shape *pLine, int iPart_Polygon, int iPart_Line, bool bAscending = true);
+	bool						Add_Line		(CSG_Shape *pPolygon, CSG_Shape *pLine, int iPart_Polygon = 0);
+	bool						Get_Part		(CSG_Point Point, CSG_Shape *pLine, int &iPart, bool &bAscending);
+
 };
 
 
