@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: MLB_Interface.cpp 2010 2014-02-21 14:43:04Z oconrad $
+ * Version $Id: MLB_Interface.cpp 2417 2015-02-19 15:14:52Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -74,7 +74,10 @@ CSG_String Get_Info(int i)
 	switch( i )
 	{
 	case MLB_INFO_Name:	default:
-		return( _TL("Shapes - Tools") );
+		return( _TL("Tools") );
+
+	case MLB_INFO_Category:
+		return( _TL("Shapes") );
 
 	case MLB_INFO_Author:
 		return( SG_T("O. Conrad, V. Olaya, V. Wichmann (c) 2002-2013") );
@@ -121,6 +124,8 @@ CSG_String Get_Info(int i)
 #include "shapes_generate.h"
 #include "shapes_convert_vertex_type.h"
 
+#include "LandUse_Scenario.h"
+
 
 //---------------------------------------------------------
 // 4. Allow your modules to be created here...
@@ -158,6 +163,8 @@ CSG_Module *		Create_Module(int i)
 	case 22:	return( new CShapes_Generate );
 	case 23:	return( new CShapes_Convert_Vertex_Type );
 
+	case 25:	return( new CLandUse_Scenario );
+
 	case 30:	return( NULL );
 	}
 
