@@ -62,17 +62,6 @@
 #include "gw_multi_regression_grid.h"
 
 
-///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
-///////////////////////////////////////////////////////////
-
-//---------------------------------------------------------
-#define GRID_SET_NODATA(g, x, y)	if( g ) { g->Set_NoData(x, y); }
-#define GRID_SET_VALUE(g, x, y, z)	if( g ) { g->Set_Value(x, y, z); }
-
-
 ///////////////////////////////////////////////////////////
 //														 //
 //														 //
@@ -91,17 +80,25 @@ CGW_Multi_Regression_Grid::CGW_Multi_Regression_Grid(void)
 
 	Set_Description	(_TW(
 		"References:\n"
-		"- Fotheringham, S.A., Brunsdon, C., Charlton, M. (2002):"
-		" Geographically Weighted Regression: the analysis of spatially varying relationships. John Wiley & Sons."
-		" <a target=\"_blank\" href=\"http://onlinelibrary.wiley.com/doi/10.1111/j.1538-4632.2003.tb01114.x/abstract\">online</a>.\n"
-		"\n"
-		"- Fotheringham, S.A., Charlton, M., Brunsdon, C. (1998):"
-		" Geographically weighted regression: a natural evolution of the expansion method for spatial data analysis."
-		" Environment and Planning A 30(11), 19051927."
-		" <a target=\"_blank\" href=\"http://www.envplan.com/abstract.cgi?id=a301905\">online</a>.\n"
-		"\n"
-		"- Lloyd, C. (2010): Spatial Data Analysis - An Introduction for GIS Users. Oxford, 206p.\n"
-	));
+	) + GWR_References);
+
+	//-----------------------------------------------------
+	pNode = Parameters.Add_Shapes(
+		NULL	, "POINTS"		, _TL("Points"),
+		_TL(""),
+		PARAMETER_INPUT, SHAPE_TYPE_Point
+	);
+
+	Parameters.Add_Table_Field(
+		pNode	, "DEPENDENT"	, _TL("Dependent Variable"),
+		_TL("")
+	);
+
+	Parameters.Add_Shapes(
+		NULL	, "RESIDUALS"	, _TL("Residuals"),
+		_TL(""),
+		PARAMETER_OUTPUT_OPTIONAL, SHAPE_TYPE_Point
+	);
 
 	//-----------------------------------------------------
 	Parameters.Add_Grid_List(
@@ -134,23 +131,6 @@ CGW_Multi_Regression_Grid::CGW_Multi_Regression_Grid(void)
 		PARAMETER_TYPE_Bool, false
 	);
 
-	pNode	= Parameters.Add_Shapes(
-		NULL	, "POINTS"		, _TL("Points"),
-		_TL(""),
-		PARAMETER_INPUT, SHAPE_TYPE_Point
-	);
-
-	Parameters.Add_Table_Field(
-		pNode	, "DEPENDENT"	, _TL("Dependent Variable"),
-		_TL("")
-	);
-
-	Parameters.Add_Shapes(
-		NULL	, "RESIDUALS"	, _TL("Residuals"),
-		_TL(""),
-		PARAMETER_OUTPUT_OPTIONAL, SHAPE_TYPE_Point
-	);
-
 	//-----------------------------------------------------
 	pNode	= Parameters.Add_Choice(
 		NULL	, "RESOLUTION"	, _TL("Model Resolution"),
@@ -172,72 +152,26 @@ CGW_Multi_Regression_Grid::CGW_Multi_Regression_Grid(void)
 	m_Weighting.Create_Parameters(&Parameters, false);
 
 	//-----------------------------------------------------
-	CSG_Parameter	*pSearch	= Parameters.Add_Node(
-		NULL	, "NODE_SEARCH"			, _TL("Search Options"),
-		_TL("")
-	);
+	m_Search.Create(&Parameters, Parameters.Add_Node(NULL, "NODE_SEARCH", _TL("Search Options"), _TL("")), 16);
 
-	pNode	= Parameters.Add_Choice(
-		pSearch	, "SEARCH_RANGE"		, _TL("Search Range"),
-		_TL(""),
-		CSG_String::Format(SG_T("%s|%s|"),
-			_TL("local"),
-			_TL("global")
-		), 1
-	);
-
-	Parameters.Add_Value(
-		pNode	, "SEARCH_RADIUS"		, _TL("Maximum Search Distance"),
-		_TL("local maximum search distance given in map units"),
-		PARAMETER_TYPE_Double	, 1000.0, 0, true
-	);
-
-	pNode	= Parameters.Add_Choice(
-		pSearch	, "SEARCH_POINTS_ALL"	, _TL("Number of Points"),
-		_TL(""),
-		CSG_String::Format(SG_T("%s|%s|"),
-			_TL("maximum number of nearest points"),
-			_TL("all points within search distance")
-		), 0
-	);
-
-	Parameters.Add_Value(
-		pNode	, "SEARCH_POINTS_MIN"	, _TL("Minimum"),
-		_TL("minimum number of points to use"),
-		PARAMETER_TYPE_Int, 4, 1, true
-	);
-
-	Parameters.Add_Value(
-		pNode	, "SEARCH_POINTS_MAX"	, _TL("Maximum"),
-		_TL("maximum number of nearest points"),
-		PARAMETER_TYPE_Int, 50, 1, true
-	);
-
-	Parameters.Add_Choice(
-		pNode	, "SEARCH_DIRECTION"	, _TL("Search Direction"),
-		_TL(""),
-		CSG_String::Format(SG_T("%s|%s|"),
-			_TL("all directions"),
-			_TL("quadrants")
-		)
-	);
+	Parameters("SEARCH_RANGE"     )->Set_Value(1);
+	Parameters("SEARCH_POINTS_ALL")->Set_Value(1);
 }
 
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 int CGW_Multi_Regression_Grid::On_Parameter_Changed(CSG_Parameters *pParameters, CSG_Parameter *pParameter)
 {
-	if(	!SG_STR_CMP(pParameter->Get_Identifier(), SG_T("POINTS")) && pParameter->asShapes() )
+	if( !SG_STR_CMP(pParameter->Get_Identifier(), "POINTS") && pParameter->asShapes() )
 	{
-		double	d	= pParameter->asShapes()->Get_Extent().Get_XRange() / 20.0;
+		m_Search.On_Parameter_Changed(pParameters, pParameter);
 
-		pParameters->Get_Parameter("RESOLUTION_VAL")->Set_Value(d);
+		pParameters->Set_Parameter("RESOLUTION_VAL", GWR_Fit_To_Density(pParameter->asShapes(), 4.0, 1));
+		pParameters->Set_Parameter("DW_BANDWIDTH"  , GWR_Fit_To_Density(pParameter->asShapes(), 4.0, 1));
 	}
 
 	return( 1 );
@@ -248,20 +182,10 @@ int CGW_Multi_Regression_Grid::On_Parameters_Enable(CSG_Parameters *pParameters,
 {
 	if(	!SG_STR_CMP(pParameter->Get_Identifier(), SG_T("RESOLUTION")) )
 	{
-		pParameters->Get_Parameter("RESOLUTION_VAL"   )->Set_Enabled(pParameter->asInt() == 1);
+		pParameters->Get_Parameter("RESOLUTION_VAL")->Set_Enabled(pParameter->asInt() == 1);
 	}
 
-	if(	!SG_STR_CMP(pParameter->Get_Identifier(), SG_T("SEARCH_RANGE")) )
-	{
-		pParameters->Get_Parameter("SEARCH_RADIUS"    )->Set_Enabled(pParameter->asInt() == 0);	// local
-		pParameters->Get_Parameter("SEARCH_POINTS_MIN")->Set_Enabled(pParameter->asInt() == 0);	// local
-	}
-
-	if(	!SG_STR_CMP(pParameter->Get_Identifier(), SG_T("SEARCH_POINTS_ALL")) )
-	{
-		pParameters->Get_Parameter("SEARCH_POINTS_MAX")->Set_Enabled(pParameter->asInt() == 0);	// maximum number of points
-		pParameters->Get_Parameter("SEARCH_DIRECTION" )->Set_Enabled(pParameter->asInt() == 0);	// maximum number of points per quadrant
-	}
+	m_Search.On_Parameters_Enable(pParameters, pParameter);
 
 	m_Weighting.Enable_Parameters(pParameters);
 
@@ -271,8 +195,6 @@ int CGW_Multi_Regression_Grid::On_Parameters_Enable(CSG_Parameters *pParameters,
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -390,8 +312,6 @@ bool CGW_Multi_Regression_Grid::On_Execute(void)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -452,32 +372,21 @@ bool CGW_Multi_Regression_Grid::Initialize(CSG_Shapes *pPoints, int iDependent,
 	}
 
 	//-----------------------------------------------------
-	m_nPoints_Min	= Parameters("SEARCH_POINTS_MIN")->asInt   ();
-	m_nPoints_Max	= Parameters("SEARCH_POINTS_ALL")->asInt   () == 0
-					? Parameters("SEARCH_POINTS_MAX")->asInt   () : 0;
-	m_Radius		= Parameters("SEARCH_RANGE"     )->asInt   () == 0
-					? Parameters("SEARCH_RADIUS"    )->asDouble() : 0.0;
-	m_Direction		= Parameters("SEARCH_DIRECTION" )->asInt   () == 0 ? -1 : 4;
-
 	m_Weighting.Set_Parameters(&Parameters);
 
-	return( m_Points.Get_Count() > m_nPredictors
-		&& ((m_nPoints_Max <= 0 && m_Radius <= 0.0) || m_Search.Create(&m_Points, -1))
-	);
+	return( m_Points.Get_Count() > m_nPredictors && m_Search.Initialize(&m_Points, -1) );
 }
 
 //---------------------------------------------------------
 void CGW_Multi_Regression_Grid::Finalize(void)
 {
+	m_Search.Finalize();
 	m_Points.Destroy();
-	m_Search.Destroy();
 }
 
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -488,18 +397,17 @@ bool CGW_Multi_Regression_Grid::Get_Model(void)
 	{
 		for(int x=0; x<m_dimModel.Get_NX(); x++)
 		{
-			double		q;
-			CSG_Vector	b;
+			CSG_Regression_Weighted	Model;
 
-			if( Get_Regression(x, y, q, b) )
+			if( Get_Model(x, y, Model) )
 			{
-				m_pQuality->Set_Value(x, y, q);
+				m_pQuality->Set_Value(x, y, Model.Get_R2());
 
-				m_pModel[m_nPredictors]->Set_Value(x, y, b[0]);
+				m_pModel[m_nPredictors]->Set_Value(x, y, Model[0]);
 
 				for(int i=0; i<m_nPredictors; i++)
 				{
-					m_pModel[i]->Set_Value(x, y, b[i + 1]);
+					m_pModel[i]->Set_Value(x, y, Model[i + 1]);
 				}
 			}
 			else
@@ -519,119 +427,45 @@ bool CGW_Multi_Regression_Grid::Get_Model(void)
 }
 
 //---------------------------------------------------------
-bool CGW_Multi_Regression_Grid::Get_Regression(int x, int y, double &Quality, CSG_Vector &b)
+bool CGW_Multi_Regression_Grid::Get_Model(int x, int y, CSG_Regression_Weighted &Model)
 {
 	//-----------------------------------------------------
-	int			i, nPoints;
-	double		zMean, zr, rss, tss;
-	CSG_Vector	z, w;
-	CSG_Matrix	Y, YtW;
-
-	if( (nPoints = Get_Variables(x, y, z, w, Y)) <= m_nPredictors )
-	{
-		return( false );
-	}
-
-	//-----------------------------------------------------
-	YtW.Create(nPoints, 1 + m_nPredictors);
-
-	for(i=0, zMean=0.0; i<nPoints; i++)
-	{
-		zMean		+= z[i];
-		YtW[0][i]	 = w[i];
-
-		for(int j=1; j<=m_nPredictors; j++)
-		{
-			YtW[j][i]	= Y[i][j] * w[i];
-		}
-	}
-
-	//-----------------------------------------------------
-	b		= (YtW * Y).Get_Inverse() * (YtW * z);
-
-	zMean	/= nPoints;
-
-	for(i=0, rss=0.0, tss=0.0; i<nPoints; i++)
-	{
-		zr	= b[0];
-
-		for(int j=1; j<=m_nPredictors; j++)
-		{
-			zr	+= b[j] * Y[i][j];
-		}
-
-		rss	+= w[i] * SG_Get_Square(z[i] - zr);
-		tss	+= w[i] * SG_Get_Square(z[i] - zMean);
-	}
-
-	Quality	= tss > 0.0 ? (tss - rss) / tss : 0.0;
-
-	//-----------------------------------------------------
-	return( true );
-}
-
-//---------------------------------------------------------
-int CGW_Multi_Regression_Grid::Get_Variables(int x, int y, CSG_Vector &z, CSG_Vector &w, CSG_Matrix &Y)
-{
 	TSG_Point	Point	= m_dimModel.Get_Grid_to_World(x, y);
-	int			nPoints	= m_Search.is_Okay() ? (int)m_Search.Select_Nearest_Points(Point.x, Point.y, m_nPoints_Max, m_Radius, m_Direction) : m_Points.Get_Count();
+	int			nPoints = m_Search.Set_Location(Point);
 
-	z.Create(nPoints);
-	w.Create(nPoints);
-	Y.Create(1 + m_nPredictors, nPoints);
+	CSG_Vector	Predictors(m_nPredictors);
+
+	Model.Destroy();
 
 	for(int iPoint=0; iPoint<nPoints; iPoint++)
 	{
 		double	ix, iy, iz;
 
-		CSG_Shape	*pPoint	= m_Search.is_Okay() && m_Search.Get_Selected_Point(iPoint, ix, iy, iz)
+		CSG_Shape	*pPoint = m_Search.Do_Use_All() && m_Search.Get_Point(iPoint, ix, iy, iz)
 			? m_Points.Get_Shape((int)iz)
 			: m_Points.Get_Shape(iPoint);
 
-		z[iPoint]		= pPoint->asDouble(0);
-		w[iPoint]		= m_Weighting.Get_Weight(SG_Get_Distance(Point, pPoint->Get_Point(0)));
-		Y[iPoint][0]	= 1.0;
-
-		for(int iPredictor=1; iPredictor<=m_nPredictors; iPredictor++)
+		for(int iPredictor=0; iPredictor<m_nPredictors; iPredictor++)
 		{
-			Y[iPoint][iPredictor]	= pPoint->asDouble(iPredictor);
+			Predictors[iPredictor]	= pPoint->asDouble(1 + iPredictor);
 		}
+
+		Model.Add_Sample(
+			m_Weighting.Get_Weight(SG_Get_Distance(Point, pPoint->Get_Point(0))),
+			pPoint->asDouble(0),
+			Predictors
+		);
 	}
 
-	return( nPoints );
+	//-----------------------------------------------------
+	return( Model.Calculate() );
 }
 
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
-//---------------------------------------------------------
-bool CGW_Multi_Regression_Grid::Set_Model(double x, double y, double &Value)
-{
-	if( !m_pModel[m_nPredictors]->Get_Value(x, y, Value, GRID_INTERPOLATION_BSpline) )
-	{
-		return( false );
-	}
-
-	double	Model, Predictor;
-
-	for(int i=0; i<m_nPredictors; i++)
-	{
-		if( !m_pModel     [i]->Get_Value(x, y, Model    , GRID_INTERPOLATION_BSpline)
-		||  !m_pPredictors[i]->Get_Value(x, y, Predictor, GRID_INTERPOLATION_NearestNeighbour) )
-		{
-			return( false );
-		}
-
-		Value	+= Model * Predictor;
-	}
-
-	return( true );
-}
-
 //---------------------------------------------------------
 bool CGW_Multi_Regression_Grid::Set_Model(void)
 {
@@ -658,13 +492,13 @@ bool CGW_Multi_Regression_Grid::Set_Model(void)
 
 			if( Set_Model(p_x, p_y, Value) )
 			{
-				GRID_SET_VALUE(pRegression, x, y, Value);
-				GRID_SET_VALUE(pQuality   , x, y, m_pQuality->Get_Value(p_x, p_y));
+				SG_GRID_PTR_SAFE_SET_VALUE(pRegression, x, y, Value);
+				SG_GRID_PTR_SAFE_SET_VALUE(pQuality   , x, y, m_pQuality->Get_Value(p_x, p_y));
 			}
 			else
 			{
-				GRID_SET_NODATA(pRegression, x, y);
-				GRID_SET_NODATA(pQuality   , x, y);
+				SG_GRID_PTR_SAFE_SET_NODATA(pRegression, x, y);
+				SG_GRID_PTR_SAFE_SET_NODATA(pQuality   , x, y);
 			}
 		}
 	}
@@ -675,11 +509,33 @@ bool CGW_Multi_Regression_Grid::Set_Model(void)
 	return( true );
 }
 
+//---------------------------------------------------------
+bool CGW_Multi_Regression_Grid::Set_Model(double x, double y, double &Value)
+{
+	if( !m_pModel[m_nPredictors]->Get_Value(x, y, Value, GRID_INTERPOLATION_BSpline) )
+	{
+		return( false );
+	}
+
+	double	Model, Predictor;
+
+	for(int i=0; i<m_nPredictors; i++)
+	{
+		if( !m_pModel     [i]->Get_Value(x, y, Model    , GRID_INTERPOLATION_BSpline)
+		||  !m_pPredictors[i]->Get_Value(x, y, Predictor, GRID_INTERPOLATION_NearestNeighbour) )
+		{
+			return( false );
+		}
+
+		Value	+= Model * Predictor;
+	}
+
+	return( true );
+}
+
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
