@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: Fit.cpp 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: Fit.cpp 2447 2015-03-19 14:43:42Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -239,7 +239,7 @@ bool CFit::On_Execute(void)
 		if (uservars[i] >='a' && uservars[i] <= 'z')
 		{
 			if (uservars[i] != 'x')
-				vars[NrVars++] = uservars[i];
+				vars[NrVars++] = (char)(uservars[i]);
 		}
 	}
 	
