@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: table_cluster_analysis.cpp 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: table_cluster_analysis.cpp 2343 2014-11-26 16:21:11Z reklov_w $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -69,17 +69,17 @@
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-CTable_Cluster_Analysis::CTable_Cluster_Analysis(void)
+CTable_Cluster_Analysis::CTable_Cluster_Analysis(bool bShapes)
 {
+	m_bShapes	= bShapes;
+
 	CSG_Parameter	*pNode;
 
 	//-----------------------------------------------------
-	Set_Name		(_TL("Cluster Analysis"));
-
-	Set_Author		(SG_T("O. Conrad (c) 2010"));
+	Set_Author		("O. Conrad (c) 2010");
 
 	Set_Description	(_TW(
-		"Cluster Analysis for grids.\n\nReferences:\n\n"
+		"Cluster Analysis for tables.\n\nReferences:\n\n"
 		                                                                                                                                                                                                                  
 		"Iterative Minimum Distance:\n"
 		"- Forgy, E. (1965):\n"
@@ -93,11 +93,22 @@ CTable_Cluster_Analysis::CTable_Cluster_Analysis(void)
 	));
 
 	//-----------------------------------------------------
-	pNode	= Parameters.Add_Table(
-		NULL	, "TABLE"		, _TL("Table"),
-		_TL(""),
-		PARAMETER_INPUT
-	);
+	if( m_bShapes )
+	{
+		Set_Name		(_TL("Cluster Analysis (Shapes)"));
+
+		pNode	= 
+		Parameters.Add_Shapes(NULL, "INPUT" , _TL("Shapes"), _TL(""), PARAMETER_INPUT);
+		Parameters.Add_Shapes(NULL, "RESULT", _TL("Result"), _TL(""), PARAMETER_OUTPUT_OPTIONAL);
+	}
+	else
+	{
+		Set_Name		(_TL("Cluster Analysis"));
+
+		pNode	=
+		Parameters.Add_Table(NULL, "INPUT"  , _TL("Table" ), _TL(""), PARAMETER_INPUT);
+		Parameters.Add_Table(NULL, "RESULT" , _TL("Result"), _TL(""), PARAMETER_OUTPUT_OPTIONAL);
+	}
 
 	Parameters.Add_Table_Fields(
 		pNode	, "FIELDS"		, _TL("Attributes"),
@@ -110,12 +121,6 @@ CTable_Cluster_Analysis::CTable_Cluster_Analysis(void)
 		true
 	);
 
-	Parameters.Add_Table(
-		NULL	, "RESULT"		, _TL("Result"),
-		_TL(""),
-		PARAMETER_OUTPUT_OPTIONAL
-	);
-
 	Parameters.Add_Table(
 		NULL	, "STATISTICS"	, _TL("Statistics"),
 		_TL(""),
@@ -182,13 +187,20 @@ bool CTable_Cluster_Analysis::On_Execute(void)
 		return( false );
 	}
 
-	if( pTable && pTable != Parameters("TABLE")->asTable() )
+	if( pTable && pTable != Parameters("INPUT")->asTable() )
 	{
-		pTable->Create(*Parameters("TABLE")->asTable());
+		if( m_bShapes )
+		{
+			((CSG_Shapes *)pTable)->Create(*Parameters("INPUT")->asShapes());
+		}
+		else
+		{
+			pTable->Create(*Parameters("INPUT")->asTable());
+		}
 	}
 	else
 	{
-		pTable	= Parameters("TABLE")->asTable();
+		pTable	= Parameters("INPUT")->asTable();
 	}
 
 	if( Cluster < 0 )
