@@ -35,7 +35,7 @@
 // You should have received a copy of the GNU General    //
 // Public License along with this program; if not,       //
 // write to the Free Software Foundation, Inc.,          //
-// 59 Temple Place - Suite 330, Boston, MA 02111-1307,   //
+// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
 // USA.                                                  //
 //                                                       //
 //-------------------------------------------------------//
@@ -116,10 +116,10 @@ public:
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-class CTable_Append_Rows : public CSG_Module  
+class CTable_Append_Cols : public CSG_Module  
 {
 public:
-	CTable_Append_Rows(void);
+	CTable_Append_Cols(void);
 
 
 protected:
