@@ -64,6 +64,19 @@
 #include "Flow.h"
 
 
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+#define SET_GRID_TO(grd, val)	if( grd )	{	grd->Assign(val);	}
+
+#define SET_GRID_CELL_VAL(x, y, grid, val)	if( grid )	{	grid->Set_Value(x, y, val);	}
+#define ADD_GRID_CELL_VAL(x, y, grid, val)	if( grid )	{	grid->Add_Value(x, y, val);	}
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 //														 //
@@ -73,101 +86,40 @@
 //---------------------------------------------------------
 CFlow::CFlow(void)
 {
-	pCatch_AspectY	= NULL;
-	bPoint			= false;
+	m_bPoint	= false;
 
 	//-----------------------------------------------------
-	Set_Description (_TW(
-		""
-	));
+	Parameters.Add_Grid(NULL, "ELEVATION" , _TL("Elevation"                                ), _TL(""), PARAMETER_INPUT);
+	Parameters.Add_Grid(NULL, "SINKROUTE" , _TL("Sink Routes"                              ), _TL(""), PARAMETER_INPUT_OPTIONAL);
+	Parameters.Add_Grid(NULL, "WEIGHT"    , _TL("Weight"                                   ), _TL(""), PARAMETER_INPUT_OPTIONAL);
 
+	Parameters.Add_Grid(NULL, "CAREA"     , _TL("Flow Accumulation"                        ), _TL(""), PARAMETER_OUTPUT);
 
-	//-----------------------------------------------------
-	// Input...
+	Parameters.Add_Grid(NULL, "VAL_INPUT" , _TL("Input for Mean over Catchment Calculation"), _TL(""), PARAMETER_INPUT_OPTIONAL);
+	Parameters.Add_Grid(NULL, "VAL_MEAN"  , _TL("Mean over Catchment"                      ), _TL(""), PARAMETER_OUTPUT);
 
-	Parameters.Add_Grid(
-		NULL	, "ELEVATION"	, _TL("Elevation"),
-		_TL(""),
-		PARAMETER_INPUT
-	);
-
-	Parameters.Add_Grid(
-		NULL	, "SINKROUTE"	, _TL("Sink Routes"),
-		_TL(""),
-		PARAMETER_INPUT_OPTIONAL
-	);
-
-	Parameters.Add_Grid(
-		NULL	, "WEIGHT"		, _TL("Weight"),
-		_TL(""),
-		PARAMETER_INPUT_OPTIONAL
-	);
-
-	Parameters.Add_Grid(
-		NULL	, "MATERIAL"	, _TL("Material"),
-		_TL(""),
-		PARAMETER_INPUT_OPTIONAL
-	);
-
-	Parameters.Add_Grid(
-		NULL	, "TARGET"		, _TL("Target"),
-		_TL(""),
-		PARAMETER_INPUT_OPTIONAL
-	);	
-
-
-	//-----------------------------------------------------
-	// Output...
-
-	Parameters.Add_Grid(
-		NULL	, "CAREA"		, _TL("Catchment Area"),
-		_TL(""),
-		PARAMETER_OUTPUT
-	);
-
-	Parameters.Add_Grid(
-		NULL	, "CHEIGHT"		, _TL("Catchment Height"),
-		_TL(""),
-		PARAMETER_OUTPUT_OPTIONAL
-	);
-
-	Parameters.Add_Grid(
-		NULL	, "CSLOPE"		, _TL("Catchment Slope"),
-		_TL(""),
-		PARAMETER_OUTPUT_OPTIONAL
-	);
-
-	Parameters.Add_Grid(
-		NULL	, "ACCU_TOT"	, _TL("Total accumulated Material"),
-		_TL(""),
-		PARAMETER_OUTPUT_OPTIONAL
-	);	
-
-	Parameters.Add_Grid(
-		NULL	, "ACCU_LEFT"	, _TL("Accumulated Material from _left_ side"),
-		_TL(""),
-		PARAMETER_OUTPUT_OPTIONAL
-	);		
-
-	Parameters.Add_Grid(
-		NULL	, "ACCU_RIGHT"	, _TL("Accumulated Material from _right_ side"),
-		_TL(""),
-		PARAMETER_OUTPUT_OPTIONAL
-	);		
+	Parameters.Add_Grid(NULL, "MATERIAL"  , _TL("Material"                                 ), _TL(""), PARAMETER_INPUT_OPTIONAL);
+	Parameters.Add_Grid(NULL, "TARGET"    , _TL("Accumulation Target"                      ), _TL(""), PARAMETER_INPUT_OPTIONAL);
+	Parameters.Add_Grid(NULL, "ACCU_TOT"  , _TL("Total accumulated Material"               ), _TL(""), PARAMETER_OUTPUT_OPTIONAL);	
+	Parameters.Add_Grid(NULL, "ACCU_LEFT" , _TL("Accumulated Material from Left Side"      ), _TL(""), PARAMETER_OUTPUT_OPTIONAL);		
+	Parameters.Add_Grid(NULL, "ACCU_RIGHT", _TL("Accumulated Material from Right Side"     ), _TL(""), PARAMETER_OUTPUT_OPTIONAL);		
 	
 	//-----------------------------------------------------
-	// Options...
-
 	Parameters.Add_Value(
 		NULL	, "STEP"		, _TL("Step"),
 		_TL(""),
 		PARAMETER_TYPE_Int, 1, 1, true
 	);
-}
 
-//---------------------------------------------------------
-CFlow::~CFlow(void)
-{}
+	Parameters.Add_Choice(
+		NULL	, "CAREA_UNIT"	, _TL("Flow Accumulation Unit"),
+		_TL(""),
+		CSG_String::Format("%s|%s|",
+			_TL("number of cells"),
+			_TL("cell area")
+		), 1
+	);
+}
 
 
 ///////////////////////////////////////////////////////////
@@ -179,121 +131,122 @@ CFlow::~CFlow(void)
 //---------------------------------------------------------
 void CFlow::Set_Point(int x, int y)
 {
-	bPoint	= true;
-	xPoint	= x;
-	yPoint	= y;
+	m_bPoint	= true;
+
+	m_xPoint	= x;
+	m_yPoint	= y;
 }
 
+
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+int CFlow::On_Parameters_Enable(CSG_Parameters *pParameters, CSG_Parameter *pParameter)
+{
+	if( !SG_STR_CMP(pParameter->Get_Identifier(), "VAL_INPUT") )
+	{
+		pParameters->Set_Enabled("VAL_MEAN", pParameter->asGrid() != NULL);
+	}
+
+	if( !SG_STR_CMP(pParameter->Get_Identifier(), "WEIGHT") )
+	{
+		pParameters->Set_Enabled("WEIGHT_GT_0", pParameter->asGrid() != NULL);
+	}
+
+	return( CSG_Module_Grid::On_Parameters_Enable(pParameters, pParameter) );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
 //---------------------------------------------------------
 bool CFlow::On_Execute(void)
 {
-	bool	bResult	= false;
-	sLong	n;
-	double	d;
-
 	//-------------------------------------------------
-	pDTM			= Parameters("ELEVATION")	->asGrid();
-	pRoute			= Parameters("SINKROUTE")	->asGrid();
-	pWeight			= Parameters("WEIGHT")		->asGrid();
-	pMaterial		= Parameters("MATERIAL")	->asGrid();
-	pTarget			= Parameters("TARGET")		->asGrid();
+	m_pDTM			= Parameters("ELEVATION")->asGrid();
+	m_pRoute		= Parameters("SINKROUTE")->asGrid();
+	m_pWeight		= Parameters("WEIGHT"   )->asGrid();
+	m_pMaterial		= Parameters("MATERIAL" )->asGrid();
+	m_pTarget		= Parameters("TARGET"   )->asGrid();
+	m_pCatch		= Parameters("CAREA"    )->asGrid();
 
-	pCatch			= Parameters("CAREA")		->asGrid();
-	DataObject_Set_Colors(pCatch, 100, SG_COLORS_WHITE_BLUE);
-	pCatch_Height	= NULL;
-	pCatch_Slope	= NULL;
-	pCatch_Aspect	= NULL;
-	pFlowPath		= NULL;
-	pAccu_Tot		= NULL;
-	pAccu_Left		= NULL;
-	pAccu_Right		= NULL;
+	m_pFlowPath		= NULL;
 
-	Step			= Parameters("STEP")		->asInt();
-
-	//-------------------------------------------------
-	On_Initialize();
-
-	//-------------------------------------------------
-	if( pCatch )
+	if( (m_pVal_Input = Parameters("VAL_INPUT")->asGrid()) != NULL
+	&&  (m_pVal_Mean  = Parameters("VAL_MEAN" )->asGrid()) != NULL )
 	{
-		pCatch			->Assign(0.0);
-	}
-
-	if( pCatch_Height )
-	{
-		pCatch_Height	->Assign(0.0);
-	}
-
-	if( pCatch_Slope )
-	{
-		pCatch_Slope	->Assign(0.0);
-	}
-
-	if( pCatch_Aspect )
-	{
-		pCatch_Aspect	->Assign(0.0);
-		pCatch_AspectY	 = SG_Create_Grid(pCatch_Aspect);
-	}
-
-	if( pFlowPath )
-	{
-		pFlowPath		->Assign(0.0);
-	}
-
-	if( pAccu_Tot )
-	{
-		pAccu_Tot		->Assign(1.0);
-	}	
-
-	if( pAccu_Left )
-	{
-		pAccu_Left		->Assign(1.0);
-	}	
-
-	if( pAccu_Right )
-	{
-		pAccu_Right		->Assign(1.0);
-	}	
-	
-	//-------------------------------------------------
-	if( bPoint )
-	{
-		bPoint	= false;
-
-		if( is_InGrid(xPoint, yPoint) )
-		{
-			Calculate(xPoint, yPoint);
-
-			On_Finalize();
-
-			for(n=0; n<Get_NCells(); n++)
-			{
-				d	= pCatch->asDouble(n);
-				pCatch->Set_Value(n, 100.0 * d);
-			}
-
-			bResult		= true;
-		}
+		m_pVal_Mean->Set_Name(CSG_String::Format("%s [%s]", m_pVal_Input->Get_Name(), _TL("Mean over Catchment")));
+		m_pVal_Mean->Set_Unit(m_pVal_Input->Get_Unit());
 	}
 	else
 	{
-		pCatch_Height	= Parameters("CHEIGHT")	->asGrid();
-		pCatch_Slope	= Parameters("CSLOPE")	->asGrid();
-		
-		pAccu_Tot		= Parameters("ACCU_TOT")	->asGrid();
-		pAccu_Left		= Parameters("ACCU_LEFT")	->asGrid();
-		pAccu_Right		= Parameters("ACCU_RIGHT")	->asGrid();
+		m_pVal_Mean	= NULL;
+	}
+
+	m_pAccu_Tot		= NULL;
+	m_pAccu_Left	= NULL;
+	m_pAccu_Right	= NULL;
+
+	m_Step			= Parameters("STEP")->asInt();
+	m_bGT_Zero		= false;
+
+	//-----------------------------------------------------
+	On_Initialize();
+
+	SET_GRID_TO(m_pCatch     , 0.0);
+	SET_GRID_TO(m_pFlowPath  , 0.0);
+	SET_GRID_TO(m_pVal_Mean  , 0.0);
+	SET_GRID_TO(m_pAccu_Tot  , 1.0);
+	SET_GRID_TO(m_pAccu_Left , 1.0);
+	SET_GRID_TO(m_pAccu_Right, 1.0);
+
+	DataObject_Set_Colors(m_pCatch, 11, SG_COLORS_WHITE_BLUE);
+	
+	//-----------------------------------------------------
+	if( m_bPoint )
+	{
+		m_bPoint	= false;
+
+		if( is_InGrid(m_xPoint, m_yPoint) )
+		{
+			Calculate(m_xPoint, m_yPoint);
+
+			On_Finalize();
+
+			m_pCatch->Multiply(100.0);	// output as percentage
+
+			return( true );
+		}
+	}
+
+	//-----------------------------------------------------
+	else
+	{
+		m_pAccu_Tot		= Parameters("ACCU_TOT"  )->asGrid();
+		m_pAccu_Left	= Parameters("ACCU_LEFT" )->asGrid();
+		m_pAccu_Right	= Parameters("ACCU_RIGHT")->asGrid();
 		
+		DataObject_Set_Colors(m_pFlowPath, 11, SG_COLORS_RED_GREY_BLUE);
+
 		Calculate();
 
 		On_Finalize();
-		Finalize();
 
-		bResult			= true;
+		_Finalize();
+
+		return( true );
 	}
 
-	//-------------------------------------------------
-	return( bResult );
+	//-----------------------------------------------------
+	return( false );
 }
 
 
@@ -306,51 +259,20 @@ bool CFlow::On_Execute(void)
 //---------------------------------------------------------
 void CFlow::Init_Cell(int x, int y)
 {
-	double	Weight, Material, Slope, Aspect;
+	double	Weight	= m_pWeight ? m_pWeight->asDouble(x, y) : 1.0;
 
-	Weight		= pWeight   ? pWeight  ->asDouble(x, y) : 1.0;
-	Material	= pMaterial ? pMaterial->asDouble(x, y) : 1.0;
+	ADD_GRID_CELL_VAL(x, y, m_pCatch, Weight);
 
-	if( pCatch )
+	if( m_pVal_Mean && !m_pVal_Input->is_NoData(x, y) )
 	{
-		pCatch			->Add_Value(x, y, Weight);
+		ADD_GRID_CELL_VAL(x, y, m_pVal_Mean, Weight * m_pVal_Input->asDouble(x, y));
 	}
 
-	if( pCatch_Height )
-	{
-		pCatch_Height	->Add_Value(x, y, Weight * pDTM->asDouble(x, y));
-	}
+	Weight	*= m_pMaterial ? m_pMaterial->asDouble(x, y) : 1.0;
 
-	if( pCatch_Slope )
-	{
-		Get_Gradient(x, y, Slope, Aspect);
-
-		pCatch_Slope	->Add_Value(x, y, Weight * Slope);
-	}
-
-	if( pCatch_Aspect && pCatch_AspectY )
-	{
-		Get_Gradient(x, y, Slope, Aspect);
-
-		pCatch_Aspect	->Add_Value(x, y, Weight * sin(Aspect));
-		pCatch_AspectY	->Add_Value(x, y, Weight * cos(Aspect));
-	}
-	
-	if( pAccu_Tot )
-	{
-		pAccu_Tot		->Set_Value(x, y, Weight * Material);
-	}	
-
-	if( pAccu_Left )
-	{
-		pAccu_Left		->Set_Value(x, y, Weight * Material);
-	}	
-
-	if( pAccu_Right )
-	{
-		pAccu_Right		->Set_Value(x, y, Weight * Material);
-	}	
-	
+	SET_GRID_CELL_VAL(x, y, m_pAccu_Tot  , Weight);
+	SET_GRID_CELL_VAL(x, y, m_pAccu_Left , Weight);
+	SET_GRID_CELL_VAL(x, y, m_pAccu_Right, Weight);
 }
 
 
@@ -361,90 +283,62 @@ void CFlow::Init_Cell(int x, int y)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-void CFlow::Finalize(void)
+void CFlow::_Finalize(void)
 {
-	for(sLong n=0; n<Get_NCells() && Set_Progress_NCells(n); n++)
+	bool	bCellsToArea	= Parameters("CAREA_UNIT")->asInt() == 1;
+
+	#pragma omp parallel for
+	for(sLong n=0; n<Get_NCells(); n++)
 	{
-		if( pDTM->is_NoData(n) )
+		if( m_pDTM->is_NoData(n) )
 		{
-			if( pCatch )		{	pCatch			->Set_NoData(n);	}
-			if( pCatch_Height )	{	pCatch_Height	->Set_NoData(n);	}
-			if( pCatch_Slope )	{	pCatch_Slope	->Set_NoData(n);	}
-			if( pCatch_Aspect )	{	pCatch_Aspect	->Set_NoData(n);	}
-			if( pFlowPath )		{	pFlowPath		->Set_NoData(n);	}
-			if( pAccu_Tot )		{	pAccu_Tot		->Set_NoData(n);	}
-			if( pAccu_Left )	{	pAccu_Left		->Set_NoData(n);	}
-			if( pAccu_Right )	{	pAccu_Right		->Set_NoData(n);	}
+			if( m_pCatch      )	{	m_pCatch     ->Set_NoData(n);	}
+			if( m_pFlowPath   )	{	m_pFlowPath  ->Set_NoData(n);	}
+			if( m_pVal_Mean   )	{	m_pVal_Mean  ->Set_NoData(n);	}
+			if( m_pAccu_Tot   )	{	m_pAccu_Tot  ->Set_NoData(n);	}
+			if( m_pAccu_Left  )	{	m_pAccu_Left ->Set_NoData(n);	}
+			if( m_pAccu_Right )	{	m_pAccu_Right->Set_NoData(n);	}
 		}
 		else
 		{
 			//---------------------------------------------
-			double	Catch	= pCatch->asDouble(n);
+			double	Catch	= m_pCatch->asDouble(n);
 
-			if( pCatch )
+			if( m_pCatch && bCellsToArea )
 			{
-				pCatch	->Set_Value(n, Catch * Get_System()->Get_Cellarea());
+				m_pCatch->Set_Value(n, Catch * Get_System()->Get_Cellarea());
 			}
 
 			if( Catch > 0.0 )
 			{
-				if( pCatch_Height )	{	pCatch_Height	->Set_Value(n, pCatch_Height->asDouble(n) / Catch - pDTM->asDouble(n));	}
-				if( pCatch_Slope )	{	pCatch_Slope	->Mul_Value(n, 1.0 / Catch);	}
-				if( pFlowPath )		{	pFlowPath		->Mul_Value(n, 1.0 / Catch);	}
+				if( m_pFlowPath )	{	m_pFlowPath->Mul_Value(n, 1.0 / Catch);	}
+				if( m_pVal_Mean )	{	m_pVal_Mean->Mul_Value(n, 1.0 / Catch);	}
 			}
 			else
 			{
-				if( pCatch_Height )	{	pCatch_Height	->Set_Value(n, pDTM->asDouble(n));	}
-				if( pCatch_Slope )	{	pCatch_Slope	->Set_Value(n, 0.0);	}
-				if( pFlowPath )		{	pFlowPath		->Set_Value(n, 0.0);	}
+				if( m_pFlowPath )	{	m_pFlowPath->Set_Value(n, 0.0);	}
+				if( m_pVal_Mean )	{	m_pVal_Mean->Set_Value(n, 0.0);	}
 			}
 
 			//---------------------------------------------
-			if( pCatch_Aspect && pCatch_AspectY )
+			if( m_pTarget )
 			{
-				double	G	= pCatch_Aspect	->asDouble(n);
-				double	H	= pCatch_AspectY->asDouble(n);
-
-				pCatch_Aspect	->Set_Value(n, G ? fmod(M_PI_270 + atan2(H, G), M_PI_360) : (H > 0 ? M_PI_270 : (H < 0 ? M_PI_090 : -1)));
-			}
-			
-			//---------------------------------------------
-			double	Material	= pMaterial ? pMaterial->asDouble(n) : 1.0;
-			double	Weight		= pWeight   ? pWeight  ->asDouble(n) : 1.0;
-
-			if( pAccu_Left && pTarget )
-			{
-				if( pTarget->is_NoData(n) )
+				if( m_pTarget->is_NoData(n) )
 				{
-					pAccu_Left	->Set_NoData(n);
+					if( m_pAccu_Left  )	{	m_pAccu_Left ->Set_NoData(n);	}
+					if( m_pAccu_Right )	{	m_pAccu_Right->Set_NoData(n);	}
 				}
 				else
 				{
-					pAccu_Left	->Add_Value(n, - 0.5 * Weight * Material);
+					double	Material	= m_pMaterial ? m_pMaterial->asDouble(n) : 1.0;
+					double	Weight		= m_pWeight   ? m_pWeight  ->asDouble(n) : 1.0;
+
+					if( m_pAccu_Left  )	{	m_pAccu_Left ->Add_Value(n, - 0.5 * Weight * Material);	}
+					if( m_pAccu_Right )	{	m_pAccu_Right->Add_Value(n, - 0.5 * Weight * Material);	}
 				}
 			}
-			
-			if( pAccu_Right && pTarget )
-			{
-				if( pTarget->is_NoData(n) )
-				{	
-					pAccu_Right	->Set_NoData(n);
-				}
-				else
-				{
-					pAccu_Right	->Add_Value(n, - 0.5 * Weight * Material);
-				}
-			}			
 		}
 	}
-
-	//-----------------------------------------------------
-	if( pCatch_AspectY )
-	{
-		delete(pCatch_AspectY);
-
-		pCatch_AspectY	= NULL;
-	}
 }
 
 
@@ -457,10 +351,117 @@ void CFlow::Finalize(void)
 //---------------------------------------------------------
 void CFlow::Get_Gradient(int x, int y, double &Slope, double &Aspect)
 {
-	pDTM->Get_Gradient(x, y, Slope, Aspect);
+	m_pDTM->Get_Gradient(x, y, Slope, Aspect);
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+void CFlow::Add_Fraction(int x, int y, int Direction, double Fraction)
+{
+	if( !is_InGrid(x, y) || Direction < 0 )
+	{
+		return;
+	}
+
+	int	ix	= Get_xTo(Direction, x);
+	int	iy	= Get_yTo(Direction, y);
+		
+	if( !is_InGrid(ix, iy) )
+	{
+		return;
+	}
+
+	ADD_GRID_CELL_VAL(ix, iy, m_pCatch   , Fraction *  m_pCatch   ->asDouble(x, y));
+	ADD_GRID_CELL_VAL(ix, iy, m_pFlowPath, Fraction * (m_pFlowPath->asDouble(x, y) + Get_Length(Direction)));
+	ADD_GRID_CELL_VAL(ix, iy, m_pVal_Mean, Fraction *  m_pVal_Mean->asDouble(x, y));
+
+	if( !m_pTarget )
+	{
+		ADD_GRID_CELL_VAL(ix, iy, m_pAccu_Tot  , Fraction * m_pAccu_Tot      ->asDouble(x, y));
+		ADD_GRID_CELL_VAL(ix, iy, m_pAccu_Left , Fraction * m_pAccu_Left     ->asDouble(x, y));
+		ADD_GRID_CELL_VAL(ix, iy, m_pAccu_Right, Fraction * m_pAccu_Right    ->asDouble(x, y));
+	}
+	else if(  m_pTarget->is_NoData(ix, iy) && m_pTarget->is_NoData(x, y) )
+	{
+		ADD_GRID_CELL_VAL(ix, iy, m_pAccu_Tot  , Fraction * m_pAccu_Tot      ->asDouble(x, y));
+		ADD_GRID_CELL_VAL(ix, iy, m_pAccu_Left , Fraction * m_pAccu_Left     ->asDouble(x, y));
+		ADD_GRID_CELL_VAL(ix, iy, m_pAccu_Right, Fraction * m_pAccu_Right    ->asDouble(x, y));
+	}
+	else if( !m_pTarget->is_NoData(ix, iy) && m_pTarget->is_NoData(x, y) )
+	{
+		bool	left, right;
+
+		Find_Sides(x, y, Direction, left, right);
+
+		ADD_GRID_CELL_VAL(ix, iy, m_pAccu_Tot  , Fraction * m_pAccu_Tot      ->asDouble(x, y));
+
+		if( left && right )
+		{
+			ADD_GRID_CELL_VAL(ix, iy, m_pAccu_Right, Fraction * m_pAccu_Right->asDouble(x, y) * 0.5);
+			ADD_GRID_CELL_VAL(ix, iy, m_pAccu_Left , Fraction * m_pAccu_Left ->asDouble(x, y) * 0.5);
+		}
+		else if( right && !left )
+		{
+			ADD_GRID_CELL_VAL(ix, iy, m_pAccu_Right, Fraction * m_pAccu_Right->asDouble(x, y));
+		}				
+		else if( left && !right )
+		{
+			ADD_GRID_CELL_VAL(ix, iy, m_pAccu_Left , Fraction * m_pAccu_Left ->asDouble(x, y));
+		}
+	}
 }
 
 //---------------------------------------------------------
+void CFlow::Add_Portion(int x, int y, int ix, int iy, int Direction)
+{
+	if( !is_InGrid(x, y) || !is_InGrid(ix, iy) )
+	{
+		return;
+	}
+
+	ADD_GRID_CELL_VAL(ix, iy, m_pCatch   , m_pCatch   ->asDouble(x, y));
+	ADD_GRID_CELL_VAL(ix, iy, m_pFlowPath, m_pFlowPath->asDouble(x, y));
+	ADD_GRID_CELL_VAL(ix, iy, m_pVal_Mean, m_pVal_Mean->asDouble(x, y));
+
+	if( m_pTarget )
+	{
+		if( m_pTarget->is_NoData(ix, iy) && m_pTarget->is_NoData(x, y) )
+		{
+			ADD_GRID_CELL_VAL(ix, iy, m_pAccu_Tot  , m_pAccu_Tot  ->asDouble(x, y));
+			ADD_GRID_CELL_VAL(ix, iy, m_pAccu_Left , m_pAccu_Left ->asDouble(x, y));
+			ADD_GRID_CELL_VAL(ix, iy, m_pAccu_Right, m_pAccu_Right->asDouble(x, y));				
+		}
+		else if( !m_pTarget->is_NoData(ix, iy) && m_pTarget->is_NoData(x, y))
+		{
+			bool	left, right;
+
+			Find_Sides(x, y, Direction, left, right);
+
+			ADD_GRID_CELL_VAL(ix, iy, m_pAccu_Tot, m_pAccu_Tot->asDouble(x, y));
+				
+			if( left && right )
+			{
+				ADD_GRID_CELL_VAL(ix, iy, m_pAccu_Right, m_pAccu_Right->asDouble(x, y) * 0.5);
+				ADD_GRID_CELL_VAL(ix, iy, m_pAccu_Left , m_pAccu_Left ->asDouble(x, y) * 0.5);
+			}
+			else if( right && !left )
+			{
+				ADD_GRID_CELL_VAL(ix, iy, m_pAccu_Right, m_pAccu_Right->asDouble(x, y));
+			}
+			else if( left && !right )
+			{
+				ADD_GRID_CELL_VAL(ix, iy, m_pAccu_Left , m_pAccu_Left ->asDouble(x, y));
+			}				
+		}
+	}
+}
+
 
 ///////////////////////////////////////////////////////////
 //		            SIDE-Algorithm                       //
@@ -515,6 +516,8 @@ void CFlow::Get_Gradient(int x, int y, double &Slope, double &Aspect)
 //														 //
 ///////////////////////////////////////////////////////////
 
+//---------------------------------------------------------
+
 ///////////////////////////////////////////////////////////
 //														 //
 // List of variables:                                    //
@@ -560,6 +563,7 @@ void CFlow::Get_Gradient(int x, int y, double &Slope, double &Aspect)
 //                           tributaries to a junction   //
 ///////////////////////////////////////////////////////////
 
+//---------------------------------------------------------
 void CFlow::Find_Sides(int x, int y, int Direction, bool &left, bool &right)
 {
 	int			FL_Dir, stream1_Dir, stream2_Dir;
@@ -579,7 +583,7 @@ void CFlow::Find_Sides(int x, int y, int Direction, bool &left, bool &right)
 	FL_Dir		        = Direction;
 	stream1_X			= Get_xTo( FL_Dir, x );
 	stream1_Y			= Get_yTo( FL_Dir, y );
-	stream1_Dir			= pTarget->asInt(stream1_X, stream1_Y);
+	stream1_Dir			= m_pTarget->asInt(stream1_X, stream1_Y);
 
 	/*--Note: At this point it is already assumed that FL_Dir points to 
 	    a grid cell that belongs to the stream network. Thus, stream1_X,
@@ -654,10 +658,10 @@ void CFlow::Find_Sides(int x, int y, int Direction, bool &left, bool &right)
 			if( is_InGrid(stream2_X, stream2_Y)	)
 			{
 				// Make sure it is not a missing-value
-				if ( ! pTarget->is_NoData( stream2_X, stream2_Y ) )
+				if ( ! m_pTarget->is_NoData( stream2_X, stream2_Y ) )
 				{
 					// Is the stream cell an upstream tributary?
-					stream2_Dir = pTarget->asInt(stream2_X, stream2_Y);
+					stream2_Dir = m_pTarget->asInt(stream2_X, stream2_Y);
 					is_upstream = stream1_X == Get_xTo(stream2_Dir, stream2_X) && 
 									stream1_Y == Get_yTo(stream2_Dir, stream2_Y);
 								
@@ -726,141 +730,10 @@ void CFlow::Find_Sides(int x, int y, int Direction, bool &left, bool &right)
 }
 
 
-//---------------------------------------------------------
-void CFlow::Add_Fraction(int x, int y, int Direction, double Fraction)
-{
-	int		ix, iy;
-
-	if( is_InGrid(x, y) )
-	{
-		ix	= Get_xTo(Direction, x);
-		iy	= Get_yTo(Direction, y);
-		
-		if( is_InGrid(ix, iy) )
-		{
-			if( pCatch )
-			{
-				pCatch			->Add_Value(ix, iy, Fraction * pCatch			->asDouble(x, y));
-			}
-
-			if( pCatch_Height )
-			{
-				pCatch_Height	->Add_Value(ix, iy, Fraction * pCatch_Height	->asDouble(x, y));
-			}
-
-			if( pCatch_Slope )
-			{
-				pCatch_Slope	->Add_Value(ix, iy, Fraction * pCatch_Slope		->asDouble(x, y));
-			}
-
-			if( pFlowPath )
-			{
-				pFlowPath		->Add_Value(ix, iy, Fraction * (pFlowPath		->asDouble(x, y) + Get_Length(Direction)));
-			}
-
-			if( pCatch_Aspect && pCatch_AspectY )
-			{
-				pCatch_Aspect	->Add_Value(ix, iy, Fraction * pCatch_Aspect	->asDouble(x, y));
-				pCatch_AspectY	->Add_Value(ix, iy, Fraction * pCatch_AspectY	->asDouble(x, y));
-			}
-			
-			if( !pTarget )
-			{
-				if( pAccu_Tot ) pAccu_Tot -> Add_Value(ix, iy, Fraction * pAccu_Tot ->asDouble(x, y));
-				if( pAccu_Left ) pAccu_Left -> Add_Value(ix, iy, Fraction * pAccu_Left ->asDouble(x, y));
-				if( pAccu_Right ) pAccu_Right -> Add_Value(ix, iy, Fraction * pAccu_Right ->asDouble(x, y));				
-			}
-			
-			if( pTarget ) if( pTarget->is_NoData(ix, iy) && pTarget->is_NoData(x, y) )
-			{
-				if( pAccu_Tot ) pAccu_Tot -> Add_Value(ix, iy, Fraction * pAccu_Tot ->asDouble(x, y));
-				if( pAccu_Left ) pAccu_Left -> Add_Value(ix, iy, Fraction * pAccu_Left ->asDouble(x, y));
-				if( pAccu_Right ) pAccu_Right -> Add_Value(ix, iy, Fraction * pAccu_Right ->asDouble(x, y));				
-			}
-			
-
-			if( pTarget ) if( !pTarget->is_NoData(ix, iy) && pTarget->is_NoData(x, y))
-			{
-				
-				bool left, right;
-
-				Find_Sides(x, y, Direction, left, right);
-			
-				if( pAccu_Tot ) pAccu_Tot -> Add_Value(ix, iy, Fraction * pAccu_Tot ->asDouble(x, y));
-				
-				if( right && !left && pAccu_Right ) pAccu_Right -> Add_Value(ix, iy, Fraction * pAccu_Right ->asDouble(x, y));
-				
-				if( left && !right && pAccu_Left ) pAccu_Left -> Add_Value(ix, iy, Fraction * pAccu_Left ->asDouble(x, y));
-				
-				if( left && right )
-				{
-					if( pAccu_Right ) pAccu_Right -> Add_Value(ix, iy, 0.5 * Fraction * pAccu_Right ->asDouble(x, y));
-
-					if( pAccu_Left ) pAccu_Left -> Add_Value(ix, iy, 0.5 * Fraction * pAccu_Left ->asDouble(x, y));			
-				}				
-			}
-		}
-	}
-}
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-void CFlow::Add_Portion(int x, int y, int ix, int iy, int Direction)
-{
-	if( is_InGrid(x, y) && is_InGrid(ix, iy) )
-	{
-		
-		if( pCatch )
-		{
-			pCatch			->Add_Value(ix, iy, pCatch			->asDouble(x, y));
-		}
-
-		if( pCatch_Height )
-		{
-			pCatch_Height	->Add_Value(ix, iy, pCatch_Height	->asDouble(x, y));
-		}
-
-		if( pCatch_Slope )
-		{
-			pCatch_Slope	->Add_Value(ix, iy, pCatch_Slope	->asDouble(x, y));
-		}
-
-		if( pFlowPath )
-		{
-			pFlowPath		->Add_Value(ix, iy, pFlowPath		->asDouble(x, y));
-		}
-
-		if( pCatch_Aspect && pCatch_AspectY )
-		{
-			pCatch_Aspect	->Add_Value(ix, iy, pCatch_Aspect	->asDouble(x, y));
-			pCatch_AspectY	->Add_Value(ix, iy, pCatch_AspectY	->asDouble(x, y));
-		}
-		
-		if( pTarget ) if( pTarget->is_NoData(ix, iy) && pTarget->is_NoData(x, y) )
-		{
-			if( pAccu_Tot ) pAccu_Tot -> Add_Value(ix, iy, pAccu_Tot ->asDouble(x, y));
-			if( pAccu_Left ) pAccu_Left -> Add_Value(ix, iy, pAccu_Left ->asDouble(x, y));
-			if( pAccu_Right ) pAccu_Right -> Add_Value(ix, iy, pAccu_Right ->asDouble(x, y));				
-		}
-		if( pTarget ) if( !pTarget->is_NoData(ix, iy) && pTarget->is_NoData(x, y))
-			{
-				
-				bool left, right;
-
-				Find_Sides(x, y, Direction, left, right);
-					
-				if( pAccu_Tot )pAccu_Tot -> Add_Value(ix, iy, pAccu_Tot->asDouble(x, y));
-				
-				if( right && !left && pAccu_Right ) pAccu_Right -> Add_Value(ix, iy, pAccu_Right->asDouble(x, y));
-				
-				if( left && !right && pAccu_Left ) pAccu_Left -> Add_Value(ix, iy, pAccu_Left->asDouble(x, y));
-				
-				if( left && right )
-				{
-					if( pAccu_Right ) pAccu_Right -> Add_Value(ix, iy, 0.5 * pAccu_Right->asDouble(x, y));
-					if( pAccu_Left ) pAccu_Left -> Add_Value(ix, iy, 0.5 * pAccu_Left->asDouble(x, y));			
-				}
-				
-			}		
-		
-	}
-}
