@@ -82,7 +82,7 @@
 //---------------------------------------------------------
 CFlow_RecursiveDown::CFlow_RecursiveDown(void)
 {
-	Set_Name		(_TL("Catchment Area (Flow Tracing)"));
+	Set_Name		(_TL("Flow Accumulation (Flow Tracing)"));
 
 	Set_Author		(SG_T("O.Conrad (c) 2001"));
 
@@ -113,7 +113,7 @@ CFlow_RecursiveDown::CFlow_RecursiveDown(void)
 	// Method...
 
 	Parameters.Add_Choice(
-		NULL	, "Method"		, _TL("Method"),
+		NULL	, "METHOD"		, _TL("Method"),
 		_TL(""),
 		CSG_String::Format(SG_T("%s|%s|%s|"),
 			_TL("Rho 8"),
@@ -139,10 +139,6 @@ CFlow_RecursiveDown::CFlow_RecursiveDown(void)
 	);
 }
 
-//---------------------------------------------------------
-CFlow_RecursiveDown::~CFlow_RecursiveDown(void)
-{}
-
 
 ///////////////////////////////////////////////////////////
 //														 //
@@ -157,11 +153,11 @@ void CFlow_RecursiveDown::On_Initialize(void)
 	double	Slope, Aspect;
 
 	//-----------------------------------------------------
-	Method			= Parameters("Method"	)->asInt();
-	DEMON_minDQV	= Parameters("MINDQV"	)->asDouble();
-	bFlowPathWeight	= Parameters("CORRECT"	)->asBool();
+	Method			= Parameters("METHOD" )->asInt();
+	DEMON_minDQV	= Parameters("MINDQV" )->asDouble();
+	bFlowPathWeight	= Parameters("CORRECT")->asBool();
 
-	pLinear			= 1 ? SG_Create_Grid(pDTM, SG_DATATYPE_Float) : NULL;
+	pLinear			= 1 ? SG_Create_Grid(m_pDTM, SG_DATATYPE_Float) : NULL;
 
 	//-----------------------------------------------------
 	Lock_Create();
@@ -179,14 +175,14 @@ void CFlow_RecursiveDown::On_Initialize(void)
 	//-----------------------------------------------------
 	case 1:	case 2:		// KRA, DEMON...
 
-		pDir		= SG_Create_Grid(pDTM, SG_DATATYPE_Char);
-		pDif		= SG_Create_Grid(pDTM, SG_DATATYPE_Float);
+		pDir		= SG_Create_Grid(m_pDTM, SG_DATATYPE_Char);
+		pDif		= SG_Create_Grid(m_pDTM, SG_DATATYPE_Float);
 
 		for(y=0; y<Get_NY() && Set_Progress(y); y++)
 		{
 			for(x=0; x<Get_NX(); x++)
 			{
-				if( !pDTM->is_NoData(x, y) )
+				if( !m_pDTM->is_NoData(x, y) )
 				{
 					Get_Gradient(x, y, Slope, Aspect);
 
@@ -206,10 +202,6 @@ void CFlow_RecursiveDown::On_Initialize(void)
 //---------------------------------------------------------
 void CFlow_RecursiveDown::On_Finalize(void)
 {
-	int		x, y, dir;
-	sLong	n;
-	double	qFlow;
-
 	//-----------------------------------------------------
 	if( pDir )
 	{
@@ -226,22 +218,26 @@ void CFlow_RecursiveDown::On_Finalize(void)
 	//-----------------------------------------------------
 	if( pLinear )
 	{
-		for(n=0; n<Get_NCells() && Set_Progress_NCells(n); n++)
+		if( m_pDTM->Set_Index() )
 		{
-			;
-
-			if( pDTM->Get_Sorted(n, x, y) && (qFlow = pLinear->asDouble(x, y)) > 0.0 )
+			for(sLong n=0; n<Get_NCells() && Set_Progress_NCells(n); n++)
 			{
-				Add_Flow(x, y, qFlow);
+				int		x, y, dir;
+				double	qFlow;
 
-				if( (dir = pDTM->Get_Gradient_NeighborDir(x, y)) >= 0 )
+				if( m_pDTM->Get_Sorted(n, x, y) && (qFlow = pLinear->asDouble(x, y)) > 0.0 )
 				{
-					x	= Get_xTo(dir, x);
-					y	= Get_yTo(dir, y);
+					Add_Flow(x, y, qFlow);
 
-					if( pDTM->is_InGrid(x, y) )
+					if( (dir = m_pDTM->Get_Gradient_NeighborDir(x, y)) >= 0 )
 					{
-						pLinear->Add_Value(x, y, qFlow);
+						x	= Get_xTo(dir, x);
+						y	= Get_yTo(dir, y);
+
+						if( m_pDTM->is_InGrid(x, y) )
+						{
+							pLinear->Add_Value(x, y, qFlow);
+						}
 					}
 				}
 			}
@@ -263,9 +259,9 @@ void CFlow_RecursiveDown::On_Finalize(void)
 //---------------------------------------------------------
 bool CFlow_RecursiveDown::Calculate(void)
 {
-	for(int y=0; y<Get_NY() && Set_Progress(y); y+=Step)
+	for(int y=0; y<Get_NY() && Set_Progress(y); y+=m_Step)
 	{	//if( !(y%2) )DataObject_Update(pFlow);
-		for(int x=0; x<Get_NX(); x+=Step)
+		for(int x=0; x<Get_NX(); x+=m_Step)
 		{
 			Calculate(x, y);
 		}
@@ -277,32 +273,21 @@ bool CFlow_RecursiveDown::Calculate(void)
 //---------------------------------------------------------
 bool CFlow_RecursiveDown::Calculate(int x, int y)
 {
-	double 	Slope, Aspect, qFlow;
+	double	qFlow;
 
-	if( !pDTM->is_NoData(x, y) && (qFlow = pWeight ? pWeight->asDouble(x, y) : 1.0) > 0.0 )
+	if( !m_pDTM->is_NoData(x, y) && (qFlow = m_pWeight ? m_pWeight->asDouble(x, y) : 1.0) > 0.0 )
 	{
-		Get_Gradient(x, y, Slope, Aspect);
-
-		Src_Height	= pDTM->asDouble(x,y);
-		Src_Slope	= Slope;
+		Src_Value	= m_pVal_Mean && !m_pVal_Input->is_NoData(x, y) ? m_pVal_Input->asDouble(x, y) : 0.0;
 
 		Add_Flow(x, y, qFlow);
+
 		Lock_Set(x, y, 1);
 
-		//-------------------------------------------------
 		switch( Method )
 		{
-		case 0:
-			Rho8_Start(		x, y, qFlow );
-			break;
-
-		case 1:
-			KRA_Start(		x, y, qFlow );
-			break;
-
-		case 2:
-			DEMON_Start(	x, y, qFlow );
-			break;
+		case 0:	Rho8_Start (x, y, qFlow);	break;
+		case 1:	KRA_Start  (x, y, qFlow);	break;
+		case 2:	DEMON_Start(x, y, qFlow);	break;
 		}
 
 		Lock_Set(x, y, 0);
@@ -321,20 +306,8 @@ bool CFlow_RecursiveDown::Calculate(int x, int y)
 //---------------------------------------------------------
 void CFlow_RecursiveDown::Add_Flow(int x, int y, double Fraction)
 {
-	if( pCatch )
-	{
-		pCatch			->Add_Value(x, y, Fraction );
-	}
-
-	if( pCatch_Height )
-	{
-		pCatch_Height	->Add_Value(x, y, Fraction * Src_Height );
-	}
-
-	if( pCatch_Slope )
-	{
-		pCatch_Slope	->Add_Value(x, y, Fraction * Src_Slope );
-	}
+	if( m_pCatch    )	{	m_pCatch   ->Add_Value(x, y, Fraction);	}
+	if( m_pVal_Mean )	{	m_pVal_Mean->Add_Value(x, y, Fraction * Src_Value );	}
 }
 
 
@@ -428,7 +401,7 @@ void CFlow_RecursiveDown::KRA_Trace(int x, int y, double qFlow, int Direction, d
 	y	= Get_yTo(Direction, y);
 
 	//-----------------------------------------------------
-	if( pDTM->is_InGrid(x, y) && !is_Locked(x, y) )
+	if( m_pDTM->is_InGrid(x, y) && !is_Locked(x, y) )
 	{
 		Lock_Set(x, y, 1);
 
@@ -562,7 +535,7 @@ void CFlow_RecursiveDown::DEMON_Trace(int x, int y, double qFlow, int Direction,
 	y	= Get_yTo(Direction, y);
 
 	//-----------------------------------------------------
-	if( pDTM->is_InGrid(x, y) && !is_Locked(x, y) )
+	if( m_pDTM->is_InGrid(x, y) && !is_Locked(x, y) )
 	{
 		Lock_Set(x, y, 1);
 
@@ -735,3 +708,12 @@ void CFlow_RecursiveDown::DEMON_Trace(int x, int y, double qFlow, int Direction,
 		Lock_Set(x, y, 0);
 	}
 }
+
+
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
