@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: HillShade.cpp 2208 2014-08-27 16:00:51Z oconrad $
+ * Version $Id: HillShade.cpp 2322 2014-11-03 21:29:57Z johanvdw $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -145,7 +145,7 @@ CHillShade::CHillShade(void)
 	Parameters.Add_Value(
 		NULL	, "NDIRS"			, _TL("Number of Directions"),
 		_TW("Number of sample directions for ambient occlusion. Divides azimuth range (270 to 0 to 90 degree) into sectors. "
-			"Declination (0 to 90 degree) is devided in (Number of Directions / 4) sectors."),
+			"Declination (0 to 90 degree) is divided in (Number of Directions / 4) sectors."),
 		PARAMETER_TYPE_Int, 8.0, 2, true
 	);
 
