@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: api_core.h 2252 2014-09-25 15:59:01Z oconrad $
+ * Version $Id: api_core.h 2710 2015-12-02 12:33:39Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -117,6 +117,7 @@
 
 //---------------------------------------------------------
 // this is defined by configure, but will not be on a normal application build
+
 #ifndef SIZEOF_LONG
 	#if defined(__alpha) || defined(__sparcv9) || defined(__LP64__) || (defined(__HOS_AIX__) && defined(_LP64))
 		#define SIZEOF_LONG		8
@@ -148,13 +149,8 @@
 #endif	// _TYPEDEF_WORD
 
 //---------------------------------------------------------
-#if SIZEOF_LONG == 4
 	typedef   signed long long	sLong;
 	typedef unsigned long long	uLong;
-#else
-	typedef   signed long		sLong;
-	typedef unsigned long		uLong;
-#endif
 
 //---------------------------------------------------------
 #if defined(_SAGA_MSW)
@@ -201,7 +197,8 @@ SAGA_API_DLL_EXPORT void *			SG_Calloc			(size_t num, size_t size);
 SAGA_API_DLL_EXPORT void *			SG_Realloc			(void *memblock, size_t size);
 SAGA_API_DLL_EXPORT void			SG_Free				(void *memblock);
 
-#define SG_FREE_SAFE(PTR)			{ if( PTR ) { SG_Free(PTR); PTR = NULL; } }
+#define SG_FREE_SAFE(PTR)			{ if( PTR ) { SG_Free (PTR); PTR = NULL; } }
+#define SG_DELETE_ARRAY(PTR)		{ if( PTR ) { delete[](PTR); PTR = NULL; } }
 
 //---------------------------------------------------------
 SAGA_API_DLL_EXPORT void			SG_Swap_Bytes		(void *Buffer, int nBytes);
@@ -351,6 +348,8 @@ public:
 	double							asDouble			(void)								const;
 	bool							asDouble			(double &Value)						const;
 
+	bool							to_ASCII			(char **pString)					const;
+
 
 protected:
 
@@ -916,13 +915,13 @@ public:
 	bool							is_Open				(void)	const	{	return( m_pStream != NULL );	}
 	bool							is_EOF				(void)	const;
 
-	long							Length				(void)	const;
+	sLong							Length				(void)	const;
 
-	bool							Seek				(long Offset, int Origin = SG_FILE_START)	const;
+	bool							Seek				(sLong Offset, int Origin = SG_FILE_START)	const;
 	bool							Seek_Start			(void)	const;
 	bool							Seek_End			(void)	const;
 
-	long							Tell				(void)	const;
+	sLong							Tell				(void)	const;
 
 	bool							Flush				(void)	const;
 
@@ -964,6 +963,8 @@ SAGA_API_DLL_EXPORT bool			SG_Dir_Exists				(const SG_Char *Directory);
 SAGA_API_DLL_EXPORT bool			SG_Dir_Create				(const SG_Char *Directory);
 SAGA_API_DLL_EXPORT CSG_String		SG_Dir_Get_Current			(void);
 SAGA_API_DLL_EXPORT CSG_String		SG_Dir_Get_Temp				(void);
+SAGA_API_DLL_EXPORT bool			SG_Dir_List_Subdirectories	(CSG_Strings &List, const CSG_String &Directory);
+SAGA_API_DLL_EXPORT bool			SG_Dir_List_Files			(CSG_Strings &List, const CSG_String &Directory, const SG_Char *Extension = NULL);
 
 SAGA_API_DLL_EXPORT bool			SG_File_Exists				(const SG_Char *FileName);
 SAGA_API_DLL_EXPORT bool			SG_File_Delete				(const SG_Char *FileName);
@@ -1373,6 +1374,8 @@ SAGA_API_DLL_EXPORT void					SG_UI_Msg_Add				(const CSG_String &Message, bool b
 SAGA_API_DLL_EXPORT void					SG_UI_Msg_Add_Error			(const CSG_String &Message);
 SAGA_API_DLL_EXPORT void					SG_UI_Msg_Add_Execution		(const CSG_String &Message, bool bNewLine, TSG_UI_MSG_STYLE Style = SG_UI_MSG_STYLE_NORMAL);
 
+SAGA_API_DLL_EXPORT void					SG_UI_ProgressAndMsg_Lock	(bool bOn);
+
 SAGA_API_DLL_EXPORT bool					SG_UI_DataObject_Add		(class CSG_Data_Object *pDataObject, int Show);
 SAGA_API_DLL_EXPORT bool					SG_UI_DataObject_Update		(class CSG_Data_Object *pDataObject, int Show, class CSG_Parameters *pParameters);
 SAGA_API_DLL_EXPORT bool					SG_UI_DataObject_Show		(class CSG_Data_Object *pDataObject, int Show);
