@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: api_file.cpp 1975 2014-02-09 14:59:42Z reklov_w $
+ * Version $Id: api_file.cpp 2711 2015-12-02 16:12:16Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -65,9 +65,19 @@
 //---------------------------------------------------------
 #include <wx/utils.h>
 #include <wx/filename.h>
+#include <wx/dir.h>
 
 #include "api_core.h"
 
+#if defined(_SAGA_VC)
+	#define SG_FILE_TELL	_ftelli64
+	#define SG_FILE_SEEK	_fseeki64
+	#define SG_FILE_SIZE	__int64
+#else
+	#define SG_FILE_TELL	ftell
+	#define SG_FILE_SEEK	fseek
+	#define SG_FILE_SIZE	long
+#endif
 
 ///////////////////////////////////////////////////////////
 //														 //
@@ -165,16 +175,12 @@ bool CSG_File::Close(void)
 }
 
 //---------------------------------------------------------
-long CSG_File::Length(void)	const
+sLong CSG_File::Length(void)	const
 {
 	if( m_pStream )
 	{
-		long	pos, len;
-
-		pos	= ftell(m_pStream);
-		fseek(m_pStream, 0, SEEK_END);
-		len	= ftell(m_pStream);
-		fseek(m_pStream, pos, SEEK_SET);
+		SG_FILE_SIZE	pos	= SG_FILE_TELL(m_pStream);	SG_FILE_SEEK(m_pStream,   0, SEEK_END);
+		SG_FILE_SIZE	len	= SG_FILE_TELL(m_pStream);	SG_FILE_SEEK(m_pStream, pos, SEEK_SET);
 
 		return( len );
 	}
@@ -189,7 +195,7 @@ bool CSG_File::is_EOF(void)	const
 }
 
 //---------------------------------------------------------
-bool CSG_File::Seek(long Offset, int Origin) const
+bool CSG_File::Seek(sLong Offset, int Origin) const
 {
 	switch( Origin )
 	{
@@ -199,25 +205,25 @@ bool CSG_File::Seek(long Offset, int Origin) const
 	case SG_FILE_END:		Origin	= SEEK_END;	break;
 	}
 
-	return( m_pStream ? !fseek(m_pStream, Offset, Origin) : false );
+	return( m_pStream ? !SG_FILE_SEEK(m_pStream, Offset, Origin) : false );
 }
 
 //---------------------------------------------------------
 bool CSG_File::Seek_Start(void) const
 {
-	return( m_pStream && fseek(m_pStream, 0, SEEK_SET) == 0 );
+	return( m_pStream && SG_FILE_SEEK(m_pStream, 0, SEEK_SET) == 0 );
 }
 
 //---------------------------------------------------------
 bool CSG_File::Seek_End(void) const
 {
-	return( m_pStream && fseek(m_pStream, 0, SEEK_END) == 0 );
+	return( m_pStream && SG_FILE_SEEK(m_pStream, 0, SEEK_END) == 0 );
 }
 
 //---------------------------------------------------------
-long CSG_File::Tell(void) const
+sLong CSG_File::Tell(void) const
 {
-	return( m_pStream ? ftell(m_pStream) : -1 );
+	return( m_pStream ? SG_FILE_TELL(m_pStream) : -1 );
 }
 
 //---------------------------------------------------------
@@ -227,21 +233,48 @@ bool CSG_File::Flush(void) const
 }
 
 //---------------------------------------------------------
-int CSG_File::Printf(const SG_Char *Format, ...)
+int CSG_File::Printf(const char *Format, ...)
 {
-	int		result	= 0;
-
-	if( m_pStream )
+	if( !m_pStream )
 	{
-		va_list	argptr;
-
-		va_start(argptr, Format);
-
-		result	= wxVfprintf(m_pStream, Format, argptr);
-
-		va_end(argptr);
+		return( 0 );
 	}
 
+#ifdef _SAGA_LINUX
+	wxString	_Format(Format);	_Format.Replace("%s", "%ls");	// workaround as we only use wide characters since wx 2.9.4 so interpret strings as multibyte
+	va_list	argptr; va_start(argptr, _Format);
+	int	result	= wxVfprintf(m_pStream, _Format, argptr);
+#else
+	va_list	argptr;	va_start(argptr, Format);
+
+	int	result	= wxVfprintf(m_pStream, Format, argptr);
+#endif
+
+	va_end(argptr);
+
+	return( result );
+}
+
+//---------------------------------------------------------
+int CSG_File::Printf(const wchar_t *Format, ...)
+{
+	if( !m_pStream )
+	{
+		return( 0 );
+	}
+
+#ifdef _SAGA_LINUX
+	wxString	_Format(Format);	_Format.Replace("%s", "%ls");	// workaround as we only use wide characters since wx 2.9.4 so interpret strings as multibyte
+	va_list	argptr; va_start(argptr, _Format);
+	int	result	= wxVfprintf(m_pStream, _Format, argptr);
+#else
+	va_list	argptr;	va_start(argptr, Format);
+
+	int	result	= wxVfprintf(m_pStream, Format, argptr);
+#endif
+
+	va_end(argptr);
+
 	return( result );
 }
 
@@ -444,7 +477,68 @@ bool			SG_Dir_Create(const SG_Char *Directory)
 //---------------------------------------------------------
 CSG_String		SG_Dir_Get_Current(void)
 {
-	return( CSG_String(wxGetCwd().wc_str()) );
+	wxString cwd = wxFileName::GetCwd();
+
+	return( CSG_String(&cwd) );
+}
+
+//---------------------------------------------------------
+CSG_String		SG_Dir_Get_Temp(void)
+{
+	wxString fname = wxFileName::GetTempDir();
+
+	return( CSG_String(&fname) );
+}
+
+//---------------------------------------------------------
+bool			SG_Dir_List_Subdirectories	(CSG_Strings &List, const CSG_String &Directory)
+{
+	List.Clear();
+
+	wxDir	Dir;
+
+	if( Dir.Open(Directory.c_str()) )
+	{
+		wxString	FileName;
+
+		if( Dir.GetFirst(&FileName, wxEmptyString, wxDIR_DIRS) )
+		{
+			do
+			{
+				List	+= SG_File_Make_Path(Directory, FileName);
+			}
+			while( Dir.GetNext(&FileName) );
+		}
+	}
+
+	return( List.Get_Count() > 0 );
+}
+
+//---------------------------------------------------------
+bool			SG_Dir_List_Files			(CSG_Strings &List, const CSG_String &Directory, const SG_Char *Extension)
+{
+	List.Clear();
+
+	wxDir	Dir;
+
+	if( Dir.Open(Directory.c_str()) )
+	{
+		wxString	FileName;
+
+		if( Dir.GetFirst(&FileName, wxEmptyString, wxDIR_FILES) )
+		{
+			do
+			{
+				if( !Extension || !*Extension || SG_File_Cmp_Extension(FileName, Extension) )
+				{
+					List	+= SG_File_Make_Path(Directory, FileName);
+				}
+			}
+			while( Dir.GetNext(&FileName) );
+		}
+	}
+
+	return( List.Get_Count() > 0 );
 }
 
 
@@ -467,7 +561,7 @@ bool			SG_File_Delete(const SG_Char *FileName)
 }
 
 //---------------------------------------------------------
-CSG_String		SG_File_Get_TmpName(const SG_Char *Prefix, const SG_Char *Directory)
+CSG_String		SG_File_Get_Name_Temp(const SG_Char *Prefix, const SG_Char *Directory)
 {
 	if( !SG_Dir_Exists(Directory) )
 	{
