@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: api_memory.cpp 1480 2012-10-05 15:52:32Z oconrad $
+ * Version $Id: api_memory.cpp 2639 2015-10-07 10:59:17Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -37,7 +37,7 @@
 // You should have received a copy of the GNU Lesser     //
 // General Public License along with this program; if    //
 // not, write to the Free Software Foundation, Inc.,     //
-// 59 Temple Place - Suite 330, Boston, MA 02111-1307,   //
+// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
 // USA.                                                  //
 //                                                       //
 //-------------------------------------------------------//
@@ -427,12 +427,12 @@ bool CSG_Array::Inc_Array		(void **pArray)
 //---------------------------------------------------------
 bool CSG_Array::Dec_Array		(bool bShrink)
 {
-	return( m_Values > 0 ? Set_Array(m_nValues - 1, bShrink) : false );
+	return( m_nValues > 0 ? Set_Array(m_nValues - 1, bShrink) : false );
 }
 
 bool CSG_Array::Dec_Array		(void **pArray, bool bShrink)
 {
-	return( m_Values > 0 ? Set_Array(m_nValues - 1, pArray, bShrink) : false );
+	return( m_nValues > 0 ? Set_Array(m_nValues - 1, pArray, bShrink) : false );
 }
 
 
