@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: api_string.cpp 1967 2014-02-05 10:15:53Z oconrad $
+ * Version $Id: api_string.cpp 2564 2015-09-01 13:06:20Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -273,25 +273,20 @@ void CSG_String::Clear(void)
 }
 
 //---------------------------------------------------------
-CSG_String CSG_String::Format(const SG_Char *Format, ...)
+CSG_String CSG_String::Format(const char *Format, ...)
 {
 	CSG_String	s;
 
-	va_list		argptr;
+	va_list	argptr;
 	
 #ifdef _SAGA_LINUX
-	/* workaround as we only use wide characters since wx 2.9.4, 
-	so interpret strings as multibyte */
-	wxString	sFormat;
-	sFormat = Format;
-	sFormat.Replace("%s", "%ls");
-
-	va_start(argptr, sFormat);
-
-	s.m_pString->PrintfV(sFormat, argptr);
+	// workaround as we only use wide characters
+	// since wx 2.9.4 so interpret strings as multibyte
+	wxString	_Format(Format);	_Format.Replace("%s", "%ls");
+	va_start(argptr, _Format);
+	s.m_pString->PrintfV(_Format, argptr);
 #else
 	va_start(argptr, Format);
-
 	s.m_pString->PrintfV(Format, argptr);
 #endif
 
@@ -301,23 +296,62 @@ CSG_String CSG_String::Format(const SG_Char *Format, ...)
 }
 
 //---------------------------------------------------------
-int CSG_String::Printf(const SG_Char *Format, ...)
+CSG_String CSG_String::Format(const wchar_t *Format, ...)
+{
+	CSG_String	s;
+
+	va_list	argptr;
+	
+#ifdef _SAGA_LINUX
+	// workaround as we only use wide characters
+	// since wx 2.9.4 so interpret strings as multibyte
+	wxString	_Format(Format);	_Format.Replace("%s", "%ls");
+	va_start(argptr, _Format);
+	s.m_pString->PrintfV(_Format, argptr);
+#else
+	va_start(argptr, Format);
+	s.m_pString->PrintfV(Format, argptr);
+#endif
+
+	va_end(argptr);
+
+	return( s );
+}
+
+//---------------------------------------------------------
+int CSG_String::Printf(const char *Format, ...)
 {
 	va_list	argptr;
 
 #ifdef _SAGA_LINUX
-	/* workaround as we only use wide characters since wx 2.9.4, 
-	so interpret strings as multibyte */
-	wxString	sFormat;
-	sFormat = Format;
-	sFormat.Replace("%s", "%ls");
-
-	va_start(argptr, sFormat);
-
-	m_pString->PrintfV(sFormat, argptr);
+	// workaround as we only use wide characters
+	// since wx 2.9.4 so interpret strings as multibyte
+	wxString	_Format(Format);	_Format.Replace("%s", "%ls");
+	va_start(argptr, _Format);
+	m_pString->PrintfV(_Format, argptr);
 #else
 	va_start(argptr, Format);
+	m_pString->PrintfV(Format, argptr);
+#endif
 
+	va_end(argptr);
+
+	return( (int)Length() );
+}
+
+//---------------------------------------------------------
+int CSG_String::Printf(const wchar_t *Format, ...)
+{
+	va_list	argptr;
+
+#ifdef _SAGA_LINUX
+	// workaround as we only use wide characters
+	// since wx 2.9.4 so interpret strings as multibyte
+	wxString	_Format(Format);	_Format.Replace("%s", "%ls");
+	va_start(argptr, _Format);
+	m_pString->PrintfV(_Format, argptr);
+#else
+	va_start(argptr, Format);
 	m_pString->PrintfV(Format, argptr);
 #endif
 
@@ -754,6 +788,43 @@ bool CSG_String::asDouble(double &Value) const
 }
 
 
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+/**
+  * This function creates a single char array and fills it
+  * with the ASCII representation of this string object.
+  * If successful pString will point to the adress of the
+  * allocated memory. Freeing memory is in response of the
+  * calling function.
+*/
+//---------------------------------------------------------
+bool CSG_String::to_ASCII(char **pString)	const
+{
+	if( is_Empty() )
+	{
+		return( false );
+	}
+
+	*pString	= (char *)SG_Malloc((1 + Length()) * sizeof(char));
+
+	if( *pString == NULL )
+	{
+		return( false );
+	}
+
+	memcpy(*pString, m_pString->ToAscii(), Length() * sizeof(char));
+
+	(*pString)[Length()]	= '\0';
+
+	return( true );
+}
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 //														 //
