@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: api_translator.cpp 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: api_translator.cpp 2005 2014-02-20 13:49:51Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -369,6 +369,78 @@ bool CSG_Translator::Get_Translation(const SG_Char *Text, CSG_String &Translatio
 }
 
 
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool SG_Set_OldStyle_Naming(void)
+{
+	//-----------------------------------------------------
+	if( SG_Get_Translator().Get_Count() > 0 )
+	{
+		return( false );
+	}
+
+	//-----------------------------------------------------
+	const char	translation[][2][256]	=
+	{
+		{	"Geoprocessing",	"Modules"	},
+		{	"Available Tools",	"Available Modules"	},
+		{	"Can't execute a tool while another runs",		"Can't execute a module while another runs"	},
+		{	"Create Tool Description Files",		"Create Module Description Files"	},
+		{	"Execute Tool",		"Execute Module"	},
+		{	"Executing tool",		"Executing module"	},
+		{	"Find and Run Tool",		"Find and Run Module"	},
+		{	"Interactive tool execution failed",		"Interactive module execution failed"	},
+		{	"Interactive tool execution has been started",		"Interactive module execution has been started"	},
+		{	"Interactive tool execution has been stopped",		"Interactive module execution has been stopped"	},
+		{	"Load Tool Library",		"Load Module Library"	},
+		{	"Please stop tool execution before exiting SAGA.",		"Please stop module execution before exiting SAGA."	},
+		{	"Run Tool",		"Run Module"	},
+		{	"SAGA Tool Libraries",		"SAGA Module Libraries"	},
+		{	"Tool",		"Module"	},
+		{	"Tool Description Source",		"Module Description Source"	},
+		{	"Tool Execution",		"Module Execution"	},
+		{	"Tool Libraries",		"Module Libraries"	},
+		{	"Tool Library",		"Module Library"	},
+		{	"Tool Manager",		"Module Manager"	},
+		{	"Tool execution failed",		"Module execution failed"	},
+		{	"Tool execution succeeded",		"Module execution succeeded"	},
+		{	"Tools",		"Modules"	},
+		{	"cannot execute interactive tool",		"cannot execute interactive module"	},
+		{	"could not execute tool",		"could not execute module"	},
+		{	"could not find tool",		"could not find module"	},
+		{	"could not initialize tool",		"could not initialize module"	},
+		{	"creating tool documentation files",		"creating module documentation files"	},
+		{	"executing tool",		"executing module"	},
+		{	"loaded tool libraries",		"loaded module libraries"	},
+		{	"tool",		"module"	},
+		{	"tool name   ",		"module name   "	},
+		{	"tools",	"modules"	},
+		{	"",	""	}
+	};
+
+	//-----------------------------------------------------
+	CSG_Table	t;
+
+	t.Add_Field("NEW", SG_DATATYPE_String);
+	t.Add_Field("OLD", SG_DATATYPE_String);
+
+	for(int i=0; *translation[i][0]; i++)
+	{
+		CSG_Table_Record	*pR	= t.Add_Record();
+
+		pR->Set_Value(0, translation[i][0]);
+		pR->Set_Value(1, translation[i][1]);
+	}
+
+	return( SG_Get_Translator().Create(&t) );
+}
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 //														 //
