@@ -103,11 +103,11 @@ CSG_Data_Collection::~CSG_Data_Collection(void)
 }
 
 //---------------------------------------------------------
-CSG_Data_Object * CSG_Data_Collection::Get(const CSG_String &File) const
+CSG_Data_Object * CSG_Data_Collection::Get(const CSG_String &File, bool bNative) const
 {
 	for(size_t i=0; i<Count(); i++)
 	{
-		if( !File.Cmp(Get(i)->Get_File_Name()) )
+		if( !File.Cmp(Get(i)->Get_File_Name(bNative)) )
 		{
 			return( Get(i) );
 		}
@@ -372,18 +372,18 @@ bool CSG_Data_Manager::Exists(CSG_Data_Object *pObject) const
 }
 
 //---------------------------------------------------------
-CSG_Data_Object *  CSG_Data_Manager::Find(const CSG_String &File) const
+CSG_Data_Object *  CSG_Data_Manager::Find(const CSG_String &File, bool bNative) const
 {
 	CSG_Data_Object	*pObject;
 
-	if( (pObject = m_pTable      ->Get(File)) != NULL )	return( pObject );
-	if( (pObject = m_pTIN        ->Get(File)) != NULL )	return( pObject );
-	if( (pObject = m_pPoint_Cloud->Get(File)) != NULL )	return( pObject );
-	if( (pObject = m_pShapes     ->Get(File)) != NULL )	return( pObject );
+	if( (pObject = m_pTable      ->Get(File, bNative)) != NULL )	return( pObject );
+	if( (pObject = m_pTIN        ->Get(File, bNative)) != NULL )	return( pObject );
+	if( (pObject = m_pPoint_Cloud->Get(File, bNative)) != NULL )	return( pObject );
+	if( (pObject = m_pShapes     ->Get(File, bNative)) != NULL )	return( pObject );
 
 	for(size_t i=0; i<Grid_System_Count(); i++)
 	{
-		if( (pObject = Get_Grid_System(i)->Get(File)) != NULL )	return( pObject );
+		if( (pObject = Get_Grid_System(i)->Get(File, bNative)) != NULL )	return( pObject );
 	}
 
 	return(	NULL );
@@ -471,6 +471,8 @@ bool CSG_Data_Manager::Add(const CSG_String &File, TSG_Data_Object_Type Type)
 bool CSG_Data_Manager::_Add_External(const CSG_String &File)
 {
 	//-----------------------------------------------------
+	bool		bResult	= false;
+
 	CSG_Module	*pImport;
 
 	if( !SG_File_Exists(File) )
@@ -479,37 +481,59 @@ bool CSG_Data_Manager::_Add_External(const CSG_String &File)
 	}
 
 	//-----------------------------------------------------
-	if(	SG_File_Cmp_Extension(File, SG_T("bmp"))
-	||	SG_File_Cmp_Extension(File, SG_T("gif"))
-	||	SG_File_Cmp_Extension(File, SG_T("jpg"))
-	||	SG_File_Cmp_Extension(File, SG_T("png"))
-	||	SG_File_Cmp_Extension(File, SG_T("pcx")) )
-	{
-		pImport	= SG_Get_Module_Library_Manager().Get_Module(SG_T("io_grid_image"), 1);	// Image Import
+	// Image Import
 
-		if( pImport && pImport->Get_Parameters()->Set_Parameter(SG_T("FILE"), File, PARAMETER_TYPE_FilePath) && pImport->Execute() )
-		{
-			return( true );
-		}
+	if(	(	SG_File_Cmp_Extension(File, SG_T("bmp"))
+		||	SG_File_Cmp_Extension(File, SG_T("gif"))
+		||	SG_File_Cmp_Extension(File, SG_T("jpg"))
+		||	SG_File_Cmp_Extension(File, SG_T("png"))
+		||	SG_File_Cmp_Extension(File, SG_T("pcx")) )
+	&&  (pImport = SG_Get_Module_Library_Manager().Get_Module("io_grid_image", 1)) != NULL
+	&&   pImport->Set_Parameter("FILE", File, PARAMETER_TYPE_FilePath) )
+	{
+		pImport->Set_Manager(this);
+		bResult	= pImport->Execute();
+		pImport->Set_Manager(&g_Data_Manager);
 	}
 
 	//-----------------------------------------------------
-	pImport	= SG_Get_Module_Library_Manager().Get_Module(SG_T("io_gdal"), 0);			// GDAL Import
+	// GDAL Import
 
-	if( pImport && pImport->Get_Parameters()->Set_Parameter(SG_T("FILES"), File, PARAMETER_TYPE_FilePath) && pImport->Execute() )
+	if( !bResult
+	&&  (pImport = SG_Get_Module_Library_Manager().Get_Module("io_gdal", 0)) != NULL
+	&&   pImport->Set_Parameter("FILES", File, PARAMETER_TYPE_FilePath) )
 	{
-		return( true );
+		pImport->Set_Manager(this);
+		bResult	= pImport->Execute();
+		pImport->Set_Manager(&g_Data_Manager);
 	}
 
 	//-----------------------------------------------------
-	pImport	= SG_Get_Module_Library_Manager().Get_Module(SG_T("io_gdal"), 3);			// OGR Import
+	// OGR Import
 
-	if( pImport && pImport->Get_Parameters()->Set_Parameter(SG_T("FILES"), File, PARAMETER_TYPE_FilePath) && pImport->Execute() )
+	if( !bResult
+	&&  (pImport = SG_Get_Module_Library_Manager().Get_Module("io_gdal", 3)) != NULL
+	&&   pImport->Set_Parameter("FILES", File, PARAMETER_TYPE_FilePath) )
 	{
-		return( true );
+		pImport->Set_Manager(this);
+		bResult	= pImport->Execute();
+		pImport->Set_Manager(&g_Data_Manager);
 	}
 
-	return( false );
+	//-----------------------------------------------------
+	// LAS Import
+
+	if( !bResult && SG_File_Cmp_Extension(File, SG_T("las"))
+	&&  (pImport = SG_Get_Module_Library_Manager().Get_Module("io_shapes_las", 1)) != NULL
+	&&   pImport->Set_Parameter("FILES", File, PARAMETER_TYPE_FilePath) )
+	{
+		pImport->Set_Manager(this);
+		bResult	= pImport->Execute();
+		pImport->Set_Manager(&g_Data_Manager);
+	}
+
+	//-----------------------------------------------------
+	return( bResult );
 }
 
 
