@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: dataobject.h 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: dataobject.h 2565 2015-09-01 13:34:02Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -186,8 +186,9 @@ public:
 
 	virtual bool					Save			(const CSG_String &File_Name, int Format = 0)	= 0;
 
-	const SG_Char *					Get_File_Name	(void)	const			{	return( m_File_Name );		}
-	int								Get_File_Type	(void)	const			{	return( m_File_Type );		}
+	void							Set_File_Name	(const CSG_String &File_Name);
+	const SG_Char *					Get_File_Name	(bool bNative = true)	const;
+	int								Get_File_Type	(void)					const;
 
 	virtual bool					Assign			(CSG_Data_Object *pObject);
 
@@ -205,8 +206,10 @@ public:
 	CSG_MetaData &					Get_MetaData	(void)	const			{	return( *m_pMetaData );		}
 	CSG_MetaData &					Get_MetaData_DB	(void)					{	return( *m_pMetaData_DB );	}
 	CSG_MetaData &					Get_History		(void)					{	return( *m_pHistory );		}
+	const CSG_MetaData &			Get_History		(void)	const			{	return( *m_pHistory );		}
 
 	CSG_Projection &				Get_Projection	(void)					{	return( m_Projection );		}
+	const CSG_Projection &			Get_Projection	(void)	const			{	return( m_Projection );		}
 
 	class CSG_Table *				asTable			(void)	{	return( Get_ObjectType() == DATAOBJECT_TYPE_Table      ? (class CSG_Table      *)this : NULL );	}
 	class CSG_Shapes *				asShapes		(void)	{	return( Get_ObjectType() == DATAOBJECT_TYPE_Shapes     ? (class CSG_Shapes     *)this : NULL );	}
@@ -224,10 +227,12 @@ public:
 		return( SG_is_NaN(Value) || (m_NoData_Value < m_NoData_hiValue ? m_NoData_Value <= Value && Value <= m_NoData_hiValue : Value == m_NoData_Value) );
 	}
 
+	bool							Save_History_to_Model	(const CSG_String &File)	const;
+
 
 protected:
 
-	void							Set_File_Name		(const CSG_String &File_Name);
+	void							Set_File_Name		(const CSG_String &File_Name, bool bNative);
 	void							Set_File_Type		(int Type)			{	m_File_Type	= Type;			}
 
 	bool							Load_MetaData		(const SG_Char *File_Name);
@@ -241,7 +246,7 @@ protected:
 
 private:
 
-	bool							m_bModified, m_bUpdate;
+	bool							m_bModified, m_bUpdate, m_File_bNative;
 
 	int								m_File_Type;
 
