@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: grid.cpp 2064 2014-03-21 13:20:57Z oconrad $
+ * Version $Id: grid.cpp 2641 2015-10-08 10:00:15Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -64,6 +64,8 @@
 
 //---------------------------------------------------------
 #include "grid.h"
+#include "data_manager.h"
+#include "module_library.h"
 
 
 ///////////////////////////////////////////////////////////
@@ -95,9 +97,18 @@ CSG_Grid * SG_Create_Grid(const CSG_Grid &Grid)
 }
 
 //---------------------------------------------------------
-CSG_Grid * SG_Create_Grid(const CSG_String &File_Name, TSG_Data_Type Type, TSG_Grid_Memory_Type Memory_Type)
+CSG_Grid * SG_Create_Grid(const CSG_String &File_Name, TSG_Data_Type Type, TSG_Grid_Memory_Type Memory_Type, bool bLoadData)
 {
-	return( new CSG_Grid(File_Name, Type, Memory_Type) );
+	CSG_Grid	*pGrid	= new CSG_Grid(File_Name, Type, Memory_Type, bLoadData);
+
+	if( pGrid->is_Valid() )
+	{
+		return( pGrid );
+	}
+
+	delete(pGrid);
+
+	return( NULL );
 }
 
 //---------------------------------------------------------
@@ -124,7 +135,16 @@ CSG_Grid * SG_Create_Grid(const CSG_Grid_System &System, TSG_Data_Type Type, TSG
 //---------------------------------------------------------
 CSG_Grid * SG_Create_Grid(TSG_Data_Type Type, int NX, int NY, double Cellsize, double xMin, double yMin, TSG_Grid_Memory_Type Memory_Type)
 {
-	return( new CSG_Grid(Type, NX, NY, Cellsize, xMin, yMin, Memory_Type) );
+	CSG_Grid	*pGrid	= new CSG_Grid(Type, NX, NY, Cellsize, xMin, yMin, Memory_Type);
+
+	if( pGrid && !pGrid->is_Valid() )
+	{
+		delete(pGrid);
+
+		pGrid	= NULL;
+	}
+
+	return( pGrid );
 }
 
 
@@ -163,12 +183,12 @@ CSG_Grid::CSG_Grid(const CSG_Grid &Grid)
   * Create a grid from file.
 */
 //---------------------------------------------------------
-CSG_Grid::CSG_Grid(const CSG_String &File_Name, TSG_Data_Type Type, TSG_Grid_Memory_Type Memory_Type)
+CSG_Grid::CSG_Grid(const CSG_String &File_Name, TSG_Data_Type Type, TSG_Grid_Memory_Type Memory_Type, bool bLoadData)
 	: CSG_Data_Object()
 {
 	_On_Construction();
 
-	Create(File_Name, Type, Memory_Type);
+	Create(File_Name, Type, Memory_Type, bLoadData);
 }
 
 //---------------------------------------------------------
@@ -234,10 +254,11 @@ void CSG_Grid::_On_Construction(void)
 	m_LineBuffer		= NULL;
 	m_LineBuffer_Count	= 5;
 
-	m_zFactor			= 1.0;
+	m_zScale			= 1.0;
+	m_zOffset			= 0.0;
 
-	m_bIndexed			= false;
 	m_Index				= NULL;
+	m_bIndex			= false;
 
 	Set_Update_Flag();
 }
@@ -252,12 +273,32 @@ void CSG_Grid::_On_Construction(void)
 //---------------------------------------------------------
 bool CSG_Grid::Create(const CSG_Grid &Grid)
 {
-	if( Create((CSG_Grid *)&Grid, ((CSG_Grid *)&Grid)->Get_Type()) )
+	if( Create(Grid.Get_System(), Grid.Get_Type()) )
 	{
-		Set_Name       (Grid.Get_Name       ());
-		Set_Description(Grid.Get_Description());
+		Set_Name              (Grid.Get_Name       ());
+		Set_Description       (Grid.Get_Description());
+		Set_Unit              (Grid.Get_Unit       ());
+		Set_NoData_Value_Range(Grid.Get_NoData_Value(), Grid.Get_NoData_hiValue());
 
-		return( Assign((CSG_Data_Object *)&Grid) );
+		for(int y=0; y<Get_NY() && SG_UI_Process_Set_Progress(y, Get_NY()); y++)
+		{
+			#pragma omp parallel for
+			for(int x=0; x<Get_NX(); x++)
+			{
+				if( Grid.is_NoData(x, y) )
+				{
+					Set_NoData(x, y);
+				}
+				else
+				{
+					Set_Value(x, y, Grid.asDouble(x, y, false));
+				}
+			}
+		}
+
+		Set_Scaling(Grid.Get_Scaling(), Grid.Get_Offset());
+
+		return( true );
 	}
 
 	return( false );
@@ -285,9 +326,94 @@ bool CSG_Grid::Create(const CSG_Grid_System &System, TSG_Data_Type Type, TSG_Gri
 }
 
 //---------------------------------------------------------
-bool CSG_Grid::Create(const CSG_String &File_Name, TSG_Data_Type Type, TSG_Grid_Memory_Type Memory_Type)
+bool CSG_Grid::Create(const CSG_String &File_Name, TSG_Data_Type Type, TSG_Grid_Memory_Type Memory_Type, bool bLoadData)
 {
-	return( _Load(File_Name, Type, Memory_Type) );
+	Destroy();
+
+	SG_UI_Msg_Add(CSG_String::Format("%s: %s...", _TL("Load grid"), File_Name.c_str()), true);
+
+	//-----------------------------------------------------
+	bool	bResult	= File_Name.BeforeFirst(':').Cmp("PGSQL") && SG_File_Exists(File_Name) && _Load(File_Name, Type, Memory_Type, bLoadData);
+
+	if( bResult )
+	{
+		// nop
+	}
+
+	//-----------------------------------------------------
+	else if( File_Name.BeforeFirst(':').Cmp("PGSQL") == 0 )	// database source
+	{
+		CSG_String	s(File_Name);
+
+		s	= s.AfterFirst(':');	CSG_String	Host  (s.BeforeFirst(':'));
+		s	= s.AfterFirst(':');	CSG_String	Port  (s.BeforeFirst(':'));
+		s	= s.AfterFirst(':');	CSG_String	DBName(s.BeforeFirst(':'));
+		s	= s.AfterFirst(':');	CSG_String	Table (s.BeforeFirst(':'));
+		s	= s.AfterFirst(':');	CSG_String	rid   (s.BeforeFirst(':').AfterFirst('='));
+
+		CSG_Module	*pModule	= SG_Get_Module_Library_Manager().Get_Module("db_pgsql", 0);	// CGet_Connections
+
+		if(	pModule != NULL )
+		{
+			SG_UI_ProgressAndMsg_Lock(true);
+
+			//---------------------------------------------
+			CSG_Table	Connections;
+			CSG_String	Connection	= DBName + " [" + Host + ":" + Port + "]";
+
+			pModule->Settings_Push();
+
+			if( pModule->On_Before_Execution() && SG_MODULE_PARAMETER_SET("CONNECTIONS", &Connections) && pModule->Execute() )	// CGet_Connections
+			{
+				for(int i=0; !bResult && i<Connections.Get_Count(); i++)
+				{
+					if( !Connection.Cmp(Connections[i].asString(0)) )
+					{
+						bResult	= true;
+					}
+				}
+			}
+
+			pModule->Settings_Pop();
+
+			//---------------------------------------------
+			if( bResult && (bResult = (pModule = SG_Get_Module_Library_Manager().Get_Module("db_pgsql", 33)) != NULL) == true )	// CPGIS_Raster_Load_Band
+			{
+				pModule->Settings_Push();
+
+				bResult	= pModule->On_Before_Execution()
+					&& SG_MODULE_PARAMETER_SET("CONNECTION", Connection)
+					&& SG_MODULE_PARAMETER_SET("TABLES"    , Table)
+					&& SG_MODULE_PARAMETER_SET("RID"       , rid)
+					&& SG_MODULE_PARAMETER_SET("GRID"      , this)
+					&& pModule->Execute();
+
+				pModule->Settings_Pop();
+			}
+
+			SG_UI_ProgressAndMsg_Lock(false);
+		}
+	}
+
+	//-----------------------------------------------------
+	if( bResult )
+	{
+		m_bCreated	= true;
+
+		Set_Modified(false);
+		Set_Update_Flag();
+
+		SG_UI_Msg_Add(_TL("okay"), false, SG_UI_MSG_STYLE_SUCCESS);
+
+		return( true );
+	}
+
+	//-----------------------------------------------------
+	Destroy();
+
+	SG_UI_Msg_Add(_TL("failed"), false, SG_UI_MSG_STYLE_FAILURE);
+
+	return( false );
 }
 
 //---------------------------------------------------------
@@ -333,7 +459,8 @@ bool CSG_Grid::Destroy(void)
 	m_Type			= SG_DATATYPE_Undefined;
 	m_Memory_Type	= GRID_MEMORY_Normal;
 
-	m_zFactor		= 1.0;
+	m_zScale		= 1.0;
+	m_zOffset		= 0.0;
 
 	m_Unit			.Clear();
 
@@ -391,15 +518,31 @@ const SG_Char * CSG_Grid::Get_Unit(void) const
 }
 
 //---------------------------------------------------------
-void CSG_Grid::Set_ZFactor(double Value)
+void CSG_Grid::Set_Scaling(double Scale, double Offset)
 {
-	m_zFactor		= Value;
+	if( (Scale != m_zScale && Scale != 0.0) || Offset != m_zOffset )
+	{
+		if( Scale != 0.0 )
+		{
+			m_zScale	= Scale;
+		}
+
+		m_zOffset	= Offset;
+
+		Set_Update_Flag();
+	}
 }
 
 //---------------------------------------------------------
-double CSG_Grid::Get_ZFactor(void) const
+double CSG_Grid::Get_Scaling(void) const
 {
-	return( m_zFactor );
+	return( m_zScale );
+}
+
+//---------------------------------------------------------
+double CSG_Grid::Get_Offset(void) const
+{
+	return( m_zOffset );
 }
 
 
@@ -467,27 +610,27 @@ bool CSG_Grid::is_Compatible(int NX, int NY, double Cellsize, double xMin, doubl
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-double CSG_Grid::Get_Value(TSG_Point Position, int Interpolation, bool bZFactor, bool bByteWise, bool bOnlyValidCells) const
+double CSG_Grid::Get_Value(TSG_Point Position, int Interpolation, bool bByteWise, bool bOnlyValidCells) const
 {
 	double	Value;
 
-	return( Get_Value(Position.x, Position.y, Value, Interpolation, bZFactor, bByteWise, bOnlyValidCells) ? Value : Get_NoData_Value() );
+	return( Get_Value(Position.x, Position.y, Value, Interpolation, bByteWise, bOnlyValidCells) ? Value : Get_NoData_Value() );
 }
 
-double CSG_Grid::Get_Value(double xPosition, double yPosition, int Interpolation, bool bZFactor, bool bByteWise, bool bOnlyValidCells) const
+double CSG_Grid::Get_Value(double xPosition, double yPosition, int Interpolation, bool bByteWise, bool bOnlyValidCells) const
 {
 	double	Value;
 
-	return( Get_Value(xPosition, yPosition, Value, Interpolation, bZFactor, bByteWise, bOnlyValidCells) ? Value : Get_NoData_Value() );
+	return( Get_Value(xPosition, yPosition, Value, Interpolation, bByteWise, bOnlyValidCells) ? Value : Get_NoData_Value() );
 }
 
-bool CSG_Grid::Get_Value(TSG_Point Position, double &Value, int Interpolation, bool bZFactor, bool bByteWise, bool bOnlyValidCells) const
+bool CSG_Grid::Get_Value(TSG_Point Position, double &Value, int Interpolation, bool bByteWise, bool bOnlyValidCells) const
 {
-	return( Get_Value(Position.x, Position.y, Value, Interpolation, bZFactor, bByteWise, bOnlyValidCells) );
+	return( Get_Value(Position.x, Position.y, Value, Interpolation, bByteWise, bOnlyValidCells) );
 }
 
 //---------------------------------------------------------
-bool CSG_Grid::Get_Value(double xPosition, double yPosition, double &Value, int Interpolation, bool bZFactor, bool bByteWise, bool bOnlyValidCells) const
+bool CSG_Grid::Get_Value(double xPosition, double yPosition, double &Value, int Interpolation, bool bByteWise, bool bOnlyValidCells) const
 {
 	if(	m_System.Get_Extent(true).Contains(xPosition, yPosition) )
 	{
@@ -523,15 +666,7 @@ bool CSG_Grid::Get_Value(double xPosition, double yPosition, double &Value, int
 				break;
 			}
 
-			if( !is_NoData_Value(Value) )
-			{
-				if( bZFactor )
-				{
-					Value	*= m_zFactor;
-				}
-
-				return( true );
-			}
+			return( !is_NoData_Value(Value) );
 		}
 	}
 
@@ -796,9 +931,10 @@ inline double CSG_Grid::_Get_ValAtPos_BSpline(int x, int y, double dx, double dy
 //---------------------------------------------------------
 inline bool CSG_Grid::_Get_ValAtPos_Fill4x4Submatrix(int x, int y, double z_xy[4][4]) const
 {
-	int		ix, iy, jx, jy, nNoData;
+	int		ix, iy, jx, jy, nNoData	= 0;
 
-	for(iy=0, jy=y-1, nNoData=0; iy<4; iy++, jy++)
+	//-----------------------------------------------------
+	for(iy=0, jy=y-1; iy<4; iy++, jy++)
 	{
 		for(ix=0, jx=x-1; ix<4; ix++, jx++)
 		{
@@ -816,32 +952,22 @@ inline bool CSG_Grid::_Get_ValAtPos_Fill4x4Submatrix(int x, int y, double z_xy[4
 	}
 
 	//-----------------------------------------------------
-	if( nNoData >= 16 )
+	if( nNoData > 0 && nNoData < 16 )	// guess missing values as average of surrounding data values
 	{
-		return( false );
-	}
-
-	if( nNoData > 0 )
-	{
-		double	t_xy[4][4];
-
-		for(iy=0; iy<4; iy++)
+		for(int i=0; nNoData>0 && i<16; i++)	// avoid the possibility of endless loop
 		{
-			for(ix=0; ix<4; ix++)
-			{
-				t_xy[ix][iy]	= z_xy[ix][iy];
-			}
-		}
+			double	t_xy[4][4];
 
-		do
-		{
 			for(iy=0; iy<4; iy++)
 			{
 				for(ix=0; ix<4; ix++)
 				{
+					t_xy[ix][iy]	= z_xy[ix][iy];
+
 					if( is_NoData_Value(z_xy[ix][iy]) )
 					{
 						int		n	= 0;
+						double	s	= 0.0;
 
 						for(jy=iy-1; jy<=iy+1; jy++)
 						{
@@ -849,18 +975,10 @@ inline bool CSG_Grid::_Get_ValAtPos_Fill4x4Submatrix(int x, int y, double z_xy[4
 							{
 								for(jx=ix-1; jx<=ix+1; jx++)
 								{
-									if( jx >= 0 && jx < 4 && !(jx == ix && jy == iy) && !is_NoData_Value(z_xy[jx][jy]) )
+									if( jx >= 0 && jx < 4 && !is_NoData_Value(z_xy[jx][jy]) )
 									{
-										if( n == 0 )
-										{
-											t_xy[ix][iy]	 = z_xy[jx][jy];
-										}
-										else
-										{
-											t_xy[ix][iy]	+= z_xy[jx][jy];
-										}
-
-										n++;
+										s	+= z_xy[jx][jy];
+										n	++;
 									}
 								}
 							}
@@ -868,10 +986,7 @@ inline bool CSG_Grid::_Get_ValAtPos_Fill4x4Submatrix(int x, int y, double z_xy[4
 
 						if( n > 0 )
 						{
-							if( n > 1 )
-							{
-								t_xy[ix][iy]	/= n;
-							}
+							t_xy[ix][iy]	= s / n;
 
 							nNoData--;
 						}
@@ -883,25 +998,23 @@ inline bool CSG_Grid::_Get_ValAtPos_Fill4x4Submatrix(int x, int y, double z_xy[4
 			{
 				for(ix=0; ix<4; ix++)
 				{
-					if( t_xy[ix][iy] != z_xy[ix][iy] )
-					{
-						z_xy[ix][iy]	= t_xy[ix][iy];
-					}
+					z_xy[ix][iy]	= t_xy[ix][iy];
 				}
 			}
 		}
-		while( nNoData > 0 );
 	}
 
-	return( true );
+	//-----------------------------------------------------
+	return( nNoData == 0 );
 }
 
 //---------------------------------------------------------
 inline bool CSG_Grid::_Get_ValAtPos_Fill4x4Submatrix(int x, int y, double z_xy[4][4][4]) const
 {
-	int		ix, iy, jx, jy, nNoData;
+	int		ix, iy, jx, jy, nNoData	= 0;
 
-	for(iy=0, jy=y-1, nNoData=0; iy<4; iy++, jy++)
+	//-----------------------------------------------------
+	for(iy=0, jy=y-1; iy<4; iy++, jy++)
 	{
 		for(ix=0, jx=x-1; ix<4; ix++, jx++)
 		{
@@ -924,35 +1037,25 @@ inline bool CSG_Grid::_Get_ValAtPos_Fill4x4Submatrix(int x, int y, double z_xy[4
 	}
 
 	//-----------------------------------------------------
-	if( nNoData >= 16 )
+	if( nNoData > 0 && nNoData < 16 )	// guess missing values as average of surrounding data values
 	{
-		return( false );
-	}
-
-	if( nNoData > 0 )
-	{
-		double	t_xy[4][4][4];
-
-		for(iy=0; iy<4; iy++)
+		for(int i=0; nNoData>0 && i<16; i++)	// avoid the possibility of endless loop
 		{
-			for(ix=0; ix<4; ix++)
-			{
-				t_xy[0][ix][iy]	= z_xy[0][ix][iy];
-				t_xy[1][ix][iy]	= z_xy[1][ix][iy];
-				t_xy[2][ix][iy]	= z_xy[2][ix][iy];
-				t_xy[3][ix][iy]	= z_xy[3][ix][iy];
-			}
-		}
+			double	t_xy[4][4][4];
 
-		do
-		{
 			for(iy=0; iy<4; iy++)
 			{
 				for(ix=0; ix<4; ix++)
 				{
+					t_xy[0][ix][iy]	 = z_xy[0][ix][iy];
+					t_xy[1][ix][iy]	 = z_xy[1][ix][iy];
+					t_xy[2][ix][iy]	 = z_xy[2][ix][iy];
+					t_xy[3][ix][iy]	 = z_xy[3][ix][iy];
+
 					if( is_NoData_Value(z_xy[0][ix][iy]) )
 					{
 						int		n	= 0;
+						double	s[4]; s[0] = s[1] = s[2] = s[3] = 0;
 
 						for(jy=iy-1; jy<=iy+1; jy++)
 						{
@@ -960,24 +1063,13 @@ inline bool CSG_Grid::_Get_ValAtPos_Fill4x4Submatrix(int x, int y, double z_xy[4
 							{
 								for(jx=ix-1; jx<=ix+1; jx++)
 								{
-									if( jx >= 0 && jx < 4 && !(jx == ix && jy == iy) && !is_NoData_Value(z_xy[0][jx][jy]) )
+									if( jx >= 0 && jx < 4 && !is_NoData_Value(z_xy[0][jx][jy]) )
 									{
-										if( n == 0 )
-										{
-											t_xy[0][ix][iy]	 = z_xy[0][jx][jy];
-											t_xy[1][ix][iy]	 = z_xy[1][jx][jy];
-											t_xy[2][ix][iy]	 = z_xy[2][jx][jy];
-											t_xy[3][ix][iy]	 = z_xy[3][jx][jy];
-										}
-										else
-										{
-											t_xy[0][ix][iy]	+= z_xy[0][jx][jy];
-											t_xy[1][ix][iy]	+= z_xy[1][jx][jy];
-											t_xy[2][ix][iy]	+= z_xy[2][jx][jy];
-											t_xy[3][ix][iy]	+= z_xy[3][jx][jy];
-										}
-
-										n++;
+										s[0]	+= z_xy[0][jx][jy];
+										s[1]	+= z_xy[1][jx][jy];
+										s[2]	+= z_xy[2][jx][jy];
+										s[3]	+= z_xy[3][jx][jy];
+										n		++;
 									}
 								}
 							}
@@ -985,13 +1077,10 @@ inline bool CSG_Grid::_Get_ValAtPos_Fill4x4Submatrix(int x, int y, double z_xy[4
 
 						if( n > 0 )
 						{
-							if( n > 1 )
-							{
-								t_xy[0][ix][iy]	/= n;
-								t_xy[1][ix][iy]	/= n;
-								t_xy[2][ix][iy]	/= n;
-								t_xy[3][ix][iy]	/= n;
-							}
+							t_xy[0][ix][iy]	= s[0] / n;
+							t_xy[1][ix][iy]	= s[1] / n;
+							t_xy[2][ix][iy]	= s[2] / n;
+							t_xy[3][ix][iy]	= s[3] / n;
 
 							nNoData--;
 						}
@@ -1003,20 +1092,17 @@ inline bool CSG_Grid::_Get_ValAtPos_Fill4x4Submatrix(int x, int y, double z_xy[4
 			{
 				for(ix=0; ix<4; ix++)
 				{
-					if( t_xy[0][ix][iy] != z_xy[0][ix][iy] )
-					{
-						z_xy[0][ix][iy]	= t_xy[0][ix][iy];
-						z_xy[1][ix][iy]	= t_xy[1][ix][iy];
-						z_xy[2][ix][iy]	= t_xy[2][ix][iy];
-						z_xy[3][ix][iy]	= t_xy[3][ix][iy];
-					}
+					z_xy[0][ix][iy]	= t_xy[0][ix][iy];
+					z_xy[1][ix][iy]	= t_xy[1][ix][iy];
+					z_xy[2][ix][iy]	= t_xy[2][ix][iy];
+					z_xy[3][ix][iy]	= t_xy[3][ix][iy];
 				}
 			}
 		}
-		while( nNoData > 0 );
 	}
 
-	return( true );
+	//-----------------------------------------------------
+	return( nNoData == 0 );
 }
 
 
@@ -1027,29 +1113,54 @@ inline bool CSG_Grid::_Get_ValAtPos_Fill4x4Submatrix(int x, int y, double z_xy[4
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-double CSG_Grid::Get_ZMin(bool bZFactor)
+bool CSG_Grid::On_Update(void)
 {
-	Update();	return( (bZFactor ? m_zFactor : 1.0) * m_zStats.Get_Minimum() );
+	if( is_Valid() )
+	{
+		m_zStats.Invalidate();
+
+		for(int y=0; y<Get_NY() && SG_UI_Process_Get_Okay(); y++)
+		{
+			for(int x=0; x<Get_NX(); x++)
+			{
+				if( !is_NoData(x, y) )
+				{
+					m_zStats.Add_Value(asDouble(x, y));
+				}
+			}
+		}
+
+		m_bIndex	= false;
+		SG_FREE_SAFE(m_Index);
+	}
+
+	return( true );
 }
 
-double CSG_Grid::Get_ZMax(bool bZFactor)
+//---------------------------------------------------------
+double CSG_Grid::Get_ZMin(void)
 {
-	Update();	return( (bZFactor ? m_zFactor : 1.0) * m_zStats.Get_Maximum() );
+	Update();	return( m_zStats.Get_Minimum() );
 }
 
-double CSG_Grid::Get_ZRange(bool bZFactor)
+double CSG_Grid::Get_ZMax(void)
 {
-	Update();	return( (bZFactor ? m_zFactor : 1.0) * m_zStats.Get_Range() );
+	Update();	return( m_zStats.Get_Maximum() );
 }
 
-double CSG_Grid::Get_ArithMean(bool bZFactor)
+double CSG_Grid::Get_ZRange(void)
 {
-	Update();	return( (bZFactor ? m_zFactor : 1.0) * m_zStats.Get_Mean() );
+	Update();	return( m_zStats.Get_Range() );
 }
 
-double CSG_Grid::Get_StdDev(bool bZFactor)
+double CSG_Grid::Get_Mean(void)
 {
-	Update();	return( (bZFactor ? m_zFactor : 1.0) * m_zStats.Get_StdDev() );
+	Update();	return( m_zStats.Get_Mean() );
+}
+
+double CSG_Grid::Get_StdDev(void)
+{
+	Update();	return( m_zStats.Get_StdDev() );
 }
 
 double CSG_Grid::Get_Variance(void)
@@ -1057,37 +1168,30 @@ double CSG_Grid::Get_Variance(void)
 	Update();	return( m_zStats.Get_Variance() );
 }
 
+//---------------------------------------------------------
+sLong CSG_Grid::Get_Data_Count(void)
+{
+	Update();	return( m_zStats.Get_Count() );
+}
+
 sLong CSG_Grid::Get_NoData_Count(void)
 {
 	Update();	return( Get_NCells() - m_zStats.Get_Count() );
 }
 
 //---------------------------------------------------------
-bool CSG_Grid::On_Update(void)
+double CSG_Grid::Get_Percentile(double Percent)
 {
-	if( is_Valid() )
+	Percent	= Percent <= 0.0 ? 0.0 : Percent >= 100.0 ? 1.0 : Percent / 100.0;
+
+	sLong	n	= (sLong)(Percent * (Get_Data_Count() - 1));
+
+	if( Get_Sorted(n, n, false) )
 	{
-		m_zStats.Invalidate();
-
-		for(int y=0; y<Get_NY(); y++)
-		{
-			SG_UI_Process_Get_Okay();
-
-			for(int x=0; x<Get_NX(); x++)
-			{
-				double	z	= asDouble(x, y);
-
-				if( !is_NoData_Value(z) )
-				{
-					m_zStats.Add_Value(z);
-				}
-			}
-		}
-
-		SG_UI_Process_Set_Ready();
+		return( asDouble(n) );
 	}
 
-	return( true );
+	return( Get_NoData_Value() );
 }
 
 
@@ -1097,183 +1201,49 @@ bool CSG_Grid::On_Update(void)
 //														 //
 ///////////////////////////////////////////////////////////
 
-//---------------------------------------------------------
-void CSG_Grid::Set_Value_And_Sort(sLong n, double Value)
-{
-	if( !m_bIndexed )
-	{
-		Set_Value(n, Value);
-
-		Set_Index(true);
-
-		return;
-	}
-
-	//-----------------------------------------------------
-	if( Value == asDouble(n) )
-		return;
-
-	sLong	i, j;
-
-	for(i=0, j=-1; i<Get_NCells() && j<0; i++)	// find index, could be faster...
-	{
-		if( n == m_Index[i] )
-		{
-			j	= i;
-		}
-	}
-
-	if( j > 0 )
-	{
-		if( Value < asDouble(n) )
-		{
-			for(i=j-1; i>=0; i--, j--)
-			{
-				if( Value < asDouble(m_Index[i]) )
-				{
-					m_Index[j]	= m_Index[i];
-				}
-				else
-				{
-					m_Index[j]	= n;
-					break;
-				}
-			}
-		}
-		else
-		{
-			for(i=j+1; i<Get_NCells(); i++, j++)
-			{
-				if( Value > asDouble(m_Index[i]) )
-				{
-					m_Index[j]	= m_Index[i];
-				}
-				else
-				{
-					m_Index[j]	= n;
-					break;
-				}
-			}
-		}
-
-		Set_Value(n, Value);
-
-		m_bIndexed	= true;
-	}
-}
-
-//---------------------------------------------------------
-void CSG_Grid::Set_Value_And_Sort(int x, int y, double Value)
-{
-	Set_Value_And_Sort(x + y * Get_NX(), Value);
-}
-
-//---------------------------------------------------------
-double CSG_Grid::Get_Percentile(double Percent, bool bZFactor)
-{
-	int		x, y;
-
-	if( Percent < 0.0 )
-	{
-		Percent	= 0.0;
-	}
-	else if( Percent > 100.0 )
-	{
-		Percent	= 100.0;
-	}
-
-	if( Get_Sorted((int)(Percent * Get_NCells() / 100.0), x, y, true) )
-	{
-		return( asDouble(x, y, bZFactor) );
-	}
-
-	return( 0.0 );
-}
-
-//---------------------------------------------------------
-bool CSG_Grid::Set_Index(bool bOn)
-{
-	if( bOn && !m_bIndexed && Get_NoData_Count() < Get_NCells() )
-	{
-		m_bIndexed	= true;
-
-		if( _Set_Index() == false )
-		{
-			Set_Index(false);
-
-			return( false );
-		}
-	}
-	else if( !bOn || Get_NoData_Count() >= Get_NCells() )
-	{
-		m_bIndexed	= false;
-
-		if( m_Index )
-		{
-			SG_Free(m_Index);
-			m_Index		= NULL;
-		}
-	}
-
-	return( m_bIndexed );
-}
-
 //---------------------------------------------------------
 #define SORT_SWAP(a,b)	{itemp=(a);(a)=(b);(b)=itemp;}
 
 bool CSG_Grid::_Set_Index(void)
 {
+	if( Get_Data_Count() <= 0 )
+	{
+		return( false );	// nothing to do
+	}
+
+	//-----------------------------------------------------
+	if( m_Index == NULL && (m_Index = (sLong *)SG_Malloc(Get_NCells() * sizeof(sLong))) == NULL )
+	{
+		SG_UI_Msg_Add_Error(_TL("could not create index: insufficient memory"));
+
+		return( false );
+	}
+
+	//-----------------------------------------------------
 	const sLong	M	= 7;
 
-	sLong	i, j, k, l, ir, n, nCells, *istack, jstack, nstack, indxt, itemp;
+	sLong	i, j, k, l, ir, n, *istack, jstack, nstack, indxt, itemp;
 	double	a;
 
 	//-----------------------------------------------------
 	SG_UI_Process_Set_Text(CSG_String::Format(SG_T("%s: %s"), _TL("Create index"), Get_Name()));
 
-	if( m_Index == NULL )
+	for(i=0, j=0, k=Get_Data_Count(); i<Get_NCells(); i++)
 	{
-		m_Index		= (sLong *)SG_Calloc(Get_NCells(), sizeof(sLong));
-
-		if( m_Index == NULL )
+		if( is_NoData(i) )
 		{
-			SG_UI_Msg_Add_Error(_TL("could not create index: insufficient memory"));
-
-			SG_UI_Process_Set_Ready();
-
-			return( false );
+			m_Index[k++]	= i;
 		}
-
-		for(i=0, l=0; i<Get_NCells(); i++)
+		else // if( !is_NoData(i) )
 		{
-			if(  is_NoData(i) )
-			{
-				m_Index[l++]	= i;
-			}
+			m_Index[j++]	= i;
 		}
-
-		for(i=0, j=l; i<Get_NCells(); i++)
-		{
-			if( !is_NoData(i) )
-			{
-				m_Index[j++]	= i;
-			}
-		}
-	}
-	else
-	{
-		l	= 0;
-	}
-
-	//-----------------------------------------------------
-	if( (nCells = Get_NCells() - l) <= 1 )
-	{
-		return( false );
 	}
 
 	//-----------------------------------------------------
+	l		= 0;
 	n		= 0;
-	ir		= Get_NCells() - 1;
+	ir		= Get_Data_Count() - 1;
 
 	nstack	= 64;
 	istack	= (sLong *)SG_Malloc(nstack * sizeof(sLong));
@@ -1283,9 +1253,13 @@ bool CSG_Grid::_Set_Index(void)
 	{
 		if( ir - l < M )
 		{
-			if( !SG_UI_Process_Set_Progress((double)(n += M - 1), (double)nCells) )
+			if( !SG_UI_Process_Set_Progress((double)(n += M - 1), (double)Get_Data_Count()) )
 			{
-				SG_Free(istack);
+				SG_FREE_SAFE(istack);
+				SG_FREE_SAFE(m_Index);
+
+				SG_UI_Msg_Add_Error(_TL("index creation stopped by user"));
+				SG_UI_Process_Set_Ready();
 
 				return( false );
 			}
@@ -1381,6 +1355,8 @@ bool CSG_Grid::_Set_Index(void)
 
 	SG_UI_Process_Set_Ready();
 
+	m_bIndex	= true;
+
 	return( true );
 }
 #undef SORT_SWAP
