@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: grid.h 2242 2014-09-22 13:27:38Z oconrad $
+ * Version $Id: grid.h 2593 2015-09-23 07:34:34Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -369,6 +369,65 @@ private:	///////////////////////////////////////////////
 };
 
 
+///////////////////////////////////////////////////////////
+//														 //
+//						CSG_Grid						 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+/**
+  * CSG_Grid is the data object created for raster handling.
+*/
+//---------------------------------------------------------
+class SAGA_API_DLL_EXPORT CSG_Grid_File_Info
+{
+//---------------------------------------------------------
+public:		///////////////////////////////////////////////
+
+	//-----------------------------------------------------
+	CSG_Grid_File_Info(void);
+
+								CSG_Grid_File_Info		(const CSG_Grid_File_Info &Info);
+	bool						Create					(const CSG_Grid_File_Info &Info);
+
+								CSG_Grid_File_Info		(const CSG_String &File_Name);
+	bool						Create					(const CSG_String &File_Name);
+
+								CSG_Grid_File_Info		(const CSG_Grid &Grid);
+	bool						Create					(const CSG_Grid &Grid);
+
+	bool						Save					(const CSG_String &File_Name                                                                      , bool bBinary = true);
+	bool						Save					(const CSG_String &File_Name                      , int xStart, int yStart, int xCount, int yCount, bool bBinary = true);
+	static bool					Save					(const CSG_String &File_Name, const CSG_Grid &Grid                                                , bool bBinary = true);
+	static bool					Save					(const CSG_String &File_Name, const CSG_Grid &Grid, int xStart, int yStart, int xCount, int yCount, bool bBinary = true);
+
+
+	//-----------------------------------------------------
+	bool						m_bFlip, m_bSwapBytes;
+
+	sLong						m_Offset;
+
+	double						m_zScale, m_zOffset, m_NoData;
+
+	TSG_Data_Type				m_Type;
+
+	CSG_String					m_Name, m_Description, m_Unit, m_Data_File;
+
+	CSG_Grid_System				m_System;
+
+	CSG_Projection				m_Projection;
+
+
+private:
+
+	void						_On_Construction		(void);
+
+	int							_Get_Key				(CSG_File &Stream, CSG_String &Line);
+
+};
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 //						CSG_Grid						 //
@@ -573,17 +632,17 @@ public:		///////////////////////////////////////////////
 
 	bool						Get_Sorted		(sLong Position, sLong &n, bool bDown = true, bool bCheckNoData = true)
 	{
-		return( (n = Get_Sorted(Position, bDown, bCheckNoData)) >= 0 );
+		return( (n = Get_Sorted(Position, bDown, false)) >= 0 && (!bCheckNoData || !is_NoData(n)) );
 	}
 
 	bool						Get_Sorted		(sLong Position, int &x, int &y, bool bDown = true, bool bCheckNoData = true)
 	{
-		if( (Position = Get_Sorted(Position, bDown, bCheckNoData)) >= 0 )
+		if( (Position = Get_Sorted(Position, bDown, false)) >= 0 )
 		{
 			x	= (int)(Position % Get_NX());
 			y	= (int)(Position / Get_NX());
 
-			return( true );
+			return( !bCheckNoData || !is_NoData(x, y) );
 		}
 
 		return( false );
@@ -675,15 +734,16 @@ public:		///////////////////////////////////////////////
 		else switch( m_Type )
 		{
 			default:	return( 0.0 );
-			case SG_DATATYPE_Byte:		Value	= ((BYTE   **)m_Values)[y][x];	break;
-			case SG_DATATYPE_Char:		Value	= ((char   **)m_Values)[y][x];	break;
-			case SG_DATATYPE_Word:		Value	= ((WORD   **)m_Values)[y][x];	break;
-			case SG_DATATYPE_Short:		Value	= ((short  **)m_Values)[y][x];	break;
-			case SG_DATATYPE_DWord:		Value	= ((DWORD  **)m_Values)[y][x];	break;
-			case SG_DATATYPE_Int:		Value	= ((int    **)m_Values)[y][x];	break;
-			case SG_DATATYPE_Float:		Value	= ((float  **)m_Values)[y][x];	break;
-			case SG_DATATYPE_Double:	Value	= ((double **)m_Values)[y][x];	break;
-			case SG_DATATYPE_Bit:		Value	=(((BYTE   **)m_Values)[y][x / 8] & m_Bitmask[x % 8]) == 0 ? 0.0 : 1.0;	break;
+			case SG_DATATYPE_Byte:		Value	= (double)((BYTE   **)m_Values)[y][x];	break;
+			case SG_DATATYPE_Char:		Value	= (double)((char   **)m_Values)[y][x];	break;
+			case SG_DATATYPE_Word:		Value	= (double)((WORD   **)m_Values)[y][x];	break;
+			case SG_DATATYPE_Short:		Value	= (double)((short  **)m_Values)[y][x];	break;
+			case SG_DATATYPE_DWord:		Value	= (double)((DWORD  **)m_Values)[y][x];	break;
+			case SG_DATATYPE_Int:		Value	= (double)((int    **)m_Values)[y][x];	break;
+			case SG_DATATYPE_Long:		Value	= (double)((sLong  **)m_Values)[y][x];	break;
+			case SG_DATATYPE_Float:		Value	= (double)((float  **)m_Values)[y][x];	break;
+			case SG_DATATYPE_Double:	Value	= (double)((double **)m_Values)[y][x];	break;
+			case SG_DATATYPE_Bit:		Value	=(double)(((BYTE   **)m_Values)[y][x / 8] & m_Bitmask[x % 8]) == 0 ? 0.0 : 1.0;	break;
 		}
 
 		if( bScaled && is_Scaled() )
@@ -730,6 +790,7 @@ public:		///////////////////////////////////////////////
 			case SG_DATATYPE_Short:		((short  **)m_Values)[y][x]	= SG_ROUND_TO_SHORT(Value);	break;
 			case SG_DATATYPE_DWord:		((DWORD  **)m_Values)[y][x]	= SG_ROUND_TO_DWORD(Value);	break;
 			case SG_DATATYPE_Int:		((int    **)m_Values)[y][x]	= SG_ROUND_TO_INT  (Value);	break;
+			case SG_DATATYPE_Long:		((sLong  **)m_Values)[y][x]	= SG_ROUND_TO_SLONG(Value);	break;
 			case SG_DATATYPE_Float:		((float  **)m_Values)[y][x]	= (float )Value;			break;
 			case SG_DATATYPE_Double:	((double **)m_Values)[y][x]	= (double)Value;			break;
 			case SG_DATATYPE_Bit:		((BYTE   **)m_Values)[y][x / 8]	= Value != 0.0
@@ -842,8 +903,6 @@ private:	///////////////////////////////////////////////
 	bool						_Load_Native			(const CSG_String &File_Name, TSG_Grid_Memory_Type aMemory_Type, bool bLoadData);
 	bool						_Save_Native			(const CSG_String &File_Name, int xA, int yA, int xN, int yN, bool bBinary = true);
 
-	int							_Load_Native_Get_Key	(CSG_File &Stream, CSG_String &Line);
-
 	bool						_Load_Surfer			(const CSG_String &File_Name, TSG_Grid_Memory_Type aMemory_Type, bool bLoadData);
 
 	//-----------------------------------------------------
@@ -878,6 +937,10 @@ private:	///////////////////////////////////////////////
 //														 //
 ///////////////////////////////////////////////////////////
 
+//---------------------------------------------------------
+#define SG_GRID_PTR_SAFE_SET_NODATA(g, x, y)	{ if( g && g->is_InGrid(x, y, false) ) { g->Set_NoData(x, y   ); } }
+#define SG_GRID_PTR_SAFE_SET_VALUE(g, x, y, z)	{ if( g && g->is_InGrid(x, y, false) ) { g->Set_Value (x, y, z); } }
+
 //---------------------------------------------------------
 /** Safe grid construction */
 SAGA_API_DLL_EXPORT CSG_Grid *		SG_Create_Grid		(void);
