@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: mat_tools.h 2208 2014-08-27 16:00:51Z oconrad $
+ * Version $Id: mat_tools.h 2589 2015-09-18 15:40:49Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -141,7 +141,8 @@
 //---------------------------------------------------------
 SAGA_API_DLL_EXPORT double		SG_Get_Square			(double Value);
 
-SAGA_API_DLL_EXPORT double		SG_Get_Rounded			(double Value, int Decimals = 0);
+SAGA_API_DLL_EXPORT double		SG_Get_Rounded							(double Value, int Decimals = 0);
+SAGA_API_DLL_EXPORT double		SG_Get_Rounded_To_SignificantFigures	(double Value, int Decimals);
 
 //---------------------------------------------------------
 SAGA_API_DLL_EXPORT int			SG_Get_Digit_Count		(int Number);
@@ -149,6 +150,11 @@ SAGA_API_DLL_EXPORT int			SG_Get_Digit_Count		(int Number);
 //---------------------------------------------------------
 SAGA_API_DLL_EXPORT	CSG_String	SG_Get_Double_asString	(double Number, int Width = -1, int Precision = -1, bool bScientific = false);
 
+//---------------------------------------------------------
+SAGA_API_DLL_EXPORT	int			SG_Compare_Int			(const void *a, const void *b);
+SAGA_API_DLL_EXPORT	int			SG_Compare_Double		(const void *a, const void *b);
+SAGA_API_DLL_EXPORT	int			SG_Compare_Char_Ptr		(const void *a, const void *b);
+
 
 ///////////////////////////////////////////////////////////
 //														 //
@@ -227,7 +233,7 @@ public:
 	bool						Add_Rows			(int nRows);
 	bool						Del_Rows			(int nRows);
 	bool						Add_Row				(double Value = 0.0);
-	bool						Del_Row				(void);
+	bool						Del_Row				(int iRow = -1);
 
 	int							Get_N				(void)	const	{	return( (int)Get_Size() );					}
 	size_t						Get_Size			(void)	const	{	return( m_Array.Get_Size() );				}
@@ -237,7 +243,8 @@ public:
 	double &					operator []			(int x)			{	return( Get_Data()[x] );	}
 	operator const double *							(void)	const	{	return( Get_Data() );		}
 
-	CSG_String					asString			(int Width = -1, int Precision = -1, bool bScientific = false)	const;
+	CSG_String					to_String			(int Width = -1, int Precision = -1, bool bScientific = false, const SG_Char *Separator = NULL)	const;
+	bool						from_String			(const CSG_String &String);
 
 	bool						is_Equal			(const CSG_Vector &Vector)	const;
 
@@ -271,6 +278,8 @@ public:
 	bool						Set_Zero			(void);
 	bool						Set_Unity			(void);
 
+	bool						Sort				(void);
+
 	double						Get_Length			(void)						const;
 	double						Get_Angle			(const CSG_Vector &Vector)	const;
 	CSG_Vector					Get_Unity			(void)						const;
@@ -349,7 +358,8 @@ public:
 	double						operator ()			(int y, int x)	const	{	return( m_z[y][x] );	}
 	double *					operator []			(int y)			const	{	return( m_z[y] );		}
 
-	CSG_String					asString			(int Width = -1, int Precision = -1, bool bScientific = false)	const;
+	CSG_String					to_String			(int Width = -1, int Precision = -1, bool bScientific = false, const SG_Char *Separator = NULL)	const;
+	bool						from_String			(const CSG_String &String);
 
 	bool						is_Square			(void)	const	{	return( m_nx > 0 && m_nx == m_ny );	}
 	bool						is_Equal			(const CSG_Matrix &Matrix)	const;
@@ -405,7 +415,7 @@ private:
 SAGA_API_DLL_EXPORT CSG_Matrix	operator *			(double Scalar, const CSG_Matrix &Matrix);
 
 //---------------------------------------------------------
-SAGA_API_DLL_EXPORT bool		SG_Matrix_LU_Decomposition	(int n,       int *Permutation,       double **Matrix                , bool bSilent = true);
+SAGA_API_DLL_EXPORT bool		SG_Matrix_LU_Decomposition	(int n,       int *Permutation,       double **Matrix                , bool bSilent = true, int *nRowChanges = NULL);
 SAGA_API_DLL_EXPORT bool		SG_Matrix_LU_Solve			(int n, const int *Permutation, const double **Matrix, double *Vector, bool bSilent = true);
 
 SAGA_API_DLL_EXPORT bool		SG_Matrix_Solve				(CSG_Matrix &Matrix, CSG_Vector &Vector, bool bSilent = true);
@@ -540,25 +550,34 @@ public:
 	CSG_Simple_Statistics(bool bHoldValues);
 	CSG_Simple_Statistics(const CSG_Simple_Statistics &Statistics);
 	CSG_Simple_Statistics(double Mean, double StdDev, sLong Count = 1000);
+	CSG_Simple_Statistics(const CSG_Vector &Values, bool bHoldValues = false);
 
 	bool						Create				(bool bHoldValues = false);
 	bool						Create				(const CSG_Simple_Statistics &Statistics);
 	bool						Create				(double Mean, double StdDev, sLong Count = 1000);
+	bool						Create				(const CSG_Vector &Values, bool bHoldValues = false);
 
 	void						Invalidate			(void);
 
-	bool						is_Evaluated		(void)	const	{	return( m_bEvaluated );	}
+	int							is_Evaluated		(void)	const	{	return( m_bEvaluated );	}
 
 	sLong						Get_Count			(void)	const	{	return( m_nValues );	}
 	double						Get_Weights			(void)	const	{	return( m_Weights );	}
 
-	double						Get_Minimum			(void)		{	if( !m_bEvaluated )	_Evaluate(); return( m_Minimum	);	}
-	double						Get_Maximum			(void)		{	if( !m_bEvaluated )	_Evaluate(); return( m_Maximum	);	}
-	double						Get_Range			(void)		{	if( !m_bEvaluated )	_Evaluate(); return( m_Range	);	}
-	double						Get_Sum				(void)		{	if( !m_bEvaluated )	_Evaluate(); return( m_Sum		);	}
-	double						Get_Mean			(void)		{	if( !m_bEvaluated )	_Evaluate(); return( m_Mean		);	}
-	double						Get_Variance		(void)		{	if( !m_bEvaluated )	_Evaluate(); return( m_Variance	);	}
-	double						Get_StdDev			(void)		{	if( !m_bEvaluated )	_Evaluate(); return( m_StdDev	);	}
+	double						Get_Minimum			(void)		{	if( m_bEvaluated < 1 )	_Evaluate(1); return( m_Minimum  );	}
+	double						Get_Maximum			(void)		{	if( m_bEvaluated < 1 )	_Evaluate(1); return( m_Maximum  );	}
+	double						Get_Range			(void)		{	if( m_bEvaluated < 1 )	_Evaluate(1); return( m_Range    );	}
+	double						Get_Sum				(void)		{	if( m_bEvaluated < 1 )	_Evaluate(1); return( m_Sum      );	}
+	double						Get_Mean			(void)		{	if( m_bEvaluated < 1 )	_Evaluate(1); return( m_Mean     );	}
+	double						Get_Variance		(void)		{	if( m_bEvaluated < 1 )	_Evaluate(1); return( m_Variance );	}
+	double						Get_StdDev			(void)		{	if( m_bEvaluated < 1 )	_Evaluate(1); return( m_StdDev   );	}
+
+	double						Get_Kurtosis		(void)		{	if( m_bEvaluated < 2 )	_Evaluate(2); return( m_Kurtosis );	}
+	double						Get_Skewness		(void)		{	if( m_bEvaluated < 2 )	_Evaluate(2); return( m_Skewness );	}
+	double						Get_SkewnessPearson	(void);
+
+	double						Get_Median			(void)		{	return( Get_Quantile(50.0) );	}
+	double						Get_Quantile		(double Quantile);
 
 	void						Add					(const CSG_Simple_Statistics &Statistics);
 
@@ -575,16 +594,18 @@ public:
 
 protected:
 
-	bool						m_bEvaluated;
+	bool						m_bSorted;
+
+	int							m_bEvaluated;
 
 	sLong						m_nValues;
 
-	double						m_Weights, m_Sum, m_Sum2, m_Minimum, m_Maximum, m_Range, m_Mean, m_Variance, m_StdDev;
+	double						m_Weights, m_Sum, m_Sum2, m_Minimum, m_Maximum, m_Range, m_Mean, m_Variance, m_StdDev, m_Kurtosis, m_Skewness;
 
 	CSG_Array					m_Values;
 
 
-	void						_Evaluate			(void);
+	void						_Evaluate			(int Level = 1);
 
 };
 
@@ -610,12 +631,11 @@ private:
 
 
 public:
-	 CSG_Class_Statistics(void);
-	~CSG_Class_Statistics(void);
+	         CSG_Class_Statistics			(void)	{	Create ();	}
+	virtual ~CSG_Class_Statistics			(void)	{	Destroy();	}
 
 	void			Create					(void);
-	void			Destroy					(void);
-
+	void			Destroy					(void)	{	m_Array.Set_Array(0, (void **)&m_Classes);	}
 	void			Reset					(void)	{	m_Array.Set_Array(0, (void **)&m_Classes, false);	}
 
 	int				Get_Count				(void)	{	return( (int)m_Array.Get_Size() );	}
@@ -651,6 +671,79 @@ public:
 
 	void			Add_Value				(double Value);
 
+	int				Get_Majority			(void);
+	bool			Get_Majority			(double &Value            )	{	int	Count; return( Get_Class(Get_Majority(), Value, Count) );	}
+	bool			Get_Majority			(double &Value, int &Count)	{	           return( Get_Class(Get_Majority(), Value, Count) && Count > 0 );	}
+
+	int				Get_Minority			(void);
+	bool			Get_Minority			(double &Value            )	{	int	Count; return( Get_Class(Get_Minority(), Value, Count) );	}
+	bool			Get_Minority			(double &Value, int &Count)	{	           return( Get_Class(Get_Minority(), Value, Count) && Count > 0 );	}
+
+
+private:
+
+	CSG_Array		m_Array;
+
+	TClass			*m_Classes;
+
+};
+
+//---------------------------------------------------------
+class SAGA_API_DLL_EXPORT CSG_Class_Statistics_Weighted
+{
+private:
+
+	typedef struct SClass
+	{
+		int		Count;
+
+		double	Value, Weight;
+	}
+	TClass;
+
+
+public:
+	         CSG_Class_Statistics_Weighted	(void)	{	Create ();	}
+	virtual ~CSG_Class_Statistics_Weighted	(void)	{	Destroy();	}
+
+	void			Create					(void);
+	void			Destroy					(void)	{	m_Array.Set_Array(0, (void **)&m_Classes);	}
+	void			Reset					(void)	{	m_Array.Set_Array(0, (void **)&m_Classes, false);	}
+
+	int				Get_Count				(void)	{	return( (int)m_Array.Get_Size() );	}
+
+	int				Get_Class_Count			(int i)	{	return( i >= 0 && i < Get_Count() ? m_Classes[i].Count : 0 );	}
+	double			Get_Class_Value			(int i)	{	return( i >= 0 && i < Get_Count() ? m_Classes[i].Value : 0 );	}
+	double			Get_Class_Weight		(int i)	{	return( i >= 0 && i < Get_Count() ? m_Classes[i].Weight : 0 );	}
+
+	bool			Get_Class				(int i, double &Value, int &Count)
+	{
+		if( i >= 0 && i < Get_Count() )
+		{
+			Count	= m_Classes[i].Count;
+			Value	= m_Classes[i].Value;
+
+			return( true );
+		}
+
+		return( false );
+	}
+
+	bool			Get_Class				(int i, int &Value, int &Count)
+	{
+		if( i >= 0 && i < Get_Count() )
+		{
+			Count	=      m_Classes[i].Count;
+			Value	= (int)m_Classes[i].Value;
+
+			return( true );
+		}
+
+		return( false );
+	}
+
+	void			Add_Value				(double Value, double Weight = 1.0);
+
 	int				Get_Majority			(void);
 	bool			Get_Majority			(double &Value);
 	bool			Get_Majority			(double &Value, int &Count);
@@ -758,35 +851,41 @@ public:
 	void						Create						(int nFeatures);
 	void						Destroy						(void);
 
-	int							Get_Feature_Count			(void)			{	return( m_nFeatures );			}
+	bool						Load						(const CSG_String &File);
+	bool						Save						(const CSG_String &File, const SG_Char *Feature_Info = NULL);
 
-	int							Get_Element_Count			(int iClass)	{	return( m_nElements[iClass] );	}
-	void						Del_Element_Count			(void);
+	bool						Train_Clr_Samples			(void);
+	bool						Train_Add_Sample			(const CSG_String &Class_ID, const CSG_Vector &Features);
+	bool						Train						(bool bClr_Samples = false);
 
-	int							Get_Class_Count				(void)			{	return( m_IDs.Get_Count() );	}
-	const CSG_String &			Get_Class_ID				(int iClass)	{	return( m_IDs[iClass] );		}
+	bool						Add_Class					(const CSG_String &Class_ID, const CSG_Vector &Mean, const CSG_Vector &Min, const CSG_Vector &Max, const CSG_Matrix &Cov);
+
+	CSG_String					Print						(void);
+
+	//-----------------------------------------------------
+	int							Get_Feature_Count			(void)			{	return( m_nFeatures );				}
+
+	int							Get_Class_Count				(void)			{	return( m_nClasses );				}
+	const CSG_String &			Get_Class_ID				(int iClass)	{	return( m_pClasses[iClass]->m_ID );	}
 
 	int							Get_Class					(const CSG_String &Class_ID);
-
-	CSG_Simple_Statistics *		Get_Statistics				(const CSG_String &Class_ID);
-	CSG_Simple_Statistics *		Get_Statistics				(int iClass)	{	return( m_Statistics[iClass] );	}
-	CSG_Simple_Statistics *		operator []					(int iClass)	{	return( m_Statistics[iClass] );	}
-
 	bool						Get_Class					(const CSG_Vector &Features, int &Class, double &Quality, int Method);
 
-	void						Set_Distance_Threshold		(double Value)	{	m_Distance_Threshold	= Value;	}
-	double						Get_Distance_Threshold		(void)			{	return( m_Distance_Threshold );		}
+	//-----------------------------------------------------
+	void						Set_Threshold_Distance		(double Value);
+	double						Get_Threshold_Distance		(void);
 
-	void						Set_Probability_Threshold	(double Value)	{	m_Probability_Threshold	= Value;	}
-	double						Get_Probability_Threshold	(void)			{	return( m_Probability_Threshold );	}
-	void						Set_Probability_Relative	(bool   Value)	{	m_Probability_Relative	= Value;	}
-	bool						Get_Probability_Relative	(void)			{	return( m_Probability_Relative );	}
+	void						Set_Threshold_Angle			(double Value);
+	double						Get_Threshold_Angle			(void);
 
-	void						Set_Angle_Threshold			(double Value)	{	m_Angle_Threshold		= Value;	}
-	double						Get_Angle_Threshold			(void)			{	return( m_Angle_Threshold );		}
+	void						Set_Threshold_Probability	(double Value);
+	double						Get_Threshold_Probability	(void);
 
-	void						Set_WTA						(int Method, bool bOn)	{	if( Method >= 0 && Method < SG_CLASSIFY_SUPERVISED_WTA ) m_bWTA[Method]	= bOn;		}
-	bool						Get_WTA						(int Method)			{	return( Method >= 0 && Method < SG_CLASSIFY_SUPERVISED_WTA ? m_bWTA[Method] : false );	}
+	void						Set_Probability_Relative	(bool   Value);
+	bool						Get_Probability_Relative	(void);
+
+	void						Set_WTA						(int Method, bool bOn);
+	bool						Get_WTA						(int Method);
 
 	static CSG_String			Get_Name_of_Method			(int Method);
 	static CSG_String			Get_Name_of_Quality			(int Method);
@@ -794,22 +893,36 @@ public:
 
 private:
 
+	//-----------------------------------------------------
+	class CClass
+	{
+	public:
+		CClass(const CSG_String &ID) : m_ID(ID)	{}
+
+		CSG_String				m_ID;
+
+		double					m_Cov_Det, m_Mean_Spectral;
+
+		CSG_Vector				m_Mean, m_Min, m_Max;
+
+		CSG_Matrix				m_Cov, m_Cov_Inv, m_Samples;
+
+
+		bool					Train						(void);
+
+	};
+
+	//-----------------------------------------------------
 	bool						m_Probability_Relative, m_bWTA[SG_CLASSIFY_SUPERVISED_WTA];
 
-	int							m_nFeatures, *m_nElements;
+	int							m_nFeatures, m_nClasses;
 
-	double						m_Distance_Threshold, m_Probability_Threshold, m_Angle_Threshold;
+	double						m_Threshold_Distance, m_Threshold_Probability, m_Threshold_Angle;
 
-	CSG_Strings					m_IDs;
+	CSG_String					m_Info;
 
-	CSG_Simple_Statistics		**m_Statistics;
+	CClass						**m_pClasses;
 
-	CSG_Vector					m_ML_s, m_SAM_l, m_BE_m;
-
-	CSG_Matrix					m_ML_a, m_ML_b, m_BE_s;
-
-
-	void						_Update						(void);
 
 	void						_Get_Binary_Encoding		(const CSG_Vector &Features, int &Class, double &Quality);
 	void						_Get_Parallel_Epiped		(const CSG_Vector &Features, int &Class, double &Quality);
@@ -1299,6 +1412,53 @@ protected:
 };
 
 
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+class SAGA_API_DLL_EXPORT CSG_Regression_Weighted
+{
+public:
+	CSG_Regression_Weighted(void);
+	virtual ~CSG_Regression_Weighted(void);
+
+	bool						Destroy				(void);
+
+	bool						Add_Sample			(double Weight, double Dependent, const CSG_Vector &Predictors);
+	int							Get_Sample_Count	(void)	const	{	return( m_Y.Get_NRows()     );	}
+	int							Get_Predictor_Count	(void)	const	{	return( m_Y.Get_NCols() - 1 );	}
+
+	bool						Calculate			(const CSG_Vector &Weights, const CSG_Vector &Dependents, const CSG_Matrix &Predictors);
+	bool						Calculate			(void);
+
+	double						Get_R2				(void)	const	{	return( m_r2   );	}
+	const CSG_Vector &			Get_RCoeff			(void)	const	{	return( m_b    );	}
+	double						Get_RCoeff			(int i)	const	{	return( m_b[i] );	}
+	double						operator []			(int i)	const	{	return( m_b[i] );	}
+
+	//-----------------------------------------------------
+	bool						Get_CrossValidation	(int nSubSamples = 0);
+
+	int							Get_CV_nSamples		(void)	const	{	return( m_CV_nSamples );	}
+	double						Get_CV_RMSE			(void)	const	{	return( m_CV_RMSE     );	}
+	double						Get_CV_NRMSE		(void)	const	{	return( m_CV_NRMSE    );	}
+	double						Get_CV_R2			(void)	const	{	return( m_CV_R2       );	}
+
+
+private:
+
+	int							m_CV_nSamples;
+
+	double						m_r2, m_CV_RMSE, m_CV_NRMSE, m_CV_R2;
+
+	CSG_Vector					m_x, m_w, m_b;
+
+	CSG_Matrix					m_Y;
+
+};
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 //				Formula Parser (A. Ringeler)			 //
