@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: metadata.cpp 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: metadata.cpp 2291 2014-10-16 13:46:55Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -136,9 +136,8 @@ CSG_MetaData::CSG_MetaData(CSG_MetaData *pParent)
 void CSG_MetaData::_On_Construction(void)
 {
 	m_pParent	= NULL;
-	m_pChildren	= NULL;
-	m_nChildren	= 0;
-	m_nBuffer	= 0;
+
+	m_Children.Create(sizeof(CSG_MetaData **), 0, SG_ARRAY_GROWTH_1);
 }
 
 //---------------------------------------------------------
@@ -150,20 +149,15 @@ CSG_MetaData::~CSG_MetaData(void)
 //---------------------------------------------------------
 void CSG_MetaData::Destroy(void)
 {
-	if( m_pChildren )
+	CSG_MetaData	**m_pChildren	= (CSG_MetaData **)m_Children.Get_Array();
+
+	for(int i=0; i<Get_Children_Count(); i++)
 	{
-		for(int i=0; i<m_nChildren; i++)
-		{
-			delete(m_pChildren[i]);
-		}
-
-		SG_Free(m_pChildren);
-
-		m_pChildren	= NULL;
-		m_nChildren	= 0;
-		m_nBuffer	= 0;
+		delete(m_pChildren[i]);
 	}
 
+	m_Children.Destroy();
+
 //	m_pParent	= NULL;
 
 //	m_Name			.Clear();
@@ -178,78 +172,166 @@ void CSG_MetaData::Destroy(void)
 //														 //
 ///////////////////////////////////////////////////////////
 
-//---------------------------------------------------------
-#define GET_GROW_SIZE(n)	(n < 64 ? 1 : (n < 1024 ? 32 : 256))
-
 //---------------------------------------------------------
 CSG_MetaData * CSG_MetaData::Add_Child(void)
 {
-	if( (m_nChildren + 1) >= m_nBuffer )
-	{
-		CSG_MetaData	**pChildren	= (CSG_MetaData **)SG_Realloc(m_pChildren, (m_nBuffer + GET_GROW_SIZE(m_nBuffer)) * sizeof(CSG_MetaData *));
+	return( Ins_Child(-1) );
+}
 
-		if( pChildren )
-		{
-			m_pChildren	= pChildren;
-			m_nBuffer	+= GET_GROW_SIZE(m_nBuffer);
-		}
-		else
-		{
-			return( NULL );
-		}
-	}
+CSG_MetaData * CSG_MetaData::Add_Child(const CSG_String &Name)
+{
+	return( Ins_Child(Name, -1) );
+}
 
-	return( m_pChildren[m_nChildren++]	= new CSG_MetaData(this) );
+CSG_MetaData * CSG_MetaData::Add_Child(const CSG_String &Name, const CSG_String &Content)
+{
+	return( Ins_Child(Name, Content, -1) );
 }
 
 CSG_MetaData * CSG_MetaData::Add_Child(const CSG_String &Name, double Content)
 {
-	return( Add_Child(Name, SG_Get_String(Content, -16)) );
+	return( Ins_Child(Name, SG_Get_String(Content, -16), -1) );
 }
 
 CSG_MetaData * CSG_MetaData::Add_Child(const CSG_String &Name, int Content)
 {
-	return( Add_Child(Name, CSG_String::Format(SG_T("%d"), Content)) );
+	return( Ins_Child(Name, CSG_String::Format(SG_T("%d"), Content), -1) );
+}
+
+CSG_MetaData * CSG_MetaData::Add_Child(const CSG_MetaData &MetaData, bool bAddChildren)
+{
+	return( Ins_Child(MetaData, -1, bAddChildren) );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+CSG_MetaData * CSG_MetaData::Ins_Child(int Position)
+{
+	if( !m_Children.Inc_Array() )
+	{
+		return( NULL );
+	}
+
+	CSG_MetaData	**pChildren	= (CSG_MetaData **)m_Children.Get_Array();
+
+	if( Position < 0 || Position >= Get_Children_Count() )
+	{
+		Position	= Get_Children_Count() - 1;
+	}
+
+	for(int i=Get_Children_Count()-1; i>Position; i--)
+	{
+		pChildren[i]	= pChildren[i - 1];
+	}
+
+	return( pChildren[Position] = new CSG_MetaData(this) );
 }
 
 //---------------------------------------------------------
-CSG_MetaData * CSG_MetaData::Add_Child(const CSG_String &Name, const SG_Char *Content)
+CSG_MetaData * CSG_MetaData::Ins_Child(const CSG_String &Name, const CSG_String &Content, int Position)
 {
-	CSG_MetaData	*pChild	= Add_Child();
+	CSG_MetaData	*pChild	= Ins_Child(Position);
 
 	if( pChild )
 	{
 		pChild->m_Name		= Name;
-		pChild->m_Content	= Content ? Content : SG_T("");
+		pChild->m_Content	= Content;
 	}
 
 	return( pChild );
 }
 
+CSG_MetaData * CSG_MetaData::Ins_Child(const CSG_String &Name, int Position)
+{
+	return( Ins_Child(Name, CSG_String(""), Position) );
+}
+
+CSG_MetaData * CSG_MetaData::Ins_Child(const CSG_String &Name, double Content, int Position)
+{
+	return( Ins_Child(Name, SG_Get_String(Content, -16), Position) );
+}
+
+CSG_MetaData * CSG_MetaData::Ins_Child(const CSG_String &Name, int Content, int Position)
+{
+	return( Ins_Child(Name, CSG_String::Format(SG_T("%d"), Content), Position) );
+}
+
+CSG_MetaData * CSG_MetaData::Ins_Child(const CSG_MetaData &MetaData, int Position, bool bAddChildren)
+{
+	CSG_MetaData	*pChild	= Ins_Child(Position);
+
+	if( pChild )
+	{
+		pChild->Assign(MetaData, bAddChildren);
+	}
+
+	return( pChild );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CSG_MetaData::Mov_Child(int from_Index, int to_Index)
+{
+	if( from_Index < 0 || from_Index >= Get_Children_Count()
+	||    to_Index < 0 ||   to_Index >= Get_Children_Count() )
+	{
+		return( false );
+	}
+
+	if( from_Index != to_Index )
+	{
+		CSG_MetaData	**pChildren	= (CSG_MetaData **)m_Children.Get_Array();
+		CSG_MetaData	*pChild	= pChildren[from_Index];
+
+		if( from_Index < to_Index )
+		{
+			for(int i=from_Index; i<to_Index; i++)
+			{
+				pChildren[i]	= pChildren[i + 1];
+			}
+		}
+		else // if( from_Index > to_Index )
+		{
+			for(int i=from_Index; i>to_Index; i--)
+			{
+				pChildren[i]	= pChildren[i - 1];
+			}
+		}
+
+		pChildren[to_Index]	= pChild;
+	}
+
+	return( true );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
 //---------------------------------------------------------
 bool CSG_MetaData::Del_Child(int Index)
 {
-	if( Index >= 0 && Index < m_nChildren )
+	if( Index >= 0 && Index < Get_Children_Count() )
 	{
-		delete(m_pChildren[Index]);
+		CSG_MetaData	**pChildren	= (CSG_MetaData **)m_Children.Get_Array();
 
-		m_nChildren--;
+		delete(pChildren[Index]);
 
-		for(int i=Index; i<m_nChildren; i++)
+		for(int i=Index, j=Index+1; j<Get_Children_Count(); i++, j++)
 		{
-			m_pChildren[i]	= m_pChildren[i + 1];
+			pChildren[i]	= pChildren[j];
 		}
 
-		if( (m_nChildren - 1) < m_nBuffer - GET_GROW_SIZE(m_nBuffer) )
-		{
-			CSG_MetaData	**pChildren	= (CSG_MetaData **)SG_Realloc(m_pChildren, (m_nBuffer - GET_GROW_SIZE(m_nBuffer)) * sizeof(CSG_MetaData *));
-
-			if( pChildren )
-			{
-				m_pChildren	= pChildren;
-				m_nBuffer	-= GET_GROW_SIZE(m_nBuffer);
-			}
-		}
+		m_Children.Dec_Array();
 
 		return( true );
 	}
@@ -258,45 +340,78 @@ bool CSG_MetaData::Del_Child(int Index)
 }
 
 //---------------------------------------------------------
-bool CSG_MetaData::Del_Children(int Depth)
+bool CSG_MetaData::Add_Children(const CSG_MetaData &MetaData)
 {
-	if( Depth == 0 )
+	if( &MetaData != this )
 	{
-		if( m_pChildren )
+		for(int i=0; i<MetaData.Get_Children_Count(); i++)
 		{
-			for(int i=0; i<m_nChildren; i++)
+			Add_Child(MetaData[i], true);
+		}
+	}
+
+	return( true );
+}
+
+//---------------------------------------------------------
+/**
+  * Deletes children if depth level is reached, i.e. if depth level equals zero
+  * all children of the node will be deleted, if it is one only the grandchildren
+  * will be affected, and so on.
+  * If Name is a valid string, only those children are taken into account that
+  * have this name.
+*/
+//---------------------------------------------------------
+bool CSG_MetaData::Del_Children(int Depth, const SG_Char *Name)
+{
+	if( Depth < 0 )
+	{
+		// nop
+	}
+	else if( Name && *Name )
+	{
+		for(int i=Get_Children_Count()-1; i>=0; i--)
+		{
+			if( Get_Child(i)->Get_Name().CmpNoCase(Name) )
 			{
-				delete(m_pChildren[i]);
+				Get_Child(i)->Del_Children(Depth, Name);
+			}
+			else if( Depth > 0 )
+			{
+				Get_Child(i)->Del_Children(Depth - 1, Name);
+			}
+			else
+			{
+				Del_Child(i);
 			}
-
-			SG_Free(m_pChildren);
-
-			m_pChildren	= NULL;
-			m_nChildren	= 0;
-			m_nBuffer	= 0;
-
-			return( true );
 		}
 	}
 	else if( Depth > 0 )
 	{
 		for(int i=0; i<Get_Children_Count(); i++)
 		{
-			Get_Child(i)->Del_Children(Depth - 1);
+			Get_Child(i)->Del_Children(Depth - 1, Name);
+		}
+	}
+	else
+	{
+		for(int i=0; i<Get_Children_Count(); i++)
+		{
+			delete(Get_Child(i));
 		}
 
-		return( true );
+		m_Children.Destroy();
 	}
 
-	return( false );
+	return( true );
 }
 
 //---------------------------------------------------------
 int CSG_MetaData::_Get_Child(const CSG_String &Name) const
 {
-	for(int i=0; i<m_nChildren; i++)
+	for(int i=0; i<Get_Children_Count(); i++)
 	{
-		if( Name.CmpNoCase(m_pChildren[i]->Get_Name()) == 0 )
+		if( Name.CmpNoCase(Get_Child(i)->Get_Name()) == 0 )
 		{
 			return( i );
 		}
@@ -311,21 +426,54 @@ int CSG_MetaData::_Get_Child(const CSG_String &Name) const
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-void CSG_MetaData::Fmt_Content(const SG_Char *Format, ...)
+bool CSG_MetaData::Cmp_Name(const CSG_String &String, bool bNoCase) const
+{
+	return( bNoCase ? !m_Name.CmpNoCase(String) : !m_Name.Cmp(String) );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+void CSG_MetaData::Fmt_Content(const char *Format, ...)
 {
 	wxString	s;
-	va_list		argptr;
 
+	va_list	argptr;
+
+#ifdef _SAGA_LINUX
+	wxString	_Format(Format);	_Format.Replace("%s", "%ls");	// workaround as we only use wide characters since wx 2.9.4 so interpret strings as multibyte
+	va_start(argptr, _Format);
+	s.PrintfV(_Format, argptr);
+#else
 	va_start(argptr, Format);
+	s.PrintfV(Format, argptr);
+#endif
 
-	if( s.PrintfV(Format, argptr) > 0 )
-	{
-		m_Content	= s.wc_str();
-	}
-	else
-	{
-		m_Content.Clear();
-	}
+	m_Content	= CSG_String(&s);
+
+	va_end(argptr);
+}
+
+//---------------------------------------------------------
+void CSG_MetaData::Fmt_Content(const wchar_t *Format, ...)
+{
+	wxString	s;
+
+	va_list	argptr;
+
+#ifdef _SAGA_LINUX
+	wxString	_Format(Format);	_Format.Replace("%s", "%ls");	// workaround as we only use wide characters since wx 2.9.4 so interpret strings as multibyte
+	va_start(argptr, _Format);
+	s.PrintfV(_Format, argptr);
+#else
+	va_start(argptr, Format);
+	s.PrintfV(Format, argptr);
+#endif
+
+	m_Content	= CSG_String(&s);
 
 	va_end(argptr);
 }
@@ -344,10 +492,10 @@ bool CSG_MetaData::Cmp_Content(const CSG_String &String, bool bNoCase) const
 //---------------------------------------------------------
 bool CSG_MetaData::Add_Property(const CSG_String &Name, const CSG_String &Value)
 {
-	if( _Get_Property(Name) < 0 )
+	if( !Value.is_Empty() &&_Get_Property(Name) < 0 )
 	{
-		m_Prop_Names	.Add(Name);
-		m_Prop_Values	.Add(Value);
+		m_Prop_Names .Add(Name );
+		m_Prop_Values.Add(Value);
 
 		return( true );
 	}
@@ -480,6 +628,11 @@ CSG_String CSG_MetaData::asText(int Flags) const
 		XML.Save(Stream);
 
 		s	= &Stream.GetString();
+
+		if( Flags == 2 )	// remove <xml>
+		{
+			s	= s.AfterFirst('\n');
+		}
 	}
 
 	return( s );
@@ -510,33 +663,24 @@ CSG_Table CSG_MetaData::asTable(int Flags) const
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CSG_MetaData::Assign(const CSG_MetaData &MetaData, bool bAppend)
+bool CSG_MetaData::Assign(const CSG_MetaData &MetaData, bool bAddChildren)
 {
-	if( &MetaData == this )
-	{
-		return( true );
-	}
-
-	int		i;
-
-	if( !bAppend )
+	if( &MetaData != this )
 	{
 		Destroy();
 
 		Set_Name	(MetaData.Get_Name   ());
 		Set_Content	(MetaData.Get_Content());
 
-		//-------------------------------------------------
-		for(i=0; i<MetaData.Get_Property_Count(); i++)
+		for(int i=0; i<MetaData.Get_Property_Count(); i++)
 		{
 			Add_Property(MetaData.Get_Property_Name(i), MetaData.Get_Property(i));
 		}
-	}
 
-	//-----------------------------------------------------
-	for(i=0; i<MetaData.Get_Children_Count(); i++)
-	{
-		Add_Child()->Assign(*MetaData.Get_Child(i), false);
+		if( bAddChildren )
+		{
+			Add_Children(MetaData);
+		}
 	}
 
 	return( true );
