@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: metadata.h 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: metadata.h 2266 2014-10-01 15:00:52Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -109,30 +109,50 @@ public:
 	bool						Load_WKT			(const CSG_String &WKT);
 	bool						Save_WKT			(      CSG_String &WKT)		const;
 
-	bool						Assign				(const CSG_MetaData &MetaData, bool bAppend = false);
-	CSG_MetaData &				operator  =			(const CSG_MetaData &MetaData)		{	Assign(MetaData, false);	return( *this );	}
-	CSG_MetaData &				operator +=			(const CSG_MetaData &MetaData)		{	Assign(MetaData, true );	return( *this );	}
+	bool						Assign				(const CSG_MetaData &MetaData, bool bAddChildren = true);
+
+	CSG_MetaData &				operator  =			(const CSG_MetaData &MetaData)		{	Assign   (MetaData, true);	return( *this );	}
+	CSG_MetaData &				operator +=			(const CSG_MetaData &MetaData)		{	Add_Child(MetaData, true);	return( *this );	}
 
 	const CSG_String &			Get_Name			(void)						const	{	return( m_Name );		}
 	void						Set_Name			(const CSG_String &Name)			{	m_Name		= Name;		}
+	bool						Cmp_Name			(const CSG_String &String, bool bNoCase =  true)	const;
 	const CSG_String &			Get_Content			(void)						const	{	return( m_Content );	}
+	const SG_Char *				Get_Content			(int Index)					const	{	return( Get_Child(Index) ? Get_Child(Index)->Get_Content().c_str() : NULL );	}
+	const SG_Char *				Get_Content			(const CSG_String &Name)	const	{	return( Get_Content(_Get_Child(Name)) );	}
 	void						Set_Content			(const CSG_String &Content)			{	m_Content	= Content;	}
-	void						Fmt_Content			(const SG_Char *Format, ...);
+	void						Fmt_Content			(const char    *Format, ...);
+	void						Fmt_Content			(const wchar_t *Format, ...);
 	bool						Cmp_Content			(const CSG_String &String, bool bNoCase = false)	const;
 
-	int							Get_Children_Count	(void)						const	{	return( m_nChildren );	}
-	CSG_MetaData *				Get_Child			(int Index)					const	{	return( Index >= 0 && Index < m_nChildren ? m_pChildren[Index] : NULL );	}
+	CSG_MetaData *				Get_Parent			(void)						const	{	return( m_pParent );	}
+
+	int							Get_Children_Count	(void)						const	{	return( (int)m_Children.Get_Size() );	}
+	CSG_MetaData *				Get_Child			(int Index)					const	{	return( Index >= 0 ? *((CSG_MetaData **)m_Children.Get_Entry((size_t)Index)) : NULL );	}
 	CSG_MetaData *				Get_Child			(const CSG_String &Name)	const	{	return( Get_Child(_Get_Child(Name)) );	}
 	CSG_MetaData *				Add_Child			(void);
-	CSG_MetaData *				Add_Child			(const CSG_String &Name, const SG_Char *Content = NULL);
-	CSG_MetaData *				Add_Child			(const CSG_String &Name, double         Content);
-	CSG_MetaData *				Add_Child			(const CSG_String &Name, int            Content);
+	CSG_MetaData *				Add_Child			(const CSG_String &Name);
+	CSG_MetaData *				Add_Child			(const CSG_String &Name, const CSG_String &Content);
+	CSG_MetaData *				Add_Child			(const CSG_String &Name, double            Content);
+	CSG_MetaData *				Add_Child			(const CSG_String &Name, int               Content);
+	CSG_MetaData *				Add_Child			(const CSG_MetaData &MetaData, bool bAddChildren = true);
+	CSG_MetaData *				Ins_Child			(                                                   int Position);
+	CSG_MetaData *				Ins_Child			(const CSG_String &Name                           , int Position);
+	CSG_MetaData *				Ins_Child			(const CSG_String &Name, const CSG_String &Content, int Position);
+	CSG_MetaData *				Ins_Child			(const CSG_String &Name, double            Content, int Position);
+	CSG_MetaData *				Ins_Child			(const CSG_String &Name, int               Content, int Position);
+	CSG_MetaData *				Ins_Child			(const CSG_MetaData &MetaData                     , int Position, bool bAddChildren = true);
+	bool						Mov_Child			(int from_Index, int to_Index);
 	bool						Del_Child			(int Index);
 	bool						Del_Child			(const CSG_String &Name)			{	return( Del_Child(_Get_Child(Name)) );	}
 
-	bool						Del_Children		(int Depth = 0);
+	bool						Add_Children		(const CSG_MetaData &MetaData);
+	bool						Del_Children		(int Depth = 0, const SG_Char *Name = NULL);
 
-	const CSG_MetaData &		operator []			(int Index)					const	{	CSG_MetaData	*pData	= Index >= 0 && Index < m_nChildren ? m_pChildren[Index] : NULL;	return( *pData );	}
+	CSG_MetaData *				operator ()			(int Index)					const	{	return(  Get_Child(Index           ) );	}
+	CSG_MetaData *				operator ()			(const CSG_String &Name)	const	{	return(  Get_Child(_Get_Child(Name)) );	}
+
+	const CSG_MetaData &		operator []			(int Index)					const	{	return( *Get_Child(Index           ) );	}
 	const CSG_MetaData &		operator []			(const CSG_String &Name)	const	{	return( *Get_Child(_Get_Child(Name)) );	}
 
 	bool						is_Valid			(void)						const	{	return( this != NULL );					}
@@ -161,9 +181,9 @@ private:
 	CSG_MetaData(CSG_MetaData *pParent);
 
 
-	int							m_nChildren, m_nBuffer;
+	CSG_Array					m_Children;
 
-	CSG_MetaData				**m_pChildren, *m_pParent, *m_pDummy;
+	CSG_MetaData				*m_pParent, *m_pDummy;
 
 	CSG_String					m_Name, m_Content;
 
