@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: module.cpp 2327 2014-11-04 14:32:44Z oconrad $
+ * Version $Id: module.cpp 2670 2015-11-02 15:36:08Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -182,6 +182,39 @@ const CSG_String & CSG_Module::Get_Author(void) const
 	return( m_Author );
 }
 
+//---------------------------------------------------------
+CSG_String CSG_Module::Get_MenuPath(bool bSolved)
+{
+	if( !bSolved )
+	{
+		return( Get_MenuPath() );
+	}
+
+	CSG_String	Menu	= Get_MenuPath();
+
+	if( Menu.Length() > 1 && Menu[1] == ':' )
+	{
+		if( Menu[0] == 'A' || Menu[0] == 'a' )	// absolute menu path, overwrites library's default menu path
+		{
+			return( Menu.AfterFirst(':') );
+		}
+
+		Menu	= Menu.AfterFirst(':');	// Menu[0] == 'R' || Menu[0] == 'r'	// menu path explicitly declared as relative to library's default menu path
+	}
+
+	if( m_Library_Menu.is_Empty() )
+	{
+		return( Menu );
+	}
+
+	if( Menu.is_Empty() )
+	{
+		return( m_Library_Menu );
+	}
+
+	return( m_Library_Menu + "|" + Menu );
+}
+
 
 ///////////////////////////////////////////////////////////
 //														 //
@@ -443,6 +476,17 @@ bool CSG_Module::Dlg_Parameters(CSG_Parameters *pParameters, const CSG_String &C
 //														 //
 ///////////////////////////////////////////////////////////
 
+//---------------------------------------------------------
+void CSG_Module::Set_Callback(bool bActive)
+{
+	Parameters.Set_Callback(bActive);
+
+	for(int i=0; i<m_npParameters; i++)
+	{
+		m_pParameters[i]->Set_Callback(bActive);
+	}
+}
+
 //---------------------------------------------------------
 void CSG_Module::Set_Manager(class CSG_Data_Manager *pManager)
 {
@@ -469,21 +513,19 @@ bool CSG_Module::Settings_Push(CSG_Data_Manager *pManager)
 		m_Settings_Stack.Create(sizeof(CSG_Parameters *));
 	}
 
-	int	n	= m_Settings_Stack.Get_Size();
+	size_t	n	= m_Settings_Stack.Get_Size();
 
 	CSG_Parameters	**pP	= (CSG_Parameters **)m_Settings_Stack.Get_Array(n + 1 + m_npParameters);
 
 	if( pP )
 	{
-		pP[n++]	= new CSG_Parameters(Parameters);	Parameters.Restore_Defaults();
+		pP[n++]	= new CSG_Parameters(Parameters); Parameters.Restore_Defaults(true); Parameters.Set_Manager(pManager);
 
 		for(int i=0; i<m_npParameters; i++)
 		{
-			pP[n++]	= new CSG_Parameters(*m_pParameters[i]);	m_pParameters[i]->Restore_Defaults();
+			pP[n++]	= new CSG_Parameters(*m_pParameters[i]); m_pParameters[i]->Restore_Defaults(true); m_pParameters[i]->Set_Manager(pManager);
 		}
 
-		Set_Manager(pManager);
-
 		return( true );
 	}
 
@@ -497,16 +539,14 @@ bool CSG_Module::Settings_Pop(void)
 
 	if( pP && (int)m_Settings_Stack.Get_Size() >= 1 + m_npParameters )
 	{
-		int	n	= m_Settings_Stack.Get_Size() - 1;
+		size_t	n	= m_Settings_Stack.Get_Size() - 1;
 
 		for(int i=m_npParameters-1; i>=0; i--, n--)
 		{
-			m_pParameters[i]->Assign_Values(pP[n]); delete(pP[n]);
+			m_pParameters[i]->Assign_Values(pP[n]); m_pParameters[i]->Set_Manager(pP[n]->Get_Manager()); delete(pP[n]);
 		}
 
-		Set_Manager(pP[n]->Get_Manager());
-
-		Parameters.Assign_Values(pP[n]); delete(pP[n]);
+		Parameters.Assign_Values(pP[n]); Parameters.Set_Manager(pP[n]->Get_Manager()); delete(pP[n]);
 
 		m_Settings_Stack.Set_Array(n);
 
@@ -556,6 +596,8 @@ bool CSG_Module::Set_Progress(double Position, double Range)
 //---------------------------------------------------------
 bool CSG_Module::Stop_Execution(bool bDialog)
 {
+	m_bExecutes	= false;
+
 	return( SG_UI_Stop_Execution(bDialog) );
 }
 
@@ -1145,6 +1187,8 @@ void _Add_XML(CSG_MetaData *pParent, CSG_Parameter *pParameter, CSG_String ID =
 #define SUMMARY_ADD_STR(label, value)	s += CSG_String::Format(SG_T("<tr><td valign=\"top\">%s</td><td valign=\"top\">%s</td></tr>"), label, value)
 #define SUMMARY_ADD_INT(label, value)	s += CSG_String::Format(SG_T("<tr><td valign=\"top\">%s</td><td valign=\"top\">%d</td></tr>"), label, value)
 
+#define SUMMARY_DO_ADD	(SG_UI_Get_Window_Main() ? Parameters(i)->do_UseInGUI() : Parameters(i)->do_UseInCMD())
+
 //---------------------------------------------------------
 CSG_String CSG_Module::Get_Summary(bool bParameters, const CSG_String &Menu, const CSG_String &Description, bool bXML)
 {
@@ -1173,7 +1217,7 @@ CSG_String CSG_Module::Get_Summary(bool bParameters, const CSG_String &Menu, con
 		{
 			for(i=0; i<Parameters.Get_Count(); i++)
 			{
-				if( Parameters(i)->is_Input() )
+				if( SUMMARY_DO_ADD && Parameters(i)->is_Input() )
  				{
 					_Add_XML(&m, Parameters(i));
 				}
@@ -1181,7 +1225,7 @@ CSG_String CSG_Module::Get_Summary(bool bParameters, const CSG_String &Menu, con
 
 			for(i=0; i<Parameters.Get_Count(); i++)
 			{
-				if( Parameters(i)->is_Output() )
+				if( SUMMARY_DO_ADD && Parameters(i)->is_Output() )
  				{
 					_Add_XML(&m, Parameters(i));
 				}
@@ -1189,7 +1233,7 @@ CSG_String CSG_Module::Get_Summary(bool bParameters, const CSG_String &Menu, con
 
 			for(i=0; i<Parameters.Get_Count(); i++)
 			{
-	 			if( Parameters(i)->is_Option()
+				if( SUMMARY_DO_ADD && Parameters(i)->is_Option()
 				&&  Parameters(i)->Get_Type() != PARAMETER_TYPE_Node
 				&&  Parameters(i)->Get_Type() != PARAMETER_TYPE_Grid_System )
  				{
@@ -1259,7 +1303,7 @@ CSG_String CSG_Module::Get_Summary(bool bParameters, const CSG_String &Menu, con
 			{
 				CSG_Parameter	*pParameter	= Parameters(i);
 
-				if( pParameter->is_Input() )
+				if( SUMMARY_DO_ADD && pParameter->is_Input() )
 				{
 					if( bFirst )
 					{
@@ -1283,7 +1327,7 @@ CSG_String CSG_Module::Get_Summary(bool bParameters, const CSG_String &Menu, con
 			{
 				CSG_Parameter	*pParameter	= Parameters(i);
 
-				if( pParameter->is_Output() )
+				if( SUMMARY_DO_ADD && pParameter->is_Output() )
 				{
 					if( bFirst )
 					{
@@ -1307,7 +1351,7 @@ CSG_String CSG_Module::Get_Summary(bool bParameters, const CSG_String &Menu, con
 			{
 				CSG_Parameter	*pParameter	= Parameters(i);
 
-				if( pParameter->is_Option() && pParameter->Get_Type() != PARAMETER_TYPE_Grid_System )
+				if( SUMMARY_DO_ADD && pParameter->is_Option() && pParameter->Get_Type() != PARAMETER_TYPE_Grid_System )
 				{
 					if( bFirst )
 					{
