@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: module.h 2284 2014-10-10 10:01:30Z oconrad $
+ * Version $Id: module.h 2412 2015-02-17 22:18:08Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -205,14 +205,22 @@ public:
 	CSG_String					Get_Summary					(bool bParameters = true, const CSG_String &Menu = "", const CSG_String &Description = "", bool bXML = false);
 
 	virtual CSG_String			Get_MenuPath				(void)	{	return( SG_T("") );	}
+	virtual CSG_String			Get_MenuPath				(bool bSolved);
 
 	int							Get_Parameters_Count		(void)	{	return( m_npParameters );	}
 	CSG_Parameters *			Get_Parameters				(void)	{	return( &Parameters );	}
 	CSG_Parameters *			Get_Parameters				(int i)	{	return( i >= 0 && i < m_npParameters ? m_pParameters[i] : NULL );	}
 	CSG_Parameters *			Get_Parameters				(const CSG_String &Identifier);
 
+	bool						Set_Parameter				(const CSG_String &Identifier, CSG_Parameter *pSource);
+	bool						Set_Parameter				(const CSG_String &Identifier, int            Value, int Type = PARAMETER_TYPE_Undefined);
+	bool						Set_Parameter				(const CSG_String &Identifier, double         Value, int Type = PARAMETER_TYPE_Undefined);
+	bool						Set_Parameter				(const CSG_String &Identifier, void          *Value, int Type = PARAMETER_TYPE_Undefined);
+	bool						Set_Parameter				(const CSG_String &Identifier, const SG_Char *Value, int Type = PARAMETER_TYPE_Undefined);
+
 	bool						Update_Parameter_States		(void);
 
+	void						Set_Callback				(bool bActive = true);
 	void						Set_Manager					(class CSG_Data_Manager *pManager);
 
 	bool						Settings_Push				(class CSG_Data_Manager *pManager = NULL);
@@ -317,7 +325,7 @@ private:
 
 	CSG_Parameters				**m_pParameters;
 
-	CSG_String					m_ID, m_Library, m_File_Name, m_Author;
+	CSG_String					m_ID, m_Library, m_Library_Menu, m_File_Name, m_Author;
 
 
 	bool						_Synchronize_DataObjects	(void);
@@ -639,16 +647,17 @@ extern "C" _SAGA_DLL_EXPORT const SG_Char *					Get_Version			(void)\
 //---------------------------------------------------------
 #define MLB_INTERFACE_INITIALIZE	extern "C" _SAGA_DLL_EXPORT bool MLB_Initialize	(const SG_Char *File_Name)\
 {\
+	int		i;\
+\
 	MLB_Interface.Set_File_Name(File_Name);\
-\
-	int		i	= 0;\
-\
-	while( MLB_Interface.Add_Module(Create_Module(i), i) ) i++;\
 \
 	for(i=0; i<MLB_INFO_User; i++)\
 	{\
 		MLB_Interface.Set_Info(i, Get_Info(i));\
 	}\
+\
+	for(i=0; MLB_Interface.Add_Module(Create_Module(i), i); i++)\
+	{}\
 \
 	return( MLB_Interface.Get_Count() > 0 );\
 }\
