@@ -73,7 +73,10 @@
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-#define GET_XML_CONTENT(md, id, def)	(md(id) ? md(id)->Get_Content() : CSG_String(def))
+#define GET_XML_CONTENT(md, id, def)		(md(id) ? md(id)->Get_Content() : CSG_String(def))
+
+#define IS_TRUE_STRING(String)				(!String.CmpNoCase("true") || !String.CmpNoCase("1"))
+#define IS_TRUE_PROPERTY(Item, Property)	(Item.Cmp_Property(Property, "true", true) || Item.Cmp_Property(Property, "1"))
 
 
 ///////////////////////////////////////////////////////////
@@ -105,9 +108,17 @@ void CSG_Module_Chain::Reset(void)
 
 	m_Chain.Destroy();
 
+	m_Conditions.Destroy();
+
 	m_Menu.Clear();
 }
 
+//---------------------------------------------------------
+void CSG_Module_Chain::Set_Library_Menu(const CSG_String &Menu)
+{
+	m_Library_Menu	= Menu;
+}
+
 
 ///////////////////////////////////////////////////////////
 //														 //
@@ -119,15 +130,35 @@ bool CSG_Module_Chain::Create(const CSG_String &File)
 	Reset();
 
 	//-----------------------------------------------------
-	if( !m_Chain.Load(File) || !m_Chain.Cmp_Name("toolchain")
-	||  SG_Compare_Version(m_Chain.Get_Property("saga-version"), "2.1.3") < 0
-	||  !m_Chain("identifier") || !m_Chain("parameters") )
+	if( File.Right(sizeof(".pyt.xml")).Make_Lower().Find(".pyt.xml") >= 0 )
 	{
-		Reset();
-
 		return( false );
 	}
 
+	if( !m_Chain.Load(File) )
+	{
+		SG_UI_Msg_Add_Error(CSG_String::Format("%s: %s", _TL("failed to load or parse xml file"), File.c_str()));
+
+		Reset();	return( false );
+	}
+
+	if( m_Chain.Cmp_Name("toolchains") )	// don't report any error, this xml-file provides info for a category of tool chains
+	{
+		Reset();	return( false );
+	}
+
+	if( !m_Chain.Cmp_Name("toolchain") || !m_Chain("identifier") || !m_Chain("parameters") )
+	{
+		SG_UI_Msg_Add_Error(CSG_String::Format("%s: %s", _TL("missing tool chain tags"), File.c_str()));
+
+		Reset();	return( false );
+	}
+
+	if( SG_Compare_Version(m_Chain.Get_Property("saga-version"), "2.1.3") < 0 )
+	{
+		SG_UI_Msg_Add_Error(CSG_String::Format("%s %s: %s", _TL("WARNING"), _TL("unsupported tool chain version"), File.c_str()));
+	}
+
 	//-----------------------------------------------------
 	m_File_Name		= File;
 
@@ -140,7 +171,7 @@ bool CSG_Module_Chain::Create(const CSG_String &File)
 
 	if( !m_Menu.is_Empty() && (m_Menu.Length() < 2 || m_Menu[1] != ':') )
 	{
-		if( m_Chain["menu"].Cmp_Property("absolute", "true", true) || m_Chain["menu"].Cmp_Property("absolute", "1") )
+		if( IS_TRUE_PROPERTY(m_Chain["menu"], "absolute") )
 			m_Menu.Prepend("A:");	// absolute path
 		else
 			m_Menu.Prepend("R:");	// relative path
@@ -151,27 +182,50 @@ bool CSG_Module_Chain::Create(const CSG_String &File)
 	{
 		const CSG_MetaData	&Parameter	= m_Chain["parameters"][i];
 
-		int			Constraint	= 0;
-		CSG_String	Value, ID	= Parameter.Get_Property("varname");
-
-		if( Parameter.Cmp_Name("input") )
-		{
-			Constraint	= PARAMETER_INPUT;
-		}
-		else if( Parameter.Cmp_Name("output") )
-		{
-			Constraint	= PARAMETER_OUTPUT;
-		}
-		else if( Parameter.Cmp_Name("option") )
-		{
-			Value		= Parameter.Get_Content("value");
-		}
+		CSG_String	ID	= Parameter.Get_Property("varname");
 
 		if( ID.is_Empty() || Parameters(ID) )
 		{
 			continue;
 		}
 
+		//-------------------------------------------------
+		int			Constraint	= 0;
+		CSG_String	Value;
+		bool		bMin = false, bMax = false;
+		double		 Min = 0.0  ,  Max = 0.0;
+
+		if( Parameter.Cmp_Name("input") )
+		{
+			Constraint	= IS_TRUE_PROPERTY(Parameter, "optional") ? PARAMETER_INPUT_OPTIONAL : PARAMETER_INPUT;
+		}
+		else if( Parameter.Cmp_Name("output") )
+		{
+			Constraint	= IS_TRUE_PROPERTY(Parameter, "optional") ? PARAMETER_OUTPUT_OPTIONAL : PARAMETER_OUTPUT;
+		}
+		else if( Parameter.Cmp_Name("option") && Parameter("value") )
+		{
+			Value	= Parameter.Get_Content("value");
+
+			bMin	= Parameter["value"].Get_Property("min", Min);
+			bMax	= Parameter["value"].Get_Property("max", Max);
+		}
+
+		//-------------------------------------------------
+		if( Parameter("condition") )
+		{
+			CSG_MetaData	&Conditions	= *m_Conditions.Add_Child(ID);
+
+			for(int j=0; j<Parameter.Get_Children_Count(); j++)	// there might be more than one condition to be checked
+			{
+				if( Parameter[j].Cmp_Name("condition") )
+				{
+					Conditions.Add_Child(Parameter[j]);
+				}
+			}
+		}
+
+		//-------------------------------------------------
 		CSG_String	Name	= Parameter.Get_Content("name"       );
 		CSG_String	Desc	= Parameter.Get_Content("description");
 
@@ -180,37 +234,50 @@ bool CSG_Module_Chain::Create(const CSG_String &File)
 		//-------------------------------------------------
 		switch( SG_Parameter_Type_Get_Type(Parameter.Get_Property("type")) )
 		{
-		case PARAMETER_TYPE_PointCloud     : Parameters.Add_PointCloud     (pParent, ID, Name, Desc, Constraint);	break;
-		case PARAMETER_TYPE_Grid           : Parameter.Cmp_Property("target", "none") 
-			                               ? Parameters.Add_Grid_Output    (   NULL, ID, Name, Desc)
-			                               : Parameters.Add_Grid           (pParent, ID, Name, Desc, Constraint);	break;
-		case PARAMETER_TYPE_Table          : Parameters.Add_Table          (pParent, ID, Name, Desc, Constraint);	break;
-		case PARAMETER_TYPE_Shapes         : Parameters.Add_Shapes         (pParent, ID, Name, Desc, Constraint);	break;
-		case PARAMETER_TYPE_TIN            : Parameters.Add_TIN            (pParent, ID, Name, Desc, Constraint);	break;
+		case PARAMETER_TYPE_Node             : Parameters.Add_Node           (pParent, ID, Name, Desc);	break;
 
-		case PARAMETER_TYPE_PointCloud_List: Parameters.Add_PointCloud_List(pParent, ID, Name, Desc, Constraint);	break;
-		case PARAMETER_TYPE_Grid_List      : Parameters.Add_Grid_List      (pParent, ID, Name, Desc, Constraint);	break;
-		case PARAMETER_TYPE_Table_List     : Parameters.Add_Table_List     (pParent, ID, Name, Desc, Constraint);	break;
-		case PARAMETER_TYPE_Shapes_List    : Parameters.Add_Shapes_List    (pParent, ID, Name, Desc, Constraint);	break;
-		case PARAMETER_TYPE_TIN_List       : Parameters.Add_TIN_List       (pParent, ID, Name, Desc, Constraint);	break;
+		case PARAMETER_TYPE_Bool             : Parameters.Add_Value          (pParent, ID, Name, Desc, PARAMETER_TYPE_Bool  , IS_TRUE_STRING(Value));	break;
+		case PARAMETER_TYPE_Int              : Parameters.Add_Value          (pParent, ID, Name, Desc, PARAMETER_TYPE_Int   , Value.asInt   (), Min, bMin, Max, bMax);	break;
+		case PARAMETER_TYPE_Double           : Parameters.Add_Value          (pParent, ID, Name, Desc, PARAMETER_TYPE_Double, Value.asDouble(), Min, bMin, Max, bMax);	break;
+		case PARAMETER_TYPE_Degree           : Parameters.Add_Value          (pParent, ID, Name, Desc, PARAMETER_TYPE_Degree, Value.asDouble(), Min, bMin, Max, bMax);	break;
 
-		case PARAMETER_TYPE_Bool           : Parameters.Add_Value          (pParent, ID, Name, Desc, PARAMETER_TYPE_Bool  ,!Value.CmpNoCase("TRUE"));	break;
-		case PARAMETER_TYPE_Int            : Parameters.Add_Value          (pParent, ID, Name, Desc, PARAMETER_TYPE_Int   , Value.asInt   ());	break;
-		case PARAMETER_TYPE_Double         : Parameters.Add_Value          (pParent, ID, Name, Desc, PARAMETER_TYPE_Double, Value.asDouble());	break;
-		case PARAMETER_TYPE_Degree         : Parameters.Add_Value          (pParent, ID, Name, Desc, PARAMETER_TYPE_Degree, Value.asDouble());	break;
+		case PARAMETER_TYPE_Range            : Parameters.Add_Range          (pParent, ID, Name, Desc, Value.BeforeFirst(';').asDouble(), Value.AfterFirst (';').asDouble(), Min, bMin, Max, bMax);	break;
+		case PARAMETER_TYPE_Choice           : Parameters.Add_Choice         (pParent, ID, Name, Desc, Parameter.Get_Content("choices"))->Set_Value(Value);	break;
 
-		case PARAMETER_TYPE_String         : Parameters.Add_String         (pParent, ID, Name, Desc, Value, false);	break;
-		case PARAMETER_TYPE_Text           : Parameters.Add_String         (pParent, ID, Name, Desc, Value,  true);	break;
+		case PARAMETER_TYPE_String           : Parameters.Add_String         (pParent, ID, Name, Desc, Value, false);	break;
+		case PARAMETER_TYPE_Text             : Parameters.Add_String         (pParent, ID, Name, Desc, Value,  true);	break;
 
-		case PARAMETER_TYPE_FilePath       : Parameters.Add_FilePath       (pParent, ID, Name, Desc, Value);	break;
+		case PARAMETER_TYPE_FilePath         : Parameters.Add_FilePath       (pParent, ID, Name, Desc, Parameter.Get_Content("filter"), Value,
+												IS_TRUE_PROPERTY(Parameter, "save"     ),
+												IS_TRUE_PROPERTY(Parameter, "directory"),
+												IS_TRUE_PROPERTY(Parameter, "multiple" ));	break;
 
-		case PARAMETER_TYPE_Choice         : Parameters.Add_Choice         (pParent, ID, Name, Desc, Parameter.Get_Content("choices"))->Set_Value(Value);	break;
-		case PARAMETER_TYPE_Range          : Parameters.Add_Range          (pParent, ID, Name, Desc, Value.BeforeFirst(';').asDouble(), Value.AfterFirst (';').asDouble());	break;
+		case PARAMETER_TYPE_Font             : break;
+		case PARAMETER_TYPE_Color            : Parameters.Add_Value          (pParent, ID, Name, Desc, PARAMETER_TYPE_Color, Value.asInt());	break;
+		case PARAMETER_TYPE_Colors           : Parameters.Add_Colors         (pParent, ID, Name, Desc);	break;
+		case PARAMETER_TYPE_FixedTable       : break;	// to do ?
 
-		case PARAMETER_TYPE_Table_Field    : Parameters.Add_Table_Field    (pParent, ID, Name, Desc, !Value.CmpNoCase("TRUE"));	break;
-		case PARAMETER_TYPE_Table_Fields   : Parameters.Add_Table_Fields   (pParent, ID, Name, Desc);	break;
+		case PARAMETER_TYPE_Grid_System      : Parameters.Add_Grid_System    (pParent, ID, Name, Desc);	break;
 
-		case PARAMETER_TYPE_Grid_System    : Parameters.Add_Grid_System    (pParent, ID, Name, Desc);	break;
+		case PARAMETER_TYPE_Table_Field      : Parameters.Add_Table_Field    (pParent, ID, Name, Desc, (!Value.CmpNoCase("true") || !Value.CmpNoCase("1")));	break;
+		case PARAMETER_TYPE_Table_Fields     : Parameters.Add_Table_Fields   (pParent, ID, Name, Desc);	break;
+
+		case PARAMETER_TYPE_Grid             : Parameter.Cmp_Property("target", "none") 
+			                                 ? Parameters.Add_Grid_Output    (   NULL, ID, Name, Desc)
+			                                 : Parameters.Add_Grid           (pParent, ID, Name, Desc, Constraint);	break;
+		case PARAMETER_TYPE_Table            : Parameters.Add_Table          (pParent, ID, Name, Desc, Constraint);	break;
+		case PARAMETER_TYPE_Shapes           : Parameters.Add_Shapes         (pParent, ID, Name, Desc, Constraint);	break;
+		case PARAMETER_TYPE_TIN              : Parameters.Add_TIN            (pParent, ID, Name, Desc, Constraint);	break;
+		case PARAMETER_TYPE_PointCloud       : Parameters.Add_PointCloud     (pParent, ID, Name, Desc, Constraint);	break;
+
+		case PARAMETER_TYPE_Grid_List        : Parameters.Add_Grid_List      (pParent, ID, Name, Desc, Constraint, !IS_TRUE_PROPERTY(Parameter, "no_system"));	break;
+		case PARAMETER_TYPE_Table_List       : Parameters.Add_Table_List     (pParent, ID, Name, Desc, Constraint);	break;
+		case PARAMETER_TYPE_Shapes_List      : Parameters.Add_Shapes_List    (pParent, ID, Name, Desc, Constraint);	break;
+		case PARAMETER_TYPE_TIN_List         : Parameters.Add_TIN_List       (pParent, ID, Name, Desc, Constraint);	break;
+		case PARAMETER_TYPE_PointCloud_List  : Parameters.Add_PointCloud_List(pParent, ID, Name, Desc, Constraint);	break;
+
+		case PARAMETER_TYPE_DataObject_Output: break;
+		case PARAMETER_TYPE_Parameters       : break;	// to do ?
 
 		default: break;
 		}
@@ -221,6 +288,34 @@ bool CSG_Module_Chain::Create(const CSG_String &File)
 }
 
 
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+int CSG_Module_Chain::On_Parameters_Enable(CSG_Parameters *pParameters, CSG_Parameter *pParameter)
+{
+	for(int iParameter=0; iParameter<m_Conditions.Get_Children_Count(); iParameter++)
+	{
+		const CSG_MetaData	&Conditions	= m_Conditions[iParameter];
+
+		if( pParameters->Get_Parameter(Conditions.Get_Name()) )
+		{
+			bool	bEnable	= true;
+
+			for(int iCondition=0; bEnable && iCondition<Conditions.Get_Children_Count(); iCondition++)
+			{
+				bEnable	= Check_Condition(Conditions[iCondition], pParameters);
+			}
+
+			pParameters->Get_Parameter(Conditions.Get_Name())->Set_Enabled(bEnable);
+		}
+	}
+
+	return( CSG_Module::On_Parameters_Enable(pParameters, pParameter) );
+}
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 ///////////////////////////////////////////////////////////
@@ -228,14 +323,16 @@ bool CSG_Module_Chain::Create(const CSG_String &File)
 //---------------------------------------------------------
 bool CSG_Module_Chain::On_Execute(void)
 {
-	bool	bResult	= true;
+	bool	bResult	= Data_Initialize();
 
-	if( Data_Initialize() )
+	if( !bResult )
 	{
-		for(int i=0; i<m_Chain["tools"].Get_Children_Count() && bResult; i++)
-		{
-			bResult	= Tool_Run(m_Chain["tools"][i]);
-		}
+		Error_Set(_TL("no data objects"));
+	}
+
+	for(int i=0; bResult && i<m_Chain["tools"].Get_Children_Count(); i++)
+	{
+		bResult	= Tool_Run(m_Chain["tools"][i]);
 	}
 
 	Data_Finalize();
@@ -256,22 +353,37 @@ bool CSG_Module_Chain::Data_Add(const CSG_String &ID, CSG_Parameter *pData)
 		return( false );
 	}
 
-	switch( pData->Get_Type() )
+	CSG_Parameter	*pParameter	= m_Data(ID);
+
+	if( pParameter )	// don't add twice with same identifier
 	{
-	case PARAMETER_TYPE_PointCloud     : m_Data.Add_PointCloud     (NULL, ID, "", "", 0)->Assign(pData);	break;
-	case PARAMETER_TYPE_Grid           : m_Data.Add_Grid           (NULL, ID, "", "", 0)->Assign(pData);	break;
-	case PARAMETER_TYPE_Table          : m_Data.Add_Table          (NULL, ID, "", "", 0)->Assign(pData);	break;
-	case PARAMETER_TYPE_Shapes         : m_Data.Add_Shapes         (NULL, ID, "", "", 0)->Assign(pData);	break;
-	case PARAMETER_TYPE_TIN            : m_Data.Add_TIN            (NULL, ID, "", "", 0)->Assign(pData);	break;
-
-	case PARAMETER_TYPE_PointCloud_List: m_Data.Add_PointCloud_List(NULL, ID, "", "", 0)->Assign(pData);	break;
-	case PARAMETER_TYPE_Grid_List      : m_Data.Add_Grid_List      (NULL, ID, "", "", 0)->Assign(pData);	break;
-	case PARAMETER_TYPE_Table_List     : m_Data.Add_Table_List     (NULL, ID, "", "", 0)->Assign(pData);	break;
-	case PARAMETER_TYPE_Shapes_List    : m_Data.Add_Shapes_List    (NULL, ID, "", "", 0)->Assign(pData);	break;
-	case PARAMETER_TYPE_TIN_List       : m_Data.Add_TIN_List       (NULL, ID, "", "", 0)->Assign(pData);	break;
-
-	default: return( false );
+		if( pParameter->Get_Type() != pData->Get_Type() )
+		{
+			return( false );
+		}
 	}
+	else switch( pData->Get_Type() )
+	{
+	case PARAMETER_TYPE_PointCloud     : pParameter	= m_Data.Add_PointCloud     (NULL, ID, "", "", 0       );	break;
+	case PARAMETER_TYPE_Grid           : pParameter	= m_Data.Add_Grid           (NULL, ID, "", "", 0       );	break;
+	case PARAMETER_TYPE_Table          : pParameter	= m_Data.Add_Table          (NULL, ID, "", "", 0       );	break;
+	case PARAMETER_TYPE_Shapes         : pParameter	= m_Data.Add_Shapes         (NULL, ID, "", "", 0       );	break;
+	case PARAMETER_TYPE_TIN            : pParameter	= m_Data.Add_TIN            (NULL, ID, "", "", 0       );	break;
+
+	case PARAMETER_TYPE_PointCloud_List: pParameter	= m_Data.Add_PointCloud_List(NULL, ID, "", "", 0       );	break;
+	case PARAMETER_TYPE_Grid_List      : pParameter	= m_Data.Add_Grid_List      (NULL, ID, "", "", 0, false);	break;
+	case PARAMETER_TYPE_Table_List     : pParameter	= m_Data.Add_Table_List     (NULL, ID, "", "", 0       );	break;
+	case PARAMETER_TYPE_Shapes_List    : pParameter	= m_Data.Add_Shapes_List    (NULL, ID, "", "", 0       );	break;
+	case PARAMETER_TYPE_TIN_List       : pParameter	= m_Data.Add_TIN_List       (NULL, ID, "", "", 0       );	break;
+
+	case PARAMETER_TYPE_DataObject_Output:
+		return( true );
+
+	default:
+		return( false );
+	}
+
+	pParameter->Assign(pData);
 
 	if( pData->is_DataObject() )
 	{
@@ -320,12 +432,19 @@ bool CSG_Module_Chain::Data_Initialize(void)
 {
 	m_Data.Set_Manager(NULL);
 
+	bool	bResult	= false;
+
 	for(int i=0; i<Parameters.Get_Count(); i++)
 	{
-		Data_Add(Parameters(i)->Get_Identifier(), Parameters(i));
+		CSG_Parameter	*pParameter	= Parameters(i);
+
+		if( !(pParameter->is_DataObject() && !pParameter->asDataObject()) && Data_Add(pParameter->Get_Identifier(), pParameter) )
+		{
+			bResult	= true;
+		}
 	}
 
-	return( m_Data.Get_Count() > 0 );
+	return( bResult );
 }
 
 //---------------------------------------------------------
@@ -347,6 +466,16 @@ bool CSG_Module_Chain::Data_Finalize(void)
 		}
 		else if( Parameters(i)->is_DataObject_List() )
 		{
+			if( Parameters(i)->is_Output() && m_Data(Parameters(i)->Get_Identifier()) )	// output lists cannot be up-to-date yet
+			{
+				CSG_Parameter	*pData	= m_Data(Parameters(i)->Get_Identifier());
+
+				for(int j=0; j<pData->asList()->Get_Count(); j++)	// csg_parameter::assign() will not work, if parameters data manager is the standard data manager because it checks for existing data sets
+				{
+					Parameters(i)->asList()->Add_Item(pData->asList()->asDataObject(j));
+				}
+			}
+
 			for(int j=0; j<Parameters(i)->asList()->Get_Count(); j++)
 			{
 				m_Data_Manager.Delete(Parameters(i)->asList()->asDataObject(j), true);
@@ -377,8 +506,7 @@ bool CSG_Module_Chain::Data_Finalize(void)
 				if( Parameter("colours") )
 				{
 					DataObject_Set_Colors(pParameter->asDataObject(), 11,
-						Parameter["colours"].Get_Content().asInt(),
-						Parameter["colours"].Cmp_Property("revert", "true", true) || Parameter["colours"].Cmp_Property("revert", "1")
+						Parameter["colours"].Get_Content().asInt(), IS_TRUE_PROPERTY(Parameter["colours"], "revert")
 					);
 				}
 			}
@@ -390,6 +518,129 @@ bool CSG_Module_Chain::Data_Finalize(void)
 }
 
 
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CSG_Module_Chain::Check_Condition(const CSG_MetaData &Condition, CSG_Parameters *pData)
+{
+	//-----------------------------------------------------
+	CSG_String	Type, Value;
+
+	if( !Condition.Cmp_Name("condition") || !Condition.Get_Property("type", Type) )
+	{
+		return( true );
+	}
+
+	if( !Condition.Get_Property("variable", Value) )
+	{
+		Value	= Condition.Get_Content();
+	}
+
+	//-----------------------------------------------------
+	if( !Type.CmpNoCase("has_gui"   ) )	// executed from saga_gui ? (tool might offer different parameters if called from saga_cmd, python etc.)
+	{
+		return( IS_TRUE_STRING(Value) ? SG_UI_Get_Window_Main() != NULL : SG_UI_Get_Window_Main() == NULL );
+	}
+
+	//-----------------------------------------------------
+	if( !Type.CmpNoCase("exists"    ) )	// data object exists
+	{
+		CSG_Parameter	*pParameter	= pData->Get_Parameter(Value);
+
+		return( pParameter && ((pParameter->is_DataObject() && pParameter->asDataObject()) || (pParameter->is_DataObject_List() && pParameter->asList()->Get_Count())) );
+	}
+
+	if( !Type.CmpNoCase("not_exists") )	// data object does not exist
+	{
+		return( pData->Get_Parameter(Value) == NULL || pData->Get_Parameter(Value)->asDataObject() == NULL );
+	}
+
+	//-----------------------------------------------------
+	CSG_Parameter	*pOption	= Parameters(Value);
+
+	if( pOption == NULL )
+	{
+		return( true );
+	}
+
+	switch( pOption->Get_Type() )
+	{
+	//-----------------------------------------------------
+	case PARAMETER_TYPE_Bool    :
+		{
+			CSG_String	Value;
+
+			if( Condition.Get_Property("value", Value) )
+			{
+				if(      !Type.CmpNoCase("=") || !Type.CmpNoCase("equal"    ) )	{	if( IS_TRUE_STRING(Value) == pOption->asBool() )	{	return( false );	}	}
+				else if( !Type.CmpNoCase("!") || !Type.CmpNoCase("not_equal") )	{	if( IS_TRUE_STRING(Value) != pOption->asBool() )	{	return( false );	}	}
+			}
+		}
+		break;
+
+	//-----------------------------------------------------
+	case PARAMETER_TYPE_Int     :
+	case PARAMETER_TYPE_Color   :
+	case PARAMETER_TYPE_Choice  :
+		{
+			int		Value;
+
+			if( Condition.Get_Property("value", Value) )
+			{
+				if(      !Type.CmpNoCase("=") || !Type.CmpNoCase("equal"    ) )	{	if( Value != pOption->asInt() )	{	return( false );	}	}
+				else if( !Type.CmpNoCase("!") || !Type.CmpNoCase("not_equal") )	{	if( Value == pOption->asInt() )	{	return( false );	}	}
+				else if( !Type.CmpNoCase("<") || !Type.CmpNoCase("less"     ) )	{	if( Value >= pOption->asInt() )	{	return( false );	}	}
+				else if( !Type.CmpNoCase(">") || !Type.CmpNoCase("greater"  ) )	{	if( Value <= pOption->asInt() )	{	return( false );	}	}
+			}
+		}
+		break;
+
+	//-----------------------------------------------------
+	case PARAMETER_TYPE_Double  :
+	case PARAMETER_TYPE_Degree  :
+		{
+			double	Value;
+
+			if( Condition.Get_Property("value", Value) )
+			{
+				if(      !Type.CmpNoCase("=") || !Type.CmpNoCase("equal"    ) )	{	if( Value != pOption->asDouble() )	{	return( false );	}	}
+				else if( !Type.CmpNoCase("!") || !Type.CmpNoCase("not_equal") )	{	if( Value == pOption->asDouble() )	{	return( false );	}	}
+				else if( !Type.CmpNoCase("<") || !Type.CmpNoCase("less"     ) )	{	if( Value >= pOption->asDouble() )	{	return( false );	}	}
+				else if( !Type.CmpNoCase(">") || !Type.CmpNoCase("greater"  ) )	{	if( Value <= pOption->asDouble() )	{	return( false );	}	}
+			}
+		}
+		break;
+
+	//-----------------------------------------------------
+	case PARAMETER_TYPE_String  :
+	case PARAMETER_TYPE_Text    :
+	case PARAMETER_TYPE_FilePath:
+		{
+			CSG_String	Value;
+
+			Condition.Get_Property("value", Value);	// no 'if', bcos empty string would return false !!
+
+			{
+				if(      !Type.CmpNoCase("=") || !Type.CmpNoCase("equal"    ) )	{	if( Value.Cmp(pOption->asString()) != 0 )	{	return( false );	}	}
+				else if( !Type.CmpNoCase("!") || !Type.CmpNoCase("not_equal") )	{	if( Value.Cmp(pOption->asString()) == 0 )	{	return( false );	}	}
+				else if( !Type.CmpNoCase("<") || !Type.CmpNoCase("less"     ) )	{	if( Value.Cmp(pOption->asString()) >= 0 )	{	return( false );	}	}
+				else if( !Type.CmpNoCase(">") || !Type.CmpNoCase("greater"  ) )	{	if( Value.Cmp(pOption->asString()) <= 0 )	{	return( false );	}	}
+			}
+		}
+		break;
+
+	//-----------------------------------------------------
+	default:
+		// nop
+		break;
+	}
+
+	return( true );
+}
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 ///////////////////////////////////////////////////////////
@@ -397,6 +648,24 @@ bool CSG_Module_Chain::Data_Finalize(void)
 //---------------------------------------------------------
 bool CSG_Module_Chain::Tool_Run(const CSG_MetaData &Tool)
 {
+	//-----------------------------------------------------
+	if( Tool.Cmp_Name("condition") )
+	{
+		if( !Check_Condition(Tool, &m_Data) )
+		{
+			return( true );
+		}
+
+		bool	bResult	= true;
+
+		for(int i=0; bResult && i<Tool.Get_Children_Count(); i++)
+		{
+			bResult	= Tool_Run(Tool[i]);
+		}
+
+		return( bResult );
+	}
+
 	//-----------------------------------------------------
 	if( !Tool.Cmp_Name("tool") || !Tool.Get_Property("library") || !Tool.Get_Property("module") )
 	{
@@ -410,8 +679,7 @@ bool CSG_Module_Chain::Tool_Run(const CSG_MetaData &Tool)
 
 	CSG_Module	*pModule;
 
-	if(	!(pModule = SG_Get_Module_Library_Manager().Get_Module(Tool.Get_Property("library"), Module        ))
-	&&  !(pModule = SG_Get_Module_Library_Manager().Get_Module(Tool.Get_Property("library"), Module.asInt())) )
+	if(	!(pModule = SG_Get_Module_Library_Manager().Get_Module(Tool.Get_Property("library"), Module)) )
 	{
 		Error_Fmt("%s [%s].[%s]", _TL("could not find tool"),  Tool.Get_Property("library"), Module.c_str());
 
@@ -426,13 +694,17 @@ bool CSG_Module_Chain::Tool_Run(const CSG_MetaData &Tool)
 
 	bool	bResult	= false;
 
-	if( !Tool_Initialize(Tool, pModule) )
+	if( !pModule->On_Before_Execution() )
 	{
-		Error_Fmt("%s [%s].[%s]", _TL("tool initialization failed"), pModule->Get_Library().c_str(), pModule->Get_Name().c_str());
+		Error_Fmt("%s [%s].[%s]", _TL("before tool execution check failed"), pModule->Get_Library().c_str(), pModule->Get_Name().c_str());
+	}
+	else if( !Tool_Initialize(Tool, pModule) )
+	{
+		Error_Fmt("%s [%s].[%s]", _TL("tool initialization failed"        ), pModule->Get_Library().c_str(), pModule->Get_Name().c_str());
 	}
 	else if( !(bResult = pModule->Execute()) )
 	{
-		Error_Fmt("%s [%s].[%s]", _TL("tool execution failed"     ), pModule->Get_Library().c_str(), pModule->Get_Name().c_str());
+		Error_Fmt("%s [%s].[%s]", _TL("tool execution failed"             ), pModule->Get_Library().c_str(), pModule->Get_Name().c_str());
 	}
 
 	Tool_Finalize(Tool, pModule);
@@ -447,34 +719,76 @@ bool CSG_Module_Chain::Tool_Run(const CSG_MetaData &Tool)
 //														 //
 ///////////////////////////////////////////////////////////
 
+//---------------------------------------------------------
+bool CSG_Module_Chain::Tool_Check_Condition(const CSG_MetaData &Tool)
+{
+	if( Tool("condition") )
+	{
+		for(int i=0; i<Tool.Get_Children_Count(); i++)	// there might be more than one condition to be checked
+		{
+			if( !Check_Condition(Tool[i], &m_Data) )
+			{
+				return( false );
+			}
+		}
+	}
+
+	return( true );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CSG_Module_Chain::Tool_Get_Parameter(const CSG_MetaData &Parameter, CSG_Module *pModule, CSG_Parameter **ppParameter, CSG_Parameter **ppParameters)
+{
+	CSG_String	ID	= Parameter.Get_Property("id");
+
+	*ppParameters	= NULL;
+
+	CSG_Parameter	*pParameters	= pModule->Get_Parameters()->Get_Parameter(Parameter.Get_Property("parms"));
+
+	if( pParameters && pParameters->Get_Type() == PARAMETER_TYPE_Parameters && pParameters->asParameters()->Get_Parameter(ID) )
+	{
+		*ppParameters	= pParameters;
+		*ppParameter	= pParameters->asParameters()->Get_Parameter(ID);
+	}
+	else if( pModule->Get_Parameters(Parameter.Get_Property("parms")) )
+	{
+		*ppParameter	= pModule->Get_Parameters(Parameter.Get_Property("parms"))->Get_Parameter(ID);
+	}
+	else
+	{
+		*ppParameter	= pModule->Get_Parameters()->Get_Parameter(ID);
+	}
+
+	return( *ppParameter != NULL );
+}
+
 //---------------------------------------------------------
 bool CSG_Module_Chain::Tool_Initialize(const CSG_MetaData &Tool, CSG_Module *pModule)
 {
 	//-----------------------------------------------------
-	pModule->Get_Parameters()->Set_Callback();	// ???!!!
-
 	int		i;
 
-	CSG_String	Tool_ID	= Tool.Get_Property("id");
-
 	//-----------------------------------------------------
-	for(i=0; i<Tool.Get_Children_Count(); i++)	// set data objects first
+	for(i=0; i<Tool.Get_Children_Count(); i++)	// set data variables first
 	{
 		const CSG_MetaData	&Parameter	= Tool[i];
 
-		CSG_String	ID	= Parameter.Get_Property("id");
+		CSG_Parameter	*pParameter, *pOwner;
 
-		CSG_Parameter	*pParameter	= pModule->Get_Parameters(Parameter.Get_Property("parms"))
-			? pModule->Get_Parameters(Parameter.Get_Property("parms"))->Get_Parameter(ID)
-			: pModule->Get_Parameters()->Get_Parameter(ID);
-
-		if( !pParameter )
+		if( !Tool_Get_Parameter(Parameter, pModule, &pParameter, &pOwner) )
 		{
+			Error_Set(CSG_String::Format("%s: %s", _TL("parameter not found"), Parameter.Get_Property("id")));
+
 			return( false );
 		}
 		else if( Parameter.Cmp_Name("option") )
 		{
-			if( Parameter.Cmp_Property("varname", "1") || Parameter.Cmp_Property("varname", "true", true) )
+			if( IS_TRUE_PROPERTY(Parameter, "varname") )
 			{	// does option want a value from tool chain parameters and do these provide one ?
 				pParameter->Assign(Parameters(Parameter.Get_Content()));
 			}
@@ -489,6 +803,8 @@ bool CSG_Module_Chain::Tool_Initialize(const CSG_MetaData &Tool, CSG_Module *pMo
 
 			if( !pData )	// each input for this tool should be available now !!!
 			{
+				Error_Set(CSG_String::Format("%s: %s", _TL("input"), Parameter.Get_Property("id")));
+
 				return( false );
 			}
 
@@ -498,6 +814,8 @@ bool CSG_Module_Chain::Tool_Initialize(const CSG_MetaData &Tool, CSG_Module *pMo
 				{
 					if( !pParameter->Assign(pData) )
 					{
+						Error_Set(CSG_String::Format("%s: %s", _TL("set input"), Parameter.Get_Property("id")));
+
 						return( false );
 					}
 				}
@@ -507,13 +825,25 @@ bool CSG_Module_Chain::Tool_Initialize(const CSG_MetaData &Tool, CSG_Module *pMo
 				}
 
 				pParameter->has_Changed();
+
+				if( pOwner )
+				{
+					pOwner->has_Changed();
+				}
 			}
 		}
 		else if( Parameter.Cmp_Name("output") )
 		{
 			if( !pParameter->Assign(m_Data(Parameter.Get_Content())) )
 			{
-				pParameter->Set_Value(DATAOBJECT_CREATE);
+				if( pParameter->is_DataObject() )
+				{
+					pParameter->Set_Value(DATAOBJECT_CREATE);
+				}
+				else if( pParameter->is_DataObject_List() )
+				{
+					pParameter->asList()->Del_Items();
+				}
 			}
 		}
 	}
@@ -523,25 +853,36 @@ bool CSG_Module_Chain::Tool_Initialize(const CSG_MetaData &Tool, CSG_Module *pMo
 	{
 		const CSG_MetaData	&Parameter	= Tool[i];
 
-		CSG_String	ID	= Parameter.Get_Property("id");
+		CSG_Parameter	*pParameter, *pOwner;
 
-		CSG_Parameter	*pParameter	= pModule->Get_Parameters(Parameter.Get_Property("parms"))
-			? pModule->Get_Parameters(Parameter.Get_Property("parms"))->Get_Parameter(ID)
-			: pModule->Get_Parameters()->Get_Parameter(ID);
-
-		if( !pParameter )
+		if( !Tool_Get_Parameter(Parameter, pModule, &pParameter, &pOwner) )
 		{
 			return( false );
 		}
 		else if( Parameter.Cmp_Name("option") )
 		{
-			if( Parameter.Cmp_Property("varname", "1") || Parameter.Cmp_Property("varname", "true", true) )
+			if( IS_TRUE_PROPERTY(Parameter, "varname") )
 			{	// does option want a value from tool chain parameters and do these provide one ?
 				pParameter->Assign(Parameters(Parameter.Get_Content()));
 			}
 			else
 			{
-				pParameter->Set_Value(Parameter.Get_Content());
+				CSG_String	Value(Parameter.Get_Content());
+
+				if( pParameter->Get_Type() == PARAMETER_TYPE_String )
+				{
+					for(int j=0; j<Parameters.Get_Count(); j++)
+					{
+						CSG_String	Var; Var.Printf("$(%s)", Parameters(j)->Get_Identifier());
+
+						if( Value.Find(Var) >= 0 )
+						{
+							Value.Replace(Var, Parameters(j)->asString());
+						}
+					}
+				}
+
+				pParameter->Set_Value(Value);
 			}
 		}
 	}
@@ -595,7 +936,7 @@ bool CSG_Module_Chain::Tool_Finalize(const CSG_MetaData &Tool, CSG_Module *pModu
 				}
 				else if( pParameter->is_DataObject_List() )
 				{
-					for(int k=0; k<m_Data.Get_Count(); k++)
+					for(int k=0; k<pParameter->asList()->Get_Count(); k++)
 					{
 						if( !Data_Exists(pParameter->asList()->asDataObject(k)) )
 						{
@@ -687,6 +1028,8 @@ bool CSG_Module_Chains::Add_Module(CSG_Module_Chain *pModule)
 	m_pModules	= (CSG_Module_Chain **)SG_Realloc(m_pModules, (m_nModules + 1) * sizeof(CSG_Module_Chain *));
 	m_pModules[m_nModules++]	= pModule;
 
+	pModule->Set_Library_Menu(Get_Info(MLB_INFO_Menu_Path));
+
 	return( true );
 }
 
