@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: module_library.h 2282 2014-10-09 15:50:47Z oconrad $
+ * Version $Id: module_library.h 2680 2015-11-06 12:27:37Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -134,15 +134,15 @@ public:
 
 	virtual int						Get_Count			(void)	const	{	return( m_pInterface ? m_pInterface->Get_Count() : 0 );	}
 
-	virtual CSG_Module *			Get_Module			(int           Index, TSG_Module_Type Type = MODULE_TYPE_Base)	const;
-	virtual CSG_Module *			Get_Module			(const SG_Char *Name, TSG_Module_Type Type = MODULE_TYPE_Base)	const;
+	virtual CSG_Module *			Get_Module			(int               Index , TSG_Module_Type Type = MODULE_TYPE_Base)	const;
+	virtual CSG_Module *			Get_Module			(const CSG_String &Module, TSG_Module_Type Type = MODULE_TYPE_Base)	const;
 
-	CSG_Module_Grid *				Get_Module_Grid				(int           Index)	const;
-	CSG_Module_Grid *				Get_Module_Grid				(const SG_Char *Name)	const;
-	CSG_Module_Interactive *		Get_Module_Interactive		(int           Index)	const;
-	CSG_Module_Interactive *		Get_Module_Interactive		(const SG_Char *Name)	const;
-	CSG_Module_Grid_Interactive *	Get_Module_Grid_Interactive	(int           Index)	const;
-	CSG_Module_Grid_Interactive *	Get_Module_Grid_Interactive	(const SG_Char *Name)	const;
+	CSG_Module_Grid *				Get_Module_Grid				(int               Index )	const;
+	CSG_Module_Grid *				Get_Module_Grid				(const CSG_String &Module)	const;
+	CSG_Module_Interactive *		Get_Module_Interactive		(int               Index )	const;
+	CSG_Module_Interactive *		Get_Module_Interactive		(const CSG_String &Module)	const;
+	CSG_Module_Grid_Interactive *	Get_Module_Grid_Interactive	(int               Index )	const;
+	CSG_Module_Grid_Interactive *	Get_Module_Grid_Interactive	(const CSG_String &Module)	const;
 
 	virtual CSG_String				Get_File_Name		(int i)	const	{	return( "" );	}
 	virtual CSG_String				Get_Menu			(int i)	const;
@@ -198,8 +198,8 @@ public:
 
 	bool							is_Loaded			(CSG_Module_Library *pLibrary)	const;
 
-	CSG_Module *					Get_Module			(const SG_Char *Library, int            Module)	const;
-	CSG_Module *					Get_Module			(const SG_Char *Library, const SG_Char *Module)	const;
+	CSG_Module *					Get_Module			(const CSG_String &Library, int               ID    )	const;
+	CSG_Module *					Get_Module			(const CSG_String &Library, const CSG_String &Module)	const;
 
 	CSG_String						Get_Summary			(int Format = SG_SUMMARY_FMT_HTML)	const;
 	bool							Get_Summary			(const CSG_String &Path)			const;
@@ -235,21 +235,21 @@ SAGA_API_DLL_EXPORT CSG_Module_Library_Manager &	SG_Get_Module_Library_Manager	(
 	\
 	if(	pModule == NULL )\
 	{\
-		Error_Set(CSG_String::Format(SG_T("%s [%s]"), _TL("could not find tool"), SG_T(LIBRARY)));\
+		SG_UI_Msg_Add_Error(CSG_String::Format(SG_T("%s [%s]"), _TL("could not find tool"), SG_T(LIBRARY)));\
 	}\
 	else\
 	{\
-		Process_Set_Text(pModule->Get_Name());\
+		SG_UI_Process_Set_Text(pModule->Get_Name());\
 		\
 		pModule->Settings_Push();\
 		\
-		if( !(CONDITION) )\
+		if( !pModule->On_Before_Execution() || !(CONDITION) )\
 		{\
-			Error_Set(CSG_String::Format(SG_T("%s [%s].[%s]"), _TL("could not initialize tool"), SG_T(LIBRARY), pModule->Get_Name().c_str()));\
+			SG_UI_Msg_Add_Error(CSG_String::Format(SG_T("%s [%s].[%s]"), _TL("could not initialize tool"), SG_T(LIBRARY), pModule->Get_Name().c_str()));\
 		}\
 		else if( !pModule->Execute() )\
 		{\
-			Error_Set(CSG_String::Format(SG_T("%s [%s].[%s]"), _TL("could not execute tool"   ), SG_T(LIBRARY), pModule->Get_Name().c_str()));\
+			SG_UI_Msg_Add_Error(CSG_String::Format(SG_T("%s [%s].[%s]"), _TL("could not execute tool"   ), SG_T(LIBRARY), pModule->Get_Name().c_str()));\
 		}\
 		else\
 		{\
@@ -282,6 +282,11 @@ SAGA_API_DLL_EXPORT CSG_Module_Library_Manager &	SG_Get_Module_Library_Manager	(
 	&&	pModule->Get_Parameters()->Get_Parameter(IDENTIFIER)->asList()->Add_Item(VALUE)\
 )
 
+#define SG_MODULE_SET_DATAOBJECT_LIST(IDENTIFIER, VALUE)	(\
+		pModule->Get_Parameters()->Get_Parameter(IDENTIFIER)\
+	&&	pModule->Get_Parameters()->Get_Parameter(IDENTIFIER)->asList()\
+	&&	pModule->Get_Parameters()->Get_Parameter(IDENTIFIER)->asList()->Assign(VALUE)\
+)
 
 ///////////////////////////////////////////////////////////
 //														 //
