@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: parameter.cpp 1756 2013-06-27 10:39:19Z reklov_w $
+ * Version $Id: parameter.cpp 2624 2015-09-30 15:33:53Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -37,7 +37,7 @@
 // You should have received a copy of the GNU Lesser     //
 // General Public License along with this program; if    //
 // not, write to the Free Software Foundation, Inc.,     //
-// 59 Temple Place - Suite 330, Boston, MA 02111-1307,   //
+// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
 // USA.                                                  //
 //                                                       //
 //-------------------------------------------------------//
@@ -95,9 +95,11 @@ CSG_Parameter::CSG_Parameter(CSG_Parameters *pOwner, CSG_Parameter *pParent, con
 	}
 
 	//-----------------------------------------------------
+	m_pData			= NULL;
+
 	switch( Type )
 	{
-	default:								m_pData	= NULL;														break;
+	default:	break;
 
 	case PARAMETER_TYPE_Node:				m_pData	= new CSG_Parameter_Node				(this, Constraint);	break;
 
@@ -148,6 +150,13 @@ CSG_Parameter::CSG_Parameter(CSG_Parameters *pOwner, CSG_Parameter *pParent, con
 		m_Children		= NULL;
 		break;
 	}
+
+	//-----------------------------------------------------
+	if( m_pParent && m_pParent->m_pData )
+	{
+		Set_UseInCMD(m_pParent->do_UseInCMD());
+		Set_UseInGUI(m_pParent->do_UseInGUI());
+	}
 }
 
 //---------------------------------------------------------
@@ -179,6 +188,49 @@ void CSG_Parameter::_Add_Child(CSG_Parameter *pChild)
 }
 
 
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+void CSG_Parameter::Set_UseInGUI(bool bDoUse)
+{
+	if( bDoUse )
+	{
+		m_pData->m_Constraint	&= ~PARAMETER_NOT_FOR_GUI;
+	}
+	else
+	{
+		m_pData->m_Constraint	|=  PARAMETER_NOT_FOR_GUI;
+	}
+
+	for(int i=0; i<Get_Children_Count(); i++)
+	{
+		Get_Child(i)->Set_UseInGUI(bDoUse);
+	}
+}
+
+//---------------------------------------------------------
+void CSG_Parameter::Set_UseInCMD(bool bDoUse)
+{
+	if( bDoUse )
+	{
+		m_pData->m_Constraint	&= ~PARAMETER_NOT_FOR_CMD;
+	}
+	else
+	{
+		m_pData->m_Constraint	|=  PARAMETER_NOT_FOR_CMD;
+	}
+
+	for(int i=0; i<Get_Children_Count(); i++)
+	{
+		Get_Child(i)->Set_UseInCMD(bDoUse);
+	}
+}
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 //														 //
@@ -198,6 +250,33 @@ bool CSG_Parameter::Set_Enabled(bool bEnabled)
 	return( bEnabled );
 }
 
+//---------------------------------------------------------
+bool CSG_Parameter::is_Enabled(void) const
+{
+	if( do_UseInGUI() == false && SG_UI_Get_Window_Main() != NULL )
+	{
+		return( false );
+	}
+
+	if( do_UseInCMD() == false && SG_UI_Get_Window_Main() == NULL )
+	{
+		return( false );
+	}
+
+	return( m_bEnabled && (Get_Parent() == NULL || Get_Parent()->is_Enabled()) );
+}
+
+//---------------------------------------------------------
+bool CSG_Parameter::Set_Children_Enabled(bool bEnabled)
+{
+	for(int i=0; i<Get_Children_Count(); i++)
+	{
+		Get_Child(i)->Set_Enabled(bEnabled);
+	}
+
+	return( true );
+}
+
 //---------------------------------------------------------
 bool CSG_Parameter::is_Option(void)	const
 {
@@ -641,9 +720,14 @@ bool CSG_Parameter::Check(bool bSilent)
 //---------------------------------------------------------
 bool CSG_Parameter::Assign(CSG_Parameter *pSource)
 {
-	m_bEnabled	= pSource->m_bEnabled;
+	if( pSource )
+	{
+		m_bEnabled	= pSource->m_bEnabled;
 
-	return( m_pData->Assign(pSource->m_pData) );
+		return( m_pData->Assign(pSource->m_pData) );
+	}
+
+	return( false );
 }
 
 //---------------------------------------------------------
@@ -654,15 +738,15 @@ CSG_MetaData * CSG_Parameter::Serialize(CSG_MetaData &Entry, bool bSave)
 		if( !is_Information() && Get_Type() != PARAMETER_TYPE_Node && Get_Type() != PARAMETER_TYPE_Undefined )
 		{
 			CSG_MetaData	*pEntry	= Entry.Add_Child(
-				is_Option()          ? SG_T("OPTION")    :
-				is_DataObject()      ? SG_T("DATA")      :
-				is_DataObject_List() ? SG_T("DATA_LIST") :
-				                       SG_T("PARAMETER")
+				is_Option         () ? "OPTION"    :
+				is_DataObject     () ? "DATA"      :
+				is_DataObject_List() ? "DATA_LIST" : "PARAMETER"
 			);
 
-			pEntry->Add_Property(SG_T("type"), Get_Type_Identifier());
-			pEntry->Add_Property(SG_T("id")  , Get_Identifier());
-			pEntry->Add_Property(SG_T("name"), Get_Name());
+			pEntry->Add_Property("type" , Get_Type_Identifier        ());
+			pEntry->Add_Property("id"   , Get_Identifier             ());
+			pEntry->Add_Property("name" , Get_Name                   ());
+			pEntry->Add_Property("parms", Get_Owner()->Get_Identifier());
 
 			m_pData->Serialize(*pEntry, bSave);
 
@@ -671,8 +755,8 @@ CSG_MetaData * CSG_Parameter::Serialize(CSG_MetaData &Entry, bool bSave)
 	}
 	else
 	{
-		if(	Entry.Cmp_Property(SG_T("type"), Get_Type_Identifier())
-		&&	Entry.Cmp_Property(SG_T("id")  , Get_Identifier()) )
+		if(	Entry.Cmp_Property("type", Get_Type_Identifier())
+		&&	Entry.Cmp_Property("id"  , Get_Identifier     ()) )
 		{
 			return( m_pData->Serialize(Entry, bSave) ? &Entry : NULL );
 		}
@@ -691,14 +775,60 @@ CSG_MetaData * CSG_Parameter::Serialize(CSG_MetaData &Entry, bool bSave)
 //---------------------------------------------------------
 CSG_Parameters_Grid_Target::CSG_Parameters_Grid_Target(void)
 {
-	Create();
+	m_pParameters	= NULL;
 }
 
 //---------------------------------------------------------
-void CSG_Parameters_Grid_Target::Create(void)
+bool CSG_Parameters_Grid_Target::Create(CSG_Parameters *pParameters, bool bAddDefaultGrid, CSG_Parameter *pNode, const CSG_String &Prefix)
 {
-	m_pUser		= NULL;
-	m_pGrid		= NULL;
+	if( pParameters == NULL )
+	{
+		return( false );
+	}
+
+	m_pParameters	= pParameters;
+	m_Prefix		= Prefix;
+
+	//-----------------------------------------------------
+	pNode	= m_pParameters->Add_Choice(
+		pNode, m_Prefix + "DEFINITION", _TL("Target Grid System"),
+		_TL(""),
+		CSG_String::Format(SG_T("%s|%s|"),
+			_TL("user defined"),
+			_TL("grid or grid system")
+		), 0
+	);
+
+	//-----------------------------------------------------
+	m_pParameters->Add_Value     (pNode, m_Prefix + "USER_XMIN", _TL("Left"    ), _TL(""), PARAMETER_TYPE_Double,   0.0);
+	m_pParameters->Add_Value     (pNode, m_Prefix + "USER_XMAX", _TL("Right"   ), _TL(""), PARAMETER_TYPE_Double, 100.0);
+	m_pParameters->Add_Value     (pNode, m_Prefix + "USER_YMIN", _TL("Bottom"  ), _TL(""), PARAMETER_TYPE_Double,   0.0);
+	m_pParameters->Add_Value     (pNode, m_Prefix + "USER_YMAX", _TL("Top"     ), _TL(""), PARAMETER_TYPE_Double, 100.0);
+	m_pParameters->Add_Value     (pNode, m_Prefix + "USER_SIZE", _TL("Cellsize"), _TL(""), PARAMETER_TYPE_Double, 1.0, 0.0, true);
+	m_pParameters->Add_Info_Value(pNode, m_Prefix + "USER_COLS", _TL("Columns" ), _TL(""), PARAMETER_TYPE_Int   , 100);
+	m_pParameters->Add_Info_Value(pNode, m_Prefix + "USER_ROWS", _TL("Rows"    ), _TL(""), PARAMETER_TYPE_Int   , 100);
+	m_pParameters->Add_Choice    (pNode, m_Prefix + "USER_FITS", _TL("Fit"     ), _TL(""),
+		CSG_String::Format(SG_T("%s|%s|"),
+			_TL("nodes"),
+			_TL("cells")
+		), 0
+	);
+
+	//-----------------------------------------------------
+	pNode	= m_pParameters->Add_Grid_System(pNode, m_Prefix + "SYSTEM", _TL("Grid System"), _TL(""));
+
+	if( !SG_UI_Get_Window_Main() )
+	{
+		m_pParameters->Add_Grid(pNode, m_Prefix + "TEMPLATE", _TL("Target System"), _TL("use this grid's system for output grids"), PARAMETER_INPUT_OPTIONAL, false);
+	}
+
+	//-----------------------------------------------------
+	if( bAddDefaultGrid )
+	{
+		Add_Grid(m_Prefix + "OUT_GRID", _TL("Target Grid"), false);
+	}
+
+	return( true );
 }
 
 
@@ -707,118 +837,123 @@ void CSG_Parameters_Grid_Target::Create(void)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CSG_Parameters_Grid_Target::Add_Parameters_User(CSG_Parameters *pParameters, bool bAddDefaultGrid)
+bool CSG_Parameters_Grid_Target::On_Parameter_Changed(CSG_Parameters *pParameters, CSG_Parameter *pParameter)
 {
-	if( pParameters == NULL )
+	if( !m_pParameters || !pParameters || m_pParameters->Get_Identifier().Cmp(pParameters->Get_Identifier()) || !pParameter )
 	{
 		return( false );
 	}
 
-	m_pUser	= pParameters;
+	CSG_Parameter	*pXMin	= pParameters->Get_Parameter(m_Prefix + "USER_XMIN");
+	CSG_Parameter	*pXMax	= pParameters->Get_Parameter(m_Prefix + "USER_XMAX");
+	CSG_Parameter	*pYMin	= pParameters->Get_Parameter(m_Prefix + "USER_YMIN");
+	CSG_Parameter	*pYMax	= pParameters->Get_Parameter(m_Prefix + "USER_YMAX");
+	CSG_Parameter	*pSize	= pParameters->Get_Parameter(m_Prefix + "USER_SIZE");
+	CSG_Parameter	*pRows	= pParameters->Get_Parameter(m_Prefix + "USER_ROWS");
+	CSG_Parameter	*pCols	= pParameters->Get_Parameter(m_Prefix + "USER_COLS");
+	CSG_Parameter	*pFits	= pParameters->Get_Parameter(m_Prefix + "USER_FITS");
 
-	pParameters->Add_Value(
-		NULL, "XMIN"		, _TL("Left")		, _TL(""), PARAMETER_TYPE_Double
-	);
+	double		Size	= pSize->asDouble();
 
-	pParameters->Add_Value(
-		NULL, "XMAX"		, _TL("Right")		, _TL(""), PARAMETER_TYPE_Double
-	);
+	CSG_Rect	r(pXMin->asDouble(), pYMin->asDouble(), pXMax->asDouble(), pYMax->asDouble());
 
-	pParameters->Add_Value(
-		NULL, "YMIN"		, _TL("Bottom")		, _TL(""), PARAMETER_TYPE_Double
-	);
-
-	pParameters->Add_Value(
-		NULL, "YMAX"		, _TL("Top")		, _TL(""), PARAMETER_TYPE_Double
-	);
-
-	pParameters->Add_Value(
-		NULL, "SIZE"		, _TL("Cellsize")	, _TL(""), PARAMETER_TYPE_Double, 1.0, 0.0, true
-	);
-
-	pParameters->Add_Info_Value(
-		NULL, "COLS"		, _TL("Columns")	, _TL(""), PARAMETER_TYPE_Int
-	);
-
-	pParameters->Add_Info_Value(
-		NULL, "ROWS"		, _TL("Rows")		, _TL(""), PARAMETER_TYPE_Int
-	);
-
-	if( bAddDefaultGrid )
+	if( m_bFitToCells )
 	{
-		pParameters->Add_Grid_Output(
-			NULL, "GRID"		, _TL("Grid")		, _TL("")
-		);
+		r.Deflate(0.5 * Size, false);
 	}
 
+	//-----------------------------------------------------
+	if(      !SG_STR_CMP(pParameter->Get_Identifier(), pFits->Get_Identifier()) )
+	{
+		if( m_bFitToCells != (pFits->asInt() == 1) )
+		{
+			m_bFitToCells	= pFits->asInt() == 1;
+		}
+	}
+	else if( !SG_STR_CMP(pParameter->Get_Identifier(), pSize->Get_Identifier()) )
+	{
+		r.m_rect.xMax	= r.Get_XMin() + Size * (int)(r.Get_XRange() / Size);
+		r.m_rect.yMax	= r.Get_YMin() + Size * (int)(r.Get_YRange() / Size);
+	}
+	else if( !SG_STR_CMP(pParameter->Get_Identifier(), pXMin->Get_Identifier()) )
+	{
+		if( r.Get_XRange() <= 0.0 )
+			r.m_rect.xMin	= r.Get_XMax() - Size * pCols->asInt();
+		else
+			r.m_rect.xMax	= r.Get_XMin() + Size * (int)(r.Get_XRange() / Size);
+	}
+	else if( !SG_STR_CMP(pParameter->Get_Identifier(), pXMax->Get_Identifier()) )
+	{
+		if( r.Get_XRange() <= 0.0 )
+			r.m_rect.xMax	= r.Get_XMin() + Size * pCols->asInt();
+		else
+			r.m_rect.xMin	= r.Get_XMax() - Size * (int)(r.Get_XRange() / Size);
+	}
+	else if( !SG_STR_CMP(pParameter->Get_Identifier(), pYMin->Get_Identifier()) )
+	{
+		if( r.Get_YRange() <= 0.0 )
+			r.m_rect.yMin	= r.Get_YMax() - Size * pRows->asInt();
+		else
+			r.m_rect.yMax	= r.Get_YMin() + Size * (int)(r.Get_YRange() / Size);
+	}
+	else if( !SG_STR_CMP(pParameter->Get_Identifier(), pYMax->Get_Identifier()) )
+	{
+		if( r.Get_YRange() <= 0.0 )
+			r.m_rect.yMax	= r.Get_YMin() + Size * pRows->asInt();
+		else
+			r.m_rect.yMin	= r.Get_YMax() - Size * (int)(r.Get_YRange() / Size);
+	}
+	else if( !SG_STR_CMP(pParameter->Get_Identifier(), pCols->Get_Identifier()) && pCols->asInt() > 0 )
+	{
+		pSize->Set_Value(Size	= r.Get_XRange() / pCols->asDouble());
+	}
+	else if( !SG_STR_CMP(pParameter->Get_Identifier(), pRows->Get_Identifier()) && pRows->asInt() > 0 )
+	{
+		pSize->Set_Value(Size	= r.Get_YRange() / pRows->asDouble());
+	}
+
+	//-----------------------------------------------------
+	pCols->Set_Value(1 + (int)((r.Get_XRange()) / Size));
+	pRows->Set_Value(1 + (int)((r.Get_YRange()) / Size));
+
+	if( m_bFitToCells )
+	{
+		r.Inflate(0.5 * Size, false);
+	}
+
+	pXMin->Set_Value(r.Get_XMin());
+	pXMax->Set_Value(r.Get_XMax());
+	pYMin->Set_Value(r.Get_YMin());
+	pYMax->Set_Value(r.Get_YMax());
+
+	pFits->Set_Value(m_bFitToCells);
+
 	return( true );
 }
 
 //---------------------------------------------------------
-bool CSG_Parameters_Grid_Target::On_User_Changed(CSG_Parameters *pParameters, CSG_Parameter *pParameter)
+bool CSG_Parameters_Grid_Target::On_Parameters_Enable(CSG_Parameters *pParameters, CSG_Parameter *pParameter)
 {
-	if( !m_pUser || !pParameters || !pParameter || SG_STR_CMP(m_pUser->Get_Identifier(), pParameters->Get_Identifier()) )
+	if( !m_pParameters || !pParameters || m_pParameters->Get_Identifier().Cmp(pParameters->Get_Identifier()) || !pParameter )
 	{
 		return( false );
 	}
 
-	CSG_Parameter	*xMin	= pParameters->Get_Parameter("XMIN");
-	CSG_Parameter	*xMax	= pParameters->Get_Parameter("XMAX");
-	CSG_Parameter	*yMin	= pParameters->Get_Parameter("YMIN");
-	CSG_Parameter	*yMax	= pParameters->Get_Parameter("YMAX");
-	CSG_Parameter	*Size	= pParameters->Get_Parameter("SIZE");
-	CSG_Parameter	*Rows	= pParameters->Get_Parameter("ROWS");
-	CSG_Parameter	*Cols	= pParameters->Get_Parameter("COLS");
-
-	if( !xMin || !xMax || !yMin || !yMax || !Size || !Rows || !Cols )
+	if( (pParameter = pParameters->Get_Parameter(m_Prefix + "DEFINITION")) == NULL )
 	{
 		return( false );
 	}
 
-	if(      !SG_STR_CMP(pParameter->Get_Identifier(), SG_T("SIZE")) )
-	{
-		xMax->Set_Value(xMin->asDouble() + ((int)((xMax->asDouble() - xMin->asDouble()) / Size->asDouble())) * Size->asDouble());
-		yMax->Set_Value(yMin->asDouble() + ((int)((yMax->asDouble() - yMin->asDouble()) / Size->asDouble())) * Size->asDouble());
-	}
-	else if( !SG_STR_CMP(pParameter->Get_Identifier(), SG_T("XMIN")) )
-	{
-		if( xMin->asDouble() >= xMax->asDouble() )
-		{
-			xMin->Set_Value(xMax->asDouble() - Cols->asInt() * Size->asDouble());
-		}
-
-		xMax->Set_Value(xMin->asDouble() + ((int)((xMax->asDouble() - xMin->asDouble()) / Size->asDouble())) * Size->asDouble());
-	}
-	else if( !SG_STR_CMP(pParameter->Get_Identifier(), SG_T("XMAX")) )
-	{
-		if( xMin->asDouble() >= xMax->asDouble() )
-		{
-			xMax->Set_Value(xMin->asDouble() + Cols->asInt() * Size->asDouble());
-		}
-
-		xMin->Set_Value(xMax->asDouble() - ((int)((xMax->asDouble() - xMin->asDouble()) / Size->asDouble())) * Size->asDouble());
-	}
-	else if( !SG_STR_CMP(pParameter->Get_Identifier(), SG_T("YMIN")) )
-	{
-		if( yMin->asDouble() >= yMax->asDouble() )
-		{
-			yMin->Set_Value(yMax->asDouble() - Rows->asInt() * Size->asDouble());
-		}
-
-		yMax->Set_Value(yMin->asDouble() + ((int)((yMax->asDouble() - yMin->asDouble()) / Size->asDouble())) * Size->asDouble());
-	}
-	else if( !SG_STR_CMP(pParameter->Get_Identifier(), SG_T("YMAX")) )
-	{
-		if( yMin->asDouble() >= yMax->asDouble() )
-		{
-			yMax->Set_Value(yMin->asDouble() + Rows->asInt() * Size->asDouble());
-		}
-
-		yMin->Set_Value(yMax->asDouble() - ((int)((yMax->asDouble() - yMin->asDouble()) / Size->asDouble())) * Size->asDouble());
-	}
-
-	Cols->Set_Value(1 + (int)((xMax->asDouble() - xMin->asDouble()) / Size->asDouble()));
-	Rows->Set_Value(1 + (int)((yMax->asDouble() - yMin->asDouble()) / Size->asDouble()));
+	pParameters->Set_Enabled(m_Prefix + "USER_XMIN", pParameter->asInt() == 0);
+	pParameters->Set_Enabled(m_Prefix + "USER_XMAX", pParameter->asInt() == 0);
+	pParameters->Set_Enabled(m_Prefix + "USER_YMIN", pParameter->asInt() == 0);
+	pParameters->Set_Enabled(m_Prefix + "USER_YMAX", pParameter->asInt() == 0);
+	pParameters->Set_Enabled(m_Prefix + "USER_SIZE", pParameter->asInt() == 0);
+	pParameters->Set_Enabled(m_Prefix + "USER_ROWS", pParameter->asInt() == 0);
+	pParameters->Set_Enabled(m_Prefix + "USER_COLS", pParameter->asInt() == 0);
+	pParameters->Set_Enabled(m_Prefix + "USER_FITS", pParameter->asInt() == 0);
+	pParameters->Set_Enabled(m_Prefix + "USER_OPTS", pParameter->asInt() == 0);
+	pParameters->Set_Enabled(m_Prefix + "SYSTEM"   , pParameter->asInt() == 1);
 
 	return( true );
 }
@@ -828,106 +963,152 @@ bool CSG_Parameters_Grid_Target::On_User_Changed(CSG_Parameters *pParameters, CS
   * Initializes the grid system from Extent and number of rows.
   * If bFitToCells is true, the extent is deflated by half a cell size,
   * so automatically adjusting the extent based on grid nodes to
-  * to an extent based on the grid cells.
+  * an extent based on the grid cells. If Rounding is greater than
+  * zero it specifies the number of significant figures to which
+  * the cell size is rounded and also adjuts the extent coordinates
+  * to be a multiple of cell size.
 */
-bool CSG_Parameters_Grid_Target::Init_User(const TSG_Rect &Extent, int Rows, bool bFitToCells)
+bool CSG_Parameters_Grid_Target::Set_User_Defined(CSG_Parameters *pParameters, const TSG_Rect &Extent, int Rows, bool bFitToCells, int Rounding)
 {
-	if( !m_pUser || Extent.xMin >= Extent.xMax || Extent.yMin >= Extent.yMax || Rows < 1 )
+	if( !SG_UI_Get_Window_Main() )	// no cancel button, so set parameters directly
+	{
+		pParameters	= m_pParameters;
+	}
+
+	if( !m_pParameters || !pParameters || m_pParameters->Get_Identifier().Cmp(pParameters->Get_Identifier()) )
 	{
 		return( false );
 	}
 
-	double	Size	= (Extent.yMax - Extent.yMin) / (double)Rows;
-
-	CSG_Rect        GridExtent(Extent);
-
-    if( bFitToCells )
-        GridExtent.Deflate(0.5 * Size, false);
-
-	m_pUser->Get_Parameter("XMIN")->Set_Value(GridExtent.Get_XMin());
-	m_pUser->Get_Parameter("XMAX")->Set_Value(GridExtent.Get_XMax());
-	m_pUser->Get_Parameter("YMIN")->Set_Value(GridExtent.Get_YMin());
-	m_pUser->Get_Parameter("YMAX")->Set_Value(GridExtent.Get_YMax());
-	m_pUser->Get_Parameter("SIZE")->Set_Value(Size);
-	m_pUser->Get_Parameter("COLS")->Set_Value(1 + (int)((GridExtent.Get_XMax() - GridExtent.Get_XMin()) / Size));
-	m_pUser->Get_Parameter("ROWS")->Set_Value(1 + (int)((GridExtent.Get_YMax() - GridExtent.Get_YMin()) / Size));
-
-	return( true );
-}
-
-bool CSG_Parameters_Grid_Target::Init_User(double xMin, double yMin, double Size, int nx, int ny)
-{
-	if( !m_pUser || Size <= 0.0 || nx <= 1 || ny <= 1 )
+	if( Rows < 1 )
 	{
-		return( false );
+		Rows	= m_pParameters->Get_Parameter(m_Prefix + "USER_ROWS")->asInt();
+
+		if( Rows < 1 )
+		{
+			Rows	= 100;
+		}
 	}
 
-	m_pUser->Get_Parameter("XMIN")->Set_Value(xMin);
-	m_pUser->Get_Parameter("XMAX")->Set_Value(xMin + Size * nx);
-	m_pUser->Get_Parameter("YMIN")->Set_Value(yMin);
-	m_pUser->Get_Parameter("YMAX")->Set_Value(yMin + Size * ny);
-	m_pUser->Get_Parameter("SIZE")->Set_Value(Size);
-	m_pUser->Get_Parameter("COLS")->Set_Value(nx);
-	m_pUser->Get_Parameter("ROWS")->Set_Value(ny);
+	CSG_Rect	r(Extent);
+
+	if( r.Get_XRange() == 0.0 && r.Get_YRange() == 0.0 )
+	{
+		r.Inflate(0.5 * Rows, false);	// assume cellsize = 1.0
+	}
+	else if( r.Get_XRange() == 0.0 )
+	{
+		double	d	= 0.5 * r.Get_YRange() / Rows;	r.m_rect.xMin	-= d;	r.m_rect.xMax	+= d;	// inflate by half cellsize
+	}
+	else if( r.Get_YRange() == 0.0 )
+	{
+		double	d	= 0.5 * r.Get_XRange() / Rows;	r.m_rect.yMin	-= d;	r.m_rect.yMax	+= d;	// inflate by half cellsize
+	}
+
+	//-----------------------------------------------------
+	m_bFitToCells	= bFitToCells;
+
+	double	Size	= r.Get_YRange() / ((m_bFitToCells ? 1 : 0) + Rows);
+
+	if( Rounding > 0 )
+	{
+		Size	= SG_Get_Rounded_To_SignificantFigures(Size, Rounding);
+
+		r.m_rect.xMin	= Size * floor(r.m_rect.xMin / Size);
+		r.m_rect.yMin	= Size * floor(r.m_rect.yMin / Size);
+		r.m_rect.yMax	= Size * ceil (r.m_rect.yMax / Size);
+	}
+
+	//-----------------------------------------------------
+	int		Cols	= (bFitToCells ? 0 : 1) + (int)(r.Get_XRange() / Size);
+
+	r.m_rect.xMax	= r.Get_XMin() + Cols * Size;
+
+	bool	bCallback	= pParameters->Set_Callback(false);
+
+	pParameters->Set_Parameter(m_Prefix + "USER_XMIN", r.Get_XMin() );
+	pParameters->Set_Parameter(m_Prefix + "USER_XMAX", r.Get_XMax() );
+	pParameters->Set_Parameter(m_Prefix + "USER_YMIN", r.Get_YMin() );
+	pParameters->Set_Parameter(m_Prefix + "USER_YMAX", r.Get_YMax() );
+	pParameters->Set_Parameter(m_Prefix + "USER_SIZE", Size         );
+	pParameters->Set_Parameter(m_Prefix + "USER_COLS", Cols         );
+	pParameters->Set_Parameter(m_Prefix + "USER_ROWS", Rows         );
+	pParameters->Set_Parameter(m_Prefix + "USER_FITS", m_bFitToCells);
+
+	pParameters->Set_Callback(bCallback);
 
 	return( true );
 }
 
 //---------------------------------------------------------
-CSG_Grid * CSG_Parameters_Grid_Target::Get_User(TSG_Data_Type Type)
+bool CSG_Parameters_Grid_Target::Set_User_Defined(CSG_Parameters *pParameters, double xMin, double yMin, double Size, int nx, int ny, bool bFitToCells)
 {
-	return( Get_User(SG_T("GRID"), Type) );
+	if( Size <= 0.0 || nx < 1 || ny < 1 )
+	{
+		return( false );
+	}
+
+	CSG_Rect	Extent(xMin, yMin, xMin + Size * nx, yMin + Size * ny);
+
+	if( bFitToCells )
+	{
+		Extent.m_rect.xMax	+= Size;
+		Extent.m_rect.yMax	+= Size;
+	}
+
+	return( Set_User_Defined(pParameters, Extent, ny, bFitToCells, 0) );
 }
 
 //---------------------------------------------------------
-CSG_Grid * CSG_Parameters_Grid_Target::Get_User(const CSG_String &Identifier, TSG_Data_Type Type)
+bool CSG_Parameters_Grid_Target::Set_User_Defined(CSG_Parameters *pParameters, CSG_Shapes *pPoints, int Scale, bool bFitToCells, int Rounding)
 {
-	CSG_Grid	*pGrid	= NULL;
-
-	if( m_pUser )
+	if( !pPoints || pPoints->Get_Count() <= 0 || pPoints->Get_Extent().Get_Area() <= 0.0 )
 	{
-		CSG_Grid_System	System(
-			m_pUser->Get_Parameter("SIZE")->asDouble(),
-			m_pUser->Get_Parameter("XMIN")->asDouble(),
-			m_pUser->Get_Parameter("YMIN")->asDouble(),
-			m_pUser->Get_Parameter("XMAX")->asDouble(),
-			m_pUser->Get_Parameter("YMAX")->asDouble()
-		);
+		return( false );
+	}
 
-		if( System.is_Valid() )
-		{
-			pGrid	= SG_Create_Grid(System, Type);
-		}
+	double	d	= sqrt(pPoints->Get_Extent().Get_Area() / pPoints->Get_Count());	// edge length of a square given as average area per point (cell size)
 
-		if( Identifier.Length() > 0 && m_pUser->Get_Parameter(Identifier) )
+	int	Rows	= (int)(0.5 + pPoints->Get_Extent().Get_YRange() / d);	if( Scale > 1 ) Rows *= Scale;
+
+	return( Set_User_Defined(pParameters, pPoints->Get_Extent(), Rows, bFitToCells, Rounding) );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CSG_Parameters_Grid_Target::Cmd_Update(const TSG_Rect &Extent)
+{
+	if( m_pParameters && !SG_UI_Get_Window_Main() )	// tool is run by saga_cmd ...
+	{
+		if( m_pParameters->Get_Manager() == &SG_Get_Data_Manager() )	// ... but tool is not run as part of a tool chain
 		{
-			m_pUser->Get_Parameter(Identifier)->Set_Value(pGrid);
+			Set_User_Defined(m_pParameters, Extent);
+
+			SG_UI_Dlg_Parameters(m_pParameters, m_pParameters->Get_Name());
+
+			return( true );
 		}
 	}
 
-	return( pGrid );
+	return( false );
 }
 
 //---------------------------------------------------------
-bool CSG_Parameters_Grid_Target::Get_System_User(CSG_Grid_System &System)
+bool CSG_Parameters_Grid_Target::Cmd_Update(CSG_Shapes *pPoints)
 {
-	if( m_pUser )
+	if( m_pParameters && !SG_UI_Get_Window_Main() )	// tool is run by saga_cmd ...
 	{
-		if( m_pUser->Get_Parameter("SIZE")->asDouble() > 0.0
-		&&	m_pUser->Get_Parameter("COLS")->asInt()    > 0
-		&&	m_pUser->Get_Parameter("ROWS")->asInt()    > 0	)
+		if( m_pParameters->Get_Manager() == &SG_Get_Data_Manager() )	// ... but tool is not run as part of a tool chain
 		{
-			On_User_Changed(m_pUser, m_pUser->Get_Parameter("SIZE"));
+			Set_User_Defined(m_pParameters, pPoints);
 
-			System.Assign(
-				m_pUser->Get_Parameter("SIZE")->asDouble(),
-				m_pUser->Get_Parameter("XMIN")->asDouble(),
-				m_pUser->Get_Parameter("YMIN")->asDouble(),
-				m_pUser->Get_Parameter("COLS")->asInt(),
-				m_pUser->Get_Parameter("ROWS")->asInt()
-			);
+			SG_UI_Dlg_Parameters(m_pParameters, m_pParameters->Get_Name());
 
-			return( System.is_Valid() );
+			return( true );
 		}
 	}
 
@@ -940,34 +1121,36 @@ bool CSG_Parameters_Grid_Target::Get_System_User(CSG_Grid_System &System)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CSG_Parameters_Grid_Target::Add_Grid_Parameter(const CSG_String &Identifier, const CSG_String &Name, bool bOptional)
+bool CSG_Parameters_Grid_Target::Add_Grid(const CSG_String &Identifier, const CSG_String &Name, bool bOptional)
 {
-	if( Identifier.Length() == 0 )
+	if( !m_pParameters || Identifier.Length() == 0 || m_pParameters->Get_Parameter(Identifier) != NULL )
 	{
 		return( false );
 	}
 
-	if( m_pUser && m_pUser->Get_Parameter(Identifier) == NULL )
+	CSG_Parameter	*pTarget	= m_pParameters->Get_Parameter(m_Prefix + "DEFINITION");
+	CSG_Parameter	*pSystem	= NULL;
+
+	for(int i=0; i<pTarget->Get_Children_Count() && !pSystem; i++)
 	{
-		m_pUser->Add_Grid_Output(NULL, Identifier, Name, _TL(""));
+		if( pTarget->Get_Child(i)->Get_Type() == PARAMETER_TYPE_Grid_System )
+		{
+			pSystem	= pTarget->Get_Child(i);
+		}
 	}
 
-	if( m_pGrid && m_pGrid->Get_Parameter(Identifier) == NULL )
-	{
-		CSG_Parameter	*pSystem	= NULL;
+	m_pParameters->Add_Grid(pSystem, Identifier, Name, _TL(""), bOptional ? PARAMETER_OUTPUT_OPTIONAL : PARAMETER_OUTPUT, false);
 
-		for(int i=0; i<m_pGrid->Get_Count() && !pSystem; i++)
+	if( bOptional && SG_UI_Get_Window_Main() )
+	{
+		CSG_Parameter	*pNode	= m_pParameters->Get_Parameter(m_Prefix + "USER_OPTS");
+
+		if( !pNode )
 		{
-			if( m_pGrid->Get_Parameter(i)->Get_Type() == PARAMETER_TYPE_Grid_System )
-			{
-				pSystem	= m_pGrid->Get_Parameter(i);
-			}
+			pNode	= m_pParameters->Add_Node(pTarget, m_Prefix + "USER_OPTS", _TL("Optional Target Grids"), _TL(""));
 		}
 
-		m_pGrid->Add_Grid(pSystem, Identifier, Name, _TL(""), SG_UI_Get_Window_Main()
-			? (bOptional ? PARAMETER_OUTPUT_OPTIONAL : PARAMETER_OUTPUT)
-			: (bOptional ? PARAMETER_INPUT_OPTIONAL  : PARAMETER_INPUT ), false
-		);
+		m_pParameters->Add_Value(pNode, Identifier + "_CREATE", Name, _TL(""), PARAMETER_TYPE_Bool, false);
 	}
 
 	return( true );
@@ -979,87 +1162,98 @@ bool CSG_Parameters_Grid_Target::Add_Grid_Parameter(const CSG_String &Identifier
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CSG_Parameters_Grid_Target::Add_Parameters_Grid(CSG_Parameters *pParameters, bool bAddDefaultGrid)
+CSG_Grid_System CSG_Parameters_Grid_Target::Get_System(void)
 {
-	if( pParameters == NULL )
+	CSG_Grid_System	System;
+
+	if( m_pParameters )
 	{
-		return( false );
+		if( m_pParameters->Get_Parameter(m_Prefix + "DEFINITION")->asInt() == 0 )	// user defined
+		{
+			double	Size	= m_pParameters->Get_Parameter(m_Prefix + "USER_SIZE")->asDouble();
+
+			CSG_Rect	r(
+				m_pParameters->Get_Parameter(m_Prefix + "USER_XMIN")->asDouble(),
+				m_pParameters->Get_Parameter(m_Prefix + "USER_YMIN")->asDouble(),
+				m_pParameters->Get_Parameter(m_Prefix + "USER_XMAX")->asDouble(),
+				m_pParameters->Get_Parameter(m_Prefix + "USER_YMAX")->asDouble()
+			);
+
+			if( m_bFitToCells )
+			{
+				r.Deflate(0.5 * Size, false);
+			}
+
+			System.Assign(Size, r);
+		}
+		else
+		{
+			CSG_Parameter	*pParameter	= m_pParameters->Get_Parameter(m_Prefix + "SYSTEM");
+
+			if( pParameter->asGrid_System() )
+			{
+				System.Assign(*pParameter->asGrid_System());
+			}
+		}
 	}
 
-	m_pGrid	= pParameters;
-
-	if( bAddDefaultGrid )
-	{
-		pParameters->Add_Grid(
-			NULL	, "GRID"	, _TL("Grid")		, _TL(""), SG_UI_Get_Window_Main() ? PARAMETER_OUTPUT : PARAMETER_INPUT_OPTIONAL, false
-		);
-	}
-
-	return( true );
-}
-
-//---------------------------------------------------------
-CSG_Grid * CSG_Parameters_Grid_Target::Get_Grid(TSG_Data_Type Type)
-{
-	return( Get_Grid(SG_T("GRID"), Type) );
+	return( System );
 }
 
 //---------------------------------------------------------
 CSG_Grid * CSG_Parameters_Grid_Target::Get_Grid(const CSG_String &Identifier, TSG_Data_Type Type)
 {
-	CSG_Parameter	*pParameter	= m_pGrid && Identifier.Length() > 0 ? m_pGrid->Get_Parameter(Identifier) : NULL;
-
-	if( !pParameter || !pParameter->Get_Parent()->asGrid_System()->is_Valid() )
+	if( !m_pParameters )
 	{
 		return( NULL );
 	}
 
-	CSG_Grid	*pGrid	= pParameter->asGrid();
+	CSG_Parameter	*pParameter	= m_pParameters->Get_Parameter(Identifier);
 
-	if( (pGrid == DATAOBJECT_NOTSET && !pParameter->is_Optional()) || pGrid == DATAOBJECT_CREATE )
+	if( !pParameter )
 	{
-		pParameter->Set_Value(pGrid	= SG_Create_Grid(*pParameter->Get_Parent()->asGrid_System(), Type));
+		return( NULL );
+	}
 
-		if( pParameter->Get_Manager() )
+	CSG_Grid_System	System(Get_System());
+
+	if( !System.is_Valid() )
+	{
+		return( NULL );
+	}
+
+	CSG_Grid	*pGrid	= NULL;
+
+	if( m_pParameters->Get_Parameter(m_Prefix + "DEFINITION")->asInt() == 0 )
+	{
+		if( m_pParameters->Get_Parameter(Identifier + "_CREATE") == NULL
+		||  m_pParameters->Get_Parameter(Identifier + "_CREATE")->asBool() )
 		{
-			pParameter->Get_Manager()->Add(pGrid);
+			pGrid	= SG_Create_Grid(System, Type);
 		}
 	}
+	else
+	{
+		pGrid	= pParameter->asGrid();
+
+		if( (pGrid == DATAOBJECT_NOTSET && !pParameter->is_Optional()) || pGrid == DATAOBJECT_CREATE )
+		{
+			pGrid	= SG_Create_Grid(System, Type);
+		}
+	}
+
+	if( pGrid && pGrid != pParameter->asGrid() )
+	{
+		pParameter->Set_Value(pGrid);
+	}
 
 	return( pGrid );
 }
 
-
-///////////////////////////////////////////////////////////
-//														 //
-///////////////////////////////////////////////////////////
-
 //---------------------------------------------------------
-bool CSG_Parameters_Grid_Target::Add_Parameters_System(CSG_Parameters *pParameters)
+CSG_Grid * CSG_Parameters_Grid_Target::Get_Grid(TSG_Data_Type Type)
 {
-	if( pParameters == NULL )
-	{
-		return( false );
-	}
-
-	m_pSystem	= pParameters;
-
-	m_pSystem->Add_Grid_System(NULL, "SYSTEM", _TL("Grid System"), _TL(""));
-
-	return( true );
-}
-
-//---------------------------------------------------------
-bool CSG_Parameters_Grid_Target::Get_System(CSG_Grid_System &System)
-{
-	if( m_pSystem && m_pSystem->Get_Parameter("SYSTEM") && m_pSystem->Get_Parameter("SYSTEM")->asGrid_System() )
-	{
-		System.Assign(*m_pSystem->Get_Parameter("SYSTEM")->asGrid_System());
-
-		return( System.is_Valid() );
-	}
-
-	return( false );
+	return( Get_Grid(m_Prefix + "OUT_GRID", Type) );
 }
 
 
