@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: parameter_data.cpp 2310 2014-10-28 10:39:25Z oconrad $
+ * Version $Id: parameter_data.cpp 2727 2015-12-13 16:02:55Z reklov_w $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -1752,6 +1752,31 @@ void CSG_Parameter_Table_Field::On_Assign(CSG_Parameter_Data *pSource)
 	m_Default	= ((CSG_Parameter_Table_Field *)pSource)->m_Default;
 }
 
+//---------------------------------------------------------
+bool CSG_Parameter_Table_Field::On_Serialize(CSG_MetaData &Entry, bool bSave)
+{
+	if( bSave )
+	{
+		Entry.Set_Property("index", asInt());
+		Entry.Set_Content(asString());
+	}
+	else
+	{
+		int	idx;
+
+		if( Entry.Get_Property("index", idx) )	// we require this check for backward compatibility, "index" was first introduced with SAGA 2.2.3 (r2671)
+		{
+			return( Set_Value(idx) );
+		}
+		else
+		{
+			return( Set_Value(Entry.Get_Content()) );
+		}
+	}
+
+	return( true );
+}
+
 
 ///////////////////////////////////////////////////////////
 //														 //
@@ -1807,7 +1832,7 @@ bool CSG_Parameter_Table_Fields::Set_Value(const CSG_String &Value)
 		else if( sValue.Length() > 0 )
 		{
 			sValue.Trim();
-			
+
 			if( sValue[0] == '[' )
 			{
 				sValue	= sValue.AfterFirst('[').BeforeLast(']');
@@ -1959,30 +1984,30 @@ bool CSG_Parameter_Data_Object::On_Serialize(CSG_MetaData &Entry, bool bSave)
 	{
 		if(	m_pDataObject == DATAOBJECT_CREATE )
 		{
-			Entry.Set_Content(SG_T("CREATE"));
+			Entry.Set_Content("CREATE");
 		}
-		else if( m_pDataObject == DATAOBJECT_NOTSET || !SG_File_Exists(m_pDataObject->Get_File_Name()) )
+		else if( m_pDataObject == DATAOBJECT_NOTSET )//|| !SG_File_Exists(m_pDataObject->Get_File_Name(false)) )
 		{
-			Entry.Set_Content(SG_T("NOT SET"));
+			Entry.Set_Content("NOT SET");
 		}
 		else
 		{
-			Entry.Set_Content(m_pDataObject->Get_File_Name());
+			Entry.Set_Content(m_pDataObject->Get_File_Name(false));
 		}
 	}
 	else
 	{
-		if( Entry.Cmp_Content(SG_T("CREATE")) )
+		if( Entry.Cmp_Content("CREATE") )
 		{
 			Set_Value(DATAOBJECT_CREATE);
 		}
-		else if( Entry.Cmp_Content(SG_T("NOT SET")) )
+		else if( Entry.Cmp_Content("NOT SET") )
 		{
 			Set_Value(DATAOBJECT_NOTSET);
 		}
 		else
 		{
-			Set_Value(m_pOwner->Get_Manager() ? m_pOwner->Get_Manager()->Find(Entry.Get_Content()) : NULL);
+			Set_Value(m_pOwner->Get_Manager() ? m_pOwner->Get_Manager()->Find(Entry.Get_Content(), false) : NULL);
 		}
 	}
 
@@ -2021,7 +2046,10 @@ bool CSG_Parameter_Data_Object_Output::Set_Value(void *Value)
 		{
 			m_pOwner->Get_Manager()->Add(m_pDataObject);
 
-			SG_UI_DataObject_Add(m_pDataObject, false);
+			if( m_pOwner->Get_Manager() == &SG_Get_Data_Manager() )	// prevent that local data manager send their data objects to gui
+			{
+				SG_UI_DataObject_Add(m_pDataObject, false);
+			}
 		}
 	}
 
@@ -2474,7 +2502,10 @@ void CSG_Parameter_List::On_Assign(CSG_Parameter_Data *pSource)
 
 	for(int i=0; i<((CSG_Parameter_List *)pSource)->Get_Count(); i++)
 	{
-		Add_Item(((CSG_Parameter_List *)pSource)->asDataObject(i));
+		if( m_pOwner->Get_Manager() != &SG_Get_Data_Manager() || SG_Get_Data_Manager().Exists(((CSG_Parameter_List *)pSource)->asDataObject(i)) )
+		{
+			Add_Item(((CSG_Parameter_List *)pSource)->asDataObject(i));
+		}
 	}
 }
 
@@ -2485,9 +2516,11 @@ bool CSG_Parameter_List::On_Serialize(CSG_MetaData &Entry, bool bSave)
 	{
 		for(int i=0; i<Get_Count(); i++)
 		{
-			if( SG_File_Exists(asDataObject(i)->Get_File_Name()) )
+			CSG_String	File	= asDataObject(i)->Get_File_Name(false);
+
+			if( File.BeforeFirst(':').Cmp("PGSQL") || SG_File_Exists(File) )
 			{
-				Entry.Add_Child(SG_T("DATA"), asDataObject(i)->Get_File_Name());
+				Entry.Add_Child(SG_T("DATA"), File);
 			}
 		}
 	}
@@ -2650,7 +2683,6 @@ CSG_Parameter_Parameters::CSG_Parameter_Parameters(CSG_Parameter *pOwner, long C
 	: CSG_Parameter_Data(pOwner, Constraint)
 {
 	m_pParameters	= new CSG_Parameters(pOwner->Get_Owner()->Get_Owner(), pOwner->Get_Name(), pOwner->Get_Description(), pOwner->Get_Identifier());
-	m_pParameters	->Set_Callback();
 }
 
 CSG_Parameter_Parameters::~CSG_Parameter_Parameters(void)
