@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: parameters.cpp 2237 2014-09-18 15:07:21Z oconrad $
+ * Version $Id: parameters.cpp 2676 2015-11-04 18:39:37Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -119,7 +119,7 @@ void CSG_Parameters::_On_Construction(void)
 	m_nParameters		= 0;
 
 	m_Callback			= NULL;
-	m_bCallback			= false;
+	m_bCallback			= true;
 
 	m_pGrid_System		= NULL;
 }
@@ -335,8 +335,10 @@ CSG_Parameter * CSG_Parameters::Add_Choice(CSG_Parameter *pParent, const CSG_Str
 	m_pData		= (CSG_Parameter_Choice *)pParameter->m_pData;
 	m_pData->Set_Items(Items);
 
+	bool	bCallback	= Set_Callback(false);
 	pParameter->Set_Value  (Default);
 	pParameter->Set_Default(Default);
+	Set_Callback(bCallback);
 
 	return( pParameter );
 }
@@ -366,8 +368,10 @@ CSG_Parameter * CSG_Parameters::Add_FilePath(CSG_Parameter *pParent, const CSG_S
 	m_pData->Set_Flag_Multiple	(bMultiple);
 	m_pData->Set_Flag_Directory	(bDirectory);
 
+	bool	bCallback	= Set_Callback(false);
 	pParameter->Set_Value  (Default);
 	pParameter->Set_Default(Default);
+	Set_Callback(bCallback);
 
 	return( pParameter );
 }
@@ -381,8 +385,10 @@ CSG_Parameter * CSG_Parameters::Add_Font(CSG_Parameter *pParent, const CSG_Strin
 
 	if( pInit && *pInit )
 	{
+		bool	bCallback	= Set_Callback(false);
 		pParameter->Set_Value  (pInit);
 		pParameter->Set_Default(pInit);
+		Set_Callback(bCallback);
 	}
 
 	return( pParameter );
@@ -773,7 +779,9 @@ CSG_Parameter * CSG_Parameters::_Add_Value(CSG_Parameter *pParent, const CSG_Str
 		}
 	}
 
-	pParameter->Set_Value(Value);
+	bool	bCallback	= Set_Callback(false);
+	pParameter->Set_Value  (Value);
+	Set_Callback(bCallback);
 
 	return( pParameter );
 }
@@ -819,8 +827,10 @@ CSG_Parameter * CSG_Parameters::_Add_String(CSG_Parameter *pParent, const CSG_St
 
 	pParameter	= _Add(pParent, Identifier, Name, Description, bLongText ? PARAMETER_TYPE_Text : PARAMETER_TYPE_String, bInformation ? PARAMETER_INFORMATION : 0);
 
+	bool	bCallback	= Set_Callback(false);
 	pParameter->Set_Value  (String);
 	pParameter->Set_Default(String);
+	Set_Callback(bCallback);
 
 	((CSG_Parameter_String *)pParameter->Get_Data())->Set_Password(bPassword);
 
@@ -970,8 +980,12 @@ bool CSG_Parameters::Del_Parameters(void)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-void CSG_Parameters::Set_Callback_On_Parameter_Changed(TSG_PFNC_Parameter_Changed Callback)
+// Callback function used to react on parameter changes.
+// Return value is the previously set callback function.
+TSG_PFNC_Parameter_Changed CSG_Parameters::Set_Callback_On_Parameter_Changed(TSG_PFNC_Parameter_Changed Callback)
 {
+	TSG_PFNC_Parameter_Changed	Previous	= m_Callback;
+
 	m_Callback	= Callback;
 
 	for(int i=0; i<m_nParameters; i++)
@@ -981,11 +995,18 @@ void CSG_Parameters::Set_Callback_On_Parameter_Changed(TSG_PFNC_Parameter_Change
 			m_Parameters[i]->asParameters()->Set_Callback_On_Parameter_Changed(Callback);
 		}
 	}
+
+	return( Previous );
 }
 
 //---------------------------------------------------------
-void CSG_Parameters::Set_Callback(bool bActive)
+// If switched off parameter changes will not invoke a
+// consecutive call to the On_Parameter_Changed function.
+// Return value is the previous state.
+bool CSG_Parameters::Set_Callback(bool bActive)
 {
+	bool	bPrevious	= m_bCallback;
+
 	m_bCallback	= bActive;
 
 	for(int i=0; i<m_nParameters; i++)
@@ -995,6 +1016,8 @@ void CSG_Parameters::Set_Callback(bool bActive)
 			m_Parameters[i]->asParameters()->Set_Callback(bActive);
 		}
 	}
+
+	return( bPrevious );
 }
 
 //---------------------------------------------------------
@@ -1100,11 +1123,23 @@ bool CSG_Parameters::Set_Parameter(const CSG_String &Identifier, const SG_Char *
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CSG_Parameters::Restore_Defaults(void)
+bool CSG_Parameters::Restore_Defaults(bool bClearData)
 {
 	for(int i=0; i<Get_Count(); i++)
 	{
 		m_Parameters[i]->Restore_Default();
+
+		if( bClearData )
+		{
+			if( m_Parameters[i]->is_DataObject() )
+			{
+				m_Parameters[i]->Set_Value(DATAOBJECT_NOTSET);
+			}
+			else if( m_Parameters[i]->is_DataObject_List() )
+			{
+				m_Parameters[i]->asList()->Del_Items();
+			}
+		}
 	}
 
 	return( true );
@@ -1268,19 +1303,26 @@ bool CSG_Parameters::DataObjects_Create(void)
 
 			if( pDataObject )
 			{
-				pDataObject->Set_Name(p->Get_Name());
-
-				if( m_pManager )
+				if( p->Set_Value(pDataObject) )
 				{
-					m_pManager->Add(pDataObject);
+					pDataObject->Set_Name(p->Get_Name());
+
+					if( m_pManager )
+					{
+						m_pManager->Add(pDataObject);
+					}
+				}
+				else
+				{
+					delete(pDataObject);
+
+					bResult	= false;
 				}
 			}
 			else
 			{
 				bResult	= p->is_Optional();
 			}
-
-			p->Set_Value(pDataObject);
 		}
 	}
 
@@ -1472,16 +1514,12 @@ bool CSG_Parameters::Get_String(CSG_String &String, bool bOptionsOnly)
 		{
 			CSG_Parameter	*p	= m_Parameters[i];
 
-			if( (!bOptionsOnly || p->is_Option()) && !p->is_Information() && !(p->Get_Type() == PARAMETER_TYPE_String && ((CSG_Parameter_String *)p->Get_Data())->is_Password()) )
+			if( (!bOptionsOnly || p->is_Option()) && p->is_Enabled() && !p->is_Information() && !(p->Get_Type() == PARAMETER_TYPE_String && ((CSG_Parameter_String *)p->Get_Data())->is_Password()) )
 			{
 				bResult	= true;
 
-			//	String.Append(CSG_String::Format(SG_T("[%s] %s: %s\n"),
-			//		p->Get_Type_Name(),
-				String.Append(CSG_String::Format(SG_T("%s: %s\n"),
-					p->Get_Name(),
-					p->asString()
-				));
+			//	String	+= CSG_String::Format("[%s] %s: %s\n", p->Get_Type_Name(), p->Get_Name(), p->asString());
+				String	+= CSG_String::Format("%s: %s\n", p->Get_Name(), p->asString());
 			}
 		}
 	}
