@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: pointcloud.cpp 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: pointcloud.cpp 2700 2015-11-24 17:18:29Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -71,7 +71,12 @@
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-#define POINTCLOUD_FILE_VERSION	"SGPC01"
+#define PC_FILE_VERSION		"SGPC01"
+
+#define PC_STR_NBYTES		32
+#define PC_DAT_NBYTES		32
+
+#define PC_GET_NBYTES(type)	(type == SG_DATATYPE_String ? PC_STR_NBYTES : type == SG_DATATYPE_Date ? PC_DAT_NBYTES : (int)SG_Data_Type_Get_Size(type))
 
 
 ///////////////////////////////////////////////////////////
@@ -209,9 +214,6 @@ void CSG_PointCloud::_On_Construction(void)
 	m_Cursor		= NULL;
 	m_bXYZPrecDbl	= true;
 
-	m_Selected		= NULL;
-	m_nSelected		= 0;
-
 	Set_NoData_Value(-999999);
 
 	Set_Update_Flag();
@@ -220,8 +222,7 @@ void CSG_PointCloud::_On_Construction(void)
 	m_Shapes.Add_Shape();
 	m_Shapes_Index	= -1;
 
-	m_Array_Points  .Create(sizeof(char *), 0, SG_ARRAY_GROWTH_3);
-	m_Array_Selected.Create(sizeof(int   ), 0, SG_ARRAY_GROWTH_3);
+	m_Array_Points.Create(sizeof(char *), 0, SG_ARRAY_GROWTH_3);
 }
 
 //---------------------------------------------------------
@@ -283,7 +284,7 @@ bool CSG_PointCloud::_Load(const CSG_String &File_Name)
 		return( false );
 	}
 
-	if( !Stream.Read(ID, 6) || strncmp(ID, POINTCLOUD_FILE_VERSION, 5) != 0 )
+	if( !Stream.Read(ID, 6) || strncmp(ID, PC_FILE_VERSION, 5) != 0 )
 	{
 		SG_UI_Msg_Add(_TL("failed"), false, SG_UI_MSG_STYLE_FAILURE);
 		SG_UI_Msg_Add_Error(_TL("incompatible file."));
@@ -356,14 +357,14 @@ bool CSG_PointCloud::_Load(const CSG_String &File_Name)
 	}
 
 	//-----------------------------------------------------
-	long		fLength	= Stream.Length();
+	sLong		fLength	= Stream.Length();
 
-	while( _Inc_Array() && Stream.Read(m_Cursor + 1, nPointBytes) && SG_UI_Process_Set_Progress(Stream.Tell(), fLength) )
+	while( _Inc_Array() && Stream.Read(m_Cursor + 1, nPointBytes) && SG_UI_Process_Set_Progress((double)Stream.Tell(), (double)fLength) )
 	{}
 
 	_Dec_Array();
 
-	Set_File_Name(File_Name);
+	Set_File_Name(File_Name, true);
 
 	Load_MetaData(File_Name);
 
@@ -404,7 +405,7 @@ bool CSG_PointCloud::_Save(const CSG_String &File_Name)
 
 	int		i, iBuffer, nPointBytes	= m_nPointBytes - 1;
 
-	Stream.Write((void *)POINTCLOUD_FILE_VERSION, 6);
+	Stream.Write((void *)PC_FILE_VERSION, 6);
 	Stream.Write(&nPointBytes	, sizeof(int));
 	Stream.Write(&m_nFields		, sizeof(int));
 
@@ -426,7 +427,7 @@ bool CSG_PointCloud::_Save(const CSG_String &File_Name)
 
 	Set_Modified(false);
 
-	Set_File_Name(sFile_Name);
+	Set_File_Name(sFile_Name, true);
 
 	Save_MetaData(File_Name);
 
@@ -434,6 +435,8 @@ bool CSG_PointCloud::_Save(const CSG_String &File_Name)
 
 	SG_UI_Msg_Add(_TL("okay"), false, SG_UI_MSG_STYLE_SUCCESS);
 
+	SG_UI_Process_Set_Ready();
+
 	return( true );
 }
 
@@ -515,7 +518,7 @@ bool CSG_PointCloud::is_Compatible(CSG_PointCloud *pPointCloud) const
 //---------------------------------------------------------
 bool CSG_PointCloud::_Add_Field(const SG_Char *Name, TSG_Data_Type Type, int iField)
 {
-	if( !Name || SG_Data_Type_Get_Size(Type) <= 0 )
+	if( !Name || PC_GET_NBYTES(Type) <= 0 )
 	{
 		return( false );
 	}
@@ -528,14 +531,14 @@ bool CSG_PointCloud::_Add_Field(const SG_Char *Name, TSG_Data_Type Type, int iFi
 	m_Field_Name  [m_nFields]	= new CSG_String(Name);
 	m_Field_Type  [m_nFields]	= Type;
 	m_Field_Stats [m_nFields]	= new CSG_Simple_Statistics();
-	m_Field_Offset[m_nFields]	= m_nFields == 0 ? 1 : m_Field_Offset[m_nFields - 1] + (int)SG_Data_Type_Get_Size(m_Field_Type[m_nFields - 1]);
+	m_Field_Offset[m_nFields]	= m_nFields == 0 ? 1 : m_Field_Offset[m_nFields - 1] + PC_GET_NBYTES(m_Field_Type[m_nFields - 1]);
 
 	if( m_nFields == 0 )
 	{
 		m_nPointBytes	= 1;
 	}
 
-	m_nPointBytes	+= (int)SG_Data_Type_Get_Size(m_Field_Type[m_nFields]);
+	m_nPointBytes	+= PC_GET_NBYTES(m_Field_Type[m_nFields]);
 	m_nFields		++;
 
 	m_Shapes.Add_Field(Name, Type);
@@ -580,7 +583,7 @@ bool CSG_PointCloud::Del_Field(int iField)
 
 	//-----------------------------------------------------
 	m_nFields		--;
-	m_nPointBytes	-= (int)SG_Data_Type_Get_Size(m_Field_Type[iField]);
+	m_nPointBytes	-= PC_GET_NBYTES(m_Field_Type[iField]);
 
 	for(i=0; i<m_nRecords; i++)
 	{
@@ -605,7 +608,7 @@ bool CSG_PointCloud::Del_Field(int iField)
 		m_Field_Name  [i]	= m_Field_Name  [i + 1];
 		m_Field_Type  [i]	= m_Field_Type  [i + 1];
 		m_Field_Stats [i]	= m_Field_Stats [i + 1];
-		m_Field_Offset[i]	= m_Field_Offset[i - 1] + (int)SG_Data_Type_Get_Size(m_Field_Type[i - 1]);
+		m_Field_Offset[i]	= m_Field_Offset[i - 1] + PC_GET_NBYTES(m_Field_Type[i - 1]);
 	}
 
 	m_Field_Name	= (CSG_String            **)SG_Realloc(m_Field_Name  , m_nFields * sizeof(CSG_String *));
@@ -629,15 +632,16 @@ bool CSG_PointCloud::_Set_Field_Value(char *pPoint, int iField, double Value)
 		{
 		default:
 		case SG_DATATYPE_Undefined:	break;
-		case SG_DATATYPE_Byte:		*((BYTE   *)pPoint)	= (BYTE  )Value;	break;
-		case SG_DATATYPE_Char:		*((char   *)pPoint)	= (char  )Value;	break;
-		case SG_DATATYPE_Word:		*((WORD   *)pPoint)	= (WORD  )Value;	break;
-		case SG_DATATYPE_Short:		*((short  *)pPoint)	= (short )Value;	break;
-		case SG_DATATYPE_DWord:		*((DWORD  *)pPoint)	= (DWORD )Value;	break;
-		case SG_DATATYPE_Int:		*((int    *)pPoint)	= (int   )Value;	break;
-		case SG_DATATYPE_Long:		*((long   *)pPoint)	= (long  )Value;	break;
-		case SG_DATATYPE_Float:		*((float  *)pPoint)	= (float )Value;	break;
+		case SG_DATATYPE_Byte  :	*((BYTE   *)pPoint)	= (BYTE  )Value;	break;
+		case SG_DATATYPE_Char  :	*((char   *)pPoint)	= (char  )Value;	break;
+		case SG_DATATYPE_Word  :	*((WORD   *)pPoint)	= (WORD  )Value;	break;
+		case SG_DATATYPE_Short :	*((short  *)pPoint)	= (short )Value;	break;
+		case SG_DATATYPE_DWord :	*((DWORD  *)pPoint)	= (DWORD )Value;	break;
+		case SG_DATATYPE_Int   :	*((int    *)pPoint)	= (int   )Value;	break;
+		case SG_DATATYPE_Long  :	*((long   *)pPoint)	= (long  )Value;	break;
+		case SG_DATATYPE_Float :	*((float  *)pPoint)	= (float )Value;	break;
 		case SG_DATATYPE_Double:	*((double *)pPoint)	= (double)Value;	break;
+		case SG_DATATYPE_String:	sprintf(    pPoint, "%f"    , Value);	break;
 		}
 
 		m_Field_Stats[iField]->Invalidate();
@@ -653,20 +657,21 @@ double CSG_PointCloud::_Get_Field_Value(char *pPoint, int iField) const
 {
 	if( pPoint && iField >= 0 && iField < m_nFields )
 	{
-		pPoint	= ((char *)pPoint) + m_Field_Offset[iField];
+		pPoint	+= m_Field_Offset[iField];
 
 		switch( m_Field_Type[iField] )
 		{
 		case SG_DATATYPE_Undefined:		default:	break;
-		case SG_DATATYPE_Byte:			return( *((BYTE   *)pPoint) );
-		case SG_DATATYPE_Char:			return( *((char   *)pPoint) );
-		case SG_DATATYPE_Word:			return( *((WORD   *)pPoint) );
-		case SG_DATATYPE_Short:			return( *((short  *)pPoint) );
-		case SG_DATATYPE_DWord:			return( *((DWORD  *)pPoint) );
-		case SG_DATATYPE_Int:			return( *((int    *)pPoint) );
-		case SG_DATATYPE_Long:			return( *((long   *)pPoint) );
-		case SG_DATATYPE_Float:			return( *((float  *)pPoint) );
-		case SG_DATATYPE_Double:		return( *((double *)pPoint) );
+		case SG_DATATYPE_Byte  :	return( *((BYTE   *)pPoint) );
+		case SG_DATATYPE_Char  :	return( *((char   *)pPoint) );
+		case SG_DATATYPE_Word  :	return( *((WORD   *)pPoint) );
+		case SG_DATATYPE_Short :	return( *((short  *)pPoint) );
+		case SG_DATATYPE_DWord :	return( *((DWORD  *)pPoint) );
+		case SG_DATATYPE_Int   :	return( *((int    *)pPoint) );
+		case SG_DATATYPE_Long  :	return( *((long   *)pPoint) );
+		case SG_DATATYPE_Float :	return( *((float  *)pPoint) );
+		case SG_DATATYPE_Double:	return( *((double *)pPoint) );
+		case SG_DATATYPE_String:	return( atof(       pPoint) );
 		}
 	}
 
@@ -674,28 +679,60 @@ double CSG_PointCloud::_Get_Field_Value(char *pPoint, int iField) const
 }
 
 //---------------------------------------------------------
-bool CSG_PointCloud::Set_Value(int iPoint, int iField, const SG_Char *Value)
+bool CSG_PointCloud::_Set_Field_Value(char *pPoint, int iField, const SG_Char *Value)
 {
-	if( iPoint >= 0 && iPoint < m_nRecords && Value && *Value )
+	if( pPoint && iField >= 0 && iField < m_nFields && Value )
 	{
-		double		d;
 		CSG_String	s(Value);
 
-		if( s.asDouble(d) )
+		switch( m_Field_Type[iField] )
 		{
-			return( _Set_Field_Value(m_Points[iPoint], iField, d) );
+		default:
+			{
+				double	d;
+
+				return( s.asDouble(d) && _Set_Field_Value(pPoint, iField, d) );
+			}
+			break;
+
+		case SG_DATATYPE_Date:
+		case SG_DATATYPE_String:
+			pPoint	+= m_Field_Offset[iField];
+			memset(pPoint, 0, PC_STR_NBYTES);
+			memcpy(pPoint, s.b_str(), s.Length() > PC_STR_NBYTES ? PC_STR_NBYTES : s.Length());
+			break;
 		}
+
+		return( true );
 	}
 
 	return( false );
 }
 
 //---------------------------------------------------------
-bool CSG_PointCloud::Get_Value(int iPoint, int iField, CSG_String &Value)	const
+bool CSG_PointCloud::_Get_Field_Value(char *pPoint, int iField, CSG_String &Value)	const
 {
-	if( iPoint >= 0 && iPoint < m_nRecords && iField >= 0 && iField < m_nFields )
+	if( pPoint && iField >= 0 && iField < m_nFields )
 	{
-		Value.Printf(SG_T("%f"), _Get_Field_Value(m_Points[iPoint], iField));
+		switch( m_Field_Type[iField] )
+		{
+		default:
+			Value.Printf("%f", _Get_Field_Value(pPoint, iField));
+			break;
+
+		case SG_DATATYPE_Date:
+		case SG_DATATYPE_String:
+			{
+				char	s[PC_STR_NBYTES + 1];
+
+				memcpy(s, pPoint + m_Field_Offset[iField], PC_STR_NBYTES);
+
+				s[PC_STR_NBYTES]	= '\0';
+
+				Value	= s;
+			}
+			break;
+		}
 
 		return( true );
 	}
@@ -828,15 +865,13 @@ bool CSG_PointCloud::Del_Points(void)
 		SG_Free(m_Points[iPoint]);
 	}
 
-	m_Array_Points  .Destroy();
-	m_Array_Selected.Destroy();
+	m_Array_Points.Destroy();
 
 	m_nRecords	= 0;
 	m_Points	= NULL;
 	m_Cursor	= NULL;
 
-	m_nSelected	= 0;
-	m_Selected	= NULL;
+	m_Selection.Set_Array(0);
 
 	return( true );
 }
@@ -894,11 +929,15 @@ bool CSG_PointCloud::On_Update(void)
 
 		_Stats_Update(0);
 		_Stats_Update(1);
+		_Stats_Update(2);
 
 		m_Extent.Assign(
 			m_Field_Stats[0]->Get_Minimum(), m_Field_Stats[1]->Get_Minimum(),
 			m_Field_Stats[0]->Get_Maximum(), m_Field_Stats[1]->Get_Maximum()
 		);
+
+		m_ZMin = m_Field_Stats[2]->Get_Minimum();
+		m_ZMax = m_Field_Stats[2]->Get_Maximum();
 	}
 
 	return( true );
@@ -950,17 +989,27 @@ CSG_Shape * CSG_PointCloud::_Set_Shape(int iPoint)
 
 		for(int i=0; i<Get_Field_Count(); i++)
 		{
-			Set_Value(i, pShape->asDouble(i));
+			switch( Get_Field_Type(i) )
+			{
+			default:
+				Set_Value(i, pShape->asDouble(i));
+				break;
+
+			case SG_DATATYPE_Date:
+			case SG_DATATYPE_String:
+				Set_Value(i, pShape->asString(i));
+				break;
+			}
 		}
 
 		Set_Value(0, pShape->Get_Point(0).x);
 		Set_Value(1, pShape->Get_Point(0).y);
-		Set_Value(2, pShape->Get_Z(0));
+		Set_Value(2, pShape->Get_Z    (0)  );
 	}
 
 	if( iPoint >= 0 && iPoint < Get_Count() )
 	{
-		if( iPoint != m_Shapes_Index )
+		if(1|| iPoint != m_Shapes_Index )
 		{
 			m_Cursor	= m_Points[iPoint];
 
@@ -969,10 +1018,28 @@ CSG_Shape * CSG_PointCloud::_Set_Shape(int iPoint)
 
 			for(int i=0; i<Get_Field_Count(); i++)
 			{
-				pShape->Set_Value(i, Get_Value(i));
+				switch( Get_Field_Type(i) )
+				{
+				default:
+					pShape->Set_Value(i, Get_Value(i));
+					break;
+
+				case SG_DATATYPE_Date:
+				case SG_DATATYPE_String:
+					{
+						CSG_String	s;
+
+						Get_Value(i, s);
+
+						pShape->Set_Value(i, s);
+					}
+					break;
+				}
 			}
 
 			m_Shapes_Index	= iPoint;
+			pShape->m_Index	= iPoint;
+			pShape->Set_Selected(is_Selected(iPoint));
 		}
 
 		m_Shapes.Set_Modified(false);
@@ -1076,51 +1143,35 @@ CSG_Shape * CSG_PointCloud::Add_Shape(CSG_Table_Record *pCopy, TSG_ADD_Shape_Cop
 //---------------------------------------------------------
 bool CSG_PointCloud::Select(int iRecord, bool bInvert)
 {
-	if( !bInvert )
+	if( !bInvert && Get_Selection_Count() > 0 )
 	{
-		for(int i=0; i<m_nSelected; i++)
+		for(size_t i=0; i<Get_Selection_Count(); i++)
 		{
-			m_Points[m_Selected[i]][0]	&= ~SG_TABLE_REC_FLAG_Selected;
+			m_Points[Get_Selection_Index(i)][0]	&= ~SG_TABLE_REC_FLAG_Selected;
 		}
 
-		m_Array_Selected.Destroy();
-		m_Selected	= NULL;
-		m_nSelected	= 0;
+		m_Selection.Destroy();
 	}
 
 	if( Set_Cursor(iRecord) )
 	{
 		if( (m_Cursor[0] & SG_TABLE_REC_FLAG_Selected) == 0 )	// select
 		{
-			if( m_Array_Selected.Set_Array(m_nSelected + 1, (void **)&m_Selected) )
+			if( _Add_Selection(iRecord) )
 			{
 				m_Cursor[0]	|= SG_TABLE_REC_FLAG_Selected;
 
-				m_Selected[m_nSelected++]	= iRecord;
-
 				return( true );
 			}
 		}
 		else													// deselect
 		{
-			m_Cursor[0]	&= ~SG_TABLE_REC_FLAG_Selected;
-
-			m_nSelected--;
-
-			for(int i=0; i<m_nSelected; i++)
+			if( _Del_Selection(iRecord) )
 			{
-				if( m_Points[iRecord] == m_Points[m_Selected[i]] )
-				{
-					for(; i<m_nSelected; i++)
-					{
-						m_Selected[i]	= m_Selected[i + 1];
-					}
-				}
+				m_Cursor[0]	&= ~SG_TABLE_REC_FLAG_Selected;
+
+				return( true );
 			}
-
-			m_Array_Selected.Set_Array(m_nSelected, (void **)&m_Selected);
-
-			return( true );
 		}
 	}
 
@@ -1155,7 +1206,7 @@ bool CSG_PointCloud::Select(TSG_Rect Extent, bool bInvert)
 		}
 	}
 
-	return( m_nSelected > 0 );
+	return( Get_Selection_Count() > 0 );
 }
 
 //---------------------------------------------------------
@@ -1183,24 +1234,24 @@ bool CSG_PointCloud::is_Selected(int iRecord)	const
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-CSG_Shape * CSG_PointCloud::Get_Selection(int Index)
+CSG_Shape * CSG_PointCloud::Get_Selection(size_t Index)
 {
-	return( Index >= 0 && Index < m_nSelected ? _Set_Shape(m_Selected[Index]) : NULL );
+	return( Index < Get_Selection_Count() ? _Set_Shape(Get_Selection_Index(Index)) : NULL );
 }
 
 //---------------------------------------------------------
 const CSG_Rect & CSG_PointCloud::Get_Selection_Extent(void)
 {
-	if( m_nSelected > 0 && Set_Cursor(m_Selected[0]) )
+	if( Get_Selection_Count() > 0 && Set_Cursor(Get_Selection_Index(0)) )
 	{
 		TSG_Rect	r;
 
 		r.xMin	= r.xMax	= Get_X();
 		r.yMin	= r.yMax	= Get_Y();
 
-		for(int i=1; i<m_nSelected; i++)
+		for(size_t i=1; i<Get_Selection_Count(); i++)
 		{
-			if( Set_Cursor(m_Selected[i]) )
+			if( Set_Cursor(Get_Selection_Index(i)) )
 			{
 				if( Get_X() < r.xMin )	r.xMin	= Get_X();	else if( Get_X() > r.xMax )	r.xMax	= Get_X();
 				if( Get_Y() < r.yMin )	r.yMin	= Get_Y();	else if( Get_Y() > r.yMax )	r.yMax	= Get_Y();
@@ -1225,36 +1276,33 @@ const CSG_Rect & CSG_PointCloud::Get_Selection_Extent(void)
 //---------------------------------------------------------
 int CSG_PointCloud::Del_Selection(void)
 {
-	if( m_nSelected <= 0 )
+	int	n	= 0;
+
+	if( Get_Selection_Count() > 0 )
 	{
-		return( 0 );
-	}
+		m_Selection.Set_Array(0);
 
-	//-----------------------------------------------------
-	int		i, n;
+		m_Cursor	= NULL;
 
-	m_Array_Selected.Set_Array(0, (void **)&m_Selected);
-	m_nSelected	= 0;
-	m_Cursor	= NULL;
-
-	for(i=0, n=0; i<m_nRecords; i++)
-	{
-		if( (m_Points[i][0] & SG_TABLE_REC_FLAG_Selected) != 0 )
+		for(int i=0; i<m_nRecords; i++)
 		{
-			SG_Free(m_Points[i]);
-		}
-		else
-		{
-			if( n < i )
+			if( (m_Points[i][0] & SG_TABLE_REC_FLAG_Selected) != 0 )
 			{
-				m_Points[n]	= m_Points[i];
+				SG_Free(m_Points[i]);
 			}
+			else
+			{
+				if( n < i )
+				{
+					m_Points[n]	= m_Points[i];
+				}
 
-			n++;
+				n++;
+			}
 		}
-	}
 
-	m_Array_Points.Set_Array(m_nRecords = n, (void **)&m_Points);
+		m_Array_Points.Set_Array(m_nRecords = n, (void **)&m_Points);
+	}
 
 	return( n );
 }
@@ -1262,29 +1310,26 @@ int CSG_PointCloud::Del_Selection(void)
 //---------------------------------------------------------
 int CSG_PointCloud::Inv_Selection(void)
 {
-	char	**pPoint;
-	int		i, n;
-
-	n	= m_nRecords - m_nSelected;
-
-	if( m_Array_Selected.Set_Array(n, (void **)&m_Selected) )
+	if( m_Selection.Set_Array((size_t)m_nRecords - Get_Selection_Count()) )
 	{
-		for(i=0, m_nSelected=0, pPoint=m_Points; i<m_nRecords; i++, pPoint++)
-		{
-			if( ((*pPoint)[0] & SG_TABLE_REC_FLAG_Selected) == 0 && m_nSelected < n )
-			{
-				m_Selected[m_nSelected++]	= i;
+		char	**pPoint	= m_Points;
 
-				(*pPoint)[0]	|= SG_TABLE_REC_FLAG_Selected;
+		for(size_t i=0, n=0; i<(size_t)m_nRecords && n<Get_Selection_Count(); i++, pPoint++)
+		{
+			if( ((*pPoint)[0] & SG_TABLE_REC_FLAG_Selected) != 0 )
+			{
+				(*pPoint)[0]	&= ~SG_TABLE_REC_FLAG_Selected;
 			}
 			else
 			{
-				(*pPoint)[0]	&= ~SG_TABLE_REC_FLAG_Selected;
+				(*pPoint)[0]	|= SG_TABLE_REC_FLAG_Selected;
+
+				_Set_Selection(i, n++);
 			}
 		}
 	}
 
-	return( m_nSelected );
+	return( Get_Selection_Count() );
 }
 
 
