@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: shapes.cpp 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: shapes.cpp 2706 2015-12-01 14:22:12Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -65,6 +65,7 @@
 //---------------------------------------------------------
 #include "shapes.h"
 #include "pointcloud.h"
+#include "module_library.h"
 
 
 ///////////////////////////////////////////////////////////
@@ -225,31 +226,99 @@ bool CSG_Shapes::Create(const CSG_String &File_Name)
 {
 	Destroy();
 
-	SG_UI_Msg_Add(CSG_String::Format(SG_T("%s: %s..."), _TL("Load shapes"), File_Name.c_str()), true);
+	SG_UI_Msg_Add(CSG_String::Format("%s: %s...", _TL("Load shapes"), File_Name.c_str()), true);
 
-	bool	bResult	= _Load_ESRI(File_Name);
-
-	Set_File_Name(File_Name);
-	Load_MetaData(File_Name);
+	//-----------------------------------------------------
+	bool	bResult	= File_Name.BeforeFirst(':').Cmp("PGSQL") && SG_File_Exists(File_Name) && _Load_ESRI(File_Name);
 
 	if( bResult )
 	{
-		SG_UI_Msg_Add(_TL("okay"), false, SG_UI_MSG_STYLE_SUCCESS);
+		Set_File_Name(File_Name, true);
+		Load_MetaData(File_Name);
 	}
-	else
+
+	//-----------------------------------------------------
+	else if( File_Name.BeforeFirst(':').Cmp("PGSQL") == 0 )	// database source
 	{
-		for(int iShape=Get_Count()-1; iShape >= 0; iShape--)
-		{
-			if( !Get_Shape(iShape)->is_Valid() )
-			{
-				Del_Shape(iShape);
-			}
-		}
+		CSG_String	s(File_Name);
 
-		SG_UI_Msg_Add(_TL("failed"), false, SG_UI_MSG_STYLE_FAILURE);
+		s	= s.AfterFirst(':');	CSG_String	Host  (s.BeforeFirst(':'));
+		s	= s.AfterFirst(':');	CSG_String	Port  (s.BeforeFirst(':'));
+		s	= s.AfterFirst(':');	CSG_String	DBName(s.BeforeFirst(':'));
+		s	= s.AfterFirst(':');	CSG_String	Table (s.BeforeFirst(':'));
+
+		CSG_Module	*pModule	= SG_Get_Module_Library_Manager().Get_Module("db_pgsql", 0);	// CGet_Connections
+
+		if(	pModule != NULL )
+		{
+			SG_UI_ProgressAndMsg_Lock(true);
+
+			//---------------------------------------------
+			CSG_Table	Connections;
+			CSG_String	Connection	= DBName + " [" + Host + ":" + Port + "]";
+
+			pModule->Settings_Push();
+
+			if( pModule->On_Before_Execution() && SG_MODULE_PARAMETER_SET("CONNECTIONS", &Connections) && pModule->Execute() )	// CGet_Connections
+			{
+				for(int i=0; !bResult && i<Connections.Get_Count(); i++)
+				{
+					if( !Connection.Cmp(Connections[i].asString(0)) )
+					{
+						bResult	= true;
+					}
+				}
+			}
+
+			pModule->Settings_Pop();
+
+			//---------------------------------------------
+			if( bResult && (bResult = (pModule = SG_Get_Module_Library_Manager().Get_Module("db_pgsql", 20)) != NULL) == true )	// CPGIS_Shapes_Load
+			{
+				pModule->Settings_Push();
+
+				bResult	= pModule->On_Before_Execution()
+					&& SG_MODULE_PARAMETER_SET("CONNECTION", Connection)
+					&& SG_MODULE_PARAMETER_SET("TABLES"    , Table)
+					&& SG_MODULE_PARAMETER_SET("SHAPES"    , this)
+					&& pModule->Execute();
+
+				pModule->Settings_Pop();
+			}
+
+			SG_UI_ProgressAndMsg_Lock(false);
+		}
 	}
 
-	return( bResult );
+	//-----------------------------------------------------
+	if( bResult )
+	{
+		Set_Modified(false);
+		Set_Update_Flag();
+
+		SG_UI_Process_Set_Ready();
+		SG_UI_Msg_Add(_TL("okay"), false, SG_UI_MSG_STYLE_SUCCESS);
+
+		return( true );
+	}
+
+	for(int iShape=Get_Count()-1; iShape>=0; iShape--)	// be kind, keep at least those shapes that have been loaded successfully
+	{
+		if( !Get_Shape(iShape)->is_Valid() )
+		{
+			Del_Shape(iShape);
+		}
+	}
+
+	if( Get_Count() <= 0 )
+	{
+		Destroy();
+	}
+
+	SG_UI_Process_Set_Ready();
+	SG_UI_Msg_Add(_TL("failed"), false, SG_UI_MSG_STYLE_FAILURE);
+
+	return( false );
 }
 
 //---------------------------------------------------------
@@ -305,7 +374,7 @@ bool CSG_Shapes::Assign(CSG_Data_Object *pObject)
 	{
 		pShapes	= (CSG_Shapes *)pObject;
 
-		Create(pShapes->Get_Type(), pShapes->Get_Name(), pShapes);
+		Create(pShapes->Get_Type(), pShapes->Get_Name(), pShapes, pShapes->Get_Vertex_Type());
 
 		for(iShape=0; iShape<pShapes->Get_Count() && SG_UI_Process_Set_Progress(iShape, pShapes->Get_Count()); iShape++)
 		{
@@ -341,15 +410,17 @@ bool CSG_Shapes::Save(const CSG_String &File_Name, int Format)
 	{
 		Set_Modified(false);
 
-		Set_File_Name(File_Name);
+		Set_File_Name(File_Name, true);
 
 		Save_MetaData(File_Name);
 
+		SG_UI_Process_Set_Ready();
 		SG_UI_Msg_Add(_TL("okay"), false, SG_UI_MSG_STYLE_SUCCESS);
 
 		return( true );
 	}
 
+	SG_UI_Process_Set_Ready();
 	SG_UI_Msg_Add(_TL("failed"), false, SG_UI_MSG_STYLE_FAILURE);
 
 	return( false );
