@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: table.cpp 1728 2013-06-13 09:37:08Z oconrad $
+ * Version $Id: table.cpp 2700 2015-11-24 17:18:29Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -37,7 +37,7 @@
 // You should have received a copy of the GNU Lesser     //
 // General Public License along with this program; if    //
 // not, write to the Free Software Foundation, Inc.,     //
-// 59 Temple Place - Suite 330, Boston, MA 02111-1307,   //
+// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
 // USA.                                                  //
 //                                                       //
 //-------------------------------------------------------//
@@ -65,6 +65,7 @@
 //---------------------------------------------------------
 #include "table.h"
 #include "shapes.h"
+#include "module_library.h"
 
 
 ///////////////////////////////////////////////////////////
@@ -169,11 +170,92 @@ CSG_Table::CSG_Table(const CSG_String &File_Name, TSG_Table_File_Type Format)
 
 bool CSG_Table::Create(const CSG_String &File_Name, TSG_Table_File_Type Format)
 {
-	return( _Load(File_Name, Format, NULL) );
+	Destroy();
+
+	SG_UI_Msg_Add(CSG_String::Format("%s: %s...", _TL("Load table"), File_Name.c_str()), true);
+
+	//-----------------------------------------------------
+	bool	bResult	= File_Name.BeforeFirst(':').Cmp("PGSQL") && SG_File_Exists(File_Name) && Load(File_Name, (int)Format, NULL);
+
+	if( bResult )
+	{
+		Set_File_Name(File_Name, true);
+		Load_MetaData(File_Name);
+	}
+
+	//-----------------------------------------------------
+	else if( File_Name.BeforeFirst(':').Cmp("PGSQL") == 0 )	// database source
+	{
+		CSG_String	s(File_Name);
+
+		s	= s.AfterFirst(':');	CSG_String	Host  (s.BeforeFirst(':'));
+		s	= s.AfterFirst(':');	CSG_String	Port  (s.BeforeFirst(':'));
+		s	= s.AfterFirst(':');	CSG_String	DBName(s.BeforeFirst(':'));
+		s	= s.AfterFirst(':');	CSG_String	Table (s.BeforeFirst(':'));
+
+		CSG_Module	*pModule	= SG_Get_Module_Library_Manager().Get_Module("db_pgsql", 0);	// CGet_Connections
+
+		if(	pModule != NULL )
+		{
+			SG_UI_ProgressAndMsg_Lock(true);
+
+			//---------------------------------------------
+			CSG_Table	Connections;
+			CSG_String	Connection	= DBName + " [" + Host + ":" + Port + "]";
+
+			pModule->Settings_Push();
+
+			if( pModule->On_Before_Execution() && SG_MODULE_PARAMETER_SET("CONNECTIONS", &Connections) && pModule->Execute() )	// CGet_Connections
+			{
+				for(int i=0; !bResult && i<Connections.Get_Count(); i++)
+				{
+					if( !Connection.Cmp(Connections[i].asString(0)) )
+					{
+						bResult	= true;
+					}
+				}
+			}
+
+			pModule->Settings_Pop();
+
+			//---------------------------------------------
+			if( bResult && (bResult = (pModule = SG_Get_Module_Library_Manager().Get_Module("db_pgsql", 12)) != NULL) == true )	// CPGIS_Table_Load
+			{
+				pModule->Settings_Push();
+
+				bResult	= pModule->On_Before_Execution()
+					&& SG_MODULE_PARAMETER_SET("CONNECTION", Connection)
+					&& SG_MODULE_PARAMETER_SET("TABLES"    , Table)
+					&& SG_MODULE_PARAMETER_SET("TABLE"     , this)
+					&& pModule->Execute();
+
+				pModule->Settings_Pop();
+			}
+
+			SG_UI_ProgressAndMsg_Lock(false);
+		}
+	}
+
+	//-----------------------------------------------------
+	if( bResult )
+	{
+		Set_Modified(false);
+		Set_Update_Flag();
+
+		SG_UI_Msg_Add(_TL("okay"), false, SG_UI_MSG_STYLE_SUCCESS);
+
+		return( true );
+	}
+
+	Destroy();
+
+	SG_UI_Msg_Add(_TL("failed"), false, SG_UI_MSG_STYLE_FAILURE);
+
+	return( false );
 }
 
 //---------------------------------------------------------
-CSG_Table::CSG_Table(const CSG_String &File_Name, TSG_Table_File_Type Format, const SG_Char *Separator)
+CSG_Table::CSG_Table(const CSG_String &File_Name, TSG_Table_File_Type Format, const SG_Char Separator)
 	: CSG_Data_Object()
 {
 	_On_Construction();
@@ -181,9 +263,9 @@ CSG_Table::CSG_Table(const CSG_String &File_Name, TSG_Table_File_Type Format, co
 	Create(File_Name, Format, Separator);
 }
 
-bool CSG_Table::Create(const CSG_String &File_Name, TSG_Table_File_Type Format, const SG_Char *Separator)
+bool CSG_Table::Create(const CSG_String &File_Name, TSG_Table_File_Type Format, const SG_Char Separator)
 {
-	return( _Load(File_Name, Format, Separator) );
+	return( Load(File_Name, (int)Format, Separator) );
 }
 
 //---------------------------------------------------------
@@ -231,11 +313,10 @@ void CSG_Table::_On_Construction(void)
 	m_nRecords		= 0;
 	m_nBuffer		= 0;
 
-	m_nSelected		= 0;
-	m_Selected		= NULL;
-
 	m_Index			= NULL;
 
+	m_Selection.Create(sizeof(size_t), 0, SG_ARRAY_GROWTH_3);
+
 	Set_Update_Flag();
 }
 
@@ -290,6 +371,14 @@ bool CSG_Table::Destroy(void)
 //														 //
 ///////////////////////////////////////////////////////////
 
+//---------------------------------------------------------
+CSG_Table & CSG_Table::operator = (const CSG_Table &Table)
+{
+	Create(Table);
+
+	return( *this );
+}
+
 //---------------------------------------------------------
 bool CSG_Table::Assign(CSG_Data_Object *pObject)
 {
@@ -564,6 +653,20 @@ int CSG_Table::Get_Field_Length(int iField)	const
 	return( Length );
 }
 
+//---------------------------------------------------------
+int CSG_Table::Get_Field(const CSG_String &Name) const
+{
+	for(int iField=0; iField<Get_Field_Count(); iField++)
+	{
+		if( !Name.Cmp(Get_Field_Name(iField)) )
+		{
+			return( iField );
+		}
+	}
+
+	return( -1 );
+}
+
 
 ///////////////////////////////////////////////////////////
 //														 //
@@ -715,7 +818,7 @@ CSG_Table_Record * CSG_Table::Ins_Record(int iRecord, CSG_Table_Record *pCopy)
 		{
 			if( is_Indexed() )
 			{
-				m_Index[i]		= m_Index[i - 1];
+				m_Index[i]	= m_Index[i - 1];
 			}
 
 			m_Records[i]			= m_Records[i - 1];
@@ -727,7 +830,8 @@ CSG_Table_Record * CSG_Table::Ins_Record(int iRecord, CSG_Table_Record *pCopy)
 			m_Index[iRecord]	= iRecord;
 		}
 
-		m_Records[iRecord]		= pRecord;
+		pRecord->m_Index	= iRecord;
+		m_Records[iRecord]	= pRecord;
 		m_nRecords++;
 
 		Set_Modified();
@@ -745,15 +849,15 @@ CSG_Table_Record * CSG_Table::Ins_Record(int iRecord, CSG_Table_Record *pCopy)
 //---------------------------------------------------------
 bool CSG_Table::Del_Record(int iRecord)
 {
-	int		i, j;
-
 	if( iRecord >= 0 && iRecord < m_nRecords )
 	{
+		int		i;
+
 		delete(m_Records[iRecord]);
 
 		m_nRecords--;
 
-		for(i=iRecord, j=iRecord+1; i<m_nRecords; i++, j++)
+		for(i=iRecord; i<m_nRecords; i++)
 		{
 			m_Records[i]			= m_Records[i + 1];
 			m_Records[i]->m_Index	= i;
@@ -1006,6 +1110,20 @@ bool CSG_Table::Set_Index(int Field_1, TSG_Table_Index_Order Order_1, int Field_
 		_Index_Destroy();
 	}
 
+	//-----------------------------------------------------
+	if( Get_Selection_Count() > 0 )
+	{
+		for(size_t i=0, n=0; i<(size_t)m_nRecords && n<Get_Selection_Count(); i++)
+		{
+			CSG_Table_Record	*pRecord	= Get_Record_byIndex(i);
+
+			if( pRecord && pRecord->is_Selected() )
+			{
+				_Set_Selection(n++, pRecord->Get_Index());
+			}
+		}
+	}
+
 	return( is_Indexed() );
 }
 
@@ -1054,9 +1172,9 @@ void CSG_Table::_Index_Create(void)
 			jstack	= 0;
 
 	//-----------------------------------------------------
-	if( m_Index == NULL )
+	if( m_Index == NULL || m_nBuffer < m_nRecords )
 	{
-		m_Index	= (int *)SG_Malloc(m_nBuffer * sizeof(int));
+		m_Index	= (int *)SG_Realloc(m_Index, (m_nBuffer < m_nRecords ? m_nRecords : m_nBuffer) * sizeof(int));
 	}
 
 	for(j=0; j<m_nRecords; j++)
@@ -1196,15 +1314,16 @@ inline int CSG_Table::_Index_Compare(int a, int b, int Field)
 	switch( m_Field_Type[m_Index_Field[Field]] )
 	{
 	case SG_DATATYPE_String:
+	case SG_DATATYPE_Date:
 		Result	= SG_STR_CMP(
-					m_Records[a]->asString(m_Index_Field[Field]),
-					m_Records[b]->asString(m_Index_Field[Field])
+				  Get_Record(a)->asString(m_Index_Field[Field]),
+				  Get_Record(b)->asString(m_Index_Field[Field])
 				);
 		break;
 
 	default:
-		Result	= m_Records[a]->asDouble(m_Index_Field[Field])
-				- m_Records[b]->asDouble(m_Index_Field[Field]);
+		Result	= Get_Record(a)->asDouble(m_Index_Field[Field])
+				- Get_Record(b)->asDouble(m_Index_Field[Field]);
 		break;
 	}
 
