@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: table.h 2064 2014-03-21 13:20:57Z oconrad $
+ * Version $Id: table.h 2713 2015-12-04 11:50:53Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -125,6 +125,7 @@ TSG_Table_Index_Order;
 class SAGA_API_DLL_EXPORT CSG_Table_Record
 {
 	friend class CSG_Table;
+	friend class CSG_PointCloud;
 
 public:
 
@@ -156,8 +157,8 @@ public:
 	short						asShort			(const CSG_String &Field)	const	{	return( (short  )asInt   ( Field) );	}
 	int							asInt			(int              iField)	const;
 	int							asInt			(const CSG_String &Field)	const;
-	sLong						asLong			(int              iField)	const	{	return( (sLong  )asDouble(iField) );	}
-	sLong						asLong			(const CSG_String &Field)	const	{	return( (sLong  )asDouble( Field) );	}
+	sLong						asLong			(int              iField)	const;
+	sLong						asLong			(const CSG_String &Field)	const;
 	float						asFloat			(int              iField)	const	{	return( (float  )asDouble(iField) );	}
 	float						asFloat			(const CSG_String &Field)	const	{	return( (float  )asDouble( Field) );	}
 	double						asDouble		(int              iField)	const;
@@ -223,8 +224,8 @@ public:
 									CSG_Table			(const CSG_String &File_Name, TSG_Table_File_Type Format = TABLE_FILETYPE_Undefined);
 	bool							Create				(const CSG_String &File_Name, TSG_Table_File_Type Format = TABLE_FILETYPE_Undefined);
 
-									CSG_Table			(const CSG_String &File_Name, TSG_Table_File_Type Format, const SG_Char *Separator);
-	bool							Create				(const CSG_String &File_Name, TSG_Table_File_Type Format, const SG_Char *Separator);
+									CSG_Table			(const CSG_String &File_Name, TSG_Table_File_Type Format, const SG_Char Separator);
+	bool							Create				(const CSG_String &File_Name, TSG_Table_File_Type Format, const SG_Char Separator);
 
 									CSG_Table			(CSG_Table *pTemplate);
 	bool							Create				(CSG_Table *pTemplate);
@@ -239,8 +240,9 @@ public:
 	virtual bool					Assign				(CSG_Data_Object *pSource);
 	bool							Assign_Values		(CSG_Table *pTable);
 
+	bool							Load				(const CSG_String &File_Name, int Format, SG_Char Separator);
 	virtual bool					Save				(const CSG_String &File_Name, int Format = 0);
-	virtual bool					Save				(const CSG_String &File_Name, int Format, const SG_Char *Separator);
+	virtual bool					Save				(const CSG_String &File_Name, int Format, SG_Char Separator);
 	bool							Serialize			(CSG_File &Stream, bool bSave);
 
 	//-----------------------------------------------------
@@ -260,9 +262,11 @@ public:
 	bool							Set_Field_Name		(int iField, const SG_Char *Name);
 	bool							Set_Field_Type		(int iField, TSG_Data_Type  Type);
 
+	sLong							Get_N				(int iField)	const	{	return( _Stats_Update(iField) ? m_Field_Stats[iField]->Get_Count()    : 0   );	}
 	double							Get_Minimum			(int iField)	const	{	return( _Stats_Update(iField) ? m_Field_Stats[iField]->Get_Minimum()  : 0.0 );	}
 	double							Get_Maximum			(int iField)	const	{	return( _Stats_Update(iField) ? m_Field_Stats[iField]->Get_Maximum()  : 0.0 );	}
 	double							Get_Range			(int iField)	const	{	return( _Stats_Update(iField) ? m_Field_Stats[iField]->Get_Range()    : 0.0 );	}
+	double							Get_Sum				(int iField)	const	{	return( _Stats_Update(iField) ? m_Field_Stats[iField]->Get_Sum()      : 0.0 );	}
 	double							Get_Mean			(int iField)	const	{	return( _Stats_Update(iField) ? m_Field_Stats[iField]->Get_Mean()     : 0.0 );	}
 	double							Get_StdDev			(int iField)	const	{	return( _Stats_Update(iField) ? m_Field_Stats[iField]->Get_StdDev()   : 0.0 );	}
 	double							Get_Variance		(int iField)	const	{	return( _Stats_Update(iField) ? m_Field_Stats[iField]->Get_Variance() : 0.0 );	}
@@ -306,8 +310,9 @@ public:
 	virtual void					Set_Modified		(bool bModified = true);
 
 	//-----------------------------------------------------
-	int								Get_Selection_Count	(void)			const	{	return( m_nSelected );	}
-	virtual CSG_Table_Record *		Get_Selection		(int Index = 0)	const	{	return( Index >= 0 && Index < m_nSelected ? Get_Record(m_Selected[Index]) : NULL );	}
+	size_t							Get_Selection_Count	(void)				const	{	return( m_Selection.Get_Size() );	}
+	size_t							Get_Selection_Index	(size_t Index = 0)	const	{	return( Index < m_Selection.Get_Size() ? *((size_t *)m_Selection.Get_Entry(Index)) : (size_t)Get_Count() );	}
+	virtual CSG_Table_Record *		Get_Selection		(size_t Index = 0)	const	{	return( Index < m_Selection.Get_Size() ? Get_Record((int)Get_Selection_Index(Index)) : NULL );	}
 
 	virtual bool					is_Selected			(int iRecord)	const	{	return( iRecord >= 0 && iRecord < m_nRecords ? m_Records[iRecord]->is_Selected() : false );	}
 
@@ -330,7 +335,7 @@ public:
 
 protected:
 
-	int								m_nFields, m_nRecords, m_nBuffer, m_nSelected;
+	int								m_nFields, m_nRecords, m_nBuffer;
 
 	TSG_Data_Type					*m_Field_Type;
 
@@ -338,11 +343,17 @@ protected:
 
 	CSG_Simple_Statistics			**m_Field_Stats;
 
+	CSG_Array						m_Selection;
+
 
 	virtual void					_On_Construction	(void);
 
 	virtual CSG_Table_Record *		_Get_New_Record		(int Index);
 
+	bool							_Add_Selection		(size_t iRecord);
+	bool							_Set_Selection		(size_t iRecord, size_t Index);
+	bool							_Del_Selection		(size_t iRecord);
+
 	bool							_Stats_Invalidate	(void)			const;
 	bool							_Stats_Invalidate	(int iField)	const;
 	virtual bool					_Stats_Update		(int iField)	const;
@@ -350,7 +361,7 @@ protected:
 
 private:
 
-	int								*m_Index, m_Index_Field[3], *m_Selected;
+	int								*m_Index, m_Index_Field[3];
 
 	TSG_Table_Index_Order			m_Index_Order[3];
 
@@ -362,9 +373,11 @@ private:
 	bool							_Inc_Array			(void);
 	bool							_Dec_Array			(void);
 
-	bool							_Load				(const CSG_String &File_Name, TSG_Table_File_Type Format, const SG_Char *Separator);
-	bool							_Load_Text			(const CSG_String &File_Name, bool bHeadline, const SG_Char *Separator);
-	bool							_Save_Text			(const CSG_String &File_Name, bool bHeadline, const SG_Char *Separator);
+	int								_Load_Text_Trim		(CSG_String &Text, const SG_Char Separator);
+
+	bool							_Load_Text			(const CSG_String &File_Name, bool bHeadline, const SG_Char Separator);
+	bool							_Save_Text			(const CSG_String &File_Name, bool bHeadline, const SG_Char Separator);
+
 	bool							_Load_DBase			(const CSG_String &File_Name);
 	bool							_Save_DBase			(const CSG_String &File_Name);
 
