@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: table_io.cpp 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: table_io.cpp 2690 2015-11-17 12:08:15Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -74,20 +74,13 @@
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CSG_Table::_Load(const CSG_String &File_Name, TSG_Table_File_Type Format, const SG_Char *Separator)
+bool CSG_Table::Load(const CSG_String &File_Name, int Format, SG_Char Separator)
 {
 	if( !SG_File_Exists(File_Name) )
 	{
 		return( false );
 	}
 
-	bool		bResult;
-	CSG_String	fName, sSeparator(Separator && *Separator ? Separator : SG_T("\t"));
-
-	Destroy();
-
-	SG_UI_Msg_Add(CSG_String::Format(SG_T("%s: %s..."), _TL("Load table"), File_Name.c_str()), true);
-
 	//-----------------------------------------------------
 	if( Format == TABLE_FILETYPE_Undefined )
 	{
@@ -95,58 +88,30 @@ bool CSG_Table::_Load(const CSG_String &File_Name, TSG_Table_File_Type Format, c
 		{
 			Format	= TABLE_FILETYPE_DBase;
 		}
-		else if( SG_File_Cmp_Extension(File_Name, SG_T("csv")) )
+		else
 		{
 			Format	= TABLE_FILETYPE_Text;
 
-			if( !Separator || *Separator == '\0' )
+			if( Separator == '\0' )
 			{
-				sSeparator	= SG_T(",");	// comma separated values
+				Separator	= SG_File_Cmp_Extension(File_Name, SG_T("csv")) ? ',' : '\t';	// comma separated values or tab spaced text
 			}
 		}
-		else //if( SG_File_Cmp_Extension(File_Name, SG_T("txt")) )
-		{
-			Format	= TABLE_FILETYPE_Text;
-		}
 	}
 
 	//-----------------------------------------------------
 	switch( Format )
 	{
-	case TABLE_FILETYPE_Text:
-		bResult	= _Load_Text (File_Name, true , sSeparator);
-		break;
+	case TABLE_FILETYPE_Text: default:
+		return( _Load_Text (File_Name, true , Separator) );
 
 	case TABLE_FILETYPE_Text_NoHeadLine:
-		bResult	= _Load_Text (File_Name, false, sSeparator);
-		break;
+		return( _Load_Text (File_Name, false, Separator) );
 
 	case TABLE_FILETYPE_DBase:
-		bResult	= _Load_DBase(File_Name);
-		break;
-
-	default:
-		bResult	= false;
+		return( _Load_DBase(File_Name) );
 	}
 
-	//-----------------------------------------------------
-	if( bResult )
-	{
-		Set_Modified(false);
-
-		Set_Update_Flag();
-
-		Set_File_Name(File_Name);
-
-		Load_MetaData(File_Name);
-
-		SG_UI_Msg_Add(_TL("okay"), false, SG_UI_MSG_STYLE_SUCCESS);
-
-		return( true );
-	}
-
-	SG_UI_Msg_Add(_TL("failed"), false, SG_UI_MSG_STYLE_FAILURE);
-
 	return( false );
 }
 
@@ -157,12 +122,9 @@ bool CSG_Table::Save(const CSG_String &File_Name, int Format)
 }
 
 //---------------------------------------------------------
-bool CSG_Table::Save(const CSG_String &File_Name, int Format, const SG_Char *Separator)
+bool CSG_Table::Save(const CSG_String &File_Name, int Format, SG_Char Separator)
 {
-	bool		bResult;
-	CSG_String	sSeparator(Separator && *Separator ? Separator : SG_T("\t"));
-
-	SG_UI_Msg_Add(CSG_String::Format(SG_T("%s: %s..."), _TL("Save table"), File_Name.c_str()), true);
+	SG_UI_Msg_Add(CSG_String::Format("%s: %s...", _TL("Save table"), File_Name.c_str()), true);
 
 	//-----------------------------------------------------
 	if( Format <= TABLE_FILETYPE_Undefined || Format > TABLE_FILETYPE_DBase )
@@ -171,30 +133,28 @@ bool CSG_Table::Save(const CSG_String &File_Name, int Format, const SG_Char *Sep
 		{
 			Format	= TABLE_FILETYPE_DBase;
 		}
-		else if( SG_File_Cmp_Extension(File_Name, SG_T("csv")) )
+		else
 		{
 			Format	= TABLE_FILETYPE_Text;
 
-			if( Separator == NULL || *Separator == '\0' )
+			if( Separator == '\0' )
 			{
-				sSeparator	= SG_T(",");	// comma separated values
+				Separator	= SG_File_Cmp_Extension(File_Name, SG_T("csv")) ? ',' : '\t';	// comma separated values or tab spaced text
 			}
 		}
-		else //if( SG_File_Cmp_Extension(File_Name, SG_T("txt")) )
-		{
-			Format	= TABLE_FILETYPE_Text;
-		}
 	}
 
 	//-----------------------------------------------------
+	bool	bResult	= false;
+
 	switch( Format )
 	{
 	case TABLE_FILETYPE_Text:
-		bResult	= _Save_Text (File_Name, true , sSeparator);
+		bResult	= _Save_Text (File_Name, true , Separator);
 		break;
 
 	case TABLE_FILETYPE_Text_NoHeadLine:
-		bResult	= _Save_Text (File_Name, false, sSeparator);
+		bResult	= _Save_Text (File_Name, false, Separator);
 		break;
 
 	case TABLE_FILETYPE_DBase:
@@ -211,7 +171,7 @@ bool CSG_Table::Save(const CSG_String &File_Name, int Format, const SG_Char *Sep
 
 		Set_File_Type(Format);
 
-		Set_File_Name(File_Name);
+		Set_File_Name(File_Name, true);
 
 		Save_MetaData(File_Name);
 
@@ -233,10 +193,31 @@ bool CSG_Table::Save(const CSG_String &File_Name, int Format, const SG_Char *Sep
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CSG_Table::_Load_Text(const CSG_String &File_Name, bool bHeadline, const SG_Char *Separator)
+int CSG_Table::_Load_Text_Trim(CSG_String &String, const SG_Char Separator)
 {
-	int			i, iField;
-	long		fLength;
+	for(size_t i=0; i<String.Length(); i++)
+	{
+		SG_Char	c	= String[i];
+
+		if( c == Separator || (c != ' ' && c != '\t' && c != '\n' && c != '\v' && c != '\f' && c != '\r') )
+		{
+			if( i > 0 )
+			{
+				String	= String.Right(String.Length() - i);
+			}
+
+			return( i );
+		}
+	}
+
+	return( 0 );
+}
+
+//---------------------------------------------------------
+bool CSG_Table::_Load_Text(const CSG_String &File_Name, bool bHeadline, const SG_Char Separator)
+{
+	int			iField;
+	sLong		fLength;
 	CSG_String	sLine, sField;
 	CSG_File	Stream;
 	CSG_Table	Table;
@@ -258,30 +239,28 @@ bool CSG_Table::_Load_Text(const CSG_String &File_Name, bool bHeadline, const SG
 	}
 
 	//-----------------------------------------------------
-	sLine	+= Separator;
+	_Load_Text_Trim(sLine, Separator);
 
-	while( (i = sLine.Find(Separator)) >= 0 )
+	while( !sLine.is_Empty() )
 	{
-		sField.Clear();
-
-		if( bHeadline )
+		if( sLine[0] == '\"' )	// value in quotas
 		{
-			sField	= sLine.Left(i);
-
-			if( sField[0] == SG_T('\"') && sField[(int)(sField.Length() - 1)] == SG_T('\"') )	// remove quota
-			{
-				sField	= sField.AfterFirst('\"').BeforeLast('\"');
-			}
+			sField	= sLine.AfterFirst('\"').BeforeFirst('\"');
+			sLine	= sLine.AfterFirst('\"').AfterFirst ('\"');
+		}
+		else
+		{
+			sField	= sLine.BeforeFirst(Separator);
 		}
 
-		if( sField.Length() == 0 )
+		sLine	= sLine.AfterFirst(Separator);	_Load_Text_Trim(sLine, Separator);
+
+		if( !bHeadline || sField.Length() == 0 )
 		{
-			sField.Printf(SG_T("F%02d"), Table.Get_Field_Count() + 1);
+			sField.Printf("F%02d", Table.Get_Field_Count() + 1);
 		}
 
 		Table.Add_Field(sField, SG_DATATYPE_String);
-
-		sLine.Remove(0, i + 1);
 	}
 
 	//-----------------------------------------------------
@@ -297,47 +276,44 @@ bool CSG_Table::_Load_Text(const CSG_String &File_Name, bool bHeadline, const SG
 		Stream.Seek_Start();
 	}
 
-	while( Stream.Read_Line(sLine) && sLine.Length() > 0 && SG_UI_Process_Set_Progress(Stream.Tell(), fLength) )
+	while( Stream.Read_Line(sLine) && sLine.Length() > 0 && SG_UI_Process_Set_Progress((double)Stream.Tell(), (double)fLength) )
 	{
 		CSG_Table_Record	*pRecord	= Table.Add_Record();
 
-		sLine	+= Separator;
+		_Load_Text_Trim(sLine, Separator);
 
-		for(iField=0; iField<Table.Get_Field_Count(); iField++)
+		for(iField=0; iField<Table.Get_Field_Count() && !sLine.is_Empty(); iField++)
 		{
-			if( (i = sLine.Find(Separator)) >= 0 )
+			if( sLine[0] == '\"' )	// value in quotas
 			{
-				sField	= sLine.Left(i);
+				sField	= sLine.AfterFirst('\"').BeforeFirst('\"');
+				sLine	= sLine.AfterFirst('\"').AfterFirst ('\"');
 
-				if( sField[0] == SG_T('\"') && sField[(int)(sField.Length() - 1)] == SG_T('\"') )	// remove quota
-				{
-					sField	= sField.AfterFirst('\"').BeforeLast('\"');
-
-					Type[iField]	= SG_DATATYPE_String;
-				}
-
-				if( Type[iField] != SG_DATATYPE_String && sField.Length() > 0 )
-				{
-					double	Value;
-
-					if( SG_SSCANF(sField, SG_T("%lf"), &Value) != 1 )
-					{
-						Type[iField]	= SG_DATATYPE_String;
-					}
-					else if( Type[iField] != SG_DATATYPE_Double && Value - (int)Value != 0.0 )
-					{
-						Type[iField]	= SG_DATATYPE_Double;
-					}
-				}
-
-				pRecord->Set_Value(iField, sField);
-
-				sLine.Remove(0, i + 1);
+				Type[iField]	= SG_DATATYPE_String;
 			}
 			else
 			{
-				break;
+				sField	= sLine.BeforeFirst(Separator);
 			}
+
+			sLine	= sLine.AfterFirst(Separator);	_Load_Text_Trim(sLine, Separator);
+
+			//---------------------------------------------
+			if( Type[iField] != SG_DATATYPE_String && !sField.is_Empty() )
+			{
+				double	Value;
+
+				if( sField.asDouble(Value) == false )
+				{
+					Type[iField]	= SG_DATATYPE_String;
+				}
+				else if( Type[iField] != SG_DATATYPE_Double && Value - (int)Value != 0.0 )
+				{
+					Type[iField]	= SG_DATATYPE_Double;
+				}
+			}
+
+			pRecord->Set_Value(iField, sField);
 		}
 	}
 
@@ -380,7 +356,7 @@ bool CSG_Table::_Load_Text(const CSG_String &File_Name, bool bHeadline, const SG
 }
 
 //---------------------------------------------------------
-bool CSG_Table::_Save_Text(const CSG_String &File_Name, bool bHeadline, const SG_Char *Separator)
+bool CSG_Table::_Save_Text(const CSG_String &File_Name, bool bHeadline, const SG_Char Separator)
 {
 	int			iField, iRecord;
 	CSG_File	Stream;
@@ -391,7 +367,7 @@ bool CSG_Table::_Save_Text(const CSG_String &File_Name, bool bHeadline, const SG
 		{
 			for(iField=0; iField<Get_Field_Count(); iField++)
 			{
-				Stream.Printf(SG_T("%s%s"), Get_Field_Name(iField), iField < Get_Field_Count() - 1 ? Separator : SG_T("\n"));
+				Stream.Printf(SG_T("%s%c"), Get_Field_Name(iField), iField < Get_Field_Count() - 1 ? Separator : '\n');
 			}
 
 			for(iRecord=0; iRecord<Get_Record_Count() && SG_UI_Process_Set_Progress(iRecord, Get_Record_Count()); iRecord++)
@@ -413,7 +389,7 @@ bool CSG_Table::_Save_Text(const CSG_String &File_Name, bool bHeadline, const SG
 						}
 					}
 
-					Stream.Printf(SG_T("%s"), iField < Get_Field_Count() - 1 ? Separator : SG_T("\n"));
+					Stream.Printf(SG_T("%c"), iField < Get_Field_Count() - 1 ? Separator : '\n');
 				}
 			}
 
