@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: tin.cpp 2194 2014-08-14 08:46:34Z oconrad $
+ * Version $Id: tin.cpp 2558 2015-08-19 14:56:36Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -179,7 +179,7 @@ bool CSG_TIN::Create(const CSG_String &File_Name)
 		Get_History().Add_Child(_TL("Created from file"), File_Name);
 		Get_History().Add_Children(Shapes.Get_History());
 
-		Set_File_Name(File_Name);
+		Set_File_Name(File_Name, true);
 		Set_Modified(false);
 		Set_Update_Flag();
 
@@ -392,7 +392,7 @@ bool CSG_TIN::Save(const CSG_String &File_Name, int Format)
 	{
 		Set_Modified(false);
 
-		Set_File_Name(File_Name);
+		Set_File_Name(File_Name, true);
 	}
 
 	return( bResult );
