@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: tin.h 911 2011-02-14 16:38:15Z reklov_w $
+ * Version $Id: tin.h 1921 2014-01-09 10:24:11Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -37,7 +37,7 @@
 // You should have received a copy of the GNU Lesser     //
 // General Public License along with this program; if    //
 // not, write to the Free Software Foundation, Inc.,     //
-// 59 Temple Place - Suite 330, Boston, MA 02111-1307,   //
+// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
 // USA.                                                  //
 //                                                       //
 //-------------------------------------------------------//
@@ -176,9 +176,12 @@ public:
 
 	double						Get_Area				(void)			{	return( m_Area );	}
 
-	bool						is_Containing			(const TSG_Point &Point);
+	bool						is_Containing			(const TSG_Point &p);
 	bool						is_Containing			(double x, double y);
 
+	bool						Get_Value				(int zField, const TSG_Point &p, double &z);
+	bool						Get_Value				(int zField, double x, double y, double &z);
+
 	bool						Get_Gradient			(int zField, double &Decline, double &Azimuth);
 
 	void						Get_CircumCircle		(TSG_Point &Point, double &Radius)	{	Point	= m_Center;	Radius	= m_Radius;	}
