@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: tin_elements.cpp 911 2011-02-14 16:38:15Z reklov_w $
+ * Version $Id: tin_elements.cpp 1921 2014-01-09 10:24:11Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -37,7 +37,7 @@
 // You should have received a copy of the GNU Lesser     //
 // General Public License along with this program; if    //
 // not, write to the Free Software Foundation, Inc.,     //
-// 59 Temple Place - Suite 330, Boston, MA 02111-1307,   //
+// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
 // USA.                                                  //
 //                                                       //
 //-------------------------------------------------------//
@@ -378,6 +378,33 @@ bool CSG_TIN_Triangle::is_Containing(double x, double y)
 	return( false );
 }
 
+//---------------------------------------------------------
+bool CSG_TIN_Triangle::Get_Value(int zField, const TSG_Point &p, double &z)
+{
+	return( Get_Value(zField, p.x, p.y, z) );
+}
+
+bool CSG_TIN_Triangle::Get_Value(int zField, double x, double y, double &z)
+{
+	CSG_Vector	B, Z(3);
+	CSG_Matrix	M(3, 3), Mt;
+
+	for(int i=0; i<3; i++)
+	{
+		M[i][0]	= 1.0;
+		M[i][1]	= m_Nodes[i]->Get_X();
+		M[i][2]	= m_Nodes[i]->Get_Y();
+		Z[i]	= m_Nodes[i]->asDouble(zField);
+	}
+
+	Mt	= M.Get_Transpose();
+	B	= (Mt * M).Get_Inverse() * (Mt * Z);
+
+	z	= B[0] + B[1] * x + B[2] * y;
+
+	return( true );
+}
+
 //---------------------------------------------------------
 bool CSG_TIN_Triangle::Get_Gradient(int zField, double &Decline, double &Azimuth)
 {
