@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: module_library.cpp 1994 2014-02-14 10:55:29Z oconrad $
+ * Version $Id: module_library.cpp 2708 2015-12-01 16:16:14Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -75,14 +75,13 @@
 //---------------------------------------------------------
 CCMD_Module::CCMD_Module(void)
 {
+	m_pLibrary	= NULL;
 	m_pModule	= NULL;
 }
 
-CCMD_Module::CCMD_Module(CSG_Module *pModule)
+CCMD_Module::CCMD_Module(CSG_Module_Library *pLibrary, CSG_Module *pModule)
 {
-	m_pModule	= NULL;
-
-	Create(pModule);
+	Create(pLibrary, pModule);
 }
 
 //---------------------------------------------------------
@@ -99,19 +98,17 @@ CCMD_Module::~CCMD_Module(void)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CCMD_Module::Create(CSG_Module *pModule)
+bool CCMD_Module::Create(CSG_Module_Library *pLibrary, CSG_Module *pModule)
 {
 	Destroy();
 
-	m_CMD.SetSwitchChars(SG_T("-"));
-
-	if( (m_pModule = pModule) != NULL )
+	if( (m_pLibrary = pLibrary) != NULL && (m_pModule = pModule) != NULL )
 	{
-		_Set_Parameters(m_pModule->Get_Parameters(), false);
+		_Set_Parameters(m_pModule->Get_Parameters());
 
 		for(int i=0; i<m_pModule->Get_Parameters_Count(); i++)
 		{
-			_Set_Parameters(m_pModule->Get_Parameters(i), true);
+			_Set_Parameters(m_pModule->Get_Parameters(i));
 		}
 
 		return( true );
@@ -123,9 +120,28 @@ bool CCMD_Module::Create(CSG_Module *pModule)
 //---------------------------------------------------------
 void CCMD_Module::Destroy(void)
 {
+	m_pLibrary	= NULL;
 	m_pModule	= NULL;
 
 	m_CMD.Reset();
+	m_CMD.SetSwitchChars("-");
+}
+
+//---------------------------------------------------------
+void CCMD_Module::Usage(void)
+{
+	if( m_pLibrary && m_pModule )
+	{
+		CMD_Print("");
+
+		wxString	sUsage = wxString::Format(SG_T("Usage: saga_cmd %s %s %s"),
+			m_pLibrary->Get_Library_Name().c_str(),
+			m_pModule ->Get_ID          ().c_str(),
+			m_CMD.GetUsageString().AfterFirst(' ').AfterFirst(' ')
+		);
+
+		SG_PRINTF(sUsage);
+	}
 }
 
 
@@ -136,21 +152,17 @@ void CCMD_Module::Destroy(void)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CCMD_Module::Execute(CSG_String sLibName, int argc, char *argv[])
+bool CCMD_Module::Execute(int argc, char *argv[])
 {
-	int		i;
-
 	//-----------------------------------------------------
-	if( !m_pModule )
+	if( !m_pLibrary || !m_pModule )
 	{
 		return( false );
 	}
 
 	if( argc <= 1 )
 	{
-		wxString sUsage = m_CMD.GetUsageString();
-		sUsage = wxString::Format(SG_T("Usage: saga_cmd %s %d %s"), sLibName.c_str(), m_pModule->Get_ID(), sUsage.AfterFirst(' ').AfterFirst(' '));
-		SG_PRINTF(sUsage);
+		Usage();
 
 		return( false );
 	}
@@ -161,39 +173,45 @@ bool CCMD_Module::Execute(CSG_String sLibName, int argc, char *argv[])
 	// We can't do it this way (passing argv as char**) because then we use an
 	// overload of the method which (re-)sets the locale from the current
 	// enviromment; in order to prevent this, we use wxString overload
-
-	wxString	sCmdLine;
-
-	for(i=1; i<argc; i++)
 	{
-		wxString	sTmp = argv[i];
+		wxString	sCmdLine;
 
-		sCmdLine += wxString::Format(SG_T("\"%s\" "), sTmp.c_str());
+		for(int i=1; i<argc; i++)
+		{
+			sCmdLine	+= wxString(i == 1 ? "\"" : " \"") + argv[i] + "\"";
+		}
+
+		m_CMD.SetCmdLine(sCmdLine);
 	}
 
-	m_CMD.SetCmdLine(sCmdLine);
+	if( m_CMD.Parse(false) != 0 )
+	{
+		Usage();
+
+		return( false );
+	}
 
 	//-----------------------------------------------------
-	bool	bResult	= _Get_Parameters(m_pModule->Get_Parameters());
+	int		i;
+
+	bool	bResult	= _Get_Parameters(m_pModule->Get_Parameters(), true);
 
 	for(i=0; bResult && i<m_pModule->Get_Parameters_Count(); i++)
 	{
-		_Get_Parameters(m_pModule->Get_Parameters(i));
+		bResult	= _Get_Parameters(m_pModule->Get_Parameters(i), true);
 	}
 
 	if( !bResult )
 	{
-		CMD_Print("");
+		Usage();
 
-		wxString sUsage = m_CMD.GetUsageString();
-		sUsage = wxString::Format(SG_T("Usage: saga_cmd %s %d %s"), sLibName.c_str(), m_pModule->Get_ID(), sUsage.AfterFirst(' ').AfterFirst(' '));
-		SG_PRINTF(sUsage);
+		return( false );
 	}
 
 	//-----------------------------------------------------
 	CMD_Set_Module(this);
 
-	if( bResult && m_pModule->On_Before_Execution() )
+	if( m_pModule->On_Before_Execution() )
 	{
 		bResult	= m_pModule->Execute();
 
@@ -257,7 +275,7 @@ wxString CCMD_Module::_Get_ID(CSG_Parameter *pParameter, const wxString &Modifie
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CCMD_Module::_Set_Parameters(CSG_Parameters *pParameters, bool bOptional)
+bool CCMD_Module::_Set_Parameters(CSG_Parameters *pParameters)
 {
 	if( !pParameters )
 	{
@@ -269,18 +287,20 @@ bool CCMD_Module::_Set_Parameters(CSG_Parameters *pParameters, bool bOptional)
 	{
 		CSG_Parameter	*pParameter	= pParameters->Get_Parameter(i);
 
+		if( pParameter->do_UseInCMD() == false )
+		{
+			continue;
+		}
+
 		wxString	Description	= pParameter->Get_Description(
 			PARAMETER_DESCRIPTION_NAME|PARAMETER_DESCRIPTION_TYPE|PARAMETER_DESCRIPTION_PROPERTIES, SG_T("\n\t")
 		).c_str();
 
-		Description.Replace(wxT("\xb2"), wxT("2"));	// unicode problem: quick'n'dirty bug fix, to be replaced
+		Description.Replace(wxT("\xb"), wxT(""));	// unicode problem: quick'n'dirty bug fix, to be replaced
 
 		if( pParameter->is_Input() || pParameter->is_Output() )
 		{
-			m_CMD.AddOption(_Get_ID(pParameter), wxEmptyString, Description, wxCMD_LINE_VAL_STRING, wxCMD_LINE_NEEDS_SEPARATOR
-			| (pParameter->is_Optional() || pParameter->is_Output() || bOptional
-			  ? wxCMD_LINE_PARAM_OPTIONAL : wxCMD_LINE_OPTION_MANDATORY
-			));
+			m_CMD.AddOption(_Get_ID(pParameter), wxEmptyString, Description, wxCMD_LINE_VAL_STRING, wxCMD_LINE_NEEDS_SEPARATOR|wxCMD_LINE_PARAM_OPTIONAL);
 		}
 
 		else if( pParameter->is_Option() && !pParameter->is_Information() )
@@ -291,11 +311,11 @@ bool CCMD_Module::_Set_Parameters(CSG_Parameters *pParameters, bool bOptional)
 				break;
 
 			case PARAMETER_TYPE_Parameters:
-				_Set_Parameters(pParameter->asParameters(), true);
+				_Set_Parameters(pParameter->asParameters());
 				break;
 
 			case PARAMETER_TYPE_Bool:
-				m_CMD.AddSwitch(_Get_ID(pParameter), wxEmptyString, Description, wxCMD_LINE_PARAM_OPTIONAL);
+				m_CMD.AddOption(_Get_ID(pParameter), wxEmptyString, Description, wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL);
 				break;
 
 			case PARAMETER_TYPE_Int:
@@ -353,17 +373,29 @@ bool CCMD_Module::_Set_Parameters(CSG_Parameters *pParameters, bool bOptional)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CCMD_Module::_Get_Parameters(CSG_Parameters *pParameters)
+bool CCMD_Module::_Get_Parameters(CSG_Parameters *pParameters, bool bInitialize)
 {
-	if( !pParameters || m_CMD.Parse(false) != 0 )
+	if( !pParameters )
 	{
 		return( false );
 	}
 
-	//-----------------------------------------------------
-	int		i;
+	if( bInitialize )
+	{
+		pParameters->Restore_Defaults();
+	}
 
-	for(i=0; i<pParameters->Get_Count(); i++)
+	//-----------------------------------------------------
+	return( _Get_Options(pParameters, bInitialize)	// options might 'disable' some input, so check all options before the input is checked!
+		&&  _Get_Input  (pParameters)
+		&&  _Get_Options(pParameters, false)		// setting input might have reset some options, so check options one more time!
+	);
+}
+
+//---------------------------------------------------------
+bool CCMD_Module::_Get_Options(CSG_Parameters *pParameters, bool bInitialize)
+{
+	for(int i=0; i<pParameters->Get_Count(); i++)
 	{
 		CSG_Parameter	*pParameter	= pParameters->Get_Parameter(i);
 
@@ -392,11 +424,27 @@ bool CCMD_Module::_Get_Parameters(CSG_Parameters *pParameters)
 				break;
 
 			case PARAMETER_TYPE_Parameters:
-				_Get_Parameters(pParameter->asParameters());
+				_Get_Parameters(pParameter->asParameters(), bInitialize);
 				break;
 
 			case PARAMETER_TYPE_Bool:
-				pParameter->Set_Value(m_CMD.Found(_Get_ID(pParameter)) ? 1 : 0);
+				if( m_CMD.Found(_Get_ID(pParameter), &s) )
+				{
+					if(      (s.ToLong(&l) && l == 1) || !s.CmpNoCase("true" ) )
+					{
+						pParameter->Set_Value(1);
+					}
+					else if( (s.ToLong(&l) && l == 0) || !s.CmpNoCase("false") )
+					{
+						pParameter->Set_Value(0);
+					}
+					else
+					{
+						CMD_Print_Error(pParameter->Get_Name(), _TL("invalid boolean value ('0', '1', 'false', 'true')"));
+
+						return( false );
+					}
+				}
 				break;
 
 			case PARAMETER_TYPE_Int:
@@ -513,10 +561,16 @@ bool CCMD_Module::_Get_Parameters(CSG_Parameters *pParameters)
 		}
 	}
 
+	//-----------------------------------------------------
 	m_pModule->Update_Parameter_States();
 
-	//-----------------------------------------------------
-	for(i=0; i<pParameters->Get_Count(); i++)
+	return( true );
+}
+
+//---------------------------------------------------------
+bool CCMD_Module::_Get_Input(CSG_Parameters *pParameters)
+{
+	for(int i=0; i<pParameters->Get_Count(); i++)
 	{
 		CSG_Parameter	*pParameter	= pParameters->Get_Parameter(i);
 
@@ -584,7 +638,16 @@ bool CCMD_Module::_Load_Input(CSG_Parameter *pParameter)
 
 	if( !m_CMD.Found(_Get_ID(pParameter), &FileName) )
 	{
-		return( pParameter->is_Optional() );
+		if( !pParameter->is_Optional() )
+		{
+			wxString	Msg(wxString::Format(_TL("The value for the option '%s' must be specified."), _Get_ID(pParameter)));
+
+			CMD_Print_Error(&Msg);
+
+			return( false );
+		}
+
+		return( true );
 	}
 
 	if( pParameter->is_DataObject() )
@@ -596,7 +659,7 @@ bool CCMD_Module::_Load_Input(CSG_Parameter *pParameter)
 			return( false );
 		}
 
-		return( pParameter->Set_Value(SG_Get_Data_Manager().Find(&FileName)) );
+		return( pParameter->Set_Value(SG_Get_Data_Manager().Find(&FileName, false)) );
 	}
 
 	else if( pParameter->is_DataObject_List() )
@@ -607,17 +670,19 @@ bool CCMD_Module::_Load_Input(CSG_Parameter *pParameter)
 
 		do
 		{
-			FileName	= FileNames.BeforeFirst	(';');
-			FileNames	= FileNames.AfterFirst	(';');
+			FileName	= FileNames.BeforeFirst(';').Trim(false);
+			FileNames	= FileNames.AfterFirst (';');
 
 			if( !SG_Get_Data_Manager().Find(&FileName) )
 			{
 				SG_Get_Data_Manager().Add(&FileName);
 			}
 
-			pParameter->asList()->Add_Item(SG_Get_Data_Manager().Find(&FileName));
+			pParameter->asList()->Add_Item(SG_Get_Data_Manager().Find(&FileName, false));
 		}
 		while( FileNames.Length() > 0 );
+
+		pParameter->has_Changed();
 	}
 
 	return( true );
@@ -681,37 +746,36 @@ bool CCMD_Module::_Save_Output(CSG_Parameters *pParameters)
 			{
 				CSG_Strings	FileNames;
 
-				while( FileName.Length() > 0 )
+				while( !FileName.IsEmpty() && FileNames.Get_Count() < pParameter->asList()->Get_Count() )
 				{
-					CSG_String	s(&FileName); s = s.BeforeFirst(';');
+					wxString	s = FileName.BeforeFirst(';'); s.Trim(true); s.Trim(false);
 
-					if( s.Length() > 0 )
+					if( !s.IsEmpty() )
 					{
-						FileNames	+= s;
-						FileName	 = FileName.AfterFirst(';');
-					}
-					else
-					{
-						FileNames	+= &FileName;
-						FileName	.Clear();
+						FileNames	+= &s;
 					}
+
+					FileName = FileName.AfterFirst(';');
 				}
 
-				int	nFileNames	= pParameter->asList()->Get_Count() <= FileNames.Get_Count() ? FileNames.Get_Count() : FileNames.Get_Count() - 1;
-
-				for(int i=0; i<pParameter->asList()->Get_Count(); i++)
+				if( FileNames.Get_Count() > 0 )	// e.g.: GRIDS=" ;;"
 				{
-					if( i < nFileNames )
+					int	nFileNames	= pParameter->asList()->Get_Count() <= FileNames.Get_Count() ? FileNames.Get_Count() : FileNames.Get_Count() - 1;
+
+					for(int i=0; i<pParameter->asList()->Get_Count(); i++)
 					{
-						pParameter->asList()->asDataObject(i)->Save(FileNames[i]);
-					}
-					else
-					{
-						pParameter->asList()->asDataObject(i)->Save(CSG_String::Format(SG_T("%s_%0*d"),
-							FileNames[FileNames.Get_Count() - 1].c_str(),
-							SG_Get_Digit_Count(pParameter->asList()->Get_Count()),
-							1 + i - nFileNames
-						));
+						if( i < nFileNames )
+						{
+							pParameter->asList()->asDataObject(i)->Save(FileNames[i]);
+						}
+						else
+						{
+							pParameter->asList()->asDataObject(i)->Save(CSG_String::Format(SG_T("%s_%0*d"),
+								FileNames[nFileNames].c_str(),
+								SG_Get_Digit_Count(pParameter->asList()->Get_Count()),
+								1 + i - nFileNames
+							));
+						}
 					}
 				}
 			}
