@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: saga_cmd.cpp 2281 2014-10-09 15:49:41Z oconrad $
+ * Version $Id: saga_cmd.cpp 2675 2015-11-04 10:39:23Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -231,9 +231,7 @@ bool		Execute(int argc, char *argv[])
 		return( false );
 	}
 
-	if( argc == 2
-	||  (  (pModule = pLibrary->Get_Module(CSG_String(argv[2])        )) == NULL
-	    && (pModule = pLibrary->Get_Module(CSG_String(argv[2]).asInt())) == NULL) )
+	if( argc == 2 || (pModule = pLibrary->Get_Module(argv[2])) == NULL )
 	{
 		Print_Modules(pLibrary);
 
@@ -680,7 +678,8 @@ void		Print_Help		(void)
 	CMD_Print(
 		"under GNU General Public License (GPL)\n"
 		"\n"
-		"____________________________\n"
+		"_____________________________________________________________________________\n"
+		"\n"
 		"Usage:\n"
 		"\n"
 		"saga_cmd [-h, --help]\n"
@@ -688,11 +687,15 @@ void		Print_Help		(void)
 		"saga_cmd [-b, --batch]\n"
 		"saga_cmd [-d, --docs]\n"
 #ifdef _OPENMP
-		"saga_cmd [-f, --flags][=qrsilpxo][-s, --story][=#][-c, --cores][=#] <LIBRARY> <MODULE> <OPTIONS>\n"
-		"saga_cmd [-f, --flags][=qrsilpxo][-s, --story][=#][-c, --cores][=#] <SCRIPT>\n"
+		"saga_cmd [-f, --flags][=qrsilpxo][-s, --story][=#][-c, --cores][=#]\n"
+		"  <LIBRARY> <MODULE> <OPTIONS>\n"
+		"saga_cmd [-f, --flags][=qrsilpxo][-s, --story][=#][-c, --cores][=#]\n"
+		"  <SCRIPT>\n"
 #else
-		"saga_cmd [-f, --flags][=qrsilpxo][-s, --story][=#] <LIBRARY> <MODULE> <module specific options...>\n"
-		"saga_cmd [-f, --flags][=qrsilpxo][-s, --story][=#] <SCRIPT>\n"
+		"saga_cmd [-f, --flags][=qrsilpxo][-s, --story][=#]\n"
+		"  <LIBRARY> <MODULE> <OPTIONS>\n"
+		"saga_cmd [-f, --flags][=qrsilpxo][-s, --story][=#]\n"
+		"  <SCRIPT>\n"
 #endif
 		"\n"
 		"[-h], [--help]   : help on usage\n"
@@ -712,18 +715,19 @@ void		Print_Help		(void)
 		"  p              : load projections dictionary\n"
 		"  x              : use XML markups for synopses and messages\n"
 		"  o              : load old style naming\n"
-		"\n"
 		"<LIBRARY>        : name of the library\n"
 		"<MODULE>         : either name or index of the tool\n"
 		"<OPTIONS>        : tool specific options\n"
 		"<SCRIPT>         : saga cmd script file with one or more tool calls\n"
 		"\n"
-		"____________________________\n"
+		"_____________________________________________________________________________\n"
+		"\n"
 		"Example:\n"
 		"\n"
 		"  saga_cmd -f=s ta_lighting 0 -ELEVATION=c:\\dem.sgrd -SHADE=c:\\shade.sgrd\n"
 		"\n"
-		"____________________________\n"
+		"_____________________________________________________________________________\n"
+		"\n"
 		"Tool libraries in the \'modules\' subdirectory of the SAGA installation\n"
 		"will be loaded automatically. Additional directories can be specified\n"
 		"by adding the environment variable \'SAGA_MLB\' and let it point to one\n"
@@ -734,7 +738,19 @@ void		Print_Help		(void)
 		"script file. Calling saga_cmd with the option \'-b\' or \'--batch\' will\n"
 		"create an example of a DOS batch script file, which might be a good starting\n"
 		"point for the implementation of your own specific work flows.\n"
-		"____________________________\n"
+		"\n"
+		"_____________________________________________________________________________\n"
+		"\n"
+		"Please provide the following reference in your work if you are using SAGA:\n"
+		"\n"
+		"Conrad, O., Bechtel, B., Bock, M., Dietrich, H., Fischer, E., Gerlitz, L.,\n"
+		"  Wehberg, J., Wichmann, V., and Boehner, J. (2015):\n"
+		"  System for Automated Geoscientific Analyses (SAGA) v. 2.1.4.\n"
+		"  Geosci. Model Dev., 8, 1991-2007, doi:10.5194/gmd-8-1991-2015.\n"
+		"\n"
+		"  http://www.geosci-model-dev.net/8/1991/2015/gmd-8-1991-2015.html\n"
+		"\n"
+		"_____________________________________________________________________________\n"
 	);
 }
 
@@ -763,7 +779,7 @@ void		Create_Example	(void)
 	Stream.Printf(
 		"@ECHO OFF\n"
 		"\n"
-		"SET FLAGS=-f=q\n"
+		"SET FLAGS=-f=s\n"
 		"REM SET SAGA=.\n"
 		"REM SET SAGA_MLB=%%SAGA%%\\_private\n"
 		"REM PATH=PATH;%%SAGA%%\n"
@@ -773,13 +789,13 @@ void		Create_Example	(void)
 		"\n"
 		"ECHO ____________________________\n"
 		"ECHO create a Gaussian landscape\n"
-		"saga_cmd %%FLAGS%% recreations_fractals 5 -GRID=dem.sgrd -NX=400 -NY=400 -H=0.75\n"
+		"saga_cmd %%FLAGS%% garden_fractals 5 -GRID=dem.sgrd -NX=400 -NY=400 -H=0.75\n"
 		"GOTO :GO\n"
 		"\n"
 		":SRTM\n"
 		"ECHO ____________________________\n"
 		"ECHO import and project srtm (geotiff)\n"
-		"saga_cmd %%FLAGS%% io_gdal              0 -FILES=srtm.tif -GRIDS=srtm -TRANSFORM\n"
+		"saga_cmd %%FLAGS%% io_gdal              0 -FILES=srtm.tif -GRIDS=srtm -TRANSFORM=true\n"
 		"saga_cmd %%FLAGS%% pj_proj4             7 -SOURCE=srtm.sgrd -GET_USER_GRID=dem.sgrd -GET_USER_SIZE=1000.0 -SOURCE_PROJ=\"+proj=longlat +datum=WGS84\" -TARGET_PROJ=\"+proj=cea +datum=WGS84 +lat_ts=0\"\n"
 		"REM alternative method 4 might need to load projection data base with EPSG codes (use --flags=p)\n"
 		"REM saga_cmd -f=qp pj_proj4             4 -SOURCE=srtm.sgrd -GET_USER_GRID=dem.sgrd -GET_USER_SIZE=1000.0 -CRS_PROJ4=\"+proj=cea +datum=WGS84 +lat_ts=0\"\n"
@@ -787,13 +803,13 @@ void		Create_Example	(void)
 		":GO\n"
 		"ECHO ____________________________\n"
 		"ECHO create contour lines from DEM\n"
-		"saga_cmd %%FLAGS%% shapes_grid          5 -INPUT=dem.sgrd -CONTOUR=contour.shp -ZSTEP=100.0\n"
+		"saga_cmd %%FLAGS%% shapes_grid          5 -GRID=dem.sgrd -CONTOUR=contour.shp -ZSTEP=100.0\n"
 		"\n"
 		"ECHO ____________________________\n"
 		"ECHO do some terrain analysis\n"
 		"saga_cmd %%FLAGS%% ta_preprocessor      2 -DEM=dem.sgrd -DEM_PREPROC=dem.sgrd\n"
 		"saga_cmd %%FLAGS%% ta_lighting          0 -ELEVATION=dem.sgrd -SHADE=shade.sgrd -METHOD=0 -AZIMUTH=-45 -DECLINATION=45\n"
-		"saga_cmd %%FLAGS%% ta_morphometry       0 -ELEVATION=dem.sgrd -SLOPE=slope.sgrd -ASPECT=aspect.sgrd -HCURV=hcurv.sgrd -VCURV=vcurv.sgrd\n"
+		"saga_cmd %%FLAGS%% ta_morphometry       0 -ELEVATION=dem.sgrd -SLOPE=slope.sgrd -ASPECT=aspect.sgrd -C_CROS=hcurv.sgrd -C_LONG=vcurv.sgrd\n"
 		"saga_cmd %%FLAGS%% ta_hydrology         0 -ELEVATION=dem.sgrd -CAREA=carea.sgrd\n"
 		"\n"
 		"ECHO ____________________________\n"
@@ -820,7 +836,7 @@ void		Create_Example	(void)
 		"ECHO cluster analysis and vectorisation\n"
 		"\n"
 		"# cluster analysis\n"
-		"imagery_classification 1 -GRIDS=%%INPUT%% -CLUSTER=cluster.sgrd -NORMALISE -NCLUSTER=5\n"
+		"imagery_classification 1 -GRIDS=%%INPUT%% -CLUSTER=cluster.sgrd -NORMALISE=true -NCLUSTER=5\n"
 		"\n"
 		"# majority filter\n"
 		"grid_filter            6 -INPUT=cluster.sgrd -RADIUS=3\n"
