@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: sgdi_dialog.cpp 911 2011-02-14 16:38:15Z reklov_w $
+ * Version $Id: sgdi_dialog.cpp 1921 2014-01-09 10:24:11Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -36,7 +36,7 @@
 // You should have received a copy of the GNU General    //
 // Public License along with this program; if not,       //
 // write to the Free Software Foundation, Inc.,          //
-// 59 Temple Place - Suite 330, Boston, MA 02111-1307,   //
+// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
 // USA.                                                  //
 //                                                       //
 //-------------------------------------------------------//
@@ -127,7 +127,7 @@ CSGDI_Dialog::CSGDI_Dialog(const wxString &Name, int Style)
 		pSizer->Add(m_pSizer_Output	, 1, wxALIGN_RIGHT|wxALL|wxEXPAND, SGDI_CTRL_SPACE);
 	}
 
-	pSizer->SetVirtualSizeHints(this);
+	pSizer->FitInside(this);
 
 	SetSizer(pSizer);
 }
