@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: sgdi_helper.h 1922 2014-01-09 10:28:46Z oconrad $
+ * Version $Id: sgdi_helper.h 2114 2014-05-08 15:31:17Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -73,13 +73,7 @@
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-#include <saga_api/saga_api.h>
-
-#ifdef _SAGA_GDI_EXPORTS
-	#define	SGDI_API_DLL_EXPORT		_SAGA_DLL_EXPORT
-#else
- 	#define	SGDI_API_DLL_EXPORT		_SAGA_DLL_IMPORT
-#endif
+#include "sgdi_core.h"
 
 
 ///////////////////////////////////////////////////////////
