@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: active.cpp 2016 2014-02-25 15:07:31Z oconrad $
+ * Version $Id: active.cpp 2629 2015-10-01 09:40:40Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -284,8 +284,6 @@ bool CACTIVE::Set_Active(CWKSP_Base_Item *pItem)
 	//-----------------------------------------------------
 	if( m_pItem == NULL )
 	{
-		STATUSBAR_Set_Text(SG_T(""), STATUSBAR_ACTIVE);
-
 		if( g_pSAGA_Frame   )	g_pSAGA_Frame->Set_Pane_Caption(this, _TL("Properties"));
 
 		if( g_pData_Buttons )	g_pData_Buttons->Refresh();
@@ -301,8 +299,6 @@ bool CACTIVE::Set_Active(CWKSP_Base_Item *pItem)
 	}
 
 	//-----------------------------------------------------
-	STATUSBAR_Set_Text(m_pItem->Get_Name(), STATUSBAR_ACTIVE);
-
 	if( g_pSAGA_Frame )	g_pSAGA_Frame->Set_Pane_Caption(this, wxString(_TL("Properties")) + ": " + m_pItem->Get_Name());
 
 	//-----------------------------------------------------
@@ -434,37 +430,9 @@ bool CACTIVE::Update_Description(void)
 	{
 		Description	= _TL("No description available");
 	}
-	else switch( m_pItem->Get_Type() )
+	else
 	{
-	default:
 		Description	= m_pItem->Get_Description();
-		break;
-
-	case WKSP_ITEM_Module:
-		{
-			wxFileName	FileName;
-
-			FileName.Assign		(((CWKSP_Module *)m_pItem)->Get_File_Name());
-			FileName.AppendDir	(FileName.GetName());
-			FileName.SetName	(wxString::Format(wxT("%s_%02d"), FileName.GetName().c_str(), m_pItem->Get_Index()));
-
-			FileName.SetExt		(wxT("html"));
-
-			if( FileName.FileExists() && m_pDescription->LoadPage(FileName.GetFullPath()) )
-			{
-				return( true );
-			}
-
-			FileName.SetExt		(wxT("htm"));
-
-			if( FileName.FileExists() && m_pDescription->LoadPage(FileName.GetFullPath()) )
-			{
-				return( true );
-			}
-
-			Description	= m_pItem->Get_Description();
-		}
-		break;
 	}
 
 	//-----------------------------------------------------
@@ -498,6 +466,8 @@ bool CACTIVE::Update(CWKSP_Base_Item *pItem, bool bSave)
 		if( !bSave )
 		{
 			Update_Description();
+
+			m_pHistory->Set_Item(pItem);
 		}
 
 		return( true );
