@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: active_history.cpp 2043 2014-03-05 17:17:20Z oconrad $
+ * Version $Id: active_history.cpp 2313 2014-10-30 09:42:40Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -70,6 +70,8 @@
 
 #include "wksp_table.h"
 #include "wksp_layer.h"
+#include "wksp_map_layer.h"
+#include "wksp_module_manager.h"
 
 #include "active.h"
 #include "active_history.h"
@@ -110,7 +112,10 @@ IMPLEMENT_CLASS(CACTIVE_History, wxTreeCtrl)
 BEGIN_EVENT_TABLE(CACTIVE_History, wxTreeCtrl)
 	EVT_RIGHT_DOWN		(CACTIVE_History::On_Mouse_RDown)
 
-	EVT_MENU			(ID_CMD_DATA_HISTORY_CLEAR	, CACTIVE_History::On_Clear)
+	EVT_MENU			(ID_CMD_DATA_HISTORY_CLEAR           , CACTIVE_History::On_Clear)
+	EVT_MENU			(ID_CMD_DATA_HISTORY_TO_MODEL        , CACTIVE_History::On_SaveAs_Model)
+	EVT_MENU			(ID_CMD_DATA_HISTORY_OPTIONS_COLLAPSE, CACTIVE_History::On_Options_Expand)
+	EVT_MENU			(ID_CMD_DATA_HISTORY_OPTIONS_EXPAND  , CACTIVE_History::On_Options_Expand)
 END_EVENT_TABLE()
 
 
@@ -136,7 +141,51 @@ CACTIVE_History::CACTIVE_History(wxWindow *pParent)
 	IMG_ADD_TO_TREECTRL(ID_IMG_WKSP_TIN);				// TIN
 	IMG_ADD_TO_TREECTRL(ID_IMG_TB_OPEN);				// FILE
 
-	Set_Item(NULL);
+	m_pItem		= NULL;
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CACTIVE_History::Set_Item(CWKSP_Base_Item *pItem)
+{
+	m_pItem	= pItem;
+
+	return( _Set_History() );
+}
+
+//---------------------------------------------------------
+bool CACTIVE_History::_Set_History(void)
+{
+	Freeze();
+
+	DeleteAllItems();
+
+	CSG_Data_Object	*pObject	= _Get_Object();
+
+	if( pObject == NULL || pObject->Get_History().Get_Children_Count() <= 0 )
+	{
+		AddRoot(_TL("no history"), IMG_ROOT);
+	}
+	else if( SG_Compare_Version(pObject->Get_History().Get_Property("saga-version"), "2.1.3") >= 0 )	// new version
+	{
+		_Add_History(AddRoot(pObject->Get_Name(), IMG_ROOT), pObject->Get_History());
+	}
+	else
+	{
+		_Add_History_OLD(AddRoot(pObject->Get_Name(), IMG_ROOT), pObject->Get_History());
+	}
+
+	Expand(GetRootItem());
+
+	Thaw();
+
+	return( true );
 }
 
 
@@ -152,6 +201,15 @@ void CACTIVE_History::On_Mouse_RDown(wxMouseEvent &event)
 	wxMenu	Menu(_TL("History"));
 
 	CMD_Menu_Add_Item(&Menu, false, ID_CMD_DATA_HISTORY_CLEAR);
+	Menu.AppendSeparator();
+	CMD_Menu_Add_Item(&Menu, false, ID_CMD_DATA_HISTORY_OPTIONS_COLLAPSE);
+	CMD_Menu_Add_Item(&Menu, false, ID_CMD_DATA_HISTORY_OPTIONS_EXPAND);
+
+	if( _Get_Object() && SG_Compare_Version(_Get_Object()->Get_History().Get_Property("saga-version"), "2.1.3") >= 0 )	// new version
+	{
+		Menu.AppendSeparator();
+		CMD_Menu_Add_Item(&Menu, false, ID_CMD_DATA_HISTORY_TO_MODEL);
+	}
 
 	PopupMenu(&Menu, event.GetPosition());
 
@@ -161,24 +219,70 @@ void CACTIVE_History::On_Mouse_RDown(wxMouseEvent &event)
 //---------------------------------------------------------
 void CACTIVE_History::On_Clear(wxCommandEvent &event)
 {
-	CWKSP_Base_Item	*pItem		= g_pACTIVE->Get_Active();
-
-	CSG_Data_Object	*pObject	= pItem && pItem->GetId().IsOk()
-	&&	(	pItem->Get_Type() == WKSP_ITEM_Table
-		||	pItem->Get_Type() == WKSP_ITEM_TIN
-		||	pItem->Get_Type() == WKSP_ITEM_PointCloud
-		||	pItem->Get_Type() == WKSP_ITEM_Shapes
-		||	pItem->Get_Type() == WKSP_ITEM_Grid )
-	? ((CWKSP_Data_Item *)pItem)->Get_Object() : NULL;
+	CSG_Data_Object	*pObject	= _Get_Object();
 
 	int	Depth	= 0;
 
 	if( pObject && DLG_Get_Number(Depth, _TL("Delete History Entries"), _TL("Depth")) )
 	{
-		pObject->Get_History().Del_Children(Depth);
+		pObject->Get_History().Del_Children(Depth, Depth > 0 ? SG_T("MODULE") : SG_T(""));
 		pObject->Set_Modified(true);
 
-		Set_Item(pItem);
+		Set_Item(g_pACTIVE->Get_Active());
+	}
+}
+
+//---------------------------------------------------------
+void CACTIVE_History::On_SaveAs_Model(wxCommandEvent &event)
+{
+	const wxString	Filter	= wxString::Format("%s|*.xml|%s|*.*",
+		_TL("XML Files"), _TL("All Files")
+	);
+
+	wxString	File;
+
+	CSG_Data_Object	*pObject	= _Get_Object();
+
+	if( pObject && pObject->Get_History().Get_Children_Count() > 0
+	&&  DLG_Save(File, _TL("Save History as Model"), Filter)
+	&&  pObject->Save_History_to_Model(&File) )
+	{
+		g_pModules->Open(File);
+	}
+}
+
+//---------------------------------------------------------
+void CACTIVE_History::On_Options_Expand(wxCommandEvent &event)
+{
+	_Expand(GetRootItem(), "Options", event.GetId() == ID_CMD_DATA_HISTORY_OPTIONS_EXPAND);
+}
+
+//---------------------------------------------------------
+void CACTIVE_History::_Expand(wxTreeItemId Node, const wxString &Name, bool bExpand)
+{
+	if( !GetItemText(Node).Cmp(Name) )
+	{
+		if( bExpand )
+		{
+			Expand(Node);
+		}
+		else
+		{
+			Collapse(Node);
+		}
+	}
+	else
+	{
+		wxTreeItemIdValue	Cookie;
+
+		wxTreeItemId	Child	= GetFirstChild(Node, Cookie);
+
+		while( Child.IsOk() )
+		{
+			_Expand(Child, Name, bExpand);
+
+			Child	= GetNextChild(Node, Cookie);
+		}
 	}
 }
 
@@ -190,43 +294,28 @@ void CACTIVE_History::On_Clear(wxCommandEvent &event)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CACTIVE_History::Set_Item(CWKSP_Base_Item *pItem)
+CSG_Data_Object * CACTIVE_History::_Get_Object(void)
 {
-	CSG_Data_Object	*pObject	= pItem && pItem->GetId().IsOk()
-	&&	(	pItem->Get_Type() == WKSP_ITEM_Table
-		||	pItem->Get_Type() == WKSP_ITEM_TIN
-		||	pItem->Get_Type() == WKSP_ITEM_PointCloud
-		||	pItem->Get_Type() == WKSP_ITEM_Shapes
-		||	pItem->Get_Type() == WKSP_ITEM_Grid )
-	? ((CWKSP_Data_Item *)pItem)->Get_Object() : NULL;
-
-	Freeze();
-
-	DeleteAllItems();
-
-	if( pObject == NULL || pObject->Get_History().Get_Children_Count() <= 0 )
+	if( m_pItem && m_pItem->GetId().IsOk() )
 	{
-		AddRoot(_TL("No history available"), IMG_ROOT);
-	}
-	else
-	{
-		_Add_History(AddRoot(pObject->Get_Name(), IMG_ROOT), pObject->Get_History());
+		if( m_pItem->Get_Type() == WKSP_ITEM_Table
+		||  m_pItem->Get_Type() == WKSP_ITEM_TIN
+		||  m_pItem->Get_Type() == WKSP_ITEM_PointCloud
+		||  m_pItem->Get_Type() == WKSP_ITEM_Shapes
+		||  m_pItem->Get_Type() == WKSP_ITEM_Grid )
+		{
+			return( ((CWKSP_Data_Item *)m_pItem)->Get_Object() );
+		}
 
-		Expand(GetRootItem());
+		if( m_pItem->Get_Type() == WKSP_ITEM_Map_Layer )
+		{
+			return( ((CWKSP_Map_Layer *)m_pItem)->Get_Layer()->Get_Object() );
+		}
 	}
 
-	Thaw();
-
-	return( true );
+	return( NULL );
 }
 
-
-///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
-///////////////////////////////////////////////////////////
-
 //---------------------------------------------------------
 int CACTIVE_History::_Get_Image(TSG_Parameter_Type Type)
 {
@@ -254,59 +343,57 @@ int CACTIVE_History::_Get_Image(const CSG_String &Type)
 	return( _Get_Image(SG_Parameter_Type_Get_Type(Type)) );
 }
 
+
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
 //---------------------------------------------------------
-bool CACTIVE_History::_Add_History(wxTreeItemId Parent, CSG_MetaData &History)
+bool CACTIVE_History::_Add_History(wxTreeItemId Parent, CSG_MetaData &Data)
 {
-	if( !Parent.IsOk() || History.Get_Children_Count() <= 0 )
+	if( !Parent.IsOk() )
 	{
 		return( false );
 	}
 
 	//-----------------------------------------------------
-	int					i, n;
-	TSG_Parameter_Type	Type;
-	CSG_String			Name;
-	CSG_MetaData		*pEntry;
-	wxTreeItemId		Node;
+	CSG_MetaData	*pModule	= Data("MODULE");
 
-	//-----------------------------------------------------
-	if( (pEntry = History.Get_Child("MODULE")) == NULL )
+	if( !pModule || !pModule->Get_Child("OUTPUT") )
 	{
-		for(i=0; i<History.Get_Children_Count(); i++)
+		if( Data("FILE") )
 		{
-			pEntry	= History.Get_Child(i);
-
-			if( pEntry->Get_Children_Count() > 0 )
-			{
-				_Add_History(AppendItem(Parent, wxString::Format("%s", pEntry->Get_Name().c_str()), IMG_NODE), *pEntry);
-			}
-			else if( !pEntry->Get_Name().Cmp(SG_T("FILE")) )
-			{
-				AppendItem(Parent, wxString::Format("%s", pEntry->Get_Content().c_str()), IMG_FILE);
-			}
-			else
-			{
-				AppendItem(Parent, wxString::Format("[%s] %s", pEntry->Get_Name().c_str(), pEntry->Get_Content().c_str()), IMG_ENTRY);
-			}
+			AppendItem(Parent, Data["FILE"].Get_Content().c_str(), IMG_FILE);
+		}
+		else
+		{
+			AppendItem(Parent, Data.Get_Name().c_str(), IMG_FILE);
 		}
 	}
 
 	//-----------------------------------------------------
 	else
 	{
-		Node	= AppendItem(Parent, pEntry->Get_Content().c_str(), IMG_TOOL);
+		int	i;
+
+		wxTreeItemId	Module	= AppendItem(Parent, wxString::Format("%s [%s]",
+			pModule->Get_Child("OUTPUT")->Get_Property("name"), pModule->Get_Property("name")
+			), IMG_TOOL
+		);
 
 		//-------------------------------------------------
-		for(i=0, n=0; i<History.Get_Children_Count(); i++)	// Options
+		wxTreeItemId	Options	= AppendItem(Module, _TL("Options"), IMG_ENTRY);
+
+		for(i=0; i<pModule->Get_Children_Count(); i++)	// Options
 		{
-			pEntry	= History.Get_Child(i);
-			Name	= pEntry->Get_Property("name");
-			Type	= SG_Parameter_Type_Get_Type(pEntry->Get_Property("type"));
+			CSG_MetaData		*pEntry	= pModule->Get_Child(i);
+			CSG_String			Name	= pEntry->Get_Property("name");
+			TSG_Parameter_Type	Type	= SG_Parameter_Type_Get_Type(pEntry->Get_Property("type"));
 
 			if( !pEntry->Get_Name().Cmp("OPTION") )
 			{
-				n++;
-
 				switch( Type )
 				{
 				case PARAMETER_TYPE_Bool:
@@ -320,21 +407,15 @@ bool CACTIVE_History::_Add_History(wxTreeItemId Parent, CSG_MetaData &History)
 				case PARAMETER_TYPE_String:
 				case PARAMETER_TYPE_Text:
 				case PARAMETER_TYPE_FilePath:
-					AppendItem(Node, wxString::Format("%s [%s: %s]",
-						Name.c_str(), SG_Parameter_Type_Get_Name(Type).c_str(),
-						pEntry->Get_Content().c_str()),
-						IMG_ENTRY
-					);
+				//	AppendItem(Options, wxString::Format("%s [%s: %s]", Name.c_str(), SG_Parameter_Type_Get_Name(Type).c_str(), pEntry->Get_Content().c_str()), IMG_ENTRY);
+					AppendItem(Options, wxString::Format("%s [%s]", Name.c_str(), pEntry->Get_Content().c_str()), IMG_ENTRY);
 					break;
 
 				case PARAMETER_TYPE_Grid_System:
 					if( pEntry->Get_Children_Count() == 0 )
 					{
-						AppendItem(Node, wxString::Format("%s [%s: %s]",
-							Name.c_str(), SG_Parameter_Type_Get_Name(Type).c_str(),
-							pEntry->Get_Content().c_str()),
-							IMG_ENTRY
-						);
+					//	AppendItem(Options, wxString::Format("%s [%s: %s]", Name.c_str(), SG_Parameter_Type_Get_Name(Type).c_str(), pEntry->Get_Content().c_str()), IMG_ENTRY);
+						AppendItem(Options, wxString::Format("%s [%s]", Name.c_str(), pEntry->Get_Content().c_str()), IMG_ENTRY);
 					}
 					break;
 
@@ -348,27 +429,32 @@ bool CACTIVE_History::_Add_History(wxTreeItemId Parent, CSG_MetaData &History)
 			}
 		}
 
-		if( n == 0 )
+		if( ItemHasChildren(Options) )
 		{
-			AppendItem(Node, _TL("no options"), IMG_ENTRY);
+			Expand(Options);
+		}
+		else
+		{
+		//	Delete(Options);
+			SetItemText(Options, _TL("No Options"));
 		}
 
 		//-------------------------------------------------
-		for(i=0; i<History.Get_Children_Count(); i++)	// Input Data
+		for(i=0; i<pModule->Get_Children_Count(); i++)	// Input Data
 		{
-			pEntry	= History.Get_Child(i);
-			Name	= pEntry->Get_Property("name");
-			Type	= SG_Parameter_Type_Get_Type(pEntry->Get_Property("type"));
+			CSG_MetaData		*pEntry	= pModule->Get_Child(i);
+			CSG_String			Name	= pEntry->Get_Property("name");
+			TSG_Parameter_Type	Type	= SG_Parameter_Type_Get_Type(pEntry->Get_Property("type"));
 
-			if( !pEntry->Get_Name().Cmp("DATA") )
+			if( !pEntry->Get_Name().Cmp("INPUT") )
 			{
-				_Add_History(AppendItem(Node, wxString::Format("%s", Name.c_str()), _Get_Image(Type)), *pEntry);
+				_Add_History(AppendItem(Module, wxString::Format("%s", Name.c_str()), _Get_Image(Type)), *pEntry);
 			}
-			else if( !pEntry->Get_Name().Cmp("DATA_LIST") )
+			else if( !pEntry->Get_Name().Cmp("INPUT_LIST") )
 			{
 				if( pEntry->Get_Children_Count() > 0 )
 				{
-					wxTreeItemId	List	= AppendItem(Node, wxString::Format("%s %s", Name.c_str(), _TL("List")), _Get_Image(Type));
+					wxTreeItemId	List	= AppendItem(Module, wxString::Format("%s %s", Name.c_str(), _TL("List")), _Get_Image(Type));
 
 					for(int j=0; j<pEntry->Get_Children_Count(); j++)
 					{
@@ -380,7 +466,149 @@ bool CACTIVE_History::_Add_History(wxTreeItemId Parent, CSG_MetaData &History)
 			}
 		}
 
-		Expand(Node);
+		Expand(Module);
+	}
+
+	Expand(Parent);
+
+	return( true );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+//					Compatibility						 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CACTIVE_History::_Add_History_OLD(wxTreeItemId Parent, CSG_MetaData &Data)
+{
+	if( !Parent.IsOk() )
+	{
+		return( false );
+	}
+
+	//-----------------------------------------------------
+	CSG_MetaData	*pModule	= Data("MODULE");
+
+	if( !pModule )
+	{
+		if( Data.Get_Children_Count() > 0 )
+		{
+			for(int i=0; i<Data.Get_Children_Count(); i++)
+			{
+				CSG_MetaData	*pEntry	= Data.Get_Child(i);
+
+				if( pEntry->Get_Children_Count() > 0 )
+				{
+					_Add_History_OLD(AppendItem(Parent, wxString::Format("%s", pEntry->Get_Name().c_str()), IMG_NODE), *pEntry);
+				}
+				else if( !pEntry->Get_Name().Cmp(SG_T("FILE")) )
+				{
+					AppendItem(Parent, wxString::Format("%s", pEntry->Get_Content().c_str()), IMG_FILE);
+				}
+				else
+				{
+					AppendItem(Parent, wxString::Format("[%s] %s", pEntry->Get_Name().c_str(), pEntry->Get_Content().c_str()), IMG_ENTRY);
+				}
+			}
+		}
+		else
+		{
+			AppendItem(Parent, Data.Get_Name().c_str(), IMG_FILE);
+		}
+	}
+
+	//-----------------------------------------------------
+	else
+	{
+		int	i;
+
+		wxTreeItemId	Module	= AppendItem(Parent, pModule->Get_Content().c_str(), IMG_TOOL);
+
+		//-------------------------------------------------
+		wxTreeItemId	Options	= AppendItem(Module, _TL("Options"), IMG_ENTRY);
+
+		for(i=0; i<Data.Get_Children_Count(); i++)	// Options
+		{
+			CSG_MetaData		*pEntry	= Data.Get_Child(i);
+			CSG_String			Name	= pEntry->Get_Property("name");
+			TSG_Parameter_Type	Type	= SG_Parameter_Type_Get_Type(pEntry->Get_Property("type"));
+
+			if( !pEntry->Get_Name().Cmp("OPTION") )
+			{
+				switch( Type )
+				{
+				case PARAMETER_TYPE_Bool:
+				case PARAMETER_TYPE_Int:
+				case PARAMETER_TYPE_Double:
+				case PARAMETER_TYPE_Degree:
+				case PARAMETER_TYPE_Choice:
+				case PARAMETER_TYPE_Range:
+				case PARAMETER_TYPE_Table_Field:
+				case PARAMETER_TYPE_Table_Fields:
+				case PARAMETER_TYPE_String:
+				case PARAMETER_TYPE_Text:
+				case PARAMETER_TYPE_FilePath:
+					AppendItem(Options, wxString::Format("%s [%s]", Name.c_str(), pEntry->Get_Content().c_str()), IMG_ENTRY);
+					break;
+
+				case PARAMETER_TYPE_Grid_System:
+					if( pEntry->Get_Children_Count() == 0 )
+					{
+						AppendItem(Options, wxString::Format("%s [%s]", Name.c_str(), pEntry->Get_Content().c_str()), IMG_ENTRY);
+					}
+					break;
+
+				case PARAMETER_TYPE_FixedTable:
+				case PARAMETER_TYPE_Parameters:
+					break;
+
+				default:
+					break;
+				}
+			}
+		}
+
+		if( ItemHasChildren(Options) )
+		{
+			Expand(Options);
+		}
+		else
+		{
+		//	Delete(Options);
+			SetItemText(Options, _TL("No Options"));
+		}
+
+		//-------------------------------------------------
+		for(i=0; i<Data.Get_Children_Count(); i++)	// Input Data
+		{
+			CSG_MetaData		*pEntry	= Data.Get_Child(i);
+			CSG_String			Name	= pEntry->Get_Property("name");
+			TSG_Parameter_Type	Type	= SG_Parameter_Type_Get_Type(pEntry->Get_Property("type"));
+
+			if( !pEntry->Get_Name().Cmp("DATA") )
+			{
+				_Add_History_OLD(AppendItem(Module, wxString::Format("%s", Name.c_str()), _Get_Image(Type)), *pEntry);
+			}
+			else if( !pEntry->Get_Name().Cmp("DATA_LIST") )
+			{
+				if( pEntry->Get_Children_Count() > 0 )
+				{
+					wxTreeItemId	List	= AppendItem(Module, wxString::Format("%s %s", Name.c_str(), _TL("List")), _Get_Image(Type));
+
+					for(int j=0; j<pEntry->Get_Children_Count(); j++)
+					{
+						_Add_History_OLD(List, *pEntry->Get_Child(j));
+					}
+
+					Expand(List);
+				}
+			}
+		}
+
+		Expand(Module);
 	}
 
 	Expand(Parent);
