@@ -20,13 +20,13 @@
 //                                                       //
 //-------------------------------------------------------//
 //                                                       //
-// This file is part of 'MicroCity: Spatial Analysis and //
-// Simulation Framework'. MicroCity is free software;you //
+// This file is part of 'SAGA - System for Automated     //
+// Geoscientific Analyses'. SAGA is free software; you   //
 // can redistribute it and/or modify it under the terms  //
 // of the GNU General Public License as published by the //
 // Free Software Foundation; version 2 of the License.   //
 //                                                       //
-// MicroCity is distributed in the hope that it will be  //
+// SAGA is distributed in the hope that it will be       //
 // useful, but WITHOUT ANY WARRANTY; without even the    //
 // implied warranty of MERCHANTABILITY or FITNESS FOR A  //
 // PARTICULAR PURPOSE. See the GNU General Public        //
@@ -295,19 +295,9 @@ void CData_Source_ODBC::On_Item_Menu(wxTreeEvent &event)
 	\
 	if(	pModule )\
 	{\
-		CSG_Parameters	P; P.Assign(pModule->Get_Parameters());\
-		\
-		pModule->Set_Manager(NULL);\
-		pModule->Get_Parameters()->Set_Callback(true);\
-		pModule->On_Before_Execution();\
-		\
-		if( (CONDITION) && pModule->Execute() )\
-		{\
-			bRetVal	= true;\
-		}\
-		\
-		pModule->Get_Parameters()->Assign_Values(&P);\
-		pModule->Set_Manager(P.Get_Manager());\
+		pModule->Settings_Push(NULL);\
+		bRetVal	= pModule->On_Before_Execution() && (CONDITION) && pModule->Execute();\
+		pModule->Settings_Pop();\
 	}\
 }
 
