@@ -20,13 +20,13 @@
 //                                                       //
 //-------------------------------------------------------//
 //                                                       //
-// This file is part of 'MicroCity: Spatial Analysis and //
-// Simulation Framework'. MicroCity is free software;you //
+// This file is part of 'SAGA - System for Automated     //
+// Geoscientific Analyses'. SAGA is free software; you   //
 // can redistribute it and/or modify it under the terms  //
 // of the GNU General Public License as published by the //
 // Free Software Foundation; version 2 of the License.   //
 //                                                       //
-// MicroCity is distributed in the hope that it will be  //
+// SAGA is distributed in the hope that it will be       //
 // useful, but WITHOUT ANY WARRANTY; without even the    //
 // implied warranty of MERCHANTABILITY or FITNESS FOR A  //
 // PARTICULAR PURPOSE. See the GNU General Public        //
@@ -84,9 +84,11 @@
 enum
 {
 	TYPE_ROOT	= 0,
+	TYPE_SERVER,
 	TYPE_SOURCE,
 	TYPE_TABLE,
 	TYPE_SHAPES,
+	TYPE_GRIDS,
 	TYPE_GRID
 };
 
@@ -94,6 +96,7 @@ enum
 enum
 {
 	IMG_ROOT	= 0,
+	IMG_SERVER,
 	IMG_SRC_CLOSED,
 	IMG_SRC_OPENED,
 	IMG_TABLE,
@@ -101,19 +104,20 @@ enum
 	IMG_POINTS,
 	IMG_LINE,
 	IMG_POLYGON,
+	IMG_GRIDS,
 	IMG_GRID
 };
 
 //---------------------------------------------------------
 enum
 {
-	DB_PGSQL_Get_Connections	= 0,
-	DB_PGSQL_Get_Connection		= 1,
-	DB_PGSQL_Del_Connection		= 2,
-	DB_PGSQL_Del_Connections	= 3,
-	DB_PGSQL_Transaction_Start	= 4,
-	DB_PGSQL_Transaction_Stop	= 5,
-	DB_PGSQL_Execute_SQL		= 6,
+	DB_PGSQL_Get_Connections	=  0,
+	DB_PGSQL_Get_Connection		=  1,
+	DB_PGSQL_Del_Connection		=  2,
+	DB_PGSQL_Del_Connections	=  3,
+	DB_PGSQL_Transaction_Start	=  4,
+	DB_PGSQL_Transaction_Stop	=  5,
+	DB_PGSQL_Execute_SQL		=  6,
 
 	DB_PGSQL_Table_List			= 10,
 	DB_PGSQL_Table_Info			= 11,
@@ -127,8 +131,12 @@ enum
 	DB_PGSQL_Shapes_SRID_Update	= 22,
 
 	DB_PGSQL_Raster_Load		= 30,
+	DB_PGSQL_Raster_Load_Band	= 33,
 	DB_PGSQL_Raster_Save		= 31,
-	DB_PGSQL_Raster_SRID_Update	= 32
+	DB_PGSQL_Raster_SRID_Update	= 32,
+
+	DB_PGSQL_DB_Create			= 35,
+	DB_PGSQL_DB_Drop			= 36
 };
 
 
@@ -139,32 +147,20 @@ enum
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-#define RUN_MODULE(MODULE, bRetVal, CONDITION)	{\
-	\
-	bRetVal	= false;\
-	\
-	CSG_Module	*pModule	= SG_Get_Module_Library_Manager().Get_Module(SG_T("db_pgsql"), MODULE);\
+#define RUN_MODULE(MODULE, bManager, CONDITION)	bool bResult = false;\
+{\
+	CSG_Module	*pModule	= SG_Get_Module_Library_Manager().Get_Module("db_pgsql", MODULE);\
 	\
 	if(	pModule )\
 	{\
-		CSG_Parameters	P; P.Assign(pModule->Get_Parameters());\
-		\
-		pModule->Set_Manager(NULL);\
-		pModule->Get_Parameters()->Set_Callback(true);\
-		pModule->On_Before_Execution();\
-		\
-		if( (CONDITION) && pModule->Execute() )\
-		{\
-			bRetVal	= true;\
-		}\
-		\
-		pModule->Get_Parameters()->Assign_Values(&P);\
-		pModule->Set_Manager(P.Get_Manager());\
+		pModule->Settings_Push(bManager ? &SG_Get_Data_Manager() : NULL);\
+		bResult	= pModule->On_Before_Execution() && (CONDITION) && pModule->Execute();\
+		pModule->Settings_Pop();\
 	}\
 }
 
 //---------------------------------------------------------
-#define SET_PARAMETER(IDENTIFIER, VALUE)	pModule->Get_Parameters()->Set_Parameter(SG_T(IDENTIFIER), VALUE)
+#define SET_PARAMETER(IDENTIFIER, VALUE)	pModule->Set_Parameter(IDENTIFIER, VALUE)
 
 
 ///////////////////////////////////////////////////////////
@@ -174,12 +170,43 @@ enum
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool	is_Connected	(const CSG_String &Server)
+bool	PGSQL_Connect			(const CSG_String &Host, const CSG_String &Port, const CSG_String &DBName)
+{
+	if( PGSQL_is_Connected(Host, Port, DBName) )
+	{
+		return( true );
+	}
+
+	wxString	Username = "postgres", Password = "postgres";
+
+	if( !DLG_Login(Username, Password, wxString::Format("%s: %s [%s:%s]", _TL("Connect to Database"), DBName.c_str(), Host.c_str(), Port.c_str())) )
+	{
+		return( false );
+	}
+
+	RUN_MODULE(DB_PGSQL_Get_Connection, false,	// CGet_Connection
+			SET_PARAMETER("PG_HOST", Host    )
+		&&	SET_PARAMETER("PG_PORT", Port    )
+		&&	SET_PARAMETER("PG_NAME", DBName  )
+		&&	SET_PARAMETER("PG_USER", Username)
+		&&	SET_PARAMETER("PG_PWD" , Password)
+	);
+
+	return( bResult );
+}
+
+//---------------------------------------------------------
+bool	PGSQL_is_Connected		(const CSG_String &Host, const CSG_String &Port, const CSG_String &DBName)
+{
+	return( PGSQL_is_Connected(DBName + " [" + Host + ":" + Port + "]") );
+}
+
+//---------------------------------------------------------
+bool	PGSQL_is_Connected		(const CSG_String &Server)
 {
-	bool		bResult;
 	CSG_Table	Connections;
 
-	RUN_MODULE(DB_PGSQL_Get_Connections, bResult, SET_PARAMETER("CONNECTIONS", &Connections));	// CGet_Connections
+	RUN_MODULE(DB_PGSQL_Get_Connections, false, SET_PARAMETER("CONNECTIONS", &Connections));	// CGet_Connections
 
 	for(int i=0; bResult && i<Connections.Get_Count(); i++)
 	{
@@ -193,6 +220,91 @@ bool	is_Connected	(const CSG_String &Server)
 }
 
 
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool	PGSQL_Get_Connections	(CSG_Strings &Servers, double vPostGIS)
+{
+	Servers.Clear();
+
+	CSG_Table	Connections;
+
+	RUN_MODULE(DB_PGSQL_Get_Connections, false, SET_PARAMETER("CONNECTIONS", &Connections));	// CGet_Connections
+
+	for(int i=0; bResult && i<Connections.Get_Count(); i++)
+	{
+		if( vPostGIS <= 0.0 || vPostGIS <= Connections[i].asDouble("PostGIS") )
+		{
+			Servers	+= Connections[i].asString(0);
+		}
+	}
+
+	return( Servers.Get_Count() > 0 );
+}
+
+//---------------------------------------------------------
+bool	PGSQL_has_Connections	(double vPostGIS)
+{
+	CSG_Strings	Servers;
+
+	return( PGSQL_Get_Connections(Servers, vPostGIS) );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool	PGSQL_Save_Table	(CSG_Table *pTable)
+{
+	CSG_Module	*pModule	= SG_Get_Module_Library_Manager().Get_Module("db_pgsql", DB_PGSQL_Table_Save);
+
+	if(	pModule && pModule->On_Before_Execution() && pModule->Set_Parameter("SHAPES", pTable)
+	&&  DLG_Parameters(pModule->Get_Parameters()) && pModule->Execute() )
+	{
+		return( true );
+	}
+
+	return( false );
+}
+
+//---------------------------------------------------------
+bool	PGSQL_Save_Shapes	(CSG_Shapes *pShapes)
+{
+	CSG_Module	*pModule	= SG_Get_Module_Library_Manager().Get_Module("db_pgsql", DB_PGSQL_Shapes_Save);
+
+	if(	pModule && pModule->On_Before_Execution() && pModule->Set_Parameter("SHAPES", pShapes)
+	&&  DLG_Parameters(pModule->Get_Parameters()) && pModule->Execute() )
+	{
+		return( true );
+	}
+
+	return( false );
+}
+
+//---------------------------------------------------------
+bool	PGSQL_Save_Grid		(CSG_Grid *pGrid)
+{
+	CSG_Module	*pModule	= SG_Get_Module_Library_Manager().Get_Module("db_pgsql", DB_PGSQL_Raster_Save);
+
+	if(	pModule && pModule->On_Before_Execution() && pModule->Set_Parameter("NAME", pGrid->Get_Name()) )
+	{
+		pModule->Get_Parameters()->Get_Parameter("GRIDS")->asList()->Del_Items();
+
+		if( pModule->Get_Parameters()->Get_Parameter("GRIDS")->asList()->Add_Item(pGrid)
+		&&  DLG_Parameters(pModule->Get_Parameters()) && pModule->Execute() )
+		{
+			return( true );
+		}
+	}
+
+	return( false );
+}
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 //														 //
@@ -203,21 +315,31 @@ bool	is_Connected	(const CSG_String &Server)
 class CData_Source_PgSQL_Data : public wxTreeItemData
 {
 public:
-    CData_Source_PgSQL_Data(int Type, const CSG_String &Value = SG_T(""), const CSG_String &Server = "")
-		: m_Type(Type), m_Value(Value), m_Server(Server)
+    CData_Source_PgSQL_Data(int Type, const CSG_String &Value = "", const CSG_String &Server = "", const CSG_String &Username = "", const CSG_String &Password = "")
+		: m_Type(Type), m_Value(Value), m_Server(Server), m_Username(Username), m_Password(Password)
 	{}
 
-	int						Get_Type		(void)	const	{	return( m_Type   );	}
-	const CSG_String &		Get_Value		(void)	const	{	return( m_Value  );	}
-	const CSG_String &		Get_Server		(void)	const	{	return( m_Server );	}
+	int						Get_Type		(void)	const	{	return( m_Type     );	}
+	const CSG_String &		Get_Value		(void)	const	{	return( m_Value    );	}
+	const CSG_String &		Get_Server		(void)	const	{	return( m_Server   );	}
+
+	void					Set_Username	(const SG_Char *Username)	{	m_Username	= Username;	}
+	const CSG_String &		Get_Username	(void)	const	{	return( m_Username     );	}
+	void					Set_Password	(const SG_Char *Password)	{	m_Password	= Password;	}
+	const CSG_String &		Get_Password	(void)	const	{	return( m_Password );	}
+
+	CSG_String				Get_Host		(void)  const	{	return( m_Server.AfterLast ('[').BeforeFirst(':') );	}
+	CSG_String				Get_Port		(void)  const	{	return( m_Server.AfterLast (':').BeforeFirst(']') );	}
+	CSG_String				Get_DBName		(void)  const	{	CSG_String s(m_Server.BeforeLast('[')); s.Trim(true); return( s );	}
+
+	bool					is_Connected	(void)	const	{	return( PGSQL_is_Connected(m_Server) );	}
 
-	bool					is_Connected	(void)	const	{	return( ::is_Connected(m_Server) );	}
 
 private:
 
     int						m_Type;
 
-	CSG_String				m_Value, m_Server;
+	CSG_String				m_Value, m_Server, m_Username, m_Password;
 
 };
 
@@ -231,6 +353,8 @@ private:
 //---------------------------------------------------------
 BEGIN_EVENT_TABLE(CData_Source_PgSQL, wxTreeCtrl)
 	EVT_MENU					(ID_CMD_DB_REFRESH				, CData_Source_PgSQL::On_Refresh)
+	EVT_MENU					(ID_CMD_DB_SOURCE_CREATE		, CData_Source_PgSQL::On_Source_Create)
+	EVT_MENU					(ID_CMD_DB_SOURCE_DROP			, CData_Source_PgSQL::On_Source_Drop)
 	EVT_MENU					(ID_CMD_DB_SOURCE_OPEN			, CData_Source_PgSQL::On_Source_Open)
 	EVT_MENU					(ID_CMD_DB_SOURCE_CLOSE			, CData_Source_PgSQL::On_Source_Close)
 	EVT_MENU					(ID_CMD_DB_SOURCE_CLOSE_ALL		, CData_Source_PgSQL::On_Sources_Close)
@@ -261,49 +385,120 @@ CData_Source_PgSQL::CData_Source_PgSQL(wxWindow *pParent)
 	: wxTreeCtrl(pParent, ID_WND_DATA_SOURCE_DATABASE)
 {
 	AssignImageList(new wxImageList(IMG_SIZE_TREECTRL, IMG_SIZE_TREECTRL, true, 0));
-	IMG_ADD_TO_TREECTRL(ID_IMG_WKSP_DB_SOURCES);		// IMG_ROOT
-	IMG_ADD_TO_TREECTRL(ID_IMG_WKSP_DB_SOURCE_OFF);		// IMG_SRC_CLOSED
-	IMG_ADD_TO_TREECTRL(ID_IMG_WKSP_DB_SOURCE_ON);		// IMG_SRC_OPENED
-	IMG_ADD_TO_TREECTRL(ID_IMG_WKSP_DB_TABLE);			// IMG_TABLE
-	IMG_ADD_TO_TREECTRL(ID_IMG_WKSP_SHAPES_POINT);		// IMG_POINT
-	IMG_ADD_TO_TREECTRL(ID_IMG_WKSP_SHAPES_POINTS);		// IMG_POINTS
-	IMG_ADD_TO_TREECTRL(ID_IMG_WKSP_SHAPES_LINE);		// IMG_LINE
+	IMG_ADD_TO_TREECTRL(ID_IMG_WKSP_DB_SOURCES    );	// IMG_ROOT
+	IMG_ADD_TO_TREECTRL(ID_IMG_WKSP_DB_SOURCES    );	// IMG_SERVER
+	IMG_ADD_TO_TREECTRL(ID_IMG_WKSP_DB_SOURCE_OFF );	// IMG_SRC_CLOSED
+	IMG_ADD_TO_TREECTRL(ID_IMG_WKSP_DB_SOURCE_ON  );	// IMG_SRC_OPENED
+	IMG_ADD_TO_TREECTRL(ID_IMG_WKSP_DB_TABLE      );	// IMG_TABLE
+	IMG_ADD_TO_TREECTRL(ID_IMG_WKSP_SHAPES_POINT  );	// IMG_POINT
+	IMG_ADD_TO_TREECTRL(ID_IMG_WKSP_SHAPES_POINTS );	// IMG_POINTS
+	IMG_ADD_TO_TREECTRL(ID_IMG_WKSP_SHAPES_LINE   );	// IMG_LINE
 	IMG_ADD_TO_TREECTRL(ID_IMG_WKSP_SHAPES_POLYGON);	// IMG_POLYGON
-	IMG_ADD_TO_TREECTRL(ID_IMG_WKSP_GRID);				// IMG_GRID
+	IMG_ADD_TO_TREECTRL(ID_IMG_WKSP_GRID_MANAGER  );	// IMG_GRIDS
+	IMG_ADD_TO_TREECTRL(ID_IMG_WKSP_GRID          );	// IMG_GRID
 
 	AddRoot(_TL("PostgreSQL Sources"), IMG_ROOT, IMG_ROOT, new CData_Source_PgSQL_Data(TYPE_ROOT));
 
 	//-----------------------------------------------------
+	SG_UI_Msg_Lock(true);
+
 	wxString	Server;
 
 	for(int i=0; CONFIG_Read(CFG_PGSQL_DIR, wxString::Format(CFG_PGSQL_SRC, i), Server); i++)
 	{
-		Update_Source(AppendItem(GetRootItem(), Server.c_str(), IMG_SRC_CLOSED, IMG_SRC_CLOSED,
-			new CData_Source_PgSQL_Data(TYPE_SOURCE, &Server, &Server)
-		));
+		wxString	User, Password;
+
+		if( Server.Find("|") > 0 )
+		{
+			User     = Server.AfterFirst ('|').BeforeFirst('|');
+			Password = Server.AfterLast  ('|');
+			Server   = Server.BeforeFirst('|');
+		}
+
+		CData_Source_PgSQL_Data	*pData	= new CData_Source_PgSQL_Data(TYPE_SOURCE, &Server, &Server, &User, &Password);
+
+		Update_Source(AppendItem(Get_Server_Item(Server, true), pData->Get_DBName().c_str(), IMG_SRC_CLOSED, IMG_SRC_CLOSED, pData));
 	}
 
 	Update_Sources();
+
+	SG_UI_Msg_Lock(false);
 }
 
 //---------------------------------------------------------
 CData_Source_PgSQL::~CData_Source_PgSQL(void)
 {
-	wxTreeItemIdValue	Cookie;
-	wxTreeItemId		Item	= GetFirstChild(GetRootItem(), Cookie);
+	long Reopen	= 0;
+
+	CONFIG_Read("/DATA", "PROJECT_DB_REOPEN", Reopen);
 
 	CONFIG_Delete(CFG_PGSQL_DIR);
 
-	for(int i=0; Item.IsOk(); )
-	{
-		CData_Source_PgSQL_Data	*pData	= (CData_Source_PgSQL_Data *)GetItemData(Item);
+	wxTreeItemIdValue srvCookie; wxTreeItemId srvItem = GetFirstChild(GetRootItem(), srvCookie);
 
-		if( pData && pData->Get_Type() == TYPE_SOURCE )
+	for(int i=0; srvItem.IsOk(); )
+	{
+		wxTreeItemIdValue Cookie; wxTreeItemId Item = GetFirstChild(srvItem, Cookie);
+
+		while( Item.IsOk() )
 		{
-			CONFIG_Write(CFG_PGSQL_DIR, wxString::Format(CFG_PGSQL_SRC, i++), pData->Get_Server().c_str());
+			CData_Source_PgSQL_Data	*pData	= (CData_Source_PgSQL_Data *)GetItemData(Item);
+
+			if( pData && pData->Get_Type() == TYPE_SOURCE )
+			{
+				CSG_String	Connection	= pData->Get_Server().c_str();
+
+				if( Reopen != 0 && pData->is_Connected() && !pData->Get_Username().is_Empty() )	// store user and password
+				{
+					Connection	+= "|" + pData->Get_Username() + "|" + pData->Get_Password();
+				}
+
+				CONFIG_Write(CFG_PGSQL_DIR, wxString::Format(CFG_PGSQL_SRC, i++), Connection.c_str());
+			}
+
+			Item	= GetNextChild(Item, Cookie);
 		}
 
-		Item	= GetNextChild(Item, Cookie);
+		srvItem	= GetNextChild(srvItem, srvCookie);
+	}
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+void CData_Source_PgSQL::Autoconnect(void)
+{
+	long Reopen	= 0;
+
+	CONFIG_Read("/DATA", "PROJECT_DB_REOPEN", Reopen);
+
+	if( Reopen != 0 )
+	{
+		wxTreeItemIdValue srvCookie; wxTreeItemId srvItem = GetFirstChild(GetRootItem(), srvCookie);
+
+		while( srvItem.IsOk() )
+		{
+			wxTreeItemIdValue Cookie; wxTreeItemId Item = GetFirstChild(srvItem, Cookie);
+
+			while( Item.IsOk() )
+			{
+				CData_Source_PgSQL_Data	*pData	= Item.IsOk() ? (CData_Source_PgSQL_Data *)GetItemData(Item) : NULL; if( pData == NULL )	return;
+
+				if( pData->Get_Type() == TYPE_SOURCE && !pData->Get_Username().is_Empty() )
+				{
+					Source_Open(pData, false);
+				}
+
+				Item	= GetNextChild(Item, Cookie);
+			}
+
+			srvItem	= GetNextChild(srvItem, srvCookie);
+		}
 	}
 }
 
@@ -320,6 +515,18 @@ void CData_Source_PgSQL::On_Refresh(wxCommandEvent &WXUNUSED(event))
 	Update_Item(GetSelection());
 }
 
+//---------------------------------------------------------
+void CData_Source_PgSQL::On_Source_Create(wxCommandEvent &WXUNUSED(event))
+{
+	Source_Create(GetSelection());
+}
+
+//---------------------------------------------------------
+void CData_Source_PgSQL::On_Source_Drop(wxCommandEvent &WXUNUSED(event))
+{
+	Source_Drop(GetSelection());
+}
+
 //---------------------------------------------------------
 void CData_Source_PgSQL::On_Source_Open(wxCommandEvent &WXUNUSED(event))
 {
@@ -386,12 +593,17 @@ void CData_Source_PgSQL::On_Item_Activated(wxTreeEvent &event)
 		Update_Sources();
 		break;
 
+	case TYPE_SERVER:
+		Update_Sources(event.GetItem());
+		break;
+
 	case TYPE_SOURCE:
 		Source_Open(event.GetItem());
 		break;
 
 	case TYPE_TABLE:
 	case TYPE_SHAPES:
+	case TYPE_GRIDS:
 	case TYPE_GRID:
 		Table_Open(event.GetItem());
 		break;
@@ -417,6 +629,14 @@ void CData_Source_PgSQL::On_Item_Menu(wxTreeEvent &event)
 	{
 	case TYPE_ROOT:
 		CMD_Menu_Add_Item(&Menu, false, ID_CMD_DB_REFRESH);
+		CMD_Menu_Add_Item(&Menu, false, ID_CMD_DB_SOURCE_CREATE);
+		CMD_Menu_Add_Item(&Menu, false, ID_CMD_DB_SOURCE_OPEN);
+		CMD_Menu_Add_Item(&Menu, false, ID_CMD_DB_SOURCE_CLOSE_ALL);
+		break;
+
+	case TYPE_SERVER:
+		CMD_Menu_Add_Item(&Menu, false, ID_CMD_DB_REFRESH);
+		CMD_Menu_Add_Item(&Menu, false, ID_CMD_DB_SOURCE_CREATE);
 		CMD_Menu_Add_Item(&Menu, false, ID_CMD_DB_SOURCE_OPEN);
 		CMD_Menu_Add_Item(&Menu, false, ID_CMD_DB_SOURCE_CLOSE_ALL);
 		break;
@@ -431,6 +651,7 @@ void CData_Source_PgSQL::On_Item_Menu(wxTreeEvent &event)
 		else
 		{
 			CMD_Menu_Add_Item(&Menu, false, ID_CMD_DB_REFRESH);
+			CMD_Menu_Add_Item(&Menu, false, ID_CMD_DB_SOURCE_DROP);
 			CMD_Menu_Add_Item(&Menu, false, ID_CMD_DB_SOURCE_CLOSE);
 			CMD_Menu_Add_Item(&Menu, false, ID_CMD_DB_SOURCE_DELETE);
 			CMD_Menu_Add_Item(&Menu, false, ID_CMD_DB_TABLE_FROM_QUERY);
@@ -439,12 +660,18 @@ void CData_Source_PgSQL::On_Item_Menu(wxTreeEvent &event)
 
 	case TYPE_TABLE:
 	case TYPE_SHAPES:
-	case TYPE_GRID:
+	case TYPE_GRIDS:
 		CMD_Menu_Add_Item(&Menu, false, ID_CMD_DB_TABLE_OPEN);
 		CMD_Menu_Add_Item(&Menu, false, ID_CMD_DB_TABLE_RENAME);
 		CMD_Menu_Add_Item(&Menu, false, ID_CMD_DB_TABLE_DELETE);
 		CMD_Menu_Add_Item(&Menu, false, ID_CMD_DB_TABLE_INFO);
 		break;
+
+	case TYPE_GRID:
+		CMD_Menu_Add_Item(&Menu, false, ID_CMD_DB_TABLE_OPEN);
+		CMD_Menu_Add_Item(&Menu, false, ID_CMD_DB_TABLE_RENAME);
+		CMD_Menu_Add_Item(&Menu, false, ID_CMD_DB_TABLE_DELETE);
+		break;
 	}
 
 	if( Menu.GetMenuItemCount() > 0 )
@@ -460,16 +687,49 @@ void CData_Source_PgSQL::On_Item_Menu(wxTreeEvent &event)
 //														 //
 ///////////////////////////////////////////////////////////
 
+//---------------------------------------------------------
+wxTreeItemId CData_Source_PgSQL::Get_Server_Item(const wxString &Server, bool bCreate)
+{
+	wxString	Name	= Server.AfterFirst('[').BeforeFirst(']');
+
+	wxTreeItemIdValue Cookie; wxTreeItemId Item = GetFirstChild(GetRootItem(), Cookie);
+
+	while( Item.IsOk() )
+	{
+		if( !Name.Cmp(GetItemText(Item)) )
+		{
+			return( Item );
+		}
+
+		Item	= GetNextChild(Item, Cookie);
+	}
+
+	if( bCreate )
+	{
+		Item	= AppendItem(GetRootItem(), Name, IMG_SERVER, IMG_SERVER, new CData_Source_PgSQL_Data(TYPE_SERVER, &Name, &Name));
+
+		SortChildren(GetRootItem());
+		Expand      (GetRootItem());
+	}
+
+	return( Item );
+}
+
 //---------------------------------------------------------
 wxTreeItemId CData_Source_PgSQL::Find_Source(const wxString &Server)
 {
-	wxTreeItemIdValue	Cookie;
+	wxTreeItemId	Item	= Get_Server_Item(Server, false);
 
-	wxTreeItemId	Item	= GetFirstChild(GetRootItem(), Cookie);
-
-	while( Item.IsOk() && Server.Cmp(GetItemText(Item)) )
+	if( Item.IsOk() )
 	{
-		Item	= GetNextChild(Item, Cookie);
+		wxTreeItemIdValue Cookie; Item = GetFirstChild(Item, Cookie);
+
+		wxString	Name	= Server.BeforeLast('['); Name.Trim(true);
+
+		while( Item.IsOk() && Name.Cmp(GetItemText(Item)) )
+		{
+			Item	= GetNextChild(Item, Cookie);
+		}
 	}
 
 	return( Item );
@@ -483,19 +743,18 @@ void CData_Source_PgSQL::Update_Item(const wxTreeItemId &Item)
 	switch( pData->Get_Type() )
 	{
 	case TYPE_ROOT:		Update_Sources();		break;
-	case TYPE_SOURCE:	Update_Source(Item);	break;
+	case TYPE_SERVER:	Update_Sources(Item);	break;
+	case TYPE_SOURCE:	Update_Source (Item);	break;
 	}
 }
 
 //---------------------------------------------------------
-void CData_Source_PgSQL::Update_Sources(void)
+void CData_Source_PgSQL::Update_Sources(const wxTreeItemId &Root)
 {
 	Freeze();
 
 	//-----------------------------------------------------
-	wxTreeItemIdValue	Cookie;
-
-	wxTreeItemId	Item	= GetFirstChild(GetRootItem(), Cookie);
+	wxTreeItemIdValue Cookie; wxTreeItemId Item = GetFirstChild(Root, Cookie);
 
 	while( Item.IsOk() )
 	{
@@ -505,10 +764,9 @@ void CData_Source_PgSQL::Update_Sources(void)
 	}
 
 	//-----------------------------------------------------
-	bool		bResult;
 	CSG_Table	Connections;
 
-	RUN_MODULE(DB_PGSQL_Get_Connections, bResult, SET_PARAMETER("CONNECTIONS", &Connections));	// CGet_Connections
+	RUN_MODULE(DB_PGSQL_Get_Connections, false, SET_PARAMETER("CONNECTIONS", &Connections));	// CGet_Connections
 
 	for(int i=0; i<Connections.Get_Count(); i++)
 	{
@@ -519,12 +777,25 @@ void CData_Source_PgSQL::Update_Sources(void)
 	}
 
 	//-----------------------------------------------------
-	SortChildren(GetRootItem());
-	Expand      (GetRootItem());
+	SortChildren(Root);
+	Expand      (Root);
 
 	Thaw();
 }
 
+//---------------------------------------------------------
+void CData_Source_PgSQL::Update_Sources(void)
+{
+	wxTreeItemIdValue Cookie; wxTreeItemId Item = GetFirstChild(GetRootItem(), Cookie);
+
+	while( Item.IsOk() )
+	{
+		Update_Sources(Item);
+
+		Item	= GetNextChild(Item, Cookie);
+	}
+}
+
 //---------------------------------------------------------
 void CData_Source_PgSQL::Update_Source(const wxString &Server)
 {
@@ -537,11 +808,11 @@ void CData_Source_PgSQL::Update_Source(const wxString &Server)
 
 	wxTreeItemId	Item	= Find_Source(Server);
 
-	if( !Item.IsOk() && is_Connected(&Server) )
+	if( !Item.IsOk() && PGSQL_is_Connected(&Server) )
 	{
-		Item	= AppendItem(GetRootItem(), Server.c_str(), IMG_SRC_OPENED, IMG_SRC_OPENED,
-			new CData_Source_PgSQL_Data(TYPE_SOURCE, &Server, &Server)
-		);
+		CData_Source_PgSQL_Data	*pData	= new CData_Source_PgSQL_Data(TYPE_SOURCE, &Server, &Server);
+
+		Item	= AppendItem(Get_Server_Item(Server, true), pData->Get_DBName().c_str(), IMG_SRC_OPENED, IMG_SRC_OPENED, pData);
 	}
 
 	Update_Source(Item);
@@ -572,10 +843,9 @@ void CData_Source_PgSQL::Update_Source(const wxTreeItemId &Item)
 		SetItemImage(Item, IMG_SRC_OPENED, wxTreeItemIcon_Normal);
 		SetItemImage(Item, IMG_SRC_OPENED, wxTreeItemIcon_Selected);
 
-		bool		bResult;
 		CSG_Table	Tables;
 
-		RUN_MODULE(DB_PGSQL_Table_List, bResult,	// CTable_List
+		RUN_MODULE(DB_PGSQL_Table_List, false,	// CTable_List
 				SET_PARAMETER("CONNECTION", pData->Get_Value())
 			&&	SET_PARAMETER("TABLES"    , &Tables)
 		);
@@ -599,11 +869,11 @@ void CData_Source_PgSQL::Update_Source(const wxTreeItemId &Item)
 				case SHAPE_TYPE_Polygon: Append_Table(Item, Tables[i].asString(0), TYPE_SHAPES, IMG_POLYGON); break;
 				}
 			}
-			else if( !s.Cmp("RASTER" ) ) Append_Table(Item, Tables[i].asString(0), TYPE_GRID  , IMG_GRID   );
+			else if( !s.Cmp("RASTER" ) ) Append_Table(Item, Tables[i].asString(0), TYPE_GRIDS , IMG_GRIDS  );
 			else if( !s.Cmp("TABLE"  ) ) Append_Table(Item, Tables[i].asString(0), TYPE_TABLE , IMG_TABLE  );
 		}
 
-		Expand      (Item);
+		Expand(Item);
 	}
 
 	Thaw();
@@ -614,7 +884,29 @@ void CData_Source_PgSQL::Append_Table(const wxTreeItemId &Parent, const SG_Char
 {
 	CData_Source_PgSQL_Data	*pData	= Parent.IsOk() ? (CData_Source_PgSQL_Data *)GetItemData(Parent) : NULL; if( pData == NULL )	return;
 
-	AppendItem(Parent, Name, Image, Image, new CData_Source_PgSQL_Data(Type, Name, pData->Get_Server()));
+	wxTreeItemId	Item	= AppendItem(Parent, Name, Image, Image, new CData_Source_PgSQL_Data(Type, Name, pData->Get_Server()));
+
+	if( Type == TYPE_GRIDS )
+	{
+		CSG_Table	Grids;
+
+		RUN_MODULE(DB_PGSQL_Table_Query, false,	// CTable_Query
+				SET_PARAMETER("CONNECTION", pData->Get_Server())
+			&&	SET_PARAMETER("TABLES"    , Name)
+			&&	SET_PARAMETER("TABLE"     , &Grids)
+			&&  SET_PARAMETER("FIELDS"    , SG_T("rid, name"))
+		);
+
+		if( bResult )
+		{
+			for(int i=0; i<Grids.Get_Count(); i++)
+			{
+				AppendItem(Item, Grids[i].asString(1), IMG_GRID, IMG_GRID,
+					new CData_Source_PgSQL_Data(TYPE_GRID, CSG_String::Format("%s:rid=%d", Name, Grids[i].asInt(0)), pData->Get_Server())
+				);
+			}
+		}
+	}
 }
 
 
@@ -624,49 +916,135 @@ void CData_Source_PgSQL::Append_Table(const wxTreeItemId &Parent, const SG_Char
 //														 //
 ///////////////////////////////////////////////////////////
 
+//---------------------------------------------------------
+bool CData_Source_PgSQL::Source_Create(const wxTreeItemId &Item)
+{
+	CData_Source_PgSQL_Data	*pData	= Item.IsOk() ? (CData_Source_PgSQL_Data *)GetItemData(Item) : NULL; if( pData == NULL )	return( false );
+
+	if( pData->Get_Type() == TYPE_ROOT
+	||  pData->Get_Type() == TYPE_SERVER )
+	{
+		CSG_Module	*pModule	= SG_Get_Module_Library_Manager().Get_Module("db_pgsql", DB_PGSQL_DB_Create);
+
+		if(	pModule && pModule->On_Before_Execution() )
+		{
+			if( pData->Get_Type() == TYPE_SERVER )
+			{
+				pModule->Set_Parameter("PG_HOST", pData->Get_Host());
+				pModule->Set_Parameter("PG_PORT", pData->Get_Port());
+			}
+
+			if( DLG_Parameters(pModule->Get_Parameters()) )
+			{
+				pModule->Execute();
+			}
+		}
+	}
+
+	return( true );
+}
+
+//---------------------------------------------------------
+bool CData_Source_PgSQL::Source_Drop(const wxTreeItemId &Item)
+{
+	static	wxString	Username = "postgres", Password = "postgres";
+
+	CData_Source_PgSQL_Data	*pData	= Item.IsOk() ? (CData_Source_PgSQL_Data *)GetItemData(Item) : NULL; if( pData == NULL )	return( false );
+
+	if( !DLG_Login(Username, Password, _TL("Drop Database")) )
+	{
+		return( false );
+	}
+
+	pData->Set_Username(Username);
+	pData->Set_Password(Password);
+
+	if( pData->Get_Type() == TYPE_SOURCE && pData->is_Connected() )
+	{
+		RUN_MODULE(DB_PGSQL_DB_Drop, true,	// CDatabase_Drop
+				SET_PARAMETER("PG_HOST", pData->Get_Host    ())
+			&&	SET_PARAMETER("PG_PORT", pData->Get_Port    ())
+			&&	SET_PARAMETER("PG_NAME", pData->Get_DBName  ())
+			&&	SET_PARAMETER("PG_USER", pData->Get_Username())
+			&&	SET_PARAMETER("PG_PWD" , pData->Get_Password())
+		);
+
+		return( bResult );
+	}
+
+	return( false );
+}
+
+//---------------------------------------------------------
+bool CData_Source_PgSQL::Source_Open(CData_Source_PgSQL_Data *pData, bool bDialog)
+{
+	static	wxString	Username = "postgres", Password = "postgres";
+
+	if( bDialog )
+	{
+		if( !DLG_Login(Username, Password) )
+		{
+			return( false );
+		}
+
+		pData->Set_Username(Username);
+		pData->Set_Password(Password);
+	}
+
+	//-----------------------------------------------------
+	MSG_General_Add(wxString::Format("%s: %s...", _TL("Connect to Database"), pData->Get_Server().c_str()), true, true);
+
+	RUN_MODULE(DB_PGSQL_Get_Connection, false,	// CGet_Connection
+			SET_PARAMETER("PG_HOST", pData->Get_Host    ())
+		&&	SET_PARAMETER("PG_PORT", pData->Get_Port    ())
+		&&	SET_PARAMETER("PG_NAME", pData->Get_DBName  ())
+		&&	SET_PARAMETER("PG_USER", pData->Get_Username())
+		&&	SET_PARAMETER("PG_PWD" , pData->Get_Password())
+	);
+
+	if( bResult )
+	{
+		MSG_General_Add(_TL("okay"), false, false, SG_UI_MSG_STYLE_SUCCESS);
+
+		return( true );
+	}
+
+	MSG_General_Add(_TL("failed"), false, false, SG_UI_MSG_STYLE_FAILURE);
+
+	return( false );
+}
+
 //---------------------------------------------------------
 void CData_Source_PgSQL::Source_Open(const wxTreeItemId &Item)
 {
 	CData_Source_PgSQL_Data	*pData	= Item.IsOk() ? (CData_Source_PgSQL_Data *)GetItemData(Item) : NULL; if( pData == NULL )	return;
 
-	if( pData->Get_Type() == TYPE_ROOT )
+	if( pData->Get_Type() == TYPE_ROOT
+	||  pData->Get_Type() == TYPE_SERVER )
 	{
-		CSG_Module	*pModule	= SG_Get_Module_Library_Manager().Get_Module(SG_T("db_pgsql"), DB_PGSQL_Get_Connection);	// CGet_Connection
+		CSG_Module	*pModule	= SG_Get_Module_Library_Manager().Get_Module("db_pgsql", DB_PGSQL_Get_Connection);	// CGet_Connection
 
-		if(	pModule && pModule->On_Before_Execution() && DLG_Parameters(pModule->Get_Parameters()) )
+		if(	pModule && pModule->On_Before_Execution() )
 		{
-			pModule->Execute();
+			if( pData->Get_Type() == TYPE_SERVER )
+			{
+				pModule->Set_Parameter("PG_HOST", pData->Get_Host());
+				pModule->Set_Parameter("PG_PORT", pData->Get_Port());
+			}
+
+			if( DLG_Parameters(pModule->Get_Parameters()) )
+			{
+				pModule->Execute();
+			}
 		}
 	}
 	else if( pData->is_Connected() )
 	{
 		Update_Source(Item);
 	}
-	else
+	else if( !Source_Open(pData, true) )
 	{
-		static	wxString	Username = "postgres", Password = "postgres";
-
-		if( DLG_Login(Username, Password) )
-		{
-			bool	bResult;
-
-			CSG_String	Host	= pData->Get_Server().AfterLast ('[').BeforeFirst(':');
-			CSG_String	Port	= pData->Get_Server().AfterLast (':').BeforeFirst(']');
-			CSG_String	Name	= pData->Get_Server().BeforeLast('['); Name.Trim(true);
-
-			RUN_MODULE(DB_PGSQL_Get_Connection, bResult,	// CGet_Connection
-					SET_PARAMETER("PG_HOST", Host)
-				&&	SET_PARAMETER("PG_PORT", Port)
-				&&	SET_PARAMETER("PG_NAME", Name)
-				&&	SET_PARAMETER("PG_USER", Username)
-				&&	SET_PARAMETER("PG_PWD" , Password)
-			);
-
-			if( !bResult )
-			{
-				DLG_Message_Show_Error(_TL("Could not connect to data source."), _TL("Connect to PostgreSQL"));
-			}
-		}
+		DLG_Message_Show_Error(_TL("Could not connect to data source."), _TL("Connect to PostgreSQL"));
 	}
 }
 
@@ -677,31 +1055,24 @@ void CData_Source_PgSQL::Source_Close(const wxTreeItemId &Item, bool bDelete)
 
 	if( pData->is_Connected() )
 	{
-		CSG_Module	*pModule	= SG_Get_Module_Library_Manager().Get_Module(SG_T("db_pgsql"), DB_PGSQL_Del_Connection);	// CDel_Connection
-
-		if( pModule )
-		{
-			pModule->On_Before_Execution();
-			pModule->Get_Parameters()->Set_Parameter("CONNECTION", pData->Get_Server());
-			pModule->Execute();
-		}
+		RUN_MODULE(DB_PGSQL_Del_Connection, true, SET_PARAMETER("CONNECTION", pData->Get_Server()));
 	}
 
 	if( bDelete )
 	{
 		Delete(Item);
 	}
+	else
+	{
+		pData->Set_Username(SG_T(""));
+		pData->Set_Password(SG_T(""));
+	}
 }
 
 //---------------------------------------------------------
 void CData_Source_PgSQL::Sources_Close(void)
 {
-	CSG_Module	*pModule	= SG_Get_Module_Library_Manager().Get_Module(SG_T("db_pgsql"), DB_PGSQL_Del_Connections);	// CDel_Connections
-
-	if( pModule )
-	{
-		pModule->Execute();
-	}
+	RUN_MODULE(DB_PGSQL_Del_Connections, true, true);
 }
 
 
@@ -721,18 +1092,14 @@ void CData_Source_PgSQL::Table_Open(const wxTreeItemId &Item)
 	{
 		CSG_Table	*pTable	= SG_Create_Table();
 
-		bool	bResult;
-
-		RUN_MODULE(DB_PGSQL_Table_Load, bResult,	// CTable_Load
+		RUN_MODULE(DB_PGSQL_Table_Load, true,	// CTable_Load
 				SET_PARAMETER("CONNECTION", pData->Get_Server())
-			&&	SET_PARAMETER("TABLES"    , pData->Get_Value())
+			&&	SET_PARAMETER("TABLES"    , pData->Get_Value ())
 			&&	SET_PARAMETER("TABLE"     , pTable)
 		);
 
 		if( bResult )
 		{
-			SG_Get_Data_Manager().Add(pTable);
-
 			g_pData->Show(pTable, 0);
 		}
 		else
@@ -746,18 +1113,14 @@ void CData_Source_PgSQL::Table_Open(const wxTreeItemId &Item)
 	{
 		CSG_Shapes	*pShapes	= SG_Create_Shapes();
 
-		bool	bResult;
-
-		RUN_MODULE(DB_PGSQL_Shapes_Load, bResult,	// CPGIS_Shapes_Load
+		RUN_MODULE(DB_PGSQL_Shapes_Load, true,	// CPGIS_Shapes_Load
 				SET_PARAMETER("CONNECTION", pData->Get_Server())
-			&&	SET_PARAMETER("TABLES"    , pData->Get_Value())
+			&&	SET_PARAMETER("TABLES"    , pData->Get_Value ())
 			&&	SET_PARAMETER("SHAPES"    , pShapes)
 		);
 
 		if( bResult )
 		{
-			SG_Get_Data_Manager().Add(pShapes);
-
 		//	g_pData->Show(pShapes, SG_UI_DATAOBJECT_SHOW_NEW_MAP);
 		}
 		else
@@ -766,18 +1129,23 @@ void CData_Source_PgSQL::Table_Open(const wxTreeItemId &Item)
 		}
 	}
 
+	//-----------------------------------------------------
+	if( pData->Get_Type() == TYPE_GRIDS )
+	{
+		RUN_MODULE(DB_PGSQL_Raster_Load, true,
+				SET_PARAMETER("CONNECTION", pData->Get_Server())
+			&&	SET_PARAMETER("TABLES"    , pData->Get_Value ())
+		);
+	}
+
 	//-----------------------------------------------------
 	if( pData->Get_Type() == TYPE_GRID )
 	{
-		CSG_Module	*pModule	= SG_Get_Module_Library_Manager().Get_Module(SG_T("db_pgsql"), DB_PGSQL_Raster_Load);	// CPGIS_Raster_Load
-
-		if( pModule )
-		{
-			pModule->On_Before_Execution();
-			pModule->Get_Parameters()->Set_Parameter("CONNECTION", pData->Get_Server());
-			pModule->Get_Parameters()->Set_Parameter("TABLES"    , pData->Get_Value ());
-			pModule->Execute();
-		}
+		RUN_MODULE(DB_PGSQL_Raster_Load, true,
+				SET_PARAMETER("CONNECTION", pData->Get_Server())
+			&&	SET_PARAMETER("TABLES"    , pData->Get_Value ().BeforeFirst(':'))
+			&&	SET_PARAMETER("WHERE"     , pData->Get_Value ().AfterFirst (':'))
+		);
 	}
 }
 
@@ -786,23 +1154,47 @@ void CData_Source_PgSQL::Table_Rename(const wxTreeItemId &Item)
 {
 	CData_Source_PgSQL_Data	*pData	= Item.IsOk() ? (CData_Source_PgSQL_Data *)GetItemData(Item) : NULL; if( pData == NULL )	return;
 
-	wxString	Name	= pData->Get_Value().c_str();
+	wxString	Name	= GetItemText(Item);
 
-	if( DLG_Get_Text(Name, _TL("Rename Table"), _TL("Name")) )
+	switch( pData->Get_Type() )
 	{
-		CSG_String	SQL	= "ALTER TABLE \"" + pData->Get_Value() + "\" RENAME TO \"" + CSG_String(&Name) + "\";";
-
-		bool	bResult;
-
-		RUN_MODULE(DB_PGSQL_Execute_SQL, bResult,
-				SET_PARAMETER("CONNECTION", pData->Get_Server())
-			&&	SET_PARAMETER("SQL"       , SQL)
-		);
-
-		if( bResult )
+	//-----------------------------------------------------
+	case TYPE_GRID:
+		if( DLG_Get_Text(Name, _TL("Rename Raster Band"), _TL("Name")) )
 		{
-			SetItemText(Item, Name);
+			CSG_String	Table	= pData->Get_Value().BeforeFirst(':');
+			CSG_String	rid		= pData->Get_Value().AfterFirst (':');
+			CSG_String	SQL	= "UPDATE \"" + Table + "\" SET name='" + CSG_String(&Name) + "' WHERE " + rid + ";";
+
+			RUN_MODULE(DB_PGSQL_Execute_SQL, false,
+					SET_PARAMETER("CONNECTION", pData->Get_Server())
+				&&	SET_PARAMETER("SQL"       , SQL)
+			);
+
+			if( bResult )
+			{
+				SetItemText(Item, Name);
+			}
 		}
+		break;
+
+	//-----------------------------------------------------
+	default:
+		if( DLG_Get_Text(Name, _TL("Rename Table"), _TL("Name")) )
+		{
+			CSG_String	SQL	= "ALTER TABLE \"" + pData->Get_Value() + "\" RENAME TO \"" + CSG_String(&Name) + "\";";
+
+			RUN_MODULE(DB_PGSQL_Execute_SQL, false,
+					SET_PARAMETER("CONNECTION", pData->Get_Server())
+				&&	SET_PARAMETER("SQL"       , SQL)
+			);
+
+			if( bResult )
+			{
+				SetItemText(Item, Name);
+			}
+		}
+		break;
 	}
 }
 
@@ -811,19 +1203,21 @@ void CData_Source_PgSQL::Table_Info(const wxTreeItemId &Item)
 {
 	CData_Source_PgSQL_Data	*pData	= Item.IsOk() ? (CData_Source_PgSQL_Data *)GetItemData(Item) : NULL; if( pData == NULL )	return;
 
-	CSG_Module	*pModule	= SG_Get_Module_Library_Manager().Get_Module(SG_T("db_pgsql"), DB_PGSQL_Table_Info);
+	CSG_Table	*pTable	= SG_Create_Table();
 
-	if( pModule )
+	RUN_MODULE(DB_PGSQL_Table_Info, true,	// CTable_Info
+			SET_PARAMETER("CONNECTION", pData->Get_Server())
+		&&	SET_PARAMETER("TABLES"    , pData->Get_Value ())
+		&&	SET_PARAMETER("TABLE"     , pTable)
+	);
+
+	if( bResult )
 	{
-		pModule->On_Before_Execution();
-		pModule->Get_Parameters()->Set_Parameter("CONNECTION", pData->Get_Server());
-		pModule->Get_Parameters()->Set_Parameter("TABLES"    , pData->Get_Value ());
-		pModule->Get_Parameters()->Set_Parameter("TABLE"     , DATAOBJECT_NOTSET  );
-		
-		if( pModule->Execute() )
-		{
-			g_pData->Show(pModule->Get_Parameters()->Get_Parameter("TABLE")->asTable());
-		}
+		g_pData->Show(pTable, 0);
+	}
+	else
+	{
+		delete(pTable);
 	}
 }
 
@@ -832,27 +1226,61 @@ void CData_Source_PgSQL::Table_Drop(const wxTreeItemId &Item)
 {
 	CData_Source_PgSQL_Data	*pData	= Item.IsOk() ? (CData_Source_PgSQL_Data *)GetItemData(Item) : NULL; if( pData == NULL )	return;
 
-	if( DLG_Message_Confirm(wxString::Format(wxT("%s [%s]"), _TL("Do you really want to delete the table"), pData->Get_Value().c_str()), _TL("Table Deletion")) )
+	wxString	Name	= GetItemText(Item);
+
+	switch( pData->Get_Type() )
 	{
-		MSG_General_Add(wxString::Format(wxT("%s: [%s] %s..."), _TL("Deleting table"), pData->Get_Server().c_str(), pData->Get_Value().c_str()), true, true);
-
-		bool	bResult;
-
-		RUN_MODULE(DB_PGSQL_Table_Drop, bResult,	// CTable_Drop
-				SET_PARAMETER("CONNECTION", pData->Get_Server())
-			&&	SET_PARAMETER("TABLES"    , pData->Get_Value())
-		);
-
-		if( bResult )
+	//-----------------------------------------------------
+	case TYPE_GRID:
+		if( DLG_Message_Confirm(wxString::Format("%s [%s]", _TL("Do you really want to delete this raster band"), Name.c_str()), _TL("Raster Band Deletion")) )
 		{
-			Delete(Item);
+			MSG_General_Add(wxString::Format("%s: [%s] %s...", _TL("Deleting raster band"), pData->Get_Server().c_str(), Name.c_str()), true, true);
 
-			MSG_General_Add(_TL("okay"), false, false, SG_UI_MSG_STYLE_SUCCESS);
+			CSG_String	Table	= pData->Get_Value().BeforeFirst(':');
+			CSG_String	rid		= pData->Get_Value().AfterFirst (':');
+			CSG_String	SQL	= "DELETE FROM \"" + Table + "\" WHERE " + rid + ";";
+
+			RUN_MODULE(DB_PGSQL_Execute_SQL, false,
+					SET_PARAMETER("CONNECTION", pData->Get_Server())
+				&&	SET_PARAMETER("SQL"       , SQL)
+			);
+
+			if( bResult )
+			{
+				Delete(Item);
+
+				MSG_General_Add(_TL("okay"), false, false, SG_UI_MSG_STYLE_SUCCESS);
+			}
+			else
+			{
+				MSG_General_Add(_TL("failed"), false, false, SG_UI_MSG_STYLE_FAILURE);
+			}
 		}
-		else
+		break;
+
+	//-----------------------------------------------------
+	default:
+		if( DLG_Message_Confirm(wxString::Format("%s [%s]", _TL("Do you really want to delete the table"), pData->Get_Value().c_str()), _TL("Table Deletion")) )
 		{
-			MSG_General_Add(_TL("failed"), false, false, SG_UI_MSG_STYLE_FAILURE);
+			MSG_General_Add(wxString::Format("%s: [%s] %s...", _TL("Deleting table"), pData->Get_Server().c_str(), pData->Get_Value().c_str()), true, true);
+
+			RUN_MODULE(DB_PGSQL_Table_Drop, false,	// CTable_Drop
+					SET_PARAMETER("CONNECTION", pData->Get_Server())
+				&&	SET_PARAMETER("TABLES"    , pData->Get_Value())
+			);
+
+			if( bResult )
+			{
+				Delete(Item);
+
+				MSG_General_Add(_TL("okay"), false, false, SG_UI_MSG_STYLE_SUCCESS);
+			}
+			else
+			{
+				MSG_General_Add(_TL("failed"), false, false, SG_UI_MSG_STYLE_FAILURE);
+			}
 		}
+		break;
 	}
 }
 
