@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: dc_helper.h 2039 2014-03-05 13:26:00Z oconrad $
+ * Version $Id: dc_helper.h 2329 2014-11-05 19:32:43Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -142,8 +142,8 @@ void			Draw_FillRect		(wxDC &dc, wxColour Color, wxRect r);
 void			Draw_Text			(wxDC &dc, int Align, int x, int y              , const wxString &Text);
 void			Draw_Text			(wxDC &dc, int Align, int x, int y, double Angle, const wxString &Text);
 
-void			Draw_Text			(wxDC &dc, int Align, int x, int y              , const wxString &Text, int Effect, wxColour Color);
-void			Draw_Text			(wxDC &dc, int Align, int x, int y, double Angle, const wxString &Text, int Effect, wxColour Color);
+void			Draw_Text			(wxDC &dc, int Align, int x, int y              , const wxString &Text, int Effect, wxColour Effect_Color, int Effect_Size = 1);
+void			Draw_Text			(wxDC &dc, int Align, int x, int y, double Angle, const wxString &Text, int Effect, wxColour Effect_Color, int Effect_Size = 1);
 
 
 ///////////////////////////////////////////////////////////
