@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: dlg_base.cpp 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: dlg_base.cpp 2002 2014-02-19 17:06:08Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -89,6 +89,7 @@ IMPLEMENT_CLASS(CDLG_Base, wxDialog)
 //---------------------------------------------------------
 BEGIN_EVENT_TABLE(CDLG_Base, wxDialog)
 	EVT_SIZE			(CDLG_Base::On_Size)
+	EVT_KEY_DOWN		(CDLG_Base::On_Key)
 
 	EVT_BUTTON			(wxID_OK		, CDLG_Base::On_Ok)
 	EVT_BUTTON			(wxID_CANCEL	, CDLG_Base::On_Cancel)
@@ -162,6 +163,19 @@ void CDLG_Base::On_Size(wxSizeEvent &event)
 	event.Skip();
 }
 
+//---------------------------------------------------------
+void CDLG_Base::On_Key(wxKeyEvent &event)
+{
+	if( event.GetKeyCode() == WXK_RETURN )
+	{
+		_Exit(true);
+	}
+	else
+	{
+		event.Skip();
+	}
+}
+
 //---------------------------------------------------------
 void CDLG_Base::Set_Positions(void)
 {
@@ -175,11 +189,6 @@ void CDLG_Base::Set_Positions(void)
 	Set_Position(wxRect(CONTROL_DIST, CONTROL_DIST, xDivide - 2 * CONTROL_DIST, yTotal));
 }
 
-//---------------------------------------------------------
-void CDLG_Base::Set_Position(wxRect r)
-{
-}
-
 
 ///////////////////////////////////////////////////////////
 //														 //
@@ -214,11 +223,6 @@ void CDLG_Base::_Exit(bool bOk)
 	}
 }
 
-//---------------------------------------------------------
-void CDLG_Base::Save_Changes(void)
-{
-}
-
 
 ///////////////////////////////////////////////////////////
 //														 //
