@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: dlg_list_base.cpp 2153 2014-06-11 11:51:04Z reklov_w $
+ * Version $Id: dlg_list_base.cpp 2447 2015-03-19 14:43:42Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -374,7 +374,7 @@ void CDLG_List_Base::On_Down(wxCommandEvent &event)
 	{
 		Selections.Sort(ArrayInt_CMP_Down);
 
-		if( Selections.Item(0) < m_pAdd->GetCount() - 1 )
+		if( (int)Selections.Item(0) < m_pAdd->GetCount() - 1 )
 		{
 			for(size_t i=0; i<Selections.GetCount(); i++)
 			{
