@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: dlg_table.cpp 2041 2014-03-05 15:08:03Z oconrad $
+ * Version $Id: dlg_table.cpp 2694 2015-11-20 16:43:15Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -69,7 +69,7 @@
 #include "helper.h"
 
 #include "dlg_table.h"
-#include "view_table_control.h"
+#include "dlg_table_control.h"
 
 
 ///////////////////////////////////////////////////////////
@@ -88,6 +88,8 @@ BEGIN_EVENT_TABLE(CDLG_Table, CDLG_Base)
 	EVT_BUTTON			(ID_BTN_LOAD			, CDLG_Table::On_Load)
 	EVT_BUTTON			(ID_BTN_SAVE			, CDLG_Table::On_Save)
 	EVT_BUTTON			(ID_BTN_ADD				, CDLG_Table::On_Add)
+	EVT_BUTTON			(ID_BTN_TABLE_FROM_WKSP	, CDLG_Table::On_WKSP_Load)
+	EVT_BUTTON			(ID_BTN_TABLE_TO_WKSP	, CDLG_Table::On_WKSP_Save)
 	EVT_BUTTON			(ID_BTN_INSERT			, CDLG_Table::On_Insert)
 	EVT_BUTTON			(ID_BTN_DELETE			, CDLG_Table::On_Delete)
 	EVT_BUTTON			(ID_BTN_DELETE_ALL		, CDLG_Table::On_Delete_All)
@@ -105,12 +107,14 @@ END_EVENT_TABLE()
 CDLG_Table::CDLG_Table(CSG_Table *pTable, wxString Caption)
 	: CDLG_Base(-1, Caption)
 {
-	m_pTable_Orig	= pTable;
-	m_pTable		= new CSG_Table(*m_pTable_Orig);
-	m_pControl		= new CVIEW_Table_Control(this, m_pTable, TABLE_CTRL_FIXED_COLS);
+	m_pTable	= pTable;
+	m_pControl	= new CDLG_Table_Control(this, m_pTable);
 
 	Add_Button(ID_BTN_LOAD);
+	Add_Button(ID_BTN_TABLE_FROM_WKSP);
+	Add_Button(-1);
 	Add_Button(ID_BTN_SAVE);
+	Add_Button(ID_BTN_TABLE_TO_WKSP);
 	Add_Button(-1);
 	Add_Button(ID_BTN_ADD);
 	Add_Button(ID_BTN_INSERT);
@@ -128,9 +132,7 @@ CDLG_Table::CDLG_Table(CSG_Table *pTable, wxString Caption)
 
 //---------------------------------------------------------
 CDLG_Table::~CDLG_Table(void)
-{
-	delete(m_pTable);
-}
+{}
 
 
 ///////////////////////////////////////////////////////////
@@ -171,7 +173,7 @@ void CDLG_Table::Set_Position(wxRect r)
 //---------------------------------------------------------
 void CDLG_Table::Save_Changes(void)
 {
-	m_pTable_Orig->Assign(m_pTable);
+	m_pTable->Assign(m_pControl->Get_Table());
 }
 
 
@@ -207,6 +209,66 @@ void CDLG_Table::On_Save(wxCommandEvent &event)
 	}
 }
 
+//---------------------------------------------------------
+void CDLG_Table::On_WKSP_Load(wxCommandEvent &event)
+{
+	CSG_Data_Collection	*pTables	= SG_Get_Data_Manager().Get_Table();
+
+	if( pTables )
+	{
+		wxArrayString	Names;
+		CSG_Table		Index;
+
+		Index.Add_Field("INDEX", SG_DATATYPE_Int);
+
+		for(size_t i=0; i<pTables->Count(); i++)
+		{
+			CSG_Table	*pTable	= (CSG_Table *)pTables->Get(i);
+
+			if( pTable->is_Compatible(m_pTable) && pTable->Get_Count() > 0 )
+			{
+				Names.Add(pTable->Get_Name());
+
+				Index.Add_Record()->Set_Value(0, i);
+			}
+		}
+
+		if( Names.Count() > 0 )
+		{
+			wxSingleChoiceDialog	dlg(MDI_Get_Top_Window(), _TL("Tables"), _TL("Load from Workspace"), Names);
+
+			if( dlg.ShowModal() == wxID_OK )
+			{
+				m_pControl->Get_Table()->Assign_Values((CSG_Table *)pTables->Get(Index[dlg.GetSelection()].asInt(0)));
+				m_pControl->Update_Table();
+
+				Refresh();
+			}
+
+			return;
+		}
+	}
+
+	DLG_Message_Show(_TL("No compatible table has been found."), CTRL_Get_Name(ID_BTN_TABLE_FROM_WKSP));
+}
+
+//---------------------------------------------------------
+void CDLG_Table::On_WKSP_Save(wxCommandEvent &event)
+{
+	wxTextEntryDialog	dlg(MDI_Get_Top_Window(), _TL("Table Name"), _TL("Save to Workspace"), _TL("Table"));
+
+	if( dlg.ShowModal() == wxID_OK && !dlg.GetValue().IsEmpty() )
+	{
+		CSG_Table	*pTable	= SG_Create_Table(*m_pControl->Get_Table());
+
+		wxString	Name(dlg.GetValue());
+
+		pTable->Set_Name(&Name);
+
+		SG_Get_Data_Manager().Add(pTable);
+	}
+}
+
 //---------------------------------------------------------
 void CDLG_Table::On_Add(wxCommandEvent &event)
 {
@@ -234,24 +296,26 @@ void CDLG_Table::On_Delete_All(wxCommandEvent &event)
 //---------------------------------------------------------
 void CDLG_Table::On_Colors(wxCommandEvent &event)
 {
-	if( m_pTable->Get_Count() > 0 )
+	if( m_pControl->Get_Table()->Get_Count() > 0 )
 	{
 		int		i;
 
-		CSG_Colors	Colors(m_pTable->Get_Count());
+		CSG_Table	*pTable	= m_pControl->Get_Table();
 
-		for(i=0; i<m_pTable->Get_Count(); i++)
+		CSG_Colors	Colors(pTable->Get_Count());
+
+		for(i=0; i<pTable->Get_Count(); i++)
 		{
-			Colors[i]	= m_pTable->Get_Record(i)->asInt(0);
+			Colors[i]	= pTable->Get_Record(i)->asInt(0);
 		}
 
 		if( DLG_Colors(&Colors) )
 		{
-			Colors.Set_Count(m_pTable->Get_Count());
+			Colors.Set_Count(pTable->Get_Count());
 
-			for(i=0; i<m_pTable->Get_Count(); i++)
+			for(i=0; i<pTable->Get_Count(); i++)
 			{
-				m_pTable->Get_Record(i)->Set_Value(0, Colors[i]);
+				pTable->Get_Record(i)->Set_Value(0, Colors[i]);
 			}
 
 			m_pControl->Update_Table();
