@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: dlg_table.h 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: dlg_table.h 2694 2015-11-20 16:43:15Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -90,23 +90,28 @@ public:
 	CDLG_Table(class CSG_Table *pTable, wxString Caption);
 	virtual ~CDLG_Table(void);
 
+
+protected:
+
+	class CSG_Table				*m_pTable;
+
+	class CDLG_Table_Control	*m_pControl;
+
+
+	void						On_Key			(wxKeyEvent     &event);
+
 	void						On_Load			(wxCommandEvent &event);
 	void						On_Save			(wxCommandEvent &event);
 
+	void						On_WKSP_Load	(wxCommandEvent &event);
+	void						On_WKSP_Save	(wxCommandEvent &event);
+
 	void						On_Add			(wxCommandEvent &event);
 	void						On_Insert		(wxCommandEvent &event);
 	void						On_Delete		(wxCommandEvent &event);
 	void						On_Delete_All	(wxCommandEvent &event);
 	void						On_Colors		(wxCommandEvent &event);
 
-
-protected:
-
-	class CSG_Table				*m_pTable, *m_pTable_Orig;
-
-	class CVIEW_Table_Control	*m_pControl;
-
-
 	virtual void				Set_Position	(wxRect r);
 
 	virtual void				Save_Changes	(void);
