@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: helper.cpp 2086 2014-04-04 09:07:51Z oconrad $
+ * Version $Id: helper.cpp 2511 2015-06-17 13:22:42Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -194,6 +194,11 @@ double		Get_Random(double loValue, double hiValue)
 //---------------------------------------------------------
 wxString	Get_FilePath_Relative(const wxString &Directory, const wxString &FileName)
 {
+	if( FileName.Find(Directory) != 0 )
+	{
+		return( FileName );
+	}
+
 	wxFileName	fn(FileName);
 
 	fn.MakeRelativeTo(Directory);
@@ -898,38 +903,52 @@ void		STATUSBAR_Set_Text(const wxString &Text, int iPane)
 //---------------------------------------------------------
 bool		Open_Application(const wxString &Reference, const wxString &Mime_Extension)
 {
-	bool		bResult		= false;
-
-	if( Reference.Length() )
+	if( Reference.IsEmpty() )
 	{
-		wxString	Extension, sReference(Reference);
-		wxFileType	*pFileType;
+		return( false );
+	}
 
-		if( Mime_Extension.Length() )
-		{
-			Extension	= Mime_Extension;
-		}
-		else if( sReference.Find(SG_T("ftp:")) == 0 || sReference.Find(SG_T("http:")) == 0 || sReference.Find(SG_T("https:")) == 0 )
-		{
-			Extension	= wxT("html");
-		}
-		else
-		{
-			wxFileName	FileName(Reference);
+	if( Reference.Find("ftp:"   ) == 0
+	||  Reference.Find("file:"  ) == 0
+	||  Reference.Find("http:"  ) == 0
+	||  Reference.Find("https:" ) == 0
+	||  Reference.Find("mailto:") == 0 )
+	{
+		return( wxLaunchDefaultBrowser(Reference) );
+	}
 
-			Extension	= FileName.GetExt();
-		}
+	//-----------------------------------------------------
+	bool	bResult	= false;
 
-		if( Extension.Length() > 0 && (pFileType = wxTheMimeTypesManager->GetFileTypeFromExtension(Extension)) != NULL )
+	wxFileName	FileName(Reference);
+
+	if( FileName.IsRelative() )	// very likely that this will not work
+	{
+		FileName.MakeAbsolute(g_pSAGA->Get_App_Path());	// this might not work either, but give it a try
+	}
+
+	//-----------------------------------------------------
+	if( Mime_Extension.IsEmpty() )
+	{
+		if( (bResult = wxLaunchDefaultApplication(FileName.GetFullPath())) == false )
+		{
+			MSG_Error_Add(wxString::Format("%s:\n%s\n", _TL("failed to launch default application"), FileName.GetFullPath().c_str()));
+		}
+	}
+
+	//-----------------------------------------------------
+	else
+	{
+		wxFileType	*pFileType	= wxTheMimeTypesManager->GetFileTypeFromExtension(Mime_Extension);
+
+		if( pFileType )
 		{
 			wxString	Command;
 
-			if( pFileType->GetOpenCommand(&Command, wxFileType::MessageParameters(Reference, wxT(""))) )
+			if( !pFileType->GetOpenCommand(&Command, wxFileType::MessageParameters(FileName.GetFullPath(), ""))
+			||  !(bResult = wxExecute(Command) == 0) )
 			{
-				MSG_Execution_Add_Line();
-				MSG_Execution_Add(Command, true, true);
-
-				bResult	= wxExecute(Command) == 0;
+				MSG_Error_Add(wxString::Format("%s:\n%s\n", _TL("command execution failed"), Command.c_str()));
 			}
 
 			delete(pFileType);
@@ -942,7 +961,7 @@ bool		Open_Application(const wxString &Reference, const wxString &Mime_Extension
 //---------------------------------------------------------
 bool		Open_WebBrowser(const wxString &Reference)
 {
-	return( Open_Application(Reference, wxT("html")) );
+	return( wxLaunchDefaultBrowser(Reference) );
 }
 
 
@@ -953,7 +972,7 @@ bool		Open_WebBrowser(const wxString &Reference)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-wxString Get_Online_Module_Description(const wxString &Library, int ID)
+wxString Get_Online_Module_Description(const wxString &Library, const wxString &ID)
 {
 	static bool	bBuisy	= false;
 
@@ -966,50 +985,49 @@ wxString Get_Online_Module_Description(const wxString &Library, int ID)
 		wxInputStream	*pStream;
 		wxHTTP			Server;
 
-		wxString	sServer	= wxT("sourceforge.net");
-		wxString	sPath	= wxT("/apps/trac/saga-gis/wiki/");
-		wxString	sRoot	= wxT("/apps/trac/saga-gis/");
+		wxString	sServer	= "sourceforge.net";
+		wxString	sPath	= SG_File_Get_Name(Library, false).c_str();
 
-		wxString	sPage	= SG_File_Get_Name(Library, false).w_str();
-
-		if( sPage.Length() > 3 && sPage[0] == wxT('l') && sPage[1] == wxT('i') && sPage[2] == wxT('b') )	// remove linux prefix 'lib'
+		if( sPath.Find("lib") == 0 )	// remove linux prefix 'lib'
 		{
-			sPage.Remove(0, 3);
+			sPath.Remove(0, 3);
 		}
 
-		if( ID >= 0 )
+		if( !ID.IsEmpty() )
 		{
-			sPage	+= wxString::Format(wxT("_%d"), ID);
+			sPath	+= "_" + ID;
 		}
 
-		if( Server.Connect(sServer) && (pStream = Server.GetInputStream(sPath + sPage)) != NULL )
+		sPath	= "/p/saga-gis/wiki/" + sPath + "/";
+
+		if( Server.Connect(sServer) && (pStream = Server.GetInputStream(sPath)) != NULL )
 		{
 			while( !pStream->Eof() )
 			{
 				Description	+= pStream->GetC();
 			}
 
-			if( Description.Find(wxT("Trac Error")) >= 0 )
+			int		n;
+
+			if( (n = Description.Find("<div class=\"markdown_content\">")) < 0 )
 			{
 				Description.Clear();
 			}
 			else
 			{
-				int		n;
+				Description.Remove(0, n);
 
-				if( (n = Description.Find(wxT("<div class=\"wikipage searchable\">"))) > 0 )
+				if( (n = Description.Find("</div>")) > 0 )
 				{
-					Description.Remove(0, n);
-
-					if( (n = Description.Find(wxT("</div>"))) > 0 )
-					{
-						Description.RemoveLast(Description.Length() - (n - 6));
-					}
+					Description.Truncate(n + 6);
 				}
 
-				Description.Replace(sRoot, wxT("http://") + sServer + sRoot);
-				Description.Replace(wxT("\n"), wxT(""));
+				Description.Replace("./attachment"      ,        "http://" + sServer + sPath + "attachment");
+				Description.Replace("href=\"/p/saga-gis", "href=\"http://" + sServer + "/p/saga-gis");
+				Description.Replace("\n", "");
 			}
+
+			delete(pStream);
 		}
 
 		bBuisy	= false;
