@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: helper.h 2086 2014-04-04 09:07:51Z oconrad $
+ * Version $Id: helper.h 2274 2014-10-02 15:44:21Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -181,16 +181,15 @@ bool				PROGRESSBAR_Set_Position		(int Position);
 bool				PROGRESSBAR_Set_Position		(double Position, double Range);
 
 //---------------------------------------------------------
-bool				Open_Application				(const wxString &Reference, const wxString &Mime_Extension = wxT(""));
+bool				Open_Application				(const wxString &Reference, const wxString &Mime_Extension = "");
 bool				Open_WebBrowser					(const wxString &Reference);
 
-wxString			Get_Online_Module_Description	(const wxString &Library, int ID = -1);
+wxString			Get_Online_Module_Description	(const wxString &Library, const wxString &ID = "");
 
 //---------------------------------------------------------
 enum
 {
 	STATUSBAR_DEFAULT	= 0,
-	STATUSBAR_ACTIVE,
 	STATUSBAR_VIEW_X,
 	STATUSBAR_VIEW_Y,
 	STATUSBAR_VIEW_Z,
