@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: project.cpp 2234 2014-09-16 15:08:19Z oconrad $
+ * Version $Id: project.cpp 2674 2015-11-03 17:03:58Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -97,6 +97,8 @@
 #include "wksp_map_graticule.h"
 #include "wksp_layer.h"
 
+#include "data_source_pgsql.h"
+
 #include "project.h"
 
 
@@ -206,6 +208,115 @@ bool CWKSP_Project::Save(const wxString &FileName, bool bSaveModified)
 }
 
 
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CWKSP_Project::Copy(void)
+{
+	wxString	FileName;
+
+	if( !DLG_Save(FileName, ID_DLG_PROJECT_SAVE) )
+	{
+		return( false );
+	}
+
+	wxFileName	Directory(FileName);
+
+	Directory.AppendDir(Directory.GetFullName());
+
+	if( !Directory.DirExists() )
+	{
+		Directory.Mkdir(wxS_DIR_DEFAULT, wxPATH_MKDIR_FULL);
+	}
+
+	size_t	i, j;
+
+	for(j=0; j<SG_Get_Data_Manager().Grid_System_Count(); j++)
+	{
+		for(i=0; i<SG_Get_Data_Manager().Get_Grid_System(j)->Count(); i++)
+		{
+			_Copy_To_File(g_pData->Get(SG_Get_Data_Manager().Get_Grid_System(j)->Get(i)), Directory.GetFullPath());
+		}
+	}
+
+	for(i=0; i<SG_Get_Data_Manager().Get_Point_Cloud()->Count(); i++)
+	{
+		_Copy_To_File(g_pData->Get(SG_Get_Data_Manager().Get_Point_Cloud()->Get(i)), Directory.GetFullPath());
+	}
+
+	for(i=0; i<SG_Get_Data_Manager().Get_Shapes()->Count(); i++)
+	{
+		_Copy_To_File(g_pData->Get(SG_Get_Data_Manager().Get_Shapes()->Get(i)), Directory.GetFullPath());
+	}
+
+	for(i=0; i<SG_Get_Data_Manager().Get_Table()->Count(); i++)
+	{
+		_Copy_To_File(g_pData->Get(SG_Get_Data_Manager().Get_Shapes()->Get(i)), Directory.GetFullPath());
+	}
+
+	for(i=0; i<SG_Get_Data_Manager().Get_TIN()->Count(); i++)
+	{
+		_Copy_To_File(g_pData->Get(SG_Get_Data_Manager().Get_TIN()->Get(i)), Directory.GetFullPath());
+	}
+
+	return( _Save(Directory.GetFullPath(), false, true) );
+}
+
+//---------------------------------------------------------
+bool CWKSP_Project::_Copy_To_File(CWKSP_Data_Item *pItem, const wxString &Directory)
+{
+	wxFileName	Path(Directory);
+
+	switch( pItem->Get_Type() )
+	{
+	case WKSP_ITEM_Table     :	Path.SetExt("txt" );	break;
+	case WKSP_ITEM_Shapes    :	Path.SetExt("shp" );	break;
+	case WKSP_ITEM_TIN       :	Path.SetExt("shp" );	break;
+	case WKSP_ITEM_PointCloud:	Path.SetExt("spc" );	break;
+	case WKSP_ITEM_Grid      :	Path.SetExt("sgrd");	break;
+	default:	return( false );
+	}
+
+	//-----------------------------------------------------
+	wxString	Name(pItem->Get_Object()->Get_Name());
+
+	Name.Replace(".", "-");
+	Name.Replace(":", "-");
+
+	Path.SetName(Name);
+
+	for(int i=1; Path.FileExists(); i++)
+	{
+		Path.SetName(Name + wxString::Format("_%d", i));
+	}
+
+	Name	= Path.GetFullPath();
+
+	return( pItem->Get_Object()->Save(&Name) );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CWKSP_Project::CopyToDB(void)
+{
+	return( true );
+}
+
+//---------------------------------------------------------
+bool CWKSP_Project::_Copy_To_Database(CWKSP_Data_Item *pItem, const wxString &Connection)
+{
+	return( true );
+}
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 //														 //
@@ -244,23 +355,27 @@ bool CWKSP_Project::_Load(const wxString &FileName, bool bAdd, bool bUpdateMenu)
 	}
 	else if( !wxFileExists(FileName) )
 	{
-		MSG_Error_Add(_TL("file does not exist.")            , true, true, SG_UI_MSG_STYLE_FAILURE);
+		MSG_Error_Add(_TL("file does not exist."            ), true, true, SG_UI_MSG_STYLE_FAILURE);
 	}
 	else if( !Project.Load(&FileName) )
 	{
-		MSG_Error_Add(_TL("could not read project file.")    , true, true, SG_UI_MSG_STYLE_FAILURE);
+		MSG_Error_Add(_TL("could not read project file."    ), true, true, SG_UI_MSG_STYLE_FAILURE);
 	}
 	else if( Project.Get_Name().Cmp(SG_T("SAGA_PROJECT")) )
 	{
-		MSG_Error_Add(_TL("invalid project file.")           , true, true, SG_UI_MSG_STYLE_FAILURE);
+		MSG_Error_Add(_TL("invalid project file."           ), true, true, SG_UI_MSG_STYLE_FAILURE);
 	}
 	else if( (pNode = Project.Get_Child(SG_T("DATA"))) == NULL || pNode->Get_Children_Count() <= 0 )
 	{
 		MSG_Error_Add(_TL("no data entries in project file."), true, true, SG_UI_MSG_STYLE_FAILURE);
 	}
+	else if( !_Load_DBConnections(*pNode) )
+	{
+		MSG_Error_Add(_TL("could not connect to database."  ), true, true, SG_UI_MSG_STYLE_FAILURE);
+	}
 	else
 	{
-		int			i;
+		int		i;
 
 		CSG_String	Version(Project.Get_Property("VERSION"));
 
@@ -284,7 +399,7 @@ bool CWKSP_Project::_Load(const wxString &FileName, bool bAdd, bool bUpdateMenu)
 		//-------------------------------------------------
 		g_pSAGA_Frame->Freeze();
 
-		if( (pNode = Project.Get_Child(SG_T("MAPS"))) != NULL && pNode->Get_Children_Count() > 0 )
+		if( (pNode = Project.Get_Child("MAPS")) != NULL && pNode->Get_Children_Count() > 0 )
 		{
 			for(int j=0; j<pNode->Get_Children_Count(); j++)
 			{
@@ -292,6 +407,12 @@ bool CWKSP_Project::_Load(const wxString &FileName, bool bAdd, bool bUpdateMenu)
 			}
 		}
 
+		switch( g_pData->Get_Parameter("PROJECT_MAP_ARRANGE")->asInt() )
+		{
+		case 1:	g_pSAGA_Frame->Tile(wxHORIZONTAL);	break;
+		case 2:	g_pSAGA_Frame->Tile(wxVERTICAL  );	break;
+		}
+
 		g_pSAGA_Frame->Thaw();
 	}
 
@@ -347,11 +468,11 @@ bool CWKSP_Project::_Save(const wxString &FileName, bool bSaveModified, bool bUp
 	//-----------------------------------------------------
 	ProjectDir	= SG_File_Get_Path(FileName).w_str();
 
-	Project.Set_Name    (SG_T("SAGA_PROJECT"));
-	Project.Add_Property(SG_T("VERSION"), SAGA_VERSION);
+	Project.Set_Name    ("SAGA_PROJECT");
+	Project.Add_Property("VERSION", SAGA_VERSION);
 
 	//-----------------------------------------------------
-	pNode		= Project.Add_Child(SG_T("DATA"));
+	pNode		= Project.Add_Child("DATA");
 
 	if( (pTables = g_pData->Get_Tables()) != NULL )
 	{
@@ -417,7 +538,7 @@ bool CWKSP_Project::_Save(const wxString &FileName, bool bSaveModified, bool bUp
 	//-----------------------------------------------------
 	if( g_pMaps->Get_Count() > 0 )
 	{
-		pNode		= Project.Add_Child(SG_T("MAPS"));
+		pNode		= Project.Add_Child("MAPS");
 
 		for(i=0; i<g_pMaps->Get_Count(); i++)
 		{
@@ -455,6 +576,63 @@ bool CWKSP_Project::_Save(const wxString &FileName, bool bSaveModified, bool bUp
 }
 
 
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CWKSP_Project::_Load_DBConnections(CSG_MetaData &Data)
+{
+	int			i;
+	CSG_Strings	Connections;
+
+	for(i=0; i<Data.Get_Children_Count(); i++)
+	{
+		CSG_String	Connection(Data[i].Get_Child("FILE") ? Data[i].Get_Child("FILE")->Get_Content() : "");
+
+		if( !Connection.BeforeFirst(':').Cmp("PGSQL") )
+		{
+			Connection	= Connection.AfterFirst(':');	CSG_String	Host  (Connection.BeforeFirst(':'));
+			Connection	= Connection.AfterFirst(':');	CSG_String	Port  (Connection.BeforeFirst(':'));
+			Connection	= Connection.AfterFirst(':');	CSG_String	DBName(Connection.BeforeFirst(':'));
+
+			Connection	= Host + ":" + Port + ":" + DBName;
+
+			bool	bAdd	= true;
+
+			for(int j=0; j<Connections.Get_Count() && bAdd; j++)
+			{
+				if( !Connection.Cmp(Connections[j]) )
+				{
+					bAdd	= false;
+				}
+			}
+
+			if( bAdd )
+			{
+				Connections	+= Connection;
+			}
+		}
+	}
+
+	for(i=0; i<Connections.Get_Count(); i++)
+	{
+		CSG_String	Host  (Connections[i].BeforeFirst(':'));
+		CSG_String	Port  (Connections[i].AfterFirst (':').BeforeLast(':'));
+		CSG_String	DBName(Connections[i].AfterLast  (':'));
+
+		if( !PGSQL_Connect(Host, Port, DBName) )
+		{
+			return( false );
+		}
+	}
+
+	return( true );
+}
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 //														 //
@@ -464,7 +642,7 @@ bool CWKSP_Project::_Save(const wxString &FileName, bool bSaveModified, bool bUp
 //---------------------------------------------------------
 bool CWKSP_Project::_Load_Data(CSG_MetaData &Entry, const wxString &ProjectDir, bool bLoad, const CSG_String &Version)
 {
-	if( Entry.Get_Name().Cmp(SG_T("DATASET")) )
+	if( Entry.Get_Name().Cmp("DATASET") || !Entry.Get_Child("FILE") || Entry.Get_Child("FILE")->Get_Content().is_Empty() )
 	{
 		return( false );
 	}
@@ -482,26 +660,54 @@ bool CWKSP_Project::_Load_Data(CSG_MetaData &Entry, const wxString &ProjectDir,
 		return( false );
 	}
 
-	if( !Entry.Get_Child(SG_T("FILE")) )
-	{
-		return( false );
-	}
+	//-----------------------------------------------------
+	wxString	File	= Entry.Get_Child("FILE")->Get_Content().c_str();
 
-	wxString	File(Get_FilePath_Absolute(ProjectDir, Entry.Get_Child(SG_T("FILE"))->Get_Content().c_str()));
-
-	if( !wxFileExists(File) )
+	if( File.Find("PGSQL") != 0 && wxFileExists(Get_FilePath_Absolute(ProjectDir, File)) )
 	{
-		return( false );
+		File	= Get_FilePath_Absolute(ProjectDir, File);
 	}
 
 	CWKSP_Base_Item	*pItem	= bLoad ? g_pData->Open(File, Type) : _Get_byFileName(File);
 
 	if( !pItem || !pItem->Get_Parameters() || !Entry.Get_Child("PARAMETERS") )
 	{
+		if( bLoad )
+		{
+			MSG_Error_Add(wxString::Format("%s [%s]", _TL("failed to load data"), File.c_str()));
+		}
+
 		return( false );
 	}
 
+	//-----------------------------------------------------
+	CSG_MetaData	*pEntry	= Entry("PARAMETERS");
+
+	for(int i=0; i<pEntry->Get_Children_Count(); i++)
+	{
+		if( !pEntry->Get_Child(i)->Get_Name().CmpNoCase("DATA") && !pEntry->Get_Child(i)->Get_Content().is_Empty() && pEntry->Get_Child(i)->Get_Content().BeforeFirst(':').Cmp("PGSQL") )
+		{
+			wxString	File(Get_FilePath_Absolute(ProjectDir, pEntry->Get_Child(i)->Get_Content().w_str()));
+
+			pEntry->Get_Child(i)->Set_Content(&File);
+
+		//	if( SG_Compare_SAGA_Version(Version) < 0 )
+			{
+				if( pEntry->Get_Child(i)->Cmp_Property("id", "OVERLAY_1") )
+				{
+					pEntry->Get_Child(i)->Set_Property("id", "OVERLAY_G");
+				}
+
+				if( pEntry->Get_Child(i)->Cmp_Property("id", "OVERLAY_2") )
+				{
+					pEntry->Get_Child(i)->Set_Property("id", "OVERLAY_B");
+				}
+			}
+		}
+	}
+
 	pItem->Get_Parameters()->Serialize(*Entry.Get_Child("PARAMETERS"), false);
+
 	pItem->Parameters_Changed();
 
 	return( true );
@@ -510,32 +716,57 @@ bool CWKSP_Project::_Load_Data(CSG_MetaData &Entry, const wxString &ProjectDir,
 //---------------------------------------------------------
 bool CWKSP_Project::_Save_Data(CSG_MetaData &Entry, const wxString &ProjectDir, CSG_Data_Object *pDataObject, CSG_Parameters *pParameters)
 {
-	if( !pDataObject || !pDataObject->Get_File_Name() || !wxFileExists(pDataObject->Get_File_Name()) )
+	if( !pDataObject || !pDataObject->Get_File_Name(false) || SG_STR_LEN(pDataObject->Get_File_Name(false)) == 0 )
 	{
 		return( false );
 	}
 
-	CSG_MetaData	*pEntry	= Entry.Add_Child(SG_T("DATASET"));
+	CSG_MetaData	*pEntry	= Entry.Add_Child("DATASET");
 
 	switch( pDataObject->Get_ObjectType() )
 	{
 	default:	return( false );
-	case DATAOBJECT_TYPE_Grid:			pEntry->Add_Property(SG_T("type"), SG_T("GRID"));	break;
-	case DATAOBJECT_TYPE_Table:			pEntry->Add_Property(SG_T("type"), SG_T("TABLE"));	break;
-	case DATAOBJECT_TYPE_Shapes:		pEntry->Add_Property(SG_T("type"), SG_T("SHAPES"));	break;
-	case DATAOBJECT_TYPE_TIN:			pEntry->Add_Property(SG_T("type"), SG_T("TIN"));	break;
-	case DATAOBJECT_TYPE_PointCloud:	pEntry->Add_Property(SG_T("type"), SG_T("POINTS"));	break;
+	case DATAOBJECT_TYPE_Grid      :	pEntry->Add_Property("type", "GRID"  );	break;
+	case DATAOBJECT_TYPE_Table     :	pEntry->Add_Property("type", "TABLE" );	break;
+	case DATAOBJECT_TYPE_Shapes    :	pEntry->Add_Property("type", "SHAPES");	break;
+	case DATAOBJECT_TYPE_TIN       :	pEntry->Add_Property("type", "TIN"   );	break;
+	case DATAOBJECT_TYPE_PointCloud:	pEntry->Add_Property("type", "POINTS");	break;
 	}
 
-	wxString	s(Get_FilePath_Relative(ProjectDir, pDataObject->Get_File_Name()));
-
-	pEntry->Add_Child("FILE", &s);
+	if( wxFileExists(pDataObject->Get_File_Name(false)) )
+	{
+		pEntry->Add_Child("FILE", SG_File_Get_Path_Relative(ProjectDir, pDataObject->Get_File_Name(false)));
+	}
+	else if( pDataObject->Get_MetaData_DB().Get_Children_Count() > 0 )
+	{
+		pEntry->Add_Child("FILE", pDataObject->Get_File_Name(false));
+	}
+	else
+	{
+		return( false );
+	}
 
 	if( pParameters )
 	{
 		pParameters->DataObjects_Check(true);
 
-		pParameters->Serialize(*pEntry->Add_Child(SG_T("PARAMETERS")), true);
+		pParameters->Serialize(*pEntry->Add_Child("PARAMETERS"), true);
+
+		//-------------------------------------------------
+		pEntry	= pEntry->Get_Child("PARAMETERS");
+
+		for(int i=0; i<pEntry->Get_Children_Count(); i++)
+		{
+			if( !pEntry->Get_Child(i)->Get_Name().CmpNoCase("DATA") )
+			{
+				CSG_String	File	= pEntry->Get_Child(i)->Get_Content();
+
+				if( File.BeforeFirst(':').Cmp("PGSQL") && SG_File_Exists(File) )
+				{
+					pEntry->Get_Child(i)->Set_Content(SG_File_Get_Path_Relative(ProjectDir, File.w_str()));
+				}
+			}
+		}
 	}
 
 	return( true );
@@ -577,7 +808,14 @@ bool CWKSP_Project::_Load_Map(CSG_MetaData &Entry, const wxString &ProjectDir)
 	{
 		if( !pNode->Get_Child(i)->Get_Name().Cmp("FILE") )
 		{
-			CWKSP_Base_Item	*pItem	= _Get_byFileName(Get_FilePath_Absolute(ProjectDir, pNode->Get_Child(i)->Get_Content().w_str()));
+			wxString	FileName(pNode->Get_Child(i)->Get_Content().w_str());
+
+			if( FileName.Find("PGSQL") != 0 )
+			{
+				FileName	= Get_FilePath_Absolute(ProjectDir, FileName);
+			}
+
+			CWKSP_Base_Item	*pItem	= _Get_byFileName(FileName);
 
 			if(	pItem &&
 			(   pItem->Get_Type() == WKSP_ITEM_Grid
@@ -602,7 +840,14 @@ bool CWKSP_Project::_Load_Map(CSG_MetaData &Entry, const wxString &ProjectDir)
 	{
 		if( !pNode->Get_Child(i)->Get_Name().Cmp("FILE") )
 		{
-			CWKSP_Base_Item	*pItem	= _Get_byFileName(Get_FilePath_Absolute(ProjectDir, pNode->Get_Child(i)->Get_Content().w_str()));
+			wxString	FileName(pNode->Get_Child(i)->Get_Content().w_str());
+
+			if( FileName.Find("PGSQL") != 0 )
+			{
+				FileName	= Get_FilePath_Absolute(ProjectDir, FileName);
+			}
+
+			CWKSP_Base_Item	*pItem	= _Get_byFileName(FileName);
 
 			if(	pItem &&
 			(   pItem->Get_Type() == WKSP_ITEM_Grid
@@ -657,11 +902,18 @@ bool CWKSP_Project::_Save_Map(CSG_MetaData &Entry, const wxString &ProjectDir, C
 		{
 			CSG_Data_Object	*pObject	= ((CWKSP_Map_Layer *)pMap->Get_Item(i))->Get_Layer()->Get_Object();
 
-			if( pObject && pObject->Get_File_Name() && wxFileExists(pObject->Get_File_Name()) )
+			if( pObject && pObject->Get_File_Name(false) && *pObject->Get_File_Name(false) )
 			{
-				wxString	s(Get_FilePath_Relative(ProjectDir, pObject->Get_File_Name()));
+				wxString	FileName(pObject->Get_File_Name(false));
 
-				pEntry->Add_Child("FILE", &s);
+				if( FileName.Find("PGSQL") == 0 )
+				{
+					pEntry->Add_Child("FILE", &FileName);
+				}
+				else if( wxFileExists(FileName) )
+				{
+					pEntry->Add_Child("FILE", SG_File_Get_Path_Relative(ProjectDir, FileName));
+				}
 			}
 		}
 
@@ -684,7 +936,7 @@ bool CWKSP_Project::_Save_Map(CSG_MetaData &Entry, const wxString &ProjectDir, C
 //---------------------------------------------------------
 CWKSP_Base_Item * CWKSP_Project::_Get_byFileName(const wxString &FileName)
 {
-	return( g_pData->Get(SG_Get_Data_Manager().Find(&FileName)) );
+	return( g_pData->Get(SG_Get_Data_Manager().Find(&FileName, false)) );
 }
 
 
@@ -766,6 +1018,8 @@ bool CWKSP_Project::_Compatibility_Load_Data(const wxString &FileName)
 		return( false );
 	}
 
+	g_pSAGA_Frame->Freeze();
+
 	g_pData->Get_Menu_Files()->Set_Update(false);
 	while( _Compatibility_Load_Data(Stream, SG_File_Get_Path(FileName).w_str()) );
 	g_pData->Get_Menu_Files()->Set_Update(true);
@@ -778,6 +1032,14 @@ bool CWKSP_Project::_Compatibility_Load_Data(const wxString &FileName)
 		while( _Compatibility_Load_Map(Stream, SG_File_Get_Path(FileName).w_str()) );
 	}
 
+	switch( g_pData->Get_Parameter("PROJECT_MAP_ARRANGE")->asInt() )
+	{
+	case 1:	g_pSAGA_Frame->Tile(wxHORIZONTAL);	break;
+	case 2:	g_pSAGA_Frame->Tile(wxVERTICAL  );	break;
+	}
+
+	g_pSAGA_Frame->Thaw();
+
 	return( true );
 }
 
