@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: project.h 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: project.h 2674 2015-11-03 17:03:58Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -89,7 +89,8 @@ public:
 	bool					Save				(bool bSaveAsOnError);
 	bool					Save				(const wxString &FileName, bool bSaveModified);
 
-	bool					Save_Modified		(class CWKSP_Base_Item *pItem, bool bSelections = false);
+	bool					Copy				(void);
+	bool					CopyToDB			(void);
 
 
 private:
@@ -101,6 +102,8 @@ private:
 	bool					_Load				(const wxString &FileName, bool bAdd, bool bUpdateMenu);
 	bool					_Save				(const wxString &FileName, bool bSaveModified, bool bUpdateMenu);
 
+	bool					_Load_DBConnections	(CSG_MetaData &Data);
+
 	bool					_Load_Data			(CSG_MetaData &Entry, const wxString &ProjectDir, bool bLoad, const CSG_String &Version);
 	bool					_Save_Data			(CSG_MetaData &Entry, const wxString &ProjectDir, class CSG_Data_Object *pDataObject, class CSG_Parameters *pParameters);
 
@@ -109,16 +112,12 @@ private:
 
 	class CWKSP_Base_Item *	_Get_byFileName		(const wxString &FileName);
 
-	static int				_Modified_Changed	(class CSG_Parameter  *pParameter, int Flags);
-	bool					_Modified_Get		(class CSG_Parameters *pParameters, class CWKSP_Base_Item *pItem, bool bSelections);
-	bool					_Modified_Get		(class CSG_Parameters *pParameters, class CWKSP_Base_Item *pItem, class CSG_Data_Object *pObject);
-	bool					_Modified_Save		(class CSG_Parameters *pParameters);
-
+	bool					_Copy_To_File		(class CWKSP_Data_Item *pData, const wxString &Directory);
+	bool					_Copy_To_Database	(class CWKSP_Data_Item *pData, const wxString &Connection);
 
 	//-----------------------------------------------------
-	bool					_Compatibility_Data	(TSG_Data_Type Type, class CSG_Parameters *pParameters, const CSG_String &Version);
+	bool	_Compatibility_Data			(TSG_Data_Type Type, class CSG_Parameters *pParameters, const CSG_String &Version);
 
-	//-----------------------------------------------------
 	bool	_Compatibility_Load_Data	(const wxString &FileName);
 	bool	_Compatibility_Load_Data	(CSG_File &Stream, const wxString &ProjectDir);
 	bool	_Compatibility_Load_Map		(CSG_File &Stream, const wxString &ProjectDir);
