@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: res_commands.cpp 2039 2014-03-05 13:26:00Z oconrad $
+ * Version $Id: res_commands.cpp 2698 2015-11-24 13:45:01Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -113,19 +113,19 @@ wxString CMD_Get_Name(int Cmd_ID)
 	case ID_CMD_WKSP_ITEM_SETTINGS_COPY:return( _TL("Copy Settings from other Layer") );
 	case ID_CMD_WKSP_ITEM_SEARCH:		return( _TL("Search for...") );
 
-	case ID_CMD_WKSP_OPEN:				return( _TL("Open") );
-	case ID_CMD_WKSP_SAVE:				return( _TL("Save All") );
-
 	case ID_CMD_INFO_CLEAR:				return( _TL("Clear") );
 	case ID_CMD_INFO_COPY:				return( _TL("Copy") );
+	case ID_CMD_INFO_SELECTALL:			return( _TL("Select All") );
 
 	case ID_CMD_DATASOURCE_REFRESH:		return( _TL("Refresh") );
 
 	case ID_CMD_DB_REFRESH:				return( _TL("Refresh") );
+	case ID_CMD_DB_SOURCE_CREATE:		return( _TL("New") );
+	case ID_CMD_DB_SOURCE_DROP:			return( _TL("Drop") );
 	case ID_CMD_DB_SOURCE_OPEN:			return( _TL("Open") );
 	case ID_CMD_DB_SOURCE_CLOSE:		return( _TL("Close") );
 	case ID_CMD_DB_SOURCE_CLOSE_ALL:	return( _TL("Close All") );
-	case ID_CMD_DB_SOURCE_DELETE:		return( _TL("Delete") );
+	case ID_CMD_DB_SOURCE_DELETE:		return( _TL("Remove") );
 	case ID_CMD_DB_TABLE_OPEN:			return( _TL("Load") );
 	case ID_CMD_DB_TABLE_DELETE:		return( _TL("Delete") );
 	case ID_CMD_DB_TABLE_RENAME:		return( _TL("Rename") );
@@ -134,18 +134,34 @@ wxString CMD_Get_Name(int Cmd_ID)
 
 	case ID_CMD_MODULES_OPEN:			return( _TL("Load Tool Library") );
 	case ID_CMD_MODULES_SEARCH:			return( _TL("Find and Run Tool") );
-	case ID_CMD_MODULES_SAVE_SCRIPT:	return( _TL("Create Script Command File") );
 	case ID_CMD_MODULES_SAVE_DOCS:		return( _TL("Create Tool Description Files") );
+	case ID_CMD_MODULES_SAVE_SCRIPT:		return( _TL("Save to Script File") );
+	case ID_CMD_MODULES_SAVE_TO_CLIPBOARD:	return( _TL("Save to Clipboard") );
+	case ID_CMD_MODULES_CHAIN_RELOAD:	return( _TL("Reload") );
+	case ID_CMD_MODULES_CHAIN_EDIT:		return( _TL("Edit") );
 
-	case ID_CMD_DATA_PROJECT_NEW:		return( _TL("Close All") );
 	case ID_CMD_DATA_PROJECT_OPEN:		return( _TL("Load Project") );
 	case ID_CMD_DATA_PROJECT_OPEN_ADD:	return( _TL("Add Project") );
+	case ID_CMD_DATA_PROJECT_BROWSE:	return( _TL("Search for Projects") );		
+	case ID_CMD_DATA_PROJECT_CLOSE:		return( _TL("Close Project") );
 	case ID_CMD_DATA_PROJECT_SAVE:		return( _TL("Save Project") );
-	case ID_CMD_DATA_PROJECT_SAVE_AS:	return( _TL("Save Project As...") );
+	case ID_CMD_DATA_PROJECT_SAVE_AS:	return( _TL("Save Project as...") );
+	case ID_CMD_DATA_PROJECT_COPY:		return( _TL("Copy Project to...") );
+	case ID_CMD_DATA_PROJECT_COPY_DB:	return( _TL("Copy Project to Database") );
+
 	case ID_CMD_DATA_LEGEND_COPY:		return( _TL("Copy Legend to Clipboard...") );
 	case ID_CMD_DATA_LEGEND_SIZE_INC:	return( _TL("Increase Legend Size") );
 	case ID_CMD_DATA_LEGEND_SIZE_DEC:	return( _TL("Decrease Legend Size") );
+
 	case ID_CMD_DATA_HISTORY_CLEAR:		return( _TL("Clear History") );
+	case ID_CMD_DATA_HISTORY_TO_MODEL:	return( _TL("Save as Tool Chain") );
+	case ID_CMD_DATA_HISTORY_OPTIONS_COLLAPSE:	return( _TL("Collpase Options") );
+	case ID_CMD_DATA_HISTORY_OPTIONS_EXPAND:	return( _TL("Expand Options") );
+
+	case ID_CMD_DATA_OPEN:				return( _TL("Open") );
+	case ID_CMD_DATA_SAVE:				return( _TL("Save") );
+	case ID_CMD_DATA_SAVEAS:			return( _TL("Save as...") );
+	case ID_CMD_DATA_SAVETODB:			return( _TL("Save to Database...") );
 	case ID_CMD_DATA_PROJECTION:		return( _TL("Spatial Reference") );
 
 	case ID_CMD_GRIDS_OPEN:
@@ -154,18 +170,6 @@ wxString CMD_Get_Name(int Cmd_ID)
 	case ID_CMD_TIN_OPEN:
 	case ID_CMD_POINTCLOUD_OPEN:		return( _TL("Load") );
 
-	case ID_CMD_GRIDS_SAVE:
-	case ID_CMD_SHAPES_SAVE:
-	case ID_CMD_TABLES_SAVE:
-	case ID_CMD_TIN_SAVE:
-	case ID_CMD_POINTCLOUD_SAVE:		return( _TL("Save") );
-
-	case ID_CMD_GRIDS_SAVEAS:
-	case ID_CMD_SHAPES_SAVEAS:
-	case ID_CMD_TABLES_SAVEAS:
-	case ID_CMD_TIN_SAVEAS:
-	case ID_CMD_POINTCLOUD_SAVEAS:		return( _TL("Save As...") );
-
 	case ID_CMD_GRIDS_SHOW:
 	case ID_CMD_SHAPES_SHOW:
 	case ID_CMD_TIN_SHOW:
@@ -184,20 +188,22 @@ wxString CMD_Get_Name(int Cmd_ID)
 
 	case ID_CMD_TABLES_DIAGRAM:			return( _TL("Diagram") );
 
-	case ID_CMD_GRIDS_SAVEAS_IMAGE:		return( _TL("Save As Image...") );
+	case ID_CMD_GRIDS_SAVEAS_IMAGE:		return( _TL("Save as Image...") );
 	case ID_CMD_GRIDS_EQUALINTERVALS:	return( _TL("Create Normalised Classification") );
-	case ID_CMD_GRIDS_FIT_MINMAX:		return( _TL("Fit Colours to Minimum/Maximum") );
-	case ID_CMD_GRIDS_FIT_STDDEV:		return( _TL("Fit Colours to Standard Deviation") );
-	case ID_CMD_GRIDS_FIT_DIALOG:		return( _TL("Fit Colours Dialog") );
+	case ID_CMD_GRIDS_FIT_MINMAX:		return( _TL("Histogram Stretch to Minimum/Maximum") );
+	case ID_CMD_GRIDS_FIT_STDDEV:		return( _TL("Histogram Stretch to Standard Deviation") );
+	case ID_CMD_GRIDS_FIT_PCTL  :		return( _TL("Histogram Stretch to Percentile") );
+	case ID_CMD_GRIDS_FIT_DIALOG:		return( _TL("Histogram Stretch Dialog") );
 	case ID_CMD_GRIDS_SEL_CLEAR:		return( _TL("Clear Selection") );
 	case ID_CMD_GRIDS_SEL_DELETE:		return( _TL("Delete Selection") );
 
-	case ID_CMD_SHAPES_EDIT_SHAPE:		return( _TL("Edit Selected Shape") );
+	case ID_CMD_SHAPES_SAVE_ATTRIBUTES:	return( _TL("Save Attributes as...") );
+	case ID_CMD_SHAPES_EDIT_SHAPE:		return( _TL("Edit Selection") );
 	case ID_CMD_SHAPES_EDIT_SEL_CLEAR:	return( _TL("Clear Selection") );
 	case ID_CMD_SHAPES_EDIT_SEL_INVERT:	return( _TL("Invert Selection") );
 	case ID_CMD_SHAPES_EDIT_ADD_SHAPE:	return( _TL("Add Shape") );
 	case ID_CMD_SHAPES_EDIT_ADD_PART:	return( _TL("Add Part") );
-	case ID_CMD_SHAPES_EDIT_DEL_SHAPE:	return( _TL("Delete Selected Shape(s)") );
+	case ID_CMD_SHAPES_EDIT_DEL_SHAPE:	return( _TL("Delete Selection") );
 	case ID_CMD_SHAPES_EDIT_DEL_PART:	return( _TL("Delete Selected Part") );
 	case ID_CMD_SHAPES_EDIT_DEL_POINT:	return( _TL("Delete Selected Point") );
 	case ID_CMD_SHAPES_EDIT_SPLIT:		return( _TL("Split with Line") );
@@ -211,9 +217,10 @@ wxString CMD_Get_Name(int Cmd_ID)
 	case ID_CMD_MAPS_SHOW:						return( _TL("Show Map") );
 	case ID_CMD_MAPS_3D_SHOW:					return( _TL("Show 3D-View") );
 	case ID_CMD_MAPS_LAYOUT_SHOW:				return( _TL("Show Print Layout") );
-	case ID_CMD_MAPS_SAVE_IMAGE:				return( _TL("Save As Image...") );
-	case ID_CMD_MAPS_SAVE_IMAGE_ON_CHANGE:		return( _TL("Save As Image when changed") );
-	case ID_CMD_MAPS_SAVE_IMAGE_TO_MEMORY:		return( _TL("Save As Image to Workspace") );
+	case ID_CMD_MAPS_SAVE_IMAGE:				return( _TL("Save as Image...") );
+	case ID_CMD_MAPS_SAVE_IMAGE_ON_CHANGE:		return( _TL("Save as Image when changed") );
+	case ID_CMD_MAPS_SAVE_IMAGE_TO_KMZ:			return( _TL("Save as KMZ...") );
+	case ID_CMD_MAPS_SAVE_IMAGE_TO_MEMORY:		return( _TL("Save as Image to Workspace") );
 	case ID_CMD_MAPS_SAVE_TO_CLIPBOARD:			return( _TL("Copy Map to Clipboard [Ctrl+C]") );
 	case ID_CMD_MAPS_SAVE_TO_CLIPBOARD_LEGEND:	return( _TL("Copy Legend to Clipboard [Ctrl+L]") );
 	case ID_CMD_MAPS_SYNCHRONIZE:				return( _TL("Synchronise Map Extents") );
@@ -225,7 +232,7 @@ wxString CMD_Get_Name(int Cmd_ID)
 	case ID_CMD_MAPS_MOVE_BOTTOM:				return( _TL("Move To Bottom") );
 	case ID_CMD_MAPS_MOVE_UP:					return( _TL("Move Up") );
 	case ID_CMD_MAPS_MOVE_DOWN:					return( _TL("Move Down") );
-	case ID_CMD_MAPS_GRID_FITCOLORS:			return( _TL("Fit Grid Colors To Map Extent") );
+	case ID_CMD_MAPS_GRID_FITCOLORS:			return( _TL("Adjust Histogram Stretch to Map Extent") );
 
 	//-----------------------------------------------------
 	// Views...
@@ -275,14 +282,14 @@ wxString CMD_Get_Name(int Cmd_ID)
 	case ID_CMD_MAP3D_STEREO_MORE:		return( _TL("Increase Eye Distance") );
 	case ID_CMD_MAP3D_STEREO:			return( _TL("Anaglyph [S]") );
 	case ID_CMD_MAP3D_INTERPOLATED:		return( _TL("Interpolated Colors") );
-	case ID_CMD_MAP3D_SAVE:				return( _TL("Save As Image...") );
-	case ID_CMD_MAP3D_SEQ_POS_ADD:		return( _TL("Add Position [A]") );
-	case ID_CMD_MAP3D_SEQ_POS_DEL:		return( _TL("Delete Last Position [D]") );
+	case ID_CMD_MAP3D_SAVE:				return( _TL("Save as Image...") );
+	case ID_CMD_MAP3D_SEQ_POS_ADD:		return( _TL("Add Position [Ctrl + A]") );
+	case ID_CMD_MAP3D_SEQ_POS_DEL:		return( _TL("Delete Last Position [Ctrl + D]") );
 	case ID_CMD_MAP3D_SEQ_POS_DEL_ALL:	return( _TL("Delete All Positions") );
 	case ID_CMD_MAP3D_SEQ_POS_EDIT:		return( _TL("Edit Positions") );
-	case ID_CMD_MAP3D_SEQ_PLAY:			return( _TL("Play Once [P]") );
-	case ID_CMD_MAP3D_SEQ_PLAY_LOOP:	return( _TL("Play Loop [L]") );
-	case ID_CMD_MAP3D_SEQ_SAVE:			return( _TL("Play And Save As Images...") );
+	case ID_CMD_MAP3D_SEQ_PLAY:			return( _TL("Play Once [Ctrl + P]") );
+	case ID_CMD_MAP3D_SEQ_PLAY_LOOP:	return( _TL("Play Loop [Ctrl + L]") );
+	case ID_CMD_MAP3D_SEQ_SAVE:			return( _TL("Play And Save as Images...") );
 
 	case ID_CMD_LAYOUT_PRINT_SETUP:		return( _TL("Print Setup") );
 	case ID_CMD_LAYOUT_PAGE_SETUP:		return( _TL("Page Setup") );
@@ -295,12 +302,14 @@ wxString CMD_Get_Name(int Cmd_ID)
 	case ID_CMD_TABLE_FIELD_SORT:		return( _TL("Sort Fields") );
 	case ID_CMD_TABLE_FIELD_RENAME:		return( _TL("Rename Fields") );
 	case ID_CMD_TABLE_FIELD_TYPE:		return( _TL("Change Field Type") );
+	case ID_CMD_TABLE_FIELD_OPEN_APP:	return( _TL("Open with external application") );
+	case ID_CMD_TABLE_FIELD_OPEN_DATA:	return( _TL("Open as data set") );
 	case ID_CMD_TABLE_RECORD_ADD:		return( _TL("Add Record") );
 	case ID_CMD_TABLE_RECORD_INS:		return( _TL("Insert Record") );
-	case ID_CMD_TABLE_RECORD_DEL:		return( _TL("Delete Selected Record(s)") );
+	case ID_CMD_TABLE_RECORD_DEL:		return( _TL("Delete Selection") );
 	case ID_CMD_TABLE_RECORD_DEL_ALL:	return( _TL("Delete All Records") );
-	case ID_CMD_TABLE_SELECTION_TO_TOP:	return( _TL("Sort Selection to Top") );
-	case ID_CMD_TABLE_SELECTION_ONLY:	return( _TL("Only Show Selection") );
+	case ID_CMD_TABLE_SELECTION_CLEAR:	return( _TL("Clear Selection") );
+	case ID_CMD_TABLE_SELECTION_ONLY:	return( _TL("Show Selection") );
 	case ID_CMD_TABLE_AUTOSIZE_COLS:	return( _TL("Fit Column Sizes") );
 	case ID_CMD_TABLE_AUTOSIZE_ROWS:	return( _TL("Fit Row Sizes") );
 
@@ -315,8 +324,9 @@ wxString CMD_Get_Name(int Cmd_ID)
 	case ID_CMD_SCATTERPLOT_AS_TABLE:		return( _TL("Convert To Table") );
 	case ID_CMD_SCATTERPLOT_TO_CLIPBOARD:	return( _TL("Copy to Clipboard") );
 
-	case ID_CMD_HISTOGRAM_CUMULATIVE:	return( _TL("Cumulative") );
-	case ID_CMD_HISTOGRAM_AS_TABLE:		return( _TL("Convert To Table") );
+	case ID_CMD_HISTOGRAM_CUMULATIVE:		return( _TL("Cumulative") );
+	case ID_CMD_HISTOGRAM_AS_TABLE:			return( _TL("Convert To Table") );
+	case ID_CMD_HISTOGRAM_TO_CLIPBOARD:		return( _TL("Copy to Clipboard") );
 	}
 
 	//-----------------------------------------------------
@@ -345,8 +355,8 @@ int CMD_Get_ImageID(int Cmd_ID)
 	case ID_CMD_FRAME_ACTIVE_SHOW:		return( ID_IMG_TB_ACTIVE );
 	case ID_CMD_FRAME_INFO_SHOW:		return( ID_IMG_TB_INFO );
 	case ID_CMD_FRAME_HELP:				return( ID_IMG_TB_HELP );
-	case ID_CMD_WKSP_OPEN:				return( ID_IMG_TB_OPEN );
-	case ID_CMD_WKSP_SAVE:				return( ID_IMG_TB_SAVE );
+	case ID_CMD_DATA_OPEN:				return( ID_IMG_TB_OPEN );
+	case ID_CMD_DATA_PROJECT_SAVE:		return( ID_IMG_TB_SAVE );
 	case ID_CMD_MODULES_OPEN:			return( ID_IMG_TB_OPEN_MODULE );
 	case ID_CMD_MODULES_SEARCH:			return( ID_IMG_NB_WKSP_MODULES );
 	case ID_CMD_TABLES_OPEN:			return( ID_IMG_TB_OPEN_TABLE );
