@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: res_commands.h 2017 2014-02-25 15:48:09Z oconrad $
+ * Version $Id: res_commands.h 2698 2015-11-24 13:45:01Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -113,8 +113,6 @@ enum ID_COMMANDS
 
 	//-----------------------------------------------------
 	ID_CMD_WKSP_FIRST,
-	ID_CMD_WKSP_OPEN,
-	ID_CMD_WKSP_SAVE,
 	ID_CMD_WKSP_ITEM_RETURN,
 	ID_CMD_WKSP_ITEM_CLOSE,
 	ID_CMD_WKSP_ITEM_SETTINGS_LOAD,
@@ -123,11 +121,14 @@ enum ID_COMMANDS
 	ID_CMD_WKSP_ITEM_SHOW,
 
 	ID_CMD_INFO_CLEAR,
+	ID_CMD_INFO_SELECTALL,
 	ID_CMD_INFO_COPY,
 
 	ID_CMD_DATASOURCE_REFRESH,
 
 	ID_CMD_DB_REFRESH,
+	ID_CMD_DB_SOURCE_CREATE,
+	ID_CMD_DB_SOURCE_DROP,
 	ID_CMD_DB_SOURCE_OPEN,
 	ID_CMD_DB_SOURCE_CLOSE,
 	ID_CMD_DB_SOURCE_CLOSE_ALL,
@@ -145,20 +146,36 @@ enum ID_COMMANDS
 	ID_CMD_MODULES_SEARCH,
 	ID_CMD_MODULES_SAVE_SCRIPT,
 	ID_CMD_MODULES_SAVE_DOCS,
+	ID_CMD_MODULES_SAVE_TO_CLIPBOARD,
+	ID_CMD_MODULES_CHAIN_RELOAD,
+	ID_CMD_MODULES_CHAIN_EDIT,
 	ID_CMD_MODULES_LAST,
 
 	ID_CMD_DATA_FIRST,
 	ID_CMD_DATA_PROJECT_RECENT_FIRST,
 	ID_CMD_DATA_PROJECT_RECENT_LAST	= RECENT_COUNT + ID_CMD_DATA_PROJECT_RECENT_FIRST,
-	ID_CMD_DATA_PROJECT_NEW,
 	ID_CMD_DATA_PROJECT_OPEN,
 	ID_CMD_DATA_PROJECT_OPEN_ADD,
+	ID_CMD_DATA_PROJECT_BROWSE,
+	ID_CMD_DATA_PROJECT_CLOSE,
 	ID_CMD_DATA_PROJECT_SAVE,
 	ID_CMD_DATA_PROJECT_SAVE_AS,
+	ID_CMD_DATA_PROJECT_COPY,
+	ID_CMD_DATA_PROJECT_COPY_DB,
+
 	ID_CMD_DATA_LEGEND_COPY,
 	ID_CMD_DATA_LEGEND_SIZE_INC,
 	ID_CMD_DATA_LEGEND_SIZE_DEC,
+
 	ID_CMD_DATA_HISTORY_CLEAR,
+	ID_CMD_DATA_HISTORY_TO_MODEL,
+	ID_CMD_DATA_HISTORY_OPTIONS_COLLAPSE,
+	ID_CMD_DATA_HISTORY_OPTIONS_EXPAND,
+
+	ID_CMD_DATA_OPEN,
+	ID_CMD_DATA_SAVE,
+	ID_CMD_DATA_SAVEAS,
+	ID_CMD_DATA_SAVETODB,
 	ID_CMD_DATA_PROJECTION,
 	ID_CMD_DATA_LAST,
 
@@ -166,8 +183,6 @@ enum ID_COMMANDS
 	ID_CMD_GRIDS_RECENT_FIRST,
 	ID_CMD_GRIDS_RECENT_LAST		= RECENT_COUNT + ID_CMD_GRIDS_RECENT_FIRST,
 	ID_CMD_GRIDS_OPEN,
-	ID_CMD_GRIDS_SAVE,
-	ID_CMD_GRIDS_SAVEAS,
 	ID_CMD_GRIDS_SAVEAS_IMAGE,
 	ID_CMD_GRIDS_SHOW,
 	ID_CMD_GRIDS_HISTOGRAM,
@@ -175,6 +190,7 @@ enum ID_COMMANDS
 	ID_CMD_GRIDS_EQUALINTERVALS,
 	ID_CMD_GRIDS_FIT_MINMAX,
 	ID_CMD_GRIDS_FIT_STDDEV,
+	ID_CMD_GRIDS_FIT_PCTL,
 	ID_CMD_GRIDS_FIT_DIALOG,
 	ID_CMD_GRIDS_SET_LUT,
 	ID_CMD_GRIDS_SEL_CLEAR,
@@ -185,14 +201,13 @@ enum ID_COMMANDS
 	ID_CMD_SHAPES_RECENT_FIRST,
 	ID_CMD_SHAPES_RECENT_LAST		= RECENT_COUNT + ID_CMD_SHAPES_RECENT_FIRST,
 	ID_CMD_SHAPES_OPEN,
-	ID_CMD_SHAPES_SAVE,
-	ID_CMD_SHAPES_SAVEAS,
 	ID_CMD_SHAPES_SHOW,
 	ID_CMD_SHAPES_HISTOGRAM,
 	ID_CMD_SHAPES_RANGE_MINMAX,
 	ID_CMD_SHAPES_RANGE_STDDEV150,
 	ID_CMD_SHAPES_RANGE_STDDEV200,
 	ID_CMD_SHAPES_SET_LUT,
+	ID_CMD_SHAPES_SAVE_ATTRIBUTES,
 	ID_CMD_SHAPES_EDIT_SHAPE,
 	ID_CMD_SHAPES_EDIT_SEL_CLEAR,
 	ID_CMD_SHAPES_EDIT_SEL_INVERT,
@@ -210,8 +225,6 @@ enum ID_COMMANDS
 	ID_CMD_TABLES_RECENT_FIRST,
 	ID_CMD_TABLES_RECENT_LAST		= RECENT_COUNT + ID_CMD_TABLES_RECENT_FIRST,
 	ID_CMD_TABLES_OPEN,
-	ID_CMD_TABLES_SAVE,
-	ID_CMD_TABLES_SAVEAS,
 	ID_CMD_TABLES_SHOW,
 	ID_CMD_TABLES_DIAGRAM,
 	ID_CMD_TABLES_SCATTERPLOT,
@@ -221,8 +234,6 @@ enum ID_COMMANDS
 	ID_CMD_TIN_RECENT_FIRST,
 	ID_CMD_TIN_RECENT_LAST			= RECENT_COUNT + ID_CMD_TIN_RECENT_FIRST,
 	ID_CMD_TIN_OPEN,
-	ID_CMD_TIN_SAVE,
-	ID_CMD_TIN_SAVEAS,
 	ID_CMD_TIN_SHOW,
 	ID_CMD_TIN_LAST,
 
@@ -230,8 +241,6 @@ enum ID_COMMANDS
 	ID_CMD_POINTCLOUD_RECENT_FIRST,
 	ID_CMD_POINTCLOUD_RECENT_LAST	= RECENT_COUNT + ID_CMD_POINTCLOUD_RECENT_FIRST,
 	ID_CMD_POINTCLOUD_OPEN,
-	ID_CMD_POINTCLOUD_SAVE,
-	ID_CMD_POINTCLOUD_SAVEAS,
 	ID_CMD_POINTCLOUD_SHOW,
 	ID_CMD_POINTCLOUD_RANGE_MINMAX,
 	ID_CMD_POINTCLOUD_RANGE_STDDEV150,
@@ -244,6 +253,7 @@ enum ID_COMMANDS
 	ID_CMD_MAPS_LAYOUT_SHOW,
 	ID_CMD_MAPS_SAVE_IMAGE,
 	ID_CMD_MAPS_SAVE_IMAGE_ON_CHANGE,
+	ID_CMD_MAPS_SAVE_IMAGE_TO_KMZ,
 	ID_CMD_MAPS_SAVE_IMAGE_TO_MEMORY,
 	ID_CMD_MAPS_SAVE_TO_CLIPBOARD,
 	ID_CMD_MAPS_SAVE_TO_CLIPBOARD_LEGEND,
@@ -330,11 +340,13 @@ enum ID_COMMANDS
 	ID_CMD_TABLE_FIELD_SORT,
 	ID_CMD_TABLE_FIELD_RENAME,
 	ID_CMD_TABLE_FIELD_TYPE,
+	ID_CMD_TABLE_FIELD_OPEN_APP,
+	ID_CMD_TABLE_FIELD_OPEN_DATA,
 	ID_CMD_TABLE_RECORD_ADD,
 	ID_CMD_TABLE_RECORD_INS,
 	ID_CMD_TABLE_RECORD_DEL,
 	ID_CMD_TABLE_RECORD_DEL_ALL,
-	ID_CMD_TABLE_SELECTION_TO_TOP,
+	ID_CMD_TABLE_SELECTION_CLEAR,
 	ID_CMD_TABLE_SELECTION_ONLY,
 	ID_CMD_TABLE_AUTOSIZE_COLS,
 	ID_CMD_TABLE_AUTOSIZE_ROWS,
@@ -359,6 +371,7 @@ enum ID_COMMANDS
 	ID_CMD_HISTOGRAM_TOOLBAR,
 	ID_CMD_HISTOGRAM_CUMULATIVE,
 	ID_CMD_HISTOGRAM_AS_TABLE,
+	ID_CMD_HISTOGRAM_TO_CLIPBOARD,
 	ID_CMD_HISTOGRAM_LAST,
 
 	ID_CMD_LAYOUT_FIRST,
