@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: res_controls.h 1939 2014-01-14 16:48:06Z oconrad $
+ * Version $Id: res_controls.h 2682 2015-11-06 12:44:37Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -168,6 +168,9 @@ enum ID_CONTROLS
 
 	ID_BTN_DEFAULTS,
 
+	ID_BTN_TABLE_TO_WKSP,
+	ID_BTN_TABLE_FROM_WKSP,
+
 	ID_BTN_COLORS,
 	ID_BTN_COLORS_MIRROR,
 	ID_BTN_COLORS_INVERT,
