@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: res_dialogs.cpp 2275 2014-10-02 15:48:59Z oconrad $
+ * Version $Id: res_dialogs.cpp 2397 2015-02-03 15:07:22Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -733,12 +733,24 @@ bool		DLG_Directory(wxString &Directory, const wxString &Caption)
 //---------------------------------------------------------
 bool		DLG_Save(wxString &File_Path, const wxString &Caption, const wxString &def_Dir, const wxString &def_File, const wxString &Filter)
 {
-	wxFileDialog	dlg(MDI_Get_Top_Window(), Caption, def_Dir, def_File, Filter, wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
+	wxString	Dir(def_Dir);
+
+	if( !wxDirExists(def_Dir) )
+	{
+		CONFIG_Read(CONFIG_GROUP_FILE_DLG, DLG_Get_FILE_Config(-1), Dir);
+	}
+
+	wxFileDialog	dlg(MDI_Get_Top_Window(), Caption, Dir, def_File, Filter, wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
 
 	if( dlg.ShowModal() == wxID_OK )
 	{
 		File_Path	= dlg.GetPath();
 
+		if( !wxDirExists(def_Dir) )
+		{
+			CONFIG_Write(CONFIG_GROUP_FILE_DLG, DLG_Get_FILE_Config(-1), SG_File_Get_Path(File_Path).w_str());
+		}
+
 		return( true );
 	}
 
@@ -752,7 +764,7 @@ bool		DLG_Save(wxString &File_Path, int ID_DLG)
 	def_Name	= SG_File_Get_Name(File_Path, true).w_str();
 	def_Dir		= SG_File_Get_Path(File_Path).w_str();
 
-	if( !wxFileExists(def_Dir) )
+	if( !wxDirExists(def_Dir) )
 	{
 		CONFIG_Read(CONFIG_GROUP_FILE_DLG, DLG_Get_FILE_Config(ID_DLG), def_Dir);
 	}
@@ -775,12 +787,24 @@ bool		DLG_Save(wxString &File_Path, const wxString &Caption, const wxString &Fil
 //---------------------------------------------------------
 bool		DLG_Open(wxString &File_Path, const wxString &Caption, const wxString &def_Dir, const wxString &def_File, const wxString &Filter)
 {
-	wxFileDialog	dlg(MDI_Get_Top_Window(), Caption, def_Dir, def_File, Filter, wxFD_OPEN|wxFD_FILE_MUST_EXIST);
+	wxString	Dir(def_Dir);
+
+	if( !wxDirExists(def_Dir) )
+	{
+		CONFIG_Read(CONFIG_GROUP_FILE_DLG, DLG_Get_FILE_Config(-1), Dir);
+	}
+
+	wxFileDialog	dlg(MDI_Get_Top_Window(), Caption, Dir, def_File, Filter, wxFD_OPEN|wxFD_FILE_MUST_EXIST);
 
 	if( dlg.ShowModal() == wxID_OK )
 	{
 		File_Path	= dlg.GetPath();
 
+		if( !wxDirExists(def_Dir) )
+		{
+			CONFIG_Write(CONFIG_GROUP_FILE_DLG, DLG_Get_FILE_Config(-1), SG_File_Get_Path(File_Path).w_str());
+		}
+
 		return( true );
 	}
 
@@ -811,13 +835,28 @@ bool		DLG_Open(wxString &File_Path, const wxString &Caption, const wxString &Fil
 //---------------------------------------------------------
 bool		DLG_Open(wxArrayString &File_Paths, const wxString &Caption, const wxString &def_Dir, const wxString &Filter)
 {
-	wxFileDialog	dlg(MDI_Get_Top_Window(), Caption, def_Dir, wxT(""), Filter, wxFD_OPEN|wxFD_FILE_MUST_EXIST|wxFD_MULTIPLE);
+	wxString	Dir(def_Dir);
+
+	if( !wxDirExists(def_Dir) )
+	{
+		CONFIG_Read(CONFIG_GROUP_FILE_DLG, DLG_Get_FILE_Config(-1), Dir);
+	}
+
+	wxFileDialog	dlg(MDI_Get_Top_Window(), Caption, Dir, wxT(""), Filter, wxFD_OPEN|wxFD_FILE_MUST_EXIST|wxFD_MULTIPLE);
 
 	if( dlg.ShowModal() == wxID_OK )
 	{
 		dlg.GetPaths(File_Paths);
 
-		return( File_Paths.GetCount() > 0 );
+		if( File_Paths.GetCount() > 0 )
+		{
+			if( !wxDirExists(def_Dir) )
+			{
+				CONFIG_Write(CONFIG_GROUP_FILE_DLG, DLG_Get_FILE_Config(-1), SG_File_Get_Path(File_Paths[0]).w_str());
+			}
+
+			return( true );
+		}
 	}
 
 	return( false );
@@ -856,8 +895,15 @@ bool		DLG_Image_Save(wxString &File_Path, int &Type, const wxString &def_Dir, co
 {
 	static	int	Filter_Index	= 3;
 
+	wxString	Dir(def_Dir);
+
+	if( !wxDirExists(def_Dir) )
+	{
+		CONFIG_Read(CONFIG_GROUP_FILE_DLG, DLG_Get_FILE_Config(-1), Dir);
+	}
+
 	wxFileDialog	dlg(
-		MDI_Get_Top_Window(), _TL("Save As Image"), def_Dir, def_File, wxString::Format(
+		MDI_Get_Top_Window(), _TL("Save As Image"), Dir, def_File, wxString::Format(
 			wxT("%s (*.bmp)|*.bmp|")
 			wxT("%s (*.jpg)|*.jpg;*.jif;*.jpeg|")
 			wxT("%s (*.tif)|*.tif;*.tiff|")
@@ -892,6 +938,11 @@ bool		DLG_Image_Save(wxString &File_Path, int &Type, const wxString &def_Dir, co
 		case 6:	Type	= wxBITMAP_TYPE_PNM;	break;
 		}
 
+		if( !wxDirExists(def_Dir) )
+		{
+			CONFIG_Write(CONFIG_GROUP_FILE_DLG, DLG_Get_FILE_Config(-1), SG_File_Get_Path(File_Path).w_str());
+		}
+
 		return( true );
 	}
 
