@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: saga.cpp 2070 2014-03-28 11:18:40Z reklov_w $
+ * Version $Id: saga.cpp 2694 2015-11-20 16:43:15Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -78,6 +78,7 @@
 #include "saga_frame.h"
 
 #include "wksp.h"
+#include "wksp_data_manager.h"
 
 
 ///////////////////////////////////////////////////////////
@@ -140,7 +141,9 @@ bool CSAGA::OnInit(void)
 
 	wxInitAllImageHandlers();
 
-	m_App_Path			= wxFileName(argv[0]).GetPath();
+	wxFileName			App_Path(argv[0]);	App_Path.MakeAbsolute();
+
+	m_App_Path			= App_Path.GetPath();
 
 #if !defined(_DEBUG)
 	wxSetAssertHandler(NULL);		// disable all wx asserts in SAGA release builds
@@ -201,6 +204,7 @@ bool CSAGA::OnInit(void)
 	}
 
 	wxSetEnv("GDAL_DRIVER_PATH", Get_App_Path() + "\\dll");
+	wxSetEnv("GDAL_DATA"       , Get_App_Path() + "\\dll\\gdal-data");
 #endif // defined(_SAGA_MSW)
 
 	//-----------------------------------------------------
@@ -263,6 +267,10 @@ bool CSAGA::OnInit(void)
 			g_pWKSP->Open(argv[i]);
 		}
 	}
+	else
+	{
+		g_pData->Initialise();
+	}
 
 	//-----------------------------------------------------
 	g_pSAGA_Frame->Show_Tips(false);
@@ -285,22 +293,41 @@ int CSAGA::OnExit(void)
 //														 //
 ///////////////////////////////////////////////////////////
 
+//---------------------------------------------------------
+#include <wx/wfstream.h>
+
 //---------------------------------------------------------
 void CSAGA::_Init_Config(void)
 {
 	wxConfigBase	*pConfig;
 
 #if defined(_SAGA_MSW)
-	wxFileName	fConfig(Get_App_Path(), "saga_gui", "ini");
+	wxFileName	fLocal(Get_App_Path(), "saga_gui", "ini");
 
-	if(	( fConfig.FileExists() && (!fConfig.IsFileReadable() || !fConfig.IsFileWritable()))
-	||	(!fConfig.FileExists() && (!fConfig.IsDirReadable () || !fConfig.IsDirWritable ())) )
+	if( ( fLocal.FileExists() && (!fLocal.IsFileReadable() || !fLocal.IsFileWritable()))
+	||  (!fLocal.FileExists() && (!fLocal.IsDirReadable () || !fLocal.IsDirWritable ())) )
 	{
-		fConfig.Assign(wxGetHomeDir(), "saga_gui", "ini");
-		//fConfig.Assign(wxFileName::GetTempDir(), "saga_gui", "ini");
+		wxFileName	fUser (wxGetHomeDir(), "saga_gui", "ini");
+	//	wxFileName	fUser (wxStandardPaths::Get().GetUserConfigDir(), "saga_gui", "ini");
+
+		if(	fLocal.FileExists() && fLocal.IsFileReadable() && !fUser.FileExists() )	// create a copy in user's home directory
+		{
+			wxFileInputStream	is(fLocal.GetFullPath());
+			wxFileOutputStream	os(fUser .GetFullPath());
+			wxFileConfig		ic(is);	ic.Save(os);
+		}
+
+		fLocal	= fUser;
 	}
 
-	pConfig = new wxFileConfig(wxEmptyString, wxEmptyString, fConfig.GetFullPath(), fConfig.GetFullPath(), wxCONFIG_USE_LOCAL_FILE|wxCONFIG_USE_GLOBAL_FILE|wxCONFIG_USE_RELATIVE_PATH);
+	if( (fLocal.FileExists() && fLocal.IsFileWritable()) || (!fLocal.FileExists() && fLocal.IsDirWritable()) )
+	{
+		pConfig = new wxFileConfig(wxEmptyString, wxEmptyString, fLocal.GetFullPath(), fLocal.GetFullPath(), wxCONFIG_USE_LOCAL_FILE|wxCONFIG_USE_GLOBAL_FILE|wxCONFIG_USE_RELATIVE_PATH);
+	}
+	else
+	{
+		pConfig	= new wxConfig;	// this might go to registry
+	}
 #else
 	pConfig	= new wxConfig;
 #endif
@@ -355,17 +382,21 @@ bool CSAGA::Process_Wait(bool bEnforce)
 	static bool			bYield	= false;
 	static wxDateTime	tYield	= wxDateTime::UNow();
 
-	if( !bYield && (bEnforce || m_Process_Frequency <= 0 || (wxDateTime::Now() - tYield).GetMilliseconds() > m_Process_Frequency) )
+	if( !bYield && (bEnforce || m_Process_Frequency <= 0 || m_Process_Frequency <= (wxDateTime::UNow() - tYield).GetMilliseconds()) )
 	{
 		bYield	= true;
 
-		//	Yield();
-		//	wxSafeYield(g_pSAGA_Frame);
+	//	Yield();
+	//	wxSafeYield(g_pSAGA_Frame);
 
 		while( Pending() && Dispatch() );
 
+		if( m_Process_Frequency > 0 )
+		{
+			tYield	= wxDateTime::UNow();
+		}
+
 		bYield	= false;
-		tYield	= wxDateTime::UNow();
 	}
 
 	return( true );
