@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: saga_frame.cpp 2047 2014-03-06 13:02:29Z oconrad $
+ * Version $Id: saga_frame.cpp 2724 2015-12-11 10:24:35Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -71,6 +71,7 @@
 #include <wx/toolbar.h>
 #include <wx/tipdlg.h>
 #include <wx/aui/aui.h>
+#include <wx/display.h>
 
 #include <saga_api/saga_api.h>
 
@@ -94,7 +95,6 @@
 
 #include "wksp_module_manager.h"
 #include "wksp_module.h"
-#include "wksp_module_menu.h"
 
 #include "wksp_data_manager.h"
 #include "wksp_data_menu_files.h"
@@ -139,10 +139,10 @@ public:
 	CSAGA_Frame_StatusBar(wxWindow *parent, wxWindowID id, long style = wxST_SIZEGRIP, const wxString& name = wxT("statusBar"))
 		: wxStatusBar(parent, id, style, name)
 	{
-		m_pProgressBar	= new wxGauge(this, ID_WND_PROGRESSBAR, 100);
+		m_pProgressBar	= new wxGauge(this, ID_WND_PROGRESSBAR, 100, wxDefaultPosition, wxDefaultSize, wxGA_HORIZONTAL|wxGA_SMOOTH);
 	}
 
-	void		On_Size		(wxSizeEvent     &event)
+	void		On_Size		(wxSizeEvent &event)
 	{
 		wxRect	r;
 
@@ -150,8 +150,6 @@ public:
 		{
 			m_pProgressBar->SetSize(r);
 		}
-
-		event.Skip();
 	}
 
 	wxGauge		*m_pProgressBar;
@@ -266,7 +264,7 @@ CSAGA_Frame::CSAGA_Frame(void)
 	SetDropTarget		(new CSAGA_Frame_DropTarget);
 
 	//-----------------------------------------------------
-	int		STATUSBAR_Sizes[STATUSBAR_COUNT]	= {	-1, -1, 90, 90, 90, -1	};
+	int		STATUSBAR_Sizes[STATUSBAR_COUNT]	= {	-1, 125, 125, 125, -1	};
 
 	CreateStatusBar		(STATUSBAR_COUNT);
 	SetStatusWidths		(STATUSBAR_COUNT, STATUSBAR_Sizes);
@@ -316,48 +314,48 @@ CSAGA_Frame::CSAGA_Frame(void)
 
 	wxString	s;
 
-	if( CONFIG_Read(wxT("/FL"), wxT("MANAGER"), s) )
+	if( CONFIG_Read("/FL", "MANAGER", s) )
 	{
 		m_pLayout->LoadPerspective(s);
 	}
 
 	_Bar_Show(m_pTB_Main, true);
 
-	//-----------------------------------------------------
 	m_pLayout->Update();
 
-#if !defined(_SAGA_LINUX)
-	Show(true);
-#endif
-
-	int		x, y, dx, dy;
+	//-----------------------------------------------------
 	long	l;
+	wxRect	r, rDefault	= wxGetClientDisplayRect();
 
-	x	= CONFIG_Read(wxT("/FL"), wxT("X" ), l) ? l : -1;
-	y	= CONFIG_Read(wxT("/FL"), wxT("Y" ), l) ? l : -1;
-	dx	= CONFIG_Read(wxT("/FL"), wxT("DX"), l) ? l : 800;
-	dy	= CONFIG_Read(wxT("/FL"), wxT("DY"), l) ? l : 600;
+	rDefault.Deflate((int)(0.1 * rDefault.GetWidth()), (int)(0.1 * rDefault.GetHeight()));
 
-	SetSize(x, y, dx, dy);
+	r.SetLeft  (CONFIG_Read("/FL", "X" , l) ? l : rDefault.GetLeft  ());
+	r.SetTop   (CONFIG_Read("/FL", "Y" , l) ? l : rDefault.GetTop   ());
+	r.SetWidth (CONFIG_Read("/FL", "DX", l) ? l : rDefault.GetWidth ());
+	r.SetHeight(CONFIG_Read("/FL", "DY", l) ? l : rDefault.GetHeight());
 
-	if( !(CONFIG_Read(wxT("/FL"), wxT("STATE"), l) && l == 0) )
+	if( wxDisplay::GetFromPoint(r.GetTopLeft    ()) == wxNOT_FOUND
+	&&  wxDisplay::GetFromPoint(r.GetTopRight   ()) == wxNOT_FOUND
+	&&  wxDisplay::GetFromPoint(r.GetBottomLeft ()) == wxNOT_FOUND
+	&&  wxDisplay::GetFromPoint(r.GetBottomRight()) == wxNOT_FOUND	)
+	{
+		r	= rDefault;
+	}
+
+	SetSize(r);
+
+	if( !(CONFIG_Read("/FL", "STATE", l) && l == 0) )
 	{
 		Maximize();
 	}
 
-#if defined(_SAGA_LINUX)
+	//-----------------------------------------------------
 	Show(true);
-#endif
 
 	Update();
 
 	//-----------------------------------------------------
-	if( g_pSAGA->argc <= 1 && g_pData->Initialise() )
-	{
-		Refresh(false);
-	}
-
-	ProgressBar_Set_Position(0);
+	m_pData_Source->Autoconnect_DB();
 }
 
 //---------------------------------------------------------
@@ -371,19 +369,20 @@ CSAGA_Frame::~CSAGA_Frame(void)
 
 	if( IsMaximized() )
 	{
-		CONFIG_Write(wxT("/FL"), wxT("STATE"), (long)1);
+		CONFIG_Write("/FL", "STATE", (long)1);
 	}
 	else
 	{
-		CONFIG_Write(wxT("/FL"), wxT("STATE"), (long)0);
-		CONFIG_Write(wxT("/FL"), wxT("X"    ), (long)GetPosition().x);
-		CONFIG_Write(wxT("/FL"), wxT("Y"    ), (long)GetPosition().y);
-		CONFIG_Write(wxT("/FL"), wxT("DX"   ), (long)GetSize().x);
-		CONFIG_Write(wxT("/FL"), wxT("DY"   ), (long)GetSize().y);
+		CONFIG_Write("/FL", "STATE", (long)0);
+
+		CONFIG_Write("/FL", "X"    , (long)GetPosition().x);
+		CONFIG_Write("/FL", "Y"    , (long)GetPosition().y);
+		CONFIG_Write("/FL", "DX"   , (long)GetSize    ().x);
+		CONFIG_Write("/FL", "DY"   , (long)GetSize    ().y);
 	}
 
 	//-----------------------------------------------------
-	CONFIG_Write(wxT("/FL"), wxT("MANAGER"), m_pLayout->SavePerspective());
+	CONFIG_Write("/FL", "MANAGER", m_pLayout->SavePerspective());
 
 	m_pLayout->UnInit();
 
@@ -421,7 +420,7 @@ void CSAGA_Frame::On_Close(wxCloseEvent &event)
 {
 	if( event.CanVeto() )
 	{
-		if( !g_pModule && DLG_Message_Confirm(ID_DLG_CLOSE) && g_pData->Close(true) )
+		if( !g_pModule && DLG_Message_Confirm(ID_DLG_CLOSE) && g_pData->Finalise() && g_pData->Close(true) )
 		{
 			g_pModules->Finalise();
 
@@ -450,6 +449,13 @@ void CSAGA_Frame::On_Close(wxCloseEvent &event)
 //---------------------------------------------------------
 void CSAGA_Frame::On_Size(wxSizeEvent &event)
 {
+	if( wxDisplay::GetFromWindow(this) == wxNOT_FOUND )
+	{
+		wxRect	r	= wxGetClientDisplayRect();	r.Deflate((int)(0.1 * r.GetWidth()), (int)(0.1 * r.GetHeight()));
+
+		SetSize(r);
+	}
+
 	event.Skip();
 }
 
@@ -752,7 +758,7 @@ bool CSAGA_Frame::Process_Set_Okay(bool bOkay)
 {
 	StatusBar_Set_Text(_TL("ready"));
 
-	ProgressBar_Set_Position(-1);
+	m_pProgressBar->SetValue(0);
 
 	g_pSAGA->Process_Set_Okay(bOkay);
 
@@ -764,28 +770,26 @@ bool CSAGA_Frame::ProgressBar_Set_Position(int Position)
 {
 	if( Position < 0 )
 	{
-		m_pProgressBar->SetValue(0);
+		Position	= 0;
 	}
-	else
+	else if( Position > 100 )
 	{
-		if( Position > 100 )
-		{
-			Position	= 100;
-		}
-
-		if( m_pProgressBar->GetValue() != Position )
-		{
-			m_pProgressBar->SetValue(Position);
-		}
+		Position	= 100;
 	}
 
-	return( Process_Get_Okay(false) );
+	if( m_pProgressBar->GetValue() != Position )
+	{
+		m_pProgressBar->SetValue(Position);
+	//	m_pProgressBar->SetLabel(wxString::Format("%d%%", Position));
+	}
+
+	return( g_pSAGA->Process_Get_Okay() );
 }
 
 //---------------------------------------------------------
 bool CSAGA_Frame::ProgressBar_Set_Position(double Position, double Range)
 {
-	return( ProgressBar_Set_Position(Range != 0.0 ? (int)(100.0 * Position / Range) : 0) );
+	return( ProgressBar_Set_Position(Range < 0.0 ? 0 : (int)(0.5 + 100.0 * Position / Range)) );
 }
 
 //---------------------------------------------------------
@@ -974,7 +978,7 @@ wxMenuBar * CSAGA_Frame::_Create_MenuBar(void)
 	wxMenuBar	*pMenuBar	= new wxMenuBar;
 
 	pMenuBar->Append(g_pData   ->Get_Menu_Files  ()->Get_Menu(), _TL("File"         ));	// 0
-	pMenuBar->Append(g_pModules->Get_Menu_Modules()->Get_Menu(), _TL("Geoprocessing"));	// 1
+	pMenuBar->Append(g_pModules->Get_Menu_Modules()            , _TL("Geoprocessing"));	// 1
 	pMenuBar->Append(pMenu_Window                              , _TL("Window"       ));	// 2
 	pMenuBar->Append(pMenu_Help                                , _TL("?"            ));	// 3
 
@@ -1113,8 +1117,8 @@ wxToolBarBase * CSAGA_Frame::_Create_ToolBar(void)
 {
 	wxToolBarBase	*pToolBar	= TB_Create(ID_TB_MAIN);
 
-	CMD_ToolBar_Add_Item(pToolBar, false, ID_CMD_WKSP_OPEN);
-	CMD_ToolBar_Add_Item(pToolBar, false, ID_CMD_WKSP_SAVE);
+	CMD_ToolBar_Add_Item(pToolBar, false, ID_CMD_DATA_OPEN);
+	CMD_ToolBar_Add_Item(pToolBar, false, ID_CMD_DATA_PROJECT_SAVE);
 	CMD_ToolBar_Add_Separator(pToolBar);
 	CMD_ToolBar_Add_Item(pToolBar, true , ID_CMD_FRAME_WKSP_SHOW);
 	CMD_ToolBar_Add_Item(pToolBar, true , ID_CMD_FRAME_ACTIVE_SHOW);
