@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: view_base.h 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: view_base.h 2061 2014-03-20 11:48:01Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -86,21 +86,28 @@
 class CVIEW_Base : public wxMDIChildFrame
 {
 public:
-	CVIEW_Base(int View_ID, wxString Caption, int Icon_ID);
+	CVIEW_Base(class CWKSP_Base_Item *pOwner, int View_ID, wxString Caption, int Icon_ID, bool bShow = true);
 	virtual ~CVIEW_Base(void);
 
-	void						On_Size				(wxSizeEvent     &event);
-	void						On_Activate			(wxActivateEvent &event);
-
-	void						On_Quit				(wxCommandEvent  &event);
+	virtual void				Do_Destroy			(void);
+	virtual void				Do_Update			(void);
 
 	virtual void				On_Command_UI		(wxUpdateUIEvent &event);
 
 
 protected:
 
+	class CWKSP_Base_Item		*m_pOwner;
+
+
+	void						Do_Show				(void);
+
 	void						Set_Size_Min		(int x, int y);
 
+	void						On_Size				(wxSizeEvent     &event);
+	void						On_Activate			(wxActivateEvent &event);
+	void						On_Quit				(wxCommandEvent  &event);
+
 
 private:
 
@@ -109,10 +116,10 @@ private:
 	wxSize						m_Size_Min;
 
 
-private:
-
-	DECLARE_EVENT_TABLE()
+	//-----------------------------------------------------
 	DECLARE_CLASS(CVIEW_Base)
+	DECLARE_EVENT_TABLE()
+
 };
 
 
