@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: view_histogram.cpp 2272 2014-10-02 15:41:39Z oconrad $
+ * Version $Id: view_histogram.cpp 2549 2015-08-13 13:10:34Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -593,10 +593,10 @@ void CVIEW_Histogram::On_AsTable(wxCommandEvent &event)
 
 		pTable->Set_Name(CSG_String::Format(SG_T("%s: %s"), _TL("Histogram"), pObject->Get_Name()));
 
-		pTable->Add_Field(_TL("CLASS" ), SG_DATATYPE_Int);
+		pTable->Add_Field(_TL("CLASS" ), SG_DATATYPE_Int   );
 		pTable->Add_Field(_TL("AREA"  ), SG_DATATYPE_Double);
-		pTable->Add_Field(_TL("COUNT" ), SG_DATATYPE_Int);
-		pTable->Add_Field(_TL("CUMUL" ), SG_DATATYPE_Int);
+		pTable->Add_Field(_TL("COUNT" ), SG_DATATYPE_Long  );
+		pTable->Add_Field(_TL("CUMUL" ), SG_DATATYPE_Long  );
 		pTable->Add_Field(_TL("NAME"  ), SG_DATATYPE_String);
 		pTable->Add_Field(_TL("MIN"   ), SG_DATATYPE_Double);
 		pTable->Add_Field(_TL("CENTER"), SG_DATATYPE_Double);
@@ -609,8 +609,8 @@ void CVIEW_Histogram::On_AsTable(wxCommandEvent &event)
 			CSG_Table_Record	*pRecord	= pTable->Add_Record();
 
 			pRecord->Set_Value(0, i + 1);
-			pRecord->Set_Value(2, pClassifier->Histogram_Get_Count     (i, false) * dArea);
-			pRecord->Set_Value(1, pClassifier->Histogram_Get_Count     (i, false));
+			pRecord->Set_Value(1, pClassifier->Histogram_Get_Count     (i, false) * dArea);
+			pRecord->Set_Value(2, pClassifier->Histogram_Get_Count     (i, false));
 			pRecord->Set_Value(3, pClassifier->Histogram_Get_Cumulative(i, false));
 			pRecord->Set_Value(4, pClassifier->Get_Class_Name          (i).wx_str());
 			pRecord->Set_Value(5, pClassifier->Get_Class_Value_Minimum (i));
