@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: view_layout_info.cpp 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: view_layout_info.cpp 2548 2015-08-13 13:07:55Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -399,7 +399,7 @@ bool CVIEW_Layout_Info::Draw(wxDC &dc)
 		//-------------------------------------------------
 		if( dc_rMap.GetWidth() > 0 )
 		{
-			m_pMap->Draw_Map(dc, dPaperToDC, dc_rMap, false);
+			m_pMap->Draw_Map(dc, dPaperToDC, dc_rMap, LAYER_DRAW_FLAG_NOEDITS);
 
 			if( dc_MapFrame > 0 )
 			{
