@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: view_map_3d.cpp 2331 2014-11-07 12:15:10Z oconrad $
+ * Version $Id: view_map_3d.cpp 2684 2015-11-06 14:38:52Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -321,12 +321,34 @@ void CVIEW_Map_3D::On_Command(wxCommandEvent &event)
 	//-----------------------------------------------------
 	case ID_CMD_MAP3D_SAVE:
 		{
-			int			FileType;
-			wxString	FileName;
+			int				FileType;
+			wxString		FileName;
+			CSG_Parameters	P(NULL, _TL("Image Resolution"), _TL(""));
 
-			if( DLG_Image_Save(FileName, FileType) )
+			P.Add_Value(NULL, "NX", _TL("Width" ), _TL(""), PARAMETER_TYPE_Int, m_pPanel->GetSize().x, 1, true);
+			P.Add_Value(NULL, "NY", _TL("Height"), _TL(""), PARAMETER_TYPE_Int, m_pPanel->GetSize().y, 1, true);
+
+			if( DLG_Image_Save(FileName, FileType) && DLG_Parameters(&P) )
 			{
-				m_pPanel->Save_asImage(&FileName);
+				Set_Buisy_Cursor(true);
+
+				if( P("NX")->asInt() == m_pPanel->GetSize().x
+				&&  P("NY")->asInt() == m_pPanel->GetSize().y )
+				{
+					m_pPanel->Save_asImage(&FileName);
+				}
+				else
+				{
+					wxSize	Size(m_pPanel->GetSize());
+
+					Freeze();
+					m_pPanel->SetSize(P("NX")->asInt(), P("NY")->asInt());
+					m_pPanel->Save_asImage(&FileName);
+					m_pPanel->SetSize(Size);
+					Thaw();
+				}
+
+				Set_Buisy_Cursor(false);
 			}
 		}
 		return;
@@ -344,7 +366,18 @@ void CVIEW_Map_3D::On_Command(wxCommandEvent &event)
 	case ID_CMD_MAP3D_SEQ_POS_DEL_ALL:	m_pPanel->Play_Pos_Clr();	return;
 	case ID_CMD_MAP3D_SEQ_PLAY       :	m_pPanel->Play_Once   ();	return;
 	case ID_CMD_MAP3D_SEQ_PLAY_LOOP  :	m_pPanel->Play_Loop   ();	return;
-	case ID_CMD_MAP3D_SEQ_SAVE       :	m_pPanel->Play_Save   ();	return;
+	case ID_CMD_MAP3D_SEQ_SAVE       :
+		{
+			int			Type;
+			wxString	File;
+
+			if( DLG_Image_Save(File, Type) )
+			{
+				m_pPanel->m_Parameters("PLAY_FILE")->Set_Value(File.wc_str());
+				m_pPanel->Play_Save();
+			}
+		}
+		return;
 
 	//-----------------------------------------------------
 	case ID_CMD_MAP3D_ROTATE_X_LESS  :	m_pPanel->Get_Projector().Set_xRotation(m_pPanel->Get_Projector().Get_xRotation() + 4.0 * M_DEG_TO_RAD);	break;
@@ -361,7 +394,7 @@ void CVIEW_Map_3D::On_Command(wxCommandEvent &event)
 	case ID_CMD_MAP3D_SHIFT_Z_LESS   :	m_pPanel->Get_Projector().Set_zShift(m_pPanel->Get_Projector().Get_zShift() - 10.0);	break;
 	case ID_CMD_MAP3D_SHIFT_Z_MORE   :	m_pPanel->Get_Projector().Set_zShift(m_pPanel->Get_Projector().Get_zShift() + 10.0);	break;
 
-//	case ID_CMD_MAP3D_DRAW_BOX       :	m_pPanel->m_Parameters("DRAW_BOX"    )->Set_Value(m_pPanel->m_Parameters("DRAW_BOX"    )->asBool() == false    );	break;
+//	case ID_CMD_MAP3D_DRAW_BOX       :	m_pPanel->m_Parameters("DRAW_BOX"    )->Set_Value(m_pPanel->m_Parameters("DRAW_BOX"    )->asBool() ==  true    );	break;
 	case ID_CMD_MAP3D_STEREO         :	m_pPanel->m_Parameters("STEREO"      )->Set_Value(m_pPanel->m_Parameters("STEREO"      )->asBool() == false    );	break;
 	case ID_CMD_MAP3D_STEREO_LESS    :	m_pPanel->m_Parameters("STEREO_DIST" )->Set_Value(m_pPanel->m_Parameters("STEREO_DIST" )->asDouble() - 2       );	break;
 	case ID_CMD_MAP3D_STEREO_MORE    :	m_pPanel->m_Parameters("STEREO_DIST" )->Set_Value(m_pPanel->m_Parameters("STEREO_DIST" )->asDouble() + 2       );	break;
@@ -445,9 +478,10 @@ void CVIEW_Map_3D::Parameters_Update(bool bFromPanel)
 		m_Parameters("CENTRAL"     )->Set_Value(m_pPanel->Get_Projector().is_Central() ? 1 : 0);
 		m_Parameters("CENTRAL_DIST")->Set_Value(m_pPanel->Get_Projector().Get_Central_Distance());
 
-		m_Parameters("BGCOLOR"     )->Set_Value(m_pPanel->m_Parameters("BGCOLOR"    )->asInt());
-		m_Parameters("STEREO"      )->Set_Value(m_pPanel->m_Parameters("STEREO"     )->asInt());
-		m_Parameters("STEREO_DIST" )->Set_Value(m_pPanel->m_Parameters("STEREO_DIST")->asDouble());
+		m_Parameters("BGCOLOR"     )->Set_Value(m_pPanel->m_Parameters("BGCOLOR"    ));
+		m_Parameters("STEREO"      )->Set_Value(m_pPanel->m_Parameters("STEREO"     ));
+		m_Parameters("STEREO_DIST" )->Set_Value(m_pPanel->m_Parameters("STEREO_DIST"));
+		m_Parameters("DRAW_BOX"    )->Set_Value(m_pPanel->m_Parameters("DRAW_BOX"   ));
 
 		m_Parameters("Z_SCALE"     )->Set_Value(m_pPanel->m_zScale);
 		m_Parameters("DEM_RES"     )->Set_Value(m_pPanel->Get_DEM_Res());
@@ -469,18 +503,14 @@ void CVIEW_Map_3D::Parameters_Update(bool bFromPanel)
 			m_Parameters("SHIFT_Z")->asDouble()
 		);
 
-		m_pPanel->Get_Projector().do_Central(
-			m_Parameters("CENTRAL")->asInt() == 1
-		);
+		m_pPanel->m_Parameters("CENTRAL"     )->Set_Value(m_Parameters("CENTRAL"     ));
+		m_pPanel->m_Parameters("CENTRAL_DIST")->Set_Value(m_Parameters("CENTRAL_DIST"));
 
-		m_pPanel->Get_Projector().Set_Central_Distance(
-			m_Parameters("CENTRAL_DIST")->asDouble()
-		);
-
-		m_pPanel->m_Parameters("BGCOLOR"    )->Set_Value(m_Parameters("BGCOLOR"    )->asInt());
-		m_pPanel->m_Parameters("STEREO"     )->Set_Value(m_Parameters("STEREO"     )->asInt());
-		m_pPanel->m_Parameters("STEREO_DIST")->Set_Value(m_Parameters("STEREO_DIST")->asDouble());
-		m_pPanel->m_Parameters("DRAPE_MODE" )->Set_Value(m_Parameters("DRAPE_MODE" )->asInt());
+		m_pPanel->m_Parameters("BGCOLOR"     )->Set_Value(m_Parameters("BGCOLOR"     ));
+		m_pPanel->m_Parameters("STEREO"      )->Set_Value(m_Parameters("STEREO"      ));
+		m_pPanel->m_Parameters("STEREO_DIST" )->Set_Value(m_Parameters("STEREO_DIST" ));
+		m_pPanel->m_Parameters("DRAPE_MODE"  )->Set_Value(m_Parameters("DRAPE_MODE"  ));
+		m_pPanel->m_Parameters("DRAW_BOX"    )->Set_Value(m_Parameters("DRAW_BOX"    ));
 
 		m_pPanel->m_zScale	= m_Parameters("Z_SCALE")->asDouble();
 
@@ -525,6 +555,12 @@ void CVIEW_Map_3D::Parameters_Create(void)
 		PARAMETER_TYPE_Double, 1.0
 	);
 
+	m_Parameters.Add_Value(
+		pNode	, "DRAW_BOX"	, _TL("Bounding Box"),
+		_TL(""),
+		PARAMETER_TYPE_Bool, false
+	);
+
 	//-----------------------------------------------------
 	pNode	= m_Parameters.Add_Node(NULL, "MAP", _TL("Map"), _TL(""));
 
@@ -575,7 +611,7 @@ void CVIEW_Map_3D::Parameters_Create(void)
 	m_Parameters.Add_Value(
 		pNode	, "CENTRAL_DIST"	, _TL("Perspectivic Distance"),
 		_TL(""),
-		PARAMETER_TYPE_Double, 200, 1, true
+		PARAMETER_TYPE_Double, 1500, 1, true
 	);
 
 	//-----------------------------------------------------
