@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: view_map_control.cpp 2027 2014-02-27 15:14:20Z oconrad $
+ * Version $Id: view_map_control.cpp 2735 2015-12-16 17:47:02Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -202,20 +202,71 @@ bool CVIEW_Map_Control::Set_Mode(int Mode)
 //---------------------------------------------------------
 inline void CVIEW_Map_Control::_Set_StatusBar(CSG_Point ptWorld)
 {
-	STATUSBAR_Set_Text(wxString::Format(wxT("X%f"), ptWorld.Get_X()), STATUSBAR_VIEW_X);
-	STATUSBAR_Set_Text(wxString::Format(wxT("Y%f"), ptWorld.Get_Y()), STATUSBAR_VIEW_Y);
+	static bool	bBuisy	= false;
 
-	if( m_Mode == MAP_MODE_DISTANCE )
+	if( bBuisy == false )
 	{
-		STATUSBAR_Set_Text(wxString::Format(wxT("D %f"), m_Distance + m_Distance_Move), STATUSBAR_VIEW_Z);
-	}
-	else if( Get_Active_Layer() )
-	{
-		STATUSBAR_Set_Text(wxString::Format(wxT("Z %s"), Get_Active_Layer()->Get_Value(ptWorld, _Get_World(2.0)).c_str()), STATUSBAR_VIEW_Z);
-	}
-	else
-	{
-		STATUSBAR_Set_Text(wxT("Z"), STATUSBAR_VIEW_Z);
+		bBuisy	= true;
+
+		CSG_Module	*pProjector	= NULL;
+
+		if( m_pMap->Get_Parameter("GCS_POSITION")->asBool() && m_pMap->Get_Projection().is_Okay() && (pProjector = SG_Get_Module_Library_Manager().Get_Module("pj_proj4", 2)) != NULL )	// Coordinate Transformation (Shapes)
+		{
+			if( pProjector->is_Executing() )
+			{
+				pProjector	= NULL;
+			}
+			else
+			{
+				SG_UI_Progress_Lock(true);
+				SG_UI_Msg_Lock     (true);
+
+				CSG_Shapes	prj(SHAPE_TYPE_Point), gcs(SHAPE_TYPE_Point); prj.Add_Shape()->Add_Point(ptWorld); prj.Get_Projection().Assign(m_pMap->Get_Projection());
+
+				pProjector->Settings_Push(NULL);
+
+				if( pProjector->Set_Parameter("CRS_PROJ4", SG_T("+proj=longlat +ellps=WGS84 +datum=WGS84"))
+				&&  pProjector->Set_Parameter("SOURCE"   , &prj)
+				&&  pProjector->Set_Parameter("TARGET"   , &gcs)
+				&&  pProjector->Execute() )
+				{
+					ptWorld	= gcs.Get_Shape(0)->Get_Point(0);
+
+					STATUSBAR_Set_Text(wxString::Format("X %s", SG_Double_To_Degree(ptWorld.Get_X()).c_str()), STATUSBAR_VIEW_X);
+					STATUSBAR_Set_Text(wxString::Format("Y %s", SG_Double_To_Degree(ptWorld.Get_Y()).c_str()), STATUSBAR_VIEW_Y);
+
+					pProjector->Settings_Pop();
+				}
+				else
+				{
+					pProjector->Settings_Pop();		pProjector	= NULL;
+				}
+
+				SG_UI_Progress_Lock(false);
+				SG_UI_Msg_Lock     (false);
+			}
+		}
+
+		if( !pProjector )
+		{
+			STATUSBAR_Set_Text(wxString::Format("X %f", ptWorld.Get_X()), STATUSBAR_VIEW_X);
+			STATUSBAR_Set_Text(wxString::Format("Y %f", ptWorld.Get_Y()), STATUSBAR_VIEW_Y);
+		}
+
+		if( m_Mode == MAP_MODE_DISTANCE )
+		{
+			STATUSBAR_Set_Text(wxString::Format(wxT("D %f"), m_Distance + m_Distance_Move), STATUSBAR_VIEW_Z);
+		}
+		else if( Get_Active_Layer() )
+		{
+			STATUSBAR_Set_Text(wxString::Format(wxT("Z %s"), Get_Active_Layer()->Get_Value(ptWorld, _Get_World(2.0)).c_str()), STATUSBAR_VIEW_Z);
+		}
+		else
+		{
+			STATUSBAR_Set_Text(wxT("Z"), STATUSBAR_VIEW_Z);
+		}
+
+		bBuisy	= false;
 	}
 }
 
@@ -523,7 +574,7 @@ void CVIEW_Map_Control::Refresh_Map(void)
 		wxMemoryDC	dc;
 
 		dc.SelectObject(m_Bitmap);
-		m_pMap->Draw_Map(dc, 1.0, _Get_Client(), true);
+		m_pMap->Draw_Map(dc, 1.0, _Get_Client());
 		dc.SelectObject(wxNullBitmap);
 
 		m_pParent->Ruler_Refresh();
@@ -610,10 +661,29 @@ void CVIEW_Map_Control::On_Key_Down(wxKeyEvent &event)
 //---------------------------------------------------------
 void CVIEW_Map_Control::On_Mouse_LDown(wxMouseEvent &event)
 {
-	bool	bCaptureMouse	= true;
-
 	m_Mouse_Down	= m_Mouse_Move	= event.GetPosition();
 
+	//-----------------------------------------------------
+	if( m_Mode != MAP_MODE_SELECT )	// clipboard copy ?
+	{
+		if( event.ControlDown() )
+		{
+			m_pMap->SaveAs_Image_Clipboard(false);
+
+			return;
+		}
+
+		if( event.AltDown() )
+		{
+			m_pMap->SaveAs_Image_Clipboard(GetClientSize().x, GetClientSize().y, -1);
+
+			return;
+		}
+	}
+
+	//-----------------------------------------------------
+	bool	bCaptureMouse	= true;
+
 	switch( m_Mode )
 	{
 	//-----------------------------------------------------
@@ -632,6 +702,7 @@ void CVIEW_Map_Control::On_Mouse_LDown(wxMouseEvent &event)
 				break;
 
 			case WKSP_ITEM_Grid:
+			case WKSP_ITEM_PointCloud:
 				m_Drag_Mode		= MODULE_INTERACTIVE_DRAG_BOX;
 				break;
 
@@ -685,6 +756,13 @@ void CVIEW_Map_Control::On_Mouse_LUp(wxMouseEvent &event)
 		ReleaseMouse();
 	}
 
+	//-----------------------------------------------------
+	if( m_Mode != MAP_MODE_SELECT && (event.ControlDown() || event.AltDown()) )	// clipboard copy ?
+	{
+		return;
+	}
+
+	//-----------------------------------------------------
 	_Draw_Inverse(m_Mouse_Down, event.GetPosition());
 	m_Drag_Mode	= MODULE_INTERACTIVE_DRAG_NONE;
 
@@ -738,19 +816,7 @@ void CVIEW_Map_Control::On_Mouse_LDClick(wxMouseEvent &event)
 		break;
 
 	//-----------------------------------------------------
-	case MAP_MODE_DISTANCE:
-		break;
-
-	//-----------------------------------------------------
-	case MAP_MODE_ZOOM:
-		break;
-
-	//-----------------------------------------------------
-	case MAP_MODE_PAN:
-		break;
-
-	//-----------------------------------------------------
-	case MAP_MODE_PAN_DOWN:
+	default:
 		break;
 	}
 }
@@ -765,6 +831,8 @@ void CVIEW_Map_Control::On_Mouse_LDClick(wxMouseEvent &event)
 //---------------------------------------------------------
 void CVIEW_Map_Control::On_Mouse_RDown(wxMouseEvent &event)
 {
+	m_pParent->Activate();
+
 	m_Mouse_Down	= m_Mouse_Move	= event.GetPosition();
 
 	_Draw_Inverse(m_Mouse_Down, event.GetPosition());
@@ -785,20 +853,7 @@ void CVIEW_Map_Control::On_Mouse_RDown(wxMouseEvent &event)
 		break;
 
 	//-----------------------------------------------------
-	case MAP_MODE_DISTANCE:
-		_Distance_Reset();
-		break;
-
-	//-----------------------------------------------------
-	case MAP_MODE_ZOOM:
-		break;
-
-	//-----------------------------------------------------
-	case MAP_MODE_PAN:
-		break;
-
-	//-----------------------------------------------------
-	case MAP_MODE_PAN_DOWN:
+	default:
 		break;
 	}
 }
@@ -818,34 +873,51 @@ void CVIEW_Map_Control::On_Mouse_RUp(wxMouseEvent &event)
 		{
 			g_pModule->Execute(_Get_World(event.GetPosition()), MODULE_INTERACTIVE_RUP, GET_KEYS(event));
 		}
-		else if( m_pMap->Find_Layer(Get_Active_Layer()) )
+		else if( m_pMap->Find_Layer(Get_Active_Layer()) && !Get_Active_Layer()->Edit_On_Mouse_Up(_Get_World(event.GetPosition()), _Get_World(1.0), GET_KEYS(event)|MODULE_INTERACTIVE_KEY_RIGHT) )
 		{
-			if(	!Get_Active_Layer()->Edit_On_Mouse_Up(_Get_World(event.GetPosition()), _Get_World(1.0), GET_KEYS(event)|MODULE_INTERACTIVE_KEY_RIGHT)
-			&&	(pMenu = Get_Active_Layer()->Edit_Get_Menu()) != NULL )
-			{
-				PopupMenu(pMenu, event.GetPosition());
-
-				delete(pMenu);
-			}
+			pMenu	= Get_Active_Layer()->Edit_Get_Menu();
+		}
+		else
+		{
+			pMenu	= m_pParent->_Create_Menu();
 		}
 		break;
 
 	//-----------------------------------------------------
 	case MAP_MODE_DISTANCE:
+		if( event.ControlDown() )	// context menu
+		{
+			pMenu	= m_pParent->_Create_Menu();
+		}
+		else	// reset
+		{
+			_Distance_Reset();
+		}
 		break;
 
-	//-----------------------------------------------------
 	case MAP_MODE_ZOOM:
-		_Zoom(_Get_World(event.GetPosition()), false);
+		if( event.ControlDown() )	// context menu
+		{
+			pMenu	= m_pParent->_Create_Menu();
+		}
+		else	// zoom out
+		{
+			_Zoom(_Get_World(event.GetPosition()), false);
+		}
 		break;
 
 	//-----------------------------------------------------
-	case MAP_MODE_PAN:
+	default:
+		pMenu	= m_pParent->_Create_Menu();
 		break;
+	}
 
 	//-----------------------------------------------------
-	case MAP_MODE_PAN_DOWN:
-		break;
+	if( pMenu != NULL )
+	{
+		PopupMenu(pMenu, event.GetPosition());
+
+		delete(pMenu);
 	}
 }
 
