@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: wksp_base_control.cpp 2231 2014-09-15 14:47:01Z oconrad $
+ * Version $Id: wksp_base_control.cpp 2419 2015-02-19 21:00:10Z oconrad $
  *********************************************************/
 	
 ///////////////////////////////////////////////////////////
@@ -83,6 +83,7 @@
 
 #include "wksp_module_control.h"
 #include "wksp_module_manager.h"
+#include "wksp_module.h"
 
 #include "wksp_data_manager.h"
 #include "wksp_data_layers.h"
@@ -91,6 +92,7 @@
 #include "wksp_map_manager.h"
 #include "wksp_map.h"
 #include "wksp_map_buttons.h"
+#include "wksp_layer.h"
 
 
 ///////////////////////////////////////////////////////////
@@ -284,6 +286,11 @@ bool CWKSP_Base_Control::_Del_Item(CWKSP_Base_Item *pItem, bool bSilent)
 		return( false );
 	}
 
+	if( pItem->Get_Type() == WKSP_ITEM_Module && ((CWKSP_Module *)pItem)->Get_Module()->Get_Type() != MODULE_TYPE_Chain )
+	{	// tool libraries can be unloaded, tools not! (except of tool chain tools)
+		return( false );
+	}
+
 	if( pItem->Get_Type() == WKSP_ITEM_Table &&	((CWKSP_Table *)pItem)->Get_Object()->Get_ObjectType() != DATAOBJECT_TYPE_Table )
 	{
 		return( false );
@@ -301,14 +308,10 @@ bool CWKSP_Base_Control::_Del_Item(CWKSP_Base_Item *pItem, bool bSilent)
 		{
 			Freeze();
 
-			if( g_pData_Buttons )
+			if( m_pManager == g_pData || m_pManager == g_pMaps )
 			{
-				g_pData_Buttons->Freeze();
-			}
-
-			if( g_pMap_Buttons )
-			{
-				g_pMap_Buttons->Freeze();
+				if( g_pData_Buttons )	{	g_pData_Buttons->Freeze();	}
+				if( g_pMap_Buttons  )	{	g_pMap_Buttons ->Freeze();	}
 			}
 
 			//---------------------------------------------
@@ -316,7 +319,7 @@ bool CWKSP_Base_Control::_Del_Item(CWKSP_Base_Item *pItem, bool bSilent)
 			AppendItem		(m_pManager->GetId(), _TL("<no items>"), 0, 0, NULL);
 			Expand			(m_pManager->GetId());
 
-			if( g_pModule_Ctrl && m_pManager->Get_Type() == WKSP_ITEM_Module_Manager )
+			if( g_pModules == m_pManager )
 			{
 				g_pModules->Update();
 			}
@@ -324,16 +327,10 @@ bool CWKSP_Base_Control::_Del_Item(CWKSP_Base_Item *pItem, bool bSilent)
 			//---------------------------------------------
 			Thaw();
 
-			if( g_pData_Buttons )
+			if( m_pManager == g_pData || m_pManager == g_pMaps )
 			{
-				g_pData_Buttons->Thaw();
-				g_pData_Buttons->Update_Buttons();
-			}
-
-			if( g_pMap_Buttons )
-			{
-				g_pMap_Buttons->Thaw();
-				g_pMap_Buttons->Update_Buttons();
+				if( g_pData_Buttons )	{	g_pData_Buttons->Thaw();	g_pData_Buttons->Update_Buttons();	}
+				if( g_pMap_Buttons  )	{	g_pMap_Buttons ->Thaw();	g_pMap_Buttons ->Update_Buttons();	}
 			}
 
 			return( true );
