@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: wksp_data_control.cpp 911 2011-02-14 16:38:15Z reklov_w $
+ * Version $Id: wksp_data_control.cpp 1921 2014-01-09 10:24:11Z oconrad $
  *********************************************************/
 	
 ///////////////////////////////////////////////////////////
@@ -35,7 +35,7 @@
 // You should have received a copy of the GNU General    //
 // Public License along with this program; if not,       //
 // write to the Free Software Foundation, Inc.,          //
-// 59 Temple Place - Suite 330, Boston, MA 02111-1307,   //
+// 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, //
 // USA.                                                  //
 //                                                       //
 //-------------------------------------------------------//
@@ -219,7 +219,7 @@ inline int CWKSP_Data_Control::_Get_Image_ID(CWKSP_Base_Item *pItem)
 		case WKSP_ITEM_Shapes:
 			switch( pItem->Get_Type() == WKSP_ITEM_Shapes
 				?	((CWKSP_Shapes *)pItem)->Get_Shapes()->Get_Type()
-				:	((CWKSP_Shapes_Type *)pItem)->Get_Shapes_Type() )
+				:	((CWKSP_Shapes_Type *)pItem)->Get_Shape_Type() )
 			{
 			default:
 			case SHAPE_TYPE_Point:			return( IMG_SHAPES_POINT );
