@@ -66,6 +66,7 @@
 
 #include "active.h"
 #include "active_parameters.h"
+#include "active_attributes.h"
 
 #include "wksp_base_control.h"
 
@@ -81,6 +82,9 @@
 #include "wksp_data_item.h"
 
 #include "view_histogram.h"
+#include "view_scatterplot.h"
+
+#include "data_source_pgsql.h"
 
 
 ///////////////////////////////////////////////////////////
@@ -100,6 +104,12 @@ CWKSP_Data_Item::CWKSP_Data_Item(CSG_Data_Object *pObject)
 //---------------------------------------------------------
 CWKSP_Data_Item::~CWKSP_Data_Item(void)
 {
+	for(int i=m_Views.GetCount()-1; i>=0; i--)
+	{
+		((CVIEW_Base *)m_Views[i])->Do_Destroy();
+	}
+
+	//-----------------------------------------------------
 	if( m_pObject )
 	{
 		CSG_Data_Object	*pObject	= m_pObject;	m_pObject	= NULL;
@@ -127,21 +137,24 @@ bool CWKSP_Data_Item::On_Command(int Cmd_ID)
 	default:
 		return( CWKSP_Base_Item::On_Command(Cmd_ID) );
 
-	case ID_CMD_TABLES_SAVE:
-	case ID_CMD_TIN_SAVE:
-	case ID_CMD_POINTCLOUD_SAVE:
-	case ID_CMD_SHAPES_SAVE:
-	case ID_CMD_GRIDS_SAVE:
+	case ID_CMD_DATA_SAVE:
 		Save(m_pObject->Get_File_Name());
 		break;
 
-	case ID_CMD_TABLES_SAVEAS:
-	case ID_CMD_TIN_SAVEAS:
-	case ID_CMD_POINTCLOUD_SAVEAS:
-	case ID_CMD_SHAPES_SAVEAS:
-	case ID_CMD_GRIDS_SAVEAS:
+	case ID_CMD_DATA_SAVEAS:
 		Save();
 		break;
+
+	case ID_CMD_DATA_SAVETODB:
+		switch( Get_Type() )
+		{
+		case WKSP_ITEM_Table :	PGSQL_Save_Table ((CSG_Table  *)m_pObject);	break;
+		case WKSP_ITEM_Shapes:	PGSQL_Save_Shapes((CSG_Shapes *)m_pObject);	break;
+		case WKSP_ITEM_Grid  :	PGSQL_Save_Grid  ((CSG_Grid   *)m_pObject);	break;
+
+		default:	break;
+		}
+		break;
 	}
 
 	return( true );
@@ -155,13 +168,13 @@ bool CWKSP_Data_Item::On_Command_UI(wxUpdateUIEvent &event)
 	default:
 		return( CWKSP_Base_Item::On_Command_UI(event) );
 
-	case ID_CMD_TABLES_SAVE:
-	case ID_CMD_TIN_SAVE:
-	case ID_CMD_POINTCLOUD_SAVE:
-	case ID_CMD_SHAPES_SAVE:
-	case ID_CMD_GRIDS_SAVE:
+	case ID_CMD_DATA_SAVE:
 		event.Enable(m_pObject->is_Modified() && m_pObject->Get_File_Name() && *(m_pObject->Get_File_Name()));
 		break;
+
+	case ID_CMD_DATA_SAVETODB:
+		event.Enable(PGSQL_has_Connections());
+		break;
 	}
 
 	return( true );
@@ -231,30 +244,29 @@ void CWKSP_Data_Item::On_Create_Parameters(void)
 //---------------------------------------------------------
 bool CWKSP_Data_Item::Save(void)
 {
-	bool		bResult;
+	int	idDlg;
+
+	switch( Get_Type() )
+	{
+	case WKSP_ITEM_Table     :	idDlg	= ID_DLG_TABLES_SAVE    ;	break;
+	case WKSP_ITEM_TIN       :
+	case WKSP_ITEM_Shapes    :	idDlg	= ID_DLG_SHAPES_SAVE    ;	break;
+	case WKSP_ITEM_PointCloud:	idDlg	= ID_DLG_POINTCLOUD_SAVE;	break;
+	case WKSP_ITEM_Grid      :	idDlg	= ID_DLG_GRIDS_SAVE     ;	break;
+
+	default:	return( false );
+	}
 
 	wxString	FileName	= m_pObject->Get_File_Name() && *m_pObject->Get_File_Name()
 		? m_pObject->Get_File_Name()
 		: m_pObject->Get_Name();
 
-	switch( Get_Type() )
+	if( DLG_Save(FileName, idDlg) && m_pObject->Save(&FileName) )
 	{
-	default:					bResult	= false;										break;
-	case WKSP_ITEM_Table:		bResult	= DLG_Save(FileName, ID_DLG_TABLES_SAVE    );	break;
-	case WKSP_ITEM_TIN:
-	case WKSP_ITEM_Shapes:		bResult	= DLG_Save(FileName, ID_DLG_SHAPES_SAVE    );	break;
-	case WKSP_ITEM_PointCloud:	bResult	= DLG_Save(FileName, ID_DLG_POINTCLOUD_SAVE);	break;
-	case WKSP_ITEM_Grid:		bResult	= DLG_Save(FileName, ID_DLG_GRIDS_SAVE     );	break;
+		return( true );
 	}
 
-	if( bResult )
-	{
-		bResult	= m_pObject->Save(&FileName);
-
-		PROCESS_Set_Okay();
-	}
-
-	return( bResult );
+	return( false );
 }
 
 //---------------------------------------------------------
@@ -287,19 +299,17 @@ bool CWKSP_Data_Item::Save(const wxString &File_Name)
 //---------------------------------------------------------
 void CWKSP_Data_Item::Parameters_Changed(void)
 {
-	static bool	bUpdates	= false;
-
-	if( !bUpdates )
+	if( !m_bUpdating )
 	{
-		bUpdates	= true;
+		m_bUpdating	= true;
 
 		On_Parameters_Changed();
 
 		CWKSP_Base_Item::Parameters_Changed();
 
-		Update_Views(true);
+		m_bUpdating	= false;
 
-		bUpdates	= false;
+		Update_Views(true);
 	}
 }
 
@@ -328,13 +338,11 @@ bool CWKSP_Data_Item::DataObject_Changed(void)
 }
 
 //---------------------------------------------------------
-#include "wksp_grid.h"
-
 bool CWKSP_Data_Item::DataObject_Changed(CSG_Parameters *pParameters)
 {
-	if( !pParameters && Get_Type() == WKSP_ITEM_Grid )
+	if( !pParameters )
 	{
-		return( ((CWKSP_Grid *)this)->Fit_Color_Range() );
+		Fit_Colors();
 	}
 
 	m_Parameters.Assign_Values(pParameters);
@@ -363,6 +371,47 @@ void CWKSP_Data_Item::On_DataObject_Changed(void)
 //														 //
 ///////////////////////////////////////////////////////////
 
+//---------------------------------------------------------
+bool CWKSP_Data_Item::Add_ScatterPlot(void)
+{
+	new CVIEW_ScatterPlot(this);
+
+	return( true );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CWKSP_Data_Item::View_Opened(wxMDIChildFrame *pView)
+{
+	if( m_Views.Index(pView) == wxNOT_FOUND )	// only add once
+	{
+		m_Views.Add(pView);
+
+		return( true );
+	}
+
+	return( false );
+}
+
+//---------------------------------------------------------
+bool CWKSP_Data_Item::View_Closes(wxMDIChildFrame *pView)
+{
+	if( m_Views.Index(pView) != wxNOT_FOUND )
+	{
+		m_Views.Remove(pView);
+
+		return( true );
+	}
+
+	return( false );
+}
+
 //---------------------------------------------------------
 bool CWKSP_Data_Item::Update_Views(bool bAll)
 {
@@ -372,6 +421,19 @@ bool CWKSP_Data_Item::Update_Views(bool bAll)
 
 		On_Update_Views(bAll);
 
+		if( bAll )
+		{
+			for(size_t i=0; i<m_Views.Count(); i++)
+			{
+				((CVIEW_Base *)m_Views[i])->Do_Update();
+			}
+		}
+
+		if( g_pACTIVE->Get_Active_Data_Item() == this )
+		{
+			g_pACTIVE->Get_Attributes()->Set_Attributes();
+		}
+
 		m_bUpdating	= false;
 
 		return( true );
