@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: wksp_data_manager.cpp 2145 2014-06-06 14:02:05Z oconrad $
+ * Version $Id: wksp_data_manager.cpp 2723 2015-12-11 10:18:03Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -135,15 +135,44 @@ CWKSP_Data_Manager::CWKSP_Data_Manager(void)
 	pNode	= m_Parameters.Add_Node(NULL, "NODE_GENERAL", _TL("General"), _TL(""));
 
 	m_Parameters.Add_Choice(
-		pNode	, "PROJECT_START"			, _TL("Start Project"),
+		pNode	, "PROJECT_START"			, _TL("Startup Project"),
 		_TL(""),
 		CSG_String::Format(SG_T("%s|%s|%s|"),
 			_TL("empty"),
-			_TL("last opened"),
-			_TL("automatically save and load")
+			_TL("last state"),
+			_TL("always ask what to do")
 		), 2
 	);
 
+	pNode_1	= m_Parameters.Add_Choice(
+		pNode	, "PROJECT_MAP_ARRANGE"		, _TL("Map Window Arrangement"),
+		_TL("initial map window arrangement after a project is loaded"),
+		CSG_String::Format(SG_T("%s|%s|%s|"),
+			_TL("Cascade"),
+			_TL("Tile Horizontally"),
+			_TL("Tile Vertically")
+		), 2
+	);
+
+#ifndef _SAGA_MSW
+	pNode_1->Set_Enabled(false);
+#endif
+
+	pNode_1	= m_Parameters.Add_Choice(
+		pNode	, "PROJECT_DB_REOPEN"		, _TL("Reopen Database Connections"),
+		_TL("Reopen PostgreSQL database connections. Warning: if set to true account information including unencrypted paswords for automatic connection will be stored."),
+		CSG_String::Format("%s|%s|",
+			_TL("no"),
+			_TL("yes")
+		), 0
+	);
+
+	m_Parameters.Add_Value(
+		pNode	, "SHOW_FILE_SOURCES"		, _TL("Show Data File Sources"),
+		_TL("Show data sources tab for file system. Disabling might speed up start-up. Changes take effect after restart."),
+		PARAMETER_TYPE_Bool, true
+	);
+
 	m_Parameters.Add_Value(
 		pNode	, "NUMBERING"				, _TL("Numbering of Data Sets"),
 		_TL("Leading zeros for data set numbering. Set to -1 for not using numbers at all."),
@@ -261,6 +290,8 @@ CWKSP_Data_Manager::CWKSP_Data_Manager(void)
 //---------------------------------------------------------
 CWKSP_Data_Manager::~CWKSP_Data_Manager(void)
 {
+	CONFIG_Write("/DATA", &m_Parameters);
+
 	delete(m_pProject);
 	delete(m_pMenu_Files);
 
@@ -275,36 +306,15 @@ CWKSP_Data_Manager::~CWKSP_Data_Manager(void)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CWKSP_Data_Manager::Initialise(void)
+wxFileName Get_SAGA_GUI_CFG(void)
 {
-	wxString	FileName;
-
-	if( m_pProject->Has_File_Name() )
-	{
-		return( m_pProject->Load(false) );
-	}
-	else
-	{
-		return( CONFIG_Read("/DATA", "PROJECT_FILE", FileName) && wxFileExists(FileName) && m_pProject->Load(FileName, false, false) );
-	}
-
-	return( false );
-}
-
-//---------------------------------------------------------
-bool CWKSP_Data_Manager::Finalise(void)
-{
-	//-----------------------------------------------------
-	CONFIG_Write("/DATA", &m_Parameters);
-
-	//-----------------------------------------------------
 #ifdef _SAGA_LINUX
 //	wxFileName	fProject(wxString(getenv( "HOME"), wxConvFile ), wxT("saga_gui"), wxT("cfg"));
 	CSG_String	sHome(getenv("HOME"));
 	wxFileName	fProject(sHome.c_str(), "saga_gui", "cfg");
 #else
 	wxFileName	fProject(g_pSAGA->Get_App_Path(), "saga_gui", "cfg");
-	wxString(getenv("HOME"));
+
 	if(	( fProject.FileExists() && (!fProject.IsFileReadable() || !fProject.IsFileWritable()))
 	||	(!fProject.FileExists() && (!fProject.IsDirReadable () || !fProject.IsDirWritable ())) )
 	{
@@ -314,33 +324,82 @@ bool CWKSP_Data_Manager::Finalise(void)
 
 	fProject.Normalize();
 
+	return( fProject );
+}
+
+//---------------------------------------------------------
+bool CWKSP_Data_Manager::Initialise(void)
+{
+	//-----------------------------------------------------
+	if( m_pProject->Has_File_Name() )	{	return( m_pProject->Load(false) );	}
 	//-----------------------------------------------------
-	if( Get_Count() == 0 || m_Parameters("PROJECT_START")->asInt() == 0 )
-	{	// empty
-		if( fProject.FileExists() )
-		{
-			wxRemoveFile(fProject.GetFullPath());
-		}
 
-		CONFIG_Write("/DATA", "PROJECT_FILE", "");
-	}
-	else if( m_Parameters("PROJECT_START")->asInt() == 1 )
-	{	// last opened
-        if( fProject.FileExists() )
-		{
-            wxRemoveFile(fProject.GetFullPath());
-		}
+	//-----------------------------------------------------
+	wxFileName	fProject, fLastState	= Get_SAGA_GUI_CFG();
 
-		CONFIG_Write(wxT("/DATA"), wxT("PROJECT_FILE"), m_pProject->Get_File_Name());
+	switch( m_Parameters("PROJECT_START")->asInt() )
+	{
+	case 0:	// empty
+		return( true );
+
+	case 1:	// last state
+		return( m_pProject->Load(fLastState.GetFullPath(), false, false) );
+
+	case 2:	// always ask what to do
+		{
+			wxArrayString	Projects;
+
+			Projects.Add(wxString::Format("[%s]", _TL("empty")));
+
+			if( fLastState.FileExists() )
+			{
+				Projects.Add(wxString::Format("[%s]", _TL("last state")));
+			}
+
+			m_pMenu_Files->Recent_Get(DATAOBJECT_TYPE_Undefined, Projects, true);
+
+			wxSingleChoiceDialog	dlg(MDI_Get_Top_Window(), _TL("Startup Project"), _TL("Select Startup Project"), Projects);
+
+			if( Projects.Count() <= 1 || dlg.ShowModal() != wxID_OK || dlg.GetSelection() == 0 )
+			{	// empty
+				return( true );
+			}
+
+			if( fLastState.FileExists() && dlg.GetSelection() == 1 )
+			{	// last state
+				return( m_pProject->Load(fLastState.GetFullPath(), false, false) );
+			}
+			else
+			{	// recently opened project
+				return( m_pProject->Load(dlg.GetStringSelection(), false, false) );
+			}
+		}
 	}
-	else
-	{	// automatically save and load
+
+	//-----------------------------------------------------
+	//wxString	FileName	= Get_FilePath_Absolute(g_pSAGA->Get_App_Path(), fProject.GetFullPath());
+
+	return( false );
+}
+
+//---------------------------------------------------------
+bool CWKSP_Data_Manager::Finalise(void)
+{
+	wxFileName	fProject	= Get_SAGA_GUI_CFG();
+
+	if( Get_Count() > 0 )
+	{	// last state
 		m_pProject->Save(fProject.GetFullPath(), false);
 
-		CONFIG_Write(wxT("/DATA"), wxT("PROJECT_FILE"), fProject.GetFullPath());
+		if( fProject.GetPath().Find(g_pSAGA->Get_App_Path()) == 0 )
+		{
+			fProject.MakeRelativeTo(g_pSAGA->Get_App_Path());
+		}
+	}
+	else if( fProject.FileExists() )
+	{
+		wxRemoveFile(fProject.GetFullPath());
 	}
-
-	m_pProject->Clr_File_Name();
 
 	return( true );
 }
@@ -413,18 +472,16 @@ wxMenu * CWKSP_Data_Manager::Get_Menu(void)
 
 	pMenu	= new wxMenu(_TL("Data"));
 
-//	CMD_Menu_Add_Item(pMenu, false, ID_CMD_WKSP_ITEM_CLOSE);
-
-//	pMenu->AppendSeparator();
-
-	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_PROJECT_NEW);
 	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_PROJECT_OPEN);
 //	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_PROJECT_OPEN_ADD);
+	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_PROJECT_CLOSE);
 
 	if( Get_Count() > 0 )
 	{
 		CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_PROJECT_SAVE);
 		CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_PROJECT_SAVE_AS);
+		CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_PROJECT_COPY);
+	//	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_PROJECT_COPY_DB);
 		pMenu->AppendSeparator();
 		CMD_Menu_Add_Item(pMenu, false, ID_CMD_WKSP_ITEM_SEARCH);
 	}
@@ -480,11 +537,14 @@ bool CWKSP_Data_Manager::On_Command(int Cmd_ID)
 		return( CWKSP_Base_Manager::On_Command(Cmd_ID) );
 
 	//-----------------------------------------------------
-	case ID_CMD_DATA_PROJECT_NEW:		Close(false);						break;
 	case ID_CMD_DATA_PROJECT_OPEN:		m_pProject->Load(false);			break;
 	case ID_CMD_DATA_PROJECT_OPEN_ADD:	m_pProject->Load(true);				break;
+	case ID_CMD_DATA_PROJECT_BROWSE:	Open_Browser();						break;
+	case ID_CMD_DATA_PROJECT_CLOSE:		Close(false);						break;
 	case ID_CMD_DATA_PROJECT_SAVE:		m_pProject->Save(true);				break;
 	case ID_CMD_DATA_PROJECT_SAVE_AS:	m_pProject->Save();					break;
+	case ID_CMD_DATA_PROJECT_COPY:		m_pProject->Copy();					break;
+	case ID_CMD_DATA_PROJECT_COPY_DB:	m_pProject->CopyToDB();				break;
 
 	//-----------------------------------------------------
 	case ID_CMD_TABLES_OPEN:			Open(DATAOBJECT_TYPE_Table);		break;
@@ -514,7 +574,7 @@ bool CWKSP_Data_Manager::On_Command_UI(wxUpdateUIEvent &event)
 	default:
 		return( CWKSP_Base_Manager::On_Command_UI(event) );
 
-	case ID_CMD_DATA_PROJECT_NEW:
+	case ID_CMD_DATA_PROJECT_CLOSE:
 		event.Enable(Get_Count() > 0 && g_pModule == NULL);
 		break;
 
@@ -680,6 +740,83 @@ bool CWKSP_Data_Manager::Open_CMD(int Cmd_ID)
 }
 
 
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CWKSP_Data_Manager::Open_Browser(void)
+{
+	//-----------------------------------------------------
+	wxString	Directory	= wxDirSelector(_TL("Search for Projects"));
+
+	if( Directory.IsEmpty() )
+	{
+		return( false );
+	}
+
+	//-----------------------------------------------------
+	wxArrayString	Projects;
+
+	Open_Browser(Projects, Directory);
+
+	if( Projects.Count() == 0 )
+	{
+		wxMessageBox(_TL("No projects in directory"), _TL("Search for Projects"), wxOK|wxICON_EXCLAMATION);
+
+		return( false );
+	}
+
+	//-----------------------------------------------------
+	wxSingleChoiceDialog	dlg(MDI_Get_Top_Window(), _TL("Open Project"), _TL("Search for Projects"), Projects);
+
+	return( dlg.ShowModal() == wxID_OK && Open(dlg.GetStringSelection()) );
+}
+
+//---------------------------------------------------------
+#include <wx/dir.h>
+
+//---------------------------------------------------------
+bool CWKSP_Data_Manager::Open_Browser(wxArrayString &Projects, const wxString &Directory)
+{
+	wxDir	Dir;
+
+	if( Dir.Open(Directory) )
+	{
+		wxString	FileName;
+
+		if( Dir.GetFirst(&FileName, wxEmptyString, wxDIR_FILES) )
+		{
+			do
+			{
+				wxFileName	fn(Dir.GetName(), FileName);
+
+				if( !fn.GetExt().CmpNoCase("sprj") )
+				{
+					Projects.Add(fn.GetFullPath());
+				}
+			}
+			while( Dir.GetNext(&FileName) );
+		}
+
+		if( Dir.GetFirst(&FileName, wxEmptyString, wxDIR_DIRS) )
+		{
+			do
+			{
+				wxFileName	fn(Dir.GetName(), FileName);
+
+				Open_Browser(Projects, fn.GetFullPath());
+			}
+			while( Dir.GetNext(&FileName) );
+		}
+	}
+
+	return( Projects.Count() > 0 );
+}
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 //														 //
@@ -929,14 +1066,14 @@ bool CWKSP_Data_Manager::Close(bool bSilent)
 {
 	if( Get_Count() == 0 )
 	{
-		Finalise();
+		m_pProject->Clr_File_Name();
 
 		return( true );
 	}
 
 	if( (bSilent || DLG_Message_Confirm(_TL("Close all data sets"), _TL("Close"))) && Save_Modified(this) )
 	{
-		Finalise();
+		m_pProject->Clr_File_Name();
 
 		g_pACTIVE->Get_Parameters()->Restore_Parameters();
 		g_pMaps->Close(true);
@@ -1105,7 +1242,7 @@ bool CWKSP_Data_Manager::Get_Parameters(CSG_Data_Object *pObject, CSG_Parameters
 {
 	CWKSP_Data_Item	*pItem	= pParameters ? Get(pObject) : NULL;
 
-	return( pItem && pParameters->Assign(pItem->Get_Parameters()) > 0 );
+	return( pItem && pParameters->Assign(pItem->Get_Parameters()) );
 }
 
 //---------------------------------------------------------
@@ -1113,7 +1250,7 @@ bool CWKSP_Data_Manager::Set_Parameters(CSG_Data_Object *pObject, CSG_Parameters
 {
 	CWKSP_Data_Item	*pItem	= pParameters ? Get(pObject) : NULL;
 
-	if( pItem && pItem->Get_Parameters()->Assign_Values(pParameters) > 0 )
+	if( pItem && pItem->Get_Parameters()->Assign_Values(pParameters) )
 	{
 		pItem->Parameters_Changed();
 
