@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: wksp_data_manager.h 1955 2014-01-23 13:49:11Z oconrad $
+ * Version $Id: wksp_data_manager.h 2501 2015-06-12 10:02:42Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -123,6 +123,8 @@ public:
 	bool							Open				(const wxString &File);
 	bool							Open				(int DataType);
 	bool							Open_CMD			(int Cmd_ID);
+	bool							Open_Browser		(void);
+	bool							Open_Browser		(wxArrayString &Projects, const wxString &Directory);
 
 	bool							Save_Modified		(class CWKSP_Base_Item *pItem = NULL, bool bSelections = false);
 	bool							Save_Modified_Sel	(void);
