@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: wksp_grid.h 2003 2014-02-19 17:08:18Z oconrad $
+ * Version $Id: wksp_grid.h 2716 2015-12-04 11:55:42Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -101,12 +101,11 @@ public:
 	virtual wxString			Get_Value				(CSG_Point ptWorld, double Epsilon);
 	virtual double				Get_Value_Range			(void);
 
-	bool						Fit_Color_Range			(void);
 	bool						Fit_Color_Range			(CSG_Rect rWorld);
 
 	virtual bool				asImage					(CSG_Grid *pImage);
 
-	virtual bool				Update					(CWKSP_Layer *pChanged)	{	return( pChanged == this || pChanged == m_pOverlay[0] || pChanged == m_pOverlay[1] );	}
+	virtual bool				Update					(CWKSP_Layer *pChanged);
 
 	virtual wxMenu *			Edit_Get_Menu			(void);
 	virtual TSG_Rect			Edit_Get_Extent			(void);
@@ -125,17 +124,15 @@ protected:
 	virtual void				On_Parameters_Changed	(void);
 	virtual int					On_Parameter_Changed	(CSG_Parameters *pParameters, CSG_Parameter *pParameter, int Flags);
 
-	virtual void				On_Draw					(CWKSP_Map_DC &dc_Map, bool bEdit);
+	virtual bool				Fit_Colors				(void);
+
+	virtual void				On_Draw					(CWKSP_Map_DC &dc_Map, int Flags);
 
 
 private:
 
-	bool						m_bOverlay;
-
 	int							m_xSel, m_ySel;
 
-	CWKSP_Grid					*m_pOverlay[2];
-
 
 	void						_LUT_Create				(void);
 
@@ -145,7 +142,7 @@ private:
 	void						_Save_Image				(void);
 
 	void						_Draw_Grid_Points		(CWKSP_Map_DC &dc_Map, int Interpolation);
-	void						_Draw_Grid_Line			(CWKSP_Map_DC &dc_Map, int Interpolation, bool bByteWise, int yDC, int axDC, int bxDC, int r, int g, int b);
+	void						_Draw_Grid_Line			(CWKSP_Map_DC &dc_Map, int Interpolation, int Mode, CWKSP_Grid *pOverlay[2], int yDC, int axDC, int bxDC);
 	void						_Draw_Grid_Cells		(CWKSP_Map_DC &dc_Map);
 
 	void						_Draw_Values			(CWKSP_Map_DC &dc_Map);
