@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: wksp_layer.cpp 2061 2014-03-20 11:48:01Z oconrad $
+ * Version $Id: wksp_layer.cpp 2700 2015-11-24 17:18:29Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -421,8 +421,8 @@ void CWKSP_Layer::On_Parameters_Changed(void)
 	m_pClassify->Set_Metric(
 		m_Parameters("METRIC_SCALE_MODE")->asInt(),
 		m_Parameters("METRIC_SCALE_LOG")->asDouble(),
-		m_Parameters("METRIC_ZRANGE")->asRange()->Get_LoVal() / (Get_Type() == WKSP_ITEM_Grid ? ((CSG_Grid *)m_pObject)->Get_ZFactor() : 1.0),
-		m_Parameters("METRIC_ZRANGE")->asRange()->Get_HiVal() / (Get_Type() == WKSP_ITEM_Grid ? ((CSG_Grid *)m_pObject)->Get_ZFactor() : 1.0)
+		m_Parameters("METRIC_ZRANGE")->asRange()->Get_LoVal(),
+		m_Parameters("METRIC_ZRANGE")->asRange()->Get_HiVal()
 	);
 
 	m_pLegend->Set_Orientation(m_Parameters("LEGEND_STYLE")->asInt() == LEGEND_VERTICAL ? LEGEND_VERTICAL : LEGEND_HORIZONTAL);
@@ -460,7 +460,7 @@ bool CWKSP_Layer::_Set_Thumbnail(bool bRefresh)
 		wxRect			r(0, 0, m_Thumbnail.GetWidth(), m_Thumbnail.GetHeight());
 		CWKSP_Map_DC	dc_Map(Get_Extent(), r, 1.0, SG_GET_RGB(255, 255, 255));
 
-		Draw(dc_Map, false);
+		Draw(dc_Map, LAYER_DRAW_FLAG_NOEDITS|LAYER_DRAW_FLAG_NOLABELS|LAYER_DRAW_FLAG_THUMBNAIL);
 
 		dc.SelectObject(m_Thumbnail);
 		dc.SetBackground(*wxWHITE_BRUSH);
@@ -628,9 +628,9 @@ bool CWKSP_Layer::do_Show(CSG_Rect const &rMap)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-void CWKSP_Layer::Draw(CWKSP_Map_DC &dc_Map, bool bEdit)
+void CWKSP_Layer::Draw(CWKSP_Map_DC &dc_Map, int Flags)
 {
-	On_Draw(dc_Map, bEdit);
+	On_Draw(dc_Map, Flags);
 }
 
 
