@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: wksp_layer_classify.cpp 2079 2014-04-02 16:37:16Z oconrad $
+ * Version $Id: wksp_layer_classify.cpp 2431 2015-03-03 17:21:59Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -89,19 +89,14 @@ CWKSP_Layer_Classify::CWKSP_Layer_Classify(void)
 	m_pLUT			= NULL;
 
 	m_HST_Count		= NULL;
+	m_HST_Cumul		= NULL;
 }
 
 //---------------------------------------------------------
 CWKSP_Layer_Classify::~CWKSP_Layer_Classify(void)
 {
-	if( m_HST_Count )
-	{
-		SG_Free(m_HST_Count);
-		m_HST_Count	= NULL;
-
-		SG_Free(m_HST_Cumul);
-		m_HST_Cumul	= NULL;
-	}
+	SG_FREE_SAFE(m_HST_Count);
+	SG_FREE_SAFE(m_HST_Cumul);
 }
 
 
@@ -473,12 +468,17 @@ void CWKSP_Layer_Classify::Metric2EqualElements(void)
 		pGrid	= ((CWKSP_Grid *)m_pLayer)->Get_Grid();
 		dClass	= (double)pGrid->Get_NCells() / (double)(nClasses);
 
+		if( !pGrid->Set_Index() )
+		{
+			return;
+		}
+
 		pGrid->Get_Sorted(0, x, y, false, false);
 		zA		= pGrid->asDouble(x, y);
 
 		for(iClass=0; iClass<nClasses-1; iClass++)
 		{
-			pGrid->Get_Sorted((int)(dClass * (iClass + 1.0)), x, y, false, false);
+			pGrid->Get_Sorted((sLong)(dClass * (iClass + 1.0)), x, y, false, false);
 			zB		= zA;
 			zA		= pGrid->asDouble(x, y);
 
@@ -515,24 +515,17 @@ void CWKSP_Layer_Classify::Metric2EqualElements(void)
 //---------------------------------------------------------
 bool CWKSP_Layer_Classify::Histogram_Update(void)
 {
-	int		i;
-
 	//-----------------------------------------------------
-	if( m_HST_Count )
-	{
-		SG_Free(m_HST_Count);
-		SG_Free(m_HST_Cumul);
-		m_HST_Count	= NULL;
-		m_HST_Cumul	= NULL;
-	}
+	SG_FREE_SAFE(m_HST_Count);
+	SG_FREE_SAFE(m_HST_Cumul);
 
 	//-----------------------------------------------------
 	if( Get_Class_Count() > 0 )
 	{
 		STATUSBAR_Set_Text(_TL("Build Histogram..."));
 
-		m_HST_Count	= (int *)SG_Calloc(Get_Class_Count(), sizeof(int));
-		m_HST_Cumul	= (int *)SG_Calloc(Get_Class_Count(), sizeof(int));
+		m_HST_Count	= (sLong *)SG_Calloc(Get_Class_Count(), sizeof(sLong));
+		m_HST_Cumul	= (sLong *)SG_Calloc(Get_Class_Count(), sizeof(sLong));
 
 		switch( m_pLayer->Get_Type() )
 		{
@@ -555,6 +548,8 @@ bool CWKSP_Layer_Classify::Histogram_Update(void)
 		PROCESS_Set_Okay();
 
 		//-------------------------------------------------
+		int		i;
+
 		for(i=0, m_HST_Maximum=0, m_HST_Total=0; i<Get_Class_Count(); i++)
 		{
 			m_HST_Cumul[i]	= (m_HST_Total += m_HST_Count[i]);
