@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: wksp_layer_classify.h 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: wksp_layer_classify.h 2431 2015-03-03 17:21:59Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -468,9 +468,9 @@ public: ///////////////////////////////////////////////////
 
 protected: ////////////////////////////////////////////////
 
-	int							m_Mode, m_zMode, m_Shade_Mode, m_Count,
-								m_UNI_Color,
-								*m_HST_Count, *m_HST_Cumul, m_HST_Maximum, m_HST_Total;
+	int							m_Mode, m_zMode, m_Shade_Mode, m_Count, m_UNI_Color;
+
+	sLong						*m_HST_Count, *m_HST_Cumul, m_HST_Maximum, m_HST_Total;
 
 	double						m_zMin, m_zRange, m_zLogRange, m_zLogMax;
 
