@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: wksp_map.cpp 2179 2014-07-07 15:24:50Z oconrad $
+ * Version $Id: wksp_map.cpp 2681 2015-11-06 12:31:05Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -455,6 +455,13 @@ void CWKSP_Map::On_Create_Parameters(void)
 		PARAMETER_TYPE_Double, 90, 0, true, 100, true
 	);
 
+	//-----------------------------------------------------
+	pNode_1	= m_Parameters.Add_Value(
+		pNode_0	, "GCS_POSITION"	, _TL("Position as Geographic Coordinates"),
+		_TL("show mouse position in statusbar using geographic coordinates if possible"),
+		PARAMETER_TYPE_Bool, false
+	);
+
 	//-----------------------------------------------------
 	pNode_1	= m_Parameters.Add_Value(
 		pNode_0	, "SCALE_SHOW"		, _TL("Scale Bar"),
@@ -835,6 +842,14 @@ void CWKSP_Map::_Set_Extent(const CSG_Rect &Extent)
 	{
 		View_Refresh(true);
 
+		for(int i=0; i<Get_Count(); i++)
+		{
+			if( Get_Item(i)->Get_Type() == WKSP_ITEM_Map_Layer )
+			{
+				((CWKSP_Map_Layer *)Get_Item(i))->Fit_Colors(Extent);
+			}
+		}
+
 		if( m_Parameters("SYNC_MAPS")->asBool() )
 		{
 			_Synchronise_Extents();
@@ -1226,8 +1241,8 @@ CSG_Point CWKSP_Map::Get_World(wxRect rClient, wxPoint ptClient)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-#define MASK_R	255
-#define MASK_G	0
+#define MASK_R	254
+#define MASK_G	255
 #define MASK_B	255
 
 //---------------------------------------------------------
@@ -1239,7 +1254,7 @@ bool CWKSP_Map::Get_Image(wxImage &Image, CSG_Rect &rWorld)
 		wxMemoryDC	dc;
 
 		dc.SelectObject(BMP);
-		Draw_Map(dc, 1.0, wxRect(0, 0, Image.GetWidth(), Image.GetHeight()), false, SG_GET_RGB(MASK_R, MASK_G, MASK_B));
+		Draw_Map(dc, 1.0, wxRect(0, 0, Image.GetWidth(), Image.GetHeight()), LAYER_DRAW_FLAG_NOEDITS, SG_GET_RGB(MASK_R, MASK_G, MASK_B));
 		dc.SelectObject(wxNullBitmap);
 
 		rWorld	= Get_World(wxRect(0, 0, Image.GetWidth(), Image.GetHeight()));
@@ -1348,7 +1363,7 @@ void CWKSP_Map::SaveAs_Image_Clipboard(int nx, int ny, int frame)
 	dc.SetBackground(*wxWHITE_BRUSH);
 	dc.Clear();
 
-	Draw_Map(dc, 1.0, r, false);
+	Draw_Map(dc, 1.0, r, LAYER_DRAW_FLAG_NOEDITS);
 
 	if( frame > 0 )
 	{
@@ -1368,6 +1383,81 @@ void CWKSP_Map::SaveAs_Image_Clipboard(int nx, int ny, int frame)
 	Set_Buisy_Cursor(false);
 }
 
+//---------------------------------------------------------
+void CWKSP_Map::SaveAs_Image_To_KMZ(int nx, int ny)
+{
+	if( nx < 1 || ny < 1 )
+	{
+		return;
+	}
+
+	//-----------------------------------------------------
+	CSG_Rect		Extent(Get_Extent());
+
+	CSG_Parameters	P(NULL, _TL("Save Map to KMZ"), _TL(""));
+
+	P.Add_FilePath(NULL, "FILE"    , _TL("File"    ), _TL(""), CSG_String::Format("%s|*.kmz|%s|*.*", _TL("KMZ Files"), _TL("All Files")), NULL, true);
+	P.Add_Value   (NULL, "CELLSIZE", _TL("Cellsize"), _TL(""), PARAMETER_TYPE_Double, SG_Get_Rounded_To_SignificantFigures(Extent.Get_XRange() / (double)nx, 2), 0.0, true);
+	P.Add_Value   (NULL, "LOAD"    , _TL("Load"    ), _TL(""), PARAMETER_TYPE_Bool  , true);
+
+	if( !DLG_Parameters(&P) || P("CELLSIZE")->asDouble() <= 0.0 )
+	{
+		return;
+	}
+
+	wxFileName	FileName(P("FILE")->asString());
+
+	if( !FileName.IsOk() )
+	{
+		DLG_Message_Show_Error(_TL("invalid file name"), _TL("Save Map to KMZ"));
+
+		return;
+	}
+
+	//-----------------------------------------------------
+	nx	= Extent.Get_XRange() / P("CELLSIZE")->asDouble();
+	ny	= Extent.Get_YRange() / P("CELLSIZE")->asDouble();
+
+	wxImage		Image(nx, ny);
+
+	if( !Get_Image(Image, Extent) )
+	{
+		return;
+	}
+
+	//-----------------------------------------------------
+	CSG_Grid	Map(SG_DATATYPE_Int, Image.GetWidth(), Image.GetHeight(), Extent.Get_XRange() / (double)Image.GetWidth(), Extent.Get_XMin(), Extent.Get_YMin());
+
+	Map.Set_Name(Get_Name().wx_str());
+	Map.Set_NoData_Value(16711935);
+	Map.Get_Projection().Create(m_Projection);
+
+	for(int y=0, yy=Map.Get_NY()-1; y<Map.Get_NY(); y++, yy--)
+	{
+		for(int x=0; x<Map.Get_NX(); x++)
+		{
+			Map.Set_Value(x, y, SG_GET_RGB(Image.GetRed(x, yy), Image.GetGreen(x, yy), Image.GetBlue(x, yy)));
+		}
+	}
+
+	//-----------------------------------------------------
+	bool	bResult;
+
+	SG_RUN_MODULE(bResult, "io_grid_image", 2,
+			SG_MODULE_PARAMETER_SET("GRID"     , &Map)
+		&&	SG_MODULE_PARAMETER_SET("FILE"     , FileName.GetFullPath().wc_str())
+		&&	SG_MODULE_PARAMETER_SET("COLOURING", 4)	// rgb coded values
+		&&	SG_MODULE_PARAMETER_SET("OUTPUT"   , 2)	// kmz file
+	);
+
+	if( bResult && P("LOAD")->asBool() )
+	{
+		FileName.SetExt("kmz");
+
+		Open_Application(FileName.GetFullPath());
+	}
+}
+
 //---------------------------------------------------------
 void CWKSP_Map::SaveAs_Image_To_Memory(int nx, int ny)
 {
@@ -1437,7 +1527,7 @@ void CWKSP_Map::SaveAs_Image_To_Grid(CSG_Grid &Grid, int Size)
 	if( Get_Image(Image, Extent) )
 	{
 		Grid.Create(SG_DATATYPE_Int, Image.GetWidth(), Image.GetHeight(), Extent.Get_XRange() / (double)Image.GetWidth(), Extent.Get_XMin(), Extent.Get_YMin());
-		Grid.Set_NoData_Value(16711935);
+		Grid.Set_NoData_Value(SG_GET_RGB(MASK_R, MASK_G, MASK_B));
 
 		for(int y=0, yy=Grid.Get_NY()-1; y<Grid.Get_NY(); y++, yy--)
 		{
@@ -1498,7 +1588,7 @@ void CWKSP_Map::_Img_Save(wxString file, int type)
 	dc.SetBackground(*wxWHITE_BRUSH);
 	dc.Clear();
 
-	Draw_Map(dc, 1.0, r, false);
+	Draw_Map(dc, 1.0, r, LAYER_DRAW_FLAG_NOEDITS);
 	Draw_Frame(dc, r, Frame);
 
 	dc.SelectObject(wxNullBitmap);
@@ -1625,7 +1715,7 @@ bool CWKSP_Map::_Set_Thumbnail(void)
 		dc.SetBackground(*wxWHITE_BRUSH);
 		dc.Clear();
 
-		Draw_Map(dc, Get_Extent(), 1.0, r, false);
+		Draw_Map(dc, Get_Extent(), 1.0, r, LAYER_DRAW_FLAG_NOEDITS|LAYER_DRAW_FLAG_NOLABELS);
 
 		dc.SelectObject(wxNullBitmap);
 
@@ -1643,16 +1733,18 @@ bool CWKSP_Map::_Set_Thumbnail(void)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-void CWKSP_Map::Draw_Map(wxDC &dc, double Zoom, const wxRect &rClient, bool bEdit, int Background)
+void CWKSP_Map::Draw_Map(wxDC &dc, double Zoom, const wxRect &rClient, int Flags, int Background)
 {
-	Draw_Map(dc, Get_World(rClient), Zoom, rClient, bEdit, Background);
+	Draw_Map(dc, Get_World(rClient), Zoom, rClient, Flags, Background);
 }
 
 //---------------------------------------------------------
-void CWKSP_Map::Draw_Map(wxDC &dc, const CSG_Rect &rWorld, double Zoom, const wxRect &rClient, bool bEdit, int Background)
+void CWKSP_Map::Draw_Map(wxDC &dc, const CSG_Rect &rWorld, double Zoom, const wxRect &rClient, int Flags, int Background)
 {
 	CWKSP_Map_DC	dc_Map(rWorld, rClient, Zoom, Background);
 
+	int	Flag_Labels	= !(Flags & LAYER_DRAW_FLAG_NOLABELS) ? 0 : LAYER_DRAW_FLAG_NOLABELS;
+
 	//-----------------------------------------------------
 	for(int i=Get_Count()-1; i>=0; i--)
 	{
@@ -1664,7 +1756,7 @@ void CWKSP_Map::Draw_Map(wxDC &dc, const CSG_Rect &rWorld, double Zoom, const wx
 
 				if( pLayer->do_Show() && pLayer->Get_Layer()->do_Show(Get_Extent()) )
 				{
-					pLayer->Get_Layer()->Draw(dc_Map, bEdit && pLayer->Get_Layer() == Get_Active_Layer());
+					pLayer->Get_Layer()->Draw(dc_Map, !(Flags & LAYER_DRAW_FLAG_NOEDITS) && pLayer->Get_Layer() == Get_Active_Layer() ? Flags : Flag_Labels);
 				}
 			}
 			break;
@@ -1710,14 +1802,25 @@ void CWKSP_Map::Draw_Frame(wxDC &dc, const CSG_Rect &rWorld, wxRect rMap, int Wi
 	r		= wxRect(rMap.GetLeft()    , rFrame.GetTop()    , rMap.GetWidth(), Width);
 	Draw_Scale(dc, r, rWorld.Get_XMin(), rWorld.Get_XMax()  , true , true , false);
 
-	r		= wxRect(rMap.GetLeft()    , rMap.GetBottom()   , rMap.GetWidth(), Width);
-	Draw_Scale(dc, r, 0.0              , rWorld.Get_XRange(), true , true , true);
-
 	r		= wxRect(rFrame.GetLeft()  , rMap.GetTop()      , Width, rMap.GetHeight());
 	Draw_Scale(dc, r, rWorld.Get_YMin(), rWorld.Get_YMax()  , false, false, false);
 
-	r		= wxRect(rMap.GetRight()   , rMap.GetTop()      , Width, rMap.GetHeight());
-	Draw_Scale(dc, r, 0.0              , rWorld.Get_YRange(), false, false, true);
+	if( is_ScaleBar() )
+	{
+		r		= wxRect(rMap.GetLeft()    , rMap.GetBottom()   , rMap.GetWidth(), Width);
+		Draw_Scale(dc, r, rWorld.Get_XMin(), rWorld.Get_XMax()  , true , true , false);
+
+		r		= wxRect(rMap.GetRight()   , rMap.GetTop()      , Width, rMap.GetHeight());
+		Draw_Scale(dc, r, rWorld.Get_YMin(), rWorld.Get_YMax()  , false, false, false);
+	}
+	else
+	{
+		r		= wxRect(rMap.GetLeft()    , rMap.GetBottom()   , rMap.GetWidth(), Width);
+		Draw_Scale(dc, r, 0.0              , rWorld.Get_XRange(), true , true , true);
+
+		r		= wxRect(rMap.GetRight()   , rMap.GetTop()      , Width, rMap.GetHeight());
+		Draw_Scale(dc, r, 0.0              , rWorld.Get_YRange(), false, false, true);
+	}
 
 	Draw_Edge(dc, EDGE_STYLE_SIMPLE, rFrame.GetLeft(), rFrame.GetTop(), rFrame.GetRight(), rFrame.GetBottom());
 }
