@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: wksp_module.h 2301 2014-10-21 08:52:29Z oconrad $
+ * Version $Id: wksp_module.h 2527 2015-06-30 15:38:35Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -105,7 +105,6 @@ public:
 
 	void							Set_Menu_ID			(int Menu_ID);
 	int								Get_Menu_ID			(void)			{	return( m_Menu_ID );	}
-	wxString						Get_Menu_Path		(void);
 
 	wxString						Get_File_Name		(void);
 
@@ -124,11 +123,16 @@ private:
 
 
 	void							_Save_to_Clipboard		(void);
-	void							_Save_to_Clipboard		(CSG_MetaData &Tool, CSG_Parameters *pParameters);
-
 	void							_Save_to_Script			(void);
-	void							_Save_to_Script_CMD		(CSG_String &Command, CSG_Parameters *pParameters);
-	void							_Save_to_Script_Python	(CSG_String &Command, CSG_Parameters *pParameters);
+
+	CSG_String						_Get_XML				(bool bHeader);
+	void							_Get_XML				(CSG_MetaData &Tool, CSG_Parameters *pParameters);
+
+	CSG_String						_Get_CMD				(bool bHeader, int Type = -1);
+	void							_Get_CMD				(CSG_String &Command, CSG_Parameters *pParameters);
+
+	CSG_String						_Get_Python				(bool bHeader);
+	void							_Get_Python				(CSG_String &Command, CSG_Parameters *pParameters);
 
 };
 
