@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: wksp_module_library.h 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: wksp_module_library.h 2254 2014-09-25 16:00:55Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -98,6 +98,10 @@ public:
 	virtual bool					On_Command		(int Cmd_ID);
 	virtual bool					On_Command_UI	(wxUpdateUIEvent &event);
 
+	class CSG_Module_Library *		Get_Library		(void)		{	return( m_pLibrary );	}
+
+	void							Update			(void);
+
 	class CWKSP_Module *			Get_Module		(int i)		{	return( (class CWKSP_Module *)Get_Item(i) );	}
 	class CWKSP_Module *			Get_Module		(class CWKSP_Module *pModule);
 	class CWKSP_Module *			Get_Module_byID	(int CMD_ID);
@@ -111,6 +115,10 @@ private:
 
 	class CSG_Module_Library		*m_pLibrary;
 
+
+	void							_Add_Modules	(void);
+	void							_Del_Modules	(void);
+
 };
 
 
