@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: wksp_module_manager.cpp 2087 2014-04-04 09:08:54Z oconrad $
+ * Version $Id: wksp_module_manager.cpp 2723 2015-12-11 10:18:03Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -91,8 +91,6 @@ CWKSP_Module_Manager	*g_pModules	= NULL;
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -147,7 +145,7 @@ CWKSP_Module_Manager::CWKSP_Module_Manager(void)
 	m_Parameters.Add_Value(
 		pNode	, "BEEP"			, _TL("Beep when finished"),
 		_TL(""),
-		PARAMETER_TYPE_Bool	, true
+		PARAMETER_TYPE_Bool	, false
 	);
 
 	m_Parameters.Add_Choice(
@@ -164,13 +162,13 @@ CWKSP_Module_Manager::CWKSP_Module_Manager(void)
 
 	m_Parameters.Add_Value(
 		pNode	, "LNG_OLDSTYLE"	, _TL("Old Style Namings"),
-		_TL("Use old style namings (e.g. 'modules' instead of 'tools'). Ignored if translation file is used. You need to restart SAGA to apply changes."),
+		_TL("Use old style namings (e.g. 'modules' instead of 'tools'). Ignored if translation file is used. You need to restart SAGA to apply the changes."),
 		PARAMETER_TYPE_Bool, false
 	);
 
 	m_Parameters.Add_FilePath(
 		pNode	, "LNG_FILE_DIC"	, _TL("Language Translations"),
-		_TL("Dictionary for translations from built-in (English) to local language (editable text table). You need to restart SAGA to apply changes."),
+		_TL("Dictionary for translations from built-in (English) to local language (editable text table). You need to restart SAGA to apply the changes."),
 		CSG_String::Format(SG_T("%s|*.lng|%s|*.txt|%s|*.*"),
 			_TL("Dictionary Files (*.lng)"),
 			_TL("Text Table (*.txt)"),
@@ -180,7 +178,7 @@ CWKSP_Module_Manager::CWKSP_Module_Manager(void)
 
 	m_Parameters.Add_FilePath(
 		pNode	, "CRS_FILE_SRS"		, _TL("CRS Database"),
-		_TL("Database with Coordinate Reference System (CRS) definitions. You have to restart SAGA to make changes take affect!"),
+		_TL("Database with Coordinate Reference System (CRS) definitions. A restart of SAGA is required to have the changes take effect!"),
 		CSG_String::Format(SG_T("%s|*.srs|%s|*.*"),
 			_TL("Spatial Reference System Files (*.srs)"),
 			_TL("All Files")
@@ -189,12 +187,21 @@ CWKSP_Module_Manager::CWKSP_Module_Manager(void)
 
 	m_Parameters.Add_FilePath(
 		pNode	, "CRS_FILE_DIC"		, _TL("CRS Dictionary"),
-		_TL("Dictionary for Proj.4/OGC WKT translations. You have to restart SAGA to make changes take affect!"),
+		_TL("Dictionary for Proj.4/OGC WKT translations. A restart of SAGA is required to have the changes take effect!"),
 		CSG_String::Format(SG_T("%s|*.dic|%s|*.*"),
 			_TL("Dictionary Files (*.dic)"),
 			_TL("All Files")
 		)
 	);
+
+	m_Parameters.Add_FilePath(
+		pNode	, "TOOL_MENUS"			, _TL("User defined tool menus"),
+		_TL("User defined tool menus."),
+		CSG_String::Format(SG_T("%s|*.xml|%s|*.*"),
+			_TL("XML Files (*.xml)"),
+			_TL("All Files")
+		)
+	);
 }
 
 //---------------------------------------------------------
@@ -208,8 +215,6 @@ CWKSP_Module_Manager::~CWKSP_Module_Manager(void)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -221,6 +226,10 @@ bool CWKSP_Module_Manager::Initialise(void)
 {
 	CONFIG_Read("/MODULES", &m_Parameters);
 
+	CONFIG_Do_Save(m_Parameters("SAVE_CONFIG")->asBool());
+
+	g_pSAGA->Process_Set_Frequency(m_Parameters("PROCESS_UPDATE")->asInt());
+
 #ifdef _OPENMP
 	SG_Set_Max_Num_Threads_Omp(m_Parameters("OMP_THREADS_MAX")->asInt());
 #endif
@@ -230,22 +239,36 @@ bool CWKSP_Module_Manager::Initialise(void)
 
 	for(int i=0; CONFIG_Read(CFG_LIBS, wxString::Format(CFG_LIBF, i), Library); i++)
 	{
-		Open(Library);
+		if( !wxFileExists(Library) )
+		{
+			wxFileName	fn(Library);	fn.MakeAbsolute(g_pSAGA->Get_App_Path());
+
+			Library	= fn.GetFullPath();
+		}
+
+		SG_Get_Module_Library_Manager().Add_Library(Library);
 	}
 
-	if( Get_Count() == 0 )
+	if( SG_Get_Module_Library_Manager().Get_Count() == 0 )
 	{
 #if defined(_SAGA_LINUX)
-	if( _Open_Directory(wxT(MODULE_LIBRARY_PATH)) == 0 )
+	if( SG_Get_Module_Library_Manager().Add_Directory(wxT(MODULE_LIBRARY_PATH), false) == 0 )
 #endif
-		_Open_Directory(g_pSAGA->Get_App_Path(), true);
+		SG_Get_Module_Library_Manager().Add_Directory(g_pSAGA->Get_App_Path(), true);
 	}
 
-	m_pMenu_Modules->Update();
+	_Update(false);
 
 	return( true );
 }
 
+//---------------------------------------------------------
+#ifdef _SAGA_MSW
+	#define GET_LIBPATH(path)	Get_FilePath_Relative(g_pSAGA->Get_App_Path(), path.c_str())
+#else
+	#define GET_LIBPATH(path)	path.c_str()
+#endif
+
 //---------------------------------------------------------
 bool CWKSP_Module_Manager::Finalise(void)
 {
@@ -253,20 +276,26 @@ bool CWKSP_Module_Manager::Finalise(void)
 
 	CONFIG_Delete(CFG_LIBS);
 
-	for(int i=0; i<Get_Count(); i++)
+	for(int i=0, n=0; i<Get_Count(); i++)
 	{
-#ifdef _SAGA_MSW
-		wxFileName	fLib(Get_Library(i)->Get_File_Name());
+		CWKSP_Module_Group	*pGroup	= Get_Group(i);
 
-		if( fLib.GetPath().Find(g_pSAGA->Get_App_Path()) == 0 )
+		for(int j=0; j<pGroup->Get_Count(); j++)
 		{
-			fLib.MakeRelativeTo(g_pSAGA->Get_App_Path());
-		}
+			CSG_Module_Library	*pLibrary	= pGroup->Get_Library(j)->Get_Library();
 
-		CONFIG_Write(CFG_LIBS, wxString::Format(CFG_LIBF, i), fLib.GetFullPath());
-#else
-		CONFIG_Write(CFG_LIBS, wxString::Format(CFG_LIBF, i), Get_Library(i)->Get_File_Name());
-#endif
+			if( pLibrary->Get_Type() == MODULE_CHAINS )
+			{
+				for(int j=0; j<pLibrary->Get_Count(); j++)
+				{
+					CONFIG_Write(CFG_LIBS, wxString::Format(CFG_LIBF, n++), GET_LIBPATH(pLibrary->Get_File_Name(j)));
+				}
+			}
+			else
+			{
+				CONFIG_Write(CFG_LIBS, wxString::Format(CFG_LIBF, n++), GET_LIBPATH(pLibrary->Get_File_Name()));
+			}
+		}
 	}
 
 	return( true );
@@ -275,8 +304,6 @@ bool CWKSP_Module_Manager::Finalise(void)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -315,8 +342,6 @@ wxMenu * CWKSP_Module_Manager::Get_Menu(void)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -416,8 +441,6 @@ void CWKSP_Module_Manager::On_Execute_UI(wxUpdateUIEvent &event)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -425,8 +448,6 @@ void CWKSP_Module_Manager::Parameters_Changed(void)
 {
 	CWKSP_Base_Item::Parameters_Changed();
 
-	g_pSAGA->Process_Set_Frequency(m_Parameters("PROCESS_UPDATE")->asInt());
-
 	if( m_Parameters("SAVE_CONFIG")->asBool() == false )
 	{
 		CONFIG_Write("/MODULES", &m_Parameters);
@@ -434,9 +455,13 @@ void CWKSP_Module_Manager::Parameters_Changed(void)
 
 	CONFIG_Do_Save(m_Parameters("SAVE_CONFIG")->asBool());
 
+	g_pSAGA->Process_Set_Frequency(m_Parameters("PROCESS_UPDATE")->asInt());
+
 #ifdef _OPENMP
 	SG_Set_Max_Num_Threads_Omp(m_Parameters("OMP_THREADS_MAX")->asInt());
 #endif
+
+	m_pMenu_Modules->Update();
 }
 
 //---------------------------------------------------------
@@ -448,51 +473,103 @@ bool CWKSP_Module_Manager::Do_Beep(void)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-int CWKSP_Module_Manager::_Open_Directory(const wxString &sDirectory, bool bOnlySubDirectories)
+CWKSP_Module_Group * CWKSP_Module_Manager::Get_Group(const wxString &Group)
 {
-	int			nOpened	= 0;
-	wxDir		Dir;
-	wxString	FileName;
-
-	if( Dir.Open(sDirectory) )
+	for(int i=0; i<Get_Count(); i++)
 	{
-		if( !bOnlySubDirectories && Dir.GetFirst(&FileName, wxEmptyString, wxDIR_FILES) )
+		if( !Group.Cmp(Get_Group(i)->Get_Name()) )
 		{
-			do
-			{	if( FileName.Find(wxT("saga_")) < 0 && FileName.Find(wxT("wx")) < 0 && FileName.Find(wxT("mingw")) < 0 )
-				if( Open(SG_File_Make_Path(Dir.GetName(), FileName, NULL).w_str()) )
-				{
-					nOpened++;
-				}
-			}
-			while( Dir.GetNext(&FileName) );
-		}
-
-		if( Dir.GetFirst(&FileName, wxEmptyString, wxDIR_DIRS) )
-		{
-			do
-			{
-				if( FileName.CmpNoCase(wxT("dll")) )	// ignore subdirectory 'dll'
-				{
-					nOpened	+= _Open_Directory(SG_File_Make_Path(Dir.GetName(), FileName, NULL).c_str());
-				}
-			}
-			while( Dir.GetNext(&FileName) );
+			return( Get_Group(i) );
 		}
 	}
 
-	return( nOpened );
+	return( NULL );
+}
+
+//---------------------------------------------------------
+CWKSP_Module_Library * CWKSP_Module_Manager::Get_Library(CSG_Module_Library *pLibrary)
+{
+	for(int i=0; i<Get_Count(); i++)
+	{
+		CWKSP_Module_Group	*pGroup	= Get_Group(i);
+
+		for(int j=0; j<pGroup->Get_Count(); j++)
+		{
+			if( pLibrary == pGroup->Get_Library(j)->Get_Library() )
+			{
+				return( pGroup->Get_Library(j) );
+			}
+		}
+	}
+
+	return( NULL );
+}
+
+//---------------------------------------------------------
+bool CWKSP_Module_Manager::_Update(bool bSyncToCtrl)
+{
+	if( !bSyncToCtrl )
+	{
+	//	Get_Control()->Freeze();
+
+		for(int i=SG_Get_Module_Library_Manager().Get_Count()-1; i>=0; i--)
+		{
+			CSG_Module_Library	*pLibrary	= SG_Get_Module_Library_Manager().Get_Library(i);
+			CWKSP_Module_Library	*pItem	= Get_Library(pLibrary);
+
+			if( !pItem )
+			{
+				CWKSP_Module_Group	*pGroup	= Get_Group(pLibrary->Get_Category().c_str());
+
+				if( !pGroup )
+				{
+					Add_Item(pGroup = new CWKSP_Module_Group(pLibrary->Get_Category().c_str()));
+				}
+
+				pGroup->Add_Library(pLibrary);
+			}
+			else if( pItem->Get_Library()->Get_Type() == MODULE_CHAINS )
+			{
+				pItem->Update();
+			}
+		}
+
+	//	Get_Control()->Thaw();
+	}
+
+	m_pMenu_Modules->Update();
+
+	return( true );
+}
+
+//---------------------------------------------------------
+bool CWKSP_Module_Manager::Update(void)
+{
+	return( _Update(true) );
 }
 
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+wxMenu * CWKSP_Module_Manager::Get_Menu_Modules(void)
+{
+	return( m_pMenu_Modules->Get_Menu() );
+}
+
+//---------------------------------------------------------
+void CWKSP_Module_Manager::Set_Recently_Used(CWKSP_Module *pModule)
+{
+	m_pMenu_Modules->Set_Recent(pModule);
+}
+
+
+///////////////////////////////////////////////////////////
 //														 //
 ///////////////////////////////////////////////////////////
 
@@ -505,12 +582,20 @@ void CWKSP_Module_Manager::Open(void)
 	{
 		MSG_General_Add_Line();
 
+		bool	bUpdate	= false;
+
 		for(size_t i=0; i<File_Paths.GetCount(); i++)
 		{
-			Open(File_Paths[i]);
+			if( SG_Get_Module_Library_Manager().Add_Library(File_Paths[i]) )
+			{
+				bUpdate	= true;
+			}
 		}
 
-		m_pMenu_Modules->Update();
+		if( bUpdate )
+		{
+			_Update(false);
+		}
 	}
 }
 
@@ -519,7 +604,9 @@ bool CWKSP_Module_Manager::Open(const wxString &File_Name)
 {
 	if( SG_Get_Module_Library_Manager().Add_Library(File_Name) )
 	{
-		return( Add_Item(new CWKSP_Module_Library(SG_Get_Module_Library_Manager().Get_Library(SG_Get_Module_Library_Manager().Get_Count() - 1))) );
+		_Update(false);
+
+		return( true );
 	}
 
 	return( false );
@@ -528,8 +615,6 @@ bool CWKSP_Module_Manager::Open(const wxString &File_Name)
 
 ///////////////////////////////////////////////////////////
 //														 //
-//														 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -537,9 +622,14 @@ bool CWKSP_Module_Manager::Exists(CWKSP_Module *pModule)
 {
 	for(int i=0; i<Get_Count(); i++)
 	{
-		if( Get_Library(i)->Get_Module(pModule) != NULL )
+		CWKSP_Module_Group	*pGroup	= Get_Group(i);
+
+		for(int j=0; j<pGroup->Get_Count(); j++)
 		{
-			return( true );
+			if( pGroup->Get_Library(j)->Get_Module(pModule) != NULL )
+			{
+				return( true );
+			}
 		}
 	}
 
@@ -553,9 +643,96 @@ CWKSP_Module * CWKSP_Module_Manager::Get_Module_byID(int CMD_ID)
 
 	for(int i=0; i<Get_Count(); i++)
 	{
-		if( (pModule = Get_Library(i)->Get_Module_byID(CMD_ID)) != NULL )
+		CWKSP_Module_Group	*pGroup	= Get_Group(i);
+
+		for(int j=0; j<pGroup->Get_Count(); j++)
 		{
-			return( pModule );
+			if( (pModule = pGroup->Get_Library(j)->Get_Module_byID(CMD_ID)) != NULL )
+			{
+				return( pModule );
+			}
+		}
+	}
+
+	return( NULL );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+CWKSP_Module_Group::CWKSP_Module_Group(const wxString &Name)
+{
+	m_Name	= Name;
+}
+
+//---------------------------------------------------------
+CWKSP_Module_Group::~CWKSP_Module_Group(void)
+{}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+wxString CWKSP_Module_Group::Get_Description(void)
+{
+	wxString	s;
+
+	s.Printf(wxT("<b>%s</b>: %d<br>"), _TL("Tool Libraries"), Get_Count());
+
+	return( s );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CWKSP_Module_Group::On_Command(int Cmd_ID)
+{
+//	switch( Cmd_ID )
+	{
+//	default:
+		return( CWKSP_Base_Manager::On_Command(Cmd_ID) );
+	}
+}
+
+//---------------------------------------------------------
+bool CWKSP_Module_Group::On_Command_UI(wxUpdateUIEvent &event)
+{
+//	switch( event.GetId() )
+	{
+//	default:
+		return( CWKSP_Base_Manager::On_Command_UI(event) );
+	}
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CWKSP_Module_Group::Add_Library(CSG_Module_Library *pLibrary)
+{
+	return( pLibrary && Add_Item(new CWKSP_Module_Library(pLibrary)) );
+}
+
+//---------------------------------------------------------
+CWKSP_Module_Library * CWKSP_Module_Group::Get_Library(CSG_Module_Library *pLibrary)
+{
+	for(int i=0; i<Get_Count(); i++)
+	{
+		if( pLibrary == Get_Library(i)->Get_Library() )
+		{
+			return( Get_Library(i) );
 		}
 	}
 
