@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: wksp_module_menu.h 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: wksp_module_menu.h 2349 2014-12-11 13:42:58Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -106,7 +106,7 @@ private:
 
 
 	void						_Update				(wxMenu *pMenu);
-	wxMenu *					_Get_SubMenu		(wxMenu *pMenu, wxString Menu_Path);
+	bool						_Get_SubMenu		(class CWKSP_Module *pModule, class CSG_MetaData *pUser);
 	wxMenu *					_Get_SubMenu_byToken(wxMenu *pMenu, wxString Token);
 	void						_Set_Recent			(wxMenu *pMenu);
 
