@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: wksp_pointcloud.cpp 1921 2014-01-09 10:24:11Z oconrad $
+ * Version $Id: wksp_pointcloud.cpp 2716 2015-12-04 11:55:42Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -73,7 +73,7 @@
 
 #include "wksp_pointcloud.h"
 #include "wksp_table.h"
-
+#include "view_table.h"
 #include "view_scatterplot.h"
 
 
@@ -87,10 +87,10 @@
 CWKSP_PointCloud::CWKSP_PointCloud(CSG_PointCloud *pPointCloud)
 	: CWKSP_Layer(pPointCloud)
 {
-	m_pTable		= new CWKSP_Table(pPointCloud);
+	m_pTable	= new CWKSP_Table(pPointCloud);
 
 	m_Edit_Attributes.Destroy();
-	m_Edit_Attributes.Add_Field(_TL("Name") , SG_DATATYPE_String);
+	m_Edit_Attributes.Add_Field(_TL("Name" ), SG_DATATYPE_String);
 	m_Edit_Attributes.Add_Field(_TL("Value"), SG_DATATYPE_String);
 
 	//-----------------------------------------------------
@@ -98,7 +98,7 @@ CWKSP_PointCloud::CWKSP_PointCloud(CSG_PointCloud *pPointCloud)
 
 	DataObject_Changed();
 
-	m_Parameters("COLORS_TYPE"  )->Set_Value(CLASSIFY_METRIC);
+	m_Parameters("COLORS_TYPE"  )->Set_Value(CLASSIFY_GRADUATED);
 	m_Parameters("METRIC_ATTRIB")->Set_Value(2);
 
 	On_Parameter_Changed(&m_Parameters, m_Parameters("METRIC_ATTRIB"), PARAMETER_CHECK_ALL);
@@ -129,17 +129,17 @@ wxString CWKSP_PointCloud::Get_Description(void)
 
 	s	+= wxT("<table border=\"0\">");
 
-	DESC_ADD_STR(_TL("Name")			, m_pObject->Get_Name());
-	DESC_ADD_STR(_TL("Description")		, m_pObject->Get_Description());
-	DESC_ADD_STR(_TL("File")			, SG_File_Exists(m_pObject->Get_File_Name()) ? m_pObject->Get_File_Name() : _TL("memory"));
-	DESC_ADD_STR(_TL("Modified")		, m_pObject->is_Modified() ? _TL("yes") : _TL("no"));
-	DESC_ADD_STR(_TL("Projection")		, m_pObject->Get_Projection().Get_Description().c_str());
-	DESC_ADD_FLT(_TL("West")			, Get_PointCloud()->Get_Extent().Get_XMin());
-	DESC_ADD_FLT(_TL("East")			, Get_PointCloud()->Get_Extent().Get_XMax());
-	DESC_ADD_FLT(_TL("West-East")		, Get_PointCloud()->Get_Extent().Get_XRange());
-	DESC_ADD_FLT(_TL("South")			, Get_PointCloud()->Get_Extent().Get_YMin());
-	DESC_ADD_FLT(_TL("North")			, Get_PointCloud()->Get_Extent().Get_YMax());
-	DESC_ADD_FLT(_TL("South-North")		, Get_PointCloud()->Get_Extent().Get_YRange());
+	DESC_ADD_STR(_TL("Name"            ), m_pObject->Get_Name());
+	DESC_ADD_STR(_TL("Description"     ), m_pObject->Get_Description());
+	DESC_ADD_STR(_TL("File"            ), SG_File_Exists(m_pObject->Get_File_Name()) ? m_pObject->Get_File_Name() : _TL("memory"));
+	DESC_ADD_STR(_TL("Modified"        ), m_pObject->is_Modified() ? _TL("yes") : _TL("no"));
+	DESC_ADD_STR(_TL("Projection"      ), m_pObject->Get_Projection().Get_Description().c_str());
+	DESC_ADD_FLT(_TL("West"            ), Get_PointCloud()->Get_Extent().Get_XMin());
+	DESC_ADD_FLT(_TL("East"            ), Get_PointCloud()->Get_Extent().Get_XMax());
+	DESC_ADD_FLT(_TL("West-East"       ), Get_PointCloud()->Get_Extent().Get_XRange());
+	DESC_ADD_FLT(_TL("South"           ), Get_PointCloud()->Get_Extent().Get_YMin());
+	DESC_ADD_FLT(_TL("North"           ), Get_PointCloud()->Get_Extent().Get_YMax());
+	DESC_ADD_FLT(_TL("South-North"     ), Get_PointCloud()->Get_Extent().Get_YRange());
 	DESC_ADD_INT(_TL("Number of Points"), Get_PointCloud()->Get_Count());
 
 	s	+= wxT("</table>");
@@ -162,8 +162,12 @@ wxMenu * CWKSP_PointCloud::Get_Menu(void)
 
 	CMD_Menu_Add_Item(pMenu, false, ID_CMD_WKSP_ITEM_CLOSE);
 	CMD_Menu_Add_Item(pMenu, false, ID_CMD_POINTCLOUD_SHOW);
-	CMD_Menu_Add_Item(pMenu, false, ID_CMD_POINTCLOUD_SAVE);
-	CMD_Menu_Add_Item(pMenu, false, ID_CMD_POINTCLOUD_SAVEAS);
+	pMenu->AppendSeparator();
+	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_SAVE);
+	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_SAVEAS);
+	pMenu->AppendSeparator();
+	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_PROJECTION);
+	pMenu->AppendSeparator();
 	CMD_Menu_Add_Item(pMenu,  true, ID_CMD_SHAPES_HISTOGRAM);
 	CMD_Menu_Add_Item(pMenu, false, ID_CMD_SHAPES_SET_LUT);
 	CMD_Menu_Add_Item(pMenu, false, ID_CMD_WKSP_ITEM_SETTINGS_COPY);
@@ -171,7 +175,7 @@ wxMenu * CWKSP_PointCloud::Get_Menu(void)
 	pMenu->AppendSeparator();
 
 	//-----------------------------------------------------
-	wxMenu	*pSubMenu	= new wxMenu(_TL("Classificaton"));
+	wxMenu	*pSubMenu	= new wxMenu(_TL("Classification"));
 
 	CMD_Menu_Add_Item(pSubMenu	, false, ID_CMD_POINTCLOUD_RANGE_MINMAX);
 	CMD_Menu_Add_Item(pSubMenu	, false, ID_CMD_POINTCLOUD_RANGE_STDDEV150);
@@ -248,6 +252,7 @@ bool CWKSP_PointCloud::On_Command(int Cmd_ID)
 		if( Get_PointCloud()->Get_Selection_Count() > 0 && DLG_Message_Confirm(_TL("Delete selected point(s)."), _TL("Edit Point Cloud")) )
 		{
 			Get_PointCloud()->Del_Selection();
+
 			Update_Views();
 		}
 		break;
@@ -423,10 +428,8 @@ int CWKSP_PointCloud::On_Parameter_Changed(CSG_Parameters *pParameters, CSG_Para
 	//-----------------------------------------------------
 	if( Flags & PARAMETER_CHECK_VALUES )
 	{
-		if(	!SG_STR_CMP(pParameter->Get_Identifier(), wxT("METRIC_ATTRIB")) )
+		if(	!SG_STR_CMP(pParameter->Get_Identifier(), "METRIC_ATTRIB") )
 		{
-			CSG_Parameters	Parameters;
-	
 			int		zField	= pParameter->asInt();
 
 			double	m	= Get_PointCloud()->Get_Mean  (zField);
@@ -441,14 +444,14 @@ int CWKSP_PointCloud::On_Parameter_Changed(CSG_Parameters *pParameters, CSG_Para
 	//-----------------------------------------------------
 	if( Flags & PARAMETER_CHECK_ENABLE )
 	{
-		if(	!SG_STR_CMP(pParameter->Get_Identifier(), SG_T("COLORS_TYPE")) )
+		if(	!SG_STR_CMP(pParameter->Get_Identifier(), "COLORS_TYPE") )
 		{
 			int	Value	= pParameter->asInt();
 
-			pParameters->Get_Parameter("NODE_UNISYMBOL")->Set_Enabled(Value == CLASSIFY_UNIQUE);
-			pParameters->Get_Parameter("NODE_LUT"      )->Set_Enabled(Value == CLASSIFY_LUT);
-			pParameters->Get_Parameter("NODE_METRIC"   )->Set_Enabled(Value == CLASSIFY_METRIC || Value == CLASSIFY_GRADUATED);
-			pParameters->Get_Parameter("NODE_RGB"	   )->Set_Enabled(Value == 4);
+			pParameters->Set_Enabled("NODE_UNISYMBOL", Value == CLASSIFY_UNIQUE);
+			pParameters->Set_Enabled("NODE_LUT"      , Value == CLASSIFY_LUT);
+			pParameters->Set_Enabled("NODE_METRIC"   , Value == CLASSIFY_METRIC || Value == CLASSIFY_GRADUATED);
+			pParameters->Set_Enabled("NODE_RGB"	     , Value == 4);
 
 			return( 1 );
 		}
@@ -464,6 +467,22 @@ void CWKSP_PointCloud::On_Update_Views(void)
 	m_pTable->Update_Views();
 }
 
+//---------------------------------------------------------
+bool CWKSP_PointCloud::Fit_Colors(void)
+{
+	if( m_Color_Field >= Get_PointCloud()->Get_Field_Count() )
+	{
+		m_Color_Field	= Get_PointCloud()->Get_Field_Count() - 1;
+	}
+
+	double	m	= Get_PointCloud()->Get_Mean  (m_Color_Field);
+	double	s	= Get_PointCloud()->Get_StdDev(m_Color_Field) * 2.0;
+
+	m_Parameters("METRIC_ZRANGE")->asRange()->Set_Range(m - s, m + s);
+
+	return( true );
+}
+
 
 ///////////////////////////////////////////////////////////
 //														 //
@@ -788,11 +807,9 @@ bool CWKSP_PointCloud::asImage(CSG_Grid *pImage)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-wxMenu * CWKSP_PointCloud::On_Edit_Get_Menu(void)
+wxMenu * CWKSP_PointCloud::Edit_Get_Menu(void)
 {
-	wxMenu	*pMenu;
-
-	pMenu	= new wxMenu;
+	wxMenu	*pMenu	= new wxMenu;
 
 	CMD_Menu_Add_Item(pMenu, false, ID_CMD_SHAPES_EDIT_DEL_SHAPE);
 	CMD_Menu_Add_Item(pMenu, false, ID_CMD_SHAPES_EDIT_SEL_CLEAR);
@@ -802,7 +819,18 @@ wxMenu * CWKSP_PointCloud::On_Edit_Get_Menu(void)
 }
 
 //---------------------------------------------------------
-bool CWKSP_PointCloud::On_Edit_On_Mouse_Up(CSG_Point Point, double ClientToWorld, int Key)
+TSG_Rect CWKSP_PointCloud::Edit_Get_Extent(void)
+{
+	if( Get_PointCloud()->Get_Selection_Count() > 0 )
+	{
+		return( Get_PointCloud()->Get_Selection_Extent().m_rect );
+	}
+
+	return( Get_PointCloud()->Get_Extent() );
+}
+
+//---------------------------------------------------------
+bool CWKSP_PointCloud::Edit_On_Mouse_Up(CSG_Point Point, double ClientToWorld, int Key)
 {
 	if( Key & MODULE_INTERACTIVE_KEY_RIGHT )
 	{
@@ -823,6 +851,7 @@ bool CWKSP_PointCloud::On_Edit_On_Mouse_Up(CSG_Point Point, double ClientToWorld
 
 		//-----------------------------------------------------
 		m_Edit_Attributes.Del_Records();
+		m_Edit_Index	= 0;
 
 		CSG_Table_Record	*pRecord	= Get_PointCloud()->Get_Selection();
 
@@ -839,6 +868,11 @@ bool CWKSP_PointCloud::On_Edit_On_Mouse_Up(CSG_Point Point, double ClientToWorld
 		//-----------------------------------------------------
 		g_pACTIVE->Get_Attributes()->Set_Attributes();
 
+		if( m_pTable->Get_View() )
+		{
+			m_pTable->Get_View()->Update_Selection();
+		}
+
 		Update_Views(false);
 	}
 
@@ -846,15 +880,47 @@ bool CWKSP_PointCloud::On_Edit_On_Mouse_Up(CSG_Point Point, double ClientToWorld
 }
 
 //---------------------------------------------------------
-bool CWKSP_PointCloud::On_Edit_Set_Attributes(void)
+bool CWKSP_PointCloud::Edit_Set_Index(int Index)
 {
-	CSG_Table_Record	*pRecord;
+	m_Edit_Attributes.Del_Records();
 
-	if( (pRecord = Get_PointCloud()->Get_Selection()) != NULL )
+	if( Index > (int)Get_PointCloud()->Get_Selection_Count() )
+	{
+		Index	= Get_PointCloud()->Get_Selection_Count();
+	}
+
+	CSG_Table_Record	*pSelection	= Get_PointCloud()->Get_Selection(Index);
+
+	if( pSelection )
+	{
+		m_Edit_Index	= Index;
+
+		for(int i=0; i<Get_PointCloud()->Get_Field_Count(); i++)
+		{
+			CSG_Table_Record	*pRecord	= m_Edit_Attributes.Add_Record();
+
+			pRecord->Set_Value(0, pSelection->Get_Table()->Get_Field_Name(i));
+			pRecord->Set_Value(1, pSelection->asString(i));
+		}
+	}
+	else
+	{
+		m_Edit_Index	= 0;
+	}
+
+	return( true );
+}
+
+//---------------------------------------------------------
+bool CWKSP_PointCloud::Edit_Set_Attributes(void)
+{
+	CSG_Table_Record	*pSelection	= Get_PointCloud()->Get_Selection(m_Edit_Index);
+
+	if( pSelection )
 	{
 		for(int i=0; i<m_Edit_Attributes.Get_Record_Count(); i++)
 		{
-			pRecord->Set_Value(i, m_Edit_Attributes.Get_Record(i)->asString(1));
+			pSelection->Set_Value(i, m_Edit_Attributes.Get_Record(i)->asString(1));
 		}
 
 		Update_Views(false);
@@ -865,12 +931,6 @@ bool CWKSP_PointCloud::On_Edit_Set_Attributes(void)
 	return( false );
 }
 
-//---------------------------------------------------------
-TSG_Rect CWKSP_PointCloud::On_Edit_Get_Extent(void)
-{
-	return( Get_PointCloud()->Get_Extent() );
-}
-
 
 ///////////////////////////////////////////////////////////
 //														 //
@@ -879,11 +939,18 @@ TSG_Rect CWKSP_PointCloud::On_Edit_Get_Extent(void)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-void CWKSP_PointCloud::On_Draw(CWKSP_Map_DC &dc_Map, bool bEdit)
+void CWKSP_PointCloud::On_Draw(CWKSP_Map_DC &dc_Map, int Flags)
 {
 	if( Get_Extent().Intersects(dc_Map.m_rWorld) != INTERSECTION_None && dc_Map.IMG_Draw_Begin(m_Parameters("DISPLAY_TRANSPARENCY")->asDouble() / 100.0) )
 	{
-		_Draw_Points	(dc_Map);
+		if( (Flags & LAYER_DRAW_FLAG_THUMBNAIL) == 0 )
+		{
+			_Draw_Points   (dc_Map);
+		}
+		else
+		{
+			_Draw_Thumbnail(dc_Map);
+		}
 
 		dc_Map.IMG_Draw_End();
 	}
@@ -967,27 +1034,35 @@ void CWKSP_PointCloud::_Draw_Points(CWKSP_Map_DC &dc_Map)
 	}
 
 	//-----------------------------------------------------
-	for(int i=0; i<Get_PointCloud()->Get_Count(); i++)
+	CSG_PointCloud	*pPoints	= Get_PointCloud();
+
+	int	Selection	= pPoints->Get_Selection_Count() > 0 ? (int)pPoints->Get_Selection_Index(m_Edit_Index) : -1;
+
+	for(int i=0; i<pPoints->Get_Count(); i++)
 	{
-		if( !Get_PointCloud()->is_NoData(i, m_Color_Field) )
+		pPoints->Set_Cursor(i);
+
+		if( !pPoints->is_NoData(m_Color_Field) )
 		{
-			TSG_Point_Z	Point	= Get_PointCloud()->Get_Point(i);
+			TSG_Point_Z	Point	= pPoints->Get_Point();
 
 			if( dc_Map.m_rWorld.Contains(Point.x, Point.y) )
 			{
 				int		x	= (int)dc_Map.xWorld2DC(Point.x);
 				int		y	= (int)dc_Map.yWorld2DC(Point.y);
 
-				if( Get_PointCloud()->is_Selected(i) )
+				if( Selection >= 0 && pPoints->is_Selected(i) )
 				{
-					_Draw_Point(dc_Map, x, y, Point.z, SG_COLOR_RED   , m_PointSize + 2);
-					_Draw_Point(dc_Map, x, y, Point.z, SG_COLOR_YELLOW, m_PointSize);
+					int	Size	= Selection == i ? 2 + m_PointSize : m_PointSize;
+
+					_Draw_Point(dc_Map, x, y, Point.z, SG_COLOR_YELLOW, Size    );
+					_Draw_Point(dc_Map, x, y, Point.z, SG_COLOR_RED   , Size + 2);
 				}
 				else
 				{
 					int		Color;
 
-					m_pClassify->Get_Class_Color_byValue(Get_PointCloud()->Get_Value(i, m_Color_Field), Color);
+					m_pClassify->Get_Class_Color_byValue(pPoints->Get_Value(m_Color_Field), Color);
 
 					_Draw_Point(dc_Map, x, y, Point.z, Color, m_PointSize);
 				}
@@ -996,6 +1071,33 @@ void CWKSP_PointCloud::_Draw_Points(CWKSP_Map_DC &dc_Map)
 	}
 }
 
+//---------------------------------------------------------
+void CWKSP_PointCloud::_Draw_Thumbnail(CWKSP_Map_DC &dc_Map)
+{
+	CSG_PointCloud	*pPoints	= Get_PointCloud();
+
+	int	n	= 1 + (int)(pPoints->Get_Count() / (2 * dc_Map.m_rDC.GetWidth() * dc_Map.m_rDC.GetHeight()));
+
+	for(int i=0; i<pPoints->Get_Count(); i+=n)
+	{
+		pPoints->Set_Cursor(i);
+
+		if( !pPoints->is_NoData(m_Color_Field) )
+		{
+			TSG_Point_Z	Point	= pPoints->Get_Point();
+
+			int	x	= (int)dc_Map.xWorld2DC(Point.x);
+			int	y	= (int)dc_Map.yWorld2DC(Point.y);
+
+			int	Color;
+
+			m_pClassify->Get_Class_Color_byValue(pPoints->Get_Value(m_Color_Field), Color);
+
+			dc_Map.IMG_Set_Pixel(x, y, Color);
+		}
+	}
+}
+
 
 ///////////////////////////////////////////////////////////
 //														 //
