@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: wksp_shapes.cpp 2202 2014-08-26 13:24:07Z oconrad $
+ * Version $Id: wksp_shapes.cpp 2668 2015-10-30 11:03:12Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -72,6 +72,8 @@
 #include "wksp_shapes.h"
 #include "wksp_table.h"
 
+#include "data_source_pgsql.h"
+
 
 ///////////////////////////////////////////////////////////
 //														 //
@@ -122,20 +124,36 @@ wxString CWKSP_Shapes::Get_Description(void)
 
 	s	+= wxT("<table border=\"0\">");
 
-	DESC_ADD_STR(_TL("Name")			, m_pObject->Get_Name());
-	DESC_ADD_STR(_TL("Description")		, m_pObject->Get_Description());
-	DESC_ADD_STR(_TL("File")			, SG_File_Exists(m_pObject->Get_File_Name()) ? m_pObject->Get_File_Name() : _TL("memory"));
-	DESC_ADD_STR(_TL("Modified")		, m_pObject->is_Modified() ? _TL("yes") : _TL("no"));
-	DESC_ADD_STR(_TL("Projection")		, m_pObject->Get_Projection().Get_Description().c_str());
-	DESC_ADD_FLT(_TL("West")			, Get_Shapes()->Get_Extent().Get_XMin());
-	DESC_ADD_FLT(_TL("East")			, Get_Shapes()->Get_Extent().Get_XMax());
-	DESC_ADD_FLT(_TL("West-East")		, Get_Shapes()->Get_Extent().Get_XRange());
-	DESC_ADD_FLT(_TL("South")			, Get_Shapes()->Get_Extent().Get_YMin());
-	DESC_ADD_FLT(_TL("North")			, Get_Shapes()->Get_Extent().Get_YMax());
-	DESC_ADD_FLT(_TL("South-North")		, Get_Shapes()->Get_Extent().Get_YRange());
-	DESC_ADD_STR(_TL("Type")			, SG_Get_ShapeType_Name(Get_Shapes()->Get_Type()).c_str());
-	DESC_ADD_STR(_TL("Vertex Type")		, Get_Shapes()->Get_Vertex_Type() == 0 ? _TL("X, Y") : Get_Shapes()->Get_Vertex_Type() == 1 ? _TL("X, Y, Z") : _TL("X, Y, Z, M"));
-	DESC_ADD_INT(_TL("Number Of Shapes"), Get_Shapes()->Get_Count());
+	DESC_ADD_STR(_TL("Name"            ), m_pObject->Get_Name());
+	DESC_ADD_STR(_TL("Description"     ), m_pObject->Get_Description());
+
+	if( SG_File_Exists(m_pObject->Get_File_Name(false)) )
+	{
+		DESC_ADD_STR(_TL("File"        ), m_pObject->Get_File_Name(false));
+
+		if( m_pObject->Get_MetaData()("GDAL_DRIVER") )
+			DESC_ADD_STR(_TL("Driver"  ), m_pObject->Get_MetaData()["GDAL_DRIVER"].Get_Content().c_str());
+	}
+	else if( m_pObject->Get_MetaData_DB().Get_Children_Count() )
+	{
+		DESC_ADD_STR(_TL("File"        ), m_pObject->Get_File_Name(false));
+	}
+	else
+	{
+		DESC_ADD_STR(_TL("File"        ), _TL("memory"));
+	}
+
+	DESC_ADD_STR(_TL("Modified"        ), m_pObject->is_Modified() ? _TL("yes") : _TL("no"));
+	DESC_ADD_STR(_TL("Projection"      ), m_pObject->Get_Projection().Get_Description().c_str());
+	DESC_ADD_FLT(_TL("West"            ), Get_Shapes()->Get_Extent().Get_XMin  ());
+	DESC_ADD_FLT(_TL("East"            ), Get_Shapes()->Get_Extent().Get_XMax  ());
+	DESC_ADD_FLT(_TL("West-East"       ), Get_Shapes()->Get_Extent().Get_XRange());
+	DESC_ADD_FLT(_TL("South"           ), Get_Shapes()->Get_Extent().Get_YMin  ());
+	DESC_ADD_FLT(_TL("North"           ), Get_Shapes()->Get_Extent().Get_YMax  ());
+	DESC_ADD_FLT(_TL("South-North"     ), Get_Shapes()->Get_Extent().Get_YRange());
+	DESC_ADD_STR(_TL("Type"            ), SG_Get_ShapeType_Name(Get_Shapes()->Get_Type()).c_str());
+	DESC_ADD_STR(_TL("Vertex Type"     ), Get_Shapes()->Get_Vertex_Type() == 0 ? _TL("X, Y") : Get_Shapes()->Get_Vertex_Type() == 1 ? _TL("X, Y, Z") : _TL("X, Y, Z, M"));
+	DESC_ADD_INT(_TL("Number of Shapes"), Get_Shapes()->Get_Count());
 
 	s	+= wxT("</table>");
 
@@ -157,8 +175,12 @@ wxMenu * CWKSP_Shapes::Get_Menu(void)
 	CMD_Menu_Add_Item(pMenu, false, ID_CMD_WKSP_ITEM_CLOSE);
 	CMD_Menu_Add_Item(pMenu, false, ID_CMD_SHAPES_SHOW);
 	pMenu->AppendSeparator();
-	CMD_Menu_Add_Item(pMenu, false, ID_CMD_SHAPES_SAVE);
-	CMD_Menu_Add_Item(pMenu, false, ID_CMD_SHAPES_SAVEAS);
+	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_SAVE);
+	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_SAVEAS);
+	if( PGSQL_has_Connections() )
+	{
+		CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_SAVETODB);
+	}
 	pMenu->AppendSeparator();
 	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_PROJECTION);
 	pMenu->AppendSeparator();
@@ -168,11 +190,11 @@ wxMenu * CWKSP_Shapes::Get_Menu(void)
 
 	pMenu->AppendSeparator();
 
-	wxMenu	*pTable	= new wxMenu(_TL("Table"));
+	wxMenu	*pTable	= new wxMenu(_TL("Attributes"));
 	CMD_Menu_Add_Item(pTable,  true, ID_CMD_TABLES_SHOW);
 	CMD_Menu_Add_Item(pTable,  true, ID_CMD_TABLES_DIAGRAM);
 	CMD_Menu_Add_Item(pTable, false, ID_CMD_TABLES_SCATTERPLOT);
-	CMD_Menu_Add_Item(pTable, false, ID_CMD_TABLES_SAVEAS);
+	CMD_Menu_Add_Item(pTable, false, ID_CMD_SHAPES_SAVE_ATTRIBUTES);
 	pMenu->Append(ID_CMD_WKSP_FIRST, _TL("Attributes"), pTable);
 
 	pMenu->Append(ID_CMD_WKSP_FIRST, _TL("Edit"), Edit_Get_Menu());
@@ -228,7 +250,7 @@ bool CWKSP_Shapes::On_Command(int Cmd_ID)
 		Add_ScatterPlot();
 		break;
 
-	case ID_CMD_TABLES_SAVEAS:
+	case ID_CMD_SHAPES_SAVE_ATTRIBUTES:
 		{
 			wxString	File(m_pObject->Get_File_Name());
 
@@ -394,6 +416,12 @@ void CWKSP_Shapes::On_Create_Parameters(void)
 		PARAMETER_TYPE_Color, SG_GET_RGB(255, 255, 255)
 	);
 
+	m_Parameters.Add_Value(
+		m_Parameters("LABEL_ATTRIB_EFFECT"), "LABEL_ATTRIB_EFFECT_SIZE"		, _TL("Size"),
+		_TL(""),
+		PARAMETER_TYPE_Int, 1, 1, true
+	);
+
 	AttributeList_Add(
 		m_Parameters("LABEL_ATTRIB")	, "LABEL_ATTRIB_SIZE_BY"	, _TL("Size by Attribute"),
 		_TL("")
@@ -435,21 +463,6 @@ void CWKSP_Shapes::On_Create_Parameters(void)
 		PARAMETER_TYPE_Int, 10, 0, true
 	);
 
-#ifdef USE_HTMLINFO
-	//-----------------------------------------------------
-	// HTML Extra info
-
-	m_Parameters.Add_Node(
-		NULL							, "NODE_EXTRAINFO"			, _TL("Html Extra Info"),
-		_TL("")
-	);
-
-	AttributeList_Add(
-		m_Parameters("NODE_EXTRAINFO")	, "EXTRAINFO_ATTRIB"		, _TL("Attribute"),
-		_TL("")
-	);
-#endif
-
 	m_Parameters("LUT")->asTable()->Set_Field_Type(LUT_MIN, SG_DATATYPE_String);
 	m_Parameters("LUT")->asTable()->Set_Field_Type(LUT_MAX, SG_DATATYPE_String);
 }
@@ -466,18 +479,14 @@ void CWKSP_Shapes::On_DataObject_Changed(void)
 {
 	CWKSP_Layer::On_DataObject_Changed();
 
-	AttributeList_Set(m_Parameters("LUT_ATTRIB")			, false);
-	AttributeList_Set(m_Parameters("METRIC_ATTRIB")		, false);
-	AttributeList_Set(m_Parameters("LABEL_ATTRIB")			, true);
-	AttributeList_Set(m_Parameters("LABEL_ATTRIB_SIZE_BY")	, true);
+	AttributeList_Set(m_Parameters("LUT_ATTRIB"          ), false);
+	AttributeList_Set(m_Parameters("METRIC_ATTRIB"       ), false);
+	AttributeList_Set(m_Parameters("LABEL_ATTRIB"        ), true );
+	AttributeList_Set(m_Parameters("LABEL_ATTRIB_SIZE_BY"), true );
 
 	_Chart_Set_Options();
 
 	m_pTable->DataObject_Changed();
-
-#ifdef USE_HTMLINFO
-	AttributeList_Set(m_Parameters("EXTRAINFO_ATTRIB") , true);
-#endif
 }
 
 //---------------------------------------------------------
@@ -497,6 +506,14 @@ void CWKSP_Shapes::On_Parameters_Changed(void)
 	case 3:	m_iColor	= m_Parameters("METRIC_ATTRIB")->asInt();	break;	// CLASSIFY_GRADUATED
 	}
 
+	if( m_Parameters("COLORS_TYPE")->asInt() == 1 )	// CLASSIFY_LUT
+	{
+		TSG_Data_Type	Type	= SG_Data_Type_is_Numeric(Get_Shapes()->Get_Field_Type(m_Parameters("LUT_ATTRIB")->asInt())) ? SG_DATATYPE_Double : SG_DATATYPE_String;
+
+		m_Parameters("LUT")->asTable()->Set_Field_Type(LUT_MIN, Type);
+		m_Parameters("LUT")->asTable()->Set_Field_Type(LUT_MAX, Type);
+	}
+
 	if( m_iColor >= Get_Shapes()->Get_Field_Count() )
 	{
 		m_iColor	= -1;
@@ -513,7 +530,10 @@ void CWKSP_Shapes::On_Parameters_Changed(void)
 		m_iLabel	= -1;
 	}
 
-	m_Label_Prec		= m_Parameters("LABEL_ATTRIB_PREC")->asInt() - 2;
+	m_Label_Prec		= m_Parameters("LABEL_ATTRIB_PREC"        )->asInt  () - 2;
+
+	m_Label_Eff_Color	= m_Parameters("LABEL_ATTRIB_EFFECT_COLOR")->asColor();
+	m_Label_Eff_Size	= m_Parameters("LABEL_ATTRIB_EFFECT_SIZE" )->asInt  ();
 
 	switch( m_Parameters("LABEL_ATTRIB_EFFECT")->asInt() )
 	{
@@ -529,8 +549,6 @@ void CWKSP_Shapes::On_Parameters_Changed(void)
 	case 9:		m_Label_Eff	= TEXTEFFECT_TOPRIGHT;		break;
 	}
 
-	m_Label_Eff_Color	= m_Parameters("LABEL_ATTRIB_EFFECT_COLOR")->asColor();
-
 	//-----------------------------------------------------
 	_Chart_Get_Options();
 }
@@ -565,8 +583,8 @@ int CWKSP_Shapes::On_Parameter_Changed(CSG_Parameters *pParameters, CSG_Paramete
 			TSG_Data_Type	Type	= SG_Data_Type_is_Numeric(Get_Shapes()->Get_Field_Type(pParameter->asInt()))
 									? SG_DATATYPE_Double : SG_DATATYPE_String;
 
-			m_Parameters("LUT")->asTable()->Set_Field_Type(LUT_MIN, Type);
-			m_Parameters("LUT")->asTable()->Set_Field_Type(LUT_MAX, Type);
+			pParameters->Get_Parameter("LUT")->asTable()->Set_Field_Type(LUT_MIN, Type);
+			pParameters->Get_Parameter("LUT")->asTable()->Set_Field_Type(LUT_MAX, Type);
 		}
 	}
 
@@ -703,6 +721,8 @@ void CWKSP_Shapes::_LUT_Create(void)
 	}
 
 	//-----------------------------------------------------
+	DataObject_Changed();
+
 	pColors	= Parameters("COLOR" )->asColors();
 	iField	= Parameters("FIELD" )->asInt();
 	Method	= !SG_Data_Type_is_Numeric(Get_Shapes()->Get_Field_Type(iField)) ? 0	// unique values
@@ -834,8 +854,6 @@ void CWKSP_Shapes::_LUT_Create(void)
 	}
 
 	//-----------------------------------------------------
-	DataObject_Changed();
-
 	m_Parameters("COLORS_TYPE")->Set_Value(CLASSIFY_LUT);	// Lookup Table
 	m_Parameters("LUT_ATTRIB" )->Set_Value(iField);
 
@@ -903,7 +921,7 @@ wxString CWKSP_Shapes::Get_Name_Attribute(void)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-void CWKSP_Shapes::On_Draw(CWKSP_Map_DC &dc_Map, bool bEdit)
+void CWKSP_Shapes::On_Draw(CWKSP_Map_DC &dc_Map, int Flags)
 {
 	//-----------------------------------------------------
 	if( Get_Extent().Intersects(dc_Map.m_rWorld) != INTERSECTION_None )
@@ -912,7 +930,7 @@ void CWKSP_Shapes::On_Draw(CWKSP_Map_DC &dc_Map, bool bEdit)
 
 		double	Transparency	= m_Parameters("DISPLAY_TRANSPARENCY")->asDouble() / 100.0;
 
-		CWKSP_Map_DC	*pDC	= Transparency > 0.0 ? new CWKSP_Map_DC(dc_Map.m_rWorld, dc_Map.m_rDC, dc_Map.m_Scale, SG_GET_RGB(255, 255, 255)) : NULL;
+		CWKSP_Map_DC	*pDC	= Transparency > 0.0 ? new CWKSP_Map_DC(dc_Map.m_rWorld, dc_Map.m_rDC, dc_Map.m_Scale, SG_GET_RGB(254, 255, 255)) : NULL;
 		CWKSP_Map_DC	&dc		= pDC ? *pDC : dc_Map;
 
 		m_Sel_Color		= Get_Color_asWX(m_Parameters("SEL_COLOR" )->asInt());
@@ -921,7 +939,7 @@ void CWKSP_Shapes::On_Draw(CWKSP_Map_DC &dc_Map, bool bEdit)
 		Draw_Initialize(dc);
 
 		//-------------------------------------------------
-		if( bEdit && (m_Edit_pShape || Get_Shapes()->Get_Selection_Count() > 0) )
+		if( (Flags & LAYER_DRAW_FLAG_NOEDITS) == 0 && (m_Edit_pShape || Get_Shapes()->Get_Selection_Count() > 0) )
 		{
 			for(iShape=0; iShape<Get_Shapes()->Get_Count(); iShape++)
 			{
@@ -992,7 +1010,7 @@ void CWKSP_Shapes::On_Draw(CWKSP_Map_DC &dc_Map, bool bEdit)
 		}
 
 		//-------------------------------------------------
-		if( m_iLabel >= 0 )
+		if( (Flags & LAYER_DRAW_FLAG_NOLABELS) == 0 && m_iLabel >= 0 )
 		{
 			int		iSize	= m_Parameters("LABEL_ATTRIB_SIZE_BY"  )->asInt();
 			double	dSize	= m_Parameters("LABEL_ATTRIB_SIZE_TYPE")->asInt() == 1 ?
@@ -1108,14 +1126,7 @@ void CWKSP_Shapes::_Draw_Label(CWKSP_Map_DC &dc_Map, CSG_Shape *pShape, int Poin
 //---------------------------------------------------------
 CSG_Parameter * CWKSP_Shapes::AttributeList_Add(CSG_Parameter *pNode, const CSG_String &Identifier, const CSG_String &Name, const CSG_String &Description)
 {
-	CSG_Parameter *pParameter;
-
-	pParameter	= m_Parameters.Add_Choice(
-		pNode, Identifier, Name, Description,
-		CSG_String::Format(SG_T("%s|"), _TL("<default>")), 0
-	);
-
-	return( pParameter );
+	return( m_Parameters.Add_Choice(pNode, Identifier, Name, Description, CSG_String(_TL("<default>")) + "|") );
 }
 
 //---------------------------------------------------------
@@ -1123,21 +1134,24 @@ void CWKSP_Shapes::AttributeList_Set(CSG_Parameter *pFields, bool bAddNoField)
 {
 	if( pFields && pFields->Get_Type() == PARAMETER_TYPE_Choice )
 	{
-		wxString	s;
+		CSG_String	Items;
 
 		for(int i=0; i<Get_Shapes()->Get_Field_Count(); i++)
 		{
-			s.Append(wxString::Format(wxT("%s|"), Get_Shapes()->Get_Field_Name(i)));
+			Items	+= CSG_String(Get_Shapes()->Get_Field_Name(i)) + "|";
 		}
 
 		if( bAddNoField )
 		{
-			s.Append(wxString::Format(wxT("%s|"), _TL("<none>")));
+			Items	+= CSG_String(_TL("<none>")) + "|";
 		}
 
-		pFields->asChoice()->Set_Items(s);
+		int			iChoice	= pFields->asInt   ();
+		CSG_String	sChoice	= pFields->asString();
 
-		if( bAddNoField )
+		pFields->asChoice()->Set_Items(Items);
+
+		if( bAddNoField && (iChoice < 0 || iChoice >= pFields->asChoice()->Get_Count() || sChoice.Cmp(pFields->asChoice()->Get_Item(iChoice))) )
 		{
 			pFields->Set_Value(Get_Shapes()->Get_Field_Count());
 		}
@@ -1154,23 +1168,18 @@ void CWKSP_Shapes::AttributeList_Set(CSG_Parameter *pFields, bool bAddNoField)
 //---------------------------------------------------------
 CSG_Parameter * CWKSP_Shapes::BrushList_Add(CSG_Parameter *pNode, const CSG_String &Identifier, const CSG_String &Name, const CSG_String &Description)
 {
-	CSG_Parameter *pParameter;
-
-	pParameter	= m_Parameters.Add_Choice(
-		pNode, Identifier, Name, Description,
-		CSG_String::Format(SG_T("%s|%s|%s|%s|%s|%s|%s|%s|"),
-			_TL("Opaque"),
-			_TL("Transparent"),
+	return( m_Parameters.Add_Choice(pNode, Identifier, Name, Description,
+		CSG_String::Format("%s|%s|%s|%s|%s|%s|%s|%s|",
+			_TL("Opaque"           ),
+			_TL("Transparent"      ),
 			_TL("Backward Diagonal"),
-			_TL("Cross Diagonal"),
-			_TL("Forward Diagonal"),
-			_TL("Cross"),
-			_TL("Horizontal"),
-			_TL("Vertical")
-		), 0
+			_TL("Cross Diagonal"   ),
+			_TL("Forward Diagonal" ),
+			_TL("Cross"            ),
+			_TL("Horizontal"       ),
+			_TL("Vertical"         )
+		), 0)
 	);
-
-	return( pParameter );
 }
 
 //---------------------------------------------------------
@@ -1178,35 +1187,29 @@ int CWKSP_Shapes::BrushList_Get_Style(int Index)
 {
 	switch( Index )
 	{
-	default:
-	case 0:	return( wxSOLID				);
-	case 1:	return( wxTRANSPARENT		);
-	case 2:	return( wxBDIAGONAL_HATCH	);
-	case 3:	return( wxCROSSDIAG_HATCH	);
-	case 4:	return( wxFDIAGONAL_HATCH	);
-	case 5:	return( wxCROSS_HATCH		);
-	case 6:	return( wxHORIZONTAL_HATCH	);
-	case 7:	return( wxVERTICAL_HATCH	);
+	default:	return( wxSOLID				);
+	case  1:	return( wxTRANSPARENT		);
+	case  2:	return( wxBDIAGONAL_HATCH	);
+	case  3:	return( wxCROSSDIAG_HATCH	);
+	case  4:	return( wxFDIAGONAL_HATCH	);
+	case  5:	return( wxCROSS_HATCH		);
+	case  6:	return( wxHORIZONTAL_HATCH	);
+	case  7:	return( wxVERTICAL_HATCH	);
 	}
 }
 
 //---------------------------------------------------------
 CSG_Parameter * CWKSP_Shapes::PenList_Add(CSG_Parameter *pNode, const CSG_String &Identifier, const CSG_String &Name, const CSG_String &Description)
 {
-	CSG_Parameter *pParameter;
-
-	pParameter	= m_Parameters.Add_Choice(
-		pNode, Identifier, Name, Description,
-		CSG_String::Format(SG_T("%s|%s|%s|%s|%s|"),
-			_TL("Solid"),
-			_TL("Dotted"),
-			_TL("Long Dashed"),
+	return( m_Parameters.Add_Choice(pNode, Identifier, Name, Description,
+		CSG_String::Format("%s|%s|%s|%s|%s|",
+			_TL("Solid"       ),
+			_TL("Dotted"      ),
+			_TL("Long Dashed" ),
 			_TL("Short Dashed"),
 			_TL("Dot And Dash")
-		), 0
+		), 0)
 	);
-
-	return( pParameter );
 }
 
 //---------------------------------------------------------
@@ -1214,12 +1217,11 @@ int CWKSP_Shapes::PenList_Get_Style(int Index)
 {
 	switch( Index )
 	{
-	default:
-	case 0:	return( wxSOLID );
-	case 1:	return( wxDOT );
-	case 2:	return( wxLONG_DASH );
-	case 3:	return( wxSHORT_DASH );
-	case 4:	return( wxDOT_DASH );
+	default:	return( wxSOLID      );
+	case  1:	return( wxDOT        );
+	case  2:	return( wxLONG_DASH  );
+	case  3:	return( wxSHORT_DASH );
+	case  4:	return( wxDOT_DASH   );
 	}
 }
 
