@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: wksp_shapes_edit.cpp 2081 2014-04-03 09:55:43Z oconrad $
+ * Version $Id: wksp_shapes_edit.cpp 2202 2014-08-26 13:24:07Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -1019,7 +1019,17 @@ void CWKSP_Shapes::Edit_Shape_Draw(CWKSP_Map_DC &dc_Map)
 		{
 			iPoint	= m_Parameters("EDIT_SNAP_DIST")->asInt();
 
+			dc_Map.dc.SetBrush(wxNullBrush);
+			dc_Map.dc.SetPen  (*wxWHITE);
+			dc_Map.dc.DrawCircle(1 + iPoint, 1 + iPoint, iPoint - 1);
+			dc_Map.dc.DrawCircle(1 + iPoint, 1 + iPoint, iPoint + 1);
+
+			dc_Map.dc.SetPen  (*wxBLACK);
 			dc_Map.dc.DrawCircle(1 + iPoint, 1 + iPoint, iPoint);
+
+			dc_Map.dc.SetBrush(*wxBLACK);
+			dc_Map.dc.SetPen  (*wxBLACK);
+			dc_Map.dc.DrawCircle(1 + iPoint, 1 + iPoint, 1);
 		}
 	}
 }
