@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: wksp_shapes_line.h 2017 2014-02-25 15:48:09Z oconrad $
+ * Version $Id: wksp_shapes_line.h 2329 2014-11-05 19:32:43Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -95,10 +95,12 @@ protected:
 
 	bool						m_bPoints;
 
-	int							m_iSize, m_Size_Type, m_Line_Style, m_Label_Freq;
+	int							m_iSize, m_Size_Type, m_Line_Style, m_Label_Freq, m_Effect;
 
 	double						m_Size, m_dSize, m_Size_Min;
 
+	wxColour					m_Effect_Color;
+
 	wxPen						m_Pen;
 
 
@@ -117,6 +119,11 @@ protected:
 	virtual int					Edit_Shape_HitTest		(CSG_Point Point, double max_Dist, int &iPart, int &iPoint);
 	virtual void				Edit_Snap_Point_ToLine	(CSG_Point Point, CSG_Point &snap_Point, double &snap_Dist, CSG_Shape *pShape);
 
+
+private:
+
+	virtual void				_Draw_Shape				(CWKSP_Map_DC &dc_Map, CSG_Shape *pShape, int xOffset = 0, int yOffset = 0);
+
 };
 
 
