@@ -1,5 +1,5 @@
 /**********************************************************
- * Version $Id: wksp_shapes_polygon.h 2017 2014-02-25 15:48:09Z oconrad $
+ * Version $Id: wksp_shapes_polygon.h 2329 2014-11-05 19:32:43Z oconrad $
  *********************************************************/
 
 ///////////////////////////////////////////////////////////
@@ -96,7 +96,7 @@ protected:
 
 	bool						m_bCentroid, m_bOutline;
 
-	wxColour					m_Sel_Color_Fill;
+	wxColour					m_Sel_Color_Fill[2];
 
 	wxPen						m_Pen;
 
